// FILE: ./assets/templates/controllers/SelectInlineController.js
var SelectInlineController = function(templateManager, $scope, $rootScope, metaDataFactory,  ScreenService, ZhUtil, SelectService, ZHPromise, maskEngine, EventChildPropagation) {

    var self = this;
    
    this.setScopeField = function(field) {
        if(!$scope.field) $scope.field = field;  
    };
    
    this.initFieldSelectInline = function(field) {
        this.setScopeField(field);
    };

    this.showCloseXIconSelect = function(row, fieldName) {
    	if(row[fieldName]) return true;
    	else return false;
    };
    
    this.showSearchIconSelect = function(row, fieldName) {
    	if(row[fieldName]) return false;
    	else return true;
    };
    
    this.clearCurrentRowField = function(fields, widget, $event, row, element, field) {
    	$event.stopPropagation();
    	if(!field.conditions) {
    		widget.currentRow = row;
	    	fields.forEach((field) => {
	    		field = field.name;
		    	if(!widget.currentRow['changed_'.concat(field)]) widget.currentRow['changed_'.concat(field)] = widget.currentRow[field]; 
		        widget.currentRow[field.concat('_inline')] = ''; 
		        widget.currentRow[field] = ''; 
		        if(widget.mode !== 'inline_edit') {
	                widget.inlineEdit();
	                templateManager.updateTemplate();
		        }
	        });
	        widget.lastInlineEditingElement = element.element;
    	}
        else {
        	let valid = true;
        	field.conditions.forEach((condition) => {
        		if(condition.operator == '=' || condition.operator == '==') {
        			valid (valid && row[condition.columnName] == condition.columnValue);
        		}
        		else if(condition.operator == '!=' || condition.operator == '<>') {
        			valid = (valid && row[condition.columnName] != condition.columnValue); 
        		}
        	});
        	if(valid) {
        		widget.currentRow = row;
		    	fields.forEach((field) => {
		    		field = field.name;
			    	if(!widget.currentRow['changed_'.concat(field)]) widget.currentRow['changed_'.concat(field)] = widget.currentRow[field]; 
			        widget.currentRow[field.concat('_inline')] = ''; 
			        widget.currentRow[field] = ''; 
			        if(widget.mode !== 'inline_edit') {
		                widget.inlineEdit();
		                templateManager.updateTemplate();
			        }
		        });
		        widget.lastInlineEditingElement = element.element;
        	}
        	else ScreenService.showMessage(field.errorMessage);
        }
    	
    };
    
    this.openField = function(field, $event ,row, isSpan) {
    	if(!field.conditions) {
    		this.openSelect(field, $event, isSpan);
    	}
        else {
        	let valid = true;
        	field.conditions.forEach((condition) => {
        		if(condition.operator == '=' || condition.operator == '==') {
        			valid (valid && row[condition.columnName] == condition.columnValue);
        		}
        		else if(condition.operator == '!=' || condition.operator == '<>') {
        			valid = (valid && row[condition.columnName] != condition.columnValue); 
        		}
        	});
        	if(valid) this.openSelect(field, $event, isSpan);
        	else ScreenService.showMessage(field.errorMessage);
        }
	};
	 
	this.openSelect = function(field, $event, isSpan) {
		if(isSpan) field.widget.lastInlineEditingElement = $event.toElement.parentElement.parentElement;
        else field.widget.lastInlineEditingElement = $event.toElement;
		var defer = ZHPromise.defer();
		var actions = [{
			"label": ScreenService.i18n("Cancel"),
			"showAsAction": "back",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"hideIcon": true,
			"events": [{
				"name": "ActionEvent",
				"code": "ScreenService.closePopup(); args.owner.widget.afterClosePopup(args.owner.widget.selectField);",
				"id": 9998
			}]
		}];

		if (field.showAddButtonAs == 'action') {
			actions.push({
				"icon": "plus",
				"label": field.addButtonText,
				"showAsAction": "always|edit|checked_rows|view",
				"isVisible": true,
				"showOnForm": true,
				"showOnList": true,
				"events": [{
					"name": "ActionEvent",
					"code": "args.owner.widget.onAdd()",
					"id": 9999
				}]
			});
		}

		innerSearch("", field);

		var template;
		if (!field.showSelectAs) {
			template = "widget/list.html";
		} else if (field.showSelectAs === "tree") {
			template = "widget/tree/grid.html";
		} else {
			template = "widget/" + field.showSelectAs + ".html";
		}

		var fields;
		if (field.fields && field.fields.length) {
			fields = field.fields;
		} else {
			fields = [{
				"label": field.sourceLabel,
				"name": field.descriptionField,
				"showOnList": true,
				"mask": field.mask
			}];
		}

		var isToReload = !!(field.selectWidget && Util.objectHasAnyValue(field.selectWidget.searchCriteria));
		if (!field.selectWidget) {
			var widgetEvents = [{
				"name": "WidgetAfterMoveRow",
				"code": "args.owner.setSelected(args);",
				"id": 9999
			}];
			field.selectWidget = metaDataFactory.widgetFactory({
				"id": 0,
				"name": "Widget",
				"label": field.label,
				"template": template,
				"container": field.widget.container,
				"isVisible": true,
				"popupNoBlock": false,
				"showDescriptionOrder": false,
				"floatingControl": {
					"openSearch": !Util.isMobile(),
					"defaultSearchField": field.defaultSearchField || field.descriptionField,
					"pdfAction": false,
					"xlsAction": false,
					"searchAction": true,
					"customizationAction": {
						"filter": false,
						"layout": false,
						"view": false
					}
				},

				"parentProperty": field.parentProperty,
				"primaryProperty": field.primaryProperty,
				"expandingProperty": field.expandingProperty,
				"breadcrumbProperty": field.breadcrumbProperty,

				"order": field.order,
				"reverse": field.reverse,
				"bringGroupsCollapsed": field.bringGroupsCollapsed,
				"orderFromGroups": field.orderFromGroups,
				"selectField": field,
				"showCheckbox": false,
				"onAdd": field.onAdd,
				"showAddButtonAs": field.showAddButtonAs,
				"addButtonText": field.addButtonText,
				"noMatchText": field.noMatchText,
				"searchNoMatchText": field.searchNoMatchText,
				"fields": fields,
				"dataSource": field.dataSource,
				"dataSourceFilter": field.dataSourceFilter,
				"itemsPerPage": field.itemsPerPage,
				"searchWithPagination": Util.isUndefinedOrNull(field.searchWithPagination) ? true : field.searchWithPagination,
				"hasPagination": Util.isUndefinedOrNull(field.hasPagination) ? true : field.hasPagination,
				"onDemand": field.onDemand,
				"allowSelectOnlyLastChild": field.allowSelectOnlyLastChild,
				"searchCriteria": field.searchCriteria,

				"actions": actions,
				"events": EventChildPropagation.propagate(widgetEvents, field),

				"styles": field.selectWidgetStyles
			});
			field.selectWidget.setSelected = setSelected;
			field.selectWidget.afterClosePopup = afterClosePopup;
			isToReload = !!(field.dataSource && field.dataSource.lazyLoad);
		}

		if (isToReload) {
			field.selectWidget.searchCriteria = {};
			field.selectWidget.searchList = "";
			field.reload().then(defer.resolve, defer.resolve);
		} else {
			defer.resolve();
		}

		ScreenService.openPopup(field.selectWidget);
		return defer.promise;	
	};
    
    
    function innerSearch(selectListSearch, field) {
		if (Util.isArray(field.filterDefinitions) && field.filterDefinitions.length) {
			var dataSource = field.dataSource;
			var filter = ZhUtil.buildFiltersInField(field, selectListSearch);
			dataSource.filter(filter, function(data) {
				dataSource.data = data;
				templateManager.updateTemplate();
			});
		}
	}
	
	var isLastChild = function(row){
		return row._childCount === 0;
	};

	
	function outDataInlineSelect(selectWidget, widgetOutData, fields, changed) {
        fields.forEach((field) => {
           field = field.name;
           if(!selectWidget.currentRow['changed_'.concat(field)]) widgetOutData.currentRow['changed_'.concat(field)] = widgetOutData.currentRow[field]; 
           widgetOutData.currentRow[field.concat('_inline')] = selectWidget.currentRow[field]; 
           if(!widgetOutData.currentRow.changed) widgetOutData.currentRow.changed = true;
           widgetOutData.currentRow[field] = selectWidget.currentRow[field]; 
        });
        if(changed == false) selectWidget.changed = true;
        if(widgetOutData.mode !== 'inline_edit') {
            widgetOutData.inlineEdit();
            templateManager.updateTemplate();
        }
	}
	
	function setSelected(args) {
	    var row = args.row;
		var field = args.owner.selectField;
		outDataInlineSelect(field.selectWidget, field.widget, field.fields, field.widget.changed);
		
		ScreenService.closePopup();

		if (field.element) {
			field.element.focus();
		}

		(field.outData ? changeByOut : changeByName)(field, row);
		field.dataSource.clearCheckedRows();
		field.dataSource.addCheckedRows(row);
		setChanged(field);

		if (field._afterSelected) {
			field._afterSelected(row);
		}
		if (field.change) {
			field.change();
		}
		
	}
	
	function afterClosePopup(field) {
		if (field.element) {
			field.element.focus();
		}
	}
	
	function changeByName(field, row) {
		var valueField = field.valueField ? field.valueField : field.name;
		var descriptionField = field.descriptionField ? field.descriptionField : field.name;
		field.widget.currentRow[valueField] = row[valueField];
		field.widget.currentRow[field.name] = row[descriptionField];
	}

	function changeByOut(field, row) {
		if (field.widget.currentRow) {
			var outData = field.outData || {};

			Object.keys(outData).forEach(function(key) {
				if (Array.isArray(outData[key])) {
					outData[key].forEach(function(valueKey) {
						field.widget.currentRow[valueKey] = row[key];
					});
				} else {
					field.widget.currentRow[outData[key]] = row[key];
				}
			});
		}
	}
	
	var setChanged = function(field) {
		field.widget.currentRow.changed = true;
	};
	
	$scope.$watchCollection('field.dataSourceFilter', propagateFieldDataSourceFilterToSelectWidget);
	function propagateFieldDataSourceFilterToSelectWidget(fieldDataSourceFilter) {
		if ($scope.field.selectWidget) {
			$scope.field.selectWidget.dataSourceFilter = fieldDataSourceFilter;
		}
	}
};


// FILE: ./assets/templates/controllers/zRegisterDfComponents.js
var zRegisterDfComponents = function(decodeTemplateFieldService, eventAggregator) {
    eventAggregator.subscribe('beforeProjectStartup', function register() {
        decodeTemplateFieldService.registerViewTemplate('pebbian-component#widget/pebbian-grid.html', 'pebbian#widget/pebbian-grid.html');
        decodeTemplateFieldService.registerViewTemplate('pebbian-component#widget/tree/grid.html', 'pebbian#widget/tree/grid.html');
    });
};

Configuration(function (ContextRegister, eventEngine) {
	ContextRegister.register('zRegisterDfComponents', zRegisterDfComponents);
});


(function registerTemplates() {
    window.templateUrls = window.templateUrls || [];
    window.templateUrls.push(
        {
            'package': 'pebbian-component',
            'baseUrl': './assets/templates/'
        }
    );
})();

// FILE: ./assets/js/utilities/ErrosAprovacaoLancamentos_Utilities.js
var errosAprovacaoLancamentos_Utilities = function(ScreenService,metaDataFactory) {
    this.errosAprovacao = function(widget,arrayErros, idAprovacao){
        this.openErrosAprovacao(widget,arrayErros, idAprovacao);
    };

    this.openErrosAprovacao = function(parentWidget, arrayErros, idAprovacao){
        var popupWidget = this.createErrosAprovacaoPopup(parentWidget, idAprovacao);
        popupWidget.dataSource.data = arrayErros;
        ScreenService.openPopup(popupWidget);
    };

    this.createErrosAprovacaoPopup = function(parentWidget, idAprovacao) {
        var widget = parentWidget.ErrosAprovacao;
        if(!widget ){
            widget = getPopUpObject(idAprovacao);
            widget.container = parentWidget.container;
            widget = metaDataFactory.widgetFactory(widget,parentWidget);
            parentWidget.ErrosAprovacao = widget;
            widget.parent = parentWidget;
            return widget;
        }else{
            return widget;
        }

    };


    var getPopUpObject = function(idAprovacao) {
        var label = idAprovacao == 1 ? "Erros na aprovacao da nota fiscal" : "Erros na auditoria da nota fiscal";
        return {
            "id": "000",
            "isVisible": true,
            "name": "errosAprovacaoLancamentos",
            "label": label,
            "template": "widget/grid.html",
            "currentRow": {},
            "showAsTab": false,
            "dataSource": {},
            "actions": [
                {
                    "id": 1,
                    "name": "cancelAction",
                    "label": "Voltar",
                    "showAsAction": "back",
                    "isVisible": true,
                    "events": [{
                        "id": 1,
                        "name": "ActionEvent",
                        "code": "ScreenService.closePopup();"
                    }]
                },
                {
                    "name": "VIEWNOTAS",
                    "isVisible": true,
                    "icon": "file-search",
                    "condition": "row.TIPODEERRO == '20'",
                    "showAsAction": "grid-right",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "Dfs56000_AprovacaoLancamentos_Controller.viewNotasBaseCalculo(args)"
                        }
                    ]
                }
            ],
            "fields": [
                {
                    "id": 83,
                    "name": "NRNOTAFISC",
                    "label": "Número",
                    "isVisible": true,
                    "sequenceInForm": 100,
                    "readOnly": false,
                    "defaultValue": null,
                    "filterProperties":{
                        "isVisible": false,
                        "class": 0,
                        "newLine": false
                    },
                    "template": "field/text-edit.html",
                    "class": 5,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {},
                    "placeholder": null,
                    "events": []
                },
                {
                    "id": 3,
                    "name": "VRNOTAFISC",
                    "label": "Valor",
                    "isVisible": true,
                    "sequenceInForm": 100,
                    "readOnly": false,
                    "defaultValue": null,
                    "filterProperties":{
                        "isVisible": false,
                        "class": 0,
                        "newLine": false
                    },
                    "template": "field/text-edit.html",
                    "align": "right",
                    "mask": {
                        "type": "viewFloat",
                        "params": {
                            "showSymbol": true,
                            "precision": 2,
                            "thousands": ".",
                            "decimal": ",",
                            "allowNegative": false
                        }
                    },
                    "class": 5,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {},
                    "placeholder": null,
                    "events": []
                },
                {
                    "id": 82,
                    "name": "DTEMISSAO",
                    "label": "Data de emissão",
                    "isVisible": true,
                    "sequenceInForm": 100,
                    "readOnly": false,
                    "defaultValue": null,
                    "template": "field/text-edit.html",
                    "class": 5,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {},
                    "placeholder": null,
                    "events": []
                },
                {
                    "id": 23,
                    "name": "DTENTRSAID",
                    "label": "Data de entrada",
                    "isVisible": true,
                    "sequenceInForm": 100,
                    "readOnly": false,
                    "defaultValue": null,
                    "template": "field/text-edit.html",
                    "class": 5,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {},
                    "placeholder": null,
                    "events": []
                },
                {
                    "id": 22,
                    "name": "DSERRO",
                    "label": "Descrição",
                    "isVisible": true,
                    "sequenceInForm": 100,
                    "readOnly": false,
                    "defaultValue": null,
                    "filterProperties":{
                        "isVisible": false,
                        "class": 0,
                        "newLine": false
                    },
                    "template": "field/text-edit.html",
                    "class": 5,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {},
                    "placeholder": null,
                    "events": []
                },
                {
                    "id": 21,
                    "name": "CDFILIAL",
                    "template": "field/text-edit.html",
                    "isVisible": false
                },
                {
                    "id": 20,
                    "name": "NRLANCTONF",
                    "template": "field/text-edit.html",
                    "isVisible": false
                }
            ],
            "widgets": []
        };
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('ErrosAprovacaoLancamentos_Utilities', errosAprovacaoLancamentos_Utilities);
});

// FILE: ./assets/js/utilities/ErrosCancelamentoAprovacaoLancamentos_Utilities.js
var errosCancelamentoAprovacaoLancamentos_Utilities = function(ScreenService,metaDataFactory) {
    this.errosCancelamento = function(widget,arrayErros){
        this.openErrosCancelamento(widget,arrayErros);
    };

    this.openErrosCancelamento = function(parentWidget, arrayErros){
        var popupWidget = createErrosCancelamentoPopup(parentWidget);
        popupWidget.dataSource.data = arrayErros;
        ScreenService.openPopup(popupWidget);
    };

    var createErrosCancelamentoPopup = function(parentWidget) {
        var widget = parentWidget.ErrosCancelamento;
        if(!widget ){
            widget = getPopUpObject();
            widget.container = parentWidget.container;
            widget = metaDataFactory.widgetFactory(widget,parentWidget);
            parentWidget.ErrosCancelamento = widget;
            widget.parent = parentWidget;
            return widget;
        }else{
            return widget;
        }

    };

    var getPopUpObject = function() {
        return {
            "id": "000",
            "isVisible": true,
            "name": "errosCancelamentoAprovacaoLancamentos",
            "label": "Erros no cancelamento",
            "template": "widget/grid.html",
            "currentRow": {},
            "showAsTab": false,
            "dataSource": {},
            "actions": [
                {
                    "id": 1,
                    "name": "cancelAction",
                    "label": "Voltar",
                    "showAsAction": "back",
                    "isVisible": true,
                    "events": [{
                        "id": 1,
                        "name": "ActionEvent",
                        "code": "ScreenService.closePopup();"
                    }]
                }
            ],
            "fields": [
                {
                    "id": 3,
                    "name": "CDFILIRECE",
                    "label": "Unidade",
                    "isVisible": true,
                    "sequenceInForm": 100,
                    "readOnly": false,
                    "defaultValue": null,
                    "descriptionOrder": 1,
                    "filterProperties":{
                        "isVisible": false,
                        "class": 0,
                        "newLine": false
                    },
                    "template": "field/text-edit.html",
                    "class": 5,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {},
                    "placeholder": null,
                    "events": []
                },
                {
                    "id": 3,
                    "name": "NRNOTAFISC",
                    "label": "Número",
                    "isVisible": true,
                    "sequenceInForm": 100,
                    "readOnly": false,
                    "defaultValue": null,
                    "filterProperties":{
                        "isVisible": false,
                        "class": 0,
                        "newLine": false
                    },
                    "template": "field/text-edit.html",
                    "class": 5,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {},
                    "placeholder": null,
                    "events": []
                },
                {
                    "id": 3,
                    "name": "VRNOTAFISC",
                    "label": "Valor",
                    "isVisible": true,
                    "sequenceInForm": 100,
                    "readOnly": false,
                    "defaultValue": null,
                    "filterProperties":{
                        "isVisible": false,
                        "class": 0,
                        "newLine": false
                    },
                    "template": "field/text-edit.html",
                    "class": 5,
                    "showOnForm": true,
                    "showOnList": true,
                    "align": "right",
                    "mask": {
                        "type": "viewFloat",
                        "params": {
                            "showSymbol": false,
                            "precision": 2,
                            "thousands": ".",
                            "decimal": ",",
                            "allowNegative": false
                        }
                    },
                    "newLine": true,
                    "dataSource": {},
                    "placeholder": null,
                    "events": []
                },
                {
                    "id": 3,
                    "name": "DSERRO",
                    "label": "Descrição",
                    "isVisible": true,
                    "sequenceInForm": 100,
                    "readOnly": false,
                    "defaultValue": null,
                    "filterProperties":{
                        "isVisible": false,
                        "class": 0,
                        "newLine": false
                    },
                    "template": "field/text-edit.html",
                    "class": 5,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {},
                    "placeholder": null,
                    "events": []
                }
            ]
        };
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('ErrosCancelamentoAprovacaoLancamentos_Utilities', errosCancelamentoAprovacaoLancamentos_Utilities);
});

// FILE: ./assets/js/utilities/ErrosReplicacaoDocumentos_Utilities.js
var errosReplicacaoDocumentos_Utilities = function(ScreenService,metaDataFactory) {
    this.errosReplicacao = function(widget,arrayErros){
        this.openErrosReplicacao(widget,arrayErros);
    };

    this.openErrosReplicacao = function(parentWidget, arrayErros){
        var popupWidget = createErrosReplicacaoPopup(parentWidget);
        popupWidget.dataSource.data = arrayErros;
        ScreenService.openPopup(popupWidget);
    };

    var createErrosReplicacaoPopup = function(parentWidget) {
        var widget = parentWidget.ErrosReplicacao;
        if(!widget ){
            widget = getPopUpObject();
            widget.container = parentWidget.container;
            widget = metaDataFactory.widgetFactory(widget,parentWidget);
            parentWidget.ErrosReplicacao = widget;
            widget.parent = parentWidget;
            return widget;
        }else{
            return widget;
        }

    };

    var getPopUpObject = function() {
        return {
            "id": "000",
            "isVisible": true,
            "name": "errosReplicacaoDocumentos",
            "label": "Erros na replicação da nota fiscal",
            "template": "widget/grid.html",
            "currentRow": {},
            "showAsTab": false,
            "dataSource": {},
            "actions": [
                {
                    "id": 1,
                    "name": "cancelAction",
                    "label": "Voltar",
                    "showAsAction": "back",
                    "isVisible": true,
                    "events": [{
                        "id": 1,
                        "name": "ActionEvent",
                        "code": "ScreenService.closePopup();"
                    }]
                }
            ],
            "fields": [
                {
                    "id": 3,
                    "name": "DSERRO",
                    "label": "Descrição",
                    "descriptionOrder": 1,
                    "isVisible": true,
                    "sequenceInForm": 100,
                    "readOnly": false,
                    "defaultValue": null,
                    "filterProperties":{
                        "isVisible": false,
                        "class": 0,
                        "newLine": false
                    },
                    "template": "field/text-edit.html",
                    "class": 5,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {},
                    "placeholder": null,
                    "events": []
                }
            ]
        };
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('ErrosReplicacaoDocumentos_Utilities', errosReplicacaoDocumentos_Utilities);
});

// FILE: ./assets/js/utilities/Lib00206_InfComplProduto_Repository.js
var Lib00206_InfComplProduto_Repository = function(restEngine) {
    this.getProximoLote = function(filter){
        return restEngine.post('df#/prox_lote', {
            "requestType": "FilterData",
            "filter": filter
        });
    };
    
    this.getProdFili= function(filter){
        return restEngine.post('df#/crud/prodfili', {
            "requestType": "FilterData",
            "filter": filter
        });
    };
    
    this.saveLote= function(row){
        return restEngine.post('df#/save_lote', {
            "requestType": "Row",
            "row": row
        });
    };
    
    this.consultaCustoQuantidadeProduto = function (
        filial, produto, dtLancamento, tipoCusto, almoxarifado, localEstoque, lote, subLote, operacao
    )
    {
        var row = {};
        row.CDFILIAL = filial;
        row.CDTIPOCUST = tipoCusto;
        row.CDPRODUTO = produto;
        row.CDALMOXARIFE = almoxarifado;
        row.CDLOCALESTOQ = localEstoque;
        row.NRLOTE = lote;
        row.NRSUBLOTE = subLote;
        row.DTLANCAMENTO = dtLancamento;
        row.CDTIPOOPER = operacao; 

        return restEngine.post('df#/validacoes_estoque/consulta_custo', {
            "requestType": "Row",
            "row": row
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Lib00206_InfComplProduto_Repository', Lib00206_InfComplProduto_Repository);
});

// FILE: ./assets/js/utilities/ValidacoesEstoque_Repository.js
var ValidacoesEstoque_Repository = function (localStorageEngine, restEngine) {
    var row = {};

    this.validaDataLancamento = function (filial, dtLancamento, operacao, sistema, dtEmissao) {

        row.CDFILIAL = filial;
        row.DTLANCTOEST = dtLancamento;
        row.OPERACAO = operacao;
        row.IDSISTEMA = sistema;
        row.DTEMISSAO = dtEmissao;

        return restEngine.post('df#/validacoes_estoque/data_lancamento', {
            "requestType": "Row",
            "row": row
        });
    };

    /* **************************************************
     * Função que requisita a execução da atualização de
     * estoque. Para maiores informações, favor verificar
     * a função de mesmo nome no Service de validações de
     * estoque.
     *
     * @param string cdfilial
     * @param string nrlancestq
     * @param boolean verifyIrregularities
     * @param boolean abortAtIrregularities
     * @param Array seqItemCodes
     *
     * @return Promise
     */
    this.atualizaEstoque = function (cdfilial, nrlancestq, verifyIrregularities, abortAtIrregularities, seqItemCodes) {
        //sets row values
        row.CDFILIAL = cdfilial;
        row.NRLANCESTQ = nrlancestq;
        row.IDVERIRREG = verifyIrregularities;
        row.IDABRIRREG = abortAtIrregularities;
        row.SEQITEMS = seqItemCodes;

        //sends a request to the backend
        return restEngine.post('df#/validacoes_estoque/atualiza_estoque', {
            "requestType": "Row",
            "row": row
        });
    };

    this.zeraInventario = function (filial, nrLancamento, idordemclc) {

        row.CDFILIAL = filial;
        row.NRLANCAMENTO = nrLancamento;
        row.IDORDEMCLC = idordemclc;

        return restEngine.post('df#/validacoes_estoque/zera_inventario', {
            "requestType": "Row",
            "row": row
        });
    };

    this.consultaCustoQuantidadeProduto = function (
        filial, produto, dtLancamento, tipoCusto, almoxarifado, localEstoque, lote, subLote, operacao
    )
    {
        row.CDFILIAL = filial;
        row.CDTIPOCUST = tipoCusto;
        row.CDPRODUTO = produto;
        row.CDALMOXARIFE = almoxarifado;
        row.CDLOCALESTOQ = localEstoque;
        row.NRLOTE = lote;
        row.NRSUBLOTE = subLote;
        row.DTLANCAMENTO = dtLancamento;
        row.CDTIPOOPER = operacao; 

        return restEngine.post('df#/validacoes_estoque/consulta_custo', {
            "requestType": "Row",
            "row": row
        });
    };
    
    this.verificaEstoqueProduto = function(row) {
         var filtro = [
            {
                "name": "P_CDFILIAL",
                "operator": "=",
                "value": row.CDFILIMOVI
            },
            {
                "name": "P_CDPRODUTO",
                "operator": "=",
                "value": row.CDPRODUTO
            },
            {
                "name": "P_NRLOTEESTQ",
                "operator": "=",
                "value": row.NRLOTEESTQ
            },
            {
                "name": "P_NRSUBLOTE",
                "operator": "=",
                "value": row.NRSUBLOTE
            },
            {
                "name": "P_DTPOSICAO",
                "operator": "=",
                "value": row.DTLANCMOVI
            }
        ];
        return restEngine.post('df#/lote_infCompl_semAlmox', {
            "requestType": "FilterData",
            "filter": filtro
        });
    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register('ValidacoesEstoque_Repository', ValidacoesEstoque_Repository);
});

// FILE: ./assets/js/utilities/Lib00101_Contabilizacao_Factory.js
var Lib00101_Contabilizacao_Factory = function(restEngine) {

    /*
     * Função para retornar a parametrização de contabilidade de acordo
     * com o filtro dado.
     */
    this.getContabilizacaoLancamentos = function(lancamentos) {
        return restEngine.post('df#/getContabilizacaoLancamentos', {
            "requestType": "DataSet",
            "dataset": lancamentos
        });
    };
    
    /*
     * Função para contabilizar o lançamento
     * com o filtro dado.
     */
    this.contabilizaLancamentos = function(lancamentos) {
        return restEngine.post('df#/contabilizaLancamentos', {
            "requestType": "DataSet",
            "dataset": lancamentos
        });
    };

    /*
     * Função para retornar a parametrização de contabilidade de acordo
     * com o filtro dado.
     */
    this.getContopectblParams = function(filter) {
        return restEngine.post('df#/crud/contopectbl', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    /*
     * Função para retornar a parametrização geral.
     */
    this.getParamGeral = function() {
        return restEngine.post('df#/crud/paramgeral', {
            "requestType": "FilterData",
            "filter": []
        });
    };

};

Configuration(function (ContextRegister){
    ContextRegister.register('contabilizacaoFactory', Lib00101_Contabilizacao_Factory);
});

// FILE: ./assets/js/utilities/Lib00101_Contabilizacao_Service.js
var Lib00101_Contabilizacao_Service = function(ZHPromise, localStorageEngine, contabilizacaoFactory) {

    /*
     * Função para remover os dataSets locais que foram setados para a tela.
     * Esta deve ser executada ao sair da tela.
     */
    this.clearLocalData = function() {
        localStorageEngine.remove("df#/erros_contabilizacao");
        localStorageEngine.remove("df#/partidas_contabilizacao");
        localStorageEngine.remove("df#/rateios_contabilizacao");
    };

    this.getContabilizacaoLancamentos = function(lancamentos) {
        return contabilizacaoFactory.getContabilizacaoLancamentos(lancamentos);
    };
    
    
    /*
     * Função para contabilizar o lançamento
     * com o filtro dado.
     */
    this.contabilizaLancamento = function(lancamentos) {
        return contabilizacaoFactory.contabilizaLancamento(lancamentos);
    };

    /*
     * Função para retornar a parametrização de contabilidade de acordo
     * com o filtro dado.
     */
    this.getContopectblParams = function(filter) {
        return contabilizacaoFactory.getContopectblParams(filter);
    };

    /*
     * Função que retorna os dados locais dos erros de contabilização.
     */
    this.getErros = function() {
        return localStorageEngine.getDataSource("df#/erros_contabilizacao");
    };

    /*
     * Função para retornar a parametrização geral.
     */
    this.getParamGeral = function() {
        return contabilizacaoFactory.getParamGeral();
    };

    /*
     * Função que retorna os dados locais das partidas.
     */
    this.getPartidas = function() {
        return localStorageEngine.getDataSource("df#/partidas_contabilizacao");
    };

    /*
     * Função para setar os dataSets localmente, retornando uma promise
     * cujo callback será executado somente após salvar todos os dataSets.
     */
    this.setLocalData = function(dataSets) {
        var promises = [
            setLocalErrosContabilizacao(dataSets.errosContabilizacao),
            setLocalPartidasContabilizacao(dataSets.partidasContabilizacao),
            setLocalRateiosContabilizacao(dataSets.rateiosContabilizacao)
        ];
        return ZHPromise.all(promises);
    };

    /*
     * Função para setar o dataSet de erros localmente.
     */
    var setLocalErrosContabilizacao = function(errosContabilizacao) {
        errosContabilizacao = errosContabilizacao || [];
        return localStorageEngine.setDataSource("df#/erros_contabilizacao", errosContabilizacao);
    };

    /*
     * Função para setar o dataSet de partidas localmente.
     */
    var setLocalPartidasContabilizacao = function(partidasContabilizacao) {
        partidasContabilizacao = partidasContabilizacao || [];
        return localStorageEngine.setDataSource("df#/partidas_contabilizacao", partidasContabilizacao);
    };

    /*
     * Função para setar o dataSet de rateios localmente.
     */
    var setLocalRateiosContabilizacao = function(rateiosContabilizacao) {
        rateiosContabilizacao = rateiosContabilizacao || [];
        return localStorageEngine.setDataSource("df#/rateios_contabilizacao", rateiosContabilizacao);
    };

};

Configuration(function (ContextRegister){
    ContextRegister.register('contabilizacaoService', Lib00101_Contabilizacao_Service);
});

// FILE: ./assets/js/utilities/Lib00206_InfComplProduto_Service.js
var Lib00206_InfComplProduto_Service = function(Lib00206_InfComplProduto_Repository, restEngine) {
    
    var TIPO_CUSTO_PADRAO = '03',
        VAZIO_EST         = ' ';
    
    this.getDtprocessa = function(){
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
        ];
        return restEngine.post("df#/crud/paramestoque_processamento", {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.getParamFilial = function(cdFiliMovi){
        var filial = cdFiliMovi || "FILIAL_LOGADA";
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filial
            }
        ];
        return restEngine.post("df#/crud/paramfilial", {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.getInfoLote = function(cdFiliMovi,cdProdMovi,nrLoteEstoq,nrSubLote){
        return this.getDtprocessa();
    };

    this.saveLote = function(row){
        row.NRLOTEESTQ = row.NRLOTEESTQ || ' ';
        row.NRSUBLOTE  = row.NRSUBLOTE  || ' ';
        row.DTVALILOTE = row.DTVALILOTE || null;
        row.DTFABRPROD = row.DTFABRPROD || null;
        row.CDAGREGACAO = row.CDAGREGACAO || null;

        var saveRow = {
            CDFILIMOVI:row.CDFILIMOVI,
            NRLOTEESTQ:row.NRLOTEESTQ,
            NRSUBLOTE:row.NRSUBLOTE,
            CDPRODESTO:row.CDPRODESTO,
            DTVALILOTE:row.DTVALILOTE,
            DTFABRPROD:row.DTFABRPROD,
            VRFATOCONV: row.VRFATOCONV,
            CDAGREGACAO: row.CDAGREGACAO
        };

        return Lib00206_InfComplProduto_Repository.saveLote(saveRow);
    };

    this.getProdFili = function(cdproduto,cdfilial){
        var filter = [{
            'name':"CDPRODUTO",
            "operator":"=",
            "value":cdproduto
        },
        {
            'name':"CDFILIAL",
            "operator":"=",
            "value":cdfilial
        }];
        return Lib00206_InfComplProduto_Repository.getProdFili(filter);
    };

    this.getProximoLote = function(cdFilial,cdProdesto){
        var filter = [
            {
                "name":"CDFILIAL",
                "operator":"=",
                "value":cdFilial
            },
            {
                "name":"DTVALILTE",
                "operator":"=",
                "value":moment().format('DD/MM/YYYY')
            },
            {
                "name":"CDPRODUTO",
                "operator":"=",
                "value":cdProdesto
            }
        ];
        return Lib00206_InfComplProduto_Repository.getProximoLote(filter);
    };
    
    this.consultaCustoQuantidadeProduto = function(filial, produto, dtLancamento, tipoCusto, almoxarifado, localEstoque, lote, subLote ,operacao) {
        if(!filial || !produto || !dtLancamento) {
            var response = {};
            response.data = {};
            response.data.error = 'Parâmetros incorretos ou insuficientes.';
            return Promise.reject(response);
        }

        almoxarifado = almoxarifado || VAZIO_EST;
        localEstoque = localEstoque || VAZIO_EST;

        lote    =    lote || VAZIO_EST;
        subLote = subLote || VAZIO_EST;

        tipoCusto = tipoCusto || TIPO_CUSTO_PADRAO;

        return Lib00206_InfComplProduto_Repository.consultaCustoQuantidadeProduto(
            filial, produto, dtLancamento, tipoCusto, almoxarifado, localEstoque, lote, subLote, operacao
        );
    };
    
    this.getDataFornImportRastXml = (keyNF, seqItProd, nrOrg) => {
        let filter = [ { "name": "NRACESSONFE", "operator": "=", "value": keyNF },
                       { "name": "NRSEQITPROD", "operator": "=", "value": seqItProd },
                       { "name": "NRORG",       "operator": "=", "value": nrOrg } ];
        
        return restEngine.post("df#/crud/intg_itemproduto", {
            "requestType": "FilterData", 
            "filter": filter
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Lib00206_InfComplProduto_Service', Lib00206_InfComplProduto_Service);
});

// FILE: ./assets/js/utilities/ValidacoesEstoque_Service.js
var ValidacoesEstoque_Service = function(ValidacoesEstoque_Repository) {

    var TIPO_OPERACAO = {
        'ENTRADA':    'E',
        'SAIDA':      'S',
        'AJUSTE_IMP': 'A'
    };

    var TIPO_SISTEMA = [
        'DFE', 'DFS', 'EST', 'FAI', 'FAZ', 'FAC', 'REC', 'PAG'
    ];

    var TIPO_CUSTO_PADRAO = '03',
        VAZIO_EST         = ' ';

    this.validaDataLancamento = function(filial, dataLancamento, operacao, sistema, dataEmissao) {
        var validaOperacaoSistema = (!TIPO_OPERACAO.hasOwnProperty(operacao)) || TIPO_SISTEMA.indexOf(sistema) === -1;

        if(!dataLancamento || !operacao || validaOperacaoSistema) {
            var response = {};
            response.data = {};
            response.data.error = 'Parâmetros incorretos ou insuficientes.';
            return Promise.reject(response);
        }

        operacao = TIPO_OPERACAO[operacao];
        dataLancamento = moment(dataLancamento, "DD/MM/YYYY").format('DD/MM/YYYY');
        dataEmissao = !dataEmissao ? dataLancamento : moment(dataEmissao, "DD/MM/YYYY").format('DD/MM/YYYY');

        return ValidacoesEstoque_Repository.validaDataLancamento(filial, dataLancamento, operacao, sistema, dataEmissao);
    };
    
    this.verificaEstoqueProduto = function(row) {
        return ValidacoesEstoque_Repository.verificaEstoqueProduto(row);  
    };


    /* **************************************************
     * Função que requisita a execução da atualização de
     * estoque. Possui como comportamento padrão a
     * atualização de todos os itens de um lançamento de
     * uma unidade, com verificação de irregularidades e
     * abortamento das operações no caso de invalidez
     *
     * Permite porém que a verificação de irregularidades
     * não seja efetuada ou que as operações não sejam
     * abortadas após a verificação através da passagem
     * dos parâmetros "verifyIrregularities" e
     * "abortAtIrregularities". Note que o último não
     * pode ser verdadeiro se o primeiro for falso.
     *
     * Permite também que sejam selecionados um ou mais
     * itens para a execução da atualização de estoque
     * através da passagem de seus códigos na variável
     * "seqItemCodes". Ela é nula no comportamento
     * padrão, o que garante que todos os itens do
     * lançamento sejam atualizados.
     *
     * @param string  cdfilial
     * @param string  nrlancestq
     * @param boolean verifyIrregularities
     * @param boolean abortAtIrregularities
     * @param Array   seqItemCodes
     *
     * @return Promise
     */
    this.atualizaEstoque = function(cdfilial, nrlancestq, verifyIrregularities, abortAtIrregularities, seqItemCodes) {
        //ensures that "seqItemCodes" will never be "undefined", as such value doesn't exist in PHP
        seqItemCodes = seqItemCodes || null;

        //sets default flag values if and only if they haven't been passed
        if(typeof verifyIrregularities === 'undefined')
            verifyIrregularities = true;
        if(typeof abortAtIrregularities === 'undefined')
            abortAtIrregularities = verifyIrregularities;

        //checks for invalid parameter values and returns rejected promise
        if(!cdfilial || !nrlancestq ||
           (typeof verifyIrregularities !== 'boolean') ||
           (typeof abortAtIrregularities !== 'boolean') ||
           (!verifyIrregularities && abortAtIrregularities)) {
            var response = {};
            response.data = {};
            response.data.error = 'Parâmetros incorretos ou insuficientes.';

            return Promise.reject(response);
        }

        //calls repository's request function, returning its promise
        return ValidacoesEstoque_Repository.atualizaEstoque(
            cdfilial, nrlancestq, verifyIrregularities,
            abortAtIrregularities, seqItemCodes
        );
    };

    this.zeraInventario = function(filial, nrLancamento, idordemclc) {
        idordemclc = (idordemclc !== 'U')? 0 :'U';

        if(!filial || !nrLancamento) {
            var response = {};
            response.data = {};
            response.data.error = 'Parâmetros incorretos ou insuficientes.';
            return Promise.reject(response);
        }

        return ValidacoesEstoque_Repository.zeraInventario(filial, nrLancamento, idordemclc);
    };

    this.consultaCustoQuantidadeProduto = function(filial, produto, dtLancamento, tipoCusto, almoxarifado, localEstoque, lote, subLote , operacao) {
        if(!filial || !produto || !dtLancamento) {
            var response = {};
            response.data = {};
            response.data.error = 'Parâmetros incorretos ou insuficientes.';
            return Promise.reject(response);
        }

        almoxarifado = almoxarifado || VAZIO_EST;
        localEstoque = localEstoque || VAZIO_EST;

        lote    =    lote || VAZIO_EST;
        subLote = subLote || VAZIO_EST;

        tipoCusto = tipoCusto || TIPO_CUSTO_PADRAO;

        return ValidacoesEstoque_Repository.consultaCustoQuantidadeProduto(
            filial, produto, dtLancamento, tipoCusto, almoxarifado, localEstoque, lote, subLote, operacao
        );
    };
};

Configuration (function(ContextRegister) {
    ContextRegister.register('ValidacoesEstoque_Service', ValidacoesEstoque_Service);
});

// FILE: ./assets/js/utilities/Df15200VisualisarNota_Controller.js
var Df15200EspelhoNotaFiscal_Controller = function(ScreenService,ZHPromise, templateManager, metaDataFactory, eventEngine) {
        
    var infoComplEvents = {
        //ladoJson <--                   ladoJS-->
        "EspelhoNotaAfterLoad":"afterLoadEspelhoNota"
    };

    Object.keys(infoComplEvents).forEach(function(eventName) {
        if (!eventEngine.hasEvent(eventName)) {
            eventEngine.registerEvent(eventName, infoComplEvents[eventName]);
        }
    });
    var rowParent;
    
    this.openEspelhoNf = function(widget){
        var widgetEspelho = widget.container.getWidget('df15200_espelho_nota_fiscal');
        var cabecalho = widget.container.getWidget('cabecalho');
        var row = widget.currentRow;
        widgetEspelho.isVisible = true;
        widgetEspelho.editing = true;
        
        controlaStatusWidgets(widget, 'df15200_espelho_nota_fiscal', false, widgetEspelho.widgets);
        widget.openTab(widgetEspelho.name);
        
        var filter = [
            {
                "name": "NRORG",
                "operator": "=",
                "value": row.NRORG
            },
            {
                "name": "NRLANCTONF",
                "operator": "=",
                "value": row.NRLANCTONF
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": row.CDFILIAL
            }
        ];
        
        cabecalho.dataSourceFilter = filter;
        cabecalho.dataSource.load(filter).then((result)=>{
            cabecalho.currentRow = result.dataset.notafiscal_lancamento_espelho[0];
            cabecalho.redraw();
            if(cabecalho.afterLoadEspelhoNota)
            cabecalho.afterLoadEspelhoNota();
            cabecalho.getParent().isVisible = true;
        });
        
        
    };
    
    this.createEspelhoNotaWidget = function(container, parentWidgetName = container.widgets[0].name) {
        var widgetEspelho;
        if(!widgetEspelho ){
            var parentWidget = container.getWidget(parentWidgetName);
            widgetEspelho = getEspelhoNota();
            widgetEspelho.container = container;
            widgetEspelho = metaDataFactory.widgetFactory(widgetEspelho, container);
            parentWidget.widgets.push(widgetEspelho);
            widgetEspelho.parent = parentWidget;
            parentWidget.showAsTab = true;
            return widgetEspelho;
        }else{
            return widgetEspelho;
        }
    };
    
    var getTipoEspelho = function(identrsaid){
        if(identrsaid === 'E')
            return "Espelho Lançamento de Entrada";
        else
            return "Espelho Lançamento de Saida";
    };
    
    this.voltarWidgetPai = function(args){
        var widgetCabecalho = templateManager.container.getWidget('cabecalho');
        var widgetEspelho = templateManager.container.getWidget('df15200_espelho_nota_fiscal');
        widgetCabecalho.editing = false;
        widgetEspelho.editing = false;
        widgetEspelho.isVisible = false;
        controlaStatusWidgets(widgetEspelho.getParent(), 'df15200_espelho_nota_fiscal', true, widgetEspelho.widgets);
        ScreenService.goBack();
    };
    
    this.getTipoRazaoSocialRemetente = function(widget){
      var idorigdestnf = widget.currentRow.IDORIGDESTNF;
      var nmrazsocforn = widget.getField("NMRAZSOCFORN"); 
      var nmrazsocclie = widget.getField("NMRAZSOCCLIE");
      var filial = widget.getField("DSFILIEMIT"); 
      

      if(idorigdestnf === 'F'){
          nmrazsocforn.isVisible = true;
          nmrazsocclie.isVisible = false;
          filial.isVisible = false;
      }
      else if(idorigdestnf === 'C'){
          nmrazsocforn.isVisible = false;
          nmrazsocclie.isVisible = true;
          filial.isVisible = false;
      }
      else if(idorigdestnf ==='L'){
          nmrazsocforn.isVisible = false;
          nmrazsocclie.isVisible = false;
          filial.isVisible = true;
      }
      nmrazsocclie.reload();
      nmrazsocforn.reload();
      filial.reload();
    };
    
    
    var controlaStatusWidgets = function(widgetPai, nomeWidgetAtual, status, ignoredWidgets) {
        var allWidgets = widgetPai.widgets;

        allWidgets.filter(widget => widget.name !== nomeWidgetAtual).forEach(widget => widget.enabled = status);

        if (widgetPai.name !== nomeWidgetAtual)
            widgetPai.enabled = status;

        if (Array.isArray(ignoredWidgets))
            allWidgets.filter(widget => ignoredWidgets.indexOf(widget.name) !== -1).forEach(widget => widget.enabled = !status);
    };
    
    var getEspelhoNota = function (identrsaid){
      return {
            "name": "df15200_espelho_nota_fiscal",
            "label": getTipoEspelho(identrsaid),
            "showMenu": true,
            "showBack": true,
            "showHeader": true,
            "showFooter": true,
            "isVisible": false,
            "template": "widget/tabbedWidget.html",
            "validaEstoque": true,
            "showAsTab": true,
            "defaultTab": "cabecalho",
            "actions": [
                {
                    "id": "9561059822659543754689",
                    "name": "actVoltarCabecalho",
                    "label": "Voltar",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "showAsAction": "back_list",
                    "events": [
                        {
                            "id": "9561059824271429790690",
                            "name": "ActionEvent",
                            "code": "Df15200EspelhoNotaFiscal_Controller.voltarWidgetPai(args);"
                        }
                    ]
                },
                {
                    "id": "9561059823439813908691",
                    "name": "actVoltarCabecalho",
                    "label": "Voltar",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "showAsAction": "back_view",
                    "events": [
                        {
                            "id": "9561059822918311058692",
                            "name": "ActionEvent",
                            "code": "Df15200EspelhoNotaFiscal_Controller.voltarWidgetPai(args);"
                        }
                    ]
                }
            ],
            "widgets": [
                                {
                	"id": "9561059823422121381514",
                	"name": "cabecalho",
                	"label": "Nota",
                	"template": "widget/form-view.html",
                	"isVisible": true,
                	"actions": [{
                			"id": "956105982814923858593",
                			"name": "actVoltarCabecalho",
                			"label": "Voltar",
                			"isVisible": true,
                			"showOnForm": true,
                			"showOnList": true,
                			"showAsAction": "back_list",
                			"events": [{
                				"id": "9561059823614018785594",
                				"name": "ActionEvent",
                				"code": "Df15200EspelhoNotaFiscal_Controller.voltarWidgetPai(args);"
                			}]
                		},
                		{
                			"id": "9561059823870483907595",
                			"name": "actVoltarCabecalho",
                			"label": "Voltar",
                			"isVisible": true,
                			"showOnForm": true,
                			"showOnList": true,
                			"showAsAction": "back_view",
                			"events": [{
                				"id": "9561059822297571925596",
                				"name": "ActionEvent",
                				"code": "Df15200EspelhoNotaFiscal_Controller.voltarWidgetPai(args);"
                			}]
                		}
                	],
                	"events": [
                		{
                			"id": "9561059821731037028598",
                			"name": "WidgetAfterLoadDataSource",
                			"code": "args.owner.activate();"
                		},
                		{
                            "name": "WidgetOnEnter", 
                            "code": "Df15200EspelhoNotaFiscal_Controller.validaRazaoSocialRemetente(args.owner.widget)" 
                		}
                	],
                	"itemsPerPage": 100,
                	"dataSource": {
                		"rest": true,
                		"lazyLoad": true,
                		"name": "df#/notafiscal_lancamento_espelho"
                	},
                	"dataSourceFilter": [],
                	"paramsNewCode": {
                		"__is_new_code": true,
                		"__new_code_pk": "LANCNF/FILIAL",
                		"__table_pk": "NRLANCTONF",
                		"__using_unit": true,
                		"__new_code_length": 10,
                		"__columns_to_return": [
                			"CDFILIAL",
                			"NRLANCTONF"
                		]
                	},
                	"startFieldGroupsOpened": true,
                	"floatingControl": {
                		"visibleModes": [
                			"list"
                		]
                	},
                	"fieldGroups": [{
                			"id": "9561059823524298938601",
                			"name": "destinatario",
                			"label": "Dados Destinatário",
                			"template": "field/text.html",
                			"collapsible": true,
                			"opened": false
                		},
                		{
                			"id": "95610598223150040602",
                			"name": "remetente",
                			"label": "Dados Remetente",
                			"template": "field/text.html",
                			"collapsible": true,
                			"opened": false
                		},
                		{
                			"id": "9561059822793304097603",
                			"name": "nforigem",
                			"label": "Nota Fiscal de Origem",
                			"template": "field/text.html",
                			"collapsible": true,
                			"opened": false
                		},
                		{
                			"id": "9561059822059547104604",
                			"name": "transportadora",
                			"label": "Transportadora",
                			"template": "field/text.html",
                			"collapsible": true,
                			"opened": false
                		}
                	],
                	"fields": [
                	    {
                			"id": "9561059821892534227515",
                			"name": "NMFILIAL",
                			"label": "Unidade",
                			"isVisible": true,
                			"showOnList": true,
                			"showOnForm": true,
                			"showOnView": true,
                			"readOnly": false,
                			"readOnlyOnEdit": true,
                			"defaultValue": null,
                			"class": "6",
                			"sequenceInList": 100,
                			"template": "zh-select-autocomplete#field/select-autocomplete.html",
                			"searchableFields": [
                				"CDFILIAL",
                				"NMFILIAL"
                			],
                			"displayedFields": [
                				"CDFILIAL",
                				"NMFILIAL"
                			],
                			"dataSource": {
                				"name": "df#/filioper_lancamento_notafiscal",
                				"rest": true
                			},
                			"dataSourceFilter": [{
                				"name": "CDOPERADOR",
                				"operator": "=",
                				"value": "OPERADOR_LOGADO"
                			}],
                			"order": "CDFILIAL",
                			"showSelectAs": "grid",
                			"fields": [{
                					"id": "9561059822204238771516",
                					"name": "CDFILIAL",
                					"label": "Código",
                					"showOnList": true
                				},
                				{
                					"id": "956105982280280815517",
                					"name": "NMFILIAL",
                					"label": "Descrição",
                					"showOnList": true
                				}
                			],
                			"valueField": "CDFILIRECE",
                			"descriptionField": "NMFILIAL",
                			"outData": {
                				"CDFILIAL": "CDFILIRECE",
                				"NMFILIAL": "NMFILIAL",
                				"CDEMPRESA": "CDEMPRESA",
                				"NMEMPRESA": "NMEMPRESA",
                				"CDINSCESTA": "CDINSCESTA",
                				"CDINSCMUNI": "CDINSCMUNI",
                				"IDTPIJURFILI": "IDTPIJURNF",
                				"IDUTBLOQOPAFPF": "IDUTBLOQOPAFPF",
                				"IDUTBLOQOPAFPG": "IDUTBLOQOPAFPG",
                				"IDUTBLOQOPAF": "IDUTBLOQOPAF",
                				"IDBLOQCMPSAFFIL": "IDBLOQCMPSAFFIL",
                				"DSPARFILFIS": "DSPARFILFIS",
                				"IDBLOQCMPSAF": "IDBLOQCMPSAF",
                				"DTMINVAZIA": "DTMINVAZIA",
                				"DTPROCINI": "DTPROCINI",
                				"DTMAXVAZIA": "DTMAXVAZIA",
                				"DTPROCFIN": "DTPROCFIN",
                				"IDBLOQAF": "IDBLOQAF",
                				"NRTAMNUMNFUTDFE": "NRTAMNUMNFUTDFE",
                				"DSENDEFILI": "DSENDEDEST",
                				"NMBAIRFILI": "NMBAIRDEST",
                				"NRCEPFILI": "NRCEPDEST",
                				"SGESTADO": "SGESTADODEST",
                				"NMMUNICIPIO": "NMMUNICIPIODEST",
                				"NRTELEFILI": "NRTELEDEST",
                				"IEREM": "IEDEST",
                				"IDGERNFIGUALXML": "IDGERNFIGUALXML",
                				"IDOBRCHAVACES": "IDOBRCHAVACES",
                				"IDPELANIMPXML": "IDPELANIMPXML",
                				"IDPERIMPSPRAF": "IDPERIMPSPRAF",
                				"IDFUNCAOFILI": "IDFUNCAOFILI",
                				"IDBLOQALTPROXML": "IDBLOQALTPROXML"
                			},
                			"validations": {
                				"required": {
                					"message": "Obrigatório"
                				}
                			},
                			"events": [{
                				"id": "9561059821682874838518",
                				"name": "FieldOnChange",
                				"code": "Dfe11000_LancamentoEntrada_Controller.unidadeFieldOnChange(args.owner.field.widget);"
                			}],
                			"filterProperties": {
                				"readOnly": false,
                				"readOnlyOnEdit": false,
                				"sequenceInForm": 200,
                				"events": []
                			}
                		},
                		{
                			"id": "9561059822234432168519",
                			"name": "NMEMPRESA",
                			"label": "Empresa",
                			"isVisible": true,
                			"showOnList": false,
                			"showOnForm": true,
                			"showOnView": true,
                			"readOnly": true,
                			"readOnlyOnEdit": true,
                			"defaultValue": null,
                			"class": "6",
                			"template": "zh-select-autocomplete#field/select-autocomplete.html",
                			"searchableFields": [
                				"CDEMPRESA",
                				"NMEMPRESA"
                			],
                			"displayedFields": [
                				"CDEMPRESA",
                				"NMEMPRESA"
                			],
                			"descriptionField": "NMEMPRESA",
                			"valueField": "CDEMPRESA",
                			"dataSource": {
                				"name": "df#/empresa_emprcntrfil",
                				"rest": true
                			},
                			"dataSourceFilter": [{
                				"name": "CDFILIAL",
                				"operator": "=",
                				"value": "@General.getFieldValue('CDFILIRECE', 'cabecalho')@"
                			}],
                			"order": "CDEMPRESA",
                			"showSelectAs": "grid",
                			"fields": [{
                					"id": "9561059822777255459520",
                					"name": "CDEMPRESA",
                					"label": "Código",
                					"showOnList": true
                				},
                				{
                					"id": "9561059822679713590521",
                					"name": "NMEMPRESA",
                					"label": "Descrição",
                					"showOnList": true
                				}
                			],
                			"outData": {
                				"CDEMPRESA": "CDEMPRESA",
                				"NMEMPRESA": "NMEMPRESA",
                				"CDINSCESTA": "CDINSCESTA",
                				"CDINSCMUNI": "CDINSCMUNI"
                			},
                			"validations": {
                				"required": {
                					"message": "Obrigatório"
                				}
                			},
                			"events": [],
                			"filterProperties": {
                				"validations": {},
                				"sequenceInForm": 300,
                				"events": []
                			}
                		},
                		{
                			"id": "9561059823233515940522",
                			"name": "NMTIPOOPER",
                			"label": "Operação",
                			"isVisible": true,
                			"showOnList": true,
                			"showOnForm": true,
                			"showOnView": true,
                			"readOnly": false,
                			"readOnlyOnEdit": true,
                			"defaultValue": null,
                			"class": "6",
                			"sequenceInList": 500,
                			"template": "zh-select-autocomplete#field/select-autocomplete.html",
                			"searchableFields": [
                				"CDTIPOOPER",
                				"NMTIPOOPER"
                			],
                			"displayedFields": [
                				"CDTIPOOPER",
                				"NMTIPOOPER"
                			],
                			"descriptionField": "NMTIPOOPER",
                			"valueField": "CDTIPOOPER",
                			"showSelectAs": "grid",
                			"dataSource": {
                				"name": "df_entrada#/tipooperac_lancamento_entrada",
                				"rest": true,
                				"lazyLoad": true
                			},
                			"order": "CDTIPOOPER",
                			"dataSourceFilter": [{
                					"name": "P_CDFILIAL",
                					"operator": "=",
                					"value": "@General.getFieldValue('CDFILIRECE', 'cabecalho')@"
                				},
                				{
                					"name": "P_IDUTBLOQOPAF",
                					"operator": "=",
                					"value": "@General.getFieldValue('IDUTBLOQOPAF', 'cabecalho')@"
                				}
                			],
                			"fields": [{
                					"id": "9561059821926225983523",
                					"name": "CDTIPOOPER",
                					"label": "Código",
                					"showOnList": true
                				},
                				{
                					"id": "9561059824263122258524",
                					"name": "NMTIPOOPER",
                					"label": "Descrição",
                					"showOnList": true
                				}
                			],
                			"outData": {
                				"CDTIPOOPER": "CDTIPOOPER",
                				"NMTIPOOPER": "NMTIPOOPER",
                				"CDMODDOCFISC": "CDMODDOCFISC",
                				"DSMODDOCFISC": "DSMODDOCFISC",
                				"IDMODDOCNFE": "IDMODDOCNFE",
                				"IDORIGDESTOP": "IDORIGDESTOP",
                				"IDSERIEAUTO": "IDSERIEAUTO",
                				"IDUTILEMISNF": "IDUTILEMISNF",
                				"IDUTILVENCOBR": "IDUTILVENCOBR",
                				"IDTIPOOPERELE": "IDTIPOOPERELE",
                				"IDOPERDEVONF": "IDOPERDEVONF",
                				"IDNFORIGOBRI": "IDNFORIGOBRI",
                				"IDUTLLISTORI": "IDUTLLISTORI",
                				"DSOPERNFORIG": "DSOPERNFORIG",
                				"IDDCORIGRELAC": "IDDCORIGRELAC",
                				"IDNFESTCANC": "IDNFESTCANC"
                			},
                			"validations": {
                				"required": {
                					"message": "Obrigatório"
                				}
                			},
                			"events": [{
                				"id": "9561059821535675353525",
                				"name": "FieldOnChange",
                				"code": "Dfe11000_LancamentoEntrada_Controller.tipoOperacaoOnChange(args);"
                			}],
                			"filterProperties": {
                				"dataSourceFilter": [{
                						"name": "P_CDFILIAL",
                						"operator": "=",
                						"value": "@Dfe11000_LancamentoEntrada_Controller.getCurrentField('CDFILIRECE');@"
                					},
                					{
                						"name": "P_IDUTBLOQOPAF",
                						"operator": "=",
                						"value": "@General.getFieldValue('IDUTBLOQOPAF', 'cabecalho', true)@"
                					}
                				],
                				"sequenceInForm": 700,
                				"readOnly": false,
                				"readOnlyOnEdit": false,
                				"validations": {},
                				"events": []
                			}
                		},
                		{
                			"id": "9561059821053990405526",
                			"name": "DSMODDOCFISC",
                			"label": "Modelo de Documento",
                			"isVisible": true,
                			"showOnList": false,
                			"showOnForm": true,
                			"showOnView": true,
                			"readOnly": true,
                			"readOnlyOnEdit": false,
                			"defaultValue": null,
                			"class": "6",
                			"template": "zh-select-autocomplete#field/select-autocomplete.html",
                			"searchableFields": [
                				"CDMODDOCFISC",
                				"DSMODDOCFISC"
                			],
                			"displayedFields": [
                				"CDMODDOCFISC",
                				"DSMODDOCFISC"
                			],
                			"dataSource": {
                				"name": "df#/moddocfisc_moddocfisctpop",
                				"rest": true
                			},
                			"dataSourceFilter": [{
                					"name": "IDENTRSAIDOP",
                					"operator": "=",
                					"value": "E"
                				},
                				{
                					"name": "CDTIPOOPER",
                					"operator": "=",
                					"value": "@General.getFieldValue('CDTIPOOPER', 'cabecalho')@"
                				}
                			],
                			"order": "CDMODDOCFISC",
                			"showSelectAs": "grid",
                			"valueField": "CDMODDOCFISC",
                			"descriptionField": "DSMODDOCFISC",
                			"outData": {
                				"CDMODDOCFISC": "CDMODDOCFISC",
                				"DSMODDOCFISC": "DSMODDOCFISC",
                				"IDMODDOCNFE": "IDMODDOCNFE"
                			},
                			"fields": [{
                					"id": "9561059823758624243527",
                					"name": "CDMODDOCFISC",
                					"label": "Código",
                					"showOnList": true
                				},
                				{
                					"id": "9561059823397387195528",
                					"name": "DSMODDOCFISC",
                					"label": "Descrição",
                					"showOnList": true
                				}
                			],
                			"validations": {
                				"required": {
                					"message": "Obrigatório"
                				}
                			},
                			"events": [{
                				"id": "9561059823165096997529",
                				"name": "FieldOnChange",
                				"code": "Dfe11000_LancamentoEntrada_Controller.modeloDeDocumentoOnChange(args);"
                			}],
                			"filterProperties": {
                				"isVisible": false,
                				"showOnList": false,
                				"showOnForm": false,
                				"showOnView": false
                			}
                		},
                		{
                			"id": "9561059823730803258530",
                			"name": "DSINSJURNF",
                			"label": "Razão Social",
                			"isVisible": true,
                			"readOnly": true,
                			"showOnForm": false,
                			"showOnList": true,
                			"showOnView": false,
                			"descriptionOrder": 2,
                			"sequenceInList": 300,
                			"filterProperties": {
                				"isVisible": false
                			},
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059821506356812531",
                			"name": "NRACESSONFE",
                			"label": "Chave de Acesso",
                			"isVisible": true,
                			"showOnList": false,
                			"showOnForm": true,
                			"showOnView": true,
                			"readOnly": true,
                			"readOnlyOnEdit": true,
                			"defaultValue": null,
                			"class": "12",
                			"template": "field/text-edit.html",
                			"dataSource": [],
                			"align": "center",
                			"mask": {
                				"type": "fix",
                				"params": {
                					"mask": "9999.9999.9999.9999.9999.9999.9999.9999.9999.9999.9999",
                					"cleanMaskOnModel": true
                				}
                			},
                			"events": [{
                				"id": "9561059823481068368532",
                				"name": "FieldOnChange",
                				"code": "Dfe11000_LancamentoEntrada_Controller.accessKeyOnChange(args);"
                			}],
                			"filterProperties": {
                				"isVisible": false,
                				"showOnList": false,
                				"showOnForm": false,
                				"showOnView": false
                			}
                		},
                		{
                			"id": "9561059823786691175533",
                			"name": "IDORIGDESTNF",
                			"label": "Tipo",
                			"isVisible": false,
                			"showOnForm": false,
                			"showOnView": false,
                			"showOnList": false,
                			"class": "3",
                			"readOnly": false,
                			"template": "field/select-native.html",
                			"newLine": true,
                			"defaultValue": null,
                			"hideEmptyValue": true,
                			"dataSource": {
                				"data": [{
                						"IDORIGDESTNF": "F",
                						"DSORIGDESTNF": "Fornecedor"
                					},
                					{
                						"IDORIGDESTNF": "C",
                						"DSORIGDESTNF": "Cliente"
                					},
                					{
                						"IDORIGDESTNF": "L",
                						"DSORIGDESTNF": "Unidade"
                					}
                				]
                			},
                			"descriptionField": "DSORIGDESTNF",
                			"valueField": "IDORIGDESTNF",
                			"events": [],
                			"filterProperties": {
                				"isVisible": true,
                				"showOnForm": true,
                				"showOnList": true,
                				"showOnView": true,
                				"sequenceInForm": 400,
                				"operator": "=",
                				"events": [{
                					"name": "FieldOnChange",
                					"code": "Dfe11000_LancamentoEntrada_Controller.idorigdestnfOnChangeFiltro(args)"
                				}]
                			}
                		},
                		{
                			"id": "9561059822488571600534",
                			"name": "NMRAZSOCCLIE",
                			"label": "Remetente",
                			"isVisible": true,
                			"showOnList": false,
                			"showOnForm": false,
                			"showOnView": false,
                			"readOnly": false,
                			"readOnlyOnEdit": true,
                			"defaultValue": null,
                			"class": "8",
                			"template": "zh-select-autocomplete#field/select-autocomplete.html",
                			"searchableFields": [
                				"NRINSJURCLIE",
                				"NMRAZSOCCLIE"
                			],
                			"displayedFields": [
                				"NRINSJURCLIE",
                				"NMRAZSOCCLIE"
                			],
                			"showSelectAs": "grid",
                			"validations": {
                				"required": {
                					"message": "Obrigatório"
                				}
                			},
                			"dataSource": {
                				"name": "df#/crud/cliente_lancamento_notafiscal",
                				"rest": true,
                				"itemsPerPage": 100
                			},
                			"order": "NRINSJURCLIE",
                			"dataSourceFilter": [],
                			"outData": {
                				"DSCLIENTE": "DSCLIENTE",
                				"CDCLIENTE": "CDCLIENTE",
                				"NRINSJURCLIE": "NRINSJURNF",
                				"NMRAZSOCCLIE": "NMRAZSOCCLIE",
                				"IDTPIJURCLIE": "IDTPIJURNF",
                				"DSENDECLIE": "DSENDEREM",
                				"NMBAIRCLIE": "NMBAIRREM",
                				"NRCEPCLIE": "NRCEPREM",
                				"SGESTADO": "SGESTADOREM",
                				"NMMUNICIPIO": "NMMUNICIPIOREM",
                				"NRTELECLIE": "NRTELEREM",
                				"IEREM": "IEREM"
                			},
                			"descriptionField": "DSCLIENTE",
                			"valueField": "CDCLIENTE",
                			"fields": [{
                					"id": "9561059821591433352535",
                					"label": "Código",
                					"template": "field/text.html",
                					"name": "NRINSJURCLIE",
                					"showOnList": true
                				},
                				{
                					"id": "956105982188980568536",
                					"label": "Nome",
                					"template": "field/text.html",
                					"name": "NMRAZSOCCLIE",
                					"showOnList": true
                				}
                			],
                			"events": [{
                				"id": "9561059823247844084537",
                				"name": "FieldOnChange",
                				"code": "Dfe11000_LancamentoEntrada_Controller.remetenteFieldOnChange(args);"
                			}],
                			"filterProperties": {
                				"class": "9",
                				"readOnly": true,
                				"readOnlyOnEdit": false,
                				"valueField": "NRINSJURCLIE",
                				"validations": {},
                				"sequenceInForm": 500,
                				"events": []
                			}
                		},
                		{
                			"id": "9561059823236252379538",
                			"name": "NMRAZSOCFORN",
                			"label": "Remetente",
                			"isVisible": true,
                			"showOnList": false,
                			"showOnForm": true,
                			"showOnView": true,
                			"readOnly": false,
                			"readOnlyOnEdit": true,
                			"defaultValue": null,
                			"class": "8",
                			"template": "zh-select-autocomplete#field/select-autocomplete.html",
                			"searchableFields": [
                				"NRINSJURFORN",
                				"NMRAZSOCFORN"
                			],
                			"displayedFields": [
                				"NRINSJURFORN",
                				"NMRAZSOCFORN"
                			],
                			"showSelectAs": "grid",
                			"validations": {
                				"required": {
                					"message": "Obrigatório"
                				}
                			},
                			"dataSource": {
                				"name": "df#/crud/fornecedor_lancamento_notafiscal",
                				"rest": true,
                				"itemsPerPage": 100
                			},
                			"order": "NRINSJURFORN",
                			"outData": {
                				"CDFORNECED": "CDFORNECED",
                				"DSFORNECED": "DSFORNECED",
                				"NRINSJURFORN": "NRINSJURNF",
                				"NMRAZSOCFORN": "NMRAZSOCFORN",
                				"IDBLQNFRELAFOAF": "IDBLQNFRELAFOAF",
                				"IDTPIJURFORN": "IDTPIJURNF",
                				"IDBLOQCMPSAFFOR": "IDBLOQCMPSAFFOR",
                				"DSENDEFORN": "DSENDEREM",
                				"NMBAIRFORN": "NMBAIRREM",
                				"NRCEPFORN": "NRCEPREM",
                				"SGESTADO": "SGESTADOREM",
                				"NMMUNICIPIO": "NMMUNICIPIOREM",
                				"NRTELEFORN": "NRTELEREM",
                				"IEREM": "IEREM"
                			},
                			"descriptionField": "NMRAZSOCFORN",
                			"valueField": "CDFORNECED",
                			"fields": [{
                					"id": "9561059822344036922539",
                					"label": "Código",
                					"template": "field/text.html",
                					"name": "NRINSJURFORN",
                					"showOnList": true
                				},
                				{
                					"id": "9561059822947279429540",
                					"label": "Nome",
                					"template": "field/text.html",
                					"name": "NMRAZSOCFORN",
                					"showOnList": true
                				}
                			],
                			"events": [{
                				"id": "9561059823807364815541",
                				"name": "FieldOnChange",
                				"code": "Dfe11000_LancamentoEntrada_Controller.remetenteFieldOnChange(args);"
                			}],
                			"filterProperties": {
                				"class": "9",
                				"readOnly": true,
                				"readOnlyOnEdit": false,
                				"valueField": "NRINSJURFORN",
                				"validations": {},
                				"sequenceInForm": 510,
                				"events": []
                			}
                		},
                		{
                			"id": "9561059821636981904542",
                			"name": "DSFILIEMIT",
                			"label": "Remetente",
                			"isVisible": true,
                			"showOnList": false,
                			"showOnForm": false,
                			"showOnView": false,
                			"readOnly": false,
                			"readOnlyOnEdit": true,
                			"defaultValue": null,
                			"class": "8",
                			"template": "zh-select-autocomplete#field/select-autocomplete.html",
                			"searchableFields": [
                				"CDFILIAL",
                				"NMFILIAL"
                			],
                			"displayedFields": [
                				"CDFILIAL",
                				"NMFILIAL"
                			],
                			"showSelectAs": "grid",
                			"validations": {
                				"required": {
                					"message": "Obrigatório"
                				}
                			},
                			"dataSource": {
                				"name": "df#/crud/filial_lancamento_notafiscal",
                				"rest": true,
                				"itemsPerPage": 100
                			},
                			"dataSourceFilter": [{
                					"name": "CDEMPRESA",
                					"operator": "=",
                					"value": "@General.getFieldValue('CDEMPRESA', 'cabecalho')@"
                				},
                				{
                					"name": "CDFILIAL",
                					"operator": "<>",
                					"value": "@General.getFieldValue('CDFILIRECE', 'cabecalho')@"
                				}
                			],
                			"order": "CDFILIAL",
                			"outData": {
                				"CDFILIAL": "CDFILIEMIT",
                				"NMFILIAL": "DSFILIEMIT",
                				"DSENDEFILI": "DSENDEREM",
                				"NMBAIRFILI": "NMBAIRREM",
                				"NRCEPFILI": "NRCEPREM",
                				"SGESTADO": "SGESTADOREM",
                				"NMMUNICIPIO": "NMMUNICIPIOREM",
                				"NRTELEFILI": "NRTELEREM",
                				"IEREM": "IEREM"
                			},
                			"descriptionField": "DSFILIEMIT",
                			"valueField": "CDFILIEMIT",
                			"fields": [{
                					"id": "9561059822014651672543",
                					"label": "Código",
                					"template": "field/text.html",
                					"name": "CDFILIAL",
                					"showOnList": true
                				},
                				{
                					"id": "9561059823504346224544",
                					"label": "Unidade",
                					"template": "field/text.html",
                					"name": "NMFILIAL",
                					"showOnList": true
                				}
                			],
                			"events": [{
                				"id": "956105982215507972545",
                				"name": "FieldOnChange",
                				"code": "Dfe11000_LancamentoEntrada_Controller.remetenteFieldOnChange(args);"
                			}],
                			"filterProperties": {
                				"class": "9",
                				"readOnly": true,
                				"readOnlyOnEdit": false,
                				"validations": {},
                				"valueField": "CDFILIEMIT",
                				"sequenceInForm": 520,
                				"dataSourceFilter": [{
                					"name": "CDFILIAL",
                					"operator": "<>",
                					"value": "@Dfe11000_LancamentoEntrada_Controller.getFieldValueFilter('CDFILIRECE')@"
                				}],
                				"events": []
                			}
                		},
                		{
                			"id": "9561059821301266445546",
                			"name": "NRNOTAFISC",
                			"label": "Número",
                			"isVisible": true,
                			"showOnList": true,
                			"showOnForm": true,
                			"showOnView": true,
                			"readOnly": false,
                			"readOnlyOnEdit": false,
                			"defaultValue": null,
                			"class": "4",
                			"sequenceInList": 200,
                			"template": "field/text-edit.html",
                			"descriptionOrder": 2,
                			"dataSource": [],
                			"validations": {
                				"required": {
                					"message": "Obrigatório"
                				},
                				"maxLength": {
                					"length": 20
                				}
                			},
                			"mask": {
                				"type": "zerofill",
                				"params": {
                					"acceptOnlyNumbers": true,
                					"maxlength": 9
                				}
                			},
                			"events": [],
                			"filterProperties": {
                				"isVisible": false,
                				"showOnList": false,
                				"showOnForm": false,
                				"showOnView": false
                			}
                		},
                		{
                			"id": "956105982341546451547",
                			"name": "DSSERIEENTR",
                			"label": "Série",
                			"isVisible": true,
                			"showOnList": true,
                			"showOnForm": true,
                			"showOnView": true,
                			"readOnly": false,
                			"defaultValue": null,
                			"class": "4",
                			"sequenceInList": 400,
                			"template": "zh-select-autocomplete#field/select-autocomplete.html",
                			"searchableFields": [
                				"CDSERIEENTR",
                				"DSSERIEENTR"
                			],
                			"displayedFields": [
                				"CDSERIEENTR",
                				"DSSERIEENTR"
                			],
                			"descriptionOrder": 3,
                			"dataSource": {
                				"name": "df#/crud/serieentr",
                				"rest": true
                			},
                			"order": "CDSERIEENTR",
                			"showSelectAs": "grid",
                			"valueField": "CDSERIEENTR",
                			"descriptionField": "DSSERIEENTR",
                			"fields": [{
                					"id": "9561059822749600719548",
                					"name": "CDSERIEENTR",
                					"template": "field/text.html",
                					"label": "Código",
                					"showOnList": true
                				},
                				{
                					"id": "9561059824199447493549",
                					"name": "DSSERIEENTR",
                					"template": "field/text.html",
                					"label": "Descrição",
                					"showOnList": true
                				}
                			],
                			"events": [],
                			"filterProperties": {
                				"isVisible": false,
                				"showOnList": false,
                				"showOnForm": false,
                				"showOnView": false
                			}
                		},
                		{
                			"id": "956105982148354564550",
                			"name": "DTEMISSAO",
                			"label": "Data de Emissão",
                			"class": "4",
                			"sequenceInList": 600,
                			"isVisible": true,
                			"showOnList": true,
                			"showOnForm": true,
                			"showOnView": true,
                			"readOnly": false,
                			"defaultValue": null,
                			"template": "field/calendar.html",
                			"dateFormat": "DD/MM/YYYY HH:mm:ss",
                			"displayFormat": "DD/MM/YYYY",
                			"validations": {
                				"required": {}
                			},
                			"mask": {
                				"type": "date",
                				"params": {
                					"format": "DD/MM/YYYY"
                				}
                			},
                			"events": [{
                				"id": "9561059824017738601551",
                				"name": "FieldOnChange",
                				"code": "Dfe11000_LancamentoEntrada_Controller.dateOnChange(args.owner.field);"
                			}],
                			"filterProperties": {
                				"class": "6",
                				"sequenceInForm": 600,
                				"dateFormat": "DD/MM/YYYY HH:mm:ss",
                				"template": "zh-interval-component#field/interval.html",
                				"readOnly": false,
                				"validations": {},
                				"intervals": [{
                						"label": "Semana Anterior",
                						"period": "last-week"
                					},
                					{
                						"label": "Semana Atual",
                						"period": "this-week"
                					},
                					{
                						"label": "Próxima Semana",
                						"period": "next-week"
                					},
                					{
                						"label": "Mês Anterior",
                						"period": "last-month"
                					},
                					{
                						"label": "Mês Atual",
                						"period": "this-month"
                					},
                					{
                						"label": "Próximo Mês",
                						"period": "next-month"
                					}
                				]
                			}
                		},
                		{
                			"id": "9561059821600580891552",
                			"name": "DTENTRSAID",
                			"label": "Data de Entrada",
                			"class": "4",
                			"sequenceInList": 610,
                			"isVisible": true,
                			"showOnList": true,
                			"showOnForm": true,
                			"showOnView": true,
                			"readOnly": false,
                			"defaultValue": null,
                			"template": "field/calendar.html",
                			"dateFormat": "DD/MM/YYYY HH:mm:ss",
                			"displayFormat": "DD/MM/YYYY",
                			"validations": {
                				"required": {
                					"message": "Obrigatório"
                				}
                			},
                			"mask": {
                				"type": "date",
                				"params": {
                					"format": "DD/MM/YYYY"
                				}
                			},
                			"events": [{
                				"id": "9561059822158174267553",
                				"name": "FieldOnChange",
                				"code": "Dfe11000_LancamentoEntrada_Controller.dateOnChange(args.owner.field);"
                			}],
                			"filterProperties": {
                				"class": "6",
                				"sequenceInForm": 610,
                				"dateFormat": "DD/MM/YYYY",
                				"template": "zh-interval-component#field/interval.html",
                				"defaultValue": "this-week",
                				"readOnly": false,
                				"validations": {
                					"requiredInterval": true
                				},
                				"intervals": [{
                						"label": "Semana Anterior",
                						"period": "last-week"
                					},
                					{
                						"label": "Semana Atual",
                						"period": "this-week"
                					},
                					{
                						"label": "Próxima Semana",
                						"period": "next-week"
                					},
                					{
                						"label": "Mês Anterior",
                						"period": "last-month"
                					},
                					{
                						"label": "Mês Atual",
                						"period": "this-month"
                					},
                					{
                						"label": "Próximo Mês",
                						"period": "next-month"
                					}
                				]
                			}
                		},
                		{
                			"id": "9561059824212437651554",
                			"name": "VRNOTAFISC",
                			"label": "Valor",
                			"isVisible": true,
                			"showOnList": true,
                			"showOnForm": true,
                			"showOnView": true,
                			"readOnly": false,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"class": "3",
                			"sequenceInList": 700,
                			"align": "right",
                			"mask": {
                				"type": "viewFloat",
                				"params": {
                					"showSymbol": false,
                					"precision": 2,
                					"thousands": ".",
                					"decimal": ","
                				}
                			},
                			"dataSource": [],
                			"validations": {
                				"required": {
                					"message": "Obrigatório"
                				}
                			},
                			"events": [],
                			"filterProperties": {
                				"isVisible": false,
                				"showOnList": false,
                				"showOnForm": false,
                				"showOnView": false
                			}
                		},
                		{
                			"id": "9561059824058233944555",
                			"name": "CBUTILIZAAF",
                			"label": "Importar Af",
                			"isVisible": true,
                			"showOnList": false,
                			"showOnForm": true,
                			"showOnView": true,
                			"readOnly": true,
                			"readOnlyOnEdit": true,
                			"defaultValue": null,
                			"class": "6",
                			"template": "field/select-native.html",
                			"hideEmptyValue": true,
                			"dataSource": {
                				"data": [{
                					"CBUTILIZAAF": "S",
                					"DSUTILIZAAF": "Realizar importação Af"
                				}]
                			},
                			"valueField": "CBUTILIZAAF",
                			"descriptionField": "DSUTILIZAAF",
                			"events": [{
                				"id": "9561059823040881826556",
                				"name": "FieldOnChange",
                				"code": "Dfe11000_LancamentoEntrada_Controller.abreImportacaoAF(args.owner.field);"
                			}],
                			"filterProperties": {
                				"isVisible": false,
                				"showOnList": false,
                				"showOnForm": false,
                				"showOnView": false
                			}
                		},
                		{
                			"id": "9561059821098516462557",
                			"name": "IDSTATUS",
                			"label": "Status",
                			"isVisible": true,
                			"showOnList": true,
                			"showOnForm": true,
                			"showOnView": true,
                			"readOnly": true,
                			"defaultValue": "D",
                			"class": "3",
                			"sequenceInList": 50,
                			"template": "field/select-native.html",
                			"hideEmptyValue": true,
                			"dataSource": {
                				"data": [{
                						"IDSTATUS": "P",
                						"DSSTATUS": "Pendente"
                					},
                					{
                						"IDSTATUS": "D",
                						"DSSTATUS": "Em digitação"
                					},
                					{
                						"IDSTATUS": "A",
                						"DSSTATUS": "Aprovada"
                					},
                					{
                						"IDSTATUS": "C",
                						"DSSTATUS": "Cancelada"
                					}
                				]
                			},
                			"valueField": "IDSTATUS",
                			"descriptionField": "DSSTATUS",
                			"events": [],
                			"filterProperties": {
                				"class": "12",
                				"operator": "=",
                				"sequenceInForm": 100,
                				"hideEmptyValue": true,
                				"readOnly": false,
                				"readOnlyOnEdit": false,
                				"defaultValue": ""
                			}
                		},
                		{
                			"id": "9561059821127486949558",
                			"name": "DSNOTAFISCORIG",
                			"label": "Número",
                			"isVisible": true,
                			"showOnList": false,
                			"showOnForm": true,
                			"showOnView": true,
                			"readOnly": true,
                			"readOnlyOnEdit": true,
                			"defaultValue": null,
                			"class": "12",
                			"fieldGroup": "nforigem",
                			"template": "zh-select-autocomplete#field/select-autocomplete.html",
                			"searchableFields": [
                				"CDSERIE",
                				"NRNOTAFISC",
                				"DTEMISSAO",
                				"VRNOTAFISC"
                			],
                			"displayedFields": [
                				"CDSERIE",
                				"NRNOTAFISC",
                				"DTEMISSAO",
                				"VRNOTAFISC"
                			],
                			"dataSource": {
                				"name": "",
                				"lazyLoad": true,
                				"rest": true
                			},
                			"order": "NRNOTAFISC",
                			"showSelectAs": "grid",
                			"fields": [{
                					"id": "9561059823298282429559",
                					"name": "CDSERIE",
                					"label": "Série",
                					"template": "field/text.html",
                					"showOnList": true
                				},
                				{
                					"id": "956105982627736121560",
                					"name": "NRNOTAFISC",
                					"label": "Número NF",
                					"template": "field/text.html",
                					"showOnList": true
                				},
                				{
                					"id": "9561059823531856442561",
                					"name": "DTEMISSAO",
                					"label": "Data de Emissão",
                					"template": "field/calendar.html",
                					"dateFormat": "DD/MM/YYYY",
                					"showOnList": true
                				},
                				{
                					"id": "9561059821046782023562",
                					"name": "VRNOTAFISC",
                					"label": "Valor",
                					"template": "field/text.html",
                					"mask": {
                						"type": "viewFloat",
                						"params": {
                							"showSymbol": false,
                							"precision": 2,
                							"thousands": ".",
                							"decimal": ","
                						}
                					},
                					"showOnList": true
                				}
                			],
                			"valueField": "DSNOTAFISCORIG",
                			"descriptionField": "DSNOTAFISCORIG",
                			"outData": {
                				"CDSERIE": "CDSERIEORIG",
                				"NRNOTAFISC": "NRNOTAFISCORIG",
                				"DSNOTAFISCORIG": "DSNOTAFISCORIG",
                				"PKNOTAFISCORIG": "PKNOTAFISCORIG"
                			},
                			"events": [],
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			}
                		},
                		{
                			"id": "9561059822864237925563",
                			"name": "NMRAZSOCTRAN",
                			"label": "Transportadora",
                			"fieldGroup": "transportadora",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"showOnForm": true,
                			"showOnList": false,
                			"showOnView": true,
                			"sequenceInForm": 200,
                			"readOnly": false,
                			"defaultValue": null,
                			"template": "zh-select-autocomplete#field/select-autocomplete.html",
                			"searchableFields": [
                				"NRINSJURTRAN",
                				"NMRAZSOCTRAN"
                			],
                			"displayedFields": [
                				"NRINSJURTRAN",
                				"NMRAZSOCTRAN"
                			],
                			"descriptionField": "NMRAZSOCTRAN",
                			"valueField": "NRINSJURTRAN",
                			"showSelectAs": "grid",
                			"class": "9",
                			"validations": {},
                			"newLine": true,
                			"dataSource": {
                				"rest": true,
                				"name": "df#/crud/transport_municipio",
                				"lazyLoad": true,
                				"itemsPerPage": 100
                			},
                			"dataSourceFilter": [{
                				"name": "P_TODOS",
                				"operator": "=",
                				"value": "T"
                			}],
                			"outData": {
                				"NMRAZSOCTRAN": "NMRAZSOCTRAN",
                				"NRINSJURTRAN": "NRINSJURTRAN",
                				"DSENDETRAN": "DSENDETRAN",
                				"NMMUNICIPIO": "NMMUNICIPIOTRAN",
                				"SGESTADO": "SGESTADOTRAN",
                				"NRINSESTTRAN": "NRINSESTTRAN",
                				"IDTPIJURTRAN": "IDTPIJURTRAN"
                			},
                			"fields": [{
                					"id": "9561059823230748608564",
                					"label": "Inscrição",
                					"name": "NRINSJURTRAN",
                					"template": "field/text-edit.html",
                					"showOnList": true
                				},
                				{
                					"id": "9561059821837399028565",
                					"label": "Nome",
                					"name": "NMRAZSOCTRAN",
                					"template": "field/text-edit.html",
                					"showOnList": true
                				}
                			],
                			"events": []
                		},
                		{
                			"id": "956105982724363521566",
                			"name": "NRINSESTTRAN",
                			"label": "Inscrição Estadual",
                			"fieldGroup": "transportadora",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 210,
                			"class": "3",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059823020499851567",
                			"name": "DSENDETRAN",
                			"label": "Endereço",
                			"fieldGroup": "transportadora",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 220,
                			"class": "5",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059824212074286568",
                			"name": "NMMUNICIPIOTRAN",
                			"label": "Município",
                			"fieldGroup": "transportadora",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 230,
                			"class": "5",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059821683668178569",
                			"name": "SGESTADOTRAN",
                			"label": "UF",
                			"fieldGroup": "transportadora",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 240,
                			"class": "2",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "956105982107674097570",
                			"name": "IDPGTOFRETE",
                			"label": "Frete",
                			"fieldGroup": "transportadora",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 250,
                			"class": "6",
                			"readOnly": false,
                			"defaultValue": null,
                			"template": "field/select-native.html",
                			"hideEmptyValue": true,
                			"dataSource": {
                				"data": [{
                						"IDPGTOFRETE": "0",
                						"DSPGTOFRETE": "Contratação do Frete Por Conta do Emitente"
                					},
                					{
                						"IDPGTOFRETE": "1",
                						"DSPGTOFRETE": "Contratação do Frete Por Conta do Destinatário/Remetente"
                					},
                					{
                						"IDPGTOFRETE": "2",
                						"DSPGTOFRETE": "Contratação do Frete Por Conta de Terceiros"
                					},
                					{
                						"IDPGTOFRETE": "3",
                						"DSPGTOFRETE": "Transporte Próprio por conta do Remetente"
                					},
                					{
                						"IDPGTOFRETE": "4",
                						"DSPGTOFRETE": "Transporte Próprio por conta do Destinatário"
                					},
                					{
                						"IDPGTOFRETE": "9",
                						"DSPGTOFRETE": "Sem frete"
                					}
                				]
                			},
                			"descriptionField": "DSPGTOFRETE",
                			"valueField": "IDPGTOFRETE",
                			"outData": {
                				"IDPGTOFRETE": "IDPGTOFRETE",
                				"DSPGTOFRETE": "DSPGTOFRETE"
                			},
                			"newLine": true,
                			"events": []
                		},
                		{
                			"id": "9561059823260696412571",
                			"name": "NRPLACAVEIC",
                			"label": "Placa Veículo",
                			"fieldGroup": "transportadora",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"maxlength": 7,
                			"isVisible": true,
                			"sequenceInForm": 260,
                			"class": "4",
                			"readOnly": false,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059822629378783572",
                			"name": "SGUFVEIC",
                			"label": "UF",
                			"fieldGroup": "transportadora",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"maxlength": 15,
                			"isVisible": true,
                			"sequenceInForm": 263,
                			"class": "2",
                			"readOnly": false,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059821191225784573",
                			"name": "DSQTDEVOLU",
                			"label": "Quantidade",
                			"fieldGroup": "transportadora",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 265,
                			"class": "6",
                			"align": "right",
                			"mask": {
                				"type": "viewFloat",
                				"params": {
                					"showSymbol": false,
                					"precision": 2,
                					"thousands": ".",
                					"decimal": ",",
                					"allowNegative": false
                				}
                			},
                			"readOnly": false,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059822784935940574",
                			"name": "DSESPECVOLU",
                			"label": "Espécie",
                			"fieldGroup": "transportadora",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 270,
                			"class": "6",
                			"readOnly": false,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059823592477374575",
                			"name": "DSMARCAVOLU",
                			"label": "Marca",
                			"fieldGroup": "transportadora",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 275,
                			"class": "6",
                			"readOnly": false,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059822253782858576",
                			"name": "DSNUMEVOLU",
                			"label": "Número",
                			"fieldGroup": "transportadora",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 280,
                			"class": "6",
                			"readOnly": false,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059823123096977577",
                			"name": "VRPBRUTOVOLU",
                			"label": "Peso bruto",
                			"fieldGroup": "transportadora",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 285,
                			"class": "6",
                			"align": "right",
                			"mask": {
                				"type": "viewFloat",
                				"params": {
                					"showSymbol": false,
                					"precision": 2,
                					"thousands": ".",
                					"decimal": ",",
                					"allowNegative": false
                				}
                			},
                			"readOnly": false,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059821661831935578",
                			"name": "VRPLIQUVOLU",
                			"label": "Peso líquido",
                			"fieldGroup": "transportadora",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 290,
                			"class": "6",
                			"align": "right",
                			"mask": {
                				"type": "viewFloat",
                				"params": {
                					"showSymbol": false,
                					"precision": 2,
                					"thousands": ".",
                					"decimal": ",",
                					"allowNegative": false
                				}
                			},
                			"readOnly": false,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059822908903780579",
                			"name": "DSENDEDEST",
                			"label": "Endereço",
                			"fieldGroup": "destinatario",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 300,
                			"class": "6",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059821714885626580",
                			"name": "NMBAIRDEST",
                			"label": "Bairro/Distrito",
                			"fieldGroup": "destinatario",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 305,
                			"class": "3",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059823241646819581",
                			"name": "NRCEPDEST",
                			"label": "CEP",
                			"fieldGroup": "destinatario",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 310,
                			"class": "3",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059821561630582582",
                			"name": "NMMUNICIPIODEST",
                			"label": "Município",
                			"fieldGroup": "destinatario",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 315,
                			"class": "4",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059823830367094583",
                			"name": "NRTELEDEST",
                			"label": "Telefone",
                			"fieldGroup": "destinatario",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 320,
                			"class": "3",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059821463488359584",
                			"name": "SGESTADODEST",
                			"label": "UF",
                			"fieldGroup": "destinatario",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 325,
                			"class": "2",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059822890558437585",
                			"name": "IEDEST",
                			"label": "Inscrição Estadual",
                			"fieldGroup": "destinatario",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 330,
                			"class": "3",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "956105982563324342586",
                			"name": "DSENDEREM",
                			"label": "Endereço",
                			"fieldGroup": "remetente",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 400,
                			"class": "6",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "956105982468895483587",
                			"name": "NMBAIRREM",
                			"label": "Bairro/Distrito",
                			"fieldGroup": "remetente",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 405,
                			"class": "3",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059821775829509588",
                			"name": "NRCEPREM",
                			"label": "CEP",
                			"fieldGroup": "remetente",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 410,
                			"class": "3",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059823047912191589",
                			"name": "NMMUNICIPIOREM",
                			"label": "Município",
                			"fieldGroup": "remetente",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 415,
                			"class": "4",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059821052269984590",
                			"name": "NRTELEREM",
                			"label": "Telefone",
                			"fieldGroup": "remetente",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 420,
                			"class": "3",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059821224041764591",
                			"name": "SGESTADOREM",
                			"label": "UF",
                			"fieldGroup": "remetente",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 425,
                			"class": "2",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"id": "9561059823123702746592",
                			"name": "IEREM",
                			"label": "Inscrição Estadual",
                			"fieldGroup": "remetente",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 430,
                			"class": "3",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"name": "NRINSJURNF",
                			"label": "CNPJ",
                			"fieldGroup": "remetente",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 435,
                			"class": "3",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		},
                		{
                			"name": "NMRAZSOCCLIE",
                			"label": "Razão Social",
                			"fieldGroup": "remetente",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"sequenceInForm": 440,
                			"isVisible": false,
                			"class": "9",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		}, 
                		{
                			"name": "NMRAZSOCFORN",
                			"label": "Razão Social",
                			"fieldGroup": "remetente",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": true,
                			"sequenceInForm": 440,
                			"class": "9",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		}, 
                		{
                			"name": "NMRAZSOCTRAN",
                			"label": "Razão Social",
                			"fieldGroup": "remetente",
                			"filterProperties": {
                				"isVisible": false,
                				"showOnForm": false
                			},
                			"isVisible": false,
                			"sequenceInForm": 440,
                			"class": "9",
                			"readOnly": true,
                			"defaultValue": null,
                			"template": "field/text-edit.html",
                			"newLine": true,
                			"dataSource": {},
                			"events": []
                		}
                	],

                	"widgets": []
                },
                {
                    "id": "9561059824031218464605",
                    "name": "itens",
                    "label": "Produtos",
                    "active": false,
                    "showOnForm": true,
                    "showAsTab": true,
                    "isVisible": true,
                    "template": "widget/master_detail/grid.html",
                    "itemsInfCompl": [],
                    "actions": [
                        {
                            "id": "9561059821412333211639",
                            "name": "actVoltarItensList",
                            "label": "Voltar",
                            "isVisible": true,
                            "showAsAction": "back_list",
                            "events": [
                                {
                                    "id": "9561059823346714147640",
                                    "name": "ActionEvent",
                                    "code": "Df15200EspelhoNotaFiscal_Controller.voltarWidgetPai(args);"
                                }
                            ]
                        },
                        {
                            "id": "9561059821391179849641",
                            "name": "actVoltarItensView",
                            "label": "Voltar",
                            "isVisible": true,
                            "showAsAction": "back_view",
                            "events": [
                                {
                                    "id": "9561059821187395334642",
                                    "name": "ActionEvent",
                                    "code": "General.backOrCancel(args);"
                                }
                            ]
                        }
                    ],
                    "floatingControl": {
                        "visibleModes": [
                            "list"
                        ]
                    },
                    "events": [],
                    "dataSource": {
                        "localStorage": true,
                        "name": "df#/lancamento_nota_itens"
                    },
                    "order": "CDARVPROD",
                    "fields": [
                        {
                            "id": "9561059824227961131615",
                            "name": "NRSEQITPROD",
                            "label": "Sequencial do Item",
                            "isVisible": false,
                            "readOnly": true,
                            "validations": {},
                            "defaultValue": null,
                            "template": "field/text-edit.html",
                            "mask": {
                                "type": "zerofill",
                                "maxlength": "4"
                            },
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "id": "9561059822157838786616",
                            "name": "CDARVPROD",
                            "label": "Código",
                            "isVisible": false,
                            "readOnly": false,
                            "class": "4",
                            "showOnForm": true,
                            "showOnList": true,
                            "showOnView": true,
                            "validations": {
                                "required": true
                            },
                            "defaultValue": null,
                            "template": "field/text-edit.html",
                            "showSelectAs": "grid",
                            "newLine": true
                        },
                        {
                            "id": "9561059823553653285617",
                            "name": "CDPRODESTO",
                            "label": "codprodesto",
                            "isVisible": false,
                            "readOnly": false,
                            "class": "4",
                            "showOnForm": true,
                            "showOnList": true,
                            "showOnView": true,
                            "validations": {
                                "required": true
                            },
                            "defaultValue": null,
                            "template": "field/text-edit.html",
                            "showSelectAs": "grid",
                            "newLine": true
                        },
                        {
                            "id": "9561059821174447206618",
                            "name": "IDCNTRESTOQ",
                            "label": "contrEstoq",
                            "isVisible": false,
                            "readOnly": false,
                            "class": "4",
                            "showOnForm": true,
                            "showOnList": true,
                            "showOnView": true,
                            "validations": {
                                "required": true
                            },
                            "defaultValue": null,
                            "template": "field/text-edit.html",
                            "showSelectAs": "grid",
                            "newLine": true
                        },
                        {
                            "id": "9561059823577571310619",
                            "name": "NMPRODUTO",
                            "label": "Produto",
                            "isVisible": true,
                            "readOnly": false,
                            "readOnlyOnEdit": true,
                            "class": "10",
                            "showOnForm": true,
                            "showOnList": true,
                            "showOnView": true,
                            "descriptionOrder": 1,
                            "validations": {
                                "required": true
                            },
                            "defaultValue": null,
                            "template": "zh-select-autocomplete#field/select-autocomplete.html",
                            "searchableFields": [
                                "CDARVPROD",
                                "NMPRODUTO"
                            ],
                            "displayedFields": [
                                "CDARVPROD",
                                "NMPRODUTO"
                            ],
                            "descriptionField": "NMPRODUTO",
                            "valueField": "CDARVPROD",
                            "newLine": true,
                            "showSelectAs": "grid",
                            "dataSource": {
                                "rest": true,
                                "name": "df#/crud/prodfili_produto",
                                "itemsPerPage": 50
                            },
                            "order": "CDARVPROD",
                            "dataSourceFilter": [
                                {
                                    "name": "CDFILIAL",
                                    "operator": "=",
                                    "value": "FILIAL_LOGADA"
                                },
                                {
                                    "name": "CDNVPRODUTO",
                                    "operator": "=",
                                    "value": "@Dfe11000_LancamentoEntrada_Controller.nivelCompletoProduto();@"
                                }
                            ],
                            "fields": [
                                {
                                    "id": "9561059823194724946620",
                                    "label": "Código",
                                    "name": "CDARVPROD",
                                    "showOnList": true
                                },
                                {
                                    "id": "9561059822281026630621",
                                    "label": "Nome",
                                    "name": "NMPRODUTO",
                                    "showOnList": true
                                }
                            ],
                            "outData": {
                                "CDPRODUTO": "CDPRODUTO",
                                "CDARVPROD": "CDARVPROD",
                                "NMPRODUTO": "NMPRODUTO",
                                "SGUNIDADE": "SGUNIDADE",
                                "VRFATOCONV": "VRFATOCONV",
                                "CDPRODESTO": "CDPRODESTO",
                                "IDCNTRESTOQ": "IDCNTRESTOQ"
                            },
                            "events": []
                        },
                        {
                            "id": "956105982743582595624",
                            "name": "SGUNIDADE",
                            "label": "UN",
                            "isVisible": true,
                            "readOnly": true,
                            "class": "2",
                            "showOnForm": true,
                            "showOnList": true,
                            "defaultValue": null,
                            "template": "field/text-edit.html",
                            "align": "center",
                            "newLine": false,
                            "sameLine": false,
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "id": "956105982607105342625",
                            "name": "VRFATOCONV",
                            "label": "Fator de Conversão",
                            "isVisible": false,
                            "readOnly": true,
                            "validations": {
                                "required": true
                            },
                            "defaultValue": null,
                            "template": "field/text-edit.html",
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "id": "9561059823253601078626",
                            "name": "CDPRODEMIS",
                            "label": "Código Emissão",
                            "isVisible": true,
                            "readOnly": false,
                            "class": "6",
                            "showOnForm": true,
                            "showOnList": true,
                            "validations": {
                                "required": true
                            },
                            "mask": {
                                "type": "fix",
                                "params": {
                                    "mask": "@getLocalVar('GER_CDPICTPROD')@"
                                }
                            },
                            "defaultValue": null,
                            "template": "field/text-edit.html",
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "id": "9561059823957252606627",
                            "name": "DSPRODEMIS",
                            "label": "Descrição Emissão",
                            "isVisible": true,
                            "readOnly": false,
                            "class": "12",
                            "showOnForm": true,
                            "showOnList": true,
                            "validations": {
                                "required": true
                            },
                            "defaultValue": null,
                            "template": "field/textarea.html",
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "id": "9561059821022390813628",
                            "name": "QTITEMORIG",
                            "label": "Quantidade Origem",
                            "isVisible": false,
                            "readOnly": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "defaultValue": null,
                            "template": "field/text-edit.html",
                            "mask": {
                                "type": "viewFloat",
                                "params": {
                                    "showSymbol": false,
                                    "precision": 3,
                                    "thousands": ".",
                                    "decimal": ",",
                                    "allowNegative": false
                                }
                            },
                            "align": "right",
                            "newLine": true,
                            "dataSource": {}
                        },
                        {
                            "id": "9561059822822592840629",
                            "name": "QTITEMPROD",
                            "label": "Quantidade",
                            "isVisible": true,
                            "readOnly": false,
                            "class": "6",
                            "showOnForm": true,
                            "showOnList": true,
                            "defaultValue": null,
                            "template": "field/text-edit.html",
                            "validations": {
                                "required": true
                            },
                            "mask": {
                                "type": "viewFloat",
                                "params": {
                                    "showSymbol": false,
                                    "precision": 3,
                                    "thousands": ".",
                                    "decimal": ",",
                                    "allowNegative": false
                                }
                            },
                            "align": "right",
                            "newLine": true,
                            "dataSource": {},
                            "events": [],
                            "modify": false
                        },
                        {
                            "id": "9561059822359790862632",
                            "name": "VRUNITITEM",
                            "label": "Vr. Unitário",
                            "isVisible": true,
                            "readOnly": false,
                            "class": "6",
                            "showOnForm": true,
                            "showOnList": true,
                            "validations": {
                                "required": true
                            },
                            "defaultValue": null,
                            "template": "field/text-edit.html",
                            "mask": {
                                "type": "viewFloat",
                                "params": {
                                    "showSymbol": false,
                                    "precision": 3,
                                    "thousands": ".",
                                    "decimal": ",",
                                    "allowNegative": false
                                }
                            },
                            "align": "right",
                            "newLine": true,
                            "dataSource": {},
                            "events": [],
                            "modify": false
                        },
                        {
                            "id": "9561059821411742318635",
                            "name": "VRTOTAITEM",
                            "label": "Valor Total",
                            "isVisible": true,
                            "readOnly": false,
                            "class": "6",
                            "showOnForm": true,
                            "showOnList": true,
                            "validations": {
                                "required": true
                            },
                            "defaultValue": null,
                            "template": "field/text-edit.html",
                            "mask": {
                                "type": "viewFloat",
                                "params": {
                                    "showSymbol": false,
                                    "precision": 2,
                                    "thousands": ".",
                                    "decimal": ",",
                                    "allowNegative": false
                                }
                            },
                            "align": "right",
                            "newLine": true,
                            "dataSource": {},
                            "events": [],
                            "modify": false
                        },
                        {
                            "id": "956105982866526066638",
                            "name": "VRDESCITEM",
                            "label": "Desconto",
                            "isVisible": true,
                            "readOnly": false,
                            "class": "6",
                            "showOnForm": true,
                            "showOnList": true,
                            "defaultValue": 0,
                            "template": "field/text-edit.html",
                            "inlineEditable": "number",
                            "mask": {
                                "type": "viewFloat",
                                "params": {
                                    "showSymbol": false,
                                    "precision": 2,
                                    "thousands": ".",
                                    "decimal": ",",
                                    "allowNegative": false
                                }
                            },
                            "align": "right",
                            "newLine": true,
                            "dataSource": {},
                            "events": [],
                            "modify": false
                        }
                    ],
                    "widgets": [
                        {
                            "id": "9561059822343248104606",
                            "name": "impostosItem",
                            "label": "Imposto",
                            "active": false,
                            "showForm": true,
                            "isVisible": true,
                            "template": "widget/grid.html",
                            "actions": [
                                {
                                    "id": "9561059821745334090611",
                                    "name": "actVolatrItemImposto",
                                    "label": "Voltar",
                                    "icon": "back_list",
                                    "isVisible": true,
                                    "showOnForm": true,
                                    "showOnList": true,
                                    "showAsAction": "back",
                                    "events": [
                                        {
                                            "id": "9561059822613871136612",
                                            "name": "ActionEvent",
                                            "code": "General.backOrCancel(args);"
                                        }
                                    ]
                                }
                            ],
                            "floatingControl": {
                                "visibleModes": [
                                    "list"
                                ]
                            },
                            "events": [
                                {
                                    "id": "9561059823146811069614",
                                    "name": "WidgetBeforeInlineEditMoveRow",
                                    "code": "Dfe11000_LancamentoEntrada_Controller.setIdvalorinfo(args.row)"
                                }
                            ],
                            "dataSource": {
                                "localStorage": true,
                                "name": "df#/lancamento_nota_imposto_item"
                            },
                            "order": "CDIMPOSTO",
                            "dataSourceFilter": [
                                {
                                    "name": "NRSEQITPROD",
                                    "operator": "=",
                                    "value": "@General.getFieldValue('NRSEQITPROD', 'itens')@"
                                }
                            ],
                            "fields": [
                                {
                                    "id": "956105982344419804607",
                                    "name": "CDIMPOSTO",
                                    "label": "Código",
                                    "isVisible": true,
                                    "readOnly": true,
                                    "showOnForm": true,
                                    "showOnList": true,
                                    "descriptionOrder": 1,
                                    "validations": {
                                        "required": true
                                    },
                                    "newLine": true,
                                    "dataSource": {},
                                    "events": []
                                },
                                {
                                    "id": "956105982369988259608",
                                    "name": "SGIMPOSTO",
                                    "label": "Imposto",
                                    "isVisible": true,
                                    "readOnly": true,
                                    "showOnForm": true,
                                    "showOnList": true,
                                    "validations": {
                                        "required": true
                                    },
                                    "newLine": true,
                                    "dataSource": {},
                                    "events": []
                                },
                                {
                                    "name": "VRBASECALCIT",
                                    "label": "BC",
                                    "isVisible": true,
                                    "readOnly": true,
                                    "showOnForm": true,
                                    "showOnList": true,
                                    "validations": {
                                        "required": true
                                    },
                                    "newLine": true,
                                    "dataSource": {},
                                    "events": []
                                },
                                {
                                    "id": "9561059824190632576609",
                                    "name": "VRALIQIMPOIT",
                                    "label": "Valor Alíquota (%)",
                                    "isVisible": true,
                                    "readOnly": true,
                                    "showOnForm": true,
                                    "showOnList": true,
                                    "validations": {
                                        "required": true
                                    },
                                    "mask": {
                                        "type": "viewFloat",
                                        "params": {
                                            "showSymbol": false,
                                            "precision": 2,
                                            "thousands": ".",
                                            "decimal": ",",
                                            "allowNegative": false
                                        }
                                    },
                                    "events": [],
                                    "inlineEditable": "number",
                                    "align": "right",
                                    "newLine": true,
                                    "dataSource": {}
                                },
                                {
                                    "id": "9561059822947995045610",
                                    "name": "VRIMPOSTOIT",
                                    "label": "Valor",
                                    "isVisible": true,
                                    "readOnly": true,
                                    "showOnForm": true,
                                    "showOnList": true,
                                    "validations": {
                                        "required": true
                                    },
                                    "mask": {
                                        "type": "viewFloat",
                                        "params": {
                                            "showSymbol": false,
                                            "precision": 2,
                                            "thousands": ".",
                                            "decimal": ",",
                                            "allowNegative": false
                                        }
                                    },
                                    "inlineEditable": "number",
                                    "align": "right",
                                    "newLine": true,
                                    "dataSource": {},
                                    "events": []
                                }, 
                                {
                                    "name": "CDFISCALIT",
                                    "label": "CF",
                                    "isVisible": true,
                                    "readOnly": true,
                                    "showOnForm": true,
                                    "showOnList": true,
                                    "validations": {
                                        "required": true
                                    },
                                    "newLine": true,
                                    "dataSource": {},
                                    "events": []
                                },
                                {
                                    "name": "CDSITUTRIBITNFE",
                                    "label": "CST NFe",
                                    "isVisible": true,
                                    "readOnly": true,
                                    "showOnForm": true,
                                    "showOnList": true,
                                    "validations": {
                                        "required": true
                                    },
                                    "newLine": true,
                                    "dataSource": {},
                                    "events": []
                                },
                                 {
                                    "name": "VRPERREDBCIT",
                                    "label": "Red BC%",
                                    "isVisible": true,
                                    "readOnly": true,
                                    "showOnForm": true,
                                    "showOnList": true,
                                    "validations": {
                                        "required": true
                                    },
                                    "newLine": true,
                                    "dataSource": {},
                                    "events": []
                                }, 
                                 {
                                    "name": "IDMODBCITNFE",
                                    "label": "Mod. BC ICMS",
                                    "isVisible": true,
                                    "readOnly": true,
                                    "showOnForm": true,
                                    "showOnList": true,
                                    "validations": {
                                        "required": true
                                    },
                                    "newLine": true,
                                    "dataSource": {},
                                    "events": []
                                }, 
                                {
                                    "name": "IDMODBCSTITNFE",
                                    "label": "Mod. BC ICMS ST",
                                    "isVisible": true,
                                    "readOnly": true,
                                    "showOnForm": true,
                                    "showOnList": true,
                                    "validations": {
                                        "required": true
                                    },
                                    "newLine": true,
                                    "dataSource": {},
                                    "events": []
                                },
                                {
                                    "name": "CDTPCREDPICOITE",
                                    "label": "Tp. Crédito",
                                    "isVisible": true,
                                    "readOnly": true,
                                    "showOnForm": true,
                                    "showOnList": true,
                                    "validations": {
                                        "required": true
                                    },
                                    "newLine": true,
                                    "dataSource": {},
                                    "events": []
                                }, 
                                {
                                    "name": "CDNATUBCALCPCIT",
                                    "label": "Natu. BC",
                                    "isVisible": true,
                                    "readOnly": true,
                                    "showOnForm": true,
                                    "showOnList": true,
                                    "validations": {
                                        "required": true
                                    },
                                    "newLine": true,
                                    "dataSource": {},
                                    "events": []
                                }, 
                                {
                                    "name": "CDREGRAIT",
                                    "label": "Regra",
                                    "isVisible": true,
                                    "readOnly": true,
                                    "showOnForm": true,
                                    "showOnList": true,
                                    "validations": {
                                        "required": true
                                    },
                                    "newLine": true,
                                    "dataSource": {},
                                    "events": []
                                }, 
                                {
                                    "name": "CDDETALHEPROD",
                                    "label": "Cód. Detalhes",
                                    "isVisible": true,
                                    "readOnly": true,
                                    "showOnForm": true,
                                    "showOnList": true,
                                    "validations": {
                                        "required": true
                                    },
                                    "newLine": true,
                                    "dataSource": {},
                                    "events": []
                                }, 
                                {
                                    "name": "CDSIMPLESNACIONAL",
                                    "label": "Tab. Simples",
                                    "isVisible": true,
                                    "readOnly": true,
                                    "showOnForm": true,
                                    "showOnList": true,
                                    "validations": {
                                        "required": true
                                    },
                                    "newLine": true,
                                    "dataSource": {},
                                    "events": []
                                }
                                
                            ],
                            "widgets": []
                        }
                    ]
                },
                {
                    "id": "956105982968636651645",
                    "name": "impostos",
                    "label": "Cálculo Imposto",
                    "active": false,
                    "showForm": true,
                    "isVisible": true,
                    "template": "widget/grid.html",
                    "actions": [
                        {
                            "id": "9561059821049537548649",
                            "name": "actVoltarImpostos",
                            "label": "Voltar",
                            "icon": "back_list",
                            "isVisible": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "showAsAction": "back",
                            "events": [
                                {
                                    "id": "956105982893903845650",
                                    "name": "ActionEvent",
                                    "code": "Df15200EspelhoNotaFiscal_Controller.voltarWidgetPai(args);"
                                }
                            ]
                        }
                    ],
                    "floatingControl": {
                        "visibleModes": [
                            "list"
                        ]
                    },
                    "events": [
                    ],
                    "dataSource": {
                        "localStorage": true,
                        "name": "df#/lancamento_nota_imposto"
                    },
                    "order": "CDIMPOSTO",
                    "fields": [
                        {
                            "id": "9561059821470849435646",
                            "name": "CDIMPOSTO",
                            "label": "Código",
                            "isVisible": true,
                            "readOnly": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "validations": {
                                "required": true
                            },
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "id": "9561059823708419978647",
                            "name": "NMIMPOSTO",
                            "label": "Imposto",
                            "isVisible": true,
                            "readOnly": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "validations": {
                                "required": true
                            },
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "name": "VRCONTABILNF",
                            "label": "Vr. Contábil",
                            "isVisible": true,
                            "readOnly": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "validations": {
                                "required": true
                            },
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "name": "VRBASECALCNF",
                            "label": "B. Cálculo",
                            "isVisible": true,
                            "readOnly": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "validations": {
                                "required": true
                            },
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "name": "VRALIQNF",
                            "label": "Alíq.(%)",
                            "isVisible": true,
                            "readOnly": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "validations": {
                                "required": true
                            },
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "id": "9561059823020542239648",
                            "name": "VRIMPOSTONF",
                            "label": "Valor",
                            "isVisible": true,
                            "readOnly": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "validations": {
                                "required": true
                            },
                            
                            "mask": {
                                "type": "viewFloat",
                                "params": {
                                    "showSymbol": false,
                                    "precision": 2,
                                    "thousands": ".",
                                    "decimal": ",",
                                    "allowNegative": false
                                }
                            },
                            "align": "right",
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "name": "CDFISCOPERNF",
                            "label": "CFOP",
                            "isVisible": true,
                            "readOnly": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "validations": {
                                "required": true
                            },
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "name": "CDSUFXCFOPNF",
                            "label": "ID",
                            "isVisible": true,
                            "readOnly": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "validations": {
                                "required": true
                            },
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        }, 
                        {
                            "name": "CDSITUTRIBNF",
                            "label": "CST",
                            "isVisible": true,
                            "readOnly": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "validations": {
                                "required": true
                            },
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        }, 
                         {
                            "name": "CDFISCALNF",
                            "label": "CF",
                            "isVisible": true,
                            "readOnly": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "validations": {
                                "required": true
                            },
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        }, 
                         {
                            "name": "DSOBSFISCIT",
                            "label": "Observação",
                            "isVisible": true,
                            "readOnly": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "validations": {
                                "required": true
                            },
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        }
                    ],
                    "widgets": []
                },
                {
                    "id": "9561059823889356817652",
                    "name": "encargos",
                    "label": "Encargo",
                    "active": false,
                    "showForm": true,
                    "isVisible": true,
                    "template": "widget/grid.html",
                    "actions": [
                        {
                            "id": "9561059821893206202657",
                            "name": "actVoltarEncargo",
                            "label": "Voltar",
                            "isVisible": true,
                            "showAsAction": "back_list|back_view",
                            "events": [
                                {
                                    "id": "9561059822465281860658",
                                    "name": "ActionEvent",
                                    "code": "Df15200EspelhoNotaFiscal_Controller.voltarWidgetPai(args);"
                                }
                            ]
                        }
                    ],
                    "floatingControl": {
                        "visibleModes": [
                            "list"
                        ]
                    },
                    "events": [
                        {
                            "id": "9561059824048216960659",
                            "name": "WidgetOnEnter",
                            "code": "Dfe11000_LancamentoEntrada_Controller.setDtsEncargoOnEnter(args);"
                        }
                    ],
                    "dataSource": {
                        "localStorage": true,
                        "name": "df#/lancamento_nota_encargo"
                    },
                    "order": "CDENCARGO",
                    "fields": [
                        {
                            "id": "9561059823836090445653",
                            "name": "CDENCARGO",
                            "label": "Código",
                            "isVisible": true,
                            "readOnly": true,
                            "showOnForm": true,
                            "showOnList": false,
                            "validations": {
                                "required": true
                            },
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "id": "9561059821181763360654",
                            "name": "NMENCARGO",
                            "label": "Encargo",
                            "isVisible": true,
                            "readOnly": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "validations": {
                                "required": true
                            },
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "id": "9561059822267349012655",
                            "name": "VRITEMENCA",
                            "label": "Valor",
                            "isVisible": true,
                            "readOnly": false,
                            "showOnForm": true,
                            "showOnList": true,
                            "validations": {
                                "required": true
                            },
                            "mask": {
                                "type": "viewFloat",
                                "params": {
                                    "showSymbol": false,
                                    "precision": 2,
                                    "thousands": ".",
                                    "decimal": ",",
                                    "allowNegative": false
                                }
                            },
                            "align": "right",
                            "inlineEditable": false,
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        }
                    ],
                    "widgets": []
                },
                {
                    "id": "9561059823971562300662",
                    "name": "vencimentos",
                    "label": "Vencimento",
                    "active": false,
                    "showForm": true,
                    "isVisible": true,
                    "template": "widget/master_detail/grid.html",
                    "reloadOnEnterTab": true,
                    "actions": [
                        {
                            "id": "9561059822157274916674",
                            "name": "actVoltarVencimentoList",
                            "label": "Voltar",
                            "isVisible": true,
                            "showAsAction": "back_list",
                            "events": [
                                {
                                    "id": "9561059822212207918675",
                                    "name": "ActionEvent",
                                    "code": "Df15200EspelhoNotaFiscal_Controller.voltarWidgetPai(args);"
                                }
                            ]
                        },
                        {
                            "id": "956105982736492246676",
                            "name": "actVoltarVencimentoView",
                            "label": "Voltar",
                            "isVisible": true,
                            "showAsAction": "back_view",
                            "events": [
                                {
                                    "id": "9561059822070128110677",
                                    "name": "ActionEvent",
                                    "code": "General.backOrCancel(args);"
                                }
                            ]
                        }
                    ],
                    "floatingControl": {
                        "visibleModes": [
                            "list"
                        ]
                    },
                    "events": [
                        {
                            "name": "WidgetOnEnter",
                            "code": "Df11000_Lancamento_Service.validaStatusVencimento(args.owner);"
                        }
                        ],
                    "dataSource": {
                        "localStorage": true,
                        "name": "df#/lancamento_nota_vencimento"
                    },
                    "order": "DTATUAVENPAG",
                    "fields": [
                        {
                            "id": "9561059821979300130663",
                            "name": "NRSEQVENPAG",
                            "label": "Sequencial Vencimento",
                            "isVisible": false,
                            "showOnForm": false,
                            "showOnView": false,
                            "showOnList": false,
                            "readOnly": true,
                            "template": "field/text-edit.html",
                            "validations": {
                                "required": true
                            },
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "id": "9561059822647082422664",
                            "name": "QTDIASVENPAG",
                            "label": "Nº de Dias",
                            "isVisible": true,
                            "showOnForm": true,
                            "showOnList": false,
                            "readOnly": false,
                            "readOnlyOnEdit": false,
                            "template": "field/number.html",
                            "class": "6",
                            "validations": {
                                "required": true,
                                "maxLength": {
                                    "length": 3
                                }
                            },
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "id": "9561059823158043017666",
                            "name": "DTATUAVENPAG",
                            "label": "Vencimento",
                            "isVisible": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "readOnly": false,
                            "readOnlyOnEdit": false,
                            "sequenceInList": "200",
                            "template": "field/calendar.html",
                            "descriptionOrder": 1,
                            "class": "6",
                            "dateFormat": "DD/MM/YYYY",
                            "displayFormat": "DD/MM/YYYY",
                            "mask": {
                                "type": "date",
                                "params": {
                                    "format": "DD/MM/YYYY"
                                }
                            },
                            "validations": {
                                "required": true
                            },
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "id": "9561059821924240393668",
                            "name": "VRATUAPAG",
                            "label": "Valor",
                            "isVisible": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "readOnly": false,
                            "readOnlyOnEdit": false,
                            "sequenceInList": "300",
                            "template": "field/text-edit.html",
                            "descriptionOrder": 2,
                            "class": "6",
                            "validations": {
                                "required": true
                            },
                            "mask": {
                                "type": "viewFloat",
                                "params": {
                                    "showSymbol": false,
                                    "precision": 2,
                                    "thousands": ".",
                                    "decimal": ",",
                                    "allowNegative": false
                                }
                            },
                            "align": "right",
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "id": "9561059821128724635669",
                            "name": "IDSTATPAG",
                            "label": "Status",
                            "readOnly": true,
                            "readOnlyOnEdit": true,
                            "isVisible": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "sequenceInList": "100",
                            "validations": {
                                "required": true
                            },
                            "defaultValue": "A",
                            "template": "field/select-native.html",
                            "class": "6",
                            "hideEmptyValue": true,
                            "newLine": false,
                            "sameLine": false,
                            "dataSource": {
                                "data": [
                                    {
                                        "IDSTATPAG": "A",
                                        "DSSTATPAG": "Em Aberto"
                                    },
                                    {
                                        "IDSTATPAG": "B",
                                        "DSSTATPAG": "Baixado"
                                    }
                                ]
                            },
                            "options": {
                                "A": "Em Aberto",
                                "B": "Baixado"
                            },
                            "valueField": "IDSTATPAG",
                            "descriptionField": "DSSTATPAG",
                            "events": []
                        },
                        {
                            "id": "9561059824059473037670",
                            "name": "DSTPCTPAGAR",
                            "label": "Tipo de Conta Pagar",
                            "readOnly": false,
                            "isVisible": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "showOnView": true,
                            "sequenceInList": "400",
                            "validations": {},
                            "defaultValue": null,
                            "template": "zh-select-autocomplete#field/select-autocomplete.html",
                            "searchableFields": [
                                "CDTPCTPAGAR",
                                "DSTPCTPAGAR"
                            ],
                            "displayedFields": [
                                "CDTPCTPAGAR",
                                "DSTPCTPAGAR"
                            ],
                            "class": "6",
                            "showSelectAs": "grid",
                            "dataSource": {
                                "rest": true,
                                "name": "df#/crud/tipoctpagar"
                            },
                            "order": "CDTPCTPAGAR",
                            "dataSourceFilter": [],
                            "descriptionField": "DSTPCTPAGAR",
                            "valueField": "CDTPCTPAGAR",
                            "outData": {
                                "CDTPCTPAGAR": "CDTPCTPAGAR",
                                "DSTPCTPAGAR": "DSTPCTPAGAR"
                            },
                            "fields": [
                                {
                                    "id": "9561059822319718050671",
                                    "label": "Código",
                                    "name": "CDTPCTPAGAR",
                                    "showOnList": true
                                },
                                {
                                    "id": "9561059822491528886672",
                                    "label": "Descrição",
                                    "name": "DSTPCTPAGAR",
                                    "showOnList": true
                                }
                            ],
                            "events": []
                        },
                        {
                            "id": "9561059823901052335673",
                            "name": "DSCONTAPAG",
                            "label": "Descrição",
                            "isVisible": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "readOnly": false,
                            "sequenceInList": "500",
                            "template": "field/textarea.html",
                            "class": "12",
                            "validations": {},
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        }
                    ],
                    "widgets": []
                },
                {
                    "id": "956105982625110706678",
                    "name": "dados_adicionais",
                    "label": "Dados Adicionais",
                    "active": false,
                    "showForm": true,
                    "isVisible": true,
                    "template": "widget/master_detail/grid.html",
                    "reloadOnEnterTab": true,
                    "actions": [
                        {
                            "id": "9561059823783977174685",
                            "name": "actVoltarDadosAdicList",
                            "label": "Voltar",
                            "isVisible": true,
                            "showAsAction": "back_list",
                            "events": [
                                {
                                    "id": "9561059821405504294686",
                                    "name": "ActionEvent",
                                    "code": "Df15200EspelhoNotaFiscal_Controller.voltarWidgetPai(args);"
                                }
                            ]
                        },
                        {
                            "id": "9561059823848070331687",
                            "name": "actVoltarDadosAdicView",
                            "label": "Voltar",
                            "isVisible": true,
                            "showAsAction": "back_view",
                            "events": [
                                {
                                    "id": "956105982996597961688",
                                    "name": "ActionEvent",
                                    "code": "General.backOrCancel(args);"
                                }
                            ]
                        }
                    ],
                    "floatingControl": {
                        "visibleModes": [
                            "list"
                        ]
                    },
                    "events": [],
                    "dataSource": {
                        "localStorage": true,
                        "name": "df#/lancamento_nota_dadosadic"
                    },
                    "order": "NRDADOADIC",
                    "fields": [
                        {
                            "id": "9561059823365897805679",
                            "name": "NRDADOADIC",
                            "label": "Sequencial",
                            "isVisible": false,
                            "showOnForm": false,
                            "showOnView": false,
                            "showOnList": false,
                            "readOnly": true,
                            "template": "field/text-edit.html",
                            "validations": {},
                            "newLine": true,
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "id": "956105982547507661680",
                            "name": "CDMENSAGEM",
                            "label": "Código",
                            "isVisible": true,
                            "class": "4",
                            "showOnForm": true,
                            "showOnView": true,
                            "showOnList": true,
                            "readOnly": false,
                            "readOnlyOnEdit": false,
                            "template": "zh-select-autocomplete#field/select-autocomplete.html",
                            "searchableFields": [
                                "CDMENSAGEM",
                                "DSMENSAGEM"
                            ],
                            "displayedFields": [
                                "CDMENSAGEM",
                                "DSMENSAGEM"
                            ],
                            "descriptionField": "DSMENSAGEM",
                            "valueField": "CDMENSAGEM",
                            "order": "CDMENSAGEM",
                            "validations": {},
                            "newLine": true,
                            "showSelectAs": "grid",
                            "dataSource": {
                                "name": "df#/crud/mensagem",
                                "rest": true
                            },
                            "fields": [
                                {
                                    "id": "9561059821099183211681",
                                    "name": "CDMENSAGEM",
                                    "label": "Código",
                                    "showOnList": true
                                },
                                {
                                    "id": "9561059822829798560682",
                                    "name": "DSMENSAGEM",
                                    "label": "Descrição",
                                    "showOnList": true
                                }
                            ],
                            "outData": {
                                "CDMENSAGEM": "CDMENSAGEM",
                                "DSMENSAGEM": "DSDADOADIC"
                            },
                            "events": []
                        },
                        {
                            "id": "9561059822181831454683",
                            "name": "DSDADOADIC",
                            "label": "Mensagem",
                            "isVisible": true,
                            "class": "8",
                            "showOnForm": true,
                            "showOnView": true,
                            "showOnList": true,
                            "readOnly": false,
                            "readOnlyOnEdit": false,
                            "template": "field/text-edit.html",
                            "validations": {
                                "required": {
                                    "message": "Obrigatório"
                                }
                            },
                            "newLine": true,
                            "dataSource": {},
                            "events": [
                                {
                                    "id": "956105982372742865684",
                                    "name": "FieldOnChange",
                                    "code": "Dfe11000_LancamentoEntrada_Controller.limpaCampoCodigoDadosAdic(args);"
                                }
                            ]
                        }
                    ],
                    "widgets": []
                }
            ],
            "id": "4889913822747347216513"
        };
          
      };  
      


};

Configuration(function(ContextRegister) {
    ContextRegister.register('Df15200EspelhoNotaFiscal_Controller', Df15200EspelhoNotaFiscal_Controller);
});

// FILE: ./assets/js/utilities/Lib00101_Contabilizacao_Controller.js
var Lib00101_Contabilizacao_Controller = function(contabilizacaoService, ZHPromise, ScreenService, templateManager, maskEngine, ZhFilterPreferences, engine) {

    /********************************************************************
     *                                                                  *
     *                          Funções gerais                          *
     *                                                                  *
     ********************************************************************/

    /*
     * Função para validação de erros de contabilização, deve ser chamada
     * em caso de erro de uma requisição de lançamento que utilize contabilização.
     * Caso tenha ocorrido algum erro, os dados são salvos localmente
     * e uma mensagem é exibida para o usuário.
     * @return Boolean
     *   Retorna true se não houve problemas de contabilização, false caso contrário.
     */
    this.checkErrosContabilizacao = function(errorResponse, errorMessage) {
        // Variável que retorna se a contabilização não ocasionou o erro.
        var isValid = true;
        // Verifica se houve uma resposta da requisição e se há algum dataset associado.
        if (errorResponse && errorResponse.data.dataset) {
            var dataSets = errorResponse.data.dataset;
            // Verificar se o dataset em questão é válido.
            // Caso a requisição retorne um array vazio, não entrará no if.
            if (!Util.isEmptyOrBlank(dataSets.errosContabilizacao)) {
                // Inicializa a mensagem com o valor padrão, caso nenhuma
                // mensagem seja passada como parâmetro.
                errorMessage = errorMessage || "Não foi possível realizar o lançamento devido a erros de contabilização.";
                // A variável de controle é alterada, pois o dataSet de erros não está vazio.
                isValid = false;
                // Array com as promises que devem ser executadas antes de abrir a tela de contabilização.
                var promises = [
                    // Seta os dataSets localmente.
                    contabilizacaoService.setLocalData(dataSets),
                    // Exibe a mensagem pro usuário informando que há erros no lançamento.
                    ScreenService.showMessage(errorMessage)
                ];
                // Executa todas as promises do array.
                ZHPromise.all(promises).then(function() {
                    // Abre a tela de contabilização.
                    engine.openWindow("df#lib00101_contabilizacao");
                });
            }
        }
        return isValid;
    };

    /*
     * Função para exibição da contabilização de lançamentos, abrindo uma tela
     * para a exibição dos dados.
     * Recebe como parâmetro um objeto ou um array de objetos, onde cada objeto
     * tem as propriedades CDEMPRESA e NRLANCCONT, indicando o código da
     * empresa e o número do lançamento, respectivamente.
     */
    this.exibeContabilizacao = function(lancamentos) {
        var isMultiplosLancamentos = true;
        // Verifica se foi enviado um lançamento somente, colocando-o em um
        // array.
        if (Util.isObject(lancamentos)) {
            lancamentos = [lancamentos];
            isMultiplosLancamentos = false;
        }
        contabilizacaoService.getContabilizacaoLancamentos(lancamentos)
        .then(function(response) {
            if (response && response.dataset) {
                var dataSets = response.dataset;
                // Verificar se o dataset em questão é válido.
                // Verifica se a requisição retorna partidas da contabilização.
                if (!Util.isEmptyOrBlank(dataSets.partidasContabilizacao)) {
                    // Salva os dados retornados e abre a tela de contabilização.
                    contabilizacaoService.setLocalData(dataSets).then(function() {
                        engine.openWindow("df#lib00101_contabilizacao");
                    });
                }
                else {
                    // Exibe mensagem de que não há contabilização para o(s)
                    // lançamento(s) selecionado(s).
                    var noDataMessage = "Não há contabilização para " + (isMultiplosLancamentos ? "os lançamentos selecionados." : "o lançamento selecionado.");
                    ScreenService.showMessage(noDataMessage);
                }
            }
        });
    };
    
    /*
     * Função para contabilizar e  exibir a contabilização de lançamentos, abrindo uma tela
     * para a exibição dos dados.
     * Recebe como parâmetro um objeto ou um array de objetos, onde cada objeto
     * tem as propriedades CDEMPRESA e NRLANCCONT, indicando o código da
     * empresa e o número do lançamento, respectivamente.
     */
    this.contabilizaEexibeContabilizacao = function(lancamentos) {
        var isMultiplosLancamentos = true;
        // Verifica se foi enviado um lançamento somente, colocando-o em um
        // array.
        if (Util.isObject(lancamentos)) {
            lancamentos = [lancamentos];
            isMultiplosLancamentos = false;
        }
        contabilizacaoService.getContabilizacaoLancamentos(lancamentos)
        .then( response => {
            if (response && response.dataset) {
                var dataSets = response.dataset;
                // Verificar se o dataset em questão é válido.
                // Verifica se a requisição retorna partidas da contabilização.
                if (!Util.isEmptyOrBlank(dataSets.partidasContabilizacao)) {
                    // Salva os dados retornados e abre a tela de contabilização.
                    contabilizacaoService.setLocalData(dataSets).then( () => {
                        engine.openWindow("df#lib00101_contabilizacao");
                    });
                }
                else {
                    contabilizacaoService.contabilizaLancamento(lancamentos).then( () => {
                        engine.openWindow("df#lib00101_contabilizacao");
                    });
                }
            }
        });
    };


    /********************************************************************
     *                                                                  *
     *                          Funções da tela                         *
     *                                                                  *
     ********************************************************************/
    // Variável para identificação se a tela está aberta para exibição de erros.
    var isErrorRoutine = true;
    // Variáveis de armazenamento de empresa e número de lançamento para o filtro.
    var cdempresa = "";
    var nrlancctbtemp = "";
    // Variável contendo os códigos sequenciais das partidas inválidas.
    // Utilizada para controlar a exibição de partidas, exibindo as inválidas com destaque.
    var partidasInvalidas = [];
    // Variável contendo os códigos sequenciais das partidas que possuem erros relacionados
    // aos seus rateios. Utilizado para o filtro na tab de rateios.
    var partidasRateiosInvalidos = [];

    this.beforeGridFilterApply = function(partidasWidget) {
        var filterWidget = partidasWidget.appliedFilter;
        cdempresa = filterWidget.currentRow.CDEMPRESA;
        nrlancctbtemp = filterWidget.currentRow.NRLANCCTBTEMP;
        checkContopectblParams(cdempresa);
    };

    /*
     * Função auxiliar para montar os dados a ser exibidos nos fields do filtro.
     */
    var buildFilterData = function(partidasWidget, partidas) {
        var empresaField = partidasWidget.getField("NMEMPRESA");
        var lancamentoField = partidasWidget.getField("NRLANCCTBTEMP");
        var empresaAux = {};
        var empresaData = [];
        var lancamentoData = [];
        // Varre todos as partidas, para buscar somente empresas distintas e
        // lançamentos distintos.
        partidas.forEach(function(partida) {
            // Verifica se a empresa da partida já foi adicionada ao objeto.
            if (!empresaAux[partida.CDEMPRESA]) {
                // Se não, adiciona, acrescentando o nome da empresa.
                empresaAux[partida.CDEMPRESA] = {NMEMPRESA: partida.NMEMPRESA};
            }
            // Verifica se o número de lançamento da partida já foi adicionado.
            if (!empresaAux[partida.CDEMPRESA][partida.NRLANCCTBTEMP]) {
                // Se não, adiciona, acrescentando informações complementares
                // e inicializando a contabilização total de créditos/débitos.
                empresaAux[partida.CDEMPRESA][partida.NRLANCCTBTEMP] = {
                    CDDOCUCONT: partida.CDDOCUCONT,
                    DTLANCCTBTEMP: partida.DTLANCCTBTEMP,
                    VRCREDLANC: partida.IDDEBCRE == 'CR' ? partida.VRLANCCTBTEMP : 0,
                    VRDEBLANC: partida.IDDEBCRE == 'CR' ? 0 : partida.VRLANCCTBTEMP
                };
            }
            else {
                // Se o lançamento já foi adicionado, adiciona o valor da
                // partida à contabilização dos valores.

                // Se a partida tiver um valor de crédito, acrescenta o valor aos créditos.
                if (partida.IDDEBCRE == 'CR') {
                    empresaAux[partida.CDEMPRESA][partida.NRLANCCTBTEMP].VRCREDLANC += partida.VRLANCCTBTEMP;
                }
                // Senão, acrescenta à contabilização de débitos.
                else {
                    empresaAux[partida.CDEMPRESA][partida.NRLANCCTBTEMP].VRDEBLANC += partida.VRLANCCTBTEMP;
                }
            }
        });

        // Varre todas as empresas distintas, adicionando ao array principal.
        Object.keys(empresaAux).forEach(function(cdempresa) {
            empresaData.push({
                CDEMPRESA: cdempresa,
                NMEMPRESA: empresaAux[cdempresa].NMEMPRESA
            });
            // Deleta o nome da empresa para restar somente os lançamentos
            // como chaves do objeto.
            delete empresaAux[cdempresa].NMEMPRESA;
            Object.keys(empresaAux[cdempresa]).forEach(function(nrlancctbtemp) {
                // Adiciona todos os lançamentos ao array principal.
                lancamentoData.push({
                    CDEMPRESA: cdempresa,
                    NRLANCCTBTEMP: nrlancctbtemp,
                    CDDOCUCONT: empresaAux[cdempresa][nrlancctbtemp].CDDOCUCONT,
                    DTLANCCTBTEMP: empresaAux[cdempresa][nrlancctbtemp].DTLANCCTBTEMP,
                    VRCREDLANC: empresaAux[cdempresa][nrlancctbtemp].VRCREDLANC,
                    VRDEBLANC: empresaAux[cdempresa][nrlancctbtemp].VRDEBLANC
               });
            });
        });

        // Seta os arrays aos dataSources dos respectivos fields.
        empresaField.dataSource.data = empresaData;
        lancamentoField.dataSource.data = lancamentoData;
    };

    /*
     * Função para alterar o filtro do field select de partidas, ao selecionar
     * os rateios que serão exibidos.
     */
    this.changeFilterPartidas = function(field) {
        var widget = field.widget;
        var partidasField = widget.getField("NRSEQCTBTEMP");
        // Seta um filtro vazio como default, caso seja selecionado todas as partidas.
        var filter = [
            {
                name: "CDEMPRESA",
                operator: "=",
                value: "@contabilizacaoController.getCdempresa()@"
            },
            {
                name: "NRLANCCTBTEMP",
                operator: "=",
                value: "@contabilizacaoController.getNrlancctbtemp()@"
            }
        ];
        // Se for selecionado o filtro das partidas com problemas de rateios, muda o filtro.
        if (field.value() === 'R') {
            filter.push({
                name: "NRSEQCTBTEMP",
                operator: "IN",
                value: partidasRateiosInvalidos
            });
        }
        // Troca o filtro e recarrega os dados do field.
        partidasField.dataSourceFilter = filter;
        partidasField.reload();
        partidasField.setValue([]);
    };

    /*
     * Função para verificar os parâmetros na tabela CONTOPECTBL de acordo com a empresa do lançamento.
     * Caso o campo IDCONTGERE não esteja setado como 'S', a tab "Rateios" não é exibida.
     * Caso o campo IDCONTCOME não esteja setado como 'S', os fields "Tipo Conta" e
     * "Código Comercial" não são exibidos.
     */
    var checkContopectblParams = function(cdempresa) {
        // Busca o código da empresa da primeira partida retornada.
        var filter = [{
            "name": "CDEMPRESA",
            "operator": "=",
            "value": cdempresa
        }];
        contabilizacaoService.getContopectblParams(filter)
        .then(function(response) {
            // Verifica se a resposta da requisição trouxe o dataSource "CONTOPECTBL".
            if (response && response.dataset && response.dataset.contopectbl) {
                // Busca a primeira linha retornada, que seria o
                // objeto da parametrização em si.
                var contopectbl = response.dataset.contopectbl[0];
                if (contopectbl) {
                    // Se o parâmetro não estiver setado, a tab de rateios é escondida.
                    var rateiosIsVisible = contopectbl.IDCONTGERE == 'S';
                    templateManager.container.getWidget("rateiosTabbed").isVisible = rateiosIsVisible;

                    // Se o parâmetro não estiver setado, os campos
                    // "Tipo Conta" e "Código Comercial" são escondidos.
                    var fieldIsVisible = contopectbl.IDCONTCOME == 'S';
                    var partidasWidget = templateManager.container.getWidget("partidas");
                    partidasWidget.getField("IDLANCCTBCOME").showOnList = fieldIsVisible;
                    partidasWidget.getField("CDLANCCTBCOME").showOnList = fieldIsVisible;
                    templateManager.updateTemplate();
                }
            }
        });
    };

    /*
     * Verifica se a rotina para a qual a tela foi aberta é a de exibição de
     * erros ou exibição de contabilidade de um lançamento já contabilizado.
     */
    this.checkExibicaoErros = function() {
        contabilizacaoService.getErros().then(function(erros) {
            // Verifica se a rotina é de erros com base nos erros retornados.
            isErrorRoutine = erros.length > 0;

            // Exibe ou esconde a action de erros conforme rotina.
            var partidasWidget = templateManager.container.getWidget("partidas");
            var errosAction = partidasWidget.actions.filter(function(action) {
                return action.name == 'erros';
            })[0];
            errosAction.isVisible = isErrorRoutine;

            var rateiosTabbedWidget = templateManager.container.getWidget("rateiosTabbed");
            var filtroRateiosField = rateiosTabbedWidget.getField("TODASPARTIDAS");
            filtroRateiosField.isVisible = isErrorRoutine;
        });
    };

    /*
     * Função para verificação de quantos lançamentos serão exibidos, para
     * verificar a necessidade do filtro.
     */
    this.checkMultiplosLancamentos = function() {
        // Busca as partidas salvas localmente.
        contabilizacaoService.getPartidas()
        .then(function(partidas) {
            cdempresa = partidas[0].CDEMPRESA;
            nrlancctbtemp = partidas[0].NRLANCCTBTEMP;
            // Verifica se há alguma partida cujo número de lançamento e empresa
            // sejam diferentes da primeira partida.
            var multiplosLancamentos = partidas.some(function(partida) {
                return partida.NRLANCCTBTEMP != nrlancctbtemp || partida.CDEMPRESA != cdempresa;
            });
            var partidasWidget = templateManager.container.getWidget("partidas");
            // Se houver mais de um lançamento, remove os valores das variáveis
            // de armazenamento de empresa e lançamento, abre o filtro e exibe
            // a action de filtrar do stripe.
            if (multiplosLancamentos) {
                cdempresa = "";
                nrlancctbtemp = "";
                buildFilterData(partidasWidget, partidas);
                templateManager.container.getWidget("stripe").actions[0].isVisible = true;
                ZhFilterPreferences.openFilterWidget(partidasWidget);
            }
            // Se não, carrega o grid principal e verifica os parâmetros
            // de contabilização da empresa do lançamento.
            else {
                partidasWidget.reload();
                checkContopectblParams(cdempresa);
            }
        });
    };

    /*
     * Função para recarregar widgets que dependem de outros valores.
     * Um caso em que é utilizada é quando há uma tabbed com fields, onde
     * sua widget só é carregada quando seus fields estão preenchidos.
     */
    this.checkReloadDependentWidget = function(currentWidget) {
        // Busca a widget filha.
        var childWidget = templateManager.container.getWidget(currentWidget.dependentWidgetName);
        // Verifica se todos os fields de que a widget filha depende estão preenchidos.
        var toReload = currentWidget.widgetDependentFields.every(function(fieldValue) {
            return !!currentWidget.currentRow[fieldValue];
        });
        // Se todos os fields estiverem preenchidos, recarrega a widget filha.
        if (toReload) {
            childWidget.reload();
        }
        // Se houver algum field não preenchido, a widget filha é resetada e fica em estado readonly.
        else {
            childWidget.dataSource.data = [];
        }
    };

    /*
     * Função chamada quando o usuário abre outra tela, ao sair desta.
     * Limpa os dataSets locais que foram setados para esta tela.
     */
    this.clearLocalData = function() {
        contabilizacaoService.clearLocalData();
    };

    /*
     * Função chamada ao alterar a empresa selecionada no filtro, caso haja
     * mais de um lançamento.
     */
    this.empresaFilterOnChange = function(filterWidget) {
        var lancamentoField = filterWidget.getField("NRLANCCTBTEMP");
        lancamentoField.setValue(null);
        if (filterWidget.currentRow.CDEMPRESA) {
            lancamentoField.readOnly = false;
            lancamentoField.reload();
        }
        else {
            lancamentoField.readOnly = true;
        }
    };

    /*
     * Função auxiliar utilizada pelos dataSourceFilters para filtrar lançamento.
     */
    this.getCdempresa = function() {
        return cdempresa;
    };

    /*
     * Função auxiliar utilizada pelos dataSourceFilters para filtrar lançamento.
     */
    this.getNrlancctbtemp = function() {
        return nrlancctbtemp;
    };

    /*
     * Função para abrir a widget contendo a lista de erros de contabilização.
     */
    this.openErrosWidget = function(widget) {
        ScreenService.openSwipe(widget.widgets[0]);
    };

    /*
     * Função para verificar se a partida em questão faz parte da lista de
     * partidas com erros. É utilizada pelas widgets de exibição de partidas
     * para destacar as partidas com erros.
     */
    this.partidaIsValid = function(nrseqctbtemp) {
        // Retorna true caso o sequencial não seja encontrado no array.
        return partidasInvalidas.indexOf(nrseqctbtemp) === -1;
    };

    /*
     * Função chamada quando o usuário abre outra tela para setar a máscara
     * de conta contábil no respectivo field.
     */
    this.setContaContabilMask = function(widget) {
        contabilizacaoService.getParamGeral().then(function(response) {
            // Verifica se a requisição foi bem-sucedida.
            if (response && response.dataset && response.dataset.paramgeral) {
                // Variável do primeiro dado da PARAMGERAL.
                var paramGeral = response.dataset.paramgeral[0];
                // Cria o objeto de máscara dos fields.
                var mask = {
                    type: "fix",
                    params: {
                    // Adicionado '?' para permitir mais caracteres em códigos inválidos.
                        mask: '?' + paramGeral.CDPICTCCTB + 'AAAAAA'
                    }
                };
                // Aplica a máscara no campo "Conta Contábil" do grid principal.
                widget.getField("CDCONTCTBL").mask = mask;

                // Busca o field de conta contábil no filtro da tab "Rateios".
                var rateiosTabbedWidget = templateManager.container.getWidget("rateiosTabbed");
                rateiosTabbedWidget.getField("NRSEQCTBTEMP").fields[1].mask = mask;
            }
        });
    };


    /*
     * Função executada ao abrir a tela para diferenciar as partidas que têm
     * erros e aquelas que têm erros de rateios.
     */
    this.setPartidasInvalidas = function() {
        // Verifica se a tela está aberta para exibição de erros de contabilização.
        if (isErrorRoutine) {
            // Recarrega os dados dos erros, conforme o lançamento filtrado.
            var errosWidget = templateManager.container.getWidget("erros");
            errosWidget.reload.then(function() {
                var data = errosWidget.dataSource.data;
                // Filtra somente os erros que estejam relacionados a uma partida.
                var errosComPartidas = data.filter(function(erro) {
                    // Retorna true caso o erro tenha um código de partida.
                    return !!erro.NRSEQCTBTEMP;
                });
                // A variável de partidas recebe somente um array com os códigos,
                // sem os objetos dos erros em si.
                partidasInvalidas = errosComPartidas.map(function(erro) {
                    // Retorna o código de cada erro.
                    return erro.NRSEQCTBTEMP;
                });

                // Esta variável recebe os códigos das partidas que tem erros relacionados
                // aos rateios. Mais especificamente as partidas dos erros de código
                // "007", "008", "009", "010" e "013".
                var codigosErrosRateios = [
                    "007",
                    "008",
                    "009",
                    "010",
                    "013"
                ];
                var errosRateios = errosComPartidas.filter(function(erro) {
                    // Retorna true caso o código do erro esteja no array.
                    return codigosErrosRateios.indexOf(erro.CDERROTEMP) !== -1;
                });
                partidasRateiosInvalidos = errosRateios.map(function(erro) {
                    // Retorna o código de cada erro.
                    return erro.NRSEQCTBTEMP;
                });
            });
        }
    };

    /*
     * Função chamada ao abrir a tela e carregar o dataSource pela primeira vez.
     * Seta os valores do cabeçalho da tela.
     */
    this.setStripeData = function(gridWidget) {
        // Como os dados do cabeçalho são os mesmos para todas as partidas,
        // a primeira partida do dataSet será utilizada como referência.
        var currentRow = gridWidget.dataSource.data[0];
        var stripeWidget = templateManager.container.getWidget("stripe");

        // Contabiliza o valor total de débitos e créditos no lançamento.
        var creditos = 0;
        var debitos = 0;
        gridWidget.dataSource.data.forEach(function(partida) {
            // Se a partida tiver um valor de crédito, acrescenta o valor aos créditos.
            if (partida.IDDEBCRE == 'CR') {
                creditos += partida.VRLANCCTBTEMP;
            }
            // Senão, acrescenta à contabilização de débitos.
            else {
                debitos += partida.VRLANCCTBTEMP;
            }
        });

        // Seta os dados na stripe.
        stripeWidget.currentRow = {};
        stripeWidget.currentRow.NMEMPRESA = currentRow.CDEMPRESA + ' - ' + currentRow.NMEMPRESA;
        stripeWidget.currentRow.CDDOCUCONT = currentRow.CDDOCUCONT;
        stripeWidget.currentRow.DTLANCCTBTEMP = currentRow.DTLANCCTBTEMP;
        stripeWidget.currentRow.VRCREDLANC = currencyMask(creditos);
        stripeWidget.currentRow.VRDEBLANC = currencyMask(debitos);
    };

    /*
     * Função temporária para colocar máscara nos valores que estão na Stripe.
     * Utilizada devido a problemas no Framework em relação às máscaras.
     * @return String
     *   Retorna o valor com a máscara do tipo currency.
     */
    var currencyMask = function(value) {
        // Verifica se é um valor válido.
        value = value || 0;
        // Cria a parametrização da máscara.
        var params = {
            type: "currency",
            params: {
                showSymbol: false,
                precision: 2,
                thousands: ".",
                decimal: ","
            }
        };
        // Aplica a máscara no valor.
        return Util.applyMask(value, params, maskEngine);
    };
};

Configuration(function (ContextRegister){
 ContextRegister.register('contabilizacaoController', Lib00101_Contabilizacao_Controller);
});

// FILE: ./assets/js/utilities/Lib00206_InfComplProduto_Controller.js
var Lib00206_InfComplProduto_Controller = function(ScreenService,ZHPromise, templateManager,metaDataFactory,Lib00206_InfComplProduto_Service,eventEngine, MessageService) {

    var infoComplEvents = {
        "InfoComplBeforeConfirm":"beforeConfirmInfoCompl",
        "InfoComplAfterConfirm":"afterConfirmInfoCompl",
        "InfoComplBeforeCancel":"beforeCancelInfoCompl",
        "InfoComplAfterCancel":"afterCancelInfoCompl"
    };
    
    var maxlote = "";
    var usaLoteAutomatico = false;

    Object.keys(infoComplEvents).forEach(function(eventName) {
        if (!eventEngine.hasEvent(eventName)) {
            eventEngine.registerEvent(eventName, infoComplEvents[eventName]);
        }
        
    });
    
     var getCabecalhoWidget = function(){
        if(templateManager.container)
            return templateManager.container.getWidget("cabecalho");
    };
    var rowParent;

    this.infComplementares = function(widget, rowItem, fornImportRastXml, atualizaEstoque){
        var defaultParams = {
            ShowLote : true,
            ShowEntr : false,
            Ajuste : false,
            AtuEstoq : true,
            VerQtEst : true,
            DocuDevo : false,
            AfPiloto : false,
            IsEntrada : true,
            IsSaida: false
        };

        rowParent = rowItem;
        defaultParams.ShowLote = rowItem.__is_new;
        var cabecalhoCurrentRow = getCabecalhoWidget().currentRow;
        var isNotaAjuste = cabecalhoCurrentRow.IDNFAJUSTE == "S" && cabecalhoCurrentRow.IDENTRSAIDOP == "S";

        Util.extend(defaultParams,widget.paramInfoCompl);  
        
        if(!rowParent.IDCNTRESTOQ)   // IDCNTRESTOQ = null ou inexistente produto controlara estoque
            rowParent.IDCNTRESTOQ = 'S';
            
        if(!(rowParent.CDFILIAL && rowParent.CDFILIMOVI && rowParent.CDPRODESTO && rowParent.CDPRODUTO && rowParent.CDARVPROD && rowParent.NMPRODUTO)){
            console.error('Row do Item incompleta');
        } else if (rowParent.IDCNTRESTOQ != 'N' && !isNotaAjuste && cabecalhoCurrentRow.CDTIPOOPER != "03") { // IDCNTRESTOQ = null produto controlara estoque
            Lib00206_InfComplProduto_Service.getParamFilial(rowParent.CDFILIMOVI).then(function(paramfilial){
                paramfilial = paramfilial.dataset.paramfilial[0];
                // IDCTRLESTQ = null filial controlara estoque
                if(paramfilial.IDCTRLESTQ != 'N' && (paramfilial.IDUTILLOTE == 'S' || paramfilial.IDUTILALMOX == 'S') && atualizaEstoque != 'N'){
                    if(paramfilial.IDUTILLOTE == 'S' && widget.container.name == 'dfe11000_lancamento_entrada')
                        defaultParams.ShowEntr = true;
                    if(widget.container.name != 'dfe11000_lancamento_entrada')
                        defaultParams.IsSaida = true;
                    this.openInfCompl(widget, defaultParams, paramfilial, fornImportRastXml);
                } else {
                    notOpenPopup(widget);
                }
            }.bind(this));
        } else {
            notOpenPopup(widget);
            
        }

    };

    this.limpaCamposInfCompl = function(widget){
        widget.fields.forEach(function(field){
            field.value(null);
        });
    };

    this.openInfCompl = function(parentWidget, params, paramfilial, fornImportRastXml){
        var popupWidget = createInfComplProdutoPopup(parentWidget);
        delete(rowParent.$error);
        Util.extend(popupWidget.currentRow,rowParent);
        //this.limpaCamposInfCompl(popupWidget);

        Lib00206_InfComplProduto_Service.getDtprocessa().then(function(success){
            var dtprocessa = success.dataset.paramestoque_processamento[0].DTPROCESSA;

            if(!rowParent.DTLANCMOVI)
                rowParent.DTLANCMOVI = dtprocessa;

            popupWidget.getField('DTVALILOTE').minDate = dtprocessa;
            usaLoteAutomatico = false;

            var filialFilter = {
                "name":"CDFILIAL",
                "operator":"=",
                "value":rowParent.CDFILIMOVI
            };

            ['CDALMOXARIFE', 'CDLOCALESTOQ'].forEach(function(fieldName) {
                var fieldFilter = popupWidget.getField(fieldName).dataSourceFilter;
                var filterIndex = fieldFilter.findIndex(function(element) {return element.name == "CDFILIAL";});

                if(filterIndex == -1)
                    fieldFilter.push(filialFilter);
                else
                    fieldFilter[filterIndex] = filialFilter;

                if(['CDLOCALESTOQ'].includes(fieldName)) {
                    filterIndex = fieldFilter.findIndex(function(element) {return element.name == "CDALMOXARIFE";});

                    if(filterIndex == -1) {
                        var almoxariFilter = {
                            "name":"CDALMOXARIFE",
                            "operator":"=",
                            "value":"@Lib00206_InfComplProduto_Controller.getInfComplFieldVal('"+parentWidget.name+"','CDALMOXARIFE')@"
                        };

                        fieldFilter.push(almoxariFilter);
                    }
                }
            });

            this.setFieldLote(popupWidget,parentWidget,params);

            if(!params.AtuEstoq){
                popupWidget.getField('CDALMOXARIFE').setValue(' ');
                popupWidget.getField('CDLOCALESTOQ').setValue(' ');
                popupWidget.getField('NRSUBLOTE').setValue(' ');
                popupWidget.getField('NRLOTEESTQ').setValue(' ');
            }

            popupWidget.params = params;
            if(params.DocuDevo) {
                popupWidget.getField('IDNOVOLOTE').value(!params.DocuDevo);
                popupWidget.getField('IDNOVOLOTE').defaultValue = !params.DocuDevo;
                popupWidget.getField('IDNOVOLOTE').readOnly = params.DocuDevo;
            } else {
                popupWidget.getField('IDNOVOLOTE').value(params.ShowEntr);
                popupWidget.getField('IDNOVOLOTE').defaultValue = params.ShowEntr;
                popupWidget.getField('IDNOVOLOTE').readOnly = !params.ShowEntr;
            }
            
            popupWidget.getField('IDNOVOLOTE').isVisible = !params.IsSaida;
            
            this.setReadOnlyField(popupWidget,params).then(function(paramfilial){
                Lib00206_InfComplProduto_Service.getProdFili(rowParent.CDPRODUTO, rowParent.CDFILIMOVI).then(function(prodfili){
                    prodfili = prodfili.dataset.prodfili[0];

                    var almoxField = popupWidget.getField('CDALMOXARIFE'),
                        localField = popupWidget.getField('CDLOCALESTOQ'),
                        selectLote = popupWidget.getField('IDNOVOLOTE'),
                        loteField = popupWidget.getField('NRLOTEESTQ'),
                        subloteField = popupWidget.getField('NRSUBLOTE'),
                        agregacaoField = popupWidget.getField('CDAGREGACAO');

                    loteField.validations.required = true;
                    subloteField.setValue(' ');
                    agregacaoField.setValue(null);

                    if(prodfili.IDUTCTRLLOTE == 'N' && paramfilial.IDUTILLOTE == 'N') {
                        delete loteField.validations.required;
                        loteField.readOnly = true;
                        loteField.setValue(' ');
                        selectLote.readOnly = true;
                        popupWidget.getField('IDNOVOLOTE').isVisible = false;
                        popupWidget.getField('NRLOTEESTQ').isVisible = false;
                        popupWidget.getField('NRSUBLOTE').isVisible = false;
                        agregacaoField.isVisible = false;
                    } else{
                        popupWidget.getField('IDNOVOLOTE').readOnly = paramfilial.IDENTRAUTLOTE == 'S' ? true : false;
                        popupWidget.getField('NRLOTEESTQ').isVisible = true;
                        popupWidget.getField('NRSUBLOTE').isVisible = true;
                        agregacaoField.isVisible = true;
                        
                        if(params.IsEntrada) 
                            loteField.readOnly = paramfilial.IDENTRAUTLOTE == 'S' ? true : false;
                        else 
                            loteField.readOnly = false;
                   
                        if(!parentWidget.currentRow.NRLOTEESTQ || parentWidget.currentRow.NRLOTEESTQ == ' ') {
                            loteField.value(null);
                            if(prodfili && params.ShowLote && (paramfilial.IDENTRSEQLOTE == 'S' || paramfilial.IDENTRAUTLOTE === 'S' ) && !fornImportRastXml) {
                                usaLoteAutomatico = true;        
                                sugereNovoLote(prodfili, params, loteField); 
                            }
                        }
                        else if(parentWidget.currentRow.NRLOTEESTQ) {
                            popupWidget.currentRow.NRLOTEESTQ = parentWidget.currentRow.NRLOTEESTQ;
                        }
                        
                        if(fornImportRastXml) controlFornImportRastXml(popupWidget, parentWidget);
                    }

                    if(prodfili.IDCNTRESTOQ == 'N'){
                        loteField.readOnly = true;
                        loteField.setValue(' ');
                        subloteField.readOnly = true;
                        subloteField.setValue(' ');
                        almoxField.readOnly = true;
                        almoxField.setValue(' ');
                        delete almoxField.validations.required;
                        localField.readOnly = true;
                        localField.setValue(' ');
                        delete localField.validations.required;
                    }
                    
                    if(!almoxField.readOnly) {
                        if(parentWidget.currentRow.CDALMOXARIFE == " " || !parentWidget.currentRow.CDALMOXARIFE) {
                            popupWidget.currentRow.CDALMOXARIFE =  prodfili.CDALMOXARIFE || paramfilial.CDALMOXARIFE || null;
                        }
                        else {
                            popupWidget.currentRow.CDALMOXARIFE = parentWidget.currentRow.CDALMOXARIFE;
                        }
						
                    } else {
                        popupWidget.currentRow.CDALMOXARIFE = " ";
                    }
                    
                    if(!localField.readOnly) {
                        if(parentWidget.currentRow.CDLOCALESTOQ == " " || !parentWidget.currentRow.CDLOCALESTOQ) {
                            popupWidget.currentRow.CDLOCALESTOQ = prodfili.CDLOCALESTOQ || paramfilial.CDLOCALESTOQ || null;
                        }
                        else {
                            popupWidget.currentRow.CDLOCALESTOQ = parentWidget.currentRow.CDLOCALESTOQ;
                        }
                    }
                    else {
                        popupWidget.currentRow.CDLOCALESTOQ = " ";
                    }
                    
                    
                    if(params.DocuDevo) {
                        almoxField.readOnly = true;
                        selectLote.readOnly = true;
                        loteField.readOnly = true;
                        
                        popupWidget.getAction('confirmAction').isVisible = false;
                        popupWidget.getAction('cancelAction').label = "Voltar";
                    } else {
                        popupWidget.getAction('confirmAction').isVisible = true;
                        popupWidget.getAction('cancelAction').label = "Cancelar";
                    }

                });
            });
            ScreenService.openPopup(popupWidget);
        }.bind(this));
    };
    
    //Seta automaticamente o NRLOTEESTQ dos produtos caso a filial tenha os parâmetros necessários
    this.setCamposMultiItens = function (widgetItens, selectedRows) {
        var deferred = ZHPromise.defer();
        var cdfilirece = widgetItens.getParent().currentRow.CDFILIRECE;
        var params = {
            ShowLote : true,
            ShowEntr : false,
            Ajuste : false,
            AtuEstoq : true,
            VerQtEst : true,
            DocuDevo : false,
            AfPiloto : false,
            IsEntrada : true,
            IsSaida: false
        };

        Util.extend(params,widgetItens.paramInfoCompl);
        
        selectedRows.forEach ((row) => {
            row.CDFILIMOVI = cdfilirece;
            row = Util.clone(row, ["__rightActions", "__leftActions"]);
        });
        
        Lib00206_InfComplProduto_Service.getParamFilial(selectedRows[0].CDFILIMOVI).then(function(paramfilial){
            paramfilial = paramfilial.dataset.paramfilial[0];
            // IDCTRLESTQ = null filial controlara estoque
            if(paramfilial.IDCTRLESTQ == null || (paramfilial.IDUTILLOTE != 'S' && paramfilial.IDUTILALMOX != 'S')){
                return;
            }
            if(paramfilial.IDUTILLOTE == 'S' && widgetItens.container.name == 'dfe11000_lancamento_entrada')
                params.ShowEntr = true;
                
            Lib00206_InfComplProduto_Service.getProdFili(selectedRows[0].CDPRODUTO, selectedRows[0].CDFILIMOVI).then(function(prodfili){
                prodfili = prodfili.dataset.prodfili[0];
                
                if(prodfili.IDUTCTRLLOTE != 'S' || paramfilial.IDUTILLOTE != 'S') {
                    return;
                }
                if(!prodfili || !params.ShowLote || (paramfilial.IDENTRSEQLOTE != 'S' && paramfilial.IDENTRAUTLOTE != 'S') ) {
                    return;
                }
                
                var countRows = 0, totalRows = selectedRows.length;
                selectedRows.forEach ((row) => {
                    sugereNovoLote(prodfili, params, row, null, true).then(newRow => {
                        row.NRLOTEESTQ = newRow.NRLOTEESTQ;
                        countRows++;
                        if (countRows == totalRows) {
                            deferred.resolve(selectedRows);
                        }
                    });
                });
            });
        });
        
        return deferred;
    };

    this.setFieldLote = function(popupWidget,parentWidget,params){
        var fieldLote = popupWidget.getField('NRLOTEESTQ');
        fieldLote.dataSourceFilter=[
                {
                    "name":"P_CDFILIAL",
                    "operator":"=",
                    "value":rowParent.CDFILIMOVI
                },{
                    "name":"P_CDPRODUTO",
                    "operator":"=",
                    "value":rowParent.CDPRODUTO
                },{
                    "name":"P_NRLOTEESTQ",
                    "operator":"=",
                    "value":" "
                },{
                    "name":"P_NRSUBLOTE",
                    "operator":"=",
                    "value":" "
                },{
                    "name":"P_DTPOSICAO",
                    "operator":"=",
                    "value":rowParent.DTLANCMOVI
                }
            ];
        if(!params.ShowEntr || !params.Ajuste ){
            fieldLote.dataSource.name = 'df#/lote_infCompl_almox';
            fieldLote.dataSourceFilter = fieldLote.dataSourceFilter.concat([
                {
                    "name":"P_CDALMOXARIFE",
                    "operator":"=",
                    "value":"@Lib00206_InfComplProduto_Controller.getInfComplFieldVal('"+parentWidget.name+"','CDALMOXARIFE')|| ' '@"
                },{
                    "name":"P_CDLOCALESTOQ",
                    "operator":"=",
                    "value":"@Lib00206_InfComplProduto_Controller.getInfComplFieldVal('"+parentWidget.name+"','CDLOCALESTOQ')|| ' '@"
                },{
                    "name":"P_IDQUANTID",
                    "operator":"=",
                    "value":"S"
                }
            ]);
        }else{
            if(params.VerQtEst){
                fieldLote.dataSource.name = 'df#/lote_infCompl_almox';
                fieldLote.dataSourceFilter = fieldLote.dataSourceFilter.concat([
                    {
                        "name":"P_CDALMOXARIFE",
                        "operator":"=",
                    "value":"@Lib00206_InfComplProduto_Controller.getInfComplFieldVal('"+parentWidget.name+"','CDALMOXARIFE')|| ' '@"
                    },{
                        "name":"P_CDLOCALESTOQ",
                        "operator":"=",
                    "value":"@Lib00206_InfComplProduto_Controller.getInfComplFieldVal('"+parentWidget.name+"','CDLOCALESTOQ')|| ' '@"
                    },{
                        "name":"P_IDQUANTID",
                        "operator":"=",
                        "value":"N"
                    }
                ]);
            }else{
                fieldLote.dataSource.name = 'df#/lote_infCompl_semAlmox';
            }
        }
    };

    function notOpenPopup(widget) {
        ZHPromise.when(widget.beforeConfirmInfoCompl && widget.AlwaysCallsBeforeEvent !== false ? widget.beforeConfirmInfoCompl() : undefined).then(function(){
            if(widget.afterConfirmInfoCompl && widget.AlwaysCallsAfterEvents !== false)
                widget.afterConfirmInfoCompl();
        });
    }
    
    var setCurrentRowWithOutData = function(currentRow, valuesToSetOnCurrentRow, outDataValue) {
        for (var key in outDataValue) {
            currentRow[outDataValue[key]] = valuesToSetOnCurrentRow[key];
        }
    };
    
    var locaisValidos = function(widget){
        var almoxarifado = widget.getField('CDALMOXARIFE');
        var local = widget.getField('CDLOCALESTOQ');
        var nlote = widget.getField('NRLOTEESTQ');
        var novolote = widget.getField('IDNOVOLOTE').value();
        var utAlmoxarifado = !almoxarifado.readOnly;
        var utLocal = !local.readOnly;
        var utLote = !nlote.readOnly || usaLoteAutomatico;
        var almoxarifadoValido = false; 
        var localValido = false;
        var loteValido = false;
        var locaisValidos = false;

        
        var arrLocais = ['CDALMOXARIFE', 'CDLOCALESTOQ', 'NRLOTEESTQ', 'NRSUBLOTE'];
        
        arrLocais.forEach( local => {
            if(widget.currentRow[local] == "")
                widget.currentRow[local] = " ";
        });
         
        if(utAlmoxarifado && almoxarifado.value() !== ' ')
            almoxarifadoValido = true;
        else
        if(!utAlmoxarifado && almoxarifado.value() === ' ')
            almoxarifadoValido = true;
                
        if(utLocal && local.value() !== ' ')
            localValido = true;
        else
        if(!utLocal && local.value() === ' ')
            localValido = true;
                
        if(utLote && nlote.value() !== ' ')
            loteValido = true;
        else
        if(!utLote && nlote.value() === ' ')
            loteValido = true;
                
        if(almoxarifadoValido && localValido && loteValido)
            locaisValidos = true;
            
        return locaisValidos;
    };
    
     this.saveLote = function(widget){
        if(widget.isValid() && locaisValidos(widget)){
            var row  = widget.currentRow;
            if(row.DTVALILOTE && row.DTFABRPROD && moment(row.DTFABRPROD,'DD/MM/YYYY').isAfter(moment(row.DTVALILOTE,'DD/MM/YYYY')) && widget.params.IsEntrada){
                ScreenService.showMessage('Data de fabricação deve ser menor ou igual a data de validade.');
            }else if(row.DTVALILOTE && moment(row.DTVALILOTE,'DD/MM/YYYY').isBefore(moment(widget.getField('DTVALILOTE').minDate,'DD/MM/YYYY')) && widget.params.IsEntrada){
                ScreenService.showMessage('Data de validade deve ser maior que a data de processamento.');
            }else{
                var parent = widget.getParent();
                
                //atualiza currentrow com posição de estoque
                var lote =    widget.getField('NRLOTEESTQ');
                var subLote = widget.getField('NRSUBLOTE');
                var loteOutData = lote.outData;
                
                lote.dataSourceFilter.forEach(function(params){
                    if(params.name == "P_NRSUBLOTE")
                        params.value = subLote.value();
                	if(params.name == "P_NRLOTEESTQ")
                	    params.value = lote.value();
                }.bind(this));
                
                ZHPromise.when(parent.beforeConfirmInfoCompl?parent.beforeConfirmInfoCompl():undefined).then(function(){
                    Lib00206_InfComplProduto_Service.saveLote(row).then(
                        function(){
                            rowParent.CDALMOXARIFE = row.CDALMOXARIFE;
                            rowParent.CDLOCALESTOQ = row.CDLOCALESTOQ;
                            rowParent.NRLOTEESTQ = row.NRLOTEESTQ;
                            rowParent.NRSUBLOTE = row.NRSUBLOTE;
                            rowParent.DTVALILOTE = row.DTVALILOTE;
                            rowParent.QTESTOQDIA = (row.QTESTOQDIA) ? row.QTESTOQDIA : 0;
                            ScreenService.closePopup();
                            parent.activate();
                            lote.reload().then(function(posEst){
                                if(posEst.dataset.lote_infCompl_almox.length > 0){
                                    var valuesToSet = posEst.dataset.lote_infCompl_almox[0];
                                    setCurrentRowWithOutData(row, valuesToSet, loteOutData);
                                    setCurrentRowWithOutData(rowParent, valuesToSet, loteOutData);
                                }else{
                                    rowParent.QTESTOQDIA = 0;
                                    rowParent.VRESTOQDIA = 0;
                                }
                                if(parent.afterConfirmInfoCompl)
                                    if(widget.params.IsSaida) {
                                        consultaCustoAfterInfCom(rowParent)
                                        .then(() => parent.afterConfirmInfoCompl(rowParent))
                                        .catch(() => parent.afterConfirmInfoCompl(rowParent));
                                    }
                                    else {
                                        parent.afterConfirmInfoCompl(rowParent);
                                    }
                                   
                            }.bind(this));
                            
                        },
                        function(error){
                            ScreenService.showMessage(error.data.error);
                        }
                    );
                });
            

            }
        }
    };

    var createInfComplProdutoPopup = function(parentWidget) {
        var widget = parentWidget.InfoCompl;
        if(!widget ){
            widget = getPopUpObject();
            widget.container = parentWidget.container;
            widget = metaDataFactory.widgetFactory(widget,parentWidget);
            parentWidget.InfoCompl = widget;
            widget.parent = parentWidget;
            return widget;
        }else{
            return widget;
        }
    };

    this.getInfComplFieldVal= function(parentName,fieldName){
        return templateManager.container.getWidget(parentName).InfoCompl.getField(fieldName).value();
    };

    this.setReadOnlyField = function(popupWidget,params){
        return Lib00206_InfComplProduto_Service.getParamFilial(popupWidget.currentRow.CDFILIMOVI).then(function(paramfilial){
            paramfilial = paramfilial.dataset.paramfilial[0];
            usaLoteAutomatico = paramfilial.IDENTRAUTLOTE === 'S';
            var idnovolote = popupWidget.getField('IDNOVOLOTE').value();
            popupWidget.getField('CDALMOXARIFE').readOnly = paramfilial.IDUTILALMOX == 'N' || !params.AtuEstoq;
            popupWidget.getField('CDLOCALESTOQ').readOnly = paramfilial.IDUTILLCESTQ == 'N' || !params.AtuEstoq;
            popupWidget.getField('DTVALILOTE').readOnly   = paramfilial.IDUTILLOTE == 'N' || !params.ShowEntr || params.DocuDevo || !idnovolote;
            popupWidget.getField('NRLOTEESTQ').readOnly   = paramfilial.IDUTILLOTE == 'N' || !params.AtuEstoq || params.DocuDevo || (paramfilial.IDENTRAUTLOTE === 'S' && params.ShowEntr);
            popupWidget.getField('CDAGREGACAO').readOnly  = popupWidget.getField('NRLOTEESTQ').readOnly || !idnovolote;
            popupWidget.getField('NRSUBLOTE').readOnly    = popupWidget.getField('NRLOTEESTQ').readOnly || !idnovolote || (paramfilial.IDENTRAUTLOTE === 'S' && params.ShowEntr);
            popupWidget.getField('DTFABRPROD').readOnly   = (paramfilial.IDUTILLOTE !== 'S') || !params.ShowEntr || params.DocuDevo || !idnovolote;

            if(paramfilial.IDUTILLOTE ==='S' && idnovolote){
                if(!popupWidget.getField('DTVALILOTE').validations.hasOwnProperty('required'))
                    popupWidget.getField('DTVALILOTE').validations = {"required": {"message": "Campo Obrigatorio"}};
            }else{
                delete popupWidget.getField('DTVALILOTE').validations.required;
            }

            if(!popupWidget.getField('CDALMOXARIFE').readOnly){
                if(!popupWidget.getField('CDALMOXARIFE').validations.hasOwnProperty('required'))
                    popupWidget.getField('NRLOTEESTQ').validations = {"required": {"message": "Campo Obrigatorio"}};
            }else{
                delete popupWidget.getField('CDALMOXARIFE').validations.required;
            }

            if(!popupWidget.getField('CDLOCALESTOQ').readOnly){
                if(!popupWidget.getField('CDLOCALESTOQ').validations.hasOwnProperty('required'))
                    popupWidget.getField('NRLOTEESTQ').validations = {"required": {"message": "Campo Obrigatorio"}};
            }else{
                delete popupWidget.getField('CDLOCALESTOQ').validations.required;
            }

            if(!popupWidget.getField('NRLOTEESTQ').readOnly){
                if(!popupWidget.getField('NRLOTEESTQ').validations.hasOwnProperty('required'))
                    popupWidget.getField('NRLOTEESTQ').validations = {"required": {"message": "Campo Obrigatorio"}};
            }else{
                delete popupWidget.getField('NRLOTEESTQ').validations.required;
            }


            if(idnovolote){
                popupWidget.getField('NRLOTEESTQ').template = 'field/text-edit.html';
                popupWidget.getField('NRLOTEESTQ').dataSourceFilter[2].value = ' ';
                popupWidget.getField('NRLOTEESTQ').dataSourceFilter[3].value = ' ';
                popupWidget.getField('NRLOTEESTQ').maxlength = maxlote;
                if(usaLoteAutomatico) {
                    if(maxlote) {
                        popupWidget.getField('NRLOTEESTQ').validations.minLength = {length : maxlote , message: "Valor abaixo do Valor Minímo Sugerido" };
                    }
                    else {
                        popupWidget.getField('NRLOTEESTQ').readOnly = false;
                    }
                }
            }else{
                popupWidget.getField('NRLOTEESTQ').template = 'zh-select-autocomplete#field/select-autocomplete.html';
                popupWidget.getField('NRLOTEESTQ').dataSourceFilter[2].value = 'TD';
                popupWidget.getField('NRLOTEESTQ').dataSourceFilter[3].value = 'TD';
                delete popupWidget.getField('NRLOTEESTQ').maxlength;
                delete popupWidget.getField('NRLOTEESTQ').validations.minLength;
            }
            
            /*let useUniLote = paramfilial.IDENTRUNILOTE == 'S'? true:false;
            this.liberaEdit(popupWidget, !useUniLote);*/
            
            popupWidget.reload();

            return paramfilial;
        });
    };

    this.idNovoLoteChange =function(widget){
        this.setReadOnlyField(widget,widget.params);

        widget.getField('NRLOTEESTQ').setValue(null);
        widget.getField('NRSUBLOTE').setValue(null);

        if(widget.getField('IDNOVOLOTE').value())
            widget.getField('NRSUBLOTE').setValue('');

        widget.getField('DTVALILOTE').setValue('');
        widget.getField('DTFABRPROD').setValue(null);
        widget.getField('CDAGREGACAO').setValue(null);
    };
    
    this.loteOnChange =function(widget){
        var novoLote = widget.getField('IDNOVOLOTE').value();
        
        if(novoLote){
            var nrLote = widget.getField('NRLOTEESTQ');
            nrLote.dataSourceFilter[2].value = nrLote.value();
        }

    };

    this.cancelaLancamento = function(widget) {
        var parent = widget.getParent();
        
        //Lib00206_InfComplProduto_Service.getParamFilial(rowParent.CDFILIMOVI).then(function(paramfilial){
           // paramfilial = paramfilial.dataset.paramfilial[0];
                ZHPromise.when(parent.beforeCancelInfoCompl?parent.beforeCancelInfoCompl():undefined).then(function(){
                    ScreenService.closePopup();
                    if(parent.afterCancelInfoCompl)
                        parent.afterCancelInfoCompl();
                });
            
        //});
    };
    
    this.almoxarifeChange = function(args) {
        var fieldLocalEstoq = args.owner.field.widget.getField('CDLOCALESTOQ');
        var fieldLote = args.owner.field.widget.getField('NRLOTEESTQ');
        var fieldSubLote = args.owner.field.widget.getField('NRSUBLOTE');
        
        if(fieldLocalEstoq.readOnly || fieldLocalEstoq.readOnlyOnEdit)
            fieldLocalEstoq.setValue(' ');
        else
            fieldLocalEstoq.setValue('');
            
        fieldLocalEstoq.reload();
        
        if(fieldLote.readOnly || fieldLote.readOnlyOnEdit)
            fieldLote.setValue(' ');
        else
            fieldLote.setValue('');
            
        fieldLote.reload();
        
        if(fieldSubLote.readOnly || fieldSubLote.readOnlyOnEdit)
            fieldSubLote.setValue(' ');
        else
            fieldSubLote.setValue('');
            
    };
    
	this.localChange = function(args) {
        
        var fieldLote = args.owner.field.widget.getField('NRLOTEESTQ');
        var fieldSubLote = args.owner.field.widget.getField('NRSUBLOTE');
        
        if(!usaLoteAutomatico){
            if(fieldLote.readOnly || fieldLote.readOnlyOnEdit)
                fieldLote.setValue(' ');
            else
                fieldLote.setValue('');
        }
            
        fieldLote.reload();
        
        if(fieldSubLote.readOnly || fieldSubLote.readOnlyOnEdit)
            fieldSubLote.setValue(' ');
        else
            fieldSubLote.setValue('');
    };
    
   /* this.liberaEdit = function(widget, status = false){
        let actionEdit = widget.getAction('liberar');
        actionEdit.readOnly = status;
        actionEdit.isVisible = !status;
    };
    */
    this.liberaJustificativa = function(widget){
        let fieldJustific = widget.getField('DSJUSTIFICATIVA');
        fieldJustific.validations = {"required": {"message": "Campo Obrigatorio"}};
        fieldJustific.isVisible = true;
        widget.getField('NRLOTEESTQ').readOnly = false;
        widget.getField('NRSUBLOTE').readOnly = false;
    };

    var getPopUpObject = function() {
        return {
            "isVisible": true,
            "name": "infComplProduto",
            "label": "Informações Complementares do Produto",
            "template": "widget/form.html",
            "currentRow": {},
            "showAsTab": false,
            "actions": [
                {
                    "id": "1899882901626109187970",
                    "name": "cancelAction",
                    "label": "Cancelar",
                    "showAsAction": "back",
                    "isVisible": true,
                    "events": [
                        {
                            "id": "189988290583399632971",
                            "name": "ActionEvent",
                            "code": "Lib00206_InfComplProduto_Controller.cancelaLancamento(args.owner.widget)"
                        }
                    ]
                },
                {
                    "id": "1899882901076133865972",
                    "name": "confirmAction",
                    "label": "Confirmar",
                    "showAsAction": "never",
                    "isVisible": true,
                    "events": [
                        {
                            "id": "1899882901577606513973",
                            "name": "ActionEvent",
                            "code": "Lib00206_InfComplProduto_Controller.saveLote(args.owner.widget)"
                        }
                    ]
                },
                {
                    "id": 1,
                    "name": "liberar",
                    "label": "Liberar Lote",
                    "isVisible": true,
                    "readOnly": true,
                    "showAsAction": "view|edit|always",
                    "icon": "key",
                    "events": [
                        {
                            "id": 1,
                            "name": "ActionEvent",
                            "code": "Lib00206_InfComplProduto_Controller.liberaJustificativa(args.owner.widget)"
                        }
                    ]
                }
            ],
            "fields": [
                {
                    "id": "1899882901326453382941",
                    "name": "NMPRODUTO",
                    "label": "Produto",
                    "isVisible": true,
                    "readOnly": true,
                    "readOnlyOnEdit": true,
                    "class": "6",
                    "showOnForm": true,
                    "showOnList": true,
                    "showOnView": true,
                    "defaultValue": null,
                    "descriptionOrder": 1,
                    "template": "field/text-edit.html"
                },
                {
                    "id": "1899882903301163737942",
                    "name": "CDARVPROD",
                    "label": "Cod. Produto",
                    "isVisible": true,
                    "readOnly": true,
                    "readOnlyOnEdit": true,
                    "class": "6",
                    "showOnForm": true,
                    "showOnList": true,
                    "showOnView": true,
                    "defaultValue": null,
                    "descriptionOrder": 1,
                    "template": "field/text-edit.html"
                },
                {
                    "id": "1899882901280710555943",
                    "name": "CDALMOXARIFE",
                    "label": "Almoxarifado",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "showOnView": true,
                    "class": 6,
                    "order": "CDALMOXARIFE",
                    "template": "zh-select-autocomplete#field/select-autocomplete.html",
                    "searchableFields": [
                        "CDALMOXARIFE",
                        "DSALMOXARIFE"
                    ],
                    "displayedFields": [
                        "CDALMOXARIFE",
                        "DSALMOXARIFE"
                    ],
                    "showSelectAs": "grid",
                    "dataSource": {
                        "rest": true,
                        "lazyLoad": true,
                        "name": "df#/crud/almoxarifado"
                    },
                    "dataSourceFilter": [
                        {
                            "name": "CDALMOXARIFE",
                            "operator": "<>",
                            "value": " "
                        }
                    ],
                    "descriptionField": "DSALMOXARIFE",
                    "valueField": "CDALMOXARIFE",
                    "outData": {
                        "CDALMOXARIFE": "CDALMOXARIFE",
                        "DSALMOXARIFE": "DSALMOXARIFE"
                    },
                    "fields": [
                        {
                            "id": "1899882903706419573944",
                            "label": "Código",
                            "name": "CDALMOXARIFE",
                            "showOnList": true,
							"template": "field/text-edit.html",
                            "isVisible": true
                        },
                        {
                            "id": "189988290938331621945",
                            "label": "Almoxarifado",
                            "name": "DSALMOXARIFE",
                            "showOnList": true,
							"isVisible": true
                        }
                    ],
                    "events": [
                        {
                            "name": "FieldOnClick",
                            "code": "args.owner.field.initializeField();"
                        },
                        {
                            "id": "1899882902027725524947",
                            "name": "FieldOnChange",
                            "code": "Lib00206_InfComplProduto_Controller.almoxarifeChange(args);"
                        }
                    ],
                    "validations": {
                        "required": {
                            "message": "Campo obrigatório"
                        }
                    }
                },
                {
                    "id": "1899882902070375588948",
                    "name": "CDLOCALESTOQ",
                    "label": "Localização",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "class": 6,
                    "order": "CDLOCALESTOQ",
                    "template": "zh-select-autocomplete#field/select-autocomplete.html",
                    "searchableFields": [
                        "CDLOCALESTOQ",
                        "DSLOCALESTOQ"
                    ],
                    "displayedFields": [
                        "CDLOCALESTOQ",
                        "DSLOCALESTOQ"
                    ],
                    "showSelectAs": "grid",
                    "dataSource": {
                        "rest": true,
                        "lazyLoad": true,
                        "name": "df#/crud/localestoque"
                    },
                    "dataSourceFilter": [
                        {
                            "name": "CDLOCALESTOQ",
                            "operator": "<>",
                            "value": " "
                        }
                    ],
                    "descriptionField": "DSLOCALESTOQ",
                    "valueField": "CDLOCALESTOQ",
                    "outData": {
                        "CDLOCALESTOQ": "CDLOCALESTOQ",
                        "DSLOCALESTOQ": "DSLOCALESTOQ"
                    },
                    "fields": [
                        {
                            "id": "1899882904123017515949",
                            "label": "Código",
                            "name": "CDLOCALESTOQ",
                            "showOnList": true
                        },
                        {
                            "id": "1899882902241420430950",
                            "label": "Descrição",
                            "name": "DSLOCALESTOQ",
                            "showOnList": true
                        }
                    ],
                    "events": [
                        {
                            "id": "1899882903471388340951",
                            "name": "FieldOnClick",
                            "code": "args.owner.field.initializeField();"
                        },
						{
                            "name": "FieldOnChange",
                            "code": "Lib00206_InfComplProduto_Controller.localChange(args);"
                        }
                    ],
                    "validations": {
                        "required": {
                            "message": "Campo obrigatório"
                        }
                    }
                },
                {
                    "id": "189988290332070133952",
                    "name": "IDNOVOLOTE",
                    "label": "Lote",
                    "isVisible": true,
                    "readOnly": false,
                    "filterProperties": {
                        "isVisible": false,
                        "class": 0,
                        "newLine": false
                    },
                    "dataSource": {
                        "data": [
                            {
                                "IDNOVOLOTE": false,
                                "DSNOVOLOTE": "Lote Existente"
                            },
                            {
                                "IDNOVOLOTE": true,
                                "DSNOVOLOTE": "Novo Lote"
                            }
                        ]
                    },
                    "valueField": "IDNOVOLOTE",
                    "descriptionField": "DSNOVOLOTE",
                    "hideEmptyValue": true,
                    "template": "field/select-native.html",
                    "class": 6,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "placeholder": null,
                    "events": [
                        {
                            "id": "1899882903965666718953",
                            "name": "FieldOnChange",
                            "code": "Lib00206_InfComplProduto_Controller.idNovoLoteChange(args.owner.field.widget)"
                        },
                        {
                            "id": "1899882902235780224954",
                            "name": "FieldBeforeSelectOpen",
                            "code": "args.owner.field.reload()"
                        }
                    ]
                },
                {
                    "id": "1899882902891348800955",
                    "name": "NRLOTEESTQ",
                    "label": "Lote",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "class": 6,
                    "order": "NRLOTEESTQ",
                    "template": "zh-select-autocomplete#field/select-autocomplete.html",
                    "searchableFields": [
                        "NRLOTEESTQ",
                        "DTVALILOTE"
                    ],
                    "displayedFields": [
                        "NRLOTEESTQ",
                        "DTVALILOTE"
                    ],
                    "showSelectAs": "grid",
                    "maxlength": 20,
                    "dataSource": {
                        "rest": true,
                        "lazyLoad": true,
                        "name": "df#/lote_infCompl_semAlmox"
                    },
                    "dataSourceFilter": [],
                    "descriptionField": "NRLOTEESTQ",
                    "valueField": "NRLOTEESTQ",
                    "outData": {
                        "NRLOTEESTQ": "NRLOTEESTQ",
                        "NRSUBLOTE": "NRSUBLOTE",
                        "DTVALILOTE": "DTVALILOTE",
                        "DTFABRPROD": "DTFABRPROD",
                        "QTESTOQDIA": "QTESTOQDIA",
                        "VRESTOQDIA": "VRESTOQDIA",
                        "CDFABRIC": "CDFABRIC",
                        "CDAGREGACAO": "CDAGREGACAO"
                    },
                    "fields": [
                        {
                            "id": "1899882901163182621956",
                            "label": "Lote",
                            "name": "NRLOTEESTQ",
                            "showOnList": true
                        },
                        {
                            "id": "1899882902077203128957",
                            "label": "Sublote",
                            "name": "NRSUBLOTE",
                            "showOnList": true
                        },
                        {
                            "id": "1899882902745803117958",
                            "label": "Validade",
                            "name": "DTVALILOTE",
                            "showOnList": true,
                            "displayFormat": "DD/MM/YYYY"
                        },
                        {
                            "id": "1899882903401287676959",
                            "label": "Almoxarifado",
                            "name": "CDALMOXARIFE",
                            "showOnList": true
                        },
                        {
                            "id": "1899882903029078872960",
                            "label": "Localização",
                            "name": "CDLOCALESTOQ",
                            "showOnList": true
                        },
                        {
                            "id": "1899882902174827513961",
                            "label": "Quantidade",
                            "name": "QTESTOQDIA",
                            "showOnList": true,
                            "mask": {
                                "type": "viewFloat",
                                "params": {
                                    "precision": 2,
                                    "showSymbol": false
                                }
                            }
                        },
                        {
                            "id": "1899882902962224615962",
                            "label": "Valor Total",
                            "name": "VRESTOQDIA",
                            "showOnList": true,
                            "mask": {
                                "type": "viewFloat",
                                "params": {
                                    "precision": "2"
                                }
                            }
                        }
                    ],
                    "events": [
                        {
                            "name": "FieldOnClick",
                            "code": "args.owner.field.initializeField();"
                        },
                        {
                            "id": "1899882902477402526964",
                            "name": "FieldOnChange",
                            "code": "Lib00206_InfComplProduto_Controller.loteOnChange(args.owner.field.widget)"
                        }
                    ],
                    "validations": {
                         "required": {
                            "message": "Campo obrigatório"
                        }
                    }
                },
                {
                    "id": "1899882902054841854965",
                    "name": "NRSUBLOTE",
                    "label": "Sublote",
                    "isVisible": true,
                    "sequenceInForm": 100,
                    "readOnly": false,
                    "defaultValue": null,
                    "maxlength": 4,
                    "filterProperties": {
                        "isVisible": false,
                        "class": 0,
                        "newLine": false
                    },
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {},
                    "placeholder": null,
                    "events": [],
                    "validations": {}
                },
                {
                    "id": "189988290908490628966",
                    "name": "DTVALILOTE",
                    "label": "Data de Validade",
                    "isVisible": true,
                    "sequenceInForm": 80,
                    "readOnly": true,
                    "defaultValue": null,
                    "filterProperties": {
                        "template": "field/interval.html",
                        "readOnly": false
                    },
                    "template": "field/calendar.html",
                    "dateFormat": "DD/MM/YYYY",
                    "class": 6,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {},
                    "placeholder": null,
                    "validations": {},
                    "events": [
                        {
                            "id": "1899882904006880584967",
                            "name": "FieldOnChange",
                            "code": "args.owner.field.widget.activate();"
                        }
                    ]
                },
                {
                    "id": "1899882902888126514968",
                    "name": "DTFABRPROD",
                    "label": "Data de Fabricação",
                    "isVisible": true,
                    "sequenceInForm": 90,
                    "readOnly": false,
                    "defaultValue": null,
                    "filterProperties": {
                        "template": "field/interval.html"
                    },
                    "template": "field/calendar.html",
                    "dateFormat": "DD/MM/YYYY",
                    "class": 6,
                    "descriptionOrder": 4,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {},
                    "placeholder": null,
                    "events": []
                },
                {
                    "id": "1899882901788656434969",
                    "name": "CDAGREGACAO",
                    "label": "Código de Agregação",
                    "isVisible": true,
                    "sequenceInForm": 110,
                    "readOnly": false,
                    "defaultValue": null,
                    "maxlength": 20,
                    "filterProperties": {
                        "isVisible": false,
                        "class": 0,
                        "newLine": false
                    },
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {},
                    "placeholder": null,
                    "events": [],
                    "validations": {}
                },
                {
                    "id": 7,
                    "name": "DSJUSTIFICATIVA",
                    "label": "Justificativa",
                    "isVisible": false,
                    "readOnly": false,
                    "defaultValue": null,
                    "maxlength": 50,
                    "filterProperties": {
                        "isVisible": false,
                        "class": 0,
                        "newLine": false
                    },
                    "template": "field/text-edit.html",
                    "class": 12,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {},
                    "placeholder": null,
                    "events": [],
                    "validations": {}
                }
            ],
            "events": [
                {
                    "id": "1899882904087446697974",
                    "name": "WidgetOnEnter",
                    "code": "args.owner.getField('CDALMOXARIFE').reload();"
                }
            ],
            "id": "2060328640427745033940"
        };
    };
    
    var sugereNovoLote = function(prodfili, params, loteField ) {
        if(prodfili && params.ShowLote) {
            Lib00206_InfComplProduto_Service.getProximoLote(rowParent.CDFILIMOVI, rowParent.CDPRODESTO).then(function(success){
                loteField.setValue(success.dataset.prox_lote.NRLOTEESTQ);
                if(success.dataset.prox_lote.NRLOTEESTQ) {
                    maxlote= success.dataset.prox_lote.NRLOTEESTQ.length;
                    loteField.maxlength = maxlote;
                    loteField.validations.minLength = {length : maxlote , message: "Valor abaixo do Valor Minímo Sugerido" };
                }
                else {
                    loteField.readOnly = false;
                }
                loteField.reload();
            });
        }
        
    };
    
    var consultaCustoAfterInfCom = function (rowParent){
        
        return new Promise((resolve, reject) => {
            var lancamentoItemArgs = templateManager.container.getActiveWidget();
             var filial = lancamentoItemArgs.getParent().currentRow.CDFILIEMIT;
             var dtLancamento = lancamentoItemArgs.getParent().currentRow.DTENTRSAID;
             var produto = lancamentoItemArgs.currentRow.CDPRODUTO;
             var operacao = lancamentoItemArgs.getParent().currentRow.CDTIPOOPER;
             
             var almoxarifado = rowParent.CDALMOXARIFE;
             var localEstoque = rowParent.CDLOCALESTOQ;
             var lote = rowParent.NRLOTEESTQ;
             var subLote =  rowParent.NRSUBLOTE;
    
             lancamentoItemArgs.currentRow.VRUNITITEM = "";
             lancamentoItemArgs.currentRow.QTITEMPROD = "";
             lancamentoItemArgs.currentRow.VRTOTAITEM = "";
    
            if (produto) {
                if(lancamentoItemArgs.currentRow.CDSERVFILI) {
                    lancamentoItemArgs.currentRow.VRUNITITEM = lancamentoItemArgs.currentRow.VRSERVFATU;
                }
                else {
                    Lib00206_InfComplProduto_Service.consultaCustoQuantidadeProduto(filial, produto, dtLancamento, null, almoxarifado, 
                                                                                    localEstoque, lote, subLote ,operacao).then(function(custoQuant) {
                        lancamentoItemArgs.currentRow.QTESTOQDIA = custoQuant.dataset.consultaCustoQuantidadeProduto.QTESTOQDIA;
                        custoQuant = custoQuant.dataset.consultaCustoQuantidadeProduto.VRCUSTOPROD;
                        lancamentoItemArgs.currentRow.VRUNITITEM = typeof(custoQuant) === "string" ? parseFloat(custoQuant) : custoQuant;
                        resolve();
                    })
                    .catch(()=>  reject());
                }
            }
            
           
        });
 
    };
    
    var controlFornImportRastXml = (popupWidget, parentWidget) => {
        let keyNF = parentWidget.getParent().currentRow.NRACESSONFE,
            nrOrg = parentWidget.getParent().currentRow.NRORG,
            seqItProd = parseInt(popupWidget.currentRow.NRSEQITPROD, 10);
            
        Lib00206_InfComplProduto_Service.getDataFornImportRastXml(keyNF, seqItProd, nrOrg).then((request) => {
            let row = request.dataset.intg_itemproduto[0],
                popupWidget = templateManager.container.getActiveWidget(),
                selectLote = popupWidget.getField('IDNOVOLOTE'),
                loteField = popupWidget.getField('NRLOTEESTQ'),
                dtValField = popupWidget.getField('DTVALILOTE'),
                dtFabField = popupWidget.getField('DTFABRPROD'),
                agregacaoField = popupWidget.getField('CDAGREGACAO'),
                subloteField = popupWidget.getField('NRSUBLOTE');
            
            selectLote.readOnly = true;
            loteField.readOnly = true;
            dtValField.readOnly = true;
            dtFabField.readOnly = true;
            agregacaoField.readOnly = true;
            subloteField.readOnly = true;
            
            if (loteField.validations.minLength) delete loteField.validations.minLength;
            
            if(row.NRLOTEPRODXML && row.DTVALPRODXML && row.DTFABPRODXML) {
                selectLote.setValue(true);
                loteField.setValue(row.NRLOTEPRODXML);
                dtValField.setValue(moment(row.DTVALPRODXML, 'DD/MM/YYYY').format('DD/MM/YYYY'));
                dtFabField.setValue(moment(row.DTFABPRODXML, 'DD/MM/YYYY').format('DD/MM/YYYY'));
                agregacaoField.setValue(row.CDAGREGPRODXML);
            } else {
                MessageService.showMessageByCode('FORNIMPORTRASTXML_DATA_NOT_FOUND');
            }
        });
    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Lib00206_InfComplProduto_Controller', Lib00206_InfComplProduto_Controller);
});

// FILE: ./assets/js/utilities/UtilDf.js
var UtilDf = function(ScreenService,ZHPromise, templateManager, metaDataFactory, Lib00206_InfComplProduto_Service, eventEngine, LibUtilities) {
    /*
	saveCollectedDataGeneric
	funcao utilizada para salvar os dados das filiais ao entrarem nas telas
	é utilizada em várias telas, por enquanto apenas nas que filtram uma única filial
    */
    this.saveCollectedDataGeneric = function (widget, description = undefined, fromFilter = false, ids = undefined) {
    	let collectionIds;
    	let valuesWidget = widget;
    	if(fromFilter){
            valuesWidget = templateManager.container.factoredWidgets.filter((widget) => { return widget._filteredWidgetId == valuesWidget.id; })[0];
        }
        
        if(valuesWidget) {
        	if(valuesWidget.currentRow){
        		if(valuesWidget.currentRow.CDFILIRECE)
        			collectionIds = valuesWidget.currentRow.CDFILIRECE;
    			else if(valuesWidget.currentRow.CDFILIEMIT)
    			    collectionIds = valuesWidget.currentRow.CDFILIEMIT;
        		else if(valuesWidget.currentRow.CDFILIAL)
        			collectionIds = valuesWidget.currentRow.CDFILIAL;
        		else
        			collectionIds = 'T';
        	}else{
        		collectionIds = 'T';
        	}
        }else{
        	collectionIds = 'T';
        }
        
        //widget instanceof Container
    	let operationId = templateManager.container.name + '/' + description;
    	
    	/*
    	* Lista de descriptions utilizadas para tentar padronizar chamadas:
    	* ContainerAfterinit
    	*/
    	/*
	    * @param $collectionIds  => ID ou ARRAY da(s) coleta(s) (Cod Filial, Cod Operador, Cod da Loja, ...)
	    * @param $operationId    => ID da operação (Retirada, Cardápio, Login, ...)
	    * @param $productId      => Código do Produto (opcional).
	    * @param $operatorCode   => Código do Operador (opcional).
	    * @param $operatorName   => Nome do Operador (opcional).
	    * @param $organizationId => Código do organização (opcional).
	    */
    	LibUtilities.saveCollectedData(collectionIds, operationId);
    };


   /*  Função para bloquear os fields filhos ligados a um field pai.
        Caso desbloqueiaComValor = true o field filho será habilitado 
        se o field pai possuir valor. No caso de valueDesbloqueia != null
        o valor para desbloquear o filho deve ser igual ao valor de valueDesbloqueia.
        
        @param field field 
    */
    this.alteraReadOnlyFilhos = function(field) {
        let widget = field.widget;
        field.filhos.forEach((filho) => {
            var desbloqueiaComValor = (filho.desbloqueiaComValor) ? filho.desbloqueiaComValor : true;
            var valueDesbloqueia    = (filho.valueDesbloqueia)    ? filho.valueDesbloqueia    : null;
            var isRequired          = (filho.isRequired)          ? filho.isRequired          : false;
            
            let filhoField = widget.getField(filho.name);
            
            
            if(valueDesbloqueia == null) {
                filhoField.readOnly =  ((field.value() == "") == desbloqueiaComValor);
                if(((field.value() == "") == desbloqueiaComValor)) widget.currentRow[filho.name] = "";
                if(filhoField.filhos) this.alteraReadOnlyFilhos(filhoField, true);
                widget.getField(filho.name).reload();
            }
            else {
                var habilita = (field.value() == valueDesbloqueia);
                if(habilita) {
                    if(filhoField.readOnly) {
                        if(filho.clearFields) {
                            filho.clearFields.forEach((clearField) => {
                                widget.currentRow[clearField] = "";
                            });
                        }
                        widget.currentRow[filho.name] = "";
                    }
                    
                    filhoField.readOnly = false;
                    
                    if(isRequired) {
                        if(filhoField.validations) {
                            filhoField.validations.required = true;
                        }
                        else {
                            filhoField.validations = {required : true};
                        }
                    }
                }
                else {
                    var filhoOutData = filhoField.outData;
                    for(var fieldOut in filhoOutData) {
                        widget.currentRow[fieldOut] = "";
                    } 
                    widget.currentRow[filho.name] = "";
                    filhoField.readOnly = true;
                    if(isRequired) {
                        if(filhoField.validations) {
                            if(filhoField.validations.required) delete filhoField.validations.required;
                        }
                    }
                }
                if(filhoField.filhos) this.alteraReadOnlyFilhos(filhoField, true);
                widget.getField(filho.name).reload();
            }
        });
    };

    /*
        Percorre uma widget procurando por fields com a propiedade "filhos".
        Para os fields com a propiedade fields a função adiciona o evento 
        de alteraReadOnlyFilhos.
        
        @param widget widget
        @param boolean filter
    */
    this.blockFieldsFilhosAndSetEvents = function(widget, filter = true) {
        let fields = widget.fields;
        fields.forEach((field) => {
            if(field.filhos) {
                field.events.push(
                    {
                        "name": "FieldOnChange",
                        "code": "UtilDf.alteraReadOnlyFilhos(args.owner.field)"
                    }
                );
                
                field.filhos.forEach((filho) => {
                    if(filter) {
                        if(!widget.getField(filho.name).filterProperties) 
                            widget.getField(filho.name).filterProperties = {};
                        
                        widget.getField(filho.name).filterProperties.readOnly = false; 
                    }
                    else  widget.getField(filho.name).readOnly = false; 
                });
            }
        });
        this.openFilterWidget(widget);
    };
    
    this.controlaStatusWidgets = function(widgetPai, nomeWidgetAtual, status, ignoredWidgets) {
        var allWidgets = widgetPai.widgets;

        allWidgets.filter(widget => widget.name !== nomeWidgetAtual).forEach(widget => widget.enabled = status);

        if (widgetPai.name !== nomeWidgetAtual)
            widgetPai.enabled = status;

        if (Array.isArray(ignoredWidgets))
            allWidgets.filter(widget => ignoredWidgets.indexOf(widget.name) !== -1).forEach(widget => widget.enabled = !status);
    };
    
    this.getFieldValueFilter = function(filedName, widgetName) {
        var container =  templateManager.container;
        if(container) {
            var filterWidget;
            var filterBforeApply = templateManager.container.getActiveWidget();
            if(filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
              filterWidget = filterBforeApply;
            else {
                var f = container.getWidget(widgetName);
                if(f) {
                     filterWidget = container.getWidget(widgetName).appliedFilter;
                }
               
            }
            
            if(filterWidget) {
                var valueField;
                if(filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
                    valueField = filterWidget.currentRow[filedName];
                else
                    valueField = filterWidget.FILTER_ROW[filedName];
                    
                return valueField || null;
            }
            else return null;
        }
        else return null;
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('UtilDf', UtilDf);
});

// FILE: ./assets/js/repositories/AprovacaoLancamentos_Repositories.js
var aprovacaoLancamentos_Repositories = function(restEngine) {
    this.aprovarNotasMarcadas = function(notas) {
        return restEngine.post('df#/aprovarNotas', {
            "requestType": "DataSet",
            "dataset": notas
        });
    };

    this.cancelarAprovacaoNotasMarcadas = function(notas) {
        return restEngine.post('df#/cancelarAprovacao', {
            "requestType": "DataSet",
            "dataset": notas
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('AprovacaoLancamentos_Repositories', aprovacaoLancamentos_Repositories);
});

// FILE: ./assets/js/repositories/Df11000_Lancamento_Repositories.js
var df11000_Lancamento_Repositories = function(localStorageEngine, restEngine, General) {

    this.montaCorpoNota = function(filial, seqLancamento, entrada_saida, statusNota) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filial
            },
            {
                "name": "NRLANCTONF",
                "operator": "=",
                "value": seqLancamento
            },
            {
                "name": "IDENTRSAIDOP",
                "operator": "=",
                "value": entrada_saida
            },
            {
                "name": "IDSTATUS",
                "operator": "=",
                "value": statusNota
            }
        ];
        return localStorageEngine.download("df#/monta_corpo_nota", filter);
    };
    
    this.getItensRateio = function(cdfilial, nrLancto) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdfilial
            },
            {
                "name": "NRLANCTONF",
                "operator": "=",
                "value": nrLancto
            }
        ];
        return localStorageEngine.download("df#/itens_rateio", filter);
    };

    this.retornaItens = function(filial, lancamento, organizacao) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filial
            },
            {
                "name": "NRLANCTONF",
                "operator": "=",
                "value": lancamento
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": organizacao
            }
        ];
        return localStorageEngine.download("df#/crud/itemproduto", filter);
    };

    this.retornaEncargos = function(filial, lancamento, organizacao) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filial
            },
            {
                "name": "NRLANCTONF",
                "operator": "=",
                "value": lancamento
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": organizacao
            }
        ];
        return localStorageEngine.download("df#/crud/itemencargo", filter);
    };
    
    this.getStatusVencimento =  function(cdfilial, nrlanbolpag){
        var filter = [
            {
                "name": "CDFILIAL", 
                "operator" : "=", 
                "value": cdfilial
            },
            {
                "name": "NRLANBOLPAG", 
                "operator" : "=", 
                "value": nrlanbolpag
            }
        ];
        return General.callRestEngine("df#/get_status_vencimentos", "FilterData", filter);
        
    };

    this.retornaVencimentos = function(filial, lancamento, organizacao) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filial
            },
            {
                "name": "NRLANCTONF",
                "operator": "=",
                "value": lancamento
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": organizacao
            }
        ];
        return localStorageEngine.download("df#/crud/itemproduto", filter);
    };

    this.retornaDadosAdicionais = function(filial, lancamento, organizacao) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filial
            },
            {
                "name": "NRLANCTONF",
                "operator": "=",
                "value": lancamento
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": organizacao
            }
        ];
        return localStorageEngine.download("df#/crud/dadosadic", filter);
    };

    this.retornaEncargosPadrao = function() {
        return localStorageEngine.download("df#/crud/encargo");
    };

    this.retornaDataProcessamento = function(filial) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filial
            }
        ];
        return localStorageEngine.download("df#/paramestoque_processamento", filter);
    };

    this.retornaItensNotaOrigem = function(filial, lancamento, organizacao) {
        var filter = [
            {
                "name": "P_CDFILIAL",
                "operator": "=",
                "value": filial
            },
            {
                "name": "P_NRLANCTONF",
                "operator": "=",
                "value": lancamento
            },
            {
                "name": "P_NRORG",
                "operator": "=",
                "value": organizacao
            }
        ];
        return localStorageEngine.download("df#/itemproduto_notafiscal_origem", filter);
    };

    this.retornaQuantidadeDevolvida = function(
        filial, lancamento, filialOrigem, lancamentoOrigem, codArvProd, lote, subLote, valorUnitario, organizacao
        ) {
        var filter = [
            {
                "name": "P_FILIAL",
                "operator": "=",
                "value": filial
            },
            {
                "name": "P_LANCTONF",
                "operator": "=",
                "value": lancamento
            },
            {
                "name": "P_CDFILIALIT",
                "operator": "=",
                "value": filialOrigem
            },
            {
                "name": "P_NRLANCTONFIT",
                "operator": "=",
                "value": lancamentoOrigem
            },
            {
                "name": "P_CDARVPROD",
                "operator": "=",
                "value": codArvProd
            },
            {
                "name": "P_NRLOTEESTQ",
                "operator": "=",
                "value": lote
            },
            {
                "name": "P_NRSUBLOTE",
                "operator": "=",
                "value": subLote
            },
            {
                "name": "P_VRUNITITEM",
                "operator": "=",
                "value": valorUnitario
            },
            {
                "name": "P_NRORG",
                "operator": "=",
                "value": organizacao
            }
        ];
        return localStorageEngine.download("df#/quantidade_devolvida_itemproduto", filter);
    };

    this.salvaCabecalho = function(cabecalho) {
        return restEngine.post('df#/crud/notafiscal/save', {
            "requestType": "Row",
            "row": cabecalho
        });
    };
    

    this.salvaItemnotafisc = function(row) {
        return restEngine.post('df#/crud/itemnotafisc/save', {
            "requestType": "Row",
            "row": row
        });
    };

    this.salvaItem = function(item) {
        return restEngine.post('df#/crud/itemproduto/save', {
            "requestType": "Row",
            "row": item
        });
    };
    
    this.salvaRateio = function(rateio) {
        return restEngine.post('df#/crud/rateionf/save', {
            "requestType": "DataSet",
            "dataset": rateio
        });
    };

    this.salvaItemProcedure = function(item) {
        return restEngine.post('df#/salvarItem',
            {
                "requestType": "FilterData",
                "filter": [
                    {
                        "name": "ITEM",
                        "operator": "=",
                        "value": item
                    }
                ]
            }
        );
    };

    this.excluiItem = function(item){
        return restEngine.post('df#/crud/itemproduto/remove', {
            "requestType": "Row",
            "row": item
        });
    };
    
    this.excluiRateio = function(rateio){
        return restEngine.post('df#/crud/rateionf/remove', {
            "requestType": "DataSet",
            "dataset": rateio
        });
    };

    this.salvaImposto = function(imposto) {
        return restEngine.post('df#/crud/impostonf/save', {
            "requestType": "DataSet",
            "dataset": imposto
        });
    };

    this.salvaEncargo = function(encargos) {
        return restEngine.post('df#/crud/itemencargo/save', {
            "requestType": "DataSet",
            "dataset": encargos
        });
    };

    this.salvaDadosAdicionais = function(dadosAdicionais) {
        return restEngine.post('df#/crud/dadosadic/save', {
            "requestType": "Row",
            "row": dadosAdicionais
        });
    };

    this.excluiDadosAdicionais = function(dadosAdicionais){
        return restEngine.post('df#/crud/dadosadic/remove', {
            "requestType": "Row",
            "row": dadosAdicionais
        });
    };

    this.fecharNota = function(cabecalho, vencimento, forcaFechamento) {
        return restEngine.post('df#/fecharNota',
            {
                "requestType": "FilterData",
                "filter": [
                    {
                        "name": "CABECALHO",
                        "operator": "=",
                        "value": cabecalho
                    },
                    {
                        "name": "VENCIMENTO",
                        "operator": "=",
                        "value": vencimento
                    },
                    {
                        "name": "FORCAFECHAMENTO",
                        "operator": "=",
                        "value": forcaFechamento
                    }
                ]
            }
        );
    };
    
    this.geraRateio = function(filial, lancamento, idEntrSaid, rateioNF, geraRatAut = 'N', realizaManut = 'P') {
        return restEngine.post('df#/geraRateio',
            {
                "requestType": "FilterData",
                "filter": [
                    {
                        "name": "FILIAL",
                        "operator": "=",
                        "value": filial
                    },
                    {
                        "name": "LANCAMENTO",
                        "operator": "=",
                        "value": lancamento
                    },
                    {
                        "name": "IDENTRSAID",
                        "operator": "=",
                        "value": idEntrSaid
                    },
                    {
                        "name": "RATEIONF",
                        "operator": "=",
                        "value": rateioNF
                    },
                    {
                        "name": "GERARATAUT",
                        "operator": "=",
                        "value": geraRatAut
                    },
                    {
                        "name": "REALIZAMANUT",
                        "operator": "=",
                        "value": realizaManut
                    }
                ]
            }
        );
    };

    this.salvaImpostosTmpItem = function(impostosItens) {
        return restEngine.post('df#/crud/trabitproimpos/save', {
            "requestType": "DataSet",
            "dataset": impostosItens
        });
    };
    
    this.salvaImpostosItem = function(impostosItens) {
        return restEngine.post('df#/crud/itemproimpos/save', {
            "requestType": "DataSet",
            "dataset": impostosItens
        });
    };
    
    this.deleteItemProImpos = function(imposto){
        return restEngine.post('df#/crud/itemproimpos/remove', {
            "requestType": "Row",
            "row": imposto
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Df11000_Lancamento_Repositories', df11000_Lancamento_Repositories);
});

// FILE: ./assets/js/repositories/Df56000_AprovacaoLancamentos_Repositories.js
var df56000_AprovacaoLancamentos_Repositories = function(restEngine) {
    this.salvarEdicaoLancamento = function(row) {
        return restEngine.post('df#/crud/notafiscal/save', {
            "requestType": "Row",
            "row": row
        });
    };
    
    this.validaEdicaoLancamento = function(currentRow) {
        return restEngine.post('df#/validaEdicaoLancamento', {
            "requestType": "Row",
            "row": currentRow
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Df56000_AprovacaoLancamentos_Repositories', df56000_AprovacaoLancamentos_Repositories);
});

// FILE: ./assets/js/repositories/Ger00400_Imposto_Repository.js
function Ger00400_Imposto_Repository(restEngine) {

	this.save = function(row) {
        return restEngine.post('df#/imposto_completo/save', {
            "requestType": "Row",
            "row": row
        });
	};

	this.saveContabilizacao = function(row) {
        return restEngine.post('df#/cctbimpforn_contctbl/save', {
            "requestType": "Row",
            "row": row
        });
	};

    this.saveBaseCalculo = function(row) {
        return restEngine.post('df#/basecalc_impostos_encargo/save', {
            "requestType": "Row",
            "row": row
        });
    };

    this.saveOutrosImpostosTipoOperacao = function(row) {
        return restEngine.post('df#/basecalctpoper_impostos/save', {
            "requestType": "Row",
            "row": row
        });
    };

    this.verificaRange = function(row) {
        var filtro = [
            {
                "name": "P_CDIMPOSTO",
                "operator": "=",
                "value": row.CDIMPOSTO
            }
        ];
        return restEngine.post('df#/dadosfiscncm_regrafiscal', {
            "requestType": "FilterData",
            "filter": filtro
        });
    };
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger00400_Imposto_Repository', Ger00400_Imposto_Repository);
});

// FILE: ./assets/js/repositories/Ger01400_Mensagem_Repository.js
Configuration(function(RepositoryFactory, ContextRegister, restEngine) {

    var Ger01400_Mensagem_Repository = RepositoryFactory.factory('df#/crud/mensagem', 'LOCAL', 1, 30000);

    Ger01400_Mensagem_Repository.getValidations = function(CDMENSAGEM) {
        return restEngine.post("df#/mensagem_validation", {
            "requestType": "Row",
            "row": {"CDMENSAGEM": CDMENSAGEM}
        });
    };

    ContextRegister.register('Ger01400_Mensagem_Repository', Ger01400_Mensagem_Repository);

});

// FILE: ./assets/js/repositories/Ger03400_ReplicacaoDocumentos_Repository.js
var Ger03400_ReplicacaoDocumentos_Repository = function (restEngine) {

    
    this.verificaControle = function(CDFILIAL) {
        var filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": CDFILIAL
            }
        ];
        return restEngine.post('df#/crud/paramfilial', {
            "requestType": "FilterData",
            "filter": filtro
        });
    };
    
    this.verificaNRREPLICDOC = function(CDFILIAL,NRREPLICDOC) {
         
         var filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": CDFILIAL
            },
            {
                "name": "NRREPLICDOC",
                "operator": "=",
                "value": NRREPLICDOC
            }
        ];
         return restEngine.post('df#/crud/replicdoc', {
            "requestType": "FilterData",
            "filter": filtro
        });
        
    };
    
    this.verificaRelacaoFilial = function(CDFILIAL) {
         
         var filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": CDFILIAL
            }
        ];
        
        return restEngine.post('df#/crud/filial', {
            "requestType": "FilterData",
            "filter": filtro
        });
        
    };
    
    this.verificaRelacaoFilialEmpresa = function(P_CDFILIAL,P_CDEMPRESA) {
         var filtro = [
            {
                    "name": "P_CDFILIAL",
                    "operator": "=",
                    "value": P_CDFILIAL
            },
            {
                    "name": "P_CDEMPRESA",
                    "operator": "=",
                    "value": P_CDEMPRESA
            }
        ];
        
        return restEngine.post('df#/filial_emprcntrfil', {
            "requestType": "FilterData",
            "filter": filtro
        });
    };
    
};

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger03400_ReplicacaoDocumentos_Repository', Ger03400_ReplicacaoDocumentos_Repository);
});

// FILE: ./assets/js/repositories/Ger03600_Rateio_Produto_Repository.js
var Ger03600_Rateio_Produto_Repository = function(restEngine){
    
    this.importRateio = function(row){
        return restEngine.post('df#/importRateio', {
            "requestType": "Row",
            "row": row
        });
    };
    
};

Configuration(function (ContextRegister){
 ContextRegister.register('Ger03600_Rateio_Produto_Repository', Ger03600_Rateio_Produto_Repository);
});

// FILE: ./assets/js/repositories/Ger03900_ErrosAvisosNf_Repository.js
function Ger03900_ErrosAvisosNf_Repository (restEngine) {

	this.findAutoRegister = function(filter) {
        return restEngine.post('df#/crud/erroavisonffil_filial', {
            "requestType": "FilterData",
            "filter": filter
        });
	};
	
	this.saveAutoRegister = function(rows) {
        return restEngine.post('df#/erroavisonffil_filial/save', {
            "requestType": "DataSet",
            "dataset": rows
        });
	};
	
	this.deleteRegisters = function(cdfiliais){
	    return restEngine.post('df#/erroavisonffil_filial/deleteRegisters', {
            "requestType": "Row",
            "row": cdfiliais
        });
	};
	
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger03900_ErrosAvisosNf_Repository', Ger03900_ErrosAvisosNf_Repository);
});

// FILE: ./assets/js/repositories/Ger05500_produtosDesoneracaoImpostos_Repositories.js
function Ger05500_produtosDesoneracaoImpostos_Repositories (restEngine) {
    var self = this;

    self.removeProdutoDesoneracaoFornecedor = function (data) {
        return restEngine.post('df#/crud/prodesoneraimfor/remove',
            {
                "requestType":"DataSet",
                "dataset": data
            }
        );
    };

    self.removeProdutoDesoneracaoCliente = function (data) {
        return restEngine.post('df#/crud/prodesoneraimcli/remove',
            {
                "requestType":"DataSet",
                "dataset": data
            }
        );
    };

    self.removeProdutoDesoneracaoFilial = function (data) {
        return restEngine.post('df#/crud/prodesoneraimfil/remove',
            {
                "requestType":"DataSet",
                "dataset": data
            }
        );
    };
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger05500_produtosDesoneracaoImpostos_Repositories', Ger05500_produtosDesoneracaoImpostos_Repositories);
});

// FILE: ./assets/js/repositories/Ger06600_TipoDeOperacao_Repository.js
var Ger06600_TipoDeOperacao_Repository = function(restEngine) {

    this.checkParametersUtilizaDocumento = function(row) {
        return restEngine.post('df#/tipooperac/getParameters', {
            "requestType": "Row",
            "row": row
        });
    };

    this.saveTipoOperac = function(row) {
        return restEngine.post('df#/tipooperac/save', {
            "requestType": "Row",
            "row": row
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("Ger06600_TipoDeOperacao_Repository", Ger06600_TipoDeOperacao_Repository);
});


// FILE: ./assets/js/repositories/Ger10200_ParametrosGrupoFiscal_Repository.js
function Ger10200_ParametrosGrupoFiscal_Repository(restEngine) {

	this.save = function(row) {
        return restEngine.post('df#/cadastramentoAutomatico/save', {
            "requestType": "Row",
            "row": row
        });
	};
	this.saveReplicacao = function(row) {
        return restEngine.post('df#/replicaoAliquotas/save', {
            "requestType": "Row",
            "row": row
        });
	};

}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger10200_ParametrosGrupoFiscal_Repository', Ger10200_ParametrosGrupoFiscal_Repository);
});

// FILE: ./assets/js/repositories/Ger33500_AgenteDeTerceirizacao_Repository.js
function Ger33500_AgenteDeTerceirizacao_Repository (restEngine, localStorageEngine) {


    this.verificaControle = function(CDFILIAL) {
        var filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": CDFILIAL
            }
        ];
        return restEngine.post('df#/crud/paramfilial', {
            "requestType": "FilterData",
            "filter": filtro
        });
    };

    this.retornaUnidadeLogada = function() {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
        ];
        return localStorageEngine.download("df#/crud/filial", filter);
    };

}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger33500_AgenteDeTerceirizacao_Repository', Ger33500_AgenteDeTerceirizacao_Repository);
});

// FILE: ./assets/js/repositories/Ger57000_TabeladDoSimplesNacional_Repository.js
function Ger57000_TabelaDoSimplesNacional_Repository(restEngine) {
    
    //save: função
    //dataSet: array de dados a serem gravados
    //name: nome da widget que sera utilizado para a rota
    //Row: tipo de requisição para uma linha
    //DataSet: Tipo de requisição para mais de uma linha

	this.save = function(dataSet, name) {
	    if(dataSet.length == 1){ 
            return this.saveRow(dataSet[0],name);
	    }
	    else{
	        return this.saveDataSet(dataSet,name);
	    }
	};
	
	this.saveRow = function(row, name){
	    return restEngine.post("df#/simplesnacional/save" + name, {
            "requestType": "Row",
            "row": row
        });
	};
	
	this.saveDataSet = function(dataSet, name){
	    return restEngine.post("df#/simplesnacional/save" + name, {
            "requestType": "DataSet",
            "dataset": dataSet
        });
	};
	
	this.deleteTabelaDeReferencia = function (row){
	  	return	restEngine.post("df#/simplesnacional/deleteTabelaDeReferencia", {
            "requestType": "Row",
            "row": row
        });
        
	};
	
	this.deleteFaixaValores = function (row){
	  	return	restEngine.post("df#/simplesnacional/deleteFaixa", {
            "requestType": "Row",
            "row": row
        });
        
	};

}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger57000_TabelaDoSimplesNacional_Repository', Ger57000_TabelaDoSimplesNacional_Repository);
});

// FILE: ./assets/js/repositories/Ger80900_ParametrizacaoFiscalCentralizada_Repository.js
var Ger80900_ParametrizacaoFiscalCentralizada_Repository = function(localStorageEngine, restEngine) {

  this.deletarRegraImposto = function(row) {
    return restEngine.post('df#/deletarRegraImposto', {
      "requestType": "Row",
      "row": row
    });
  };
  
  this.deletarRegraCfop = function(row) {
    return restEngine.post('df#/deletarRegraCfop', {
      "requestType": "Row",
      "row": row
    });
  };
  
  this.consultarErrosRegra = function(row) {
    return restEngine.post('df#/consultarErrosRegra', {
      "requestType": "Row",
      "row": row
    });
  };
  
  this.getNewCodeRegra = function(row) {
    return restEngine.post('df#/getNewCodeRegra', {
      "requestType": "Row",
      "row": row
    });
  };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Ger80900_ParametrizacaoFiscalCentralizada_Repository', Ger80900_ParametrizacaoFiscalCentralizada_Repository);
});

// FILE: ./assets/js/repositories/Ger97500_TipoOpercaoCfop_Repository.js
function Ger97500_TipoOpercaoCfop_Repository(restEngine) {

	this.delete = function(row) {
        return restEngine.post('df#/deletarTpopercfop', {
            "requestType": "Row",
            "row": row
        });
	};

}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger97500_TipoOpercaoCfop_Repository', Ger97500_TipoOpercaoCfop_Repository);
});

// FILE: ./assets/js/repositories/Ger98500_TipoGestaoFiscal_Repository.js
function Ger98500_TipoGestaoFiscal_Repository(restEngine) {

	this.delete = function(row) {
        return restEngine.post('df#/deletetarTipoGestaoFiscalUnidade', {
            "requestType": "Row",
            "row": row
        });
	};

	this.verifica = function(row) {
        return restEngine.post('df#/verificaUnidadeRelacionada', {
            "requestType": "Row",
            "row": row
        });
	};

}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger98500_TipoGestaoFiscal_Repository', Ger98500_TipoGestaoFiscal_Repository);
});

// FILE: ./assets/js/repositories/Ger98600_CodigoDeSituacaoDaOperacaoDoSimplesNacional_Repository.js
function Ger98600_CodigoDeSituacaoDaOperacaoDoSimplesNacional_Repository(restEngine) {

	this.save = function(row) {
        return restEngine.post('df#/codfiscopersn_allDescriptions/save', {
            "requestType": "Row",
            "row": row
        });
	};

}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger98600_CodigoDeSituacaoDaOperacaoDoSimplesNacional_Repository', Ger98600_CodigoDeSituacaoDaOperacaoDoSimplesNacional_Repository);
});

// FILE: ./assets/js/repositories/Lib98004_EmailsAdicionais_Repository.js
function Lib98004_EmailsAdicionais_Repository(requestDataSourceEngine, General) {

	this.saveEmail = function(row) {
        return General.callRestEngine('/findEmailsAdicionais/save', "Row", row);
	};
	
	this.removeEmail = function(row) {
        return General.callRestEngine('/findEmailsAdicionais/remove', "Row", row);
	};

}

Configuration(function (ContextRegister) {
	ContextRegister.register('Lib98004_EmailsAdicionais_Repository', Lib98004_EmailsAdicionais_Repository);
});

// FILE: ./assets/modules/entrada/assets/js/repositories/Dfe11000_LancamentoEntrada_Repositories.js
var Dfe11000_LancamentoEntrada_Repositories = function(localStorageEngine, restEngine) {

    this.excluirNotaEntrada = function(filial, sequencialNota) {
        var row = {
            "CDFILIAL": filial,
            "NRLANCTONF": sequencialNota
        };
        return restEngine.post('df_entrada#/excluir_nota_entrada', {
            "requestType": "Row",
            "row": row
        });
    };

    this.salvaVencimento = function(vencimento) {
        return restEngine.post('df#/crud/contapagar/save', {
            "requestType": "Row",
            "row": vencimento
        });
    };

    this.excluiVencimento = function(vencimento){
        return restEngine.post('df#/crud/contapagar/remove', {
            "requestType": "Row",
            "row": vencimento
        });
    };

    this.calculaImpostoItemProduto = function(cabecalhoRow, lancamentoItemRow) {
        var rows = [cabecalhoRow, lancamentoItemRow];
        return restEngine.post('df#/calcula_imposto_item_produto', {
            "requestType": "DataSet",
            "dataset": rows
        });
    };
    
    this.validaParamProduto = function(row){
        return restEngine.post('df_entrada#/validaParamProduto', {
            "requestType": "Row",
            "row": row
        });
    };
    this.validaAF = function(row){
        return restEngine.post('df_entrada#/validaAF', {
            "requestType": "Row",
            "row": row
        });
    };

    this.realizaImportacaoItensNotaOrigem = function(cabecalhoRow) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cabecalhoRow.CDFILIAL
            },
            {
                "name": "NRLANCTONF",
                "operator": "=",
                "value": cabecalhoRow.NRLANCTONF
            },
            {
                "name": "PKNOTAFISCORIG",
                "operator": "=",
                "value": (Array.isArray(cabecalhoRow.PKNOTAFISCORIG))? cabecalhoRow.PKNOTAFISCORIG : [cabecalhoRow.PKNOTAFISCORIG]
            }
        ];

        return restEngine.post('ger#/crud/itens_notas_origem_entrada', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.retornaParambloqnfafil = function(filial, entradaSaida, tipoOperac) {
        if (!filial)
            filial = 'FILIAL_LOGADA';

        var filter = [{
            "name": "CDFILIAL",
            "operator": "=",
            "value": filial
        }, {
            "name": "IDENTRSAIDOP",
            "operator": "=",
            "value": entradaSaida
        }, {
            "name": "CDTIPOOPER",
            "operator": "=",
            "value": tipoOperac
        }];

        return restEngine.post('df#/crud/parambloqnfafil', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.realizaImportacaoAf = function(cabecalhoRow) {
        return restEngine.post('df_entrada#/realiza_importacao_af', {
            "requestType": "Row",
            "row": cabecalhoRow
        });
    };

    this.buscaFornecedorBloqTpOperac = function(tipoOperacao, nrinsjurforn) {
        var filter = [
            {
                "name": "CDTIPOOPER",
                "operator": "=",
                "value": tipoOperacao
            },
            {
                "name": "P_NRINSJURFORN",
                "operator": "=",
                "value": nrinsjurforn
            },
            {
                "name": "IDLIBERATPOPFOR",
                "operator": "=",
                "value": 'N'
            }
        ];
        return restEngine.post('df_entrada#/parambloqnfafor_fornecedorparamsbloq', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.salvaImpostosNotaFiscal = function(impostos) {
        return restEngine.post('df_entrada#/salva_impostos_nota_fiscal', {
            "requestType": "DataSet",
            "dataset": impostos
        });
    };

    this.limpaTrabsLancamento = function(cdfilial, nrlanctonf) {
        return restEngine.post('df_entrada#/remove_trabs_lancamento_entrada', {
            "requestType": "Row",
            "row": {
                'CDFILIAL': cdfilial,
                'NRLANCTONF': nrlanctonf
            }
        });
    };

    this.validacaoSalvaCabecalhoEntrada = function(cabecalhoRow) {
        return restEngine.post('df_entrada#/validacao_salva_cabecalho_entrada', {
            "requestType": "Row",
            "row": cabecalhoRow
        });
    };
    
    this.geraSenhaLancRetroativo = function(lancRetroativoRow) {
        return restEngine.post('df_entrada#/gera_senha_lanc_retroativo', {
            "requestType": "Row",
            "row": lancRetroativoRow
        });
    };

    this.buscaNfChaveAcesso = function(nracessonfe) {
        var filter = [
            {
                'name': 'P_NRACESSONFE',
                'operator': '=',
                'value': nracessonfe
            }, {
                'name': 'P_IDENTRSAIDOP',
                'operator': '=',
                'value': 'E'
            }
        ];
        return restEngine.post('df_entrada#/intg_notafiscal_lancamento', {
            "requestType": "FilterData",
            "filter": filter
        });
    };
    
    this.buscaNfChaveAcessoPorCampos = function(params){
        var filter = [
            {
                'name': 'P_DTEMISSAO',
                'operator': '=',
                'value': params.DTEMISSAO
            },
            {
                'name': 'P_CDSERIEENTR',
                'operator': '=',
                'value': params.CDSERIEENTR
            },
            {
                'name': 'P_NRNOTAFISC',
                'operator': '=',
                'value': params.NRNOTAFISC
            },
            {
                'name': 'P_NRINSJURNF',
                'operator': '=',
                'value': params.NRINSJURNF
            }
        ];
        return restEngine.post('df_entrada#/intg_notafiscal_lancamento_campos', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.realizaImportacaoChaveAcesso = function(cabecalhoRow) {
        return restEngine.post('df_entrada#/realiza_importacao_chave_acesso', {
            "requestType": "Row",
            "row": cabecalhoRow
        });
    };

    this.realizaImportacaoAfChaveAcesso = function(cabecalhoRow) {
        return restEngine.post('df_entrada#/realiza_importacao_af_chave_acesso', {
            "requestType": "Row",
            "row": cabecalhoRow
        });
    };

    this.salvaRelacionamentoItensXml = function(relacionamentoRow) {
        return restEngine.post('df_entrada#/salva_relacionamento_itens_xml', {
            "requestType": "Row",
            "row": relacionamentoRow
        });
    };

    this.removerRelacionamentoItemXml = function(cdprodutoxml, cdforneced, cdfilial, cdproduto) {
        var row = {
            'CDPRODUTOXML' : cdprodutoxml,
            'CDFORNECED' : cdforneced,
            'CDFILIAL':cdfilial,
            'CDPRODUTO':cdproduto
        };

        return restEngine.post('df_entrada#/remove_relacionamento_item', {
            "requestType": "Row",
            "row": row
        });
    };

    this.salvaMultiItens = function(novosItens, itensAlterados, currentRowParentWidget) {
        var rows = [novosItens, itensAlterados, currentRowParentWidget];
        return restEngine.post('df_entrada#/salva_multi_itens_lancamento_entrada', {
            "requestType": "DataSet",
            "dataset": rows
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Dfe11000_LancamentoEntrada_Repositories', Dfe11000_LancamentoEntrada_Repositories);
});


// FILE: ./assets/modules/entrada/assets/js/repositories/Dfe36000_EntradasTransferenciaDoOperadorLogistico_Repository.js
var Dfe36000_EntradasTransferenciaDoOperadorLogistico_Repository = function(restEngine){

};
Configuration(function (ContextRegister){
    ContextRegister.register('Dfe36000_EntradasTransferenciaDoOperadorLogistico_Repository', Dfe36000_EntradasTransferenciaDoOperadorLogistico_Repository);
});


// FILE: ./assets/modules/entrada/assets/js/repositories/Dfe51000_ImportacaoConsultaNotaFiscal_Repositories.js
var Dfe51000_ImportacaoConsultaNotaFiscal_Repositories = function(localStorageEngine, restEngine, General) {
    this.gerarDanfe = function(row){
        var filter = [
            {
                "name": "ROW",
                "operator": "=",
                "value": row
            }
        ];
        
        return General.callRestEngine("df_entrada#/gerarDanfe", "FilterData", filter);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Dfe51000_ImportacaoConsultaNotaFiscal_Repositories', Dfe51000_ImportacaoConsultaNotaFiscal_Repositories);
});

// FILE: ./assets/modules/entrada/assets/js/repositories/Dfe58000_ManifestacaoDestinatario_Repositories.js
var Dfe58000_ManifestacaoDestinatario_Repositories = function(localStorageEngine, restEngine, General) {
    this.montaConfigTransmissao = function(row) {
        var filter = [
            {
                "name": "ROW",
                "operator": "=",
                "value": row
            }
        ];
        return localStorageEngine.download("df_entrada#/config_manifestacao", filter);
    };
    
    this.setConfigTransmissao = function(row) {
        return localStorageEngine.save("df_entrada#/config_manifestacao", row);
    };

    this.sincronizarNFe = function(config, row) {
        var filter = [
            {
                "name": "CONFIG",
                "operator": "=",
                "value": config
            },
            {
                "name": "OPCOESSINC",
                "operator": "=",
                "value": row
            }
        ];
        return General.callRestEngine("df_entrada#/sincronizarNfeManifestacao", "FilterData", filter);
    };

    this.manifestarNfe = function(notasfiscais, config, evento, justificativa) {
        var filter = [
            {
                "name": "NOTAS",
                "operator": "=",
                "value": notasfiscais
            },
            {
                "name": "CONFIG",
                "operator": "=",
                "value": config
            },
            {
                "name": "EVENTO",
                "operator": "=",
                "value": evento
            },
            {
                "name": "JUSTIFICATIVA",
                "operator": "=",
                "value": justificativa
            }
        ];
        return localStorageEngine.download("df_entrada#/manifestarNfe", filter);
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('Dfe58000_ManifestacaoDestinatario_Repositories', Dfe58000_ManifestacaoDestinatario_Repositories);
});

// FILE: ./assets/modules/saida/assets/js/repositories/Dfe43200_ParametrosDocumentosFiscaisSaida_Repository.js
function Dfe43200_ParametrosDocumentosFiscaisSaida_Repository(restEngine) {

	this.saveParametrosFilial = function(row) {
        return restEngine.post('df_saida#/paramfilial_saida/save', {
            "requestType": "Row",
            "row": row
        });
	};
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Dfe43200_ParametrosDocumentosFiscaisSaida_Repository', Dfe43200_ParametrosDocumentosFiscaisSaida_Repository);
});

// FILE: ./assets/modules/saida/assets/js/repositories/Dfs11000_LancamentoSaida_Repositories.js
var dfs11000_LancamentoSaida_Repositories = function(localStorageEngine, restEngine) {

    this.salvaVencimento = function(vencimento) {
        return restEngine.post('df#/crud/contareceber/save', {
            "requestType": "Row",
            "row": vencimento
        });
    };

    this.excluiVencimento = function(vencimento){
        return restEngine.post('df#/crud/contareceber/remove', {
            "requestType": "Row",
            "row": vencimento
        });
    };

    this.excluirNota = function(filial, lancamento) {
        return restEngine.post('df_saida#/excluir_nota_saida',
            {
                "requestType": "FilterData",
                "filter": [
                    {
                        "name": "P_CDFILIAL",
                        "operator": "=",
                        "value": filial
                    },
                    {
                        "name": "P_LANCAMENTO",
                        "operator": "=",
                        "value": lancamento
                    }
                ]
            }
        );
    };

    this.retornaClienteFilial = function(filial, cliente) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filial
            },
            {
                "name": "CDCLIENTE",
                "operator": "=",
                "value": cliente
            }
        ];
        return localStorageEngine.download("df#/crud/clienfilial", filter);
    };
    
    this.saveDataUpdate = function (rowsUpdate, NEWDTENTRSAID) {
        var data = {};
        data.rowsUpdate = rowsUpdate;
        data.newDate = NEWDTENTRSAID;
        
        return restEngine.post('df_saida#/notafiscal_dataupdate/save', {
            "requestType": "Row",
            "row": data
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Dfs11000_LancamentoSaida_Repositories', dfs11000_LancamentoSaida_Repositories);
});

// FILE: ./assets/modules/saida/assets/js/repositories/Dfs49000_TransmissaoNfe_Repositories.js
var dfs49000_TransmissaoNfe_Repositories = function(localStorageEngine, restEngine) {
    this.montaConfigTransmissao = function(row) {
        var filter = [
            {
                "name": "ROW",
                "operator": "=",
                "value": row
            }
        ];
        return localStorageEngine.download("df#/config_transmissao", filter);
    };

    this.statusServico = function(notasfiscais, config) {
        var filter = [
            {
                "name": "NOTAS",
                "operator": "=",
                "value": notasfiscais
            },
            {
                "name": "CONFIG",
                "operator": "=",
                "value": config
            }
        ];
        return localStorageEngine.download("df_saida#/statusServico", filter);
    };

    this.transmiteNotas = function(notasfiscais, config, tipo) {
        var filter = [
            {
                "name": "NOTAS",
                "operator": "=",
                "value": notasfiscais
            },
            {
                "name": "CONFIG",
                "operator": "=",
                "value": config
            },
            {
                "name": "TIPO",
                "operator": "=",
                "value": tipo
            }
        ];
        return localStorageEngine.download("df_saida#/transmiteNotas", filter);
    };

    this.atualizaLote = function(lotesMarcados, config) {
        var filter = [
            {
                "name": "LOTESMARCADOS",
                "operator": "=",
                "value": lotesMarcados
            },
            {
                "name": "CONFIG",
                "operator": "=",
                "value": config
            }
        ];
        return localStorageEngine.download("df_saida#/atualizaLote", filter);
    };

    this.retornaDataProcessamento = function(filial) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filial
            }
        ];
        return localStorageEngine.download("df#/paramestoque_processamento", filter);
    };

    this.cancelarTransmissao = function(row, config) {
        var filter = [
            {
                "name": "ROW",
                "operator": "=",
                "value": row
            },
            {
                "name": "CONFIG",
                "operator": "=",
                "value": config
            }
        ];
        return localStorageEngine.download("df_saida#/cancelarTransmissao", filter);
    };

    this.enviarCartaCorrecao = function(notafiscal, dscorrecaonfe, nrseqcce, config) {
        var filter = [
            {
                "name": "NOTAFISCAL",
                "operator": "=",
                "value": notafiscal
            },
            {
                "name": "DSCORRECAONFE",
                "operator": "=",
                "value": dscorrecaonfe
            },
            {
                "name": "NRSEQCCE",
                "operator": "=",
                "value": nrseqcce
            },
            {
                "name": "CONFIG",
                "operator": "=",
                "value": config
            }
        ];
        return localStorageEngine.download("df_saida#/enviarCartaCorrecao", filter);
    };
    
    this.enviarEmailCC = function(notafiscal, config, tipo, nrseqcce) {
        var filter = [
            {
                "name": "NOTAFISCAL",
                "operator": "=",
                "value": notafiscal
            },
            {
                "name": "TIPO",
                "operator": "=",
                "value": tipo
            },
            {
                "name": "NRSEQCCE",
                "operator": "=",
                "value": nrseqcce
            },
            {
                "name": "CONFIG",
                "operator": "=",
                "value": config
            }
        ];
        return localStorageEngine.download("df_saida#/enviarEmailCC", filter);
    };

    this.visualizarDanfe = function(notasMarcadas, config) {
        var filter = [
            {
                "name": "NOTASMARCADAS",
                "operator": "=",
                "value": notasMarcadas
            },
            {
                "name": "CONFIG",
                "operator": "=",
                "value": config
            }
        ];
        return localStorageEngine.download("df_saida#/visualizarDanfe", filter);
    };

    this.downloadDanfe = function(notasMarcadas, config) {
        var filter = [
            {
                "name": "NOTASMARCADAS",
                "operator": "=",
                "value": notasMarcadas
            },
            {
                "name": "CONFIG",
                "operator": "=",
                "value": config
            }
        ];
        return localStorageEngine.download("df_saida#/downloadDanfe", filter);
    };

    this.downloadXML = function(notasMarcadas, config) {
        var filter = [
            {
                "name": "NOTASMARCADAS",
                "operator": "=",
                "value": notasMarcadas
            },
            {
                "name": "CONFIG",
                "operator": "=",
                "value": config
            }
        ];
        return localStorageEngine.download("df_saida#/downloadXML", filter);
    };

    this.enviarEmailPendenteNFsAceitas = function(notas, config) {
        var filter = [
            {
                "name": "NOTAS",
                "operator": "=",
                "value": notas
            },
            {
                "name": "CONFIG",
                "operator": "=",
                "value": config
            }
        ];
        return localStorageEngine.download("df_saida#/enviarEmailPendenteNFsAceitas", filter);
    };

    this.enviarEmailNF = function(nota, config, tipo) {
        var filter = [
            {
                "name": "NOTA",
                "operator": "=",
                "value": nota
            },
            {
                "name": "CONFIG",
                "operator": "=",
                "value": config
            },
            {
                "name": "TIPO",
                "operator": "=",
                "value": tipo
            }
        ];
        return localStorageEngine.download("df_saida#/enviarEmailNF", filter);
    };

    this.salvaProdutorRural = function(row) {
        return restEngine.post('df#/crud/nfprodrural/save', {
            "requestType": "Row",
            "row": row
        });
    };

    this.excluirProdutorRural = function(row) {
        return restEngine.post('df#/crud/nfprodrural/remove', {
            "requestType": "Row",
            "row": row
        });
    };

    this.getMaxNrseqcce = function(filial, lancamento){
        var filter = [
            {
                "name": "FILIAL",
                "operator": "=",
                "value": filial
            },
            {
                "name": "LANCAMENTO",
                "operator": "=",
                "value": lancamento
            }
        ];
        return localStorageEngine.download("df_saida#/maxNrseqcce", filter);
    };

    this.alteraDataEmissao = function(row) {
        return restEngine.post('df#/crud/notafiscal/save', {
            "requestType": "Row",
            "row": row
        });
    };
    
    this.requestModulesMetadata = () => {
        let modulesPackage = window.modulesPackage || "modules";
        return restEngine.requestMetaData(modulesPackage + "#json/modules.json").then(function (response) {
            return response.data;
        });
    };
    
    this.aprovarNotasTransmissao = (notas) => {
        let filter = [{"name": "NOTAS", "operator": "=", "value": notas}];
        
        return localStorageEngine.download("df#/aprovarNotasTransmissao", filter);
    };
    
    this.emailAutomaticoTransmissao = (notas, config) => {
        let filter = [{"name": "NOTAS",  "operator": "=", "value": notas},
                      {"name": "CONFIG", "operator": "=", "value": config}];
        
        return localStorageEngine.download("df#/emailAutomaticoTransmissao", filter);
    };
    
    this.consultaSituacaoNF = (row, config) => {
        let filter = [{"name": "NOTA",   "operator": "=", "value": row},
                      {"name": "CONFIG", "operator": "=", "value": config}];
        
        return localStorageEngine.download("df#/consultaSituacaoNF", filter);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Dfs49000_TransmissaoNfe_Repositories', dfs49000_TransmissaoNfe_Repositories);
});

// FILE: ./assets/modules/saida/assets/js/repositories/Dfs51000_InutilizacaoNotaFiscalEletronica_Repository.js
var Dfs51000_InutilizacaoNotaFiscalEletronica_Repository = function(restEngine) {

    this.inutilizarNF = function(rows,config) {
        var filtro = {};
        filtro.config = config;
        filtro.rows = rows;

        return restEngine.post('df_saida#/confirmarInutilizacaoNf', {
            "requestType": "DataSet",
            "dataset": [filtro]
        });
    };

    this.confirmarAgendamento = function(row,config) {
        var filtro = {};
        filtro.config = config;
        filtro.row = row;

        return restEngine.post('df_saida#/confirmarAgendamentoNf', {
            "requestType": "DataSet",
            "dataset": [filtro]
        });
    };

    this.buscaAgendamento = function(filtro) {
        return restEngine.post('df_saida#/gravaintinutnfe_agendamento', {
            "requestType": "FilterData",
            "filter": filtro
        });
    };

};


Configuration(function(ContextRegister) {
    ContextRegister.register('Dfs51000_InutilizacaoNotaFiscalEletronica_Repository', Dfs51000_InutilizacaoNotaFiscalEletronica_Repository);
});

// FILE: ./assets/modules/saida/assets/js/repositories/Dfs74000_CalculoReceitaBrutaSimplesNacional_Repository.js
var Dfs74000_CalculoReceitaBrutaSimplesNacional_Repository = function(restEngine) {

    this.buscaMesPosterior = function (row){
        return restEngine.post('df_saida#/buscaMesPosteriorReceitaBruta',{
            "requestType": "Row",
            "row": row
        });
    };

    this.calculaReceitaBrutaMes = function (row){
        return restEngine.post('df_saida#/calculaReceitaBrutaMes',{
            "requestType": "Row",
            "row": row
        });
    };

    this.bloquerMesReceita = function (row){
        return restEngine.post('df_saida#/bloquerMesReceita',{
            "requestType": "Row",
            "row": row
        });
    };

    this.buscaNotasFiscaisMes = function (row){
        return restEngine.post('df_saida#/buscaNotasFiscaisMes',{
            "requestType": "Row",
            "row": row
        });
    };

    this.saveReceitaBruta = function (row){
        return restEngine.post('df_saida#/saveReceitaBruta',{
            "requestType": "Row",
            "row": row
        });
    };

    this.deleteMesReceita = function (row){
        return restEngine.post('df_saida#/deleteMesReceita',{
            "requestType": "Row",
            "row": row
        });
    };

    this.validacoesSave = function (row){
        return restEngine.post('df_saida#/validacoesSave',{
            "requestType": "Row",
            "row": row
        });
    };

};


Configuration(function(ContextRegister) {
    ContextRegister.register('Dfs74000_CalculoReceitaBrutaSimplesNacional_Repository', Dfs74000_CalculoReceitaBrutaSimplesNacional_Repository);
});

// FILE: ./assets/modules/relatorios/assets/js/repositories/Adm03000_ReNotasFiscais_Repository.js
function Adm03000_ReNotasFiscais_Repository(restEngine) {
	this.parametersForFilial = function(filtro){
		return restEngine.post('df_relatorios#/paramfilial_forAdm03000', {
            "requestType": "FilterData",
            "filter": filtro
        });
	};

	this.parametersForNotaFiscal = function(filtro){
		return restEngine.post('df_relatorios#/notafiscal_forAdm03000', {
            "requestType": "FilterData",
            "filter": filtro
        });
	};

	this.geraRelatorio = function(row){
    return restEngine.post('df_relatorios#/geraRelatorioDoRe',
        {
            "requestType":"Row",
            "row": row
        }
    );
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('Adm03000_ReNotasFiscais_Repository', Adm03000_ReNotasFiscais_Repository);
});


// FILE: ./assets/modules/relatorios/assets/js/repositories/Dfe13000_Espelho_de_notas_fiscais_Repository.js
 function Dfe13000_Espelho_de_notas_fiscais_Repository(restEngine) {

	this.geraRelatorio = function(rows){
        return restEngine.post('df_relatorios#/gerarEspelhoNF',
            {
                "requestType":"Row",
                "row": rows
            }
        );
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('espelhoNF_Repository', Dfe13000_Espelho_de_notas_fiscais_Repository);
});



// FILE: ./assets/modules/relatorios/assets/js/repositories/Dfe16000_ConsistenciasAfNf_Repository.js
function Dfe16000_ConsistenciasAfNf_Repository(restEngine) {

	this.generateReport = function(row) {
        return restEngine.post('df_relatorios#/notafiscal_consistencias_af_nf', {
            "requestType": "Row",
            "row": row
        });
	};

	this.getMainQueryData = function(params) {
        return restEngine.post('df_relatorios#/notafiscal_consistencias_af_nf/getMainQueryData', {
            "requestType": "Row",
            "row": params
        });
	};

}

Configuration(function(ContextRegister) {
	ContextRegister.register('Dfe16000_ConsistenciasAfNf_Repository', Dfe16000_ConsistenciasAfNf_Repository);
});


// FILE: ./assets/modules/relatorios/assets/js/repositories/Dfe23000_ComprasRealizadasPorMes_Repository.js
var Dfe23000_ComprasRealizadasPorMes_Repository = function(restEngine) {
	this.getParamFilial = function(cdfilial){
        return restEngine.post('df#/paramfilial_paramvalidacoes', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "P_CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                }
            ]
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Dfe23000_ComprasRealizadasPorMes_Repository', Dfe23000_ComprasRealizadasPorMes_Repository);
});

// FILE: ./assets/modules/relatorios/assets/js/repositories/Dfe26000_InflacaoInterna_Repository.js
var Dfe26000_InflacaoInterna_Repository = function(restEngine) {

    this.generateReport = function(row){
        return restEngine.post('df_relatorios#/inflacao_interna_report',
            {
                "requestType":"Row",
                "row": row
            }
        );
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Dfe26000_InflacaoInterna_Repository', Dfe26000_InflacaoInterna_Repository);
});

// FILE: ./assets/js/services/AprovacaoLancamentos_Service.js
var aprovacaoLancamentos_Service = function(AprovacaoLancamentos_Repositories) {
    this.aprovarNotasMarcadas = function(notas){
        return AprovacaoLancamentos_Repositories.aprovarNotasMarcadas(notas);
    };
    
    this.cancelarAprovacaoNotasMarcadas = function(notas){
        return AprovacaoLancamentos_Repositories.cancelarAprovacaoNotasMarcadas(notas);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("AprovacaoLancamentos_Service", aprovacaoLancamentos_Service);
});

// FILE: ./assets/js/services/Df11000_Lancamento_Service.js
var df11000_Lancamento_Service = function(localStorageEngine, Df11000_Lancamento_Repositories, Dfs11000_LancamentoSaida_Repositories, ZHPromise, Lib00206_InfComplProduto_Service, ScreenService) {

    var self = this;
    var lancamento = {
        "CABECALHO": [],
        "ITENS": [],
        "IMPOSTO": [],
        "ENCARGO": [],
        "VENCIMENTO": [],
        "DADOSADICIONAIS": [],
        "RATEIO": [],
        'ITENSRATEIO': []
    };

    this.criaDetalhesLancamento = function(consultaLancamentoRow) {
        this.setDataSourceCabecalho(consultaLancamentoRow);
        lancamento.CABECALHO = consultaLancamentoRow;
        var filial = consultaLancamentoRow.CDFILIAL;
        var seqLancamento = consultaLancamentoRow.NRLANCTONF;
        var organizacao = consultaLancamentoRow.NRORG;
        var entrada_saida = consultaLancamentoRow.IDENTRSAIDOP;
        var statusNota = consultaLancamentoRow.IDSTATUS;
        return Df11000_Lancamento_Repositories.montaCorpoNota(filial, seqLancamento, entrada_saida, statusNota).then(function() {
            var infoLancamentoSaida = [
                getDataSourceItens(), getDataSourceImposto(), getDataSourceEncargo(), getDataSourceVencimento(), getDataSourceDadosAdicionais(), getDataSourceItensRateio(), getDataSourceRateio()
            ];

            return ZHPromise.all(infoLancamentoSaida).then(function(){
                return lancamento;
            });
        });
    };

    var getDataSourceCabecalho = function() {
        return localStorageEngine.getDataSource("df#/lancamento_nota_cabecalho").then(function(cabecalho){
            lancamento.CABECALHO = cabecalho[0];
            return cabecalho[0];
        });
    };

    var getDataSourceItens = function() {
        return localStorageEngine.getDataSource("df#/lancamento_nota_itens").then(function(itens){
            lancamento.ITENS = itens;
            return itens;
        });
    };

    var getDataSourceItensImpostos = function() {
        return localStorageEngine.getDataSource("df#/lancamento_nota_imposto_item").then(function(itensImpostos) {
            return itensImpostos;
        });
    };

    this.getDataSourceItensImpostos = function() {
        return getDataSourceItensImpostos();
    };

    var getDataSourceImposto = function() {
        return localStorageEngine.getDataSource("df#/lancamento_nota_imposto").then(function(imposto){
            lancamento.IMPOSTO = imposto;
            return imposto;
        });
    };

    var getDataSourceEncargo = function() {
        return localStorageEngine.getDataSource("df#/lancamento_nota_encargo").then(function(encargo){
            lancamento.ENCARGO = encargo;
            return encargo;
        });
    };

    var getDataSourceVencimento = function() {
        return localStorageEngine.getDataSource("df#/lancamento_nota_vencimento").then(function(vencimento){
            lancamento.VENCIMENTO = vencimento;
            return vencimento;
        });
    };

    var getDataSourceDadosAdicionais = function() {
        return localStorageEngine.getDataSource("df#/lancamento_nota_dadosadic").then(function(dadosAdic){
            lancamento.DADOSADICIONAIS = dadosAdic;
            return dadosAdic;
        });
    };
    
    var getDataSourceItensRateio = function() {
        return localStorageEngine.getDataSource("df#/lancamento_nota_itens_rateionf").then(function(itemRat){
            lancamento.ITENSRATEIO = itemRat;
            return itemRat;
        });
    };
    
    var getDataSourceRateio = function() {
        return localStorageEngine.getDataSource("df#/lancamento_nota_rateionf").then(function(rateio){
            lancamento.RATEIO = rateio;
            return rateio;
        });
    };
    
    this.getDataSourceRateio = function() {
        return getDataSourceRateio();
    };

    var buscaItens = function(consultaLancamentoRow) {
        return Df11000_Lancamento_Repositories.retornaItens(consultaLancamentoRow.CDFILIAL,
            consultaLancamentoRow.NRLANCTONF, consultaLancamentoRow.NRORG).then(function(itens){
            lancamento.ITENS = itens;
        });
    };

    var buscaEncargos = function(consultaLancamentoRow) {
        return Df11000_Lancamento_Repositories.retornaEncargos(consultaLancamentoRow.CDFILIAL,
            consultaLancamentoRow.NRLANCTONF, consultaLancamentoRow.NRORG).then(function(encargos){
            lancamento.ENCARGO = encargos;
        });
    };

    var buscaVencimentos = function(consultaLancamentoRow) {
        return Df11000_Lancamento_Repositories.retornaVencimentos(consultaLancamentoRow.CDFILIAL,
            consultaLancamentoRow.NRLANCTONF, consultaLancamentoRow.NRORG).then(function(vencimentos){
            lancamento.VENCIMENTO = vencimentos;
        });
    };

    var buscaDadosAdicionais = function(consultaLancamentoRow) {
        return Df11000_Lancamento_Repositories.retornaDadosAdicionais(consultaLancamentoRow.CDFILIAL,
            consultaLancamentoRow.NRLANCTONF, consultaLancamentoRow.NRORG).then(function(dadosAdicionais){
            lancamento.DADOSADICIONAIS = dadosAdicionais;
        });
    };

    this.buscaEncargosPadrao = function() {
        return Df11000_Lancamento_Repositories.retornaEncargosPadrao();
        //    .then(function(encargosPadrao){
        //    lancamento.ENCARGO = encargosPadrao;
        //});
    };

    this.buscaOperadorLogado = function() {
        var filter = [
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": "OPERADOR_LOGADO"
            }
        ];

        return localStorageEngine.download("df#/crud/operador", filter);
    };

    this.buscaDataProcessamento = function(filial) {
        return Df11000_Lancamento_Repositories.retornaDataProcessamento(filial);
    };

    this.buscaItensNotaOrigem = function(consultaLancamentoRow) {
        return Df11000_Lancamento_Repositories.retornaItensNotaOrigem(consultaLancamentoRow.CDFILIAL, consultaLancamentoRow.NRLANCTONF, consultaLancamentoRow.NRORG);
    };

    this.buscaQuantidadeDevolvida = function(cabecalhoRow, detalhesItemWidget) {
        var filial = cabecalhoRow.CDFILIAL;
        var lancamento = cabecalhoRow.NRLANCTONF;
        var filialOrigem = cabecalhoRow.CDFILINOTAORIG;
        var lancamentoOrigem = cabecalhoRow.NRLANCNOTAORIG;
        var codArvProd = detalhesItemWidget.currentRow.CDARVPROD;
        var lote = detalhesItemWidget.currentRow.NRLOTEESTQ;
        var subLote = detalhesItemWidget.currentRow.NRSUBLOTE;
        var valorUnitario = detalhesItemWidget.currentRow.VRUNITITEM;
        var organizacao = cabecalhoRow.NRORG;

        return Df11000_Lancamento_Repositories.retornaQuantidadeDevolvida(
            filial, lancamento, filialOrigem, lancamentoOrigem, codArvProd, lote, subLote, valorUnitario, organizacao
        );
    };

    this.removeDataSourceLancamentoNota = function() {
        const dataSources = [
            'lancamento_nota',
            'lancamento_nota_cabecalho',
            'lancamento_nota_itens',
            'itemproduto_notafiscal_origem',
            'lancamento_nota_imposto_item',
            'lancamento_nota_imposto',
            'lancamento_nota_encargo',
            'lancamento_nota_dadosadic',
            'lancamento_nota_vencimento',
            'lancamento_nota_itens_rateionf',
            'lancamento_nota_rateionf'
        ];

        let promises = dataSources.map(dataSource => localStorageEngine.remove(`df#/${dataSource}`));

        return ZHPromise.all(promises);
    };

    this.inicializaLancamento = function(){
        lancamento.CABECALHO = [];
        lancamento.ITENS = [];
        lancamento.IMPOSTO = [];
        lancamento.ENCARGO = [];
        lancamento.VENCIMENTO = [];
        lancamento.DADOSADICIONAIS = [];
        lancamento.RATEIO = [];
        lancamento.ITENSRATEIO = [];
    };

    this.atualizaLancamento = function(){
        var infoLancamentoSaida = [
            getDataSourceCabecalho(), getDataSourceItens(), getDataSourceImposto(), getDataSourceEncargo(), getDataSourceVencimento(), getDataSourceDadosAdicionais(), getDataSourceItensRateio(), getDataSourceRateio()
        ];

        return ZHPromise.all(infoLancamentoSaida).then(function(){
            return lancamento;
        });
    };

    this.setDataSourceLancamentoNota = function() {
        return localStorageEngine.save("df#/lancamento_nota", lancamento);
    };

    this.setDataSourceCabecalho = function(cabecalho) {
        return localStorageEngine.setDataSource("df#/lancamento_nota_cabecalho", cabecalho);
    };

    this.setDataSourceItem = function(itens) {
        return localStorageEngine.save("df#/lancamento_nota_itens", itens);
    };
    
    this.setDataSourceRateio = function(rateio) {
        return localStorageEngine.save("df#/lancamento_nota_rateionf", rateio);
    };

    this.setDataSourceItensRateio = function(itensRateio) {
        return localStorageEngine.save("df#/lancamento_nota_itens_rateionf", itensRateio);
    };

    this.getItensRateio = function(cdfilial, nrLancto){
        return Df11000_Lancamento_Repositories.getItensRateio(cdfilial, nrLancto);
    };

    this.setDataSourceItensImpostos = function(impostos) {
        return localStorageEngine.save("df#/lancamento_nota_imposto_item", impostos);
    };

    this.excluiItemDataSource = function(seqItem) {
        var filter = [
            {
                "name": "NRSEQITPROD",
                "operator": "=",
                "value": seqItem
            }
        ];
        return localStorageEngine.remove("df#/lancamento_nota_itens", filter);
    };
    
    this.excluiRateioDataSource = function(rowRateio) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": rowRateio.CDFILIAL
            },
            {
                "name": "NRLANCTONF",
                "operator": "=",
                "value": rowRateio.NRLANCTONF
            },
            {
                "name": "NRSEQITPROD",
                "operator": "=",
                "value": rowRateio.NRSEQITPROD
            },
            {
                "name": "CDFILICCUST",
                "operator": "=",
                "value": rowRateio.CDFILICCUST
            },
            {
                "name": "CDCENTCUSNF",
                "operator": "=",
                "value": rowRateio.CDCENTCUSNF
            }
        ];
        return localStorageEngine.remove("df#/lancamento_nota_rateionf", filter);
    };
    
    this.excluiMultiRateioDataSource = function(datasetRateio) {
        var arrayCDFILICUST = [];
        var arrayCDCENTCUSNF = [];
        var arrayNRSEQITPROD =[];
        datasetRateio.forEach(row=> {
            arrayCDFILICUST.push(row.CDFILICCUST);
            arrayCDCENTCUSNF.push(row.CDCENTCUSNF);
            arrayNRSEQITPROD.push(row.NRSEQITPROD);
        });
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": datasetRateio[0].CDFILIAL
            },
            {
                "name": "NRLANCTONF",
                "operator": "=",
                "value": datasetRateio[0].NRLANCTONF
            },
            {
                "name": "NRSEQITPROD",
                "operator": "IN",
                "value": arrayNRSEQITPROD
            },
            {
                "name": "CDFILICCUST",
                "operator": "IN",
                "value": arrayCDFILICUST
            },
            {
                "name": "CDCENTCUSNF",
                "operator": "IN",
                "value": arrayCDCENTCUSNF
            }
        ];
        return localStorageEngine.remove("df#/lancamento_nota_rateionf", filter);
    };


    this.setDataSourceImposto = function(imposto) {
        return localStorageEngine.setDataSource("df#/lancamento_nota_imposto", imposto);
    };

    this.setDataSourceEncargo = function(encargo) {
        return localStorageEngine.setDataSource("df#/lancamento_nota_encargo", encargo);
    };

    this.setDataSourceVencimento = function(vencimento) {
        vencimento.SEQVENCIMENTO = vencimento.NRSEQVENREC || vencimento.NRSEQVENPAG;
        return localStorageEngine.save("df#/lancamento_nota_vencimento", vencimento);
    };

    this.excluiVencimentoDataSource = function(seqVencimento) {
        var filter = [
            {
                "name": "SEQVENCIMENTO",
                "operator": "=",
                "value": seqVencimento
            }
        ];
        return localStorageEngine.remove("df#/lancamento_nota_vencimento", filter);
    };

    this.setDataSourceDadosAdicionais = function(dadosadic) {
        return localStorageEngine.save("df#/lancamento_nota_dadosadic", dadosadic);
    };

    this.excluiDadosAdicionaisDataSource = function(seqDadosAdicionais) {
        var filter = [
            {
                "name": "NRDADOADIC",
                "operator": "=",
                "value": seqDadosAdicionais
            }
        ];
        return localStorageEngine.remove("df#/lancamento_nota_dadosadic", filter);
    };

    this.salvaCabecalho = function(cabecalho) {
        return Df11000_Lancamento_Repositories.salvaCabecalho(cabecalho);
    };

    this.salvaItemnotafisc = function(row) {
        return Df11000_Lancamento_Repositories.salvaItemnotafisc(row);
    };

    this.calculaDataVencimento = function(numDias, dataEmissao){
        return moment(dataEmissao, 'DD/MM/YYYY').add(numDias, "days").format("DD/MM/YYYY");
    };

    this.calculaNumeroDiasVencimento = function(dataVencimento, dataEmissao){
        return moment(dataVencimento, "DD/MM/YYYY").diff(moment(dataEmissao, "DD/MM/YYYY"), 'days');
    };

    this.calculaValorTotal = function(quantidade, valorUnitario) {
        quantidade = removeMaskString(quantidade);
        valorUnitario = removeMaskString(valorUnitario);
        if (valorUnitario > 0 && quantidade > 0 )
            return quantidade * valorUnitario;
            
        return 0;
    };

    var removeMaskString = function(field) {
        return typeof(field) === "string" ? parseFloat(field.replace(",", ".")) : field;
        //field = (typeof(field) === "string" && field.indexOf('.') === -1) ? $.currency.unmaskString(field) : field;
        //field = Math.abs(field);
        //return field;
    };

    this.calculaValorUnitario = function(quantidade, valorTotal) {
        // Atribui valor 0 caso a quantidade/valorTotal passados sejam valores
        // inválidos, como null ou undefined.
        quantidade = removeMaskString(quantidade) || 0;
        valorTotal = removeMaskString(valorTotal) || 0;

        if (valorTotal > 0 && quantidade > 0 )
            return valorTotal / quantidade;

    };
    
    this.verificaNotaDevolucao = function(cabecalho) {
        return this.verificaUtilizaNotaOrigem(cabecalho) &&
               !this.verificaComplemento(cabecalho);
    };
    
    this.verificaComplemento = function (cabecalho) {
        return cabecalho.CDTIPOOPER == '03'; //REGRA PODE MUDAR!;
    };

    this.verificaUtilizaNotaOrigem = function(cabecalho) {
        return ((cabecalho.IDOPERDEVONF == 'S') &&
                !!cabecalho.NRINSJURNF);
    };

    var validacoesNotaOrigem = function(quantidade, quantidadeOrigem) {
        return (quantidade > quantidadeOrigem);
    };

    var validaCamposItemIgualZero = function(quantidade, valorUnitario, valorTotal) {
        var reject = [];
        if (parseFloat(quantidade) === 0)
            reject.push('"Quantidade"');

        if (parseFloat(valorUnitario) === 0)
            reject.push('"Vr. Unitário"');

        if (parseFloat(valorTotal) === 0)
            reject.push('"Valor Total"');
        return reject;
    };

    var validaDescontoMaiorValorTotal = function(desconto, valorTotal) {
        desconto = desconto || 0;
        valorTotal = valorTotal || 0;
        return (parseFloat(desconto) <= parseFloat(valorTotal));
    };

    this.getProxSeqItens = function() {
        return setMaxSequencialItens().then(function(maxSeqItens){
            var proxSeqItens = parseInt(maxSeqItens) + 1;
            var str = "" + proxSeqItens;
            var pad = "0000";
            return pad.substring(0, pad.length - str.length) + str;
        });
    };

    var setMaxSequencialItens = function() {
        return getDataSourceItens().then(function(itens){
            var maxSeqItens = '0000';
            itens.forEach(function(item){
                if(parseInt(item.NRSEQITPROD) > parseInt(maxSeqItens))
                    maxSeqItens = item.NRSEQITPROD;
            });
            return maxSeqItens;
        });
    };

    this.salvaItem = function(cabecalho, item) {
        if(item.hasOwnProperty('__rightActions')){
            delete item.__rightActions;
            delete item.__leftActions;
        }
        //TODO - Agrupar as validações das notas de origem em somente uma função.
        item.VRDESCITEM = item.VRDESCITEM || 0;
        if(this.verificaUtilizaNotaOrigem(cabecalho) && (cabecalho.IDNFAJUSTE != "S") && !this.verificaComplemento(cabecalho) ){
            if (validacoesNotaOrigem(item.QTITEMPROD, item.QTITEMORIG))
                return ZHPromise.reject("Quantidade informada deve ser menor ou igual a quantidade de origem");

            if(item.__is_new){
                if (validacoesNotaOrigem(parseFloat(item.QTITEMPROD) + parseFloat(item.QTITEMDEVO), item.QTITEMORIG))
                    return ZHPromise.reject("Quantidade deve ser menor ou igual à quantidade original. Quantidade já devolvida: " + item.QTITEMDEVO.toString());
            }
            else{
                var quatidadeJaDevolvida = parseFloat(item.QTITEMDEVO) - parseFloat(item.QTITEMPRODBEFOREEDIT);
                if (validacoesNotaOrigem(parseFloat(item.QTITEMPROD) + parseFloat(quatidadeJaDevolvida), item.QTITEMORIG))
                    return ZHPromise.reject("Quantidade deve ser menor ou igual à quantidade original. Quantidade já devolvida: " + quatidadeJaDevolvida.toString());
            }
        }
        
        if (!this.verificaNotaDevolucao(cabecalho) && (cabecalho.IDNFAJUSTE != "S") && !(this.verificaComplemento(cabecalho))) {
            var reject = validaCamposItemIgualZero(item.QTITEMPROD, item.VRUNITITEM, item.VRTOTAITEM);
            if (reject.length > 0)
                return ZHPromise.reject(reject); 
        }

        if (!validaDescontoMaiorValorTotal(item.VRDESCITEM, item.VRTOTAITEM))
            return ZHPromise.reject("Desconto do item deve ser menor que o valor total do item");

        if(item.__is_new) {
            return Df11000_Lancamento_Repositories.salvaItemProcedure(item);
        } else
            return Df11000_Lancamento_Repositories.salvaItem(item);
    };
    
    this.salvaRateio = function(excluirDataset){
        var deferred = ZHPromise.defer();
        var somaRateios = 0;
        var vrContProdutos = 0;
        
        getDataSourceItensRateio().then(itensRateio => {
            getDataSourceRateio().then(rateio => {

                itensRateio.forEach(itens =>{
                    vrContProdutos += Number(itens.VRCONTITEM);
                   rateio.forEach(row =>{
                       if(itens.NRSEQITPROD === row.NRSEQITPROD){
                            somaRateios += Number(row.VRRATENF);
                            row.VRPERRAT = (Number(row.VRRATENF) * 100) / Number(itens.VRCONTITEM);
                       }
                   }) ;
                });
                
                if (somaRateios.toFixed(2) === vrContProdutos.toFixed(2)){
                    Df11000_Lancamento_Repositories.excluiRateio(excluirDataset).then(()=> {
                        Df11000_Lancamento_Repositories.salvaRateio(rateio).then(() => {
                            this.setDataSourceRateio(rateio)
                            .then(this.getDataSourceRateio());
                            deferred.resolve();    
                        }, error => {
                            deferred.reject(error.data.error);
                        });
                    });
                }else{
                    deferred.reject("Total do valor dos rateios deve ser igual ao valor contábil.");
                }
            });    
        });
        
        return deferred;
    };

    this.excluiItem = function(item){
        if(item.hasOwnProperty('__rightActions')){
            delete item.__rightActions;
            delete item.__leftActions;
        }
        return Df11000_Lancamento_Repositories.excluiItem(item);
    };
    
    this.excluiRateio = function(rateio){
        if(rateio.hasOwnProperty('__rightActions')){
            delete rateio.__rightActions;
            delete rateio.__leftActions;
        }
        return Df11000_Lancamento_Repositories.excluiRateio(rateioitem);
    };

    this.salvaImposto = function(imposto) {
        return Df11000_Lancamento_Repositories.salvaImposto(imposto);
    };

    this.salvaEncargo = function(encargos) {
        return Df11000_Lancamento_Repositories.salvaEncargo(encargos);
    };

    this.getProxSeqDadosAdicionais = function() {
        return setMaxSequencialDadosAdicionais().then(function(maxSeqDadosAdicionais){
            var proxSeqDadosAdicionais = parseInt(maxSeqDadosAdicionais) + 1;
            var str = "" + proxSeqDadosAdicionais;
            var pad = "000";
            return pad.substring(0, pad.length - str.length) + str;
        });
    };

    var setMaxSequencialDadosAdicionais = function() {
        return getDataSourceDadosAdicionais().then(function(dadosAdicionais){
            var maxSeqDadosAdicionais = '000';
            dadosAdicionais.forEach(function(dados){
                if(parseInt(dados.NRDADOADIC) > parseInt(maxSeqDadosAdicionais))
                    maxSeqDadosAdicionais = dados.NRDADOADIC;
            });
            return maxSeqDadosAdicionais;
        });
    };

    this.salvaDadosAdicionais = function(dadosAdicionais) {
        return Df11000_Lancamento_Repositories.salvaDadosAdicionais(dadosAdicionais);
    };

    this.excluiDadosAdicionais = function(dadosAdicionais){
        return Df11000_Lancamento_Repositories.excluiDadosAdicionais(dadosAdicionais);
    };

    var converteDataParaComparacao = function(data) {
        return moment(data, 'DD/MM/YYYY').format('MM-DD-YYYY');
    };

    this.validaDataVencimento = function(dataLancamento, dataVencimento, dataProcessamento) {
        dataLancamento = converteDataParaComparacao(dataLancamento);
        dataVencimento = converteDataParaComparacao(dataVencimento);

        if (moment(dataVencimento).isBefore(dataLancamento) && !moment(dataLancamento).isSame(dataVencimento)) {
            return "Data de Vencimento deve ser maior ou igual a data de saída";
        }
        dataProcessamento = converteDataParaComparacao(dataProcessamento);
        if (moment(dataVencimento).isBefore(dataProcessamento)  && !moment(dataProcessamento).isSame(dataVencimento))
            return "Data de Vencimento informada anterior a data de processamento";

        if (Math.abs(moment(dataLancamento).diff(moment(dataVencimento), 'days')) > 60)
            return "Data de Vencimento superior a %1 dias da data de saída".replace("%1", 60);
    };
    
    /*this.validaDataIgual = function (dataAtual, datasource){
        var vencimentos = datasource.map(f => f.DTATUAVENPAG);
        var count = 0;
        vencimentos.forEach(function(data) { if(dataAtual === data) count++;  });
        if (count > 1) return true; else return false;
        
    };*/
    

    this.validaFecharNota = function(cabecalhoRow){
        var deferred = ZHPromise.defer();
        var erro = false;
        if(lancamento.ITENS.length === 0 && lancamento.CABECALHO.CDTIPOOPER !== "03"){
            deferred.reject("Item(s) da nota fiscal devem ser informado(s)");
                erro = true;
        }else{
            Lib00206_InfComplProduto_Service.getParamFilial(lancamento.ITENS[0].CDFILIMOVI).then(function(paramfilial){
                paramfilial = paramfilial.dataset.paramfilial[0];
                if(paramfilial.IDCTRLESTQ != null && (paramfilial.IDUTILLOTE == 'S' || paramfilial.IDUTILALMOX == 'S') && lancamento.CABECALHO.CDTIPOOPER != "03" && lancamento.CABECALHO.IDNFAJUSTE != "S") {   
                    lancamento.ITENS.forEach(produto => {
                        if(produto.CDALMOXARIFE === ' ' && paramfilial.IDUTILALMOX == 'S'){
                            deferred.reject("Almoxarifado do Produto: '"+produto.NMPRODUTO+"' deve ser informado.");
                            erro = true;
                        }
                        if(produto.CDLOCALESTOQ === ' ' && paramfilial.IDUTILLCESTQ == 'S'){
                            deferred.reject("Local de estoque do produto: '"+produto.NMPRODUTO+"' deve ser informado.");
                            erro = true;
                        }
                        if(produto.NRLOTEESTQ === ' ' && paramfilial.IDUTILLOTE == 'S'){
                            deferred.reject("Lote do produto: '"+produto.NMPRODUTO+"' deve ser informado.");
                            erro = true;
                        }
                    });
                }
            if(cabecalhoRow.IDUTILVENCOBR === 'S' && lancamento.VENCIMENTO.length === 0){
                deferred.reject("Vencimento deve ser informado"); 
                erro = true;
            }
                
            if(!erro)
                deferred.resolve();
            });
        }
            
        return deferred;
    };

    this.fecharNota = function(cabecalhoRow, forcaFechamento = false) {
        this.criaDetalhesLancamento(cabecalhoRow);
        return Df11000_Lancamento_Repositories.fecharNota(lancamento.CABECALHO, lancamento.VENCIMENTO, forcaFechamento);
    };
    
    this.geraRateio = function(geraRatAut = 'N', realizaManut = 'P', onEnter = 'N') {
        var filial = lancamento.CABECALHO.CDFILIAL,
        nrLancamento = lancamento.CABECALHO.NRLANCTONF, 
        idEntrSaid = lancamento.CABECALHO.IDENTRSAIDOP, 
        rateioNF = lancamento.RATEIO;
        
        if(onEnter == 'N'){
            if(rateioNF.length > 0){
                geraRatAut = 'S';
                ScreenService.confirmMessage(
                    ScreenService.i18n('Deseja regerar o rateio por produto de forma proporcional?'),
                            'question',
                            (sim) => Df11000_Lancamento_Repositories.geraRateio(filial, nrLancamento, idEntrSaid, rateioNF, geraRatAut, 'N'),
                            (nao) => Df11000_Lancamento_Repositories.geraRateio(filial, nrLancamento, idEntrSaid, rateioNF, geraRatAut, 'S')
                );
            }else{
                geraRatAut = 'S'; realizaManut = 'N';
                return Df11000_Lancamento_Repositories.geraRateio(filial, nrLancamento, idEntrSaid, rateioNF, geraRatAut, realizaManut);
            }
        }else{
             return Df11000_Lancamento_Repositories.geraRateio(filial, nrLancamento, idEntrSaid, [], 'N', 'P');
        }
        
        
    };

    this.salvaImpostosTmpItem = function(impostosItem) {

        var impostosTemp = [];
        var trabEntity = {
            'CDFILIMPOIT': 'CDFILIAL',
            'NRLANCIMPOIT': 'NRLANCTONF',
            'NRSEQIMPOIT': 'NRSEQITPROD',
            'CDIMPOSTOIT': 'CDIMPOSTO',
            'VRALIQIMPOIT': 'VRALIQIMPOIT',
            'VRIMPOSIT': 'VRIMPOSTOIT',
            'IDVALORINFO': 'IDVALORINFIT',
            'VRADICALIQFECP' : 'VRADICALIQFECP'
        };
        var columns = Object.keys(trabEntity);

        impostosTemp = Util.clone(impostosItem);
        
        impostosTemp = impostosTemp.filter( imposto => imposto.NRITPROIMPOS == '1');

        impostosTemp.forEach(function(imposto) {
            columns.forEach(function(column) {
                if (!(column in imposto)) {
                    imposto[column] = imposto[trabEntity[column]];
                    delete imposto[trabEntity[column]];
                }
            });
            imposto.NRSEQITPRO = 1;
            imposto.IDAVISIMPOIT = 'S';
        });
        return Df11000_Lancamento_Repositories.salvaImpostosTmpItem(impostosTemp);
    };
    
    this.salvaImpostosItem = function(impostosItem){
        var impostosTemp = [];
        var itemproimposEntity= {
            'CDFILIAL': 'CDFILIAL',
            'NRLANCTONF': 'NRLANCTONF',
            'NRSEQITPROD': 'NRSEQITPROD',
            'CDIMPOSTO': 'CDIMPOSTO',
            'VRALIQITEM': 'VRALIQIMPOIT',
            'VRIMPOSTOIT': 'VRIMPOSTOIT',
            'IDVALORINFIT': 'IDVALORINFIT'
        };
        var columns = Object.keys(itemproimposEntity);
        impostosTemp = Util.clone(impostosItem);
        impostosTemp.forEach(function(imposto) {
            columns.forEach(function(column) {
                if (!(column in imposto)) {
                    imposto[column] = imposto[itemproimposEntity[column]];
                    delete imposto[itemproimposEntity[column]];
                }
            });
            imposto.NRSEQITPRO = 1;
            imposto.IDAVISIMPOIT = 'S';
        });
        return Df11000_Lancamento_Repositories.salvaImpostosItem(impostosTemp);
    };

    this.calculaTotalImpostos = function() {
        var impostosItens = getDataSourceItensImpostos();
        return impostosItens.then(function(impostos) {
            var calcImposto = {};
            impostos.forEach(function(imposto) {
                var cdimposto = imposto.CDIMPOSTO;
                var valorImposto = imposto.VRIMPOSTOIT;
                if (cdimposto in calcImposto) {
                    calcImposto[cdimposto].VRIMPOSTONF = (parseFloat(calcImposto[cdimposto].VRIMPOSTONF) + parseFloat(valorImposto));
                } else {
                    delete imposto.NRSEQITPROD;
                    delete imposto.VRIMPOSTOIT;
                    calcImposto[cdimposto] = imposto;
                    calcImposto[cdimposto].VRIMPOSTONF = valorImposto;
                }
            });
            var calcTotalImpostos = [];
            for (var obj in calcImposto) {
                calcTotalImpostos.push(calcImposto[obj]);
            }
            return self.setDataSourceImposto(calcTotalImpostos);
        }.bind(self));
    };

    var setMaxSequencialVencimento = function() {
        return getDataSourceVencimento().then(function(vencimentos){
            var maxSeqVencimento = '00';
            vencimentos.forEach(function(vencimento) {
                var seqVencimento = vencimento.NRSEQVENPAG || vencimento.NRSEQVENREC;
                if (parseInt(seqVencimento) > parseInt(maxSeqVencimento))
                    maxSeqVencimento = seqVencimento;
            });
            return maxSeqVencimento;
        });
    };

    this.getProxSeqVencimento = function() {
        return setMaxSequencialVencimento().then(function(maxSeqVencimento) {
            var proxSeqVencimento = parseInt(maxSeqVencimento) + 1;
            var str = "" + proxSeqVencimento;
            var pad = "00";
            return pad.substring(0, pad.length - str.length) + str;
        });
    };

    this.validaDataEntradaEmissao = function(dtentrada, dtemissao) {
        dtentrada = moment(dtentrada, 'DD/MM/YYYY');
        dtemissao = moment(dtemissao, 'DD/MM/YYYY');

        var diferencaDatas = dtentrada.diff(dtemissao);
        if (diferencaDatas < 0)
            return 'A data de entrada da nota deve ser maior ou igual à data de emissão da mesma';
    };

    this.somaTotalItens = function(itens) {
        var valorTotal = 0;

        itens.forEach(function(item) {
            valorTotal += item.VRTOTAITEM;
        });
        return valorTotal;
    };

    this.validaTotalEncargos = function(encargos, valorTotalItens) {
        var valorTotalEncargos = 0;
        valorTotalItens = valorTotalItens || 0;

        encargos.forEach(function(encargo) {
            if (encargo.IDENCARGO === "D")
                valorTotalEncargos += parseFloat(encargo.VRITEMENCA);
            else
                valorTotalItens += parseFloat(encargo.VRITEMENCA);
        });

        if (valorTotalEncargos > valorTotalItens)
            return false;

        return true;
    };


    this.validaStatusVencimento = function(widget){
        var fieldStatus = widget.getField('IDSTATPAG');
        if(widget.dataSource.data.length > 0 ){
            if(!widget.dataSource.data[0].EXISTEBOLETO){
                var cdfilial = widget.dataSource.data[0].CDFILIAL;
                var nrlanbolpag = widget.dataSource.data[0].NRLANBOLPAG;
                Df11000_Lancamento_Repositories.getStatusVencimento(cdfilial, nrlanbolpag).then(
                    response => {
                        var existeBoleto = response.dataset.status_vencimento[0];
                        if(existeBoleto == 'S'){
                            fieldStatus.dataSource.data[0].DSSTATPAG = "Liberado";
                        }
                        else{
                            fieldStatus.dataSource.data[0].DSSTATPAG = "Em Aberto";
                        }
                        fieldStatus.reload();
                        //widget.reload();
                    }
                ).catch(
                    error => {
                        ScreenService.showMessage(error.data.error);
                    }
                );
            }
            else{
                if(widget.dataSource.data[0].EXISTEBOLETO == 'S'){
                    fieldStatus.dataSource.data[0].DSSTATPAG = "Liberado";
                }
                else{
                    fieldStatus.dataSource.data[0].DSSTATPAG = "Em Aberto";
                }
                fieldStatus.reload();

            }
        }
    };

    this.retornaImpostosRetidos = function() {
        return self.calculaTotalImpostos().then(function(totalImpostos) {
            return totalImpostos.filter(function(imposto) {
                    return (imposto.IDDESCRFONTE === 'R' && imposto.IDTRATIMPO == 'R');
            });
        });
    };

    this.somaTotalImpostosRetidos = function() {
        var valorImpososRetidos = 0;
        return self.retornaImpostosRetidos().then(function(impostos) {
            impostos.forEach(function(imposto) {
                    valorImpososRetidos += parseFloat(imposto.VRIMPOSTONF);
                }
            );
            return valorImpososRetidos;
        });
    };

    this.calculaValorVencimento = function(dataSourceVencimento, valorTotalNota, valorImpostosRetidos) {
        var valorTotalVenc = 0, valorSegerido = 0;

        valorTotalNota -= valorImpostosRetidos;

        dataSourceVencimento.forEach(function(value) {
            valorTotalVenc += parseFloat(value.VRATUAPAG || 0);
        });
        valorSegerido = valorTotalNota - valorTotalVenc;

        return (valorSegerido > 0) ? valorSegerido : 0;
    };
    
    this.validaValorVencimento = function(dataSourceVencimento, valorTotalNota, valorImpostosRetidos) {
        var valorTotalVenc = 0, valorSegerido = 0;

        valorTotalNota -= valorImpostosRetidos;

        dataSourceVencimento.forEach(function(value) {
            valorTotalVenc += parseFloat(value.VRATUAPAG || 0);
        });
        valorSegerido = valorTotalNota - valorTotalVenc;

        return valorSegerido;
    };
    
    this.setDefaultNovaNota = function(defaultNota) {
        return localStorageEngine.setDataSource("df#/defaultNovaNota", defaultNota);
    };
    
    this.getDefaultNovaNota = function() {
        return localStorageEngine.getDataSource("df#/defaultNovaNota");
    };
    
    this.setDefaultNovaNotaSaida = function(defaultNota) {
        return localStorageEngine.setDataSource("df#/defaultNovaNotaSaida", defaultNota);
    };
    
    this.getDefaultNovaNotaSaida = function() {
        return localStorageEngine.getDataSource("df#/defaultNovaNotaSaida");
    };
    
    this.deleteItemProImpos = function(imposto){
        return Df11000_Lancamento_Repositories.deleteItemProImpos(imposto);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Df11000_Lancamento_Service", df11000_Lancamento_Service);
});


// FILE: ./assets/js/services/Df56000_AprovacaoLancamentos_Service.js
var df56000_AprovacaoLancamentos_Service = function(Df56000_AprovacaoLancamentos_Repositories) {
    this.salvarEdicaoLancamento = function(row) {
        return Df56000_AprovacaoLancamentos_Repositories.salvarEdicaoLancamento(row);
    };
    
    this.validaEdicaoLancamento = function(currentRow){
        return Df56000_AprovacaoLancamentos_Repositories.validaEdicaoLancamento(currentRow);
    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Df56000_AprovacaoLancamentos_Service", df56000_AprovacaoLancamentos_Service);
});

// FILE: ./assets/js/services/Ger00400_Imposto_Service.js
function Ger00400_Imposto_Service(Ger00400_Imposto_Repository) {

	this.salvar = function(row){
		return Ger00400_Imposto_Repository.save(row);
	};

	this.salvarContabilizacao = function(row){
		return Ger00400_Imposto_Repository.saveContabilizacao(row);
	};

	this.salvarBaseCalculo = function(row){
		return Ger00400_Imposto_Repository.saveBaseCalculo(row);
	};

	this.salvarOutrosImpostosTipoOperacao = function(row){
		return Ger00400_Imposto_Repository.saveOutrosImpostosTipoOperacao(row);
	};

	this.salvarApuracaoImpostos = function(row){
		return Ger00400_Imposto_Repository.saveApuracaoImpostos(row);
	};

	this.verificaRange = function(row){
		return Ger00400_Imposto_Repository.verificaRange(row);
	};
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger00400_Imposto_Service', Ger00400_Imposto_Service);
});

// FILE: ./assets/js/services/Ger01400_Mensagem_Service.js
var Ger01400_Mensagem_Service = function (ZHPromise, Ger01400_Mensagem_Repository) {
    var self = this;

    var requestValidations = function(CDMENSAGEM) {
        var deferred = ZHPromise.defer();

        Ger01400_Mensagem_Repository.getValidations(CDMENSAGEM).then(function(response) {
            if(response.dataset.mensagem_validation){
                deferred.resolve(response.dataset.mensagem_validation);
            } else {
                deferred.reject({"error": {"message": "Request error."}});
            }
        });

        return deferred;
    };

    self.validateDeletion = function(row) {
        var deferred = ZHPromise.defer();

        requestValidations(row.CDMENSAGEM).then(function(validations) {
            var blockDeletion = (validations.IDBLOCKDELETION == 'S');
            
            deferred.resolve(blockDeletion);
        }, function(error) {
            deferred.reject(error);
        });

        return deferred;
    };
};

Configuration (function(ContextRegister) {
    ContextRegister.register('Ger01400_Mensagem_Service', Ger01400_Mensagem_Service);
});     

// FILE: ./assets/js/services/Ger03400_ReplicacaoDocumentos_Service.js
function Ger03400_ReplicacaoDocumentos_Service(Ger03400_ReplicacaoDocumentos_Repository)  {

    
    this.verificaControle = function(CDFILIAL){
        return Ger03400_ReplicacaoDocumentos_Repository.verificaControle(CDFILIAL); 
    };
    
    this.verificaNRREPLICDOC = function(CDFILIAL,NRREPLICDOC) {
        return Ger03400_ReplicacaoDocumentos_Repository.verificaNRREPLICDOC(CDFILIAL,NRREPLICDOC); 
    };
    
    this.verificaRelacaoFilial = function(CDFILIAL) {
        return Ger03400_ReplicacaoDocumentos_Repository.verificaRelacaoFilial(CDFILIAL); 
    };
    this.verificaRelacaoFilialEmpresa = function(P_CDFILIAL, P_CDEMPRESA) {
        return Ger03400_ReplicacaoDocumentos_Repository.verificaRelacaoFilialEmpresa(P_CDFILIAL, P_CDEMPRESA);
    };
    
    
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger03400_ReplicacaoDocumentos_Service', Ger03400_ReplicacaoDocumentos_Service);
});

// FILE: ./assets/js/services/Ger03600_Rateio_Produto_Service.js
function Ger03600_Rateio_Produto_Service(metaDataFactory, templateManager, General, Ger03600_Rateio_Produto_Repository, Df11000_Lancamento_Service, Df11000_Lancamento_Repositories, eventEngine, ScreenService) {
	
	var self = this;
	var excluirDataset = [];
	
	this.alteraVisibilidadeRateio = function(nota) {
        var notaRow = nota.row;
        if((notaRow.IDBLOQUEADO !== 'S' || notaRow.IDDOCFINANC === 'L') && notaRow.IDSTATUS !== 'C'){
            nota.owner.getAction('actRateioProduto').isVisible = true;
        }else{
            nota.owner.getAction('actRateioProduto').isVisible = false;
        }
    };
	
	this.abreRateioProduto = function(nota, widget = null) {
        var widgetNota = widget ? widget : nota.owner.widget;
        var widgetRateioProduto = templateManager.container.getWidget('GER03600RateioPorProduto');
        
        widgetRateioProduto.isVisible = true;
        widgetNota.openTab('GER03600RateioPorProduto');
        ScreenService.openSwipe(widgetRateioProduto).then(
            (success) => {
                General.backOrCancel(nota);
                this.controlaVisibilidadeRateio(nota, widget);
            }
        );
    };
    
    this.controlaVisibilidadeRateio = function(args, widget = null){
        widget = args ? args.owner : widget;
        
        
        switch(widget.name){
            case 'GER03600RateioPorProduto' :
                widget.widgets[0].getAction('actNovoRateio').isVisible= true;
                widget.widgets[0].getAction('actExcluirMultiRateio').isVisible= true;
                widget.widgets[0].getAction('actExcluirTodosRateios').isVisible= false;
                break;
            case 'actVoltarRateioList' :
            case 'actVoltarRateioView' :
                widget.getParent().getAction('actNovoRateio').isVisible= false;
                widget.getParent().getAction('actExcluirMultiRateio').isVisible= false;
                widget.getParent().getAction('actExcluirTodosRateios').isVisible= true;
                break;
            case 'actRateioProduto' :
                var widgetNota = args.owner.widget;
                widgetNota.enabled = false;
                General.changeWidgetAcessibility(widgetNota.widgets, false);
                break;
            case 'actFecharRateio' :
                var lancamentoRateioWidget = args.owner.widget;
                var widgetItensRateio = lancamentoRateioWidget.parent;
                var cabecalhoWidget = widgetItensRateio.parent;
                lancamentoRateioWidget.getAction('actFecharRateio').isVisible = false;
                widgetItensRateio.isVisible = false;
                cabecalhoWidget.enabled = true;
                General.changeWidgetAcessibility(cabecalhoWidget.widgets, true);
                break;
            case 'cabecalho' :
                General.changeWidgetAcessibility(widget.widgets, true);
                widget.container.getWidget('GER03600RateioPorProduto').isVisible = false;
                widget.enabled = true;
                break;
        }
    };
    
    this.actNovoRateioOnClick = function(lancamentoRateioArgs) {
        lancamentoRateioArgs.owner.widget.newRow();
        var lancamentoRateioWidget = lancamentoRateioArgs.owner.widget;
        var currRow = lancamentoRateioWidget.currentRow;
        var prodRateio = lancamentoRateioWidget.parent.currentRow;
        
        currRow.CDFILIAL = prodRateio.CDFILIAL;
        currRow.NRLANCTONF = prodRateio.NRLANCTONF;
        currRow.NRSEQITPROD = prodRateio.NRSEQITPROD;

        
    };
    
    this.actExcluirRateioOnClick = function(excluirRateioArgs) {
        var lancamentoRateioWidget = excluirRateioArgs.owner.widget;
        ScreenService.confirmMessage(
            ScreenService.i18n('Excluir Registro?'),
            'question',
            () => {
                const {owner: {widget}, row} = excluirRateioArgs;
                
                lancamentoRateioWidget.changed = true;
                excluirDataset.push(row);
               /* Df11000_Lancamento_Service.excluiRateio(row)
                    .then(() => */Df11000_Lancamento_Service.excluiRateioDataSource(row)/*)*/
                    .then(Df11000_Lancamento_Service.atualizaLancamento)
                    .then(() => {
                        Df11000_Lancamento_Service.setDataSourceLancamentoNota();
                        General.backOrCancel(excluirRateioArgs);
                        widget.reload();
                    });
            });
    };
    
    this.actExcluirMultiRateio = function(lancamentoRateioArgs){
        var lancamentoRateioWidget = lancamentoRateioArgs.owner.getParent();
        var widgetRateio = lancamentoRateioArgs.owner.widget.container.getWidget('gridItensRateio');
        var rateiosSelecionados;
        
        
        if(lancamentoRateioArgs.owner.name === 'actExcluirMultiRateio'){
            rateiosSelecionados = lancamentoRateioArgs.owner.widget.getCheckedRows();
        }else{
            rateiosSelecionados = Df11000_Lancamento_Service.getDataSourceRateio();
        }
        
        if(widgetRateio._hasCheckedRows() || lancamentoRateioArgs.owner.name === 'actExcluirTodosRateios'){
            ScreenService.confirmMessage(
                ScreenService.i18n('Excluir Registros?'),
                'question',
                () => {
                
                    widgetRateio.changed = true;
                    lancamentoRateioWidget.getAction('actFecharRateio').isVisible = true;
                    rateiosSelecionados.forEach(row =>{
                        excluirDataset.push(row);
                    });
                    lancamentoRateioWidget.unSelectAll();
                    Df11000_Lancamento_Service.excluiMultiRateioDataSource(rateiosSelecionados)
                    .then(Df11000_Lancamento_Service.atualizaLancamento)
                    .then(() => widgetRateio.reload());
                });
        }
        ScreenService.closePopup();
    };
    
    this.actEditarRateioOnClick = function(lancamentoRateioArgs) {
        var lancamentoRateioWidget = lancamentoRateioArgs.owner.widget;
        var lancamentoRateioRow = lancamentoRateioArgs.row;
        if(lancamentoRateioRow.hasOwnProperty('__rightActions')){
            delete lancamentoRateioRow.__rightActions;
            delete lancamentoRateioRow.__leftActions;
        }
        General.edit(lancamentoRateioWidget);
        controlaStatusWidgets(lancamentoRateioWidget, lancamentoRateioWidget.name, false);
    };
    
    this.actSalvarRateioOnClick = function(lancamentoRateioArgs) {
        var lancamentoRateioWidget = lancamentoRateioArgs.owner.widget;

        if (!lancamentoRateioWidget.isValid())
            return lancamentoRateioArgs.e.preventDefault();

  
  
        lancamentoRateioWidget.changed = true;
        Df11000_Lancamento_Service.setDataSourceRateio(lancamentoRateioWidget.dataSource.data) // este set está errado
            .then(Df11000_Lancamento_Service.atualizaLancamento)
            .then(() => {
                lancamentoRateioWidget.view();
                controlaStatusWidgets(lancamentoRateioWidget, lancamentoRateioWidget.name, true);
                lancamentoRateioWidget.getAction('actFecharRateio').isVisible = true;
                ScreenService.goBack();
            });
        
    };
    
    /*this.actFecharRateio = function(rateioArgs) {
        var lancamentoRateioWidget = rateioArgs.owner.widget;
        var rateioDataset = lancamentoRateioWidget.dataSource.data;
        var widgetItensRateio = lancamentoRateioWidget.parent;
        var cabecalhoWidget = widgetItensRateio.parent;
        
        Df11000_Lancamento_Service.salvaRateio(excluirDataset).then(() => {

            rateioDataset.forEach(row => {
                if(row.__is_new) {
                    row.__is_new = false;
                    row.__createdLocal = false;
                }
            });

            Df11000_Lancamento_Service.setDataSourceRateio(lancamentoRateioWidget.dataSource.data) 
                .then(Df11000_Lancamento_Service.atualizaLancamento)
                .then(() => {
                    lancamentoRateioWidget.view();
                    controlaStatusWidgets(lancamentoRateioWidget, lancamentoRateioWidget.name, true);
                    lancamentoRateioWidget.changed = false;
                    excluirDataset = [];
                    this.controlaVisibilidadeRateio(rateioArgs);
                    ScreenService.goBack();
                    ScreenService.closePopup();
                    cabecalhoWidget.openTab('cabecalho');
                });
        }, error => {
            if (Array.isArray(error)) {
                var stringCamposObrig = error.join(', ');
                var mensagem = 'Os campos %1 deve(m) ser maior que zero.'.replace('%1', stringCamposObrig);
                return ScreenService.showMessage(mensagem);
            } else if (typeof error === 'string')
                return ScreenService.showMessage(error);
            return ScreenService.showMessage(error.data.error);
        });
    }; */
    
    this.filtraItemRateio = function(args){
        var widgetItensRateio = args.owner;
        var widgetRateio = widgetItensRateio.container.getWidget('gridItensRateio');
        
        widgetRateio.dataSourceFilter= [
            {
                "name": "NRSEQITPROD",
                "operator": "=",
                "value": args.row.NRSEQITPROD
            }
        ];
    };
    
    this.actImportarRateio = function(args){
        var widgetRateio = args.owner.widget.container.getWidget('gridItensRateio');
        var popupWidget = this.popUpRateio(widgetRateio);
        
        ScreenService.openPopup(popupWidget);

    };
	
    this.importarRateio = function(importacaoArgs){
        var widget = importacaoArgs.owner.widget;
        if(widget.isValid()){
            var file = widget.currentRow.FILEXLS;
            var tipoImportacao = widget.currentRow.IMPORTTYPE;
            var itensRateio = widget.parent.parent.dataSource.data;
            var pos = file[0].b64File.indexOf(',') + 1;
            file = {
                'name': file[0].name,
                'content': file[0].b64File.substr(pos)
            };
            var row = {
                'ITENSRATEIO': itensRateio,
                'TIPORATEIO': tipoImportacao,
                'FILE': file
            };
            
            Ger03600_Rateio_Produto_Repository.importRateio(row).then(function(success) {
                var rateiosExcluir = [];
                if(success.dataset.importRateio){
                    var rateioImportado = success.dataset.importRateio;
                    rateioImportado.forEach(row =>{
                        row.__is_new = true;
                    });
                    Df11000_Lancamento_Service.getDataSourceRateio().then(function(ratAtual) {
                        ratAtual.forEach(row =>{
                            rateiosExcluir.push(row);
                        });
                        Df11000_Lancamento_Repositories.excluiRateio(rateiosExcluir)
                        .then(Df11000_Lancamento_Service.setDataSourceRateio(rateioImportado))
                        .then(Df11000_Lancamento_Service.atualizaLancamento())
                        .then(function(){
                            widget.parent.reload();
                            widget.parent.getAction('actFecharRateio').isVisible = true;
                            ScreenService.closePopup();
                            ScreenService.closePopup();
                        }.bind(widget));
                        
                    }.bind(widget));
                }
            }.bind(widget),
            function(error){
                if(error.data.error){
                    ScreenService.showMessage('Arquivo ou Formato Inválido \n'+error.data.error);
                }
            });
                    
            }
    };
	
	this.popUpRateio = function(parentWidget){
        var widget = parentWidget.importaRateio;
        if(!widget ){
            widget = getPopUpRateio();
            widget.container = parentWidget.container;
            widget = metaDataFactory.widgetFactory(widget,parentWidget);
            parentWidget.importaRateio = widget;
            widget.parent = parentWidget;
            return widget;
        }else{
            return widget;
        }
    };
    
    var controlaStatusWidgets = function(widgetPai, nomeWidgetAtual, status, ignoredWidgets) {
        var allWidgets = widgetPai.widgets;

        allWidgets.filter(widget => widget.name !== nomeWidgetAtual).forEach(widget => widget.enabled = status);

        if (widgetPai.name !== nomeWidgetAtual)
            widgetPai.enabled = status;

        if (Array.isArray(ignoredWidgets))
            allWidgets.filter(widget => ignoredWidgets.indexOf(widget.name) !== -1).forEach(widget => widget.enabled = !status);
    };
    

	var getPopUpRateio = function() {
        return {
			"id": "000",
            "name": "importacaoRateio",
            "label": "Importação de Rateio",
            "isVisible": true,
            "template": "widget/form.html",
            "dataSourceFilter": [],
			"currentRow": {},
            "showAsTab": false,
            "dataSource": {},
            "fields": [
                {
					"id": 1,
                    "name": "IMPORTTYPE",
                    "label": "Tipo de importação",
                    "isVisible": true,
                    "class": 12,
                    "template": "field/select-native.html",
                    "valueField": "IMPORTTYPE",
                    "descriptionField": "IMPORTNAME",
                    "hideEmptyValue": true,
                    "dataSource": {
                        "data": [
                            {
                                "IMPORTTYPE": "P",
                                "IMPORTNAME": "Por Percentual"
                            },
                            {
                                "IMPORTTYPE": "V",
                                "IMPORTNAME": "Por Valor"
                            }
                        ]
                    },
                    "validations": {
                        "required": {}
                    }
                },
                {
					"id": 1,
                    "name": "FILEXLS",
                    "isVisible": true,
                    "class": 12,
                    "template": "field/file-upload.html",
                    "validations": {
                        "extensions": {
                            "value": [
                                ".xls",
                                ".xlsx"
                            ]
                        },
                        "required": {}
                    },
                    "events": []
                }
            ],
            "actions": [
				{
					"id": 1,
					"name": "import",
					"label": "Importar",
					"showAsAction": "never",
					"isVisible": true,
					"events": [
						{
							"id": 1,
							"name": "ActionEvent",
							"code": "Ger03600_Rateio_Produto_Controller.importarRateio(args);"
						}
					]
				},
				{
                	"id": 1,
                    "name": "cancelAction",
                    "label": "Voltar",
                    "showAsAction": "back",
                    "isVisible": true,
                    "events": [
                    	{
	                        "id": 1,
	                        "name": "ActionEvent",
	                        "code": "ScreenService.closePopup();"
                    	}
                    ]
                }
			],
            "events": []
        };
    };
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger03600_Rateio_Produto_Service', Ger03600_Rateio_Produto_Service);
});

// FILE: ./assets/js/services/Ger03900_ErrosAvisosNf_Service.js
function Ger03900_ErrosAvisosNf_Service(Ger03900_ErrosAvisosNf_Repository) {

	var descriptions = {
		AV01: "Fornecedor da nota fiscal difere do fornecedor da AF.",
        AV02: "Percentual de Imposto da tabela difere do percentual informado.",
        AV03: "Percentual Imposto do item difere do valor Imposto do Item.",
        AV04: "Imposto do item da entrada com código fiscal inválido.",
        AV05: "Imposto do item com situação tributária(CST) inválida.",
        AV06: "Imposto do item com cód. fiscal operação - CFOp inválido.",
        AV07: "Valor de Imposto informado diferente da tabela.",
        AV08: "Imposto da NF difere da alíquota do fornecedor.",
        AV09: "Item com código da situação tributária inválido .",
        AV10: "Item com código fiscal de operação inválido.",
        AV11: "Quantidade de produto informada difere da quantidade prevista na AF.",
        AV12: "Data de Entrada da nota fiscal difere da Data Programada na AF.",
        AV13: "Valor unitário do produto informado difere do valor unitário previsto na AF.",
        AV14: "Quantidade de produto informada difere da quantidade recebida.",
        AV15: "Data de vencimento da nota fiscal difere da data programada na AF.",
        AV16: "Item informado na nota fiscal difere do produto programado na AF.",
        AV17: "Data de Vencimento informada anterior a data de processamento.",
        ER01: "Imposto da NF inválido.",
        ER02: "Total nota fiscal difere soma dos itens.",
        ER03: "Total de vencimentos difere valor total nota fiscal.",
        ER04: "Ítem com valor unitário igual a zero.",
        ER05: "Soma das entradas sem nota fiscal difere do valor total da nota.",
        ER06: "Soma das saídas sem nota fiscal difere do valor total da nota."
	};

	this.getDescription = function(codigo) {
		return descriptions[codigo];
	};

	this.findAutoRegister = function(filter) {
		return Ger03900_ErrosAvisosNf_Repository.findAutoRegister(filter);
	};

	this.saveAutoRegister = function(rows) {
		return Ger03900_ErrosAvisosNf_Repository.saveAutoRegister(rows);
	};
	
	this.deleteRegisters = function(cdfiliais){
		return Ger03900_ErrosAvisosNf_Repository.deleteRegisters(cdfiliais);
	};
}
Configuration(function (ContextRegister) {
	ContextRegister.register('Ger03900_ErrosAvisosNf_Service', Ger03900_ErrosAvisosNf_Service);
});

// FILE: ./assets/js/services/Ger05500_produtosDesoneracaoImpostos_Service.js
function Ger05500_produtosDesoneracaoImpostos_Service(Ger05500_produtosDesoneracaoImpostos_Repositories) {
    self = this;

    self.removeProdutoDesoneracaoFornecedor = function(data) {
        return Ger05500_produtosDesoneracaoImpostos_Repositories.removeProdutoDesoneracaoFornecedor(data);
    };

    self.removeProdutoDesoneracaoCliente = function(data) {
        return Ger05500_produtosDesoneracaoImpostos_Repositories.removeProdutoDesoneracaoCliente(data);
    };

    self.removeProdutoDesoneracaoFilial = function(data) {
        return Ger05500_produtosDesoneracaoImpostos_Repositories.removeProdutoDesoneracaoFilial(data);
    };
}
Configuration(function (ContextRegister) {
	ContextRegister.register('Ger05500_produtosDesoneracaoImpostos_Service', Ger05500_produtosDesoneracaoImpostos_Service);
});

// FILE: ./assets/js/services/Ger06600_TipoDeOperacao_Service.js
var Ger06600_TipoDeOperacao_Service = function(Ger06600_TipoDeOperacao_Repository, ZHPromise) {

    this.checkParametersUtilizaDocumento = function(row) {
        var deferred = ZHPromise.defer();
        Ger06600_TipoDeOperacao_Repository.checkParametersUtilizaDocumento(row).then(function(response) {
            var parameters = response.dataset.parameters;
            var isValid = parameters.IDGERICMSTDESPA != 'S' && parameters.IDGERIPIDESPA != 'S';
            deferred.resolve(isValid);
        }, function(error) {
            deferred.reject(error);
        });
        return deferred.promise;

    };

    this.saveTipoOperac = function(row) {
        return Ger06600_TipoDeOperacao_Repository.saveTipoOperac(row);
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("Ger06600_TipoDeOperacao_Service", Ger06600_TipoDeOperacao_Service);
});


// FILE: ./assets/js/services/Ger06800_SerieDocumentosEntrada_Service.js
var Ger06800_SerieDocumentosEntrada_Service = function() {

    this.validaCodigoNfeScan = function(codigo){
        if(codigo){
            var resul = false;
            if(codigo>='900' && codigo<='999')
                resul =true;
            return resul;
        }else{
            return true;
        }

    };
    this.validaCodigoNfe = function(codigo){
        if(codigo){
            var resul = false;
            if(codigo>='1' && codigo<='889')
                resul =true;
            return resul;
        }
        else{
            return true;
        }
    };

};
Configuration(function(ContextRegister) {
	ContextRegister.register("Ger06800_SerieDocumentosEntrada_Service", Ger06800_SerieDocumentosEntrada_Service);
});


// FILE: ./assets/js/services/Ger07800_CodigoEspecificadorSubstituicaoTributaria_Service.js
var Ger07800_CodigoEspecificadorSubstituicaoTributaria_Service = function(General) {

    this.getSegmentoMercadoria = function(cdsegmento) {
        var filter = [
            {
                "name": "CDSEGMENTO",
                "operator": "=",
                "value": cdsegmento
            }   
       ];
       
       return General.callRestEngine("df#/crud/segmercabens", "FilterData", filter);
    };
    
    this.getCestBySegmento = function(cdsegmento){
        var filter = [
            {
                "name": "CDSEGMENTO",
                "operator": "=",
                "value": cdsegmento
            }   
       ];
       
       return General.callRestEngine("df#/crud/codespsubtri", "FilterData", filter);
    };
    
    this.getCest = function(cditem, cdcest) {
        var filter = [
            {
                "name": "CDITEM",
                "operator": "=",
                "value": cditem
            },
            {
                "name": "CDCEST",
                "operator": "=",
                "value": cdcest
            } 
       ];
       
       return General.callRestEngine("df#/crud/codespsubtri", "FilterData", filter);
    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Ger07800_CodigoEspecificadorSubstituicaoTributaria_Service", Ger07800_CodigoEspecificadorSubstituicaoTributaria_Service);
});


// FILE: ./assets/js/services/Ger10200_ParametrosGrupoFiscal_Service.js
function Ger10200_ParametrosGrupoFiscal_Service(Ger10200_ParametrosGrupoFiscal_Repository) {
	this.concatenaCampo = function(cdgrupofisc, dspargerfis){
		if(!dspargerfis){
				dspargerfis="  "+cdgrupofisc;
		}
		else{
				if(dspargerfis.length==1){
						dspargerfis = dspargerfis.substr(0,1)+' '+cdgrupofisc;
				}else{
					dspargerfis = dspargerfis.substr(0,2) + cdgrupofisc;
				}

		}
		return dspargerfis;
	};
	this.salvarCadastramentoAutomatico = function(row){
			return Ger10200_ParametrosGrupoFiscal_Repository.save(row);
	};
	this.salvarReplicacaoAliquotas = function(row){
			return Ger10200_ParametrosGrupoFiscal_Repository.saveReplicacao(row);
	};

}
Configuration(function (ContextRegister) {
	ContextRegister.register('Ger10200_ParametrosGrupoFiscal_Service', Ger10200_ParametrosGrupoFiscal_Service);
});

// FILE: ./assets/js/services/Ger33500_AgenteDeTerceirizacao_Service.js
function Ger33500_AgenteDeTerceirizacao_Service(Ger33500_AgenteDeTerceirizacao_Repository)  {


    this.verificaControle = function(CDFILIAL){
        return Ger33500_AgenteDeTerceirizacao_Repository.verificaControle(CDFILIAL);
    };

    this.buscaFilial = function(){
        return Ger33500_AgenteDeTerceirizacao_Repository.retornaUnidadeLogada();
    };
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger33500_AgenteDeTerceirizacao_Service', Ger33500_AgenteDeTerceirizacao_Service);
});

// FILE: ./assets/js/services/Ger57000_TabeladDoSimplesNacional_Service.js
function Ger57000_TabelaDoSimplesNacional_Service(Ger57000_TabelaDoSimplesNacional_Repository) {

    this.save = function (dataSet, name) {
        if (name !== "FaixaDeValores") {
            return Ger57000_TabelaDoSimplesNacional_Repository.save(dataSet, name);
        }
        return Ger57000_TabelaDoSimplesNacional_Repository.saveDataSet(dataSet, name);
    };
    
    this.deleteTabelaDeReferencia = function(widget){
        var row = widget.currentRow;
        return Ger57000_TabelaDoSimplesNacional_Repository.deleteTabelaDeReferencia(row);
    };
    
    this.deleteFaixaValores = function(widget){
        var row = widget.currentRow;
        return Ger57000_TabelaDoSimplesNacional_Repository.deleteFaixaValores(row);
    };

    this.verificarFaixaDeValores = function (widget) {
        var dataVigencia  = moment('01/01/2018','DD/MM/YYYY');
        var dataInicial   = moment(widget.parent.parent.currentRow.DTINIVIGENCIA,'DD/MM/YYYY');
        
        var aliquotaTotal = parseFloat(widget.currentRow.VRALIQSIMPLES);
        var soma          = 0.00;
        var impostos      = widget.widgets[0];
       
        impostos.dataSource.data.forEach(
            function (row) {
                soma += this.formataValoresDecimais(row.VRALIQSIMPLES) ;
            }.bind(this)
        );
        
        if(dataInicial>= dataVigencia){
            if(soma ===100)
                return this.verificarRateioIsss(impostos);
            else
                return 'REPARTTRIB_INV';
        }else{
            if(aliquotaTotal === soma)
                return this.verificarRateioIsss(impostos);
            else
                return 'ALIQUOTA_SUM';
        }
        
    
    };
    
    this.verificarRateioIsss = function(impostos){
        var soma = 0.00;
        impostos.dataSource.data.forEach(
            function (row) {
                soma += this.formataValoresDecimais(row.VRPERRATISS) ;
            }.bind(this)
        );
        if(soma === 0 || soma ===100)
            return true;
        else 
            return 'REPARTTRIBISS_INV';
    };

    //Widget: Impostos_form
    this.verificarImpostos = function (widget) {
        var data = widget.dataSource.data;
        var currentRow = widget.currentRow;
        var numberOfRegisters = 0;
        data.forEach(
            function (imposto) {
                if (imposto.CDIMPOSTO === currentRow.CDIMPOSTO) {
                    numberOfRegisters++;
                }
            }
        );
        if (numberOfRegisters < 2) {
            return true;
        } else {
            return false;
        }
    };
    
    this.formataValoresDecimais = function(valor){
        var valorFormatado = 0;
        valorFormatado = (valor.toString().search(',') != '-1') ?
            ((parseFloat(valor.toString().split(',')[0]) > 0) ? parseFloat(valor.toString().replace('.', '').replace(',', '.')) : parseFloat(valor.toString().replace(',', '.'))) :
            parseFloat(valor);
        return valorFormatado;
    };

}

Configuration(function (ContextRegister) {
    ContextRegister.register('Ger57000_TabelaDoSimplesNacional_Service', Ger57000_TabelaDoSimplesNacional_Service);
});

// FILE: ./assets/js/services/Ger80900_ParametrizacaoFiscalCentralizada_Service.js
var Ger80900_ParametrizacaoFiscalCentralizada_Service = function(localStorageEngine, ZHPromise, Ger80900_ParametrizacaoFiscalCentralizada_Repository) {

  var self = this;

  this.getMaiorSequencialDataset = function(dataset, field){
    var maxNrseq = 0;
    dataset.forEach(function(row){
      if(row[field] > maxNrseq)
        maxNrseq = row[field];
    });
    return maxNrseq;
  };

  this.zerofill = function(number, width){
    width -= number.toString().length;
    if ( width > 0 )
    {
      return new Array( width + (/\./.test( number ) ? 2 : 1) ).join( '0' ) + number;
    }
    return number + ""; // always return a string
  };

  this.deletarRegraImposto = function(row){
    return Ger80900_ParametrizacaoFiscalCentralizada_Repository.deletarRegraImposto(row);
  };

  this.deletarRegraCfop = function(row){
    return Ger80900_ParametrizacaoFiscalCentralizada_Repository.deletarRegraCfop(row);
  };

  this.consultarErrosRegra = function(row){
    return Ger80900_ParametrizacaoFiscalCentralizada_Repository.consultarErrosRegra(row);
  };
  
  this.getNewCodeRegra = function(row){
    return Ger80900_ParametrizacaoFiscalCentralizada_Repository.getNewCodeRegra(row);
  };

};

Configuration(function(ContextRegister) {
  ContextRegister.register("Ger80900_ParametrizacaoFiscalCentralizada_Service", Ger80900_ParametrizacaoFiscalCentralizada_Service);
});

// FILE: ./assets/js/services/Ger81100_AgrupamentoImpostos_Service.js
var Ger81100_AgrupamentoImpostos_Service = function(General) {

  this.getRegrasFiscais = function(cdgrupimposto) {
      
    var filter = [
            {
                "name": "CDGRUPIMPOSTO",
                "operator": "=",
                "value": cdgrupimposto
            }
	    ];
	
    return General.callRestEngine('df#/crud/regrafiscal_agrupimposto', 'FilterData', filter).then(
        response => {
            var regras = [];
            response.dataset.regrafiscal_agrupimposto.forEach( row => {
               regras.push(row.CDREGRAFISC); 
            });
            return regras;
        }
    );

  };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('Ger81100_AgrupamentoImpostos_Service', Ger81100_AgrupamentoImpostos_Service);
});

// FILE: ./assets/js/services/Ger97500_TipoOpercaoCfop_Service.js
function Ger97500_TipoOpercaoCfop_Service(Ger97500_TipoOpercaoCfop_Repository) {

	this.deletar = function(row){
			return Ger97500_TipoOpercaoCfop_Repository.delete(row);
	};

}
Configuration(function (ContextRegister) {
	ContextRegister.register('Ger97500_TipoOpercaoCfop_Service', Ger97500_TipoOpercaoCfop_Service);
});

// FILE: ./assets/js/services/Ger98500_TipoGestaoFiscal_Service.js
function Ger98500_TipoGestaoFiscal_Service(Ger98500_TipoGestaoFiscal_Repository) {

	this.deletar = function(row){
		return Ger98500_TipoGestaoFiscal_Repository.delete(row);
	};

	this.verificaUnidadeRelacionada = function(row){
		return Ger98500_TipoGestaoFiscal_Repository.verifica(row);
	};

}
Configuration(function (ContextRegister) {
	ContextRegister.register('Ger98500_TipoGestaoFiscal_Service', Ger98500_TipoGestaoFiscal_Service);
});

// FILE: ./assets/js/services/Ger98600_CodigoDeSituacaoDaOperacaoDoSimplesNacional_Service.js
function Ger98600_CodigoDeSituacaoDaOperacaoDoSimplesNacional_Service(Ger98600_CodigoDeSituacaoDaOperacaoDoSimplesNacional_Repository) {

	this.save = function(row) {
        return Ger98600_CodigoDeSituacaoDaOperacaoDoSimplesNacional_Repository.save(row);
	};

}
Configuration(function (ContextRegister) {
	ContextRegister.register('Ger98600_CodigoDeSituacaoDaOperacaoDoSimplesNacional_Service', Ger98600_CodigoDeSituacaoDaOperacaoDoSimplesNacional_Service);
});

// FILE: ./assets/js/services/Lib98004_EmailsAdicionais_Service.js
function Lib98004_EmailsAdicionais_Service(Lib98004_EmailsAdicionais_Repository) {

	this.saveEmail = function(tipoDestino, cdCliente, cdFilial, cdForneced, idtpijurtran, nrinsjurtran, email, idEmail) {
	    var row = {
	        IDORIGEM:       tipoDestino,
	        CDCLIENTE:      cdCliente,
	        CDFILIAL:       cdFilial,
	        CDFORNECED:     cdForneced,
	        IDTPIJURTRAN:   idtpijurtran,
	        NRINSJURTRAN:   nrinsjurtran,
	        DSEMAIL:        email,
	        IDEMAIL:        idEmail
	    };
        return Lib98004_EmailsAdicionais_Repository.saveEmail(row);
	};
	
	this.removeEmail = function(idEmail) {
	    var row = {
	        IDEMAIL:        idEmail
	    };
        return Lib98004_EmailsAdicionais_Repository.removeEmail(row);
	};

}
Configuration(function (ContextRegister) {
	ContextRegister.register('Lib98004_EmailsAdicionais_Service', Lib98004_EmailsAdicionais_Service);
});

// FILE: ./assets/modules/entrada/assets/js/services/Dfe11000_LancamentoEntrada_Service.js
var Dfe11000_LancamentoEntrada_Service = function(localStorageEngine, Dfe11000_LancamentoEntrada_Repositories, General) {

    this.excluirNotaEntrada = function (lancamentoRow) {
        var filial = lancamentoRow.CDFILIAL;
        var sequencial = lancamentoRow.NRLANCTONF;
        return Dfe11000_LancamentoEntrada_Repositories.excluirNotaEntrada(filial, sequencial);
    };
    
    this.excluiVencimento = function(vencimentoRow) {
        return Dfe11000_LancamentoEntrada_Repositories.excluiVencimento(vencimentoRow);
    };
    
    this.salvaVencimento = function(vencimentoRow) {
        return Dfe11000_LancamentoEntrada_Repositories.salvaVencimento(vencimentoRow);
    };
    
    this.isAFTipoOper = function(tipooper, filial) {
        var filter = [
            {
                "name": "CDTIPOOPER",
                "operator": "=",
                "value": tipooper
            },
            {
                "name": "IDENTRSAIDOP",
                "operator": "=",
                "value": 'E'
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filial
            }
        ];
        return General.callRestEngine("df#/crud/parambloqnfafil", "FilterData", filter).then(success =>{
            if(success.dataset.parambloqnfafil[0]){
                var parameter = success.dataset.parambloqnfafil[0];
                    if(parameter.IDEXIGEAF == 'S')
                        return true;
                    else
                        return false;
            }
            return false;
        });
    };

    this.excluiVencimento = function(vencimentoRow) {
        return Dfe11000_LancamentoEntrada_Repositories.excluiVencimento(vencimentoRow);
    };

    this.excluirNota = function(consultaLancamentoRow) {
        return Dfe11000_LancamentoEntrada_Repositories.excluirNota(consultaLancamentoRow.CDFILIAL, consultaLancamentoRow.NRLANCTONF);
    };

    this.calculaImpostoItemProduto = function(cabecalhoRow, lancamentoItemRow) {
        return Dfe11000_LancamentoEntrada_Repositories.calculaImpostoItemProduto(cabecalhoRow, lancamentoItemRow);
    };
    
    this.validaParamProduto = function(cdfilial, cdarvprod) {
        var row = {
            "CDFILIAL" : cdfilial,
            "CDARVPROD": cdarvprod
        };
        return Dfe11000_LancamentoEntrada_Repositories.validaParamProduto(row);
    };

    this.realizaImportacaoItensNotaOrigem = function(cabecalhoRow) {
        return Dfe11000_LancamentoEntrada_Repositories.realizaImportacaoItensNotaOrigem(cabecalhoRow);
    };
    
    this.validaAF = function(row){
        return Dfe11000_LancamentoEntrada_Repositories.validaAF(row);
    };

    this.realizaImportacaoAf = function(cabecalhoRow) {
        return Dfe11000_LancamentoEntrada_Repositories.realizaImportacaoAf(cabecalhoRow);
    };

    this.validaFornecedorBloqTpOperac = function(tipoOperacao, nrinsjurforn) {
        return Dfe11000_LancamentoEntrada_Repositories.buscaFornecedorBloqTpOperac(tipoOperacao, nrinsjurforn);
    };

    this.geraSenhaLancRetroativo = function(lancRetroativoRow){
        return Dfe11000_LancamentoEntrada_Repositories.geraSenhaLancRetroativo(lancRetroativoRow);
    };
    
    this.tipoEdicao = function(row){
        let filter = [];
        return General.callRestEngine("df#/crud/paramestoque", "FilterData", filter).then((response) => {
            return 'N';
/*            let permiteAlteracao = response.dataset.paramestoque[0].IDLANCDTMEN,
            difDiasAntPermitidos = parseInt(response.dataset.paramestoque[0].NRDIASENTR),
            dtLancto             = row.DTENTRSAID,
            dtProcessamento      = row.DTPROCESSA,
            difDias              = dtProcessamento.diff(dtLancto, 'days');
            
            if(permiteAlteracao == 'S'){
                if(difDiasAntPermitidos === null || difDiasAntPermitidos === undefined || difDias > difDiasAntPermitidos){
                    return 'S'; //Edição com senha
                } else{
                    return 'N'; //Edição normal
                }
            } else{
                return 'B'; //Edição bloqueada
            }*/
        });
    };

    this.salvaImpostosNotaFiscal = function(impostos) {

        var impostosTemp = [];
        var trabEntity = {
            "CDFILIMPTOT"  : "CDFILIAL",
            "NRLANCIMPTOT" : "NRLANCTONF",
            "CDIMPOSTOT"   : "CDIMPOSTO",
            "VRIMPOSTOT"   : "VRIMPOSTONF"
        };

        var columns = Object.keys(trabEntity);

        impostosTemp = Util.clone(impostos);

        impostosTemp.forEach(function(imposto) {
            columns.forEach(function(column) {
                if (!(column in imposto)) {
                    imposto[column] = imposto[trabEntity[column]];
                    delete imposto[trabEntity[column]];
                }
            });
            imposto.__is_new = true;
            imposto.VRBCALIMPTOT = 0;
        });
        
        return Dfe11000_LancamentoEntrada_Repositories.salvaImpostosNotaFiscal(impostosTemp);
    };

    this.validaInscEstadual = function(row){
        return General.callRestEngine("df_entrada#/validaInscEstadual", "Row", row);
    };
    
    this.limpaTrabsLancamento = function(cdfilial, nrlantonf) {
        return Dfe11000_LancamentoEntrada_Repositories.limpaTrabsLancamento(cdfilial, nrlantonf);
    };

    var converteDataParaComparacao = function(data) {
        return moment(data, 'DD/MM/YYYY').format('MM-DD-YYYY');
    };

    this.validaDataVencimento = function(dataEmissao, dataVencimento) {
        dataEmissao = converteDataParaComparacao(dataEmissao);
        dataVencimento = converteDataParaComparacao(dataVencimento);

        if (moment(dataVencimento).isBefore(dataEmissao) && !moment(dataEmissao).isSame(dataVencimento))
            return "Data de Vencimento deve ser maior ou igual a data de emissão";
    };

    this.validacaoSalvaCabecalhoEntrada = function(cabecalhoRow) {
        return Dfe11000_LancamentoEntrada_Repositories.validacaoSalvaCabecalhoEntrada(cabecalhoRow);
    };

    this.buscaChaveAcesso = function(nracessonfe) {
        return Dfe11000_LancamentoEntrada_Repositories.buscaNfChaveAcesso(nracessonfe);
    };
    
    this.buscaChaveAcessoPorCampos = function(params){
        return Dfe11000_LancamentoEntrada_Repositories.buscaNfChaveAcessoPorCampos(params);
    };
    
    this.getFilial = function(){
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
        ];
        return General.callRestEngine("df#/crud/filial", "FilterData", filter);
    };

    this.realizaImportacaoChaveAcesso = function(cabecalhoRow) {
        return Dfe11000_LancamentoEntrada_Repositories.realizaImportacaoChaveAcesso(cabecalhoRow);
    };

    var verificaLançamentoComImportacao = function(cabecalhoRow) {
        const ACCESS_KEY = !!cabecalhoRow.NRACESSONFE && (cabecalhoRow.IDPELANIMPXML != 'S' || !cabecalhoRow.MANUAL_POSTING);

        return {
            chaveAcesso: ACCESS_KEY,
            af: 'AF' in cabecalhoRow,
            todos: 'AF' in cabecalhoRow && ACCESS_KEY
        };
    };

    this.realizaImportacaoAfChaveAcesso = function(cabecalhoRow) {
        return Dfe11000_LancamentoEntrada_Repositories.realizaImportacaoAfChaveAcesso(cabecalhoRow);
    };

    this.realizaImportacoes = function(cabecalhoRow) {
        var lancamentoComImportacao = verificaLançamentoComImportacao(cabecalhoRow);
        if(!!cabecalhoRow.__is_new) {
            if(lancamentoComImportacao.todos)
                return this.realizaImportacaoAfChaveAcesso(cabecalhoRow);
            else if(lancamentoComImportacao.af)
                return this.realizaImportacaoAf(cabecalhoRow);
            else if(lancamentoComImportacao.chaveAcesso)
                return this.realizaImportacaoChaveAcesso(cabecalhoRow);
        }
    };

    this.salvaRelacionamentoItensXml = function(relacionamentoRow) {
        return Dfe11000_LancamentoEntrada_Repositories.salvaRelacionamentoItensXml(relacionamentoRow);
    };

    this.removerRelacionamentoItemXml = function(cdprodutoxml, cdforneced,cdfilial, cdproduto) {
          return Dfe11000_LancamentoEntrada_Repositories.removerRelacionamentoItemXml(cdprodutoxml, cdforneced,cdfilial, cdproduto);
    };

    this.salvaMultiItens = function(novosItens, itensAlterados, currentRowParentWidget) {
        return Dfe11000_LancamentoEntrada_Repositories.salvaMultiItens(novosItens, itensAlterados, currentRowParentWidget);
    };
    
    this.atualizaItemAf = function(item) {
        return General.callRestEngine("df_entrada#/atualizaItemAf", "Row", item);
    };
    
    this.excluirItemAf = function(item) {
        return General.callRestEngine("df_entrada#/excluirItemAf", "Row", item);
    };
    
    this.setFornImportRastXml = function(IDIMPRASTXML) {
        let row = {IDIMPRASTXML};
        return localStorageEngine.setDataSource("df_entrada#/fornImportRastXml", row);
    };
    
    this.getFornImportRastXml = function() {
        return localStorageEngine.getDataSource("df_entrada#/fornImportRastXml").then((success) => {
            return success[0];
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfe11000_LancamentoEntrada_Service", Dfe11000_LancamentoEntrada_Service);
});

// FILE: ./assets/modules/entrada/assets/js/services/Dfe36000_EntradasTransferenciaDoOperadorLogistico_Service.js
var Dfe36000_EntradasTransferenciaDoOperadorLogistico_Service = function(Dfe36000_EntradasTransferenciaDoOperadorLogistico_Repository, General){
    
    this.buildFilter = function(fieldsName, currentRow) {
        let filter = [];
        fieldsName.forEach((fieldName) => {
            filter.push({
                "name": "P_".concat(fieldName),
                "operator": "=",
                "value": (currentRow[fieldName]) ? currentRow[fieldName] : 'T'
            }); 
        });
        return filter;
    };
    
    this.getParamGeral = function() {
        return General.callRestEngine("df#/crud/paramgeral", "FilterData", []);
    };


    this.getFilial = function(){
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
        ];
        return General.callRestEngine("df#/crud/filial", "FilterData", filter);
    };
    
};

Configuration(function (ContextRegister){
    ContextRegister.register('Dfe36000_EntradasTransferenciaDoOperadorLogistico_Service', Dfe36000_EntradasTransferenciaDoOperadorLogistico_Service);
});


// FILE: ./assets/modules/entrada/assets/js/services/Dfe51000_ImportacaoConsultaNotaFiscal_Service.js
var Dfe51000_ImportacaoConsultaNotaFiscal_Service = function(Dfe51000_ImportacaoConsultaNotaFiscal_Repositories, Dfe36000_EntradasTransferenciaDoOperadorLogistico_Repository, General, ZHPromise){
    
   this.buildFilter = function(fieldsName, currentRow, insstr = false) {
        var deferred = ZHPromise.defer();
        let filter = [];
        fieldsName.forEach((fieldName) => {
            if(fieldName != "NRNOTA") {
                filter.push({
                    "name": "P_".concat(fieldName),
                    "operator": "=",
                    "value": (currentRow[fieldName]) ? (Array.isArray(currentRow[fieldName]) ? montaInStr(currentRow[fieldName]) : currentRow[fieldName]) : 'T'
                }); 
            }
            else {
                filter.push({
                    "name": "P_".concat(fieldName),
                    "operator": "=",
                    "value": (currentRow.NRNOTAFISCINI || currentRow.NRNOTAFISCFIN) ? '' : 'T'
                }); 
            }
        });
        deferred.resolve(filter);
        return deferred.promise;
    };
    
    var montaInStr = function(row) {
        var strReturn = '_';
        row.forEach((value) => {
            strReturn += value + '_';
        });
        return strReturn;
    };
    
    this.gerarDanfe = function(row){
        return Dfe51000_ImportacaoConsultaNotaFiscal_Repositories.gerarDanfe(row);
    };
};

Configuration(function (ContextRegister){
    ContextRegister.register('Dfe51000_ImportacaoConsultaNotaFiscal_Service', Dfe51000_ImportacaoConsultaNotaFiscal_Service);
});


// FILE: ./assets/modules/entrada/assets/js/services/Dfe58000_ManifestacaoDestinatario_Service.js
var Dfe58000_ManifestacaoDestinatario_Service = function(localStorageEngine, Dfe58000_ManifestacaoDestinatario_Repositories, General, ScreenService) {
    
    var getDataSourceConfigTransmissao = function() {
        return localStorageEngine.getDataSource("df_entrada#/config_manifestacao").then(function(transmissao){
            return transmissao[0].config;
        });
    };
    
    this.openConsultaNsuNfe = function(widget, wdgNsu) {
        
        return getDataSourceConfigTransmissao().then(function(config){
            var today = moment().format('DD/MM/YYYY');
            wdgNsu.currentRow.DTRECBTO = {};
            wdgNsu.currentRow.DTRECBTO.start = today.replace(today.substr(0,2), "01");
            wdgNsu.currentRow.DTRECBTO.end   = today;
        
            wdgNsu.widgets[0].dataSourceFilter[0].value = config.cnpj;
            wdgNsu.widgets[0].dataSourceFilter[1].value = config.tpAmb;
            wdgNsu.widgets[0].dataSourceFilter[2].value = today.replace(today.substr(0,2), "01");
            wdgNsu.widgets[0].dataSourceFilter[3].value = today;
            ScreenService.openPopup(wdgNsu);
            
            wdgNsu.widgets[0].activate();
            wdgNsu.widgets[0].list();
            wdgNsu.widgets[0].reload();
        });
    };
             

    this.montaConfigTransmissao = function(row){
        return Dfe58000_ManifestacaoDestinatario_Repositories.montaConfigTransmissao(row);
    };

    this.manifestarNfe = function (notasfiscais, evento, justificativa) {
        return getDataSourceConfigTransmissao().then(function(config){
            return Dfe58000_ManifestacaoDestinatario_Repositories.manifestarNfe(notasfiscais, config, evento, justificativa);
        });
    };

    this.gerarXML = function (notasfiscais) {
        return getDataSourceConfigTransmissao().then(function(config){
            var filter = [
                {
                    "name": "NOTAS",
                    "operator": "=",
                    "value": notasfiscais
                },
                {
                    "name": "CONFIG",
                    "operator": "=",
                    "value": config
                }
            ];
            return General.callRestEngine("df_entrada#/gerarXmlNfeManifestacao", "FilterData", filter);
        });
    };

    this.sincronizarNFe = function (row) {
        return getDataSourceConfigTransmissao().then(function(config){
            return Dfe58000_ManifestacaoDestinatario_Repositories.sincronizarNFe(config, row);
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfe58000_ManifestacaoDestinatario_Service", Dfe58000_ManifestacaoDestinatario_Service);
});

// FILE: ./assets/modules/entrada/assets/js/services/Dfe64000_IntegracaoCfopXmlComTipoDeOperacao_Service.js
var Dfe64000_IntegracaoCfopXmlComTipoDeOperacao_Service = function(ScreenService, MessageService, General) {
    
    this.errorRequest = function(error) {
        if (error.data) {
            ScreenService.showMessage(error.data.error);
        } else {
            ScreenService.showMessage(error);
        }
        ScreenService.hideLoader();
    };
    
    this.salvar = function(widget){
        var row = widget.currentRow;
        var routeName = 'df_entrada#/integracao_cfopxml_x_tipooperacao/save' + (widget.name === "Dfe64000_grid_filial" ? "Filial" : "");
        
        if (widget.isValid()) {
            General.callRestEngine(routeName, "Row", row).then(function() {
				widget.activate();
				widget.list();
				ScreenService.successNotification("Registro Salvo com Sucesso");
				ScreenService.goBack();
				widget.reload();
			}.bind(this), function(callback){
			    var notaUtilizada = callback.data.error;
			    var msg = "OP_BLOQUEADA";
                MessageService.showMessageByCode(msg, {"notaUtilizada": notaUtilizada});
        });
        }
    };
    
    this.deletar = function(widget){
        var row = widget.currentRow;
        var routeName = 'df_entrada#/integracao_cfopxml_x_tipooperacao/remove' + (widget.name === "Dfe64000_grid_filial" ? "Filial" : "");
        
        General.callRestEngine(routeName, "Row", row).then(function() {
			widget.activate();
			widget.list();  
			ScreenService.successNotification("Registro Excluído com Sucesso");
			ScreenService.goBack();
			widget.reload();
		}.bind(this), this.errorRequest);
    };
    
    this.getFilialLogada = function() {
        return General.callRestEngine( "df#/filioper_lancamento_notafiscal", "FilterData", 
                            [{name: "CDFILIAL", operator: "=", value: "FILIAL_LOGADA"},
                            {name: "CDOPERADOR", operator: "=", value: "OPERADOR_LOGADO"}]);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfe64000_IntegracaoCfopXmlComTipoDeOperacao_Service", Dfe64000_IntegracaoCfopXmlComTipoDeOperacao_Service);
});

// FILE: ./assets/modules/saida/assets/js/services/fileRepository.js
Configuration(function(RepositoryFactory, ContextRegister)
{
	var fileRepository = RepositoryFactory.factory("df_saida#/uploadFile", "LOCAL");

	ContextRegister.register('fileRepository', fileRepository);
});

// FILE: ./assets/modules/saida/assets/js/services/Dfe43200_ParametrosDocumentosFiscaisSaida_Service.js
function Dfe43200_ParametrosDocumentosFiscaisSaida_Service(Dfe43200_ParametrosDocumentosFiscaisSaida_Repository) {

	this.salvarParametrosFilial = function(row){
		return Dfe43200_ParametrosDocumentosFiscaisSaida_Repository.saveParametrosFilial(row);
	};

	this.concatenaCampoParametrosFilial = function(row){
		var campo = row.DSPAR;
		var campoCompleto = row.DSPARFILFIS;
		if(campo){
			if (!campoCompleto) {
				campoCompleto = "  " + campo;
			} else {
				if(campoCompleto.length ==1) {
					campoCompleto= campoCompleto+" "+campo;
				} else if(campoCompleto.length==2) {
					campoCompleto = campoCompleto+campo;
				} else {
					var parteinicial = campoCompleto.substr(0, 2);
					var partefinal = campoCompleto.substr(3);
					campoCompleto = parteinicial + campo + partefinal;
				}
			}
		}
		return campoCompleto;
	};
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Dfe43200_ParametrosDocumentosFiscaisSaida_Service', Dfe43200_ParametrosDocumentosFiscaisSaida_Service);
});

// FILE: ./assets/modules/saida/assets/js/services/Dfs11000_LancamentoSaida_Service.js
var dfs11000_LancamentoSaida_Service = function(localStorageEngine, Dfs11000_LancamentoSaida_Repositories, ZHPromise, General) {

    this.salvaVencimento = function(vencimento) {
        return Dfs11000_LancamentoSaida_Repositories.salvaVencimento(vencimento);
    };

    this.excluiVencimento = function(vencimento){
        return Dfs11000_LancamentoSaida_Repositories.excluiVencimento(vencimento);
    };

    this.excluirNota = function(consultaLancamentoRow) {
        return Dfs11000_LancamentoSaida_Repositories.excluirNota(consultaLancamentoRow.CDFILIAL, consultaLancamentoRow.NRLANCTONF);
    };

    var getDataSourceVencimento = function() {
        return localStorageEngine.getDataSource("df#/lancamento_nota_vencimento").then(function(vencimento){
            return vencimento;
        });
    };
    
    this.getFilial = function(){
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
        ];
        return General.callRestEngine("df#/crud/filial", "FilterData", filter);
    };

    this.getProxSeqVencimento = function(){
        return setMaxSequencialVencimento().then(function(maxSeqVencimento){
            var proxSeqVencimento = parseInt(maxSeqVencimento) + 1;
            var str = "" + proxSeqVencimento;
            
            var pad = "00";
            return pad.substring(0, pad.length - str.length) + str;
        });
    };

    var setMaxSequencialVencimento = function() {
        return getDataSourceVencimento().then(function(vencimentos){
            var maxSeqVencimento = '00';
            vencimentos.forEach(function(vencimento){
                if(parseInt(vencimento.NRSEQVENREC) > parseInt(maxSeqVencimento))
                    maxSeqVencimento = vencimento.NRSEQVENREC;
            });
            return maxSeqVencimento;
        });
    };

    this.retornaClienteFilial = function(filial, cliente) {
        return Dfs11000_LancamentoSaida_Repositories.retornaClienteFilial(filial, cliente);
    };
    
    //----------------------------------- DFS11300 Replicação de Documentos ---------------------------------------------------//
    
    /**
     * Replica o documento através de chamada
     * do backend.
     * 
     * @param {object} widget
     * 
     * return {promisse}
     */
    this.replicarDocumento = function(dataset) {
        return General.callRestEngine('df#/replicacaoManual', 'DataSet', dataset);
    };
    
    this.copiarNF = function(row){
      return  General.callRestEngine('df_saida#/copiarNF', 'Row', row);
    };
    
    this.saveDataUpdate = function (rowsUpdate, NEWDTENTRSAID) {
        return Dfs11000_LancamentoSaida_Repositories.saveDataUpdate(rowsUpdate, NEWDTENTRSAID);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfs11000_LancamentoSaida_Service", dfs11000_LancamentoSaida_Service);
});

// FILE: ./assets/modules/saida/assets/js/services/Dfs49000_TransmissaoNfe_Service.js
var dfs49000_TransmissaoNfe_Service = function(localStorageEngine, Dfs49000_TransmissaoNfe_Repositories) {
    this.montaConfigTransmissao = function(row){
        return Dfs49000_TransmissaoNfe_Repositories.montaConfigTransmissao(row);
    };
    
    this.buscaOperadorLogado = function() {
        var filter = [
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": "OPERADOR_LOGADO"
            }
        ];

        return localStorageEngine.download("df#/crud/operador", filter);
    };
    
    // Função que contém lista dos parâmetros gerais de Pedido que vem para este módulo
    this.getParamsOutModule = function() {
        
        var params = ['PED_CDFILIAL','PED_CDEMPRESA','PED_NRINSJURNF','PED_IDORIGDESTNF','PED_IDSTATUSNFE','PED_VRNOTAFISC'];
        
        return params;
        
    };
    
    // Função que retorna todas as variaveis de Pedido que ficam na cache
    this.getLocalVarsFilterOutModule = function() {
        
        var localVars = {};
        
        var params = this.getParamsOutModule();
        
        localVars.DTENTRSAID = {};
        
        localVars.DTENTRSAID.start = localStorageEngine.getLocalVar('PED_DTENTRSAID.start');
        localVars.DTENTRSAID.end = localStorageEngine.getLocalVar('PED_DTENTRSAID.end');
        
        params.forEach(function(param){
            
            localVars[param.substring(4,param.length)] = localStorageEngine.getLocalVar(param);
            
        });
        
        return localVars;
        
    };
    
    // Função que apaga todas as variaveis de PEDIDO que ficam na cache
    this.clearLocalVarsOutModule = function() {
        
        var params = this.getParamsOutModule();
        
        localStorage.removeItem('PED_DTENTRSAID.start');
        localStorage.removeItem('PED_DTENTRSAID.end');
        
        params.forEach(function(param){
            
            localStorage.removeItem(param);
            
        });
        
    };
    
    this.buscaDataProcessamento = function(filial) {
        return Dfs49000_TransmissaoNfe_Repositories.retornaDataProcessamento(filial);
    };

    this.statusServico = function(notasfiscais){
        return getDataSourceConfigTransmissao().then(function(config){
            return Dfs49000_TransmissaoNfe_Repositories.statusServico(notasfiscais, config);
        });
    };

    this.transmiteNotas = function(notasfiscais, tipo){
        return getDataSourceConfigTransmissao().then(function(config){
            return Dfs49000_TransmissaoNfe_Repositories.transmiteNotas(notasfiscais, config, tipo);
        });
    };

    this.atualizaLote = function(lotesMarcados){
        return getDataSourceConfigTransmissao().then(function(config){
            return Dfs49000_TransmissaoNfe_Repositories.atualizaLote(lotesMarcados, config);
        });
    };

    this.cancelarTransmissao = function(row){
        return getDataSourceConfigTransmissao().then(function(config){
            return Dfs49000_TransmissaoNfe_Repositories.cancelarTransmissao(row, config);
        });
    };

    this.enviarCartaCorrecao = function(notafiscal, dscorrecaonfe, nrseqcce){
        return getDataSourceConfigTransmissao().then(function(config){
            return Dfs49000_TransmissaoNfe_Repositories.enviarCartaCorrecao(notafiscal, dscorrecaonfe, nrseqcce, config);
        });
    };

    this.visualizarDanfe = function(notasMarcadas){
        return getDataSourceConfigTransmissao().then(function(config){
            return Dfs49000_TransmissaoNfe_Repositories.visualizarDanfe(notasMarcadas, config);
        });
    };

    this.downloadDanfe = function(notasMarcadas){
        return getDataSourceConfigTransmissao().then(function(config){
            return Dfs49000_TransmissaoNfe_Repositories.downloadDanfe(notasMarcadas, config);
        });
    };

    this.downloadXML = function(notasMarcadas){
        return getDataSourceConfigTransmissao().then(function(config){
            return Dfs49000_TransmissaoNfe_Repositories.downloadXML(notasMarcadas, config);
        });
    };

    this.enviarEmailPendenteNFsAceitas = function(notas){
        return getDataSourceConfigTransmissao().then(function(config){
            return Dfs49000_TransmissaoNfe_Repositories.enviarEmailPendenteNFsAceitas(notas, config);
        });
    };

    this.enviarEmailNF = function(nota, tipo){
        return getDataSourceConfigTransmissao().then(function(config){
            return Dfs49000_TransmissaoNfe_Repositories.enviarEmailNF(nota, config, tipo);
        });
    };
    
    this.enviarEmailCC = function(nota, nrseqcce, tipo){
        return getDataSourceConfigTransmissao().then(function(config){
            return Dfs49000_TransmissaoNfe_Repositories.enviarEmailCC(nota, config, tipo, nrseqcce);
        });
    };

    this.alteraDataEmissao = function(row) {
        return Dfs49000_TransmissaoNfe_Repositories.alteraDataEmissao(row);
    };

    this.getProxSeqProdRural = function(data) {
        var maxSeqProdRural = 0;
        if(data.length){
            data.forEach(function(row){
                if(row.NRSEQPRODRURAL > maxSeqProdRural)
                    maxSeqProdRural = row.NRSEQPRODRURAL;
            });
        }
        return maxSeqProdRural + 1;
    };

    this.getProxSeqCartaCorrecao = function(data) {
        var maxSeqProdRural = 0;
        if(data.length){
            data.forEach(function(row){
                if(row.NRSEQPRODRURAL > maxSeqProdRural)
                    maxSeqProdRural = row.NRSEQPRODRURAL;
            });
        }
        return maxSeqProdRural + 1;
    };

    this.salvaProdutorRural = function(row) {
        return Dfs49000_TransmissaoNfe_Repositories.salvaProdutorRural(row);
    };

    this.excluirProdutorRural = function(row) {
        return Dfs49000_TransmissaoNfe_Repositories.excluirProdutorRural(row);
    };

    this.getConfig = function() {
        return getDataSourceConfigTransmissao();  
    };

    var getDataSourceConfigTransmissao = function() {
        return localStorageEngine.getDataSource("df#/config_transmissao").then(function(transmissao){
            return transmissao[0].config;
        });
    };

    this.getMaxNrseqcce = function(filial, lancamento){
        return Dfs49000_TransmissaoNfe_Repositories.getMaxNrseqcce(filial, lancamento);
    };
    
    this.requestModulesMetadata = () => {
        return Dfs49000_TransmissaoNfe_Repositories.requestModulesMetadata();
    };
    
    this.aprovarNotasTransmissao = (notas) => {
        return Dfs49000_TransmissaoNfe_Repositories.aprovarNotasTransmissao(notas);
    };
    
    this.emailAutomaticoTransmissao = (notas, config) => {
        return Dfs49000_TransmissaoNfe_Repositories.emailAutomaticoTransmissao(notas, config);
    };
    
    this.errosNotasTransmissao = (erros) => {
        let arrayErros = [];
        
        for(let i in erros) {
            arrayErros[i] = {CDFILIAL:      erros[i].CDFILIAL,
                             NRLANCTONF:    erros[i].NRLANCTONF,
                             IDENTRSAIDOP:  erros[i].IDENTRSAIDOP,
                             NRORG:         erros[i].NRORG,
                             TPERRO:        erros[i].TPERRO};
            
            if(Array.isArray(erros[i].DSERRO)) {
                for(let index in erros[i].DSERRO) {
                    if(erros[i].DSERRO[index].DSERRO) {
                        arrayErros[i].DSERRO = erros[i].DSERRO[index].DSERRO; //Retorna apenas o primeiro erro encontrado do lançamento.
                        break;
                    }
                }
            }
            else arrayErros[i].DSERRO = erros[i].DSERRO;
        }
        return arrayErros;
    };
    
    this.consultaSituacaoNF = (row, config) => {
        return Dfs49000_TransmissaoNfe_Repositories.consultaSituacaoNF(row, config);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfs49000_TransmissaoNfe_Service", dfs49000_TransmissaoNfe_Service);
});

// FILE: ./assets/modules/saida/assets/js/services/Dfs51000_InutilizacaoNotaFiscalEletronica_Service.js
var Dfs51000_InutilizacaoNotaFiscalEletronica_Service = function(Dfs51000_InutilizacaoNotaFiscalEletronica_Repository,restEngine, templateManager, Dfs49000_TransmissaoNfe_Repositories) {

    this.confirmarInutilizacao = function(rows) {
        return this.getDataSourceConfigTransmissao().then(function(config){
            return Dfs51000_InutilizacaoNotaFiscalEletronica_Repository.inutilizarNF(rows, config);
        });
    };

    this.confirmarAgendamento = function(row) {
        return this.getDataSourceConfigTransmissao().then(function(config){
            return Dfs51000_InutilizacaoNotaFiscalEletronica_Repository.confirmarAgendamento(row, config);
        });
    };

    this.buscaAgendamento = function(row,config) {
        var filtro = [];
        var empresa = {
            "name": "P_CDEMPRESA",
            "operator":"=",
            "value":row.CDEMPRESA
        };
        var cnpj = {
            "name": "P_NRCNPJINSESTNFE",
            "operator":"=",
            "value":row.NRINSJURIEST
        };
        var serie = {
            "name": "P_CDSERNFE",
            "operator":"=",
            "value":row.CDSERIENFE
        };
        var ambiente = {
            "name": "P_IDTPAMBIENTENFE",
            "operator":"=",
            "value":config.tpAmb
        };
        filtro.push(empresa);
        filtro.push(cnpj);
        filtro.push(serie);
        filtro.push(ambiente);
        return Dfs51000_InutilizacaoNotaFiscalEletronica_Repository.buscaAgendamento(filtro);
    };

    this.aplicaJustificativa = function(justificativa, widgets,filtro){
        var inscricaoEstadual = filtro.FILTER_ROW.CDINSCESTA;
        var inscricaoMunicipal= filtro.FILTER_ROW.CDINSCMUNI;
        widgets.forEach(function(widget) {
            widget.JUSTIFICATIVA = justificativa;
            widget.CDINSCESTA = inscricaoEstadual;
            widget.CDINSCMUNI = inscricaoMunicipal;
        });
    };

    this.getDataSourceConfigTransmissao = function() {
        return Dfs49000_TransmissaoNfe_Repositories.montaConfigTransmissao(templateManager.container.getWidget('gridNotasFiscais').appliedFilter.FILTER_ROW).then(function(transmissao){
            return transmissao[0].config;
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfs51000_InutilizacaoNotaFiscalEletronica_Service", Dfs51000_InutilizacaoNotaFiscalEletronica_Service);
});

// FILE: ./assets/modules/saida/assets/js/services/Dfs61000_TransmissaoNfse_Service.js
var Dfs61000_TransmissaoNfse_Service = function(ZHPromise, restEngine, General) {
    
    this.montarLotes = function(rows, size) {
        var deferred = ZHPromise.defer();
        var groups = [], i;
        for (i = 0; i < rows.length; i += size) {
            groups.push(rows.slice(i, i + size));
        }
        deferred.resolve(groups);
        return deferred.promise;
    };
    
    this.enviarLoteRps = function(lote) {
        var options = {
            "requestType": "DataSet",
            "dataset": lote
        };
        return restEngine.post("df_saida#/enviarLoteRps", options);
    };
    
    this.visualizarNFSe = function(nota) {
        return General.callRestEngine("df_saida#/visualizarNFSe", "Row", nota);
    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfs61000_TransmissaoNfse_Service", Dfs61000_TransmissaoNfse_Service);
});

// FILE: ./assets/modules/saida/assets/js/services/Dfs71000_DevolucaoDasTransferenciasDoOperadorLogistico_Service.js
var Dfs71000_DevolucaoDasTransferenciasDoOperadorLogistico_Service = function(General){
    
    this.buildFilter = function(fieldsName, currentRow) {
        let filter = [];
        fieldsName.forEach((fieldName) => {
            filter.push({
                "name": "P_".concat(fieldName),
                "operator": "=",
                "value": (currentRow[fieldName]) ? currentRow[fieldName] : 'T'
            }); 
        });
        return filter;
    };
    
    this.getParamGeral = function() {
        return General.callRestEngine("df#/crud/paramgeral", "FilterData", []);
    };
    
    this.getDtProcessa = function(){
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
        ];
        return General.callRestEngine("df#/crud/paramestoque_processamento", "FilterData", filter);
    };
    
    this.getFilial = function(){
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
        ];
        return General.callRestEngine("df#/crud/filial", "FilterData", filter);
    };
    
    
    this.dataAtualFormatada = function(){
        var data = new Date();
        var dia = data.getDate() + 1;
        if (dia.toString().length == 1)
          dia = "0"+dia;
        var mes = data.getMonth()+1;
        if (mes.toString().length == 1)
          mes = "0"+mes;
        var ano = data.getFullYear();  
        return dia+"/"+mes+"/"+ano;
    };
};
Configuration(function (ContextRegister){
    ContextRegister.register('Dfs71000_DevolucaoDasTransferenciasDoOperadorLogistico_Service', Dfs71000_DevolucaoDasTransferenciasDoOperadorLogistico_Service);
});


// FILE: ./assets/modules/saida/assets/js/services/Dfs74000_CalculoReceitaBrutaSimplesNacional_Service.js
var Dfs74000_CalculoReceitaBrutaSimplesNacional_Service = function(Dfs74000_CalculoReceitaBrutaSimplesNacional_Repository) {

    this.calculaValorTotalRceitaBruta = function(row){
        if(!row.VRSALDOCOMPOE){
            row.VRSALDOCOMPOE='0';
        }
        if(!row.VRSALDODEDUZ){
            row.VRSALDODEDUZ='0';
        }
        row.VRSALDOCOMPOE = String(row.VRSALDOCOMPOE).replace(',','.');
        row.VRSALDODEDUZ  = String(row.VRSALDODEDUZ).replace(',','.');

        row.VRSALDOCOMPOE = parseFloat(row.VRSALDOCOMPOE);
        row.VRSALDODEDUZ  = parseFloat(row.VRSALDODEDUZ);

        row.VRRECEITABR = row.VRSALDOCOMPOE - row.VRSALDODEDUZ;

        if(row.VRRECEITABR<0){
            row.VRSALDONEGMES = row.VRRECEITABR *(-1);
            row.VRRECEITABR = 0;
        }else{
            row.VRSALDONEGMES = 0;
        }
    };

    this.verificaValorAlterado  = function(row){
        var valorCompoem       = row.VRSALDOCOMPOE;
        var valorAntigoCompoem = row.VRSALDOCOMPOE_OLD;
        var valorDeduz         = row.VRSALDODEDUZ;
        var valorAntigoDeduz   = row.VRSALDODEDUZ_OLD;
        //caso seja alterado o valor(compoem ou deduz) deve setar o campo IDINFMANUAL para M 'Manual'
        if(valorCompoem !== valorAntigoCompoem || valorDeduz !== valorAntigoDeduz){
            row.IDINFMANUAL='M';
        }
    };

    this.buscaMesPosterior = function (row){
        return Dfs74000_CalculoReceitaBrutaSimplesNacional_Repository.buscaMesPosterior(row);
    };

    this.calculaReceitaBrutaMes = function (row){
        return Dfs74000_CalculoReceitaBrutaSimplesNacional_Repository.calculaReceitaBrutaMes(row);
    };

    this.bloquerMesReceita = function (row){
        return Dfs74000_CalculoReceitaBrutaSimplesNacional_Repository.bloquerMesReceita(row);
    };

    this.buscaNotasFiscaisMes = function (row){
        return Dfs74000_CalculoReceitaBrutaSimplesNacional_Repository.buscaNotasFiscaisMes(row);
    };

    this.saveReceitaBruta = function (row){
        return Dfs74000_CalculoReceitaBrutaSimplesNacional_Repository.saveReceitaBruta(row);
    };

    this.deleteMesReceita = function (row){
        return Dfs74000_CalculoReceitaBrutaSimplesNacional_Repository.deleteMesReceita(row);
    };

    this.validacoesSave = function (row){
        return Dfs74000_CalculoReceitaBrutaSimplesNacional_Repository.validacoesSave(row);
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfs74000_CalculoReceitaBrutaSimplesNacional_Service", Dfs74000_CalculoReceitaBrutaSimplesNacional_Service);
});

// FILE: ./assets/modules/saida/assets/js/services/Dfs85000_ManifestoDocumentoEletronico_Service.js
var Dfs85000_ManifestoDocumentoEletronico_Service = function(General, ZHPromise, localStorageEngine){
    
   this.buildFilter = function(fieldsName, currentRow, insstr = false) {
        var deferred = ZHPromise.defer();
        let filter = [];
        fieldsName.forEach((fieldName) => {
            var value;
            if(fieldName != "NRMDFEINI" && fieldName != "NRMDFEFIN") {
                value = 'T';
            }
            else {
                value = '';
            }
            
            if (currentRow[fieldName]) {
                if (Array.isArray(currentRow[fieldName])) {
                    if (currentRow[fieldName].length) {
                        value = montaInStr(currentRow[fieldName]);
                    }
                }
                else {
                    value = currentRow[fieldName];
                }
            }
            
            filter.push({
                "name": "P_".concat(fieldName),
                "operator": "=",
                "value": value
            }); 
        });
        deferred.resolve(filter);
        return deferred.promise;
    };
    
    this.setDefaultMdfe = function(defaultMdfe) {
        return localStorageEngine.setDataSource("df#/defaultMdfe", defaultMdfe);
    };
    
    this.getDefaultMdfe = function() {
        return localStorageEngine.getDataSource("df#/defaultMdfe");
    };
    
    var montaInStr = function(row) {
        var strReturn = '_';
        row.forEach((value) => {
            strReturn += value + '_';
        });
        return strReturn;
    };
    
    this.saveNotasRelacionadas = function (widget) {
        var rows = widget.getCheckedRows();
        var widgetManifesto = widget.container.getWidget('manifesto_eletronico');
        
        var cdviatura = widgetManifesto.currentRow.CDVIATURA;
        var nrlanctomdfe = widgetManifesto.currentRow.NRLANCTOMDFE;
        
        rows.forEach( row => {
            row.CDVIATURA = cdviatura;
            row.NRLANCTOMDFE = nrlanctomdfe;
            row.__is_new = true;
        });
        
        return General.callRestEngine("df#/notasRelacionadasMdfe/save", "DataSet", rows);
    };
    

};

Configuration(function (ContextRegister){
    ContextRegister.register('Dfs85000_ManifestoDocumentoEletronico_Service', Dfs85000_ManifestoDocumentoEletronico_Service);
});


// FILE: ./assets/modules/saida/assets/js/services/Dfs88000_Conhecimento_de_transporte_eletronico_Service.js
function Dfs88000_Conhecimento_de_transporte_eletronico_Service(General, ZHPromise, localStorageEngine) {
    
    this.setDefaultCte = function(defaultMdfe) {
        return localStorageEngine.setDataSource("df#/defaultCte", defaultMdfe);
    };
    
    this.getDefaultCte = function() {
        return localStorageEngine.getDataSource("df#/defaultCte");
    };
    
    this.buildFilter = function(row) {
        var filter = [];

        for (let fieldName of ['REMETCDCLIENTE', 'REMETCDFORNECED', 'EXPCDCLIENTE', 'EXPCDFORNECED', 'RECEBCDCLIENTE', 'RECEBCDFORNECED', 'DESTCDCLIENTE', 'DESTCDFORNECED', 'TOMADCDCLIENTE', 'TOMADCDFORNECED']) {
            if (row[fieldName]) {
                filter.push( { "name": "P_"+fieldName, "value": row[fieldName], "operator": "=" } );
            }
        }
        
        for (let fieldName of ['CDEMPRESA', 'IDSTATUSCTE', 'IDCADDESTINATA', 'IDCADTOMADOR', 'IDCADREMETENTE', 'IDCADEXPEDIDOR', 'IDCADRECEBEDOR']) {
            if (row[fieldName]) {
                filter.push( { "name": fieldName, "value": row[fieldName], "operator": "=" } );
            }
        }
    
        if (row.NRINSESTEMIT) {
            filter.push( { "name": "CDINSCESTA", "value": row.NRINSESTEMIT, "operator": "=" } );
        }
        
        if (row.DTEMISSAO) {
            filter.push( { "name": "DTEMISSAO", "value": [row.DTEMISSAO.start, moment(row.DTEMISSAO.end, "DD/MM/YY hh:mm:ss").add(1, "days").format("DD/MM/YYYY")], "operator": "BETWEEN" } );
        }
        
        return filter;
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Dfs88000_Conhecimento_de_transporte_eletronico_Service", Dfs88000_Conhecimento_de_transporte_eletronico_Service);
});


// FILE: ./assets/modules/saida/assets/js/services/Grs15000_ExportacaoNfFaturamentoSap_Service.js
var Grs15000_ExportacaoNfFaturamentoSap_Service = function() {
    
    this.validaStatusNf = function (notas){
        var notasImportdas      = true;
        
        notas.forEach(function (nota) {
            if (nota.IMPORT === 'X') 
                notasImportdas = false;
        });
        
        return notasImportdas;
    };
    
    this.preparaRow = function (row){
        var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];  
        // Verifica se é um array de rows;
        if( Array.isArray(row) ){
            var rows =[];
            row.forEach(function(linhaSelecionada) {
                rows.push( Util.clone(linhaSelecionada, ignoredKeys) );
            });
            return rows;
        }else{
            return Util.clone(row, ignoredKeys);    
        }
        
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("Grs15000_ExportacaoNfFaturamentoSap_Service", Grs15000_ExportacaoNfFaturamentoSap_Service);
});

// FILE: ./assets/modules/relatorios/assets/js/services/Adm03000_ReNotasFiscais_Service.js
function Adm03000_ReNotasFiscais_Service(localStorageEngine, Adm03000_ReNotasFiscais_Repository) {
	var paramFilial;
	var paramNf;

	this.setParametersFilial = function() {
		var filial = arguments[0] || getLocalVar('CDFILIAL') || 'FILIAL_LOGADA',
		    filtroFilial = [
		    	{
					"name": "CDFILIAL",
					"operator": "=",
					"value": filial
				}
			];

		return Adm03000_ReNotasFiscais_Repository.parametersForFilial(filtroFilial).then(function(result) {
			var parametros = result.dataset.filial_logada;
			paramFilial = parametros[0];
			paramNf = parametros[1];

			// console.log(paramNf[0]);

			var params =[];
			params.push(this.checaAlmoxarifado());
			params.push(paramFilial.CDFILIAL);
			params.push(paramFilial.NMFILIAL);
			params.push(paramNf.DATARE);

			return params;
		}.bind(this));
	};

    this.allInvoicesImportedByXML = () => paramNf.IDNFGERADAVIAXML != 'N';

	this.getValueValidSoma = function(nameSomatorio) {
		if(nameSomatorio === 'NRCOUNTCGC') {
			return paramNf.NRCOUNTCGC;
		} else if(nameSomatorio === 'NRCOUNTEMIS') {
			return paramNf.NRCOUNTEMIS;
		} else if(nameSomatorio === 'VRTOTANOTA') {
			return paramNf.VRTOTANOTA;
		} else if(nameSomatorio === 'NRCOUNTNUM') {
			return paramNf.NRCOUNTNUM;
		}
	};

	this.setParamNota = function(cdfilial, cdalmoxarife, cdlocalestoq, datare) {
		if(paramFilial.IDUTILLCESTQ === 'N') {
			cdlocalestoq = ' ';
		}

		if(paramFilial.IDUTILALMOX === 'N') {
			cdalmoxarife = ' ';
		}

		var filtro = [
			{
				"name": "CDFILIAL",
				"operator": "=",
				"value": cdfilial
			},
			{
				"name": "DATARE",
				"operator": "=",
				"value": datare
			},
			{
				"name": "CDALMOXARIFE",
				"operator": "=",
				"value": cdalmoxarife
			},
			{
				"name": "CDLOCALESTOQ",
				"operator": "=",
				"value": cdlocalestoq
			}
		];

		return Adm03000_ReNotasFiscais_Repository.parametersForNotaFiscal(filtro).then(function(result) {
				var newParam = result.dataset.notafiscal_forAdm03000;
				paramNf = newParam[0] || {
                    IDNFGERADAVIAXML: null,
					NRCOUNTCGC: null,
					NRCOUNTEMIS: null,
					VRTOTANOTA: null,
					NRCOUNTNUM: null
				};
		});
	};

	this.checaAlmoxarifado = function() {
		if(paramFilial.IDUTILALMOX === 'S') {
			return true;
		} else {
			return false;
		}
	};

	this.checaUtilizaLocal = function() {
		if(paramFilial.IDUTILLCESTQ === 'S') {
			return true;
		} else {
			return false;
		}
	};

	this.checaIdconscgcre = function() {
		if(paramFilial.IDCONSCGCRE === 'S') {
			return true;
		} else {
			return false;
		}
	};

	this.checaIdconsemisre = function() {
		if(paramFilial.IDCONSEMISRE  === 'S') {
			return true;
		} else {
			return false;
		}
	};

	this.checaIdconstotare = function() {
		if(paramFilial.IDCONSTOTARE === 'S') {
			return true;
		} else {
			return false;
		}
	};

	this.checaIdconsnumere = function() {
		if(paramFilial.IDCONSNUMERE === 'S') {
			return true;
		} else {
			return false;
		}
	};

	this.checaNotaEmpty = function() {
		if(paramNf.NRCOUNTCGC !== null) {
			return false;
		}

		if(paramNf.NRCOUNTEMIS !== null) {
			return false;
		}

		if(paramNf.NRCOUNTNUM !== null) {
			return false;
		}

		if(paramNf.VRTOTANOTA !== null) {
			return false;
		}

		return true;
	};

	this.buidlFilter = function(row) {
		if(row.CDALMOXARIFE === null) {
			row.CDALMOXARIFE = "T";
		}

		if(row.CDLOCALESTOQ === null) {
			row.CDLOCALESTOQ = "T";
		}

		return Adm03000_ReNotasFiscais_Repository.geraRelatorio(row);
	};

	this.retornaOperadorLogado = function() {
        var filter = [
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": "OPERADOR_LOGADO"
            }
        ];

        return localStorageEngine.download("df#/crud/operador", filter);
    };

	this.getNrorg = function() {
		return this.retornaOperadorLogado().then(function(operador) {
			return operador[0].NRORG;
		});
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('Adm03000_ReNotasFiscais_Service', Adm03000_ReNotasFiscais_Service);
});


// FILE: ./assets/modules/relatorios/assets/js/services/Dfe16000_ConsistenciasAfNf_Service.js
function Dfe16000_ConsistenciasAfNf_Service(ZHPromise, Dfe16000_ConsistenciasAfNf_Repository) {

	this.generateReport = function(row) {
		return Dfe16000_ConsistenciasAfNf_Repository.generateReport(row);
	};

	this.getMainQueryData = function(row) {
		var params = {
			P_DTINICIAL: row.P_DTINICIAL,
			P_DTFINAL: row.P_DTFINAL,
			L_CDFILIRECE: row.L_CDFILIRECE
		};
		return Dfe16000_ConsistenciasAfNf_Repository.getMainQueryData(params);
	};

	this.checkDataExistance = function(row) {
		var deferred = ZHPromise.defer();
		var params = {
			P_DTINICIAL: row.P_DTINICIAL,
			P_DTFINAL: row.P_DTFINAL,
			L_CDFILIRECE: row.L_CDFILIRECE
		};
		Dfe16000_ConsistenciasAfNf_Repository.getMainQueryData(params).then((function(data) {
			var thereIsData = ((!!data.dataset) && (data.dataset.mainQueryData.length > 0)) ? true : false;
			deferred.resolve(thereIsData);
		}), function(error) {
			deferred.reject(error);
		});
		return deferred;
	};

}

Configuration(function(ContextRegister) {
	ContextRegister.register('Dfe16000_ConsistenciasAfNf_Service', Dfe16000_ConsistenciasAfNf_Service);
});


// FILE: ./assets/modules/relatorios/assets/js/services/Dfe23000_ComprasRealizadasPorMes_Service.js
var Dfe23000_ComprasRealizadasPorMes_Service = function(Dfe23000_ComprasRealizadasPorMes_Repository) {
    var idutilalmox;

    this.setParamFilial = function(cdfilial) {
        return Dfe23000_ComprasRealizadasPorMes_Repository.getParamFilial(cdfilial).then(function(banco) {
            var paramFilial = banco.dataset.paramfilial_paramvalidacoes[0];
            idutilalmox = paramFilial.IDUTILALMOX;
        });
    };

    this.utilizaAlmox = function() {
        if(idutilalmox === 'S'){
            return true;
        } else {
            return false;
        }
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Dfe23000_ComprasRealizadasPorMes_Service', Dfe23000_ComprasRealizadasPorMes_Service);
});

// FILE: ./assets/modules/relatorios/assets/js/services/Dfe26000_InflacaoInterna_Service.js
var Dfe26000_InflacaoInterna_Service = function(Dfe26000_InflacaoInterna_Repository) {

    this.generateReport = function(filterRow){
        var paransReport = {};
        if(filterRow.REPORT_NAME === 'Inflação Interna'){
            paransReport = {
                "REPORT_NAME": 'DFE26000',
                "TIPOCUSTO":filterRow.TIPOCUSTO,
                "NMTIPOCUSTO":filterRow.NMTIPOCUSTO,
                "CDOPERENTR":filterRow.CDOPERENTR,
                "CDPRODFINAL":filterRow.CDPRODFINAL,
                "CDFILIAL":filterRow.CDFILIAL,
                "DATAINICIAL":filterRow.DATAINICIAL,
                "DATAFINAL":filterRow.DATAFINAL,
                "CDPRODINI":filterRow.CDPRODINI,
                "TAMNIVEL":filterRow.TAMNIVEL
            };
        } else if(filterRow.REPORT_NAME === 'Inflação de Compras'){
            paransReport = {
                "REPORT_NAME": 'DFE26001',
                "INIMES1": filterRow.INIMES1,
                "INIMES2": filterRow.INIMES2,
                "FINMES1": filterRow.FINMES1,
                "FINMES2": filterRow.FINMES2,
                "CDOPERENTR": filterRow.CDOPERENTR,
                "CDFILIAL": filterRow.CDFILIAL,
                "CDPRODINI": filterRow.CDPRODINI,
                "CDPRODFINAL": filterRow.CDPRODFINAL,
                "TAMNIVEL": filterRow.TAMNIVEL
            };
        }else if(filterRow.REPORT_NAME === 'Inflação Interna Consolidada'){
            paransReport = {
                "REPORT_NAME": 'DFE26000_Consolidada',
                "TIPOCUSTO":filterRow.TIPOCUSTO,
                "NMTIPOCUSTO":filterRow.NMTIPOCUSTO,
                "CDOPERENTR":filterRow.CDOPERENTR,
                "CDPRODFINAL":filterRow.CDPRODFINAL,
                "CDGRUPFILI":filterRow.CDGRUPFILI,
                "DATAINICIAL":filterRow.DATAINICIAL,
                "DATAFINAL":filterRow.DATAFINAL,
                "CDPRODINI":filterRow.CDPRODINI,
                "TAMNIVEL":filterRow.TAMNIVEL
            };
        } else {
            paransReport = {
                "REPORT_NAME": 'DFE26002',
                "INIMES1": filterRow.INIMES1,
                "INIMES2": filterRow.INIMES2,
                "FINMES1": filterRow.FINMES1,
                "FINMES2": filterRow.FINMES2,
                "CDOPERENTR": filterRow.CDOPERENTR,
                "CDFILIAL": filterRow.CDFILIAL,
                "CDPRODINI": filterRow.CDPRODINI,
                "CDPRODFINAL": filterRow.CDPRODFINAL,
                "TAMNIVEL": filterRow.TAMNIVEL
            };
        }
        return Dfe26000_InflacaoInterna_Repository.generateReport(paransReport);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Dfe26000_InflacaoInterna_Service', Dfe26000_InflacaoInterna_Service);
});

// FILE: ./assets/js/controllers/Dfs37000_ProdutosDiferimentoImposto_Controller.js
function Dfs37000_ProdutosDiferimentoImposto_Controller(General, ScreenService, templateManager) {
    
    this.newDiferimento = function(widget) {
        var newWidget = templateManager.container.getWidget('PERCENTUAL');
        var message = "Preencha os campos de país e estado antes de adicionar.";
        if(!(newWidget.getField('SGESTADO').value())||
        !(newWidget.getField('NMPAIS').value())) 
        {
            ScreenService.showMessage(message);
        }  
        else {
            widget.newRow();  
            
        }
    };
    
    
    this.newDiferimentoCliente = function(widget) {
        var newWidget = templateManager.container.getWidget('EXCECAOPERCENTUAL');
        var message = "Preencha os campos de país, estado e cliente antes de adicionar.";
        
        if(!(newWidget.getField('SGESTADO').value()) ||
           !(newWidget.getField('NMPAIS').value()) ||
           !(newWidget.getField('NMRAZSOCCLIE').value())) {
            ScreenService.showMessage(message);
        }  
        else {
            widget.newRow();  
        }  
    };
    
    
    
    this.saveDiferimento = function(widget) {
       
        if(widget.isValid()){
            if(widget.getField('VRPERDIFERE').value() >= 0 && widget.getField('VRPERDIFERE').value() <= 100) {
                widget.currentRow.CDPAIS = widget.getParent().getField('CDPAIS').value();
                widget.currentRow.SGESTADO = widget.getParent().getField('SGESTADO').value();
                if(widget.name === 'DADOSPERCENTUALCLIENTE') {
                    widget.currentRow.CDCLIENTE = widget.getParent().getField('CDCLIENTE').value();
                }
        		widget.saveRow().then(function(){
        			if(widget.currentRow.__is_new === true) {
        			    ScreenService.goBack();
        			    widget.list();
        			    widget.reload();
        			    widget.currentRow.__is_new = false;
        			}
        			else {
            			widget.view();
            			widget.reload();
        			}
        		}, function(calback){
        			ScreenService.showMessage(calback.data.error);
    			});
            }
            else {
                var message = "Percentual(%) deve estar entre 0 e 100";
                widget.getField('VRPERDIFERE').clearValue();
                ScreenService.showMessage(message);
                widget.reload();
            }
    	}
    };
    this.deleteDiferimento = function(widget) {
        General.deleteCrud(widget);
    };
     
    this.editDiferimento = function(widget) {
        
        widget.backupRow = Util.clone(widget.currentRow);
        widget.edit();
        
    };
    
    this.fieldOnChangePais = function() {
        var widget = templateManager.container.getWidget('PERCENTUAL');
        var dadosWidget = templateManager.container.getWidget('DADOSPERCENTUAL');
        widget.getField('SGESTADO').clearValue();
        widget.getField('NMESTADO').clearValue();
        widget.getField('NMESTADO').initializeField();
        
        if(!widget.getField('NMPAIS').value()) {
            widget.getField('NMESTADO').readOnly = true;
            
        }
        else {
            widget.getField('NMESTADO').readOnly = false;
            
            
        }
        var field = widget.getField('NMESTADO');
        field.reload();
        widget.reload();
        dadosWidget.reload();
    };
    
    this.fieldOnChangeEstado = function() {
        var dadosWidget = templateManager.container.getWidget('DADOSPERCENTUAL');
        dadosWidget.reload();
        
    };
    
    this.fieldOnChangePaisCliente = function() {
        var widget = templateManager.container.getWidget('EXCECAOPERCENTUAL');
        var dadosWidget = templateManager.container.getWidget('DADOSPERCENTUALCLIENTE');
        widget.getField('SGESTADO').clearValue();
        widget.getField('NMESTADO').clearValue();
        widget.currentRow.NRINSJURCLIE = '';
        widget.getField('NMRAZSOCCLIE').clearValue();
        widget.getField('NMRAZSOCCLIE').readOnly = true;
        widget.getField('NMESTADO').initializeField();
        
        if(!widget.getField('NMPAIS').value()) {
            
            widget.getField('NMESTADO').readOnly = true;
            
        }
        else {
            
            widget.getField('NMESTADO').readOnly = false;
            
        }
        
        var field = widget.getField('NMESTADO');
        field.reload();     
        widget.reload();
        dadosWidget.reload();
    };
    
    this.fieldOnChangeEstadoCliente = function() {
        var widget = templateManager.container.getWidget('EXCECAOPERCENTUAL');
        var dadosWidget = templateManager.container.getWidget('DADOSPERCENTUALCLIENTE');
        widget.currentRow.NRINSJURCLIE = '';
        widget.getField('NMRAZSOCCLIE').clearValue();
        
        if(!widget.getField('NMESTADO').value()) {
            widget.getField('NMRAZSOCCLIE').readOnly = true;
            
        }
        else {
            widget.getField('NMRAZSOCCLIE').readOnly = false;
            
        }
        
        var field = widget.getField('NMRAZSOCCLIE');
        field.reload();
        widget.reload();
        dadosWidget.reload();
    };
    
    this.fieldOnChangeCliente = function() {
        var widget = templateManager.container.getWidget('EXCECAOPERCENTUAL');
        var field = widget.getField('NMRAZSOCCLIE');
        var dadosWidget = templateManager.container.getWidget('DADOSPERCENTUALCLIENTE');
        field.reload();
        widget.reload(); 
        dadosWidget.reload();
    };
}
Configuration(function (ContextRegister) {
    ContextRegister.register('Dfs37000_ProdutosDiferimentoImposto_Controller', Dfs37000_ProdutosDiferimentoImposto_Controller);
});

// FILE: ./assets/js/controllers/Dfs56000_AprovacaoLancamentos_Controller.js
var dfs56000_AprovacaoLancamentos_Controller = function(General, templateManager, ScreenService, ZhFilterPreferences, Df11000_Lancamento_Service, AprovacaoLancamentos_Service, Df56000_AprovacaoLancamentos_Service, ErrosAprovacaoLancamentos_Utilities, ErrosCancelamentoAprovacaoLancamentos_Utilities, contabilizacaoController, ZHPromise) {
    
    var isAprovacao = 1;
    
    var requestError = function(error){
        var errorMessage = "Ocorreu um erro ao tentar realizar a operação.";
        if (error && error.data) {
            errorMessage = error.data.error;
        }
        ScreenService.showMessage(errorMessage, "ERROR");
    };

    this.validacoesWidgetOnEnterOnce = function(aprovacaoLancamentosArgs) {
        var aprovacaoLancamentosWidget = aprovacaoLancamentosArgs.owner;
            
        if (sessionStorage.getItem('CDEMPRESA') &&  sessionStorage.getItem('NMEMPRESA') && sessionStorage.getItem('DTEMISSAO') && sessionStorage.getItem('IDAPROVACAO') && sessionStorage.getItem('IDENTRSAIDOP') ){
            filtroPainelDeControle(aprovacaoLancamentosArgs);
            sessionStorage.clear();
        }else{
            setDefaultFilialFilter(aprovacaoLancamentosWidget).then(defaultFilter =>{
                defaultFilter.IDENTRSAIDOP = "E";
                ZhFilterPreferences.setOriginalFilter(aprovacaoLancamentosWidget.id, defaultFilter);
                ZhFilterPreferences.openFilterWidget(aprovacaoLancamentosWidget, defaultFilter).then(() => {
                 templateManager.container.getActiveWidget().getField("CDFILIAL").reload();
                });
            });
        }
       
    };
    
    var filtroPainelDeControle = function(args) {
        var aprovacaoLancamento    = args.owner;
        let IDAPROVACAO =  sessionStorage.getItem('IDAPROVACAO');
        let CDEMPRESA =  sessionStorage.getItem('CDEMPRESA');
        let NMEMPRESA =  sessionStorage.getItem('NMEMPRESA'); 
        let IDENTRSAIDOP =  sessionStorage.getItem('IDENTRSAIDOP');
        let DTENTRSAID =  JSON.parse(sessionStorage.getItem('DTEMISSAO'));
        let dataSourceFilter = aprovacaoLancamento.dataSourceFilter;
        let defaultFilter = {};
        
     dataSourceFilter.forEach(function(data) {
            
            if(data.name === 'P_IDAPROVACAO'  ){
                data.value = IDAPROVACAO;
            }else if(data.name === 'P_CDEMPRESA' ){
                data.value = CDEMPRESA;
            }else if(data.name === 'P_IDENTRSAIDOP' ){
                data.value = IDENTRSAIDOP;
            }else if(data.name === 'P_IDSTATUS' ){
                data.value = 'P';
            }else if(data.name === 'P_CDFILIAL' ){
                data.value = 'TD';
            }
        });
        
        dataSourceFilter.push( {operator: "=", name: "P_DTENTRSAIDINI", value: DTENTRSAID.start});
        dataSourceFilter.push( {operator: "=", name: "P_DTENTRSAIDFIN", value: DTENTRSAID.end});
        dataSourceFilter.push( {operator: "=", name: "P_IDAVISOS", value: "TD"});
        dataSourceFilter.push( {operator: "=", name: "P_IDERROS", value: "TD"});
        
        defaultFilter.IDENTRSAIDOP = IDENTRSAIDOP;
        defaultFilter.IDSTATUS = 'P';
        defaultFilter.IDAPROVACAO = IDAPROVACAO;
        defaultFilter.DTENTRSAID = DTENTRSAID;
        defaultFilter.CDEMPRESA = CDEMPRESA;
        defaultFilter.NMEMPRESA = NMEMPRESA;
    
        args.e.preventDefault();
        aprovacaoLancamento.dataSourceFilter = dataSourceFilter;
        aprovacaoLancamento.reload().then(() => {
            aprovacaoLancamento.activate();
            ZhFilterPreferences.setOriginalFilter(aprovacaoLancamento.id, defaultFilter);
        });
    };
           
            
        
        
    this.changeIdAprovacao = function(field) {
        if(field.value() == '1') field.widget.container.label = "Aprovação de Lançamentos";
        else {
            field.widget.container.label = "Auditoria de Lançamentos";
        }   
    };
    

    var setDefaultFilialFilter = function(cabecalhoWidget) {
        var popupFilter = Util.clone(cabecalhoWidget.currentRow);
        var promise = ZHPromise.defer(); 
        var filter = [
           {name: "CDOPERADOR", operator: "=",value: "OPERADOR_LOGADO"},
           {name: "CDFILIAL", operator: "=", value: "FILIAL_LOGADA"}
        ];
        
        General.callRestEngine( "df#/filioper_lancamento_notafiscal", "FilterData", filter).
        then( (filialLogada) =>  {

            var filial;
            
            //Utiliza o primeiro dataset retornado. 
            //Necessário pois o dataset pode ser filioper_lancamento_notafiscal ou perfacessfili_lancamento_notafiscal
            for (let datasetName in filialLogada.dataset) {
                if (filialLogada.dataset.hasOwnProperty(datasetName)) {
                    filial = filialLogada.dataset[datasetName][0];
            	    break;
                }
            }
            
            if (filial) {
                popupFilter.NMFILIAL = [filial.NMFILIAL];
                popupFilter.CDFILIAL = [filial.CDFILIAL];
                popupFilter.NMEMPRESA = filial.NMEMPRESA;
                popupFilter.CDEMPRESA = filial.CDEMPRESA;
            }
            else {
                popupFilter.NMFILIAL = '';
                popupFilter.CDFILIAL = '';
                popupFilter.NMEMPRESA = '';
                popupFilter.CDEMPRESA = '';
            }
            
            popupFilter.CDINSCESTA   =  filial.CDINSCESTA;
            popupFilter.CDINSCMUNI   =  filial.CDINSCMUNI;
            popupFilter.IDAPROVACAO  =  "1";
            popupFilter.IDSITUACAO   =  "";
            popupFilter.IDSTATUS     =  "";
            popupFilter.IDORIGDESTNF =  "";
            
            popupFilter.DTENTRSAID = {start: moment().format("DD/MM/YYYY"), end: moment().format("DD/MM/YYYY")};        
                    
            promise.resolve(popupFilter);
            
        }); 
  
        return promise;
    };
    
    
    var setCurrentRowWithOutDataFilial = function(filialOut, filialField, currentRow) {
        var defaultValue = filialField.dataSource.data.find(function(data) {
            return data.CDFILIAL == filialOut;
        });

        var filialOutData = filialField.outData;

        setCurrentRowWithOutData(currentRow, defaultValue, filialOutData);
    };

    var setCurrentRowWithOutData = function(currentRow, valuesToSetOnCurrentRow, outDataValue) {
        for (var key in outDataValue) {
            let data = valuesToSetOnCurrentRow ? [valuesToSetOnCurrentRow[key]] : [];
            currentRow[outDataValue[key]] = data;
        }
    };
    
    this.getDataProcessamento = function(filialSelecionada){
        var promise = ZHPromise.defer();
        Df11000_Lancamento_Service.buscaDataProcessamento(filialSelecionada).then(function(processamento){
            var dtProcessFormat = moment(processamento[0].DTPROCESSA, 'DD/MM/YYYY').format('DD/MM/YYYY');
            promise.resolve(dtProcessFormat);
        });
        return promise;
    };
    
    this.setCurrentRowWithOutData = function(currentRow, valuesToSetOnCurrentRow, outDataValue) {
        for (var key in outDataValue) {
            currentRow[outDataValue[key]] = valuesToSetOnCurrentRow[key];
        }
    };

    var buscaOperadorLogado = function() {
        return Df11000_Lancamento_Service.buscaOperadorLogado();
    };

    this.getCdoperadorLogado = function() {
        return buscaOperadorLogado().then(function(operador){
            return operador[0].CDOPERADOR;
        });
    };

    this.empresaOnChange = function(aprovacaoLancamentosArgs) {
        validaCamposInscricao(aprovacaoLancamentosArgs);
        var unidade = aprovacaoLancamentosArgs.owner.field.widget.getField("CDFILIAL");
        aprovacaoLancamentosArgs.row.CDFILIAL = "TD";
        aprovacaoLancamentosArgs.row.NMFILIAL = "TD";
        unidade.reload();
    };
    

    var validaCamposInscricao = function(aprovacaoLancamentosArgs){
        var inscEsta = aprovacaoLancamentosArgs.owner.field.widget.getField("CDINSCESTA");
        var inscMuni = aprovacaoLancamentosArgs.owner.field.widget.getField("CDINSCMUNI");

        inscEsta.readOnly = !aprovacaoLancamentosArgs.row.NMEMPRESA;
        inscMuni.readOnly = !aprovacaoLancamentosArgs.row.NMEMPRESA;

        if(inscEsta.readOnly){
            aprovacaoLancamentosArgs.row.CDINSCESTA = '';
            aprovacaoLancamentosArgs.row.CDINSCMUNI = '';
        }
    };
    
    this.vencimentoOnEnter = function(args){
        var widget = args.owner;
        Df11000_Lancamento_Service.validaStatusVencimento(widget);
    };

    var validaCampoEmpresa = function(aprovacaoLancamentosArgs) {
        var empresa = aprovacaoLancamentosArgs.owner.field.widget.getField("NMEMPRESA");
        empresa.readOnly = aprovacaoLancamentosArgs.row.CDFILIAL;
    };

    var validaCampoDsserie = function(aprovacaoLancamentosArgs) {
        var dsserie = aprovacaoLancamentosArgs.owner.field.widget.getField("DSSERIE");
        dsserie.readOnly = !aprovacaoLancamentosArgs.row.CDFILIAL;
    };

    this.unidadeOnChange = function(aprovacaoLancamentosArgs) {
        validaCampoEmpresa(aprovacaoLancamentosArgs);
        validaCampoDsserie(aprovacaoLancamentosArgs);
        validaCamposInscricao(aprovacaoLancamentosArgs);
    };
    

    var getAprovacaoLancamentosWidget = function(){
        if(templateManager.container)
            return templateManager.container.getWidget("aprovacaoLancamentos");
    };

    this.getCurrentField = function(field) {
        
        var aprovacaoLancamentosWidget = getAprovacaoLancamentosWidget();
        if (aprovacaoLancamentosWidget)
            return !aprovacaoLancamentosWidget.currentRow[field] ? null : aprovacaoLancamentosWidget.currentRow[field];
    };

    this.getEmpresaFiltrada = function(){
        var aprovacaoLancamentosWidget = getAprovacaoLancamentosWidget();
        if(aprovacaoLancamentosWidget){
            return aprovacaoLancamentosWidget.appliedFilter.FILTER_ROW.CDEMPRESA;
        }
    };
    
    this.inicializaAnexoAprovLanc = function(args){
        var filter = [
            {
                "name": "P_CDFILIAL",
                "operator": "=",
                "value": args.owner.getParent().currentRow.CDFILIAL
            },
            {
                "name": "P_NRLANCTONF",
                "operator": "=",
                "value": args.owner.getParent().currentRow.NRLANCTONF
            }
            
        ];

        General.callRestEngine("df#/crud/anexo_obrigatorio", "FilterData", filter).then((success) => {
            if(success.dataset.anexo_obrigatorio[0].IDBLOQANEXO) {
                if(success.dataset.anexo_obrigatorio[0].IDBLOQANEXO == 0) {
                    args.owner.getAction("actNovoAnexo").isVisible = false;
                    args.owner.getAction("actDeleteAnexo").isVisible = false;
                }
                else if(success.dataset.anexo_obrigatorio[0].IDBLOQANEXO == 1 && args.owner.getParent().currentRow.IDSTATUS == 'A') {
                    args.owner.getAction("actNovoAnexo").isVisible = false;     
                    args.owner.getAction("actDeleteAnexo").isVisible = false;
                }                
                else if(success.dataset.anexo_obrigatorio[0].IDBLOQANEXO == 2 && (args.owner.getParent().currentRow.IDBLOQNFFIN == 'L')) {
                    args.owner.getAction("actNovoAnexo").isVisible = false;     
                    args.owner.getAction("actDeleteAnexo").isVisible = false;
                }
                else {
                    args.owner.getAction("actNovoAnexo").isVisible = true;    
                    args.owner.getAction("actDeleteAnexo").isVisible = true;
                }
            }
        });
        args.owner.reload();
    };
    
    this.salvarAnexo = function(widget){
        if(widget.isValid()){
            var row = widget.currentRow;
            var widgetReload = widget.parent.parent;
            row.CDFILIAL = widgetReload.currentRow.CDFILIAL;
            row.NRLANCTONF = widgetReload.currentRow.NRLANCTONF;
            row = Util.clone(widget.currentRow,['__rightActions']);
            General.callRestEngine('df_entrada#/salvar_Anexo', "Row", row).then(function() {
                widget.reload();
                ScreenService.closeSwipe();
                widgetReload.resizeGridColumns(true);
            }.bind(this), this.errorRequest);
        }
    };
    
    this.setDataSourceFilters = function(filter){
        var aprovacaoLancamentosWidget = getAprovacaoLancamentosWidget();
        var filterRow = [];
            filterRow.push(filter.owner.appliedFilter.FILTER_ROW);
        var teste = filterRow.map(e =>Object.entries(e))[0];
        
        //popup[0] = name
        //popup[1] = value
        if(aprovacaoLancamentosWidget.appliedFilter){
            //filter.e.filters.forEach(popup =>{
            teste.forEach(popup =>{
              aprovacaoLancamentosWidget.dataSourceFilter.forEach(dataSourceFilter => {
                  if(dataSourceFilter.name.split('P_')[1] === popup[0])
                    if(popup[1]) 
                        dataSourceFilter.value = popup[1];
                    else dataSourceFilter.value = 'TD';
                //filter.e.filters.splice(filter.e.filters.indexOf(popup), 1);
              }); 
            });
        }
        aprovacaoLancamentosWidget.reload();
        
        let aprovacao = filter.owner.appliedFilter.FILTER_ROW.IDAPROVACAO == '1';
        
        this.isAprovacao = aprovacao;
        
        aprovacaoLancamentosWidget.getAction('actAprovarLancamento').isVisible = aprovacao;
        aprovacaoLancamentosWidget.getAction('actCancelarAprovacao').isVisible = aprovacao;
        
        aprovacaoLancamentosWidget.getAction('actAuditarLancamento').isVisible = !aprovacao;
    };
    
    this.criaDataSourceFilterAfterApply = function(args) {
        args.e.preventDefault();
        var aprovacaoLancamentosWidget = args.owner;
        var dataSourceFilter = aprovacaoLancamentosWidget.dataSourceFilter;

        montaDatasFiltro(dataSourceFilter);
        montaIdsituacaoFiltro(dataSourceFilter);

        aprovacaoLancamentosWidget.dataSourceFilter = aprovacaoLancamentosWidget.dataSourceFilter.filter(function(field){return field.name[1] == '_' || field.name == 'CDOPERADOR';});
        this.setDataSourceFilters(args);
    };

    var montaDatasFiltro = function(dataSourceFilter){
        var camposData = ['P_DTENTRSAIDINI', 'P_DTENTRSAIDFIN', 'P_DTEMISSAOINI', 'P_DTEMISSAOFIN', 'P_IDAVISOS', 'P_IDERROS'];
        var datasRemover = dataSourceFilter.filter(function(field){
            return camposData.indexOf(field.name) >= 0;
        });
        if(datasRemover && datasRemover.length >= 0){
            datasRemover.forEach(function(data){
                dataSourceFilter.splice(dataSourceFilter.indexOf(data), 1);
            });
        }
        var datasFiltro = dataSourceFilter.filter(function(value) {
            return value.name === "DTENTRSAID" || value.name === 'DTEMISSAO';
        });
        if(datasFiltro.length === 1){
            var dataEmissao = {
                "name": "DTEMISSAO",
                "value": {
                    0: "TD", 1: "TD"
                }
            };
            datasFiltro.push(dataEmissao);
        }
        var filterDefault = {
            'operator': '='
        };
        datasFiltro.forEach(function(data) {
            filterDefault.name = 'P_' + data.name + 'INI';
            filterDefault.value = data.value[0];
            dataSourceFilter.push(Util.clone(filterDefault));
            filterDefault.name = 'P_' + data.name + 'FIN';
            filterDefault.value = data.value[1];
            dataSourceFilter.push(Util.clone(filterDefault));

            if(dataSourceFilter.indexOf(data) > 0)
                dataSourceFilter.splice(dataSourceFilter.indexOf(data), 1);
        });
    };

    var montaIdsituacaoFiltro = function(dataSourceFilter){
        var situacao = dataSourceFilter.filter(function(value) {
            return value.name === "IDSITUACAO";
        });
        var filterDefault = {
            'operator': '='
        };

        if(situacao.length === 0){
            filterDefault.name = 'P_IDAVISOS';
            filterDefault.value = 'TD';
            dataSourceFilter.push(Util.clone(filterDefault));
            filterDefault.name = 'P_IDERROS';
            filterDefault.value = 'TD';
            dataSourceFilter.push(Util.clone(filterDefault));
        }
        else{
            switch (situacao[0].value.match(/[a-zA-Z]+/g)[0]) {
                case 'E':
                    filterDefault.name = 'P_IDAVISOS';
                    filterDefault.value = 'TD';
                    dataSourceFilter.push(Util.clone(filterDefault));
                    filterDefault.name = 'P_IDERROS';
                    filterDefault.value = '1';
                    dataSourceFilter.push(Util.clone(filterDefault));
                    break;
                case 'A':
                    filterDefault.name = 'P_IDAVISOS';
                    filterDefault.value = '1';
                    dataSourceFilter.push(Util.clone(filterDefault));
                    filterDefault.name = 'P_IDERROS';
                    filterDefault.value = 'TD';
                    dataSourceFilter.push(Util.clone(filterDefault));
                    break;
                case 'O':
                    filterDefault.name = 'P_IDAVISOS';
                    filterDefault.value = '0';
                    dataSourceFilter.push(Util.clone(filterDefault));
                    filterDefault.name = 'P_IDERROS';
                    filterDefault.value = '0';
                    dataSourceFilter.push(Util.clone(filterDefault));
                    break;
            }

            dataSourceFilter.splice(dataSourceFilter.indexOf(situacao[0]), 1);
        }
    };

    this.validacoesAfterMoveRow = function(widget) {
        checaVisibilidadeActions(widget, widget.currentRow.IDAPROVACAO === '1', widget.currentRow.IDSTATUS === 'P', widget.currentRow.IDSTATUSNFE === 'P', widget.currentRow.IDNFGERADAVIAXML != 'N');
    };

    this.idorigdestOnChange = function(filterArgs) {
        alteraVisibilidadeCampoInscricaoFiltro(filterArgs);
        limpaCamposOutDataSelectsTpOperacFiltro(filterArgs);
    };

    this.identrsaidopOnChange = function(filterArgs) {
        alteraVisibilidadeCampoSerieFiltro(filterArgs);
        alteraVisibilidadeCamposInscricao(filterArgs);
        alteraVisibilidadeCampoUnidadeOrigem(filterArgs);
        alteraDatasoureceVencimento(filterArgs);
        this.alteraVisiblidadeCamposVencimento(filterArgs);
        alteraLabelFieldData(filterArgs);
    };

    var alteraVisibilidadeCampoInscricaoFiltro = function(filterArgs) {
        var filtroWidget = filterArgs.owner.field.widget;
        var cliente = filtroWidget.getField("NMRAZSOCCLIE");
        var fornecedor = filtroWidget.getField("NMRAZSOCFORN");
        var filial = filtroWidget.getField("DSFILIRECE");
        var tipoDestino = filterArgs.row.IDORIGDESTNF;

        switch (tipoDestino) {
            case "C":
                validacaoCamposOutTpOperacFilter(cliente, fornecedor, filial);
                break;
            case "F":
                validacaoCamposOutTpOperacFilter(fornecedor, cliente, filial);
                break;
            case "L":
                validacaoCamposOutTpOperacFilter(filial, fornecedor, cliente);
                break;
            default:
                filial.clearValue();
                filial.readOnly = true;
                cliente.clearValue();
                cliente.readOnly = true;
                fornecedor.clearValue();
                fornecedor.readOnly = true;
        }
    };

    var alteraVisibilidadeCampoSerieFiltro = function(filterArgs) {
        var filtroWidget = filterArgs.owner.field.widget;
        var dsserieentr = filtroWidget.getField("DSSERIEENTR");
        var dsserie = filtroWidget.getField("DSSERIE");
        var entrada_saida = filterArgs.row.IDENTRSAIDOP;

        if(entrada_saida === 'E'){
            filterArgs.row.CDSERIE = '';
            filterArgs.row.DSSERIE = '';
            dsserie.isVisible = false;
            dsserie.showOnForm = false;
            dsserieentr.isVisible = true;
            dsserieentr.showOnForm = true;
        }
        else{
            filterArgs.row.CDSERIEENTR = '';
            filterArgs.row.DSSERIEENTR = '';
            dsserieentr.isVisible = false;
            dsserieentr.showOnForm = false;
            dsserie.isVisible = true;
            dsserie.showOnForm = true;
        }
    };

    var alteraVisibilidadeCamposInscricao = function(filterArgs) {
        var filtroWidget = filterArgs.owner.field.widget;
        var cdinscesta = filtroWidget.getField("CDINSCESTA");
        var cdinscmuni = filtroWidget.getField("CDINSCMUNI");
        var entrada_saida = filterArgs.row.IDENTRSAIDOP;

        if(entrada_saida === 'E'){
            cdinscmuni.isVisible = false;
            cdinscmuni.showOnForm = false;
            cdinscesta.isVisible = false;
            cdinscesta.showOnForm = false;
            // filterArgs.row.CDINSCESTA = '';
            // filterArgs.row.CDINSCMUNI = '';
        }
        else{
            cdinscesta.isVisible = true;
            cdinscesta.showOnForm = true;
            cdinscmuni.isVisible = true;
            cdinscmuni.showOnForm = true;
        }
    };

    var alteraVisibilidadeCampoUnidadeOrigem = function(filterArgs) {
        var filtroWidget = filterArgs.owner.field.widget;
        var dsfiliemit = filtroWidget.getField("DSFILIEMIT");
        var entrada_saida = filterArgs.row.IDENTRSAIDOP;

        if(entrada_saida === 'E'){
            dsfiliemit.isVisible = false;
            dsfiliemit.showOnForm = false;
        }
        else{
            dsfiliemit.isVisible = true;
            dsfiliemit.showOnForm = true;
        }
    };

    var alteraDatasoureceVencimento = function(args) {
        var vencimentoWidget = templateManager.container.getWidget('vencimentos');
        var row = templateManager.container.getWidget('aprovacaoLancamentos').currentRow;
        var entrada_saida = args.row.IDENTRSAIDOP? args.row.IDENTRSAIDOP : row.IDENTRSAIDOP;

        var filter = [
                        {
                            "name": "CDFILIAL",
                            "operator": "=",
                            "value": row.CDFILIAL
                        },
                        {
                            "name": "NRLANCTONF",
                            "operator": "=",
                            "value": row.NRLANCTONF
                        }
                    ];
                    
        vencimentoWidget.dataSource.name = "";
        vencimentoWidget.dataSource.data = [];
        vencimentoWidget.dataSourceFilter = filter;
        
        if(entrada_saida === 'E'){
            vencimentoWidget.dataSource.name = 'df#/crud/contapagar';
        }
        else{
            vencimentoWidget.dataSource.name = 'df#/crud/contareceber';
        }    

    };
    
    var alteraLabelFieldData = function(args) {
        var entrada_saida = args.row.IDENTRSAIDOP;
        
        args.owner.field.widget.getField('DTENTRSAID').label = (entrada_saida == 'E') ? 'Data Entrada' : 'Data Saída';
        templateManager.container.getWidget('aprovacaoLancamentos').getField('DTENTRSAID').label = (entrada_saida == 'E') ? 'Data Entrada' : 'Data Saída';
        args.owner.field.widget.reload();
    };

    this.alteraVisiblidadeCamposVencimento = function(filterArgs) {
        var vencimentoWidget = templateManager.container.getWidget('vencimentos');
        var dtatuavenpag = vencimentoWidget.getField("DTATUAVENPAG");
        var vratuapag = vencimentoWidget.getField("VRATUAPAG");
        var idstatpag = vencimentoWidget.getField("IDSTATPAG");
        var dtatuavenrec = vencimentoWidget.getField("DTATUAVENREC");
        var vratuarec = vencimentoWidget.getField("VRATUAREC");
        var idstatrec = vencimentoWidget.getField("IDSTATREC");
        var row = templateManager.container.getWidget('aprovacaoLancamentos').currentRow;
        var entrada_saida = filterArgs.row.IDENTRSAIDOP? filterArgs.row.IDENTRSAIDOP : row.IDENTRSAIDOP;

        alteraDatasoureceVencimento(filterArgs); 
    
        if(vencimentoWidget.dataSource.name == 'df#/crud/contareceber' && entrada_saida == "S" ) {
            dtatuavenpag.isVisible = false;
            vratuapag.isVisible = false;
            idstatpag.isVisible = false;
            dtatuavenrec.isVisible = true;
            vratuarec.isVisible = true;
            idstatrec.isVisible = true;

        }
        else {
            vencimentoWidget.dataSource.name = 'df#/crud/contapagar';
            dtatuavenpag.isVisible = true;
            vratuapag.isVisible = true;
            idstatpag.isVisible = true;
            dtatuavenrec.isVisible = false;
            vratuarec.isVisible = false;
            idstatrec.isVisible = false;

        }
        
        vencimentoWidget.reload();
        
    };

    var limpaCamposOutDataSelectsTpOperacFiltro = function(filterArgs)
    {
        var filtroWidget = filterArgs.owner.field.widget;
        var aprovacaoRow = filtroWidget.currentRow;
        var cliente = filtroWidget.getField("NMRAZSOCCLIE");
        var fornecedor = filtroWidget.getField("NMRAZSOCFORN");
        var filial = filtroWidget.getField("DSFILIRECE");

        filial.value(null);

        cliente.value(null);
        aprovacaoRow.CDCLIENTE = null;

        fornecedor.value(null);
        aprovacaoRow.CDFORNECED = null;
        aprovacaoRow.IDSITUCADA = null;
        aprovacaoRow.IDBLQNFRELAFOAF = null;

        aprovacaoRow.NRINSJURNF = null;
        aprovacaoRow.NMINSJURNF = null;
        aprovacaoRow.IDTPINSJURNF = null;
    };

    var validacaoCamposOutTpOperacFilter = function(princialField) {
        princialField.showOnView = true;
        princialField.showOnForm = true;
        princialField.isVisible = true;
        princialField.readOnly = false;

        for (var i = 1; i < arguments.length; i++) {
            arguments[i].showOnView = false;
            arguments[i].showOnForm = false;
            arguments[i].isVisible = false;
            arguments[i].readOnly = true;
        }
    };

    var checaVisibilidadeActions = function(widgetPai, telaAprovacao, pendente, NfePendente, geradaViaXml){
        var actAlterarDataEmissao = widgetPai.getAction('actAlterarDataEmissao'),
            actCompletarNF = widgetPai.getAction('actCompletarNF');
        
        var exibirBotoes = pendente && NfePendente && !geradaViaXml;
        actAlterarDataEmissao.isVisible = exibirBotoes;
        actCompletarNF.isVisible        = exibirBotoes && telaAprovacao;
    };

    /*
        idtipo = 1 -> Alterar data de emissão
        idtipo = 2 -> Inserir número e data de emissão em nota fiscal não eletrônica
    */
    this.actEditarOnClick = function(aprovacaoLancamentosArgs, idtipo = 1) {
        var aprovacaoLancamentosWidget = aprovacaoLancamentosArgs.owner.widget;
        if(idtipo !== 2) {
            aprovacaoLancamentosWidget.getField("NRNOTAFISC").readOnly = true;
            delete aprovacaoLancamentosWidget.getField("NRNOTAFISC").validations.required;
        }  else {
            aprovacaoLancamentosWidget.getField("NRNOTAFISC").readOnly = false;
            aprovacaoLancamentosWidget.getField("NRNOTAFISC").validations.required = true;
        }
        
        
        General.edit(aprovacaoLancamentosArgs.owner.widget);
    };

    this.cancelaEdicao = function(args){
        General.backOrCancel(args);
    };

    this.salvarEdicaoLancamento = function(args){
        let currentRow = args.row,
            widget     = args.owner.widget;
        
        if(!widget.isValid()){
            return;
        }
        
        Df56000_AprovacaoLancamentos_Service.validaEdicaoLancamento(currentRow).then( 
            response => {
                if(response.dataset.numero_valido[0] === 'S' && response.dataset.dtemissao_valida[0] === 'S'){
                    Df56000_AprovacaoLancamentos_Service.salvarEdicaoLancamento(currentRow).then(
                        () => {
                        args.owner.widget.view();
                        ScreenService.successNotification('Nota alterada com sucesso');
                        this.deleteMaskNumeroNota(widget);
                    });
                }
            }
        ).catch(
            error => {
                let message = error.data.error;
                
                ScreenService.showMessage(message);
                this.deleteMaskNumeroNota(widget);
            }    
        );
    };
    
    this.viewNotasBaseCalculo = function(args) {
        let widget = templateManager.container.getWidget('wdg_notasBaseCalc');
        
        ScreenService.openPopup(widget);
        widget.dataSourceFilter[0].value = args.row.CDFILIAL;
        widget.dataSourceFilter[1].value = args.row.NRLANCTONF;
        widget.reload();
        widget.isVisible = true;
    };

    this.aprovarNotasMarcadas = function(args){
        var notasMarcadas = args.owner.widget.getCheckedRows();
        var notas = notasMarcadas.filter(function(notafiscal){
            return notafiscal.IDSTATUS === "P";
        });

        if(notas.length > 0){
            let existeDtemissao = true,
                existeNrnotafisc = true;
                
            for(let i = 0; i < notas.length; i++){
                if(!notas[i].DTEMISSAO){
                    existeDtemissao = false;
                    break;
                }
                
                if(!notas[i].NRNOTAFISC){
                    existeNrnotafisc = false;
                    break;
                }
            }
            
            if(!existeNrnotafisc){
                ScreenService.showMessage("Informar o número das notas a serem aprovadas!");
            }
            else if(!existeDtemissao){
                ScreenService.showMessage("Informar a data de emissão das notas a serem aprovadas!");
            }
            else{
                AprovacaoLancamentos_Service.aprovarNotasMarcadas(notas).then((errosAprovacao) => {
                    args.owner.widget.reload() ;
                    if(errosAprovacao.dataset.errosAprovacao) {
                        if(errosAprovacao.dataset.errosAprovacao.length > 0)
                            ErrosAprovacaoLancamentos_Utilities.errosAprovacao(args.owner.widget, errosAprovacao.dataset.errosAprovacao, this.isAprovacao);
                        else {
                            ScreenService.successNotification('Nota(s) aprovada(s) com sucesso.');
                        }
                    }
                    else {
                        ScreenService.successNotification('Nota(s) aprovada(s) com sucesso.');
                    }
                })
                .catch(function(errorResponse) {
                    if (contabilizacaoController.checkErrosContabilizacao(errorResponse)) {
                        requestError(errorResponse);
                    }
                });
            }
        }
        else{
            ScreenService.showMessage("Nenhuma nota pendente foi selecionada!");
        }
    };
    
    this.auditarNotasMarcadas = function(args) {
        var notasMarcadas = args.owner.widget.getCheckedRows();
    
        General.callRestEngine("df#/auditarNotas", "DataSet", notasMarcadas).then(errosAuditoria => {
             args.owner.widget.reload() ;
            if(errosAuditoria.dataset.errosAuditoria) {
                if(errosAuditoria.dataset.errosAuditoria.length > 0)
                    ErrosAprovacaoLancamentos_Utilities.errosAprovacao(args.owner.widget, errosAuditoria.dataset.errosAuditoria, this.isAprovacao);
                else {
                    ScreenService.successNotification('Nota(s) auditada(s) com sucesso.');
                }
            }
            else {
                ScreenService.successNotification('Nota(s) auditada(s) com sucesso.');
            }
            
        }).catch((error) => {
            
        });
        
    };

    this.cancelarAprovacaoNotasMarcadas = function(args){
        var notasMarcadas = args.owner.widget.getCheckedRows();
        var notas = notasMarcadas.filter(function(notafiscal){
            return notafiscal.IDSTATUS === "A";
        });

        if(notas.length > 0){
            AprovacaoLancamentos_Service.cancelarAprovacaoNotasMarcadas(notas).then(function(errosCancelamento){
                args.owner.widget.reload();
                if(Object.keys(errosCancelamento.dataset.errosCancelamento).length > 0)
                    ErrosCancelamentoAprovacaoLancamentos_Utilities.errosCancelamento(args.owner.widget, errosCancelamento.dataset.errosCancelamento);
                else ScreenService.successNotification("Aprovação da(s) nota(s) cancelada com sucesso.");
            }, requestError);
        }
        else{
            ScreenService.showMessage("Nenhuma nota aprovada foi marcada!");
        }
    };
    
    var getMainWidget = function(){
        if(templateManager.container)
            return templateManager.container.getWidget("aprovacaoLancamentos");
    };
    
    this.getFieldValueFilter = function(fieldName, widgetName) {
        var container =  templateManager.container;
        if(container) {
            var filterWidget;
            var filterBforeApply = templateManager.container.getActiveWidget();
            if(filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
              filterWidget = filterBforeApply;
            else {
                if(filterBforeApply.lastActiveWidget.name == 'zhFilterPreferencesSelectConditions'){
                    filterWidget = filterBforeApply.lastActiveWidget;
                    return filterWidget.currentRow[fieldName];
                }
                else{
                    var f = container.getWidget(widgetName);
                    if(f) {
                         filterWidget = container.getWidget(widgetName).appliedFilter;
                    }
                }
                    
               
            }
            
            if(filterWidget) {
                var valueField;
                if(filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
                    valueField = filterWidget.currentRow[fieldName];
                else
                    valueField = filterWidget.FILTER_ROW[fieldName];
                    
                return valueField || null;
            }
            
               else return null;
        }
        else return null;
    };
    
    this.abrirWidgetContabilizacao = function(widget){
        let lancamento = widget.currentRow;
        contabilizacaoController.exibeContabilizacao(lancamento);
    };
    
    this.editar = function(widget){
        General.edit(widget);
    };
    
    this.verificaStatus = function (args){
        let IDSTATUS =  General.getFieldValue('IDSTATUS','aprovacaoLancamentos');
        let widget = templateManager.container.getActiveWidget();
        let actions = widget.actions;
        actions = actions.filter( action => action.name === "editPendentes" ? true : false );
        if(IDSTATUS === "P"){
            widget.getField("VRATUAPAG").readOnly = true;
            actions[0].readOnly = false;
            widget.reload();
        }else {
            widget.getField("VRATUAPAG").readOnly = false;
            actions[0].readOnly = true;
            widget.reload();
        }
        
    };
    
    this.editMaskNumeroNota = function(widget) {
        var quantCaracteres = widget.currentRow.MAXTAMNF || 9;
        var nrNotaFiscField = widget.getField('NRNOTAFISC');

        nrNotaFiscField.mask = {"type": "zerofill",
                                    "params": {
                                        "acceptOnlyNumbers": true,
                                        "allowEmpty": true
                                    }
                               };
        nrNotaFiscField.mask.params.maxlength = quantCaracteres;
        nrNotaFiscField.maxlength = quantCaracteres;
    };
    
    this.deleteMaskNumeroNota = function(widget) {
        var filter =  [{"name": "CDFILIAL", "operator": "=", "value": widget.currentRow.CDFILIEMIT}],
            nrNotaFiscField = widget.getField('NRNOTAFISC'),
            nrTamNumNfUt,
            quantCaracteres;
        
        General.callRestEngine("df#/crud/paramfilial", "FilterData", filter).then((success) => {
            if(success.dataset.paramfilial[0]) 
                nrTamNumNfUt = success.dataset.paramfilial[0].NRTAMNUMNFUT;
            
            quantCaracteres = nrTamNumNfUt ? nrTamNumNfUt : 9;
            nrNotaFiscField.mask.params.maxlength = quantCaracteres;
            nrNotaFiscField.validations.maxLength.length = nrTamNumNfUt ? nrTamNumNfUt : 20;
            nrNotaFiscField.reload();
        });
    };
    
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfs56000_AprovacaoLancamentos_Controller", dfs56000_AprovacaoLancamentos_Controller);
});

// FILE: ./assets/js/controllers/Fis02300_ModeloDocumentoFiscal_Controller.js
function Fis02300_ModeloDocumentoFiscal_Controller(templateManager, ScreenService, General) {

	this.checkTipoRegFiscalDataExists = function(field) {
		if (!field.dataSource.data) {
			ScreenService.showMessage("Não há tipo de registro fiscal cadastrado.");
		}
	};
	
	this.toggleReadOnlyTipoOperacao = function() {
		var widget = templateManager.container.getWidget('unidadeTabbed');
		if(widget.currentRow.IDENTRSAIDOP == undefined || widget.currentRow.IDENTRSAIDOP == "") 
			widget.getField('NMTIPOOPER').readOnly = true;
		else 
			widget.getField('NMTIPOOPER').readOnly = false;
		widget.reload();
	};
	
	this.deleteModeloDocumentoFiscal = function(widget) {
		var self = this;
		
		ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
			widget.removeRow().then(function () {
				self.reloadSelect();
				widget.reload();
				ScreenService.closeSwipe().then(function () {
					widget.activate();
				});
			}, function (error) {
				if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
					ScreenService.showMessage('Exclusão não permitida! O registro possui registros filhos.');
				} else {
					ScreenService.showMessage(error.data.error);
				}
			});
		});
	};

	this.entradaSaidaOnChange = function(widget) {
		var tipoOperField = widget.getField("NMTIPOOPER");
		
		if (widget.name != "modDocFiscTipoOper"){
			var modDocFiscTipoOperWidget = templateManager.container.getWidget("modDocFiscTipoOper");
			modDocFiscTipoOperWidget.dataSource.data = null;
		}
		tipoOperField.dataSourceFilter[0].value = widget.currentRow.IDENTRSAIDOP;
		tipoOperField.reload();
		tipoOperField.setValue("");
	};

	this.hasDuplicateKeys = function(widget) {
		var arr = widget.dataSource.data.filter(function(data) {
			return ((data.CDTIPOOPER === widget.currentRow.CDTIPOOPER) &&
				(data.IDENTRSAIDOP === widget.currentRow.IDENTRSAIDOP) &&
				(data.CDMODDOCFISC === widget.currentRow.CDMODDOCFISC));
		});
		
		return (arr.length > 1);
	};

	this.reloadSelect = function() {
		templateManager.container.getWidget("modDocFiscTipoOper").getField("DSMODDOCFISC").reload();
	};

	this.setDefaultTipoOper = function(childWidget) {
		var widget = childWidget.getParent();
		
		childWidget.currentRow.IDENTRSAIDOP = widget.currentRow.IDENTRSAIDOP || "E";
		childWidget.currentRow.CDTIPOOPER = widget.currentRow.CDTIPOOPER;
		childWidget.currentRow.NMTIPOOPER = widget.currentRow.NMTIPOOPER;
		childWidget.getField("NMTIPOOPER").dataSourceFilter[0].value = childWidget.currentRow.IDENTRSAIDOP;
		childWidget.getField("NMTIPOOPER").reload();
	};

	this.saveModeloDocumentoFiscal = function(widget) {
		if (widget.isValid()) {
			var self = this;
			widget.saveRow().then(function (success) {
				self.reloadSelect();
				if (widget.currentRow.__is_new) {
					widget.list(true);
					ScreenService.goBack();
				} else {
					widget.view();
				}
				widget.reload();
				widget.activate();
			}, function (error) {
				var errorMessage = 'Timeout';
				if (error.data) {
					errorMessage = error.data.error;
				}
				ScreenService.showMessage(errorMessage);
			});
		}
	};

	this.mudaVisibilidadeBloqueio = function(widget) {
		if(widget.getField("IDOBRIGANEXO").value()=='S')
			widget.getField("IDBLOQANEXO").readOnly = false;
		else
			widget.getField("IDBLOQANEXO").readOnly = true;
	};

	this.saveModeloDocumentoFiscalTipoOper = function(widget) {
		if (widget.isValid()) {
			if (!this.hasDuplicateKeys(widget)) {
				if(widget.currentRow.IDOBRIGANEXO == 'S'){
					widget.currentRow.IDBLOQNFFIN = 'B';
					widget.currentRow.CDOPERACTPG = getLocalVar('CDOPERADOR');
					widget.currentRow.CDOPERACTPGCAN = getLocalVar('CDOPERADOR');
					widget.currentRow.DTLIBERACAOFIN  = moment().format('DD/MM/YYYY h:mm:ss');
				}
				else {
					
					widget.currentRow.IDBLOQNFFIN = 'L';
					widget.currentRow.CDOPERACTPG = getLocalVar('CDOPERADOR');
					widget.currentRow.DTLIBERACAOFIN  = moment().format('DD/MM/YYYY h:mm:ss');
				}
				widget.dataSourceFilter[0].value = widget.currentRow.IDENTRSAIDOP;
				widget.dataSourceFilter[1].value = widget.currentRow.CDTIPOOPER;
				widget.getParent().currentRow.IDENTRSAIDOP = widget.currentRow.IDENTRSAIDOP;
				widget.getParent().currentRow.CDTIPOOPER = widget.currentRow.CDTIPOOPER;
				widget.getParent().currentRow.NMTIPOOPER = widget.currentRow.NMTIPOOPER;
				General.saveCrud(widget);
			}
			else {
				ScreenService.showMessage("Modelo de Documento Fiscal já cadastrado para o tipo de operação selecionado.");
			}
		}
	};

	this.updateGridFilter = function(widget) {
		var childWidget = templateManager.container.getWidget("modDocFiscTipoOper");
		
		childWidget.dataSourceFilter[0].value = widget.currentRow.IDENTRSAIDOP || "E";
		childWidget.dataSourceFilter[1].value = widget.currentRow.CDTIPOOPER;
		childWidget.reload();
	};
	
	this.cancelStatusNfe = function (widget) {
		widget.reload();
        widget.list();
	};
	
	this.voltar = function (widget) {
		ScreenService.goBack();
		widget.reload();
	};
	
	this.saveStatusNfe = function (widget) {
		var itens = widget.dataSource.data;
        var itensModificados = itens.filter(item => item.changed_IDMODDOCNFE);
        
        itensModificados.forEach (item => {
            item.changed = true;
        });
        
        widget.dataSource.save().then(function () {
            itensModificados.forEach (item => {
                //@todo remover delete item.changed quando desmarcaLinhasEditadas for implementada
                delete item.changed_IDMODDOCNFE;
                delete item.changed;
            });
            widget.activate();
            widget.list(true);
            ScreenService.successNotification("Todas as alterações foram salvas com sucesso.");
            widget.reload();
        });
	};
	    
	this.controlaStatusWidgets = function(mainWidget, nomeWidgetAtual, status){
        mainWidget.widgets.forEach(function(widg) {
            if(nomeWidgetAtual !== widg.name)
                widg.enabled = status;
        });

        if(mainWidget.name !== nomeWidgetAtual)
            mainWidget.enabled = status;
    };
    
    this.toggleFocused = function (widget, status) {
    	if(widget.name != 'modDocFisc') {
    		delete widget.getField('DSMODDOCFISC').focused;
    		widget.getField('IDENTRSAIDOP').focused = true;
    	} else {
	    	if(status == false) {
		    	delete widget.getField('DSMODDOCFISC').focused;
		    	widget.getField('CDMODDOCFISC').focused = !status;
		   } else {
		   		delete widget.getField('CDMODDOCFISC').focused;
		    	widget.getField('DSMODDOCFISC').focused = status;
		   } 
    	}
	};
	
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Fis02300_ModeloDocumentoFiscal_Controller', Fis02300_ModeloDocumentoFiscal_Controller);
});

// FILE: ./assets/js/controllers/Ger00400_Imposto_Controller.js
var  Ger00400_Imposto_Controller = function(templateManager,ScreenService, MessageService, General,ZHPromise,Ger00400_Imposto_Service){

    this.afterMoveRowImposto = function(args){
        var widgetContabilizacao = templateManager.container.getWidget('tabbedContabilizacao');
        this.controlaCampoTratamentoVisuzalizacao(args);
        this.controlaCampoApuracaoImposto(args);
        this.verificaCampoTramento(args);
        if(!args.owner.currentRow.__is_new){
            this.recarregarWidgetFilhas(widgetContabilizacao);
        }
    };

    this.recarregarWidgetFilhas  = function(widgetPai){
        widgetPai.widgets.forEach(function(widget) {
            if(widget.name == 'padrao'){
                widget.currentRow = widgetPai.getParent().currentRow;
                widget.reload();
            }
        });
    };

    this.controlaCampoTratamento = function(args){
        var widget = templateManager.container.getWidget('gridImposto');
        var campoTratamento = widget.getField('IDTRATIMPO');
        var campoDescontar = widget.getField('IDDESCRFONTE');
        var campoValor = widget.getField('IDACRPRODENTRNF');
        var campoAjuste = widget.getField('DSAJUCTREC');
        var campoAjusteReceber = widget.getField('DSAJUCTREC');
        var campoAjustePagar = widget.getField('DSAJUCTPAG');

        switch(campoTratamento.value() ){
            case 'A':
                campoValor.value('N');
                campoValor.isVisible = true;
                campoValor.showOnView = true;
                campoValor.showOnForm = true;
                campoDescontar.isVisible = false;
                campoDescontar.showOnView = false;
                campoDescontar.showOnForm = false;
                campoDescontar.readOnly = true;
                campoDescontar.value('');
                campoAjusteReceber.readOnly = true;
                campoAjusteReceber.isVisible = false;
                campoAjusteReceber.showOnView = false;
                campoAjusteReceber.showOnForm = false;
                campoAjusteReceber.value('');
                widget.currentRow.CDAJUCTREC = '';
                delete(campoAjusteReceber.validations.required);
                campoAjustePagar.isVisible = false;
                campoAjustePagar.showOnView = false;
                campoAjustePagar.showOnForm = false;
                campoAjustePagar.value('');
                widget.currentRow.CDAJUCTPAG = '';
                delete(campoAjustePagar.validations.required);
                break;
            case 'D':
                campoValor.isVisible = false;
                campoValor.showOnView = false;
                campoValor.showOnForm = false;
                campoValor.value('N');
                campoDescontar.isVisible = true;
                campoDescontar.showOnView = true;
                campoDescontar.showOnForm = true;
                campoDescontar.readOnly = true;
                campoDescontar.value('');
                campoAjusteReceber.readOnly = true;
                campoAjusteReceber.isVisible = true;
                campoAjusteReceber.showOnView = true;
                campoAjusteReceber.showOnForm = true;
                campoAjusteReceber.value('');
                widget.currentRow.CDAJUCTREC = '';
                delete(campoAjusteReceber.validations.required);
                campoAjustePagar.isVisible = false;
                campoAjustePagar.showOnView = false;
                campoAjustePagar.showOnForm = false;
                campoAjustePagar.value('');
                widget.currentRow.CDAJUCTPAG = '';
                delete(campoAjustePagar.validations.required);
                break;
            case 'V':
                campoValor.isVisible = false;
                campoValor.showOnView = false;
                campoValor.showOnForm = false;
                campoValor.value('N');
                campoDescontar.isVisible = true;
                campoDescontar.showOnView = true;
                campoDescontar.showOnForm = true;
                campoDescontar.readOnly = true;
                campoDescontar.value('');
                campoAjusteReceber.readOnly = true;
                campoAjusteReceber.isVisible = true;
                campoAjusteReceber.showOnView = true;
                campoAjusteReceber.showOnForm = true;
                campoAjusteReceber.value('');
                widget.currentRow.CDAJUCTREC = '';
                delete(campoAjusteReceber.validations.required);
                campoAjustePagar.isVisible = false;
                campoAjustePagar.showOnView = false;
                campoAjustePagar.showOnForm =false;
                campoAjustePagar.value('');
                widget.currentRow.CDAJUCTPAG = '';
                delete(campoAjustePagar.validations.required);
                break;
            case 'R':
                campoValor.isVisible = false;
                campoValor.showOnView = false;
                campoValor.showOnForm = false;
                campoValor.value('N');
                campoDescontar.isVisible = true;
                campoDescontar.showOnView = true;
                campoDescontar.showOnForm = true;
                campoDescontar.readOnly = false;
                campoDescontar.value('A');
                campoAjusteReceber.readOnly = false;
                campoAjusteReceber.isVisible = true;
                campoAjusteReceber.showOnView = true;
                campoAjusteReceber.showOnForm = true;
                campoAjusteReceber.validations.required = true;
                campoAjustePagar.isVisible = false;
                campoAjustePagar.showOnView = false;
                campoAjustePagar.showOnForm = false;
                campoAjustePagar.value('');
                widget.currentRow.CDAJUCTPAG = '';
                delete(campoAjustePagar.validations.required);
                break;
        }
        this.controlaCampoVisuzalizacaoImposto(args);
        this.controlaCampoDecontar(args);
    };

    this.controlaCampoTratamentoVisuzalizacao = function(args){
        var widget = templateManager.container.getWidget('gridImposto');
        var campoTratamento = widget.getField('IDTRATIMPO');
        var campoDescontar = widget.getField('IDDESCRFONTE');
        var campoValor = widget.getField('IDACRPRODENTRNF');
        var campoAjusteReceber = widget.getField('DSAJUCTREC');
        var campoAjustePagar = widget.getField('DSAJUCTPAG');

        switch(campoTratamento.value() ){
            case 'A':
                campoValor.value('N');
                campoValor.isVisible = true;
                campoValor.showOnView = true;
                campoValor.showOnForm = true;
                campoDescontar.isVisible = false;
                campoDescontar.showOnView = false;
                campoDescontar.showOnForm = false;
                campoDescontar.readOnly = true;

                campoAjusteReceber.readOnly = true;
                campoAjusteReceber.isVisible = false;
                campoAjusteReceber.showOnView = false;
                campoAjusteReceber.showOnForm = false;

                delete(campoAjusteReceber.validations.required);
                campoAjustePagar.isVisible = false;
                campoAjustePagar.showOnView = false;
                campoAjustePagar.showOnForm = false;
                campoAjustePagar.value('');
                widget.currentRow.CDAJUCTPAG = '';
                delete(campoAjustePagar.validations.required);
                break;
            case 'D':
                campoValor.isVisible = false;
                campoValor.showOnView = false;
                campoValor.showOnForm = false;
                campoDescontar.isVisible = true;
                campoDescontar.showOnView = true;
                campoDescontar.showOnForm = true;
                campoDescontar.readOnly = true;

                campoAjusteReceber.readOnly = true;
                campoAjusteReceber.isVisible = true;
                campoAjusteReceber.showOnView = true;
                campoAjusteReceber.showOnForm = true;


                delete(campoAjusteReceber.validations.required);
                campoAjustePagar.isVisible = false;
                campoAjustePagar.showOnView = false;
                campoAjustePagar.showOnForm = false;

                widget.currentRow.CDAJUCTPAG = '';
                delete(campoAjustePagar.validations.required);
                break;
            case 'V':
                campoValor.isVisible = false;
                campoValor.showOnView = false;
                campoValor.showOnForm = false;
                campoDescontar.isVisible = true;
                campoDescontar.showOnView = true;
                campoDescontar.showOnForm = true;
                campoDescontar.readOnly = true;

                campoAjusteReceber.readOnly = true;
                campoAjusteReceber.isVisible = true;
                campoAjusteReceber.showOnView = true;
                campoAjusteReceber.showOnForm = true;
                delete(campoAjusteReceber.validations.required);
                campoAjustePagar.isVisible = false;
                campoAjustePagar.showOnView = false;
                campoAjustePagar.showOnForm = false;
                delete(campoAjustePagar.validations.required);
                break;
            case 'R':
                campoValor.isVisible = false;
                campoValor.showOnView = false;
                campoValor.showOnForm = false;
                campoDescontar.isVisible = true;
                campoDescontar.showOnView = true;
                campoDescontar.showOnForm = true;
                campoDescontar.readOnly = false;

                campoAjusteReceber.readOnly = false;
                campoAjusteReceber.isVisible = true;
                campoAjusteReceber.showOnView = true;
                campoAjusteReceber.showOnForm = true;
                campoAjusteReceber.validations.required = true;
                campoAjustePagar.isVisible = false;
                campoAjustePagar.showOnView = false;
                campoAjustePagar.showOnForm = false;
                delete(campoAjustePagar.validations.required);
                break;
        }
        this.controlaCampoVisuzalizacaoImposto(args);
        this.controlaCampoDecontar(args);
    };

    this.verificaCampoTramento = function(args){
        var widget = templateManager.container.getWidget('gridImposto');
        var campoTratamento = widget.getField('IDTRATIMPO');
        var campoDescontar = widget.getField('IDDESCRFONTE');
        var campoAjusteReceber = widget.getField('DSAJUCTREC');
        var campoAjustePagar = widget.getField('DSAJUCTPAG');

        if(campoTratamento.value() == 'A'){
            campoAjusteReceber.isVisible = false;
            campoAjusteReceber.showOnForm = false;
            campoAjusteReceber.showOnView = false;
        }
    };

    this.controlaCampoDecontar = function(args){
        var widget = templateManager.container.getWidget('gridImposto');
        var campoDescontar = widget.getField('IDDESCRFONTE');
        var campoAjusteReceber = widget.getField('DSAJUCTREC');
        var campoAjustePagar = widget.getField('DSAJUCTPAG');

        switch( campoDescontar.value() ){
            case 'A':
                campoAjusteReceber.readOnly = false;
                campoAjusteReceber.validations.required = true;
                campoAjusteReceber.isVisible = true;
                campoAjusteReceber.showOnView = true;
                campoAjusteReceber.showOnForm = true;

                campoAjustePagar.isVisible = false;
                campoAjustePagar.readOnly = true;
                campoAjustePagar.showOnView = false;
                campoAjustePagar.showOnForm = false;
                delete(campoAjustePagar.validations.required);
                campoAjustePagar.value('');
                widget.currentRow.CDAJUCTPAG = '';
                break;

            case 'P':
                campoAjustePagar.readOnly = false;
                campoAjustePagar.validations.required= true;
                campoAjustePagar.isVisible=true;
                campoAjustePagar.showOnView = true;
                campoAjustePagar.showOnForm = true;

                campoAjusteReceber.isVisible = false;
                campoAjusteReceber.showOnView = false;
                campoAjusteReceber.showOnForm = false;
                delete(campoAjusteReceber.validations.required);
                campoAjusteReceber.readOnly = true;
                campoAjusteReceber.value('');
                widget.currentRow.CDAJUCTREC = '';
                break;

            case 'R':
                campoAjustePagar.readOnly = true;
                delete(campoAjustePagar.validations.required);
                campoAjustePagar.isVisible = false;
                 campoAjustePagar.showOnView = false;
                campoAjustePagar.showOnForm = false;
                campoAjustePagar.value('');
                widget.currentRow.CDAJUCTPAG = '';

                campoAjusteReceber.isVisible = true;
                campoAjusteReceber.showOnView = true;
                campoAjusteReceber.showOnForm = true;
                delete(campoAjusteReceber.validations.required);
                campoAjusteReceber.readOnly = true;
                campoAjusteReceber.value('');
                widget.currentRow.CDAJUCTREC = '';

                break;
        }
    };

    this.controlaCampoValorMensal = function(args){
        var widget = templateManager.container.getWidget('gridImposto');
        var campoControle = widget.getField('IDUTVRMENSBC');
        var campoValor = widget.getField('VRMINBASECAL');
        if(campoControle.value() == 'N'){
            campoValor.value(0);
            campoValor.readOnly = true;
        }
        else{
            campoValor.readOnly = false;
            campoValor.value(0);
        }
    };

    this.controlaCampoValorQtd = function(args){
        var widget = templateManager.container.getWidget('gridImposto');
        var campoControle = widget.getField('IDUTVLRQTDEIMPO');
        var campoParcela = widget.getField('IDUTPARCDED');
        var campo1Reducao = widget.getField('ID1UTREDUBC');
        var campoReducao = widget.getField('IDUTREDUBC');
        var campoConsideraVlrMensal = widget.getField('IDUTVRMENSBC');
        var campoConsideraVlrImposto = widget.getField('IDVRIMPSTRIB');
        var campoImpostoSitTrib = widget.getField('NMIMPOSTRIB');
        var campoValor = widget.getField('VRMINBASECAL');
        if(campoControle.value() == 'S'){
            campoValor.value(0);
            campoValor.readOnly = true;
            campoParcela.value('N');
            campoParcela.readOnly = true;
            campo1Reducao.value('N');
            campo1Reducao.readOnly = true;
            campoConsideraVlrMensal.value('N');
            campoConsideraVlrMensal.readOnly = true;
            campoConsideraVlrImposto.value('N');
            campoConsideraVlrImposto.readOnly = true;
            campoImpostoSitTrib.value('');
            campoImpostoSitTrib.readOnly = true;
            campoReducao.value('N');
            campoReducao.readOnly = true;
        }else{
            campoValor.readOnly = true;
            campoValor.value(0);
            campoConsideraVlrMensal.value('N');
            campo1Reducao.readOnly = false;
            campo1Reducao.value('N');
            campoReducao.readOnly = false;
            campoReducao.value('N');
            campoConsideraVlrMensal.readOnly = false;
            campoConsideraVlrImposto.readOnly = false;
            campoConsideraVlrImposto.value('N');
            campoImpostoSitTrib.readOnly = false;
            campoParcela.readOnly = false;
            campoParcela.value('N');
        }
    };

    this.controlaCampoVisuzalizacaoImposto = function(args){
        var widget = templateManager.container.getWidget('gridImposto');
        var campoVisulizacao = widget.getField('IDPRODNOTA');
        var campoTratamento = widget.getField('IDTRATIMPO');
        var campoControla = widget.getField('IDACRPRODENTRNF');
        if(campoTratamento.value() == 'A'){
            if(campoVisulizacao.value() == 'P'){
                campoControla.readOnly = false;
                campoControla.value('N');
            }else{
                campoControla.readOnly = true;
                campoControla.value('N');
            }
        }
    };

    this.controlaCampoApuracaoImposto = function(args){
        var widget = templateManager.container.getWidget('gridImposto');
        var campoApuracao = widget.getField('IDFORMAPUR');
        var campoConsidera = widget.getField('IDCONSVRIMPAG');
        campoConsidera.showOnForm = true;
        campoConsidera.showOnView = true;
        if(campoApuracao.value() == 'P'){
            campoConsidera.isVisible = true;
            //campoConsidera.value('T');
        }else{
            campoConsidera.isVisible = false;
            campoConsidera.value('');

        }
    };

    this.realizaControleCampos = function(args){
        //this.controlaCampoTratamento(args);
        //this.controlaCampoDecontar(args);
        this.controlaCampoValorQtd(args);
        this.controlaCampoValorMensal(args);
        this.controlaCampoVisuzalizacaoImposto(args);
        this.controlaCampoApuracaoImposto(args);
        this.controlaCampoTratamentoVisuzalizacao(args);
        this.verificaCampoTramento(args);
    };

    this.widgetOnEnter = function(args){
        args.owner.currentRow = args.owner.dataSource.data[0];
    };

    this.widgetOnEnterContabilizacao = function(args){
        var widgetImposto = templateManager.container.getWidget('gridImposto');
        var widget = args.owner;
        if(!args.owner.dataSource.data[0]){
            var currentRow = {
                __is_new:false,
                "CDIMPOSTO": widgetImposto.currentRow.CDIMPOSTO,
                CONTAATIVA: "", 
                CONTADESPESA: "",
                CONTAPASSIVA: "",
                CONTATRIBUTARIA: ""
            };
            widget.dataSource.data.push(currentRow);
            widget.currentRow = currentRow;
        }else{
            args.owner.currentRow = args.owner.dataSource.data[0];
        }
    };

    this.editar = function(widget){
        widget.template = widget.templateToForm;
        General.edit(widget);
    };

    this.editarContabilizacao = function(widget){
        widget.template = widget.templateToForm;
        General.edit(widget);
        widget.getParent().edit();
    };

    this.cancelarContabilizacao = function(widget){
        for (var i in widget.backupRow) {
            widget.currentRow[i] = widget.backupRow[i];
        }
        widget.template = widget.templateToView;
        widget.view();
        widget.getParent().view();
    };

    this.cancelar = function(widget){
        for (var i in widget.backupRow) {
            widget.currentRow[i] = widget.backupRow[i];
        }
        widget.template = widget.templateToView;
        widget.view();
    };

    this.editarWidgetPadrao = function(widget){
        var widgetImposto = widget.getParent().getParent();

        this.editar(widget);
        this.controlaVisibilidadeWidgetsFilhas('padrao', 'tabbedContabilizacao', false, false);

        widgetImposto.enabled = true;
        widgetImposto.edit();
        this.ciraCurrenRowAntiga();
    };

    this.editarImposto = function(args){
        var widget = args.owner.widget;
        var widgetPadrao = templateManager.container.getWidget('padrao');
        General.edit(widget);
        this.realizaControleCampos(args);
        this.controlaVisibilidadeWidgetsFilhas('gridImposto', 'gridImposto', false, true);
        this.ciraCurrenRowAntiga();


        this.editarWidgetPadrao(widgetPadrao);
        widgetPadrao.getParent().enabled = true;
    };

    this.controlaWidgetAdicionar = function(mainWidget, status){
        mainWidget.widgets.forEach(function(widget) {
            widget.enabled = status;
        });
        var widgetHabilitada = templateManager.container.getWidget('tabbedContabilizacao');
        widgetHabilitada.enabled = true;
        widgetHabilitada.widgets.forEach(function(widget) {
            if(widget.name == 'padrao'){
                widget.enabled = true;
            }else{
               widget.enabled = status;

            }
        });
    };

    this.controlaVisibilidadeWidgetsFilhas = function(nomeWidgetFilha, nomeWidgetPai, status,statusWidgetImposto){
        var widget = templateManager.container.getWidget('gridImposto');
        widget.enabled = statusWidgetImposto;
        widget.widgets.forEach(function(widg) {
            if(nomeWidgetPai !== widg.name){
                widg.enabled = status;
            }
            if(nomeWidgetPai == widg.name){
                widg.widgets.forEach(function(w) {
                    if(nomeWidgetFilha !== w.name)
                        w.enabled = status;
                });

            }
        });
    };

    this.cancelarWidgetPadrao = function(widget){
        var widgetImposto = templateManager.container.getWidget('gridImposto');
        this.cancelarImposto(widgetImposto,widget);
        widget.getParent().view();
    };

    this.cancelarImposto = function(widgetImposto, widgetPadrao){
        this.cancelar(widgetPadrao);
        this.cancelarWidget(widgetImposto);
    };

    this.adicionarImposto = function(args, widgetImposto,widgetPadrao){
        widgetImposto.newRow();
        widgetPadrao.template = widgetPadrao.templateToForm;
        widgetPadrao.setCurrentRow({});
        widgetPadrao.currentRow.__is_new = true;
        widgetPadrao.currentRow.IDTIPCONTAB = "";
        widgetPadrao.edit();

        this.realizaControleCampos(args);
        this.controlaWidgetAdicionar(widgetImposto, false);
        this.visibilidadeAjuste();
    };

    this.cancelarWidget = function(widget){
        if(widget.isNewRow()) {
            widget.dataSource.data.pop();
            widget.currentRow = {};
            widget.list(true);
            ScreenService.goBack();
        }
        else{
            widget.view();
            for (var i in widget.backupRow) {
                widget.currentRow[i] = widget.backupRow[i];
            }
        }
    };

    this.salvarImposto = function(args){
        var widgetImposto = templateManager.container.getWidget('gridImposto');
        var widgetPadrao = templateManager.container.getWidget('padrao');
        var widgetContabilizacao = templateManager.container.getWidget('tabbedContabilizacao');
        if(widgetImposto.currentRow.__is_new){
            //adicionando  novo imposto
            if(widgetImposto.isValid()){
                if(widgetPadrao.currentRow.IDTIPCONTAB){
                    this.criaCurrentRowImposto(widgetImposto,widgetPadrao);
                    if(!widgetImposto.currentRow.IDCONSVRIMPAG){
                        widgetImposto.currentRow.IDCONSVRIMPAG = 'T';
                    }
                    if(!widgetImposto.currentRow.IDATUCCTBITIMP){
                        widgetImposto.currentRow.IDATUCCTBITIMP = 'N';
                    }
                    if(!widgetImposto.currentRow.IDUTMUNIORIG){
                        widgetImposto.currentRow.IDUTMUNIORIG = 'N';
                    }
                    widgetImposto.currentRow.__columns_to_return = ['NRORG'];
                    Ger00400_Imposto_Service.salvar(widgetImposto.currentRow).then(function(callback) {
                        this.controlaWidgetAdicionar(widgetImposto,true);
                        widgetImposto.currentRow.__is_new = false;
                        widgetPadrao.currentRow.__is_new  = false;
                        this.cancelar(widgetPadrao);
                        widgetImposto.list(true);
                        ScreenService.goBack();
                        widgetImposto.reload();
                        
                    }.bind(this),
                    function(callback){
                        if(callback.data){
                            widgetImposto.currentWidget = widgetImposto;
                            widgetImposto.activate();
                            ScreenService.showMessage(callback.data.error);
                        }
                    });
                }
                else{
                    widgetContabilizacao.currentWidget = widgetContabilizacao;
                    widgetContabilizacao.activate();
                    widgetPadrao.activate();
                    if(widgetPadrao.isValid()){
                    }
                }
            }
            else{
                widgetImposto.currentWidget = widgetImposto;
                widgetImposto.activate();
            }
        }
        else{
            //editando Imposto
            if(widgetImposto.isValid()){
                if(!widgetImposto.currentRow.IDUTMUNIORIG){
                    widgetImposto.currentRow.IDUTMUNIORIG = 'N';
                }
                if(!widgetImposto.currentRow.IDCONSVRIMPAG){
                    widgetImposto.currentRow.IDCONSVRIMPAG = 'T';
                }
                if(!widgetImposto.currentRow.IDATUCCTBITIMP){
                    widgetImposto.currentRow.IDATUCCTBITIMP = 'N';
                }
                widgetImposto.currentRow.IDTIPCONTAB = widgetPadrao.currentRow.IDTIPCONTAB;
                widgetImposto.currentRow.CDCCTBLATIV = widgetPadrao.currentRow.CDCCTBLATIV;
                widgetImposto.currentRow.CDCCTBLDESP = widgetPadrao.currentRow.CDCCTBLDESP;
                widgetImposto.currentRow.CDCCTBLPASS = widgetPadrao.currentRow.CDCCTBLPASS;
                widgetImposto.currentRow.CDCCTBLDADM = widgetPadrao.currentRow.CDCCTBLDADM;
                Ger00400_Imposto_Service.salvar(widgetImposto.currentRow).then(function(callback) {
                    if(callback.dataset){
                        var data = widgetImposto.dataSource.data;
                        var impostosRelacionados = callback.dataset.imposto;
                        if(impostosRelacionados){
                            for(var i =0;i<data.length;i++){
                                for(var j =0;j<impostosRelacionados.length;j++){
                                    if( data[i].CDIMPOSTO == impostosRelacionados[j].CDIMPOSTO  && data[i].CDIMPOSTO !== widgetImposto.currentRow.CDIMPOSTO ){
                                        data[i].IDORGARRECAD = impostosRelacionados[j].IDORGARRECAD;
                                        data[i].IDIMPORECUP = impostosRelacionados[j].IDIMPORECUP;
                                        data[i].IDFORMAPUR = impostosRelacionados[j].IDFORMAPUR;
                                        data[i].IDCONSVRIMPAG = impostosRelacionados[j].IDCONSVRIMPAG;
                                    }
                                }
                            }
                        }

                    }
                    if(callback.messages){
                        let message = "ATUALIZA_PARAMETRIZACAO";
                        var DIMPOSTONFE = callback.messages[0].message.CDIMPOSTO;
                        var SGIMPOSTONFE = callback.messages[0].message.SGIMPOSTONFE;
                        MessageService.showMessageByCode(message, {"DIMPOSTONFE": DIMPOSTONFE, "SGIMPOSTONFE": SGIMPOSTONFE});
                    }
                    this.controlaWidgetAdicionar(widgetImposto,true);
                    widgetPadrao.view(true);
                    widgetPadrao.template = widgetPadrao.templateToView;
                    widgetImposto.view(true);

                }.bind(this),
                function(callback){
                    if(callback.data){
                        ScreenService.showMessage(callback.data.error);
                    }
                });
            }
        }
    };

    this.salvarContabilizacao = function(args){
        var widget = args.owner.widget;

        switch (widget.name){
            case'pessoaJuridica':
                widget.currentRow.IDTPIJURIMPO = 'J';
                break;
            case 'pessoaFisica':
                widget.currentRow.IDTPIJURIMPO = 'F';
                break;
            case 'inscricaoJuridica':
                widget.currentRow.IDTPIJURIMPO = 'I';
                break;
            case 'gridOutros':
                widget.currentRow.IDTPIJURIMPO = 'O';
                break;
            default :
                break;
        }
        Ger00400_Imposto_Service.salvarContabilizacao(widget.currentRow).then(function(callback) {
            this.controlaVisibilidadeWidgetsFilhas(widget.name,'tabbedContabilizacao',true,true);

            widget.template = widget.templateToView;
            widget.view();
            widget.activate();
            widget.getParent().view();

        }.bind(this),
        function(callback){
            if(callback.data){
                ScreenService.showMessage(callback.data.error);
            }
        });
    };

    this.salvarBaseCalculo = function(args){
        var widget = args.owner.widget;
        var widgetImposto = templateManager.container.getWidget('gridImposto');
        switch (widget.name){
            case'gridEncardo':
                widget.currentRow.IDTIPOBCALC = 'E';
                widget.currentRow.TIPOWIDGET = 'ENCARGO';
                break;
            case 'outrosImpostosPadrao':
                widget.currentRow.IDTIPOBCALC = 'I';
                widget.currentRow.TIPOWIDGET = 'PADRAO';
                break;
           default :
                break;
        }
        var paramsNewCode = widget.paramsNewCode;
        var keysParamsNewCode = Object.keys(paramsNewCode);
        var currentRow = widget.currentRow;
        keysParamsNewCode.forEach(function(value) {
            currentRow[value] = paramsNewCode[value];
        });
        widget.currentRow.CDIMPOSTO = widgetImposto.currentRow.CDIMPOSTO;
        currentRow.__new_code_pk = 'BASE DE CALCULO/IMPOSTO ' + widget.currentRow.CDIMPOSTO;
        if(widget.isValid()){
            Ger00400_Imposto_Service.salvarBaseCalculo(widget.currentRow).then(function(callback) {
                if(widget.currentRow.__is_new){
                    widget.currentRow.__is_new = false;
                    widget.list(true);
                    ScreenService.goBack();
                    if(callback.dataset.columnsToReturn){
                        widget.currentRow.NRORG = callback.dataset.columnsToReturn[0].NRORG;
                        widget.currentRow.NRSEQUBCALC = callback.dataset.columnsToReturn[0].NRSEQUBCALC;
                    }
                    widget.reload();
                }
                else{
                    widget.view();
                }
            }.bind(this),
            function(callback){
                if(callback.data){
                    ScreenService.showMessage(callback.data.error);
                }
            });
        }
    };

    this.salvarOutrosImpostosTipoOperacao = function(args){
        var widget = args.owner.widget;
        var widgetImposto = templateManager.container.getWidget('gridImposto');
        var tabbedOutrosImpostos = templateManager.container.getWidget('tabbedOutrosImpostosTipoOperacao');

        var paramsNewCode = widget.paramsNewCode;
        var keysParamsNewCode = Object.keys(paramsNewCode);
        var currentRow = widget.currentRow;
        keysParamsNewCode.forEach(function(value) {
            currentRow[value] = paramsNewCode[value];
        });
        widget.currentRow.CDIMPOSTO = widgetImposto.currentRow.CDIMPOSTO;
        widget.currentRow.IDENTRSAIDOP = tabbedOutrosImpostos.currentRow.IDENTRSAIDOP;
        widget.currentRow.CDTIPOOPER = tabbedOutrosImpostos.currentRow.CDTIPOOPER;


        currentRow.__new_code_pk = 'BASECALCTPOPER' + widget.currentRow.IDENTRSAIDOP + widget.currentRow.CDTIPOOPER + widget.currentRow.CDIMPOSTO;
        if(widget.isValid()){
            Ger00400_Imposto_Service.salvarOutrosImpostosTipoOperacao(widget.currentRow).then(function(callback) {
                if(widget.currentRow.__is_new){
                    widget.list(true);
                    ScreenService.goBack();
                    ScreenService.closeSwipe();
                    widget.currentRow.__is_new = false;
                    if(callback.dataset.columnsToReturn){
                        widget.currentRow.NRORG = callback.dataset.columnsToReturn[0].NRORG;
                        widget.currentRow.NRSEQBCALTPOP = callback.dataset.columnsToReturn[0].NRSEQBCALTPOP;
                    }
                    widget.reload();
                }
                else{
                    widget.view();
                }
            }.bind(this),
            function(callback){
                if(callback.data){
                    ScreenService.showMessage(callback.data.error);
                }
            });
        }
    };

    /*this.recarregaWidgetPadrao = function(args){
        var widgetImposto = templateManager.container.getWidget('gridImposto');
        var widget = templateManager.container.getWidget('padrao');
        if(widgetImposto.mode!=='edit'){
            widget.currentRow = widgetImposto.currentRow;
            widget.reload();
        }
    };
*/
    this.recarregaWidgetContabilizacao = function(nameWidget){
        var widgetImposto = templateManager.container.getWidget('gridImposto');
        var widget = templateManager.container.getWidget(nameWidget);
        widget.dataSourceFilter[0].value = widgetImposto.currentRow.CDIMPOSTO;
        if(widget.mode !== 'edit'){
            widget.reload([], false);
        }
    };

    this.criaCurrentRowImposto = function(widgetImposto,widgetPadrao){
        for(var nome in widgetPadrao.currentRow){
            widgetImposto.currentRow[nome] = widgetPadrao.currentRow[nome];
        }
    };

    this.recarregarField = function(nomeWidget, nomeField){
        templateManager.container.getWidget(nomeWidget).getField(nomeField).reload();
    };

    this.ciraCurrenRowAntiga = function(){
        var widgetImposto = templateManager.container.getWidget('gridImposto');
        widgetImposto.currentRow.vlrAntigo_IDORGARRECAD = widgetImposto.currentRow.IDORGARRECAD;
        widgetImposto.currentRow.vlrAntigo_IDIMPORECUP = widgetImposto.currentRow.IDIMPORECUP;
        widgetImposto.currentRow.vlrAntigo_IDFORMAPUR = widgetImposto.currentRow.IDFORMAPUR;
        widgetImposto.currentRow.vlrAntigo_IDUTRANGEMB = widgetImposto.currentRow.IDUTRANGEMB;
        if(!widgetImposto.currentRow.IDCONSVRIMPAG){
            widgetImposto.currentRow.vlrAntigo_IDCONSVRIMPAG = 'T';
        }else{
            widgetImposto.currentRow.vlrAntigo_IDCONSVRIMPAG = widgetImposto.currentRow.IDCONSVRIMPAG;
        }
    };

    this.controlaCampoTipoOperacao = function(args){
        var widgetOutrosImpostos = templateManager.container.getWidget('gridOutrosImpostosTipoOperacao');
        var currentRow           = args.owner.field.widget.currentRow;
        var campoTipoOperac      = args.owner.field.widget.getField('NMTIPOOPER');
        
        currentRow.NMTIPOOPER    = null;
        currentRow.CDTIPOOPER    = null;
        campoTipoOperac.readOnly = false;
        campoTipoOperac.reload();
        
        widgetOutrosImpostos.getAction('adicionarRow').isVisible = false;
    };

    this.recarregarWidgetOutrosImpostos = function(args){
        var widgetOutrosImpostos = templateManager.container.getWidget('gridOutrosImpostosTipoOperacao');
        var currentRow = args.owner.owner.widget.currentRow;
        
        widgetOutrosImpostos.getAction('adicionarRow').isVisible = currentRow.IDENTRSAIDOP && currentRow.CDTIPOOPER;
        widgetOutrosImpostos.reload();
    };

    this.visibilidadeAjuste = function(){
        var widget = templateManager.container.getWidget('gridImposto');
        var campoAjusteReceber = widget.getField('DSAJUCTREC');
        var campoAjustePagar = widget.getField('DSAJUCTPAG');
        campoAjusteReceber.isVisible = false;
        campoAjustePagar.isVisible = false;
    };

    this.verificaRange = function(args){
        var widgetImposto =  templateManager.container.getWidget('gridImposto');
        var campoRange = widgetImposto.getField('IDUTRANGEMB');
        if(widgetImposto.currentRow.vlrAntigo_IDUTRANGEMB == 'S' && widgetImposto.currentRow.IDUTRANGEMB == 'N'){
            Ger00400_Imposto_Service.verificaRange(widgetImposto.currentRow).then(function(callback) {
                var msg = ' ';
                var resul = callback.dataset.dadosfiscncm_regrafiscal;

                if(resul.length > 0){
                    for(var i = 0; i < resul.length; i++){
                        if(resul.length == '1'){
                            msg = msg + parseInt(callback.dataset.dadosfiscncm_regrafiscal[i].CDREGRAFISC);
                        }else{
                            if(i == resul.length-1){
                                msg = msg + parseInt(callback.dataset.dadosfiscncm_regrafiscal[i].CDREGRAFISC);
                            }else{
                                msg = msg + parseInt(callback.dataset.dadosfiscncm_regrafiscal[i].CDREGRAFISC) + ',';

                           }
                        }
                    }
                    campoRange.value('S');
                    let txt = "IMP_POSSUI_CAD_RANGE";
                    MessageService.errorNotificationByCode(txt, {"PARAMETRO": msg});
                }


            }.bind(this),
            function(callback){
                if(callback.data){
                    ScreenService.showMessage(callback.data.error);
                }
            });
        }
    };

};

Configuration(function(ContextRegister){
    ContextRegister.register('Ger00400_Imposto_Controller', Ger00400_Imposto_Controller);
});

// FILE: ./assets/js/controllers/Ger01400_Mensagem_Controller.js
var Ger01400_Mensagem_Controller = function ( restEngine, ScreenService, templateManager, General, ZhReports, Ger01400_Mensagem_Service, ZhFilterPreferences, MessageService) {
    var self = this;

    self.deleteMessage = function(widget) {
        Ger01400_Mensagem_Service.validateDeletion(widget.currentRow).then(function(shouldBlock) {
            if(shouldBlock){
                MessageService.showMessageByCode("EXCLUSAO_MENSAGEM");
            }
            else {
                MessageService.confirmMessageByCode("REGISTRO_EXCLUIR").then( function () {
                    widget.removeRow().then(function() {
                        ScreenService.closeSwipe();
                        widget.reload();
                    });
                });
            }
            
        }.bind(self));
    };

    self.changeWidgetAcessibility = function(widgets, enabled) {
        widgets.forEach(function(widget){
            widget.enabled = enabled;
        });

        templateManager.updateTemplate();
    };

    self.masterDetailCancelForm = function(widget) {
        // var widget = templateManager.container.getActiveWidget();

        if(self.formChangeTemplate(widget) && widget.getParent()) {
            var widgets = widget.getParent().widgets;

            self.changeWidgetAcessibility(widgets.concat(widget.getParent()), true);
        } else self.changeWidgetAcessibility(widget.widgets, true);

        if(widget.currentRow.__is_new) {
            var indexOfCurrentRow = widget.dataSource.data.indexOf(widget.currentRow);
            if(indexOfCurrentRow !== -1) {
                widget.dataSource.data.pop();
            }
            widget.list();

            ScreenService.closeSwipe();
        } else {
            Util.extend(widget.currentRow, widget.backupRow);
            widget.view();
        }
    };

    self.masterDetailEdit = function(widget) {
        // var widget = templateManager.container.getActiveWidget();
        this.toggleFocused(widget, true);      
        widget.backupRow = Util.clone(widget.currentRow);
        widget.edit();

    };

    self.masterDetailGoBackToList = function(widget) {
        ScreenService.closeSwipe();
        widget.list();
        widget.reload();
    };

    self.masterDetailNewRow = function(widget) {
        // var widget = templateManager.container.getActiveWidget();
        this.toggleFocused(widget, false);  
        widget.newRow();
        widget.reload();
    };

    self.masterDetailSave = function(widget) {
        // var widget = templateManager.container.getActiveWidget();

        if(widget.isValid()) {
            widget.currentRow.changed = true;

            if(widget.currentRow.__is_new) {
                widget.currentRow.NRORGINCLUSAO = (widget.currentRow.NRORGINCLUSAO)? widget.currentRow.NRORGINCLUSAO : getLocalVar('NRORG');
                widget.currentRow.CDOPERINCLUSAO = (widget.currentRow.CDOPERINCLUSAO)? widget.currentRow.CDOPERINCLUSAO : getLocalVar('CDOPERADOR');
            } else {
                widget.currentRow.NRORGULTATU = (widget.currentRow.NRORGULTATU)? widget.currentRow.NRORGULTATU : getLocalVar('NRORG');
                widget.currentRow.CDOPERULTATU = (widget.currentRow.CDOPERULTATU)? widget.currentRow.CDOPERULTATU : getLocalVar('CDOPERADOR');
            }

            return widget.saveRow().then(function(response){
                widget.currentRow.changed = false;

                if(widget.currentRow.__is_new) {
                    ScreenService.closeSwipe();
                    widget.list(true);
                    ScreenService.goBack();
                    widget.currentRow.__is_new = false;
                }else{
                     widget.view();
                }
                widget.activate();
                if(self.formChangeTemplate(widget) && widget.getParent()) {
                    var widgets = widget.getParent().widgets;
                    self.changeWidgetAcessibility(widgets.concat(widget.getParent()), true);
                }
                else {
                    self.changeWidgetAcessibility(widget.widgets, true);
                }

                //widget.reload();
            }.bind(self), function(error) {
                var errorMessage = 'Timeout';

                if (error.data) errorMessage = error.data.error;

                ScreenService.showMessage(errorMessage);
            });
            
        } else return false;
    };

    self.formChangeTemplate = function(widget) {
        // var widget = templateManager.container.getActiveWidget();
        if((widget.template != "widget/form.html")&&(widget.template != "widget/form-view.html")) return false;

        if(!widget.templateToForm) widget.templateToForm = "widget/form.html";
        if(!widget.templateToView) widget.templateToForm = "widget/form-view.html";

        if(widget.template == widget.templateToForm) widget.template = widget.templateToView;
        else if(widget.template == widget.templateToView) widget.template = widget.templateToForm;

        templateManager.updateTemplate();

        return true;
    };
    
    this.saveStatus = function (widget) {
        
        var allStatus = widget.dataSource.data;
        var statusModificados = allStatus.filter(status => status.changed_IDSITUACAOMENS);
        
        statusModificados.forEach(status => {
            status.changed = true;
        });
        
        widget.dataSource.save().then(function () {
            statusModificados.forEach ( status => {
                delete status.changed_IDSITUACAOMENS;
                delete status.changed;
            });
            widget.activate();
            widget.list(true);
            ScreenService.goBack();
            ScreenService.closeSwipe();
            ScreenService.successNotification("Parâmetros salvos com sucesso.");
        });
    };
    
    this.cancelStatus = function(widget) {
        widget.reload();
        widget.list();
    };
    
    this.openFilter = function(widget) {
        ZhFilterPreferences.openFilterWidget(widget, true).then(response => {
            var filter = templateManager.container.getActiveWidget();
            var clearFilter = filter.actions.filter(action => action.name === 'clear')[0];
            clearFilter.shortcutKey = 'ctrl+backspace';
            clearFilter.tooltip = 'Limpar Filtro (CTRL + Backspace)';
            filter.reload();
        
        });
    };
    
    //@toggleFocused recebe o status false quando não está no modo de edição e ao abrir o filtro
    this.toggleFocused = function(widget, status) {
        
        if(status == true) {
            widget.getField('DSMENSAGEM').focused = true;
            delete widget.getField('CDMENSAGEM').focused;
        } else {
            widget.getField('CDMENSAGEM').focused = true;
            delete widget.getField('DSMENSAGEM').focused;
        }
        
    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Ger01400_Mensagem_Controller', Ger01400_Mensagem_Controller);
});

// FILE: ./assets/js/controllers/Ger03400_ReplicacaoDocumentos_Controller.js
function Ger03400_ReplicacaoDocumentos_Controller(General, ScreenService, templateManager, Ger03400_ReplicacaoDocumentos_Service) {

    var mensagens = {
        unidadeIgual: 'Unidade de Destino e Origem não podem ser iguais',
        empresaNaoRelacionada: 'Empresas das filiais de destino e origem não estão relacionadas',
        codigoExistente: 'Código já existente',
        filtro: 'Filtre antes de adicionar.'

    };

    this.adicionarFolha = function(widget) {
        if(!widget.dataSourceFilter[0]) {
            ScreenService.showMessage(mensagens.filtro);
        }
        else widget.newRow();
    };

    // Edita uma Operação
    this.editOperacao = function(widget) {
        widget.getField('NMTIPOOPER').readOnly = false;
        General.edit(widget);
    };

    // Limpa o campo Tipo de operação
    this.clearOper = function(widget) {
        var newWidget = templateManager.container.getWidget('FOLHADADOS');
        
        newWidget.getField('NMTIPOOPER').readOnly = false;
        newWidget.getField('NMTIPOOPER').clearValue();
        newWidget.getField('NMTIPOOPER').reload();
    };

    // Salva replicação de documentos
    this.saveNewReplicDoc = function(widget) {
        if (!widget.isValid()){
            widget.activate();
        }
        else {
            widget.getField('CDFILIORIGRD').label = 'Unidade';
            widget.getField('CDFILIDESTRD').label = 'Unidade';
            var folhaWidget = templateManager.container.getWidget('FOLHADADOS');
            widget.currentRow.CDFILIAL = folhaWidget.currentRow.CDFILIAL;
            widget.currentRow.NRREPLICDOC = folhaWidget.currentRow.NRREPLICDOC;

            if (!widget.currentRow.CDALMOXARIFE) widget.currentRow.CDALMOXARIFE = ' ';
            if (!widget.currentRow.CDLOCALESTOQ) widget.currentRow.CDLOCALESTOQ = ' ';

            if(widget.currentRow.IDENTRSAIDOP === 'E') {
                widget.currentRow.CDALMODESTRD = widget.currentRow.CDALMOXARIFE;
                widget.currentRow.CDLESTDESTRD = widget.currentRow.CDLOCALESTOQ;

                widget.currentRow.CDALMOORIGRD = " ";
                widget.currentRow.CDLESTORIGRD = " ";
            }
            if(widget.currentRow.IDENTRSAIDOP === 'S') {
                widget.currentRow.CDALMOORIGRD = widget.currentRow.CDALMOXARIFE;
                widget.currentRow.CDLESTORIGRD = widget.currentRow.CDLOCALESTOQ;


                widget.currentRow.CDALMODESTRD = " ";
                widget.currentRow.CDLESTDESTRD = " ";
            }


           General.saveCrud(widget).then(function(response){
                widget.getField('CDSERIE').readOnly = true;
                widget.getField('NMTIPOOPER').readOnly = true;
                widget.getField('DSALMOXARIFE').readOnly = true;
                widget.getField('DSLOCALESTOQ').readOnly = true;

            });


        }

    };


    // Cancela uma nova Replicação
    this.cancelNewReplicDoc = function(widget) {
        widget.getField('CDSERIE').readOnly = true;
        widget.getField('NMTIPOOPER').readOnly = true;
        widget.getField('DSALMOXARIFE').readOnly = true;
    };

    this.fieldOnChangeNrReplicDoc = function(widget) {
        var filtro = widget.appliedFilter;
        var cdfilial = filtro.FILTER_ROW.CDFILIAL;

        Ger03400_ReplicacaoDocumentos_Service.verificaNRREPLICDOC(cdfilial,widget.currentRow.NRREPLICDOC).then(function(response) {

              if(response.dataset.replicdoc.length !== 0) {

                    ScreenService.showMessage(mensagens.codigoExistente);
                    widget.getField('NRREPLICDOC').clearValue();
                    widget.reload();

                }

        });


    };

    // Salva uma nova Operação
    this.saveNewOperacao = function(widget) {
        var filtro = widget.appliedFilter;
        widget.currentRow.CDFILIAL = filtro.FILTER_ROW.CDFILIAL;
        widget.currentRow.NMFILIAL = filtro.FILTER_ROW.NMFILIAL;
        if (!widget.isValid()){
            // Valida widget
            widget.activate();
        }
        else {
            widget.getField('IDENTRSAIDOP').label = 'E/S';
            widget.getField('NMTIPOOPER').readOnly = true;
            General.saveCrud(widget);
        }

    };

    this.editReplicDoc = function(widget) {
        var serie = widget.getField('CDSERIE');
        
        serie.readOnly = false;
        this.readOnlyAlmoxDocumento(widget);
        this.changeFieldSerie(serie, widget.currentRow.IDENTRSAIDOP);

        var cdfilial = (widget.currentRow.IDENTRSAIDOP == 'S') ? 'CDFILIORIGRD' : 'CDFILIDESTRD';

        if(widget.currentRow.IDENTRSAIDOP === 'S' ) {
            this.setFilterSerieSaida(widget.fields[5], widget.currentRow[cdfilial]);
        }

        widget.backupRow = Util.clone(widget.currentRow);
        General.edit(widget);
    };

    this.relacionaFilialDestinoAOrigem = function(){
        var documentosgerados = templateManager.container.getWidget('DOCUMENTOSGERADOS');
        
        if (documentosgerados.getField('NMFILIDEST').value()){
            var filter = [
                {'name': 'CDEMPRESA', 'operator': '=', 'value': documentosgerados.currentRow.CDEMPRESA},
                {'name': 'CDFILIAL', 'operator': '=', 'value': documentosgerados.currentRow.CDFILIDESTRD}
            ];
            General.callRestEngine('df#/crud/filial_relacionaDestinoAOrigem', 'FilterData', filter)
            .then(function(filial){
                if (filial.dataset.filial_relacionaDestinoAOrigem.length == 0){
                    documentosgerados.getField('NMFILIDEST').setValue('');
                    ScreenService.showMessage('Empresa da filial de origem deve estar relacionada à Filial de Destino');
                }
            })
            .catch(function(erro){
                ScreenService.showMessage(erro.data.error);
            });
        }
    };

    this.onChangeFilialDocGerado = function(field) {
        var widget  = field.widget;
        var cdfilial = (field.name == 'NMFILIORIG') ? 'CDFILIORIGRD' : 'CDFILIDESTRD';
        var cdfilial2 = (field.name == 'NMFILIORIG') ? 'CDFILIDESTRD' : 'CDFILIORIGRD';
        var filialSerie = (widget.currentRow.IDENTRSAIDOP == 'S') ? 'CDFILIORIGRD' : 'CDFILIDESTRD';
        
        if(widget.currentRow[cdfilial] && widget.currentRow[cdfilial2]) {
            if(widget.currentRow[cdfilial] === widget.currentRow[cdfilial2]) {
                widget.currentRow[cdfilial] = '';
                widget.currentRow[field.name] = '';
                ScreenService.showMessage('Filial de destino e origem não podem ser iguais');
            }
            else {
                if(widget.currentRow.IDENTRSAIDOP === 'S') this.setFilterSerieSaida(widget.fields[5], widget.currentRow[cdfilial]);
            }
        }
        else if(widget.currentRow.IDENTRSAIDOP === 'S' && cdfilial === 'CDFILIORIGRD' && widget.currentRow[cdfilial]) {
            this.setFilterSerieSaida(widget.fields[5], widget.currentRow[cdfilial]);
        }
        if(widget.currentRow.IDENTRSAIDOP && widget.currentRow[filialSerie]) widget.getField('CDSERIE').readOnly = false;
        else widget.getField('CDSERIE').readOnly = true;
        this.readOnlyAlmoxDocumento(widget);
    };

    /**
     * Altera o readOnly e filtro dos campos ligados
     * ao tipo de operação(DSALMOXARIFE, DSLOCALESTOQ, NMTIPOOPER, CDSERIE).
     *
     * @param {object} field field de IDENTRSAIDOP (destino ou origem)
     * da widget documentos gerados.
    */
    this.onChangeIdEntrSaidaOp = function(field) {
        var widget = field.widget;
        var cdfilial = (field.value() == 'S') ? 'CDFILIORIGRD' : 'CDFILIDESTRD';
        var operacao = widget.getField('NMTIPOOPER');
        var almoxarifado = widget.getField('DSALMOXARIFE');
        var localizacao  = widget.getField('DSLOCALESTOQ');
        
        if(!field.value()) {
            operacao.readOnly = true;
            almoxarifado.readOnly = true;
            localizacao.readOnly = true;
        }
        else {
            this.changeFieldSerie(widget.getField('CDSERIE'), field.value());
            if(widget.currentRow[cdfilial]) {
                widget.fields[5].readOnly = false;
                widget.currentRow.CDSERIE = '';
                if(field.value() === 'S') this.setFilterSerieSaida(widget.fields[5], widget.currentRow[cdfilial]);
                if(widget.currentRow.IDENTRSAIDOP && widget.currentRow[cdfilial]) widget.getField('CDSERIE').readOnly = false;
                else widget.getField('CDSERIE').readOnly = true;
                almoxarifado.dataSourceFilter[0].value = "@General.getFieldValue('"+cdfilial+"','DOCUMENTOSGERADOS')@";
                almoxarifado.outData.CDALMOXARIFE = (field.value() == 'S') ? ["CDALMOXARIFE", "CDALMOORIGRD"] : ["CDALMOXARIFE", "CDALMODESTRD"];
                localizacao.dataSourceFilter[0].value = "@General.getFieldValue('"+cdfilial+"','DOCUMENTOSGERADOS')@";
            }
            this.readOnlyAlmoxDocumento(widget);
            operacao.readOnly = false;
        }
        operacao.reload();
        almoxarifado.setValue('');
        widget.currentRow.CDALMOXARIFE = '';
        operacao.setValue('');
        widget.currentRow.CDTIPOOPER   = '';
        localizacao.setValue('');
        widget.currentRow.CDLOCALESTOQ = '';
    };

    /**
     * Altera o readOnly e do campo de almoxarifado('DSALMOXARIFE')
     * de acordo com a filial e o tipo de operação('E' ou 'S').
     *
     * @param {object} widget  widget documentos gerados.
    */

    this.readOnlyAlmoxDocumento = function(widget) {
        var almoxarifado = widget.getField('DSALMOXARIFE');
        
        if(widget.currentRow.IDENTRSAIDOP) {
            var cdfilial = (widget.currentRow.IDENTRSAIDOP == 'S') ? 'CDFILIORIGRD' : 'CDFILIDESTRD';
            almoxarifado.dataSourceFilter[0].value = "@General.getFieldValue('"+cdfilial+"','DOCUMENTOSGERADOS')@";
            if(widget.currentRow[cdfilial]) {
                var filter = [
                    {
                        "name":"CDFILIAL",
                        "operator":"=",
                        "value": widget.currentRow[cdfilial]
                    }
                ];
                General.callRestEngine('df#/crud/paramfilial', 'FilterData', filter).then(function(success){
                    if(success.dataset.paramfilial.length > 0) {
                        if(success.dataset.paramfilial[0].IDUTILALMOX === 'S') {
                            almoxarifado.readOnly = false;
                            almoxarifado.reload();
                            this.readOnlyLocalDocumento(widget);
                        }
                    }
                    else almoxarifado.readOnly = true;
                }.bind(this));
            }
            else almoxarifado.readOnly = true;
        }
        else almoxarifado.readOnly = true;
    };

    /**
     * Altera o readOnly e do campo de localização('DSLOCALESTOQ')
     * de acordo com a filial e o tipo de operação('E' ou 'S').
     *
     * @param {object} widget  widget documentos gerados.
    */
    this.readOnlyLocalDocumento = function(widget) {
        var local = widget.getField('DSLOCALESTOQ');
        
        if(widget.currentRow.IDENTRSAIDOP) {
            var cdfilial = (widget.currentRow.IDENTRSAIDOP == 'S') ? 'CDFILIORIGRD' : 'CDFILIDESTRD';
            if(widget.currentRow[cdfilial] && widget.currentRow.CDALMOXARIFE) {
                local.dataSourceFilter[0].value = "@General.getFieldValue('"+cdfilial+"','DOCUMENTOSGERADOS')@";
                var filter = [
                    {
                        "name":"CDFILIAL",
                        "operator":"=",
                        "value": widget.currentRow[cdfilial]
                    }
                ];
                General.callRestEngine('df#/crud/paramfilial', 'FilterData', filter).then(function(success){
                    if(success.dataset.paramfilial[0].IDUTILLCESTQ === 'S') {
                        local.readOnly = false;
                        local.reload();
                    }
                });
            }
            else local.readOnly = true;
        }
        else local.readOnly = true;
    };

    /**
     * Altera o filtro do field de serie com os valores
     * referentes a filial. - Somente para IDENTRSAIDOP = 'S' -
     *
     * @param {object} fieldSerie field 'CDSERIE' da
     * widget documentos gerados.
     * @param {string} filial código da filial
    */
    this.setFilterSerieSaida = function(fieldSerie, filial) {
        var filter = [
            {
                "name":"CDFILIAL",
                "operator":"=",
                "value": filial
            }
        ];
        
        General.callRestEngine('df#/crud/filial', 'FilterData', filter).then(function(success){
            var rowFilial = success.dataset.filial;
            fieldSerie.dataSourceFilter[0].value =  rowFilial[0].CDEMPRESA;
            fieldSerie.dataSourceFilter[1].value =  rowFilial[0].CDINSCESTA;
            fieldSerie.dataSourceFilter[2].value =  rowFilial[0].CDINSCMUNI;
            fieldSerie.reload();
        });
    };

    /**
     * Altera o filed de 'CDSERIE' de acordo com
     * IDENTRSAIDOP.
     *
     * @param {object} fieldSerie field 'CDSERIE' da
     * widget documentos gerados.
     * @param {string} IDENTRSAIDOP 'E' para entrada
     * 'S' para saída
    */
    this.changeFieldSerie = function(field, IDENTRSAIDOP) {
        if(IDENTRSAIDOP === 'E') {
            field.valueFields = ["CDSERIEENTR"];
            field.descriptionFields = ["CDSERIEENTR"];
            field.displayedFields = ["CDSERIEENTR", "DSSERIEENTR"];
            field.searchableFields = ["CDSERIEENTR", "DSSERIEENTR"];
            field.outData = {"CDSERIEENTR": "CDSERIE", "DSSERIEENTR": "DSSERIE"};
            field.dataSource.name = "df#/crud/serieentr";
            field.order = "CDSERIEENTR";
            field.dataSourceFilter = [];
            field.fields[0].isVisible = true;
            field.fields[1].isVisible = true;
            field.fields[2].isVisible = false;
            field.fields[3].isVisible = false;
            if(field.selectWidget)  {
                field.selectWidget.fields[0].isVisible = true;
                field.selectWidget.fields[1].isVisible = true;
                field.selectWidget.fields[2].isVisible = false;
                field.selectWidget.fields[3].isVisible = false;
                field.selectWidget.reload();
                field.selectWidget.redraw();
            }
        }
        else if(IDENTRSAIDOP === 'S') {
            field.valueFields = ["CDSERIE"];
            field.descriptionFields = ["CDSERIE"];
            field.displayedFields = ["CDSERIE", "DSSERIE"];
            field.searchableFields = ["CDSERIE", "DSSERIE"];
            field.outData = {"CDSERIE": "CDSERIE"};
            field.dataSource.name = "df#/crud/serienf";
            field.order = "CDSERIE";
            field.fields[0].isVisible = false;
            field.fields[1].isVisible = false;
            field.fields[2].isVisible = true;
            field.fields[3].isVisible = true;
            if(field.selectWidget)  {
                field.selectWidget.outData = {"CDSERIE": "CDSERIE", "DSSERIE": "DSSERIE"};
                field.selectWidget.fields[0].isVisible = false;
                field.selectWidget.fields[1].isVisible = false;
                field.selectWidget.fields[2].isVisible = true;
                field.selectWidget.fields[3].isVisible = true;
                field.selectWidget.reload();
                field.selectWidget.redraw();
            }
            field.dataSourceFilter = [
                {
                    "name": "CDEMPRESA",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "CDINSCESTA",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "CDINSCMUNI",
                    "operator": "=",
                    "value": ""
                }
            ];
        }
        field.reload();
    };
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger03400_ReplicacaoDocumentos_Controller', Ger03400_ReplicacaoDocumentos_Controller);
});

// FILE: ./assets/js/controllers/Ger03600_Rateio_Produto_Controller.js
function Ger03600_Rateio_Produto_Controller(metaDataFactory, MessageService, templateManager, General, Ger03600_Rateio_Produto_Repository, Df11000_Lancamento_Service, Df11000_Lancamento_Repositories, eventEngine, ScreenService) {
	
	var self = this;
	var excluirDataset = [];
	
	this.alteraVisibilidadeRateio = function(nota) {
        var notaRow = nota.row;
        if((notaRow.IDBLOQUEADO !== 'S' || notaRow.IDDOCFINANC === 'L') && notaRow.IDSTATUS !== 'C' && notaRow.IDSTATUS !== 'D' && notaRow.IDSTATUS !== 'L'){
            nota.owner.getAction('actRateioProduto').isVisible = true;
        }else{
            nota.owner.getAction('actRateioProduto').isVisible = false;
        }
    };
    
    this.preLoadAutocomplete = function(widget){
      if(widget.dataSource.data.length === 0)
        widget.reload();
    };
	
	this.onChangeUnidade = function(widget) {
	    
	    if(widget.currentRow.CDFILICCUST) {
	        widget.getField('NMCENTCUST').reload();
	        widget.getField('NMCENTCUST').readOnly = false;
	    }
	    else {
	        widget.getField('NMCENTCUST').readOnly = true;
	    }
	    
        widget.currentRow.NMCENTCUST = '';
        widget.currentRow.CDCENTCUST = '';
	};
	
	this.abreRateioProduto = function(nota, widget = null) {
        var widgetNota = widget ? widget : nota.owner.widget;
        var widgetRateioProduto = templateManager.container.getWidget('GER03600RateioPorProduto');
        
        widgetRateioProduto.isVisible = true;
        widgetRateioProduto.enabled = true;
        widgetRateioProduto.reload();
        widgetNota.openTab('GER03600RateioPorProduto');
    };
    
    this.geraRateioOnEnter = function(widget){
        if(widget.geraRateio){
            widget.geraRateio = false;
            Df11000_Lancamento_Service.geraRateio('N', 'P', 'S')
            .then( 
                () => {
                    widget.geraRateio = true;
                    widget.reload();
                } 
            );
        }
    };
    
    this.controlaVisibilidadeRateio = function(args){
        var caller = args.owner;
        var widgetRateio = templateManager.container.getWidget('GER03600RateioPorProduto');
        var widgetItensRateio = templateManager.container.getWidget('gridItensRateio');
        var widgetCabecalhoNota = templateManager.container.getWidget('cabecalho');
        
        
        switch(caller.name){
            case 'GER03600RateioPorProduto' :
                widgetItensRateio.getAction('actNovoRateio').isVisible = true;
                widgetItensRateio.getAction('actExcluirMultiRateio').isVisible = true;
                //widgetItensRateio.getAction('actExcluirTodosRateios').isVisible = true;
                widgetItensRateio.getAction('actSalvarRateio').isVisible = true;
                widgetItensRateio.getAction('actCancelarRateio').isVisible = true;
                widgetItensRateio.getAction('actEditarRateio').isVisible = true;
                break;
            case 'actVoltarRateioList' :
            case 'actVoltarRateioView' :
                //widget.getParent().getAction('actNovoRateio').isVisible= false;
                break;
            case 'actRateioProduto' :
                widgetCabecalhoNota.enabled = false;
                General.changeWidgetAcessibility(widgetCabecalhoNota.widgets, false);
                break;
            case 'actFecharRateio' :
                widgetItensRateio.getAction('actFecharRateio').isVisible = false;
                widgetItensRateio.isVisible = false;
                widgetCabecalhoNota.enabled = true;
                General.changeWidgetAcessibility(widgetCabecalhoNota.widgets, true);
                break;
            case 'cabecalho' :
                General.changeWidgetAcessibility(widgetCabecalhoNota.widgets, true);
                widgetRateio.isVisible = false;
                widgetCabecalhoNota.enabled = true;
                this.alteraVisibilidadeRateio(args);
                break;
            case 'actSalvarCabecalho':
                widgetCabecalhoNota.getAction('actRateioProduto').isVisible = false;
        }
    };
    
    this.actNovoRateioOnClick = function(lancamentoRateioArgs) {
        lancamentoRateioArgs.owner.widget.newRow();
        var lancamentoRateioWidget = lancamentoRateioArgs.owner.widget;
        var currRow = lancamentoRateioWidget.currentRow;
        var prodRateio = lancamentoRateioWidget.getParent().currentRow;
        
        currRow.CDFILIAL = prodRateio.CDFILIAL;
        currRow.NRLANCTONF = prodRateio.NRLANCTONF;
        currRow.NRSEQITPROD = prodRateio.NRSEQITPROD;

        
    };
    
    this.actExcluirRateioOnClick = function(excluirRateioArgs) {
        var lancamentoRateioWidget = excluirRateioArgs.owner.widget;
        ScreenService.confirmMessage(
            ScreenService.i18n('Excluir Registro?'),
            'question',
            () => {
                const {owner: {widget}, row} = excluirRateioArgs;
                
                lancamentoRateioWidget.changed = true;
                excluirDataset.push(row);
                    Df11000_Lancamento_Service.excluiRateioDataSource(row)
                    .then(Df11000_Lancamento_Service.atualizaLancamento)
                    .then(() => {
                        Df11000_Lancamento_Service.setDataSourceLancamentoNota();
                        General.backOrCancel(excluirRateioArgs);
                        widget.reload();
                    });
            });
    };
    
    this.actExcluirMultiRateio = function(lancamentoRateioArgs){
        //var lancamentoRateioWidget = lancamentoRateioArgs.owner.getParent();
        //var lancamentoRateioWidget = lancamentoRateioArgs.owner.widget.container.getWidget('GER03600RateioPorProduto');
        //var widgetItensRateio = lancamentoRateioArgs.owner.widget.container.getWidget('gridItensRateio');
        var lancamentoRateioWidget = lancamentoRateioArgs.owner.widget.container.getWidget('GER03600RateioPorProduto');
        var widgetItensRateio = lancamentoRateioArgs.owner.widget.container.getWidget('gridItensRateio');
        var rateiosSelecionados;
        
        
        Df11000_Lancamento_Service.getDataSourceRateio().then((localStorageRateio) => {
            if(lancamentoRateioArgs.owner.name === 'actExcluirMultiRateio'){
                rateiosSelecionados = lancamentoRateioArgs.owner.widget.getCheckedRows();
            }else{
                rateiosSelecionados = localStorageRateio;
            }
            
            if(widgetItensRateio._hasCheckedRows() || lancamentoRateioArgs.owner.name === 'actExcluirTodosRateios'){
                ScreenService.confirmMessage(
                    ScreenService.i18n('Excluir Registros?'),
                    'question',
                    () => {
                    
                        widgetItensRateio.changed = true;
                        widgetItensRateio.getAction('actFecharRateio').isVisible = true;
                        rateiosSelecionados.forEach(row =>{
                            excluirDataset.push(row);
                        });
                        widgetItensRateio.unSelectAll();
                        Df11000_Lancamento_Service.excluiMultiRateioDataSource(rateiosSelecionados)
                        .then(Df11000_Lancamento_Service.atualizaLancamento)
                        .then(() =>{
                            this.controlaVisibilidadeRateio(lancamentoRateioArgs);
                            //widgetItensRateio.getAction('actExcluirTodosRateios');
                            widgetItensRateio.reload();
                        });
                    });
            }
        });
        
        ScreenService.closePopup();
    };
    
    this.actEditarRateioOnClick = function(lancamentoRateioArgs) {
        var lancamentoRateioWidget = lancamentoRateioArgs.owner.widget;
        var lancamentoRateioRow = lancamentoRateioArgs.row;
        if(lancamentoRateioRow.hasOwnProperty('__rightActions')){
            delete lancamentoRateioRow.__rightActions;
            delete lancamentoRateioRow.__leftActions;
        }
        General.edit(lancamentoRateioWidget);
        controlaStatusWidgets(lancamentoRateioWidget, lancamentoRateioWidget.name, false);
    };
    
    this.actSalvarRateioOnClick = function(lancamentoRateioArgs) {
        var lancamentoRateioWidget = lancamentoRateioArgs.owner.widget;

        if (!lancamentoRateioWidget.isValid())
            return lancamentoRateioArgs.e.preventDefault();

  
  
        lancamentoRateioWidget.changed = true;
        Df11000_Lancamento_Service.setDataSourceRateio(lancamentoRateioWidget.dataSource.data) // este set está errado?
            .then(Df11000_Lancamento_Service.atualizaLancamento)
            .then(() => {
                lancamentoRateioWidget.view();
                controlaStatusWidgets(lancamentoRateioWidget, lancamentoRateioWidget.name, true);
                lancamentoRateioWidget.getAction('actFecharRateio').isVisible = true;
                ScreenService.goBack();
            });
        
    };
    
    this.actFecharRateio = function(rateioArgs) {
        
        var lancamentoRateioWidget = templateManager.container.getWidget('gridItensRateio');
        var rateioDataset = lancamentoRateioWidget.dataSource.data;
        var widgetItensRateio = templateManager.container.getWidget('GER03600RateioPorProduto');
        var cabecalhoWidget = templateManager.container.getWidget('cabecalho');
        
        Df11000_Lancamento_Service.salvaRateio(excluirDataset).then((success) => {

            rateioDataset.forEach(row => {
                if(row.__is_new) {
                    row.__is_new = false;
                    row.__createdLocal = false;
                }
            });

            Df11000_Lancamento_Service.setDataSourceRateio(rateioDataset) 
                .then(Df11000_Lancamento_Service.atualizaLancamento()
                .then(() => {
                    lancamentoRateioWidget.unSelectAll();
                    lancamentoRateioWidget.list();
                    controlaStatusWidgets(lancamentoRateioWidget, lancamentoRateioWidget.name, true);
                    lancamentoRateioWidget.changed = false;
                    excluirDataset = [];
                    this.controlaVisibilidadeRateio(rateioArgs);
                    ScreenService.goBack();
                    ScreenService.closePopup();
                    cabecalhoWidget.openTab('cabecalho');
                }));
        }, error => {
            if (Array.isArray(error)) {
                var stringCamposObrig = error.join(', ');
                var mensagem = "CAMPOS_MAIORES_ZERO";
                return MessageService.showMessageByCode(mensagem, {"PARAMETRO": stringCamposObrig});
            } else if (typeof error === 'string')
                return ScreenService.showMessage(error);
            return ScreenService.showMessage(error.data.error);
        });
    };
    
    
    this.filtraItemRateio = function(args){
        var widgetItensRateio = args.owner;
        var widgetRateio = widgetItensRateio.container.getWidget('gridItensRateio');
        var widgetRateioTabbed = widgetItensRateio.container.getWidget('tabbedRateio');
        
        widgetRateio.dataSourceFilter= [
            {
                "name": "NRSEQITPROD",
                "operator": "=",
                "value": args.row.NRSEQITPROD
            }
        ];
        widgetRateioTabbed.currentRow = widgetItensRateio.currentRow;
        widgetRateio.isVisible = true;
        ScreenService.openSwipe(widgetRateioTabbed);
            
    };
    
    this.gridItensRateioActivate = function(widget){
            var widgetRateio = widget.container.getWidget('gridItensRateio');
            widget.openTab('gridItensRateio');
            widgetRateio.activate();
            widgetRateio.reload();
    };
    
    this.actImportarRateio = function(args){
        var widgetRateio = args.owner.widget.container.getWidget('gridItensRateio');
        var popupWidget = this.popUpRateio(widgetRateio);
        
        ScreenService.openPopup(popupWidget);

    };
	
    this.importarRateio = function(importacaoArgs){
        var widget = importacaoArgs.owner.widget;
        if(widget.isValid()){
            var file = widget.currentRow.FILEXLS;
            var tipoImportacao = widget.currentRow.IMPORTTYPE;
            var itensRateio = widget.getParent().getParent().dataSource.data;
            var pos = file[0].b64File.indexOf(',') + 1;
            file = {
                'name': file[0].name,
                'content': file[0].b64File.substr(pos)
            };
            var row = {
                'ITENSRATEIO': itensRateio,
                'TIPORATEIO': tipoImportacao,
                'FILE': file
            };
            
            Ger03600_Rateio_Produto_Repository.importRateio(row).then(function(success) {
                var rateiosExcluir = [];
                if(success.dataset.importRateio){
                    var rateioImportado = success.dataset.importRateio;
                    rateioImportado.forEach(row =>{
                        row.__is_new = true;
                    });
                    Df11000_Lancamento_Service.getDataSourceRateio().then(function(ratAtual) {
                        ratAtual.forEach(row =>{
                            rateiosExcluir.push(row);
                        });
                        Df11000_Lancamento_Repositories.excluiRateio(rateiosExcluir)
                        .then(Df11000_Lancamento_Service.setDataSourceRateio(rateioImportado))
                        .then(Df11000_Lancamento_Service.atualizaLancamento())
                        .then(function(){
                            widget.getParent().reload();
                            widget.getParent().getAction('actFecharRateio').isVisible = true;
                            ScreenService.closePopup();
                            ScreenService.closePopup();
                        }.bind(widget));
                        
                    }.bind(widget));
                }
            }.bind(widget),
            function(error){
                if(error.data.error){
                    ScreenService.showMessage('Arquivo ou Formato Inválido \n'+error.data.error);
                }
            });
                    
            }
    };
    
    this.nivelCompletoProduto = function() {
        return General.getLevelByMask('GER_CDPICTPROD','1').length;
    };
    
    var removeDataSourcesConsultaLancEntradaActionEvent = function() {
        Df11000_Lancamento_Service.removeDataSourceLancamentoNota();
    };

    //Evento chamado ao sair da tela de lançamento de nota - deprecated on list
    this.voltarListaLancamentosNotas = function(args) {
        removeDataSourcesConsultaLancEntradaActionEvent();
        General.backOrCancel(args);
    };
	
	this.popUpRateio = function(parentWidget){
        var widget = parentWidget.importaRateio;
        if(!widget ){
            widget = getPopUpRateio();
            widget.container = parentWidget.container;
            widget = metaDataFactory.widgetFactory(widget,parentWidget);
            parentWidget.importaRateio = widget;
            widget.parent = parentWidget;
            return widget;
        }else{
            return widget;
        }
    };
    
    var controlaStatusWidgets = function(widgetPai, nomeWidgetAtual, status, ignoredWidgets) {
        var allWidgets = widgetPai.widgets;

        allWidgets.filter(widget => widget.name !== nomeWidgetAtual).forEach(widget => widget.enabled = status);

        if (widgetPai.name !== nomeWidgetAtual)
            widgetPai.enabled = status;

        if (Array.isArray(ignoredWidgets))
            allWidgets.filter(widget => ignoredWidgets.indexOf(widget.name) !== -1).forEach(widget => widget.enabled = !status);
    };
    

	var getPopUpRateio = function() {
        return {
			"id": "000",
            "name": "importacaoRateio",
            "label": "Importação de Rateio",
            "isVisible": true,
            "template": "widget/form.html",
            "dataSourceFilter": [],
			"currentRow": {},
            "showAsTab": false,
            "dataSource": {},
            "fields": [
                {
					"id": 1,
                    "name": "IMPORTTYPE",
                    "label": "Tipo de importação",
                    "isVisible": true,
                    "class": 12,
                    "template": "field/select-native.html",
                    "valueField": "IMPORTTYPE",
                    "descriptionField": "IMPORTNAME",
                    "hideEmptyValue": true,
                    "dataSource": {
                        "data": [
                            {
                                "IMPORTTYPE": "P",
                                "IMPORTNAME": "Por Percentual"
                            },
                            {
                                "IMPORTTYPE": "V",
                                "IMPORTNAME": "Por Valor"
                            }
                        ]
                    },
                    "validations": {
                        "required": {}
                    }
                },
                {
					"id": 1,
                    "name": "FILEXLS",
                    "isVisible": true,
                    "class": 12,
                    "template": "field/file-upload.html",
                    "validations": {
                        "extensions": {
                            "value": [
                                ".xls",
                                ".xlsx"
                            ]
                        },
                        "required": {}
                    },
                    "events": []
                }
            ],
            "actions": [
				{
					"id": 1,
					"name": "import",
					"label": "Importar",
					"showAsAction": "never",
					"isVisible": true,
					"events": [
						{
							"id": 1,
							"name": "ActionEvent",
							"code": "Ger03600_Rateio_Produto_Controller.importarRateio(args);"
						}
					]
				},
				{
                	"id": 1,
                    "name": "cancelAction",
                    "label": "Voltar",
                    "showAsAction": "back",
                    "isVisible": true,
                    "events": [
                    	{
	                        "id": 1,
	                        "name": "ActionEvent",
	                        "code": "ScreenService.closePopup();"
                    	}
                    ]
                }
			],
            "events": []
        };
    };
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger03600_Rateio_Produto_Controller', Ger03600_Rateio_Produto_Controller);
});

// FILE: ./assets/js/controllers/Ger03900_ErrosAvisosNf_Controller.js
function Ger03900_ErrosAvisosNf_Controller(Ger03900_ErrosAvisosNf_Service, General, ScreenService, templateManager) {

	this.setDescription = function(widget) {
		var codigo = widget.currentRow.CDERROAVISO || widget.currentRow.CDERROAVISOFIL;
		let field = widget.name == "errosAvisos" ? widget.getField("CDERROAVISO") : widget.getField("CDERROAVISOFIL");
		
		field.setValue(codigo);
	};
	
	var hasDuplicateKeys = function(widget) {
		var codFilial = widget.currentRow.CDFILIAL;
		var codErrAviso = widget.currentRow.CDERROAVISOFIL;
		var arr = widget.dataSource.data.filter(function(data) {
			return ((data.CDERROAVISOFIL === codErrAviso.CDERROAVISOFIL) && (data.CDFILIAL === codFilial.CDFILIAL));
		});
		return (arr.length > 1);
	};
	
	this.save = function(widget){
    	if(widget.isValid()){
    		widget.saveRow().then(function(){
        		widget.reload();
				widget.view();
    			var currentRow = widget.currentRow;
    			currentRow.__is_new = false;
    			currentRow.changed = false;
    		}, function(calback){
    			ScreenService.showMessage(calback.data.error);
			});
    	}
    };
    
    this.editErrosEAvisos = function(widget){
    		
    	General.edit(widget);
    };
    
	this.openAutoRegister = function() {
		var tabbedWidget = templateManager.container.getWidget("autoTabbed");
		var autoWidget = templateManager.container.getWidget("cadastrAutoWidget");
		ScreenService.closePopup();
		autoWidget.newRow();
		this.validateUnidadesField(autoWidget);
		this.filialOrigemOnChange(autoWidget);
		autoWidget.getField("NMFILIALORIGEM").reload();
		ScreenService.openSwipe(tabbedWidget);
		autoWidget.activate();
	};
	
	this.saveErroAvisoUnidade = function(widget) {
		if (widget.isValid()) {
			if (!hasDuplicateKeys(widget)) {
				General.saveCrud(widget);
			}
			else {
				ScreenService.showMessage("Erro/Aviso já Cadastrado para a Unidade Selecionada.");
			}
		}
	};
	
	this.filialOrigemOnChange = function(widget){
		let filialDestinoField = widget.getField("NMFILIALDESTINO");
		if(widget.currentRow.CDFILIALORIGEM){
			filialDestinoField.dataSourceFilter = [
				{
                    name: "CDFILIAL",
                    operator: "!=",
                    value: widget.currentRow.CDFILIALORIGEM
                }	
			];
		}
		filialDestinoField.reload();
	};
	
	this.toggleUnidadesReadOnly = function(widget){
		let filialDestinoField = widget.getField("NMFILIALDESTINO"),
			currentRow = widget.currentRow;
		
		filialDestinoField.readOnly = !currentRow.CDFILIALORIGEM || (currentRow.SELECAO == "T");
		if(filialDestinoField.readOnly){
			filialDestinoField.setValue('');
		}
	};
	
	this.saveAutoRegister = function(args) {
		let widget = args.owner.widget;
		
		if(!widget.isValid()){
			return;
		}
		
		let filter  = [
	            {
	                "name": "CDFILIAL",
	                "operator": "=",
	                "value": widget.currentRow.CDFILIALORIGEM
	            }
        	],
        	destinos = [];
        
        Ger03900_ErrosAvisosNf_Service.findAutoRegister(filter).then( response => {
        	let filiais = widget.getField("NMFILIALDESTINO").dataSource.data,
	        	errosEAvisosOrigem = response.dataset.erroavisonffil_filial,
	        	rowsToSave = [];
        	
        	destinos = (widget.currentRow.SELECAO == 'T') ? filiais.map( filial => {return filial.CDFILIAL;}) 
        												  : widget.currentRow.CDFILIALDESTINO;
        	//Se errosEAvisos for vazio limpar registros
        	if(errosEAvisosOrigem.length > 0){
        		let i = 0;
        		errosEAvisosOrigem.forEach( errosEAvisos => {
	        		errosEAvisos.SOBRESCREVER = widget.currentRow.SOBRESCREVER;
		        	destinos.forEach( destino => {
		        		rowsToSave[i] = Util.clone(errosEAvisos);
		        		rowsToSave[i++].CDFILIAL = destino;
		        	});
	        	});
	        	
	        	Ger03900_ErrosAvisosNf_Service.saveAutoRegister(rowsToSave).then( () => {
                	let widgetErrosEAvisosUnidade = templateManager.container.getWidget("errosAvisosUnidade");
                	
					ScreenService.showMessage("Registros salvos com sucesso.");
					ScreenService.closeSwipe();
					widgetErrosEAvisosUnidade.activate();
					widgetErrosEAvisosUnidade.reload();
					
	        	}).catch( error => {
	        		let erro = error.data ? error.data.error : error;
	        		
	        		ScreenService.showMessage(erro);
	        	});
        	} else{
        		ScreenService.confirmMessage('A unidade de origem não possui erros ou avisos, os registros das unidades de destino serão excluídos. Deseja continuar?', 'question',
	                (success) =>  {
	                    Ger03900_ErrosAvisosNf_Service.deleteRegisters(destinos)
	                    .then( () => {
	                    	let widgetErrosEAvisosUnidade = templateManager.container.getWidget("errosAvisosUnidade");
	                    	
							ScreenService.showMessage("Registros salvos com sucesso.");
							ScreenService.closeSwipe();
							widgetErrosEAvisosUnidade.activate();
							widgetErrosEAvisosUnidade.reload();
							
			        	})
	                    .catch((error) => {
	                    	let erro = error.data ? error.data.error : error;
	        		
	        				ScreenService.showMessage(erro);
	                    });
	                },
	                function(error) {
	                     return ScreenService.errorNotification(error, 'ERROR');
	                }
	            ); 
        	}
        	
        });
		
	};
	
	this.setDefaultUnidade = function(childWidget) {
		var widget = childWidget.parent;
		childWidget.currentRow.CDFILIAL = widget.currentRow.CDFILIAL;
		childWidget.currentRow.NMFILIAL = widget.currentRow.NMFILIAL;
		childWidget.getField("CDERROAVISOFIL").reload();
		childWidget.currentRow.CDERROAVISOFIL = "AV01";
		childWidget.currentRow.IDERROAVISOFIL = "C";
	};
	
	this.validateUnidadesField = function(widget) {
		var unidadesField = widget.getField("NMFILIALDESTINO");
		
		unidadesField.setValue(null);
		if (widget.currentRow.SELECAO === "L") {
			unidadesField.validations.required = {message: "Campo obrigatório"};
			unidadesField.readOnly = false;
		}
		else {
			unidadesField.validations = {};
			unidadesField.readOnly = true;
		}

		unidadesField.reload();
	};
	
	this.updateGridFilter = function(widget) {
		var childWidget = templateManager.container.getWidget("errosAvisosUnidade");
		childWidget.dataSourceFilter[0].value = widget.currentRow.CDFILIAL;
		childWidget.reload();
	};
	
	this.closeAutoRegister = function(widget) {
		ScreenService.closeSwipe();
		templateManager.container.getWidget('errosAvisosUnidade').activate();
	};
	
	this.afterSelect = function(field) {
		if (field.value()) {
			field.getParent().widgets[0].getActions()[0].isVisible = true;
		} else{
			field.getParent().widgets[0].getActions()[0].isVisible = false;
		}
	};
	
}
Configuration(function (ContextRegister) {
	ContextRegister.register('Ger03900_ErrosAvisosNf_Controller', Ger03900_ErrosAvisosNf_Controller);
});

// FILE: ./assets/js/controllers/Ger05500_produtosDesoneracaoImpostos_Controller.js
function Ger05500_produtosDesoneracaoImpostos_Controller(General, ScreenService, templateManager, Ger05500_produtosDesoneracaoImpostos_Service) {

    var self = this;

    self.validaActionAdicionar = function(widget){
        if (widget.getField('NMPRODUTO').value()) {
            widget.widgets.forEach(function(widgetFilho){
                widgetFilho.getAction('ADD').isVisible = true;
            });
        }else{
            widget.widgets.forEach(function(widgetFilho){
                widgetFilho.getAction('ADD').isVisible = false;
            });
        }
    };

    self.salvarWidget = function(widget){
        widget.currentRow.CDPRODUTO = widget.getParent().currentRow.CDPRODUTO;
        General.widgetSave(widget);
    };

    self.deleteProdutos = function(widget) {
        ScreenService.confirmMessage('A exclusão do produto irá excluir também os fornecedores, clientes e filiais relacionados. Deseja realmente excluir?', 'question', function (success) {
            General.callRestEngine('df#/delete_produtos_desoneracao', 'Row', {"CDPRODUTO": widget.getParent().currentRow.CDPRODUTO}).then(function(){
                ScreenService.closeSwipe().then(function() {
                    widget.getParent().getParent().reload();
                    widget.getParent().getParent().activate();
                });
            });
        });
    };

    self.validaCamposRequiredSave = function (widget, widgetAtiva){
        if (!widget.isValid()){
            widget.activate();
        } else if (!widgetAtiva.isValid()) {
            widgetAtiva.activate();
        }
    };

    self.preparaWidgetTabbed = function(widget, isNew){
        var tabbed = templateManager.container.getWidget('produtos_desoneracao_tabbed');
        if (isNew) {
            tabbed.widgets.forEach(function(widgetFilho){
                widgetFilho.dataSourceFilter[0].value = null;
            });
            tabbed.currentRow.NMPRODUTO = null;
            tabbed.currentRow.CDPRODUTO = null;
        }else{
            tabbed.widgets.forEach(function(widgetFilho){
                widgetFilho.dataSourceFilter[0].value = widget.currentRow.CDPRODUTO;
            });
            tabbed.currentRow.NMPRODUTO = widget.currentRow.NMPRODUTO;
            tabbed.currentRow.CDPRODUTO = widget.currentRow.CDPRODUTO;
        }
        templateManager.container.getActiveWidget().reload();
    };

    self.ativarWidgetTabbed = function(widget){
        ScreenService.openSwipe(widget.widgets[0]);
        var tabbed = templateManager.container.getWidget('produtos_desoneracao_tabbed');
        tabbed.activate();
        tabbed.reload();
        self.validaActionAdicionar(tabbed);
    };

    self.newProdutoDesoneracao = function(widget){
        var widget_tabbed = templateManager.container.getWidget('produtos_desoneracao_tabbed');
        widget_tabbed.getField('NMPRODUTO').readOnly = false;
        self.preparaWidgetTabbed(widget, true);
        ScreenService.openSwipe(widget_tabbed).then(function(){
            self.validaActionAdicionar(widget_tabbed);
        });
    };

    self.tabbedGoBackToList = function(widget){
        ScreenService.closeSwipe().then(function() {
            var widget_tabbed = templateManager.container.getWidget('produtos_desoneracao_tabbed');
            widget_tabbed.getField('NMPRODUTO').readOnly = true;
            widget.list();
            widget.reload();
            widget.activate();
        });

    };
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger05500_produtosDesoneracaoImpostos_Controller', Ger05500_produtosDesoneracaoImpostos_Controller);
});

// FILE: ./assets/js/controllers/Ger06600_TipoDeOperacao_Controller.js
var Ger06600_TipoDeOperacao_Controller = function(ZhFilterPreferences, ScreenService, templateManager, Ger06600_TipoDeOperacao_Service, General) {

    this.openFilterWidget = function(widget){
        let currentRow = widget.currentRow;
        
        currentRow.IDENTRSAIDOP = 'E';
        ZhFilterPreferences.openFilterWidget(widget, currentRow);  
    };

    var applyDefaultValues = function(parentWidget) {
        parentWidget.widgets.forEach(function(childWidget) {
            childWidget.fields.forEach(function(field) {
                field.applyDefaultValue();
            });
        });
    };
    
    var controlaStatusWidgets = function(status) {
         var parentWidget = templateManager.container.getWidget("tipoOperac");
        var widgetAtiva = templateManager.container.getActiveWidget();
        var allWidgets = parentWidget.widgets;
        widgetAtiva.enabled = true;
        allWidgets.filter(widget => widget.name !== widgetAtiva.name).forEach(widget => widget.enabled = status);

        if (parentWidget.name !== widgetAtiva.name)
            parentWidget.enabled = status;

    };


    this.cancelEdit = function(args) {
        var parentWidget = templateManager.container.getWidget("tipoOperac");
        General.backOrCancel(args);
        parentWidget.widgets.forEach(function(childWidget) {
            if (childWidget.name != "filterPopup") {
                childWidget.template = childWidget.templateToView;
            }
        });
        controlaStatusWidgets(true);
        General.changeWidgetAcessibility(parentWidget.widgets, true);
    };

    this.mudaVisibilidadeBloqueio = function(widget){
		if(widget.getField("IDOBRIGANEXO").value()=='S'){
			widget.getField("IDBLOQANEXO").readOnly = false;
		    widget.currentRow.IDBLOQANEXO = "";
		    widget.getField("IDBLOQANEXO").reload();
		}
		else
			widget.getField("IDBLOQANEXO").readOnly = true;
	};

    this.changeFieldsVisibility = function(parentWidget) {
        var showFieldsEntrada = parentWidget.dataSourceFilter[0].value === "E";

        parentWidget.widgets.forEach( (childWidget) => {
            childWidget.fields.forEach( (field) => {
                if (field.entradaSaida === "E") {
                    field.showOnForm = showFieldsEntrada;
                    field.showOnView = showFieldsEntrada;
                }
                else if (field.entradaSaida === "S") {
                    field.showOnForm = !showFieldsEntrada;
                    field.showOnView = !showFieldsEntrada;
                }
            });
        });

        var fieldNumNfLancamento = templateManager.container.getWidget("fiscal").getField("IDPERNUMENF");
        fieldNumNfLancamento.showOnForm = true;
        fieldNumNfLancamento.readOnly = showFieldsEntrada;
        templateManager.container.getWidget('faturamento').showAsTab = !showFieldsEntrada;
        templateManager.container.getWidget('controleQualidade').showAsTab = !showFieldsEntrada;
        
        /*tipo deoperação 10 por tratamento interno do sistema não gera financeiro.*/
        templateManager.container.getWidget("financeiro").showAsTab = parentWidget.currentRow.CDTIPOOPER != "10";
        templateManager.container.getWidget("estoque").getField("IDATUESTTPOP").readOnlyOnEdit = (parentWidget.currentRow.CDTIPOOPER === "10"
                                                                                            || parentWidget.currentRow.CDTIPOOPER === "11");

        
        
    };

    this.deleteTipoOperac = function(parentWidget) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function () {
            parentWidget.removeRow().then(function () {
                parentWidget.reload();
                ScreenService.closeSwipe().then(function () {
                    parentWidget.activate();
                });
            }, function (error) {
                if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                    ScreenService.showMessage('Exclusão não permitida! O registro possui registros filhos.');
                } else {
                    ScreenService.showMessage(error.data.error);
                }
            });
        });
    };

    this.editTipoOperac = function(widget) {
        var parentWidget = templateManager.container.getWidget("tipoOperac");
        widget.edit();
        parentWidget.backupRow = Util.clone(parentWidget.currentRow);
        parentWidget.widgets.forEach(function(childWidget) {
            controlaStatusWidgets(false);
            childWidget.edit();
            childWidget.template = childWidget.templateToForm;
        });
    };
    
    this.estoqueOnEnterTab = function(widget) {
        var currentRow = widget.currentRow;
        var toReadOnly = false;
        
        if (currentRow.CDTIPOOPER == '05' || currentRow.CDTIPOOPER == '11') {
            currentRow.IDATUESTTPOP = 'N';
            toReadOnly = true;
        }
        widget.getField("IDATUESTTPOP").readOnly = toReadOnly;
    };

    this.fiscalOnEnterTab = function(widget) {
        var currentRow = widget.currentRow;
        var toReadOnly = false;
        var utilizaVencimento = currentRow.IDUTILVENC == 'S';
        var isSaida = currentRow.IDENTRSAIDOP == 'S';
        var permInfNF = currentRow.IDPERNUMENF == 'S';
        var documentoOrigField = isSaida ?
            widget.getField("NMTIPOOPER_DSOPERNFORIG") : widget.getField("NMTIPOOPER_DSOPERNFORIG_ENT");
            
        if (!isSaida && currentRow.CDTIPOOPER == '13') {
            currentRow.IDOPERDEVONF = 'S';
            currentRow.IDNFORIGOBRI = 'N';
            currentRow.IDUTLLISTORI = 'N';
            toReadOnly = true;
        }
        
        widget.getField("IDUTLLISTORI").readOnly = toReadOnly;
        
        if (isSaida && currentRow.CDTIPOOPER == '06') {
            currentRow.IDOPERDEVONF = 'S';
            currentRow.IDNFORIGOBRI = 'S';
            toReadOnly = true;
        }
        
        widget.getField("IDOPERDEVONF").readOnly = toReadOnly;
        widget.getField("IDNFORIGOBRI").readOnly = toReadOnly;
        
        if(utilizaVencimento) {
            currentRow.IDNFESTCANC = 'N';
        }
        
        widget.getField("IDNFESTCANC").readOnly   = utilizaVencimento;
        widget.getField("IDAPROVAUTOPS").readOnly = (!isSaida || currentRow.IDNFPRESTSERV != 'S');
        widget.getField("IDUTCTRLAIDF").readOnly  = !permInfNF;
        
        widget.getField("IDORIGDESTOP").label = isSaida ? "Destino" : "Origem";

        documentoOrigField.reload();
        
        if(currentRow.DSOPERNFORIG) {
            this.setCampoSelectMultiplo(widget);
        }
        this.changeFieldFiscal(widget);
        this.validateUtilizaDocumentoOrigem(widget, true);
        this.idnfestcancOnChange(widget);
        this.permiteFornDesc(widget);
    };
    
    this.changeFieldFiscal = function(widget) {
        var currentRow = widget.currentRow;
        if(currentRow.IDNFESTCANC == 'S' && currentRow.IDNOTADEVO == 'N') {
            widget.getField("IDNFESTCANC").readOnly = false;  
            widget.getField("IDNOTADEVO").readOnly = true;
        }
        else if(currentRow.IDNFESTCANC == 'N' && currentRow.IDNOTADEVO == 'S') {
            widget.getField("IDNFESTCANC").readOnly = true;
            widget.getField("IDNOTADEVO").readOnly = false;
        } else {
            widget.getField("IDNFESTCANC").readOnly = false;
            widget.getField("IDNOTADEVO").readOnly = false;
        }
        widget.reload();

    };
    
    this.setCampoSelectMultiplo = function(widget){
        var isSaida = widget.currentRow.IDENTRSAIDOP == 'S';
        var documentoOrigField = isSaida ?
            widget.getField("NMTIPOOPER_DSOPERNFORIG") : widget.getField("NMTIPOOPER_DSOPERNFORIG_ENT");
            
        // Retira os '_' do inicio e do fim se existirem e tranforma em um array
            if(!Array.isArray(widget.currentRow.DSOPERNFORIG)) {
                widget.currentRow.DSOPERNFORIG = widget.currentRow.DSOPERNFORIG.replace(/^\_|\_$/g, '').split("_");
            }
            
            documentoOrigField.setValue(widget.currentRow.DSOPERNFORIG);
    };

    this.idnfestcancOnChange = function(widget) {
        var currentRow = widget.currentRow;
        var isValid = currentRow.IDNFESTCANC != 'S' && currentRow.IDOPERDEVONF == 'S';
        widget.getField('IDUTLLISTORI').readOnly = !(isValid);
        currentRow.IDUTLLISTORI = isValid ? 'S' : 'N';
        if (widget.isNewRow()) {
            currentRow.IDCONTABOPER = currentRow.IDNFESTCANC == 'S' ? 'N' : 'S';
        }
    };
    
    this.docOrigemOnChange = function(widget){
        let currentRow = widget.currentRow;
        let numeroMaximo = 5;
        
        if(currentRow.DSOPERNFORIG.length > numeroMaximo){
            ScreenService.showMessage("Só é possível adicionar no máximo " + numeroMaximo + " documentos de origem. Retirados os " + (currentRow.DSOPERNFORIG.length - numeroMaximo) + " últimos documentos.");
            currentRow.DSOPERNFORIG = currentRow.DSOPERNFORIG.slice(0, numeroMaximo);
        }
        
        this.setCampoSelectMultiplo(widget);
    };

    this.newRow = function(widget) {
        if (widget.dataSourceFilter[0]) {
            General.changeWidgetAcessibility(widget.widgets, false);
            widget.newRow();
            widget.widgets.forEach(function(childWidget) {
                childWidget.newRow();
                childWidget.activate();
                childWidget.template = childWidget.templateToForm;
            });
            this.setWidgetsCurrentRow(widget);
            applyDefaultValues(widget);
            this.fiscalOnEnterTab(templateManager.container.getWidget('fiscal'));
            this.estoqueOnEnterTab(templateManager.container.getWidget('estoque'));
        }
        else {
            ScreenService.showMessage("É necessário filtrar por Entrada/Saída.");
        }
    };

    this.onLoadDataSource = function(widget) {
        if (!widget.dataSourceFilter[0]) {
            ScreenService.showMessage(ScreenService.i18n("É necessário filtrar por Entrada/Saída."));
        }
        else {
            this.changeFieldsVisibility(widget);
        }
    };

    this.saveTipoOperac = function(parentWidget) {
        var isSaida = parentWidget.currentRow.IDENTRSAIDOP == 'S';
        var widgetFiscal = parentWidget.widgets.filter(a => a.name === "fiscal" )[0];
        var documentoOrig = isSaida ? widgetFiscal.currentRow.NMTIPOOPER_DSOPERNFORIG
                                    : widgetFiscal.currentRow.NMTIPOOPER_DSOPERNFORIG_ENT;
        
        if (parentWidget.isValid()) {
            var currentRow = parentWidget.currentRow;
            if(currentRow.NMTIPOCUSTO == null)
                currentRow.NMTIPOCUSTO = "";
            else if(documentoOrig == '' && widgetFiscal.currentRow.IDNFORIGOBRI == 'S') 
                ScreenService.showMessage(ScreenService.i18n("Documento de Origem deve ser informado."));

            else if (validateOnSave(parentWidget, currentRow)) {
                currentRow.__columns_to_return = ["NRORG"];
                currentRow.IDCONTABOPER = currentRow.IDCONTABOPER || 'S';
                currentRow.CDTIPOOPER = parentWidget.getField("CDTIPOOPER").value().toUpperCase();
                currentRow.IDOBRIGANEXO = currentRow.IDOBRIGANEXO || 'N';
                
                if(currentRow.CDEXTIPOOPER) {
                    currentRow.CDEXTIPOOPER = parentWidget.getField("CDEXTIPOOPER").value().toUpperCase();
                }
                
                if(Array.isArray(currentRow.DSOPERNFORIG)) {
                    currentRow.DSOPERNFORIG = '_' + currentRow.DSOPERNFORIG.join("_") + '_';
                }
                
                 /*“Atualiza Estoque” sempre como “Sim” para a operação 10 e bloquear alteração,
                    uma vez que esta operação, por tratamento interno do sistema sempre atualiza estoque.
                    “Atualiza Estoque” sempre como "Não" para a operação 11 e bloquear alteração,
                    uma vez que esta operação é associada ao tipo de operação 10 */
                    
                if(currentRow.CDTIPOOPER === "10"){
                    currentRow.IDATUESTTPOP = "S";
                }else if(currentRow.CDTIPOOPER === "11"){
                    currentRow.IDATUESTTPOP = "N";
                }
                
                Ger06600_TipoDeOperacao_Service.saveTipoOperac(currentRow).then(function(successData) {
                    parentWidget.widgets.forEach(function(childWidget) {
                        childWidget.view();
                        childWidget.template = childWidget.templateToView;
                    });
        
                    if (parentWidget.currentRow.__is_new) {
                        parentWidget.list(true);
                        ScreenService.goBack();
                        parentWidget.reload();
                    } else {
                        parentWidget.view();
                    }

                    controlaStatusWidgets(true);   
                    parentWidget.enabled = true;
                    General.changeWidgetAcessibility(parentWidget.widgets, true);
                    

                }, function(error) {
                    var errorMessage = 'Timeout';
                    if (error.data) {
                        errorMessage = error.data.error;
                    }
                    ScreenService.showMessage(ScreenService.i18n(errorMessage));
                    General.changeWidgetAcessibility(parentWidget.widgets, true);
                });
            }
        }
        else {
            parentWidget.currentWidget = parentWidget;
        }
    };
    
    this.idNFPrestservOnChange = function(widget) {
        let currentRow = widget.currentRow;
        if(currentRow.IDENTRSAIDOP == 'E'){
            if(currentRow.IDNFPRESTSERV == 'S'){
                widget.getField("IDAPROVAUTOPS").readOnly = false;  
            } else{
                widget.getField("IDAPROVAUTOPS").readOnly = true;
                widget.getField("IDAPROVAUTOPS").setValue('N');
            }
             
        }
        
    };
    
    this.setDefaultTipoCusto = function(widget) {
        //
    };

    this.setDefaultIdEntradaSaida = function(widget) {
        widget.currentRow.IDENTRSAIDOP = widget.dataSourceFilter[0].value;
        
    };

    this.setWidgetsCurrentRow = function(parentWidget) {
        parentWidget.widgets.forEach(function(childWidget) {
            childWidget.currentRow = parentWidget.currentRow;
        });
    };

    this.toggleEntradaSaidaField = function(widget) {
        if (widget.name !== "tipoOperac") {
            var idEntradaSaida = widget.currentRow.IDENTRSAIDOP;
            var tipoOperacField;
            if(widget.name == "zhFilterPreferencesSelectConditions"){
                tipoOperacField = widget.getField("NMTIPOOPER");
                widget.currentRow.NMTIPOOPER ='';
            } else {
                tipoOperacField = widget.getField("CDTIPOOPER");
                widget.currentRow.CDTIPOOPER ='';
            }
            if (idEntradaSaida) {
                tipoOperacField.dataSourceFilter[0].value = idEntradaSaida;
                tipoOperacField.readOnly = false;
            }
            else {
                tipoOperacField.readOnly = true;
            }
            tipoOperacField.reload();

        }
        else {
            this.changeFieldsVisibility(widget);
        }
    };

    var validateOnSave = function(parentWidget, currentRow) {
        var isValid = true;
        var fiscalWidget = templateManager.container.getWidget('fiscal');
        

        if (currentRow.IDNFESTCANC == 'S' && currentRow.IDOPERDEVONF != 'S') {
            parentWidget.currentWidget = fiscalWidget;
            ScreenService.showMessage(ScreenService.i18n("Atenção! Tipo de operação de estorno de cancelamento deve utilizar documento de origem."));
            isValid = false;
        }
        else if (currentRow.IDNFESTCANC == 'S' && currentRow.IDNFORIGOBRI != 'S') {
            parentWidget.currentWidget = fiscalWidget;
            ScreenService.showMessage(ScreenService.i18n("Operação bloqueada! Tipo de operação de estorno de cancelamento deve utilizar documento de origem obrigatório."));
            isValid = false;
        } 

        return isValid;
    };

    this.validateUtilizaDocumentoOrigem = function(widget, fromOnEnterTab) {
        var currentRow = widget.currentRow;
        var isSaida = widget.currentRow.IDENTRSAIDOP == 'S';
        var documentoOrigField = isSaida ?
            "NMTIPOOPER_DSOPERNFORIG" : "NMTIPOOPER_DSOPERNFORIG_ENT";
        
        if (currentRow.IDOPERDEVONF !== 'S') {
            Ger06600_TipoDeOperacao_Service.checkParametersUtilizaDocumento(currentRow).then(function(isValid) {
                if (!isValid) {
                    if (!fromOnEnterTab) {
                        ScreenService.showMessage(ScreenService.i18n("Operação bloqueada! Existe parametrização de despesa acessória. Favor verificar controle fiscal."));
                        currentRow.IDOPERDEVONF = 'S';
                    }
                    if (currentRow.IDENTRSAIDOP == 'E' && currentRow.CDTIPOOPER == '13') {
                        widget.getField("IDUTLLISTORI").readOnly = true;
                        widget.getField("IDNFORIGOBRI").readOnly = true;
                    }
                    else if (currentRow.IDENTRSAIDOP == 'S' && currentRow.CDTIPOOPER == '06') {
                        widget.getField("IDUTLLISTORI").readOnly = false;
                        widget.getField("IDNFORIGOBRI").readOnly = true;
                    }
                    else {
                        if (currentRow.IDNFESTCANC == 'S') {
                            currentRow.IDUTLLISTORI = 'N';
                        }
                        widget.getField("IDUTLLISTORI").readOnly = false || currentRow.IDNFESTCANC == 'S';
                        widget.getField("IDNFORIGOBRI").readOnly = false;
                    }
                    if (currentRow.IDENTRSAIDOP == 'S') {
                        currentRow.IDDCORIGRELAC = 'N';
                    }
                    widget.getField("IDDCORIGRELAC").readOnly = false || currentRow.IDENTRSAIDOP == 'S';
                    templateManager.container.getWidget('contabil').getField("IDCONTCONFORIG").readOnly = false;
                    widget.getField(documentoOrigField).readOnly = false;
                    if(widget.getField(documentoOrigField).validations.required){
                        delete(widget.getField(documentoOrigField).validations.required);
                    }
                
                }
                else {
                    currentRow.IDDCORIGRELAC = 'N';
                    currentRow.IDNFORIGOBRI = 'N';
                    currentRow.IDUTLLISTORI = 'N';
                    currentRow.IDCONTCONFORIG = 'N';
                    if (currentRow.NMTIPOOPER_DSOPERNFORIG) {
                        currentRow.NMTIPOOPER_DSOPERNFORIG = '';
                    }
                    else {
                        currentRow.NMTIPOOPER_DSOPERNFORIG_ENT = '';
                    }
                    currentRow.DSOPERNFORIG = '';
                    currentRow.IDUTCUSORIGEST = 'N';
                    widget.getField("IDDCORIGRELAC").readOnly = true;
                    widget.getField("IDNFORIGOBRI").readOnly = true;
                    widget.getField("IDUTLLISTORI").readOnly = true;
                    templateManager.container.getWidget('contabil').getField("IDCONTCONFORIG").readOnly = true;
                    widget.getField(documentoOrigField).readOnly = true;
                    widget.getField(documentoOrigField).validations.required = {};
                    widget.getField(documentoOrigField).validations.required.message = "Campo obrigatório";
                    templateManager.container.getWidget("estoque").getField("IDUTCUSORIGEST").readOnly = true;
                    if(widget.getField(documentoOrigField).validations.required.message){
                        delete(widget.getField(documentoOrigField).validations.required.message);
                    }
                    delete(widget.getField(documentoOrigField).validations.required);
                
                   
                }
            });
            if(widget.getField(documentoOrigField).validations.required){
               if(widget.getField(documentoOrigField).validations.required.message){
                    delete(widget.getField(documentoOrigField).validations.required.message);
                }
                delete(widget.getField(documentoOrigField).validations.required);
            }
            if(widget.currentRow.NMTIPOOPER_DSOPERNFORIG) {
                widget.currentRow.NMTIPOOPER_DSOPERNFORIG = null;
            }
            else {
                widget.currentRow.NMTIPOOPER_DSOPERNFORIG_ENT = null;
            }
        }
        else {
            if (currentRow.IDENTRSAIDOP == 'E' && currentRow.CDTIPOOPER == '13') {
                widget.getField("IDUTLLISTORI").readOnly = true;
                widget.getField("IDNFORIGOBRI").readOnly = true;
            }
            else if (currentRow.IDENTRSAIDOP == 'S' && currentRow.CDTIPOOPER == '06') {
                widget.getField("IDUTLLISTORI").readOnly = false;
                widget.getField("IDNFORIGOBRI").readOnly = true;
            }
            else {
                if (currentRow.IDNFESTCANC == 'S') {
                    currentRow.IDUTLLISTORI = 'N';
                }
                widget.getField("IDUTLLISTORI").readOnly = false || currentRow.IDNFESTCANC == 'S';
                widget.getField("IDNFORIGOBRI").readOnly = false;
            }
            if (currentRow.IDENTRSAIDOP == 'S') {
                currentRow.IDDCORIGRELAC = 'N';
            }
            widget.getField("IDDCORIGRELAC").readOnly = false || currentRow.IDENTRSAIDOP == 'S';
            templateManager.container.getWidget('contabil').getField("IDCONTCONFORIG").readOnly = false;
            widget.getField(documentoOrigField).readOnly = false;
            if(currentRow.IDNFORIGOBRI == 'S') {
                widget.getField(documentoOrigField).validations.required = {};
                widget.getField(documentoOrigField).validations.required.message = "Campo obrigatório";
            }
            currentRow.IDNFORIGOBRI = 'S';
            templateManager.container.getWidget("estoque").getField("IDUTCUSORIGEST").readOnly = false;
        }
    };
    
    this.financeiroOnEnterTab = function(widget) {
        let utilizaVencimento = widget.currentRow.IDUTILVENC == 'S';
        
        widget.getField("IDGERAVENC").readOnly     = !utilizaVencimento;
        widget.getField("IDUTILVENCOBR").readOnly  = !utilizaVencimento;
        widget.getField("IDGERAVENCNFAP").readOnly = !utilizaVencimento;
    };
    
    this.utilVectoOnChange = function (wid) {
        let row = wid.currentRow;
        let wfis = templateManager.container.getWidget("fiscal");
        let west = templateManager.container.getWidget("estoque");
        let isSaida = wid.currentRow.IDENTRSAIDOP == 'S';
        let documentoOrigField = isSaida ?
            "NMTIPOOPER_DSOPERNFORIG" : "NMTIPOOPER_DSOPERNFORIG_ENT";
        
        if (wid.currentRow.IDUTILVENC == 'N') {
            wfis.getField("IDNFESTCANC").readOnly = false;
            wfis.getField("IDOPERDEVONF").readOnly = false;
            wfis.getField("IDNFORIGOBRI").readOnly = false;
            wfis.getField(documentoOrigField).readOnly = false;
            west.getField("IDUTCUSORIGEST").readOnly = false;
            
            row.IDGERAVENC = 'N';
            row.IDGERAVENCNFAP = 'N';
            row.IDUTILVENCOBR = 'N';
            wid.getField("IDGERAVENC").readOnly = true;
            wid.getField("IDGERAVENCNFAP").readOnly = true;
            wid.getField("IDUTILVENCOBR").readOnly = true;
        }
        else {
            row.IDNFESTCANC = 'N';
            row.IDOPERDEVONF = 'N';
            row.IDUTCUSORIGEST = 'N';
            row.IDNFORIGOBRI = 'N';
            row.DSOPERNFORIG = '';
            if(row.NMTIPOOPER_DSOPERNFORIG) {
                row.NMTIPOOPER_DSOPERNFORIG = '';
            }
            else {
                row.NMTIPOOPER_DSOPERNFORIG_ENT = '';
            }
            wfis.getField("IDNFESTCANC").readOnly = true;
            //wfis.getField("IDOPERDEVONF").readOnly = true;
            wfis.getField("IDNFORIGOBRI").readOnly = true;
            wfis.getField(documentoOrigField).readOnly = true;
            west.getField("IDUTCUSORIGEST").readOnly = true;
            
            wid.getField("IDGERAVENC").readOnly = false;
            wid.getField("IDGERAVENCNFAP").readOnly = false;
            wid.getField("IDUTILVENCOBR").readOnly = false;
        }
    };
    
    this.permiteNFLancOnChange = function (wid) {
        let row = wid.currentRow;
        
        if (row.IDPERNUMENF == 'N') {
            row.IDUTCTRLAIDF = 'N';
            wid.getField("IDUTCTRLAIDF").readOnly = true;
        }
        else {
            wid.getField("IDUTCTRLAIDF").readOnly = false;
        }
    };
    
    this.permiteFornDesc = function (widget){
        var permiteCnpj =  widget.getField("IDINFCPFCNPJNF").value() === "S";
        var fornPadrao = widget.getField("NMRAZSOCFORN");
        var descOp = widget.getField("DSOPERNFAVISTA");
        
        fornPadrao.readOnly = !permiteCnpj;
        descOp.readOnly = !permiteCnpj;
        
        fornPadrao.reload();
        descOp.reload();
    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Ger06600_TipoDeOperacao_Controller", Ger06600_TipoDeOperacao_Controller);
});


// FILE: ./assets/js/controllers/Ger07800_CodigoEspecificadorSubstituicaoTributaria_Controller.js
function Ger07800_CodigoEspecificadorSubstituicaoTributaria_Controller(ScreenService, ZhFilterPreferences, General, templateManager, ZHPromise, Ger07800_CodigoEspecificadorSubstituicaoTributaria_Service) {

    this.onChangeCEST = function(widget) {
       if(widget.currentRow.CDCEST) {
            Ger07800_CodigoEspecificadorSubstituicaoTributaria_Service.getSegmentoMercadoria(widget.currentRow.CDCEST.substr(0, 2)).then( success => {
               if(success.dataset.segmercabens[0]) {
                   widget.currentRow.CDSEGMENTO = success.dataset.segmercabens[0].CDSEGMENTO;
               }
               else {
                   var error = "O segmento para esse CEST ainda não foi cadastrado. Favor realizar este cadastro!";
                   widget.currentRow.CDSEGMENTO = '';
                   ScreenService.showMessage(error);
               }
           });
       }
    };
   
    this.setDefaultCodigo = function(childWidget) {
        var widget = childWidget.parent;
        childWidget.currentRow.CDCEST = widget.currentRow.CDCEST;
        childWidget.currentRow.CDSEGMENTO = widget.currentRow.CDSEGMENTO;
    };
    
    this.saveCodigo = function(args){
        let widget = args.owner.widget,
        currentRow = widget.currentRow;
        
        if(!widget.isValid()){
            return;
        } else if(!currentRow.CDSEGMENTO){
            ScreenService.showMessage("O segmento para este CEST ainda não foi cadastrado. Favor realizar este cadastro no botão \"Segmentos de Mercadorias\".");
        } else {
            Ger07800_CodigoEspecificadorSubstituicaoTributaria_Service.getCest(currentRow.CDITEM, currentRow.CDCEST).then( response => {
                if(response.dataset.codespsubtri.length > 0 && currentRow.__is_new){
                    ScreenService.showMessage("Código Especificador da Substituição Tributária (CEST) já cadastrado.");
                } else if (currentRow.CDSEGMENTO == currentRow.CDCEST.substr(0,2) && currentRow.CDITEM.substr(0,3) == currentRow.CDCEST.substr(3,3)){
                    General.saveRow(args, false);
                    widget.view();
                } else {
                    ScreenService.showMessage("Dígitos do CEST não estão de acordo com o número do segmento ou do item.");
                }
            });
        }
        
        args.owner.widget.reload();
        
    };
    
    this.saveSegmento = function(args){
        let widget = args.owner.widget;
        if(widget.isValid()){
            Ger07800_CodigoEspecificadorSubstituicaoTributaria_Service.getSegmentoMercadoria(widget.currentRow.CDSEGMENTO).then( response => {
                if(response.dataset.segmercabens.length > 0 && widget.currentRow.__is_new){
                    ScreenService.showMessage("Segmento já cadastrado.");
                    widget.currentRow.CDSEGMENTO = '';
                } else{
                    General.saveRow(args, false);
                    widget.view();
                }
            });
        }
    };
    
    this.deleteSegmento = function(args){
        let widget = args.owner.widget,
        currentRow = widget.currentRow;
        
        Ger07800_CodigoEspecificadorSubstituicaoTributaria_Service.getCestBySegmento(currentRow.CDSEGMENTO).then(response => {
            if(response.dataset.codespsubtri.length > 0){
                ScreenService.showMessage("Operação bloqueada. Existe CEST cadastrado para este segmento.");
            } else{
                General.deleteCrud(args.owner.widget);
            }
        });
    };
    
}
Configuration(function (ContextRegister) {
    ContextRegister.register("Ger07800_CodigoEspecificadorSubstituicaoTributaria_Controller", Ger07800_CodigoEspecificadorSubstituicaoTributaria_Controller);
});

// FILE: ./assets/js/controllers/Ger10200_ParametrosGrupoFiscal_Controller.js
var Ger10200_ParametrosGrupoFiscal_Controller = function(Ger10200_ParametrosGrupoFiscal_Service,ScreenService, templateManager,General){

	this.carregarWidget = function(args){
		var widget = args.owner;
		widget.reload();
	};
	this.salvar = function(args){
		var fechaSwipe = false;
		var widget = args.owner.widget;
		var dataSource = args.owner.widget.dataSource.name;
		var dataSourceGeral = dataSource.substr(4,(dataSource.indexOf('_')-4) ) ;
		
		if(widget.isValid()){
			if(widget.currentRow.__is_new === true){
				fechaSwipe = true;
			}
			widget.currentRow.__is_new = false;
			widget.dataSource.name= "df#/crud/"+dataSourceGeral;
			if(widget.name=="gridImpostoFilial"){
				widget.dataSource.name=dataSource;
			}
			General.saveCrud(widget).then(function(Success) {
	            widget.getAction('cadastramentoAutomatico').isVisible = true;
	            widget.dataSource.name=dataSource;
	           	if(fechaSwipe){
	                ScreenService.closeSwipe();
	                widget.reload();
	            }
	        },function(callback){
				//widget.currentRow.__is_new = true;
				if(callback.data){
					ScreenService.showMessage(callback.data.error);
				}
				widget.dataSource.name=dataSource;
		  });
		}
	};
	
	this.voltarWidget = function(args){
		var widget = args.owner.widget;
		widget.actions.filter(a => a.name === "cadastramentoAutomatico" )[0].isVisible = true;
		widget.actions.filter(c => c.name === "Voltar")[0].isVisible = false;
		ScreenService.goBack(widget);	
	};
	
	this.botaoCadastramentoAutomatico = function(args){
		var widget = args.owner;
		var actionEdit = widget.actions.filter(b => b.name === "edit")[0];
		if(actionEdit.isVisible == true){
			widget.actions.filter(a => a.name === "cadastramentoAutomatico" )[0].isVisible = false;
			widget.actions.filter(c => c.name === "Voltar")[0].isVisible = true;
		}
		widget.reload();
	};

	

	this.salvarCadastramentoAutomatico = function(args){
			var widget = args.owner.widget;
			widget.currentRow.__is_new = false;
			if(widget.name=='cadastramentoAutomaticoImpostoInscricaoEstadual'){
				if(widget.currentRow.TIPOCLIENTE !=='2'|| widget.currentRow.TIPOFORNECEDOR!=='2'){
					if(widget.isValid()){
						Ger10200_ParametrosGrupoFiscal_Service.salvarCadastramentoAutomatico(widget.currentRow).then(function(Success) {
					    ScreenService.closePopup();
						args.owner.widget.active = false;
						var gridFornecedor = templateManager.container.getWidget('tabbedFornecedorImpostoInscricaoEstadual');
						var gridCliente = templateManager.container.getWidget('tabbedClienteImpostoInscricaoEstadual');
						if(widget.currentRow.TIPOTELA=='FORNECEDOR'){
							gridFornecedor.active = true;
							gridFornecedor.reload();
							gridCliente.reload();
							gridFornecedor.activate();
							gridFornecedor.list();
							gridFornecedor.getField('CDIMPOSTO').isVisible = false;
							gridFornecedor.getField('NMIMPOSTO').isVisible = false;
							gridFornecedor.getField('NMEMPRESA').isVisible = false;
							gridFornecedor.getField('CDEMPRESA').isVisible = false;
							gridFornecedor.getField('CDINSCESTA').isVisible = false;
							gridFornecedor.getField('NRINSJURIEST').isVisible = false;
							gridFornecedor.reload();
							gridCliente.reload();
						}
						if(widget.currentRow.TIPOTELA=='CLIENTE'){
							gridCliente.active = true;
							gridCliente.reload();
							gridFornecedor.reload();
							gridCliente.activate();
							gridCliente.list();
							gridCliente.getField('CDIMPOSTO').isVisible = false;
							gridCliente.getField('NMIMPOSTO').isVisible = false;
							gridCliente.getField('NMEMPRESA').isVisible = false;
							gridCliente.getField('CDEMPRESA').isVisible = false;
							gridCliente.getField('CDINSCESTA').isVisible = false;
							gridCliente.getField('NRINSJURIEST').isVisible = false;
							gridCliente.reload();
							gridFornecedor.reload();
						}
						ScreenService.closePopup();
					},function(callback){
						if(callback.data){
							ScreenService.showMessage(callback.data.error);
						}
					});
				  }
				}else{
					ScreenService.showMessage('Nenhum Cliente e Nenhum Fornecedor Selecionado.');
				}
			}
			else{
				if(widget.name=="cadastramentoAutomaticoTipoContrato" || widget.name=="cadastramentoAutomaticoImpostoFilial"){
					widget.currentRow.CDAUTO="1";
				}
				General.saveCrud(widget).then(function(Success) {
					ScreenService.closePopup();
					args.owner.widget.active = false;
					args.owner.widget.getParent().active = true;
					widget.getParent().reload();
					widget.getParent().activate();
					ScreenService.closeSwipe();
					widget.list();
					widget.reload();
					widget.redraw();
			  },function(callback){
				if(callback.data){
						ScreenService.showMessage(callback.data.error);
					}
			  });
			}
	};
	
		this.verificarTipoCadastramento = function(args){
		var widget = args.owner.widget;
		var salvarCadastro = this.salvarCadastramentoAutomatico;
		if(widget.currentRow.TIPOCLIENTE =='0' && widget.currentRow.TIPOFORNECEDOR=='0'){
			ScreenService.confirmMessage("Atenção! O cadastramento automático será realizado em todos os clientes e todos os fornecedores. Confirma Operação?", "question", function(success){
				salvarCadastro(args);
			});
		}
		else if(widget.currentRow.TIPOCADASTRO =='2' && widget.currentRow.TIPOCLIENTE =='0' && widget.currentRow.TIPOFORNECEDOR=='0'){
			ScreenService.confirmMessage("Atenção! O cadastramento automático excluirá o grupo fiscal de todos os clientes e fornecedores. Confirma operação?", "question", function(success){
				salvarCadastro(args);
			});
		}
		else
			salvarCadastro(args);
	};
	
	
	this.salvarReplicacaoAliquotas = function(args){
			var widget = args.owner.widget;
			widget.currentRow.__is_new = false;
			if(widget.isValid()){
				ScreenService.confirmMessage("Atenção! Ao realizar a replicação todo cadastramento preexistente será apagado. Deseja confirmar ?", "question", function(success){
					Ger10200_ParametrosGrupoFiscal_Service.salvarReplicacaoAliquotas(widget.currentRow).then(function(Success) {
					ScreenService.closePopup();
					args.owner.widget.active = false;
					if(widget.currentRow.TIPOTELA=='FORNECEDOR'){
						var gridFornecedor = templateManager.container.getWidget('tabbedFornecedorImpostoInscricaoEstadual');
						gridFornecedor.active = true;
						gridFornecedor.reload();
						gridFornecedor.activate();
					}
					if(widget.currentRow.TIPOTELA=='CLIENTE'){
						var gridCliente = templateManager.container.getWidget('tabbedClienteImpostoInscricaoEstadual');
						gridCliente.active = true;
						gridCliente.reload();
						gridCliente.activate();
					}
						ScreenService.closePopup();
					},function(callback){
					if(callback.data){
							ScreenService.showMessage(callback.data.error);
						}
				});
				});


		  }
	};
	this.salvarTipoContrato = function(args){
		var widget = args.owner.widget;
		widget.currentRow.__is_new = false;
		widget.currentRow.CDAUTO="0";
		General.saveCrud(widget).then(function(Success) {
			args.owner.widget.getAction('cadastramentoAutomatico').isVisible = true;
	    },function(callback){
			if(callback.data){
				ScreenService.showMessage(callback.data.error);
			}
			//widget.reload();
	  });
	};
	this.salvarImpostoInscricaoEstadual = function(args){
		var widget = args.owner.widget;
		
		if(widget.isValid()){
			widget.currentRow.CDAUTO="0";
			widget.getField('CDIMPOSTO').isVisible = false;
			widget.getField('NMIMPOSTO').isVisible = false;
			widget.getField('NMEMPRESA').isVisible = false;
			widget.getField('CDEMPRESA').isVisible = false;
			widget.getField('CDINSCESTA').isVisible = false;
			widget.getField('NRINSJURIEST').isVisible = false;
			templateManager.container.getWidget('tabbedClienteImpostoInscricaoEstadual').activate();
			templateManager.container.getWidget('tabbedFornecedorImpostoInscricaoEstadual').activate();
			General.saveCrud(widget).then(function(Success) {
				args.owner.widget.getAction('cadastramentoAutomatico').isVisible = true;
				args.owner.widget.getAction('replicacaoAliquota').isVisible = true;
				args.owner.widget.getAction('adicionarRow').isVisible = true;
		    },function(callback){
	    		if(callback.data){
	    			ScreenService.showMessage(callback.data.error);
	    		}else{
	    			var str = callback;
	    			var error = str.replace("Error in data save. ", "");
	    			ScreenService.showMessage(error);
	    		}
	  
			    //widget.reload();
		    });
		}else{
			ScreenService.showMessage('Preencha todos os campos corretamente!');
		}
	};

    this.cancelarCadastramentoAutomatico = function(args){
		var widget = args.owner.widget;
		if(widget.name=='cadastramentoAutomaticoImpostoInscricaoEstadual'){
			var gridFornecedor = templateManager.container.getWidget('tabbedFornecedorImpostoInscricaoEstadual');
			var gridCliente = templateManager.container.getWidget('tabbedClienteImpostoInscricaoEstadual');
			if(widget.currentRow.TIPOTELA=='FORNECEDOR'){
				gridFornecedor.getField('CDIMPOSTO').isVisible = false;
				gridFornecedor.getField('NMIMPOSTO').isVisible = false;
				gridFornecedor.getField('NMEMPRESA').isVisible = false;
				gridFornecedor.getField('CDEMPRESA').isVisible = false;
				gridFornecedor.getField('CDINSCESTA').isVisible = false;
				gridFornecedor.getField('NRINSJURIEST').isVisible = false;
			}
			if(widget.currentRow.TIPOTELA=='CLIENTE'){
				gridCliente.active = true;
				gridCliente.reload();
				gridFornecedor.reload();
				gridCliente.activate();
				gridCliente.list();
				gridCliente.getField('CDIMPOSTO').isVisible = false;
				gridCliente.getField('NMIMPOSTO').isVisible = false;
				gridCliente.getField('NMEMPRESA').isVisible = false;
				gridCliente.getField('CDEMPRESA').isVisible = false;
				gridCliente.getField('CDINSCESTA').isVisible = false;
				gridCliente.getField('NRINSJURIEST').isVisible = false;
			}
		}
		ScreenService.closePopup();
		args.owner.widget.active = false;
		args.owner.widget.getParent().active = true;
		
		//args.owner.widget.getParent().reload();
		args.owner.widget.getParent().activate();
		//ScreenService.closeSwipe();

	};
	this.salvarImpostoFilial = function(args){
		var widget = args.owner.widget;
		if(widget.isValid()){
			widget.currentRow.CDAUTO="0";
			General.saveCrud(widget).then(function(Success) {
			    args.owner.widget.getAction('cadastramentoAutomatico').isVisible = true;
			},function(callback){
				if(callback.data){
					ScreenService.showMessage(callback.data.error);
				}
				//widget.reload();
		    });
		}
	};
	
	this.salvarOperacaoInterestadual = function(args){
			var widget = args.owner.widget;
			var dataSource = args.owner.widget.dataSource.name;
			var dataSourceGeral = dataSource.substr(4,(dataSource.indexOf('_')-4) ) ;
			widget.currentRow.DSPARGERFIS = Ger10200_ParametrosGrupoFiscal_Service.concatenaCampo(widget.currentRow.CDGRUPOFISC, widget.currentRow.DSPARGERFIS );
			widget.currentRow.__is_new = false;
			widget.dataSource.name= "df#/crud/"+dataSourceGeral;
			General.saveCrud(widget).then(function(Success) {
				widget.dataSource.name=dataSource;
				widget.template = widget.templateToView;
					widget.view();
					widget.activate();
				widget.reload();
		  },function(callback){
				if(callback.data){
						ScreenService.showMessage(callback.data.error);
					}
				widget.dataSource.name=dataSource;
				widget.reload();
		  });
	};
	this.fieldOnChangeImpostoFilial = function(){
		var gridImpostoFilial = templateManager.container.getWidget('gridImpostoFilial');
		var tabbedImpostoFilial = templateManager.container.getWidget('tabbedImpostoFilial');
		if(!tabbedImpostoFilial.getField('NMIMPOSTO').value()){
			gridImpostoFilial.getAction('adicionarRow').isVisible = false;
		}else{
			gridImpostoFilial.getAction('adicionarRow').isVisible = true;

		}
		gridImpostoFilial.reload();
	};
	
	this.fieldOnChangeCampoEmpresaImpostoInscricaoEstadual = function(args){
		var widget  = args.owner.owner.widget;
			if(widget.getField('NMEMPRESA').value()){
					if(widget.name=='tabbedImpostoInscricaoEstadual'){
						widget.getField('CDINSCESTA').value('');
						widget.getField('CDINSCESTA').readOnly=false;
						widget.currentRow.CDINSCESTA = '';
					}
					else {
						widget.getField('NRINSJURIEST').value('');
						widget.getField('NRINSJURIEST').readOnly=false;
						widget.currentRow.NRINSJURIEST = '';
					}
			}
			else{
				if(widget.name=='tabbedImpostoInscricaoEstadual'){
						widget.getField('CDINSCESTA').value('');
						widget.getField('CDINSCESTA').readOnly=true;
						widget.currentRow.CDINSCESTA = '';
					}
					else {
						widget.getField('NRINSJURIEST').value('');
						widget.getField('NRINSJURIEST').readOnly=true;
						widget.currentRow.NRINSJURIEST = '';
					}
			}
			if(widget.name=='tabbedImpostoInscricaoEstadual'){
				widget.getField('CDINSCESTA').reload();
			}
			else {
				widget.getField('NRINSJURIEST').reload();
			}
	};
	this.fieldOnChangeCampoEmpresaDestino = function(args){
		var widget  = args.owner.owner.widget;
			if(!widget.getField('NMEMPRESADESTINO').value()){
				widget.getField('NMREPRINSESTDESTINO').value('');
				//widget.getField('CDINSCESTADESTINO').value('');
				widget.getField('NMREPRINSESTDESTINO').readOnly=true;
			}
			else{
				widget.getField('NMREPRINSESTDESTINO').value('');
				//widget.getField('CDINSCESTADESTINO').value('');
				widget.getField('NMREPRINSESTDESTINO').readOnly=false;
			}
			widget.getField('NMREPRINSESTDESTINO').reload();


	};
	this.recarregarWidgetImpostoIsncricaoEstadual = function(){
		var tabbedImpostoInscricaoEstadual = templateManager.container.getWidget('tabbedImpostoInscricaoEstadual');
		var tabbedFornecedorImpostoInscricaoEstadual = templateManager.container.getWidget('tabbedFornecedorImpostoInscricaoEstadual');
		var tabbedClienteImpostoInscricaoEstadual = templateManager.container.getWidget('tabbedClienteImpostoInscricaoEstadual');
		var currentRow = tabbedImpostoInscricaoEstadual.currentRow;
		if(currentRow.NMIMPOSTO && currentRow.NMEMPRESA && currentRow.NMREPRINSEST){
			tabbedFornecedorImpostoInscricaoEstadual.getAction('adicionarRow').isVisible = true;
			tabbedClienteImpostoInscricaoEstadual.getAction('adicionarRow').isVisible = true;
			tabbedFornecedorImpostoInscricaoEstadual.reload();
			tabbedClienteImpostoInscricaoEstadual.reload();
		}else{
			tabbedClienteImpostoInscricaoEstadual.reload();
			tabbedFornecedorImpostoInscricaoEstadual.reload();

		}
	};
	
	this.grupoFiscalTabClienteOnChange = function(args) {
		var widget = args.owner.field.widget;
		var CDGrupoFiscal = widget.getField("CDGRUPOFISC").value();
		var NMRAZSocial = widget.getField("NMRAZSOCCLIE");
		var NRInscClie = widget.getField("NRINSJURCLIE");
		NMRAZSocial.setValue("");
		NRInscClie.setValue("");
		
		if (CDGrupoFiscal != null && CDGrupoFiscal != '' ) {
			NMRAZSocial.readOnly = false;
			NMRAZSocial.readOnlyOnEdit = false;
			NMRAZSocial.dataSourceFilter.forEach(row => {
				if(row.name === "CDGRUPOFISC") {
					row.value = CDGrupoFiscal;
					row.operator = "=";
				}
			} );
		}
		else {
			NMRAZSocial.readOnly = true;
			NMRAZSocial.readOnlyOnEdit = true;
		}
	};
	
	this.grupoFiscalTabUnidadeOnChange = function(args) {
		var widget = args.owner.field.widget;
		var CDGrupoFiscal = widget.getField("CDGRUPOFISC").value();
		var cdf = widget.getField("CDFILIAL");
		var und = widget.getField("NMFILIAL");
		und.setValue("");
		cdf.setValue("");

		if (CDGrupoFiscal != null && CDGrupoFiscal != '' ) {
			und.readOnly = false;
			und.dataSourceFilter.forEach(row => {
				if(row.name === "CDGRUPOFISC") {
					row.value = CDGrupoFiscal;
					row.operator = "=";
				}
			} );
		}
		else {
			und.readOnly = true;
		}
	};
	
	this.grupoFiscalTabTipoOpOnChange = function(args) {
		var widget = args.owner.field.widget;
		var CDGrupoFiscal = widget.getField("CDGRUPOFISC").value();
		var nome = widget.getField("NMTIPOOPER");
		var cod = widget.getField("CDTIPOOPER");
		var es = widget.getField("IDENTRSAIDOP");
		
		nome.setValue("");
		cod.setValue("");
		es.setValue("");

		if (CDGrupoFiscal != null && CDGrupoFiscal != '' ) {
			nome.readOnly = false;
			nome.dataSourceFilter.forEach(row => {
				if(row.name === "CDGRUPOFISC") {
					row.value = CDGrupoFiscal;
					row.operator = "=";
				}
			} );
		}
		else {
			nome.readOnly = true;
		}
	};

	this.editar = function(widget){
		widget.template = widget.templateToForm;
		General.edit(widget);
	};
	this.cancelar = function(widget){
		for (var i in widget.backupRow) {
			widget.currentRow[i] = widget.backupRow[i];
		}
		widget.template = widget.templateToView;
		widget.view();
		widget.activate();
	};
	this.criaCurrentRowTipoOperacao= function(args){
		var widget = args.owner.owner.widget;
		var tamanhoCheckedRows = args.owner.field.dataSource.checkedRows.length;
		var rowsChecadas =args.owner.field.dataSource.checkedRows;
		var array =[];
		for(var i = 0; i<tamanhoCheckedRows;i++){
				var tipoOperacao ={CDTIPOOPER:null, IDENTRSAIDOP:null};
				tipoOperacao.CDTIPOOPER = rowsChecadas[i].CDTIPOOPER;
				tipoOperacao.IDENTRSAIDOP = rowsChecadas[i].IDENTRSAIDOP;
				array[i] = tipoOperacao;
		}
		widget.currentRow.tipoOperacao= array;
	};
	this.criaCurrentRowInscricaoEstadual= function(args){
		var widget = args.owner.owner.widget;
		var tamanhoCheckedRows = args.owner.field.dataSource.checkedRows.length;
		var rowsChecadas =args.owner.field.dataSource.checkedRows;
		var array =[];
		for(var i = 0; i<tamanhoCheckedRows;i++){
				var inscricaoEstadual ={CDEMPRESA:null, CDINSCESTA:null};
				inscricaoEstadual.CDEMPRESA = rowsChecadas[i].CDEMPRESA;
				inscricaoEstadual.CDINSCESTA = rowsChecadas[i].CDINSCESTA;
				array[i] = inscricaoEstadual;
		}
		widget.currentRow.inscricaoEstadual= array;
	};
	this.WidgetOnEnter = function(args){
		args.owner.currentRow = args.owner.dataSource.data[0];
	};
	
	this.verificaWidgetAnterior = function(args){
		var widget = args.owner;
		var wdgAnterior = localStorage.getItem("wdgAnterior"); 
		var voltarAction;
		if(wdgAnterior===null){
			widget.widgets.forEach (widget => {
        	    voltarAction = widget.actions.filter(b => b.name === "Voltar" )[0];
        	    if(voltarAction) voltarAction.isVisible = false;
	        	    if(widget.name === "tabbedImpostoFilial" || widget.name === "tabbedImpostoInscricaoEstadual"){
	        	    	widget.widgets.forEach (widget => {
	        	    		voltarAction = widget.actions.filter(b => b.name === "Voltar" )[0];
	        	    		if(voltarAction) voltarAction.isVisible = false;
	        			 });
	        	    }
        		});
		}
		else{
			widget.reload();
			localStorage.removeItem("wdgAnterior");
		}
	};
	
	this.validacaoCamposCadastramentoAutmatico = function(args){
		var gridCadastramentoAutomatico = args.owner.owner.widget;
		var nome = args.owner.owner.widget.name;
		nome = nome.substr(23);
		var tipoCadastramento = gridCadastramentoAutomatico.getField('TIPOCADASTRO').value();
		var grupoFiscal = gridCadastramentoAutomatico.getField('DSGRUPOFISC');
		var  nomeCadastro= gridCadastramentoAutomatico.getField(nome);
		switch(tipoCadastramento){
			case "0":
				nomeCadastro.isVisible = false;
				nomeCadastro.value('');
				delete(gridCadastramentoAutomatico.getField(nome).validations.required );
				gridCadastramentoAutomatico.getField('DSGRUPOFISC').validations.required =true;
				grupoFiscal.readOnly = false;
				break;
			case "1":
				nomeCadastro.isVisible=true;
				nomeCadastro.reload();
				gridCadastramentoAutomatico.getField(nome).validations.required =true;
				grupoFiscal.readOnly =false;
				gridCadastramentoAutomatico.getField('DSGRUPOFISC').validations.required =true;
				break;
			case "2":
				nomeCadastro.isVisible=false;
				nomeCadastro.value('');
				delete(gridCadastramentoAutomatico.getField(nome).validations.required );
				grupoFiscal.isVisible =false;
				delete(gridCadastramentoAutomatico.getField('DSGRUPOFISC').validations.required );
				grupoFiscal.value('');
				break;
			case "3":
				nomeCadastro.isVisible=true;
				nomeCadastro.reload();
				gridCadastramentoAutomatico.getField(nome).validations.required =true;
				grupoFiscal.readOnly =true;
				delete(gridCadastramentoAutomatico.getField('DSGRUPOFISC').validations.required );
				grupoFiscal.value('');
				break;
			case "4":
				nomeCadastro.isVisible=false;
				nomeCadastro.value('');
				delete(gridCadastramentoAutomatico.getField(nome).validations.required );
				grupoFiscal.readOnly =false;
				gridCadastramentoAutomatico.getField('DSGRUPOFISC').validations.required =true;
				break;
			default: break;
		}
	};
	this.ActionCadastramentoAutomaticoHabilitada = function(args){
		var widget = args.owner.widget;
		widget.getAction('cadastramentoAutomatico').isVisible = true;
		if(widget.name=='tabbedFornecedorImpostoInscricaoEstadual' || widget.name=='tabbedClienteImpostoInscricaoEstadual'){
			widget.getAction('replicacaoAliquota').isVisible = true;
			widget.getField('CDIMPOSTO').isVisible = false;
			widget.getField('NMIMPOSTO').isVisible = false;
			widget.getField('NMEMPRESA').isVisible = false;
			widget.getField('CDEMPRESA').isVisible = false;
			widget.getField('CDINSCESTA').isVisible = false;
			widget.getField('NRINSJURIEST').isVisible = false;
		}
	};
	this.ActionCadastramentoAutomaticoDesabilitada = function(args){
		var widget = args.owner.widget;
		widget.getAction('cadastramentoAutomatico').isVisible = false;
		if(widget.name=='tabbedFornecedorImpostoInscricaoEstadual' || widget.name=='tabbedClienteImpostoInscricaoEstadual'){
			widget.getAction('replicacaoAliquota').isVisible = false;
		}

	};
	this.validacaoCamposCadastramentoAutmaticoTipoContrato = function(args){
		var gridCadastramentoAutomatico = args.owner.owner.widget;
		var nome = args.owner.owner.widget.name;
		nome = nome.substr(23);
		var tipoCadastramento = gridCadastramentoAutomatico.getField('TIPOCADASTRO').value();
		var grupoFiscal = gridCadastramentoAutomatico.getField('DSGRUPOFISC');
		var  nomeCadastro= gridCadastramentoAutomatico.getField(nome);
		switch(tipoCadastramento){
			case "0":
				gridCadastramentoAutomatico.getField('DSGRUPOFISC').validations.required =true;
				grupoFiscal.setValue(null);
				grupoFiscal.readOnly = false;
				grupoFiscal.reload();
				break;
			case "1":
				grupoFiscal.readOnly =false;
				delete(gridCadastramentoAutomatico.getField('DSGRUPOFISC').validations.required );
				grupoFiscal.setValue(null);
				grupoFiscal.reload();
				break;
			case "2":
				gridCadastramentoAutomatico.getField('DSGRUPOFISC').validations.required =true;
				grupoFiscal.setValue(null);
				grupoFiscal.readOnly = false;
				grupoFiscal.reload();
				break;
			default: break;
		}
	};
	this.verificaTipoCadastramentoAutomaticoParaInscricao = function(args){
		var widget = args.owner.owner.widget;
		var tipoCadastroInscricao = widget.getField('TIPOINSCRICAO').value();
		var listaInscricao;
		if(widget.name=='replicacaoAliquotas'){
			listaInscricao = widget.getField('NMREPRINSESTDESTINO');
		}else{
			listaInscricao = widget.getField('InscricaoEstadual');
		}

		switch(tipoCadastroInscricao){
			case'0':
				listaInscricao.value('');
				listaInscricao.isVisible = false;
				delete(listaInscricao.validations.required);
				break;
			case'1':
				listaInscricao.isVisible = true;
				listaInscricao.reload();
				listaInscricao.validations.required = true;
				break;
			case  null:
				listaInscricao.value('');
				listaInscricao.isVisible = false;
				delete(listaInscricao.validations.required);
				break;
			default: break;
		}
	};
	this.verificaTipoCadastramentoAutomaticoParaGrupoFiscal = function(){
		var gridCadastramentoAutomatico = templateManager.container.getWidget('replicacaoAliquotas');
		var tipoCadastroGrupo = gridCadastramentoAutomatico.getField('TIPOGRUPO').value();
		var listaGrupo = gridCadastramentoAutomatico.getField('GrupoFiscal');
		switch(tipoCadastroGrupo){
			case'0':
				listaGrupo.value('');
				listaGrupo.isVisible = false;
				delete(listaGrupo.validations.required);
				break;
			case'1':
				listaGrupo.isVisible = true;
				listaGrupo.validations.required = true;
				break;
			case  null:
				listaGrupo.value('');
				listaGrupo.isVisible = false;
				delete(listaGrupo.validations.required);
				break;
			default: break;
		}
	};
	this.verificaTipoCadastramentoAutomaticoParaClientes = function(){
		var gridCadastramentoAutomatico = templateManager.container.getWidget('cadastramentoAutomaticoImpostoInscricaoEstadual');
		var tipoCadastroCliente = gridCadastramentoAutomatico.getField('TIPOCLIENTE').value();
		var listaCliente = gridCadastramentoAutomatico.getField('Clientes');
		switch(tipoCadastroCliente){
			case'0':
				listaCliente.value('');
				listaCliente.isVisible = false;
				delete(listaCliente.validations.required);
				break;
			case'1':
				listaCliente.isVisible = true;
				listaCliente.reload();
				listaCliente.validations.required = true;
				break;
			case'2':
				listaCliente.value('');
				listaCliente.isVisible = false;
				delete(listaCliente.validations.required);
				break;
				case null:
				listaCliente.value('');
				listaCliente.isVisible = false;
				delete(listaCliente.validations.required );
				break;
			default: break;
		}
	};
	this.verificaTipoCadastramentoAutomaticoParaFornecedor = function(){
		var gridCadastramentoAutomatico = templateManager.container.getWidget('cadastramentoAutomaticoImpostoInscricaoEstadual');
		var tipoCadastroFornecedor = gridCadastramentoAutomatico.getField('TIPOFORNECEDOR').value();
		var listaFornecedor = gridCadastramentoAutomatico.getField('Fornecedores');
		switch(tipoCadastroFornecedor){
			case'0':
				listaFornecedor.value('');
				listaFornecedor.isVisible = false;
				delete(listaFornecedor.validations.required );
				break;
			case'1':
				listaFornecedor.isVisible = true;
				listaFornecedor.reload();
				listaFornecedor.validations.required = true;
				break;
			case'2':
				listaFornecedor.value('');
				listaFornecedor.isVisible = false;
				delete(listaFornecedor.validations.required);
				break;
			case null:
				listaFornecedor.value('');
				listaFornecedor.isVisible = false;
				delete(listaFornecedor.validations.required );
				break;
			default: break;
		}
	};
	this.verificatipoCadastroAutomaticoImpostoInscricaoEstadual = function(){
		var gridCadastramentoAutomatico = templateManager.container.getWidget('cadastramentoAutomaticoImpostoInscricaoEstadual');
		var tipoCadastro = gridCadastramentoAutomatico.getField('TIPOCADASTRO').value();
		var grupoFiscal = gridCadastramentoAutomatico.getField('DSGRUPOFISC');
		var tipoCadastroCliente = gridCadastramentoAutomatico.getField('TIPOCLIENTE');
		var tipoCadastroFornecedor = gridCadastramentoAutomatico.getField('TIPOFORNECEDOR');
		var lista ={value:'1', description:'Lista'};
		var listaFornecedor = gridCadastramentoAutomatico.getField('Fornecedores');
		var listaCliente = gridCadastramentoAutomatico.getField('Clientes');

		switch(tipoCadastro){
			case'0':
				grupoFiscal.validations.required = true;
				grupoFiscal.isVisible = true;
				grupoFiscal.readOnly = false;
				grupoFiscal.value('');
				grupoFiscal.reload();
				if(tipoCadastroCliente.dataSource.data.length==2){
						tipoCadastroCliente.dataSource.data.push(lista);
				}
				if(tipoCadastroFornecedor.dataSource.data.length==2){
					tipoCadastroFornecedor.dataSource.data.push(lista);
				}
				break;
			case'1':
				delete(grupoFiscal.validations.required);
				grupoFiscal.isVisible = false;
				grupoFiscal.value('');
				grupoFiscal.reload();
				if(tipoCadastroCliente.dataSource.data.length==2){
						tipoCadastroCliente.dataSource.data.push(lista);
				}
				if(tipoCadastroFornecedor.dataSource.data.length==2){
					tipoCadastroFornecedor.dataSource.data.push(lista);
				}
				break;
			case'2':
				grupoFiscal.validations.required = true;
				grupoFiscal.readOnly = false;
				grupoFiscal.isVisible = true;
				grupoFiscal.value('');
				grupoFiscal.reload();
				if(tipoCadastroCliente.dataSource.data.length==3){
					tipoCadastroCliente.dataSource.data.pop();
					listaCliente.isVisible=false;
					listaCliente.value('');
				}
				if(tipoCadastroFornecedor.dataSource.data.length==3){
						tipoCadastroFornecedor.dataSource.data.pop();
						listaFornecedor.isVisible=false;
						listaFornecedor.value('');
				}

				break;
			default: break;
		}
	};
	this.tipoWidgetFornecedor = function(){
		var gridCadastramentoAutomatico = templateManager.container.getWidget('cadastramentoAutomaticoImpostoInscricaoEstadual');
		gridCadastramentoAutomatico.currentRow.TIPOTELA='FORNECEDOR';
	};
	this.tipoWidgetCliente = function(){
		var gridCadastramentoAutomatico = templateManager.container.getWidget('cadastramentoAutomaticoImpostoInscricaoEstadual');
		gridCadastramentoAutomatico.currentRow.TIPOTELA='CLIENTE';
	};
	this.tipoWidgetFornecedorReplicacaoAliquota = function(){
		var gridReplicacaoAliquota = templateManager.container.getWidget('replicacaoAliquotas');
		gridReplicacaoAliquota.currentRow.TIPOTELA='FORNECEDOR';
	};
	this.tipoWidgetClienteReplicacaoAliquota = function(){
		var gridReplicacaoAliquota = templateManager.container.getWidget('replicacaoAliquotas');
		gridReplicacaoAliquota.currentRow.TIPOTELA='CLIENTE';
	};

	/*this.validaCadastramentoAutomatico = function(widget){
		
		if (Array.isArray(widget.dataSource.data)) {
			if (widget.dataSource.data.length > 0) {
				widget.getAction('cadastramentoAutomatico').isVisible = true;
			} else {
				widget.getAction('cadastramentoAutomatico').isVisible = false;
			}
		}else{
			widget.getAction('cadastramentoAutomatico').isVisible = false;
		}
		
	};*/
	
	this.apagarGrupoFiscal = function(args){
		var widget = args.owner.widget;
		var dataSource = widget.dataSource.name;
		var dataSourceGeral = dataSource.substr(4,(dataSource.indexOf('_')-4) ) ;
		ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
			widget.currentRow.CDGRUPOFISC=null;
			widget.dataSource.name= "df#/crud/"+dataSourceGeral;
			widget.saveRow().then(function () {
				widget.dataSource.name=dataSource;
				ScreenService.showMessage('Deletado com Sucesso.');
				ScreenService.closeSwipe().then(function () {
					widget.activate();
					widget.reload();
				});
			}, function (error) {
				widget.dataSource.name=dataSource;
				if(error){
					ScreenService.showMessage(error);
				}
			});
		});
    };
    this.apagarGrupoFiscalFilialImposto = function(args){
        /*var widget = args.owner.widget;
        var dataSource = widget.dataSource.name;
        var dataSourceGeral = dataSource.substr(4,(dataSource.indexOf('_')-4) ) ;
        widget.dataSource.name= "df#/crud/"+dataSourceGeral;
        General.deleteCrud(widget).then(function () {
            widget.dataSource.name=dataSource;
            widget.reload();
        }, function (error) {
            widget.dataSource.name=dataSource;
            if(callback.data){
                ScreenService.showMessage(callback.data.error);
            }
        });
        */

	};
	
	this.reload = function(name){
		if(name == 'tabbedImpostoFilial'){
			templateManager.container.getWidget('gridImpostoFilial').reload();
		}
		if(name == 'tabbedImpostoInscricaoEstadual'){
			var widget = templateManager.container.getWidget('tabbedFornecedorImpostoInscricaoEstadual');
			var widgetCliente = templateManager.container.getWidget('tabbedClienteImpostoInscricaoEstadual');
			widget.reload();
			widgetCliente.reload();
		}
	};
	
	this.setValueInscEstadual = function(name){
		var widget;
		if(name == 'tabbedFornecedorImpostoInscricaoEstadual'){
			widget = templateManager.container.getWidget('tabbedFornecedorImpostoInscricaoEstadual');
		}else if(name == 'tabbedClienteImpostoInscricaoEstadual'){
			widget = templateManager.container.getWidget('tabbedClienteImpostoInscricaoEstadual');
		}
		widget.getField('CDIMPOSTO').isVisible = true;
		widget.getField('NMIMPOSTO').isVisible = true;
		widget.getField('NMEMPRESA').isVisible = true;
		widget.getField('CDEMPRESA').isVisible = true;
		widget.getField('CDINSCESTA').isVisible = true;
		widget.getField('NRINSJURIEST').isVisible = true;
	};
	
	this.changeFieldInsc = function(args){
		var widget = args.owner.field.widget;
		var emp = widget.getField('NMEMPRESA').value();
		if(emp === ''){
			widget.getField('NRINSJURIEST').setValue('');
			widget.getField('CDINSCESTA').setValue('');
			widget.getField('CDINSCESTA').reload();
			widget.getField('NRINSJURIEST').reload();
		}
		else
		widget.getField('NRINSJURIEST').reload();
	};
	
	this.fixFiltroFilial = function(widget) {
		for (let key in widget.dataSourceFilter) {
			if (widget.dataSourceFilter[key].name == "CDFILIAL") {
				delete widget.dataSourceFilter[key];
				widget.dataSourceFilter.length--;
			}
		}
		if (widget.appliedFilter) {
			var filialFiltro = widget.appliedFilter.FILTER_ROW.CDFILIAL;
			if (filialFiltro) {
				var filterFilial = {name: "CDFILIAL",
								operator: "=",
								value: filialFiltro};
				widget.dataSourceFilter.push(filterFilial);
			}
			widget.reload();
		}
	};

};
Configuration(function (ContextRegister){
	ContextRegister.register('Ger10200_ParametrosGrupoFiscal_Controller', Ger10200_ParametrosGrupoFiscal_Controller);
});

// FILE: ./assets/js/controllers/Ger16200_ConsistenciasFiscais_Controller.js
var Ger16200_ConsistenciasFiscais_Controller = function(Ger10200_ParametrosGrupoFiscal_Service,ScreenService, templateManager,General, ZhFilterPreferences){

	this.controlaMensagemCampoStripe = function (widget, preference){
		var filterRow = preference.FILTER_ROW;
		var NMIMPOSTO;
		
		NMIMPOSTO = filterRow.CDIMPOSTO;
		NMIMPOSTO = NMIMPOSTO.concat(' - ');
		NMIMPOSTO = NMIMPOSTO.concat(filterRow.NMIMPOSTO);
		widget.getParent().getWidget('stripe').currentRow.NMIMPOSTO = NMIMPOSTO;
		widget.activate();
	};
	
	this.openFilter = function(widget, defaultFilter) {
        ZhFilterPreferences.openFilterWidget(widget, defaultFilter).then(response => {
            var filter = templateManager.container.getActiveWidget();
            var clearFilter = filter.actions.filter(action => action.name === 'clear')[0];
            var confirmFilter = filter.actions.filter(action => action.name === 'never')[0];
            
            clearFilter.shortcutKey = 'ctrl+backspace';
            clearFilter.tooltip = 'Limpar Filtro (CTRL + Backspace)';
            
            confirmFilter.shortcutKey = 'ctrl + enter';
            filter.reload();
            
        });
    };

};
Configuration(function (ContextRegister){
	ContextRegister.register('Ger16200_ConsistenciasFiscais_Controller', Ger16200_ConsistenciasFiscais_Controller);
});

// FILE: ./assets/js/controllers/Ger33500_AgenteDeTerceirizacao_Controller.js
function Ger33500_AgenteDeTerceirizacao_Controller(Ger33500_AgenteDeTerceirizacao_Service, General, templateManager, ScreenService, MessageService) {

    this.buscaFilialLogada = function(widget) {
        this.buscaFilial().then(function(response_filial) {
            var filial = response_filial.dataset.filial[0];
            widget.getParent().getField('CDFILIAL').setValue(filial.CDFILIAL);
            widget.getParent().getField('NMFILIAL').setValue(filial.NMFILIAL);
            widget.getParent().redraw();
            this.fieldOnChangeFilial(widget.getParent());
        }.bind(this));
    };

    this.buscaFilial = function() {
        return Ger33500_AgenteDeTerceirizacao_Service.buscaFilial().then(function(filial) {
            return { dataset: { filial: filial } };
        });
    };
    this.fieldOnChangeRazao = function(widget) {
        var inscricao;
        if (widget.name === 'FORNECEDOR') {
            inscricao = General.inscricaoFormat(widget.currentRow.NRINSJURFORN);
            widget.currentRow.NRINSJURFORN = inscricao.value;
        }
        else if (widget.name === 'CLIENTE') {
            inscricao = General.inscricaoFormat(widget.currentRow.NRINSJURCLIE);
            widget.currentRow.NRINSJURCLIE = inscricao.value;


        }
    };

    this.fieldOnChangeFilial = function(widget) {
        if (widget.currentRow.CDFILIAL) {
            var cdFilial = widget.currentRow.CDFILIAL;
            widget.redraw();
            widget.reload();
            Ger33500_AgenteDeTerceirizacao_Service.verificaControle(cdFilial).then(function(response) {
                if (response.dataset.paramfilial[0].IDUTILALMOX === 'N') {
                    MessageService.showMessageByCode('UNIDADE_NAO_CONTROLA_ALMOXARIFADO').then(() => {
                        widget.currentRow.CDFILIAL = '';
                        widget.currentRow.NMFILIAL = '';
                    });
                }
                var fornecedor = templateManager.container.getWidget('FORNECEDOR');
                var cliente = templateManager.container.getWidget('CLIENTE');
                cliente.currentRow.CDFILIAL = fornecedor.currentRow.CDFILIAL = widget.currentRow.CDFILIAL;

                fornecedor.reload();
                cliente.reload();
            });
        }
        else {
            templateManager.container.getWidget('FORNECEDOR').reload();
            templateManager.container.getWidget('CLIENTE').reload();
        }
    };

    this.maskInscricao = function(widget) {
        var dataSource = widget.dataSource;
        var i = 0;
        dataSource.forEach(function(row) {
            var inscricao;
            if (widget.owner.name === 'FORNECEDOR') {
                inscricao = General.inscricaoFormat(row.NRINSJURFORN);
                widget.dataSource[i].NRINSJURFORN = inscricao.value;
                widget.dataSource[i].NRINSJURFORNSMASK = inscricao.parameter;
            }
            else if (widget.owner.name === 'CLIENTE') {
                inscricao = General.inscricaoFormat(row.NRINSJURCLIE);
                widget.dataSource[i].NRINSJURCLIE = inscricao.value;
                widget.dataSource[i].NRINSJURCLIENSMASK = inscricao.parameter;
            }
            i++;
        });
    };

    this.fieldOnChangeAlmoxarifado = function(widget, field) {
        var cdFilial = widget.getParent().currentRow.CDFILIAL;
        Ger33500_AgenteDeTerceirizacao_Service.verificaControle(cdFilial).then(function(response) {
            if (response.dataset.paramfilial[0].IDUTILLCESTQ === 'S') {
                if (widget.name === 'FORNECEDOR') {
                    if (field.name === 'DSALMOXARIFE') {
                        widget.getField('DSLOCALESTOQ').readOnly = false;
                    }
                    else if (field.name === 'DSALMOXTERCFOR') {
                        widget.getField('DSLOCALTERCFOR').readOnly = false;
                    }
                    else if (field.name === 'DSALMOXINFOFOR') {
                        widget.getField('DSLOCALINFOFOR').readOnly = false;
                    }
                }
                else if (widget.name === 'CLIENTE') {
                    if (field.name === 'DSALMOXARIFE') {
                        widget.getField('DSLOCALESTOQ').readOnly = false;
                    }
                    else if (field.name === 'DSALMOXTERCCLI') {
                        widget.getField('DSLOCALTERCCLI').readOnly = false;
                    }
                    else if (field.name === 'DSALMOXINFOCLI') {
                        widget.getField('DSLOCALINFOCLI').readOnly = false;
                    }
                }
            }

        });
        if (widget.name === 'FORNECEDOR') {
            if (field.value()) {
                if (field.name === 'DSALMOXARIFE') {
                    widget.getField('DSALMOXARIFE').validations = {
                        "required": {
                            "message": "Campo Obrigatorio"
                        }
                    };
                    widget.getField('DSALMOXINFOFOR').validations = {};
                    widget.getField('DSALMOXTERCFOR').validations = {
                        "required": {
                            "message": "Campo Obrigatorio"
                        }
                    };
                }
                else if (field.name === 'DSALMOXINFOFOR') {
                    if (!widget.getField('DSALMOXARIFE').value()) {
                        widget.getField('DSALMOXARIFE').validations = {};
                        widget.getField('DSALMOXTERCFOR').validations = {};
                        widget.getField('DSALMOXINFOFOR').validations = {
                            "required": {
                                "message": "Campo Obrigatorio"
                            }
                        };
                    }
                }
            }
            else {
                if ((field.name === 'DSALMOXARIFE' && !widget.currentRow.DSALMOXINFOFOR) ||
                    (field.name === 'DSALMOXINFOFOR' && !widget.currentRow.DSALMOXARIFE)) {
                    widget.getField('DSALMOXARIFE').validations = {
                        "required": {
                            "message": "Campo Obrigatorio"
                        }
                    };
                    widget.getField('DSALMOXTERCFOR').validations = {};
                    widget.getField('DSALMOXINFOFOR').validations = {
                        "required": {
                            "message": "Campo Obrigatorio"
                        }
                    };
                }
                else if (!(widget.getField('DSALMOXARIFE').value()) && widget.getField('DSALMOXINFOFOR').value()) {
                    widget.getField('DSALMOXARIFE').validations = {};
                    widget.getField('DSALMOXTERCFOR').validations = {};
                    widget.getField('DSALMOXINFOFOR').validations = {
                        "required": {
                            "message": "Campo Obrigatorio"
                        }
                    };

                }
                else if (!(widget.getField('DSALMOXINFOFOR').value()) && widget.getField('DSALMOXARIFE').value()) {
                    widget.getField('DSALMOXARIFE').validations = {
                        "required": {
                            "message": "Campo Obrigatorio"
                        }
                    };
                    widget.getField('DSALMOXINFOFOR').validations = {};
                    widget.getField('DSALMOXTERCFOR').validations = {
                        "required": {
                            "message": "Campo Obrigatorio"
                        }
                    };

                }
            }
        }
        else if (widget.name === 'CLIENTE') {
            if (field.value()) {
                if (field.name === 'DSALMOXARIFE') {
                    widget.getField('DSALMOXARIFE').validations = {
                        "required": {
                            "message": "Campo Obrigatorio"
                        }
                    };
                    widget.getField('DSALMOXINFOCLI').validations = {};
                    widget.getField('DSALMOXTERCCLI').validations = {
                        "required": {
                            "message": "Campo Obrigatorio"
                        }
                    };
                }
                else if (field.name === 'DSALMOXINFOCLI') {
                    if (!widget.getField('DSALMOXARIFE').value()) {
                        widget.getField('DSALMOXARIFE').validations = {};
                        widget.getField('DSALMOXTERCCLI').validations = {};
                        widget.getField('DSALMOXINFOCLI').validations = {
                            "required": {
                                "message": "Campo Obrigatorio"
                            }
                        };
                    }
                }
            }
            else {
                if ((field.name === 'DSALMOXARIFE' && !widget.currentRow.DSALMOXINFOCLI) ||
                    (field.name === 'DSALMOXINFOCLI' && !widget.currentRow.DSALMOXARIFE)) {

                    widget.getField('DSALMOXARIFE').validations = {
                        "required": {
                            "message": "Campo Obrigatorio"
                        }
                    };
                    widget.getField('DSALMOXTERCCLI').validations = {};
                    widget.getField('DSALMOXINFOCLI').validations = {
                        "required": {
                            "message": "Campo Obrigatorio"
                        }
                    };
                }
                else if (!(widget.getField('DSALMOXARIFE').value()) && widget.getField('DSALMOXINFOCLI').value()) {
                    widget.getField('DSALMOXARIFE').validations = {};
                    widget.getField('DSALMOXTERCCLI').validations = {};
                    widget.getField('DSALMOXINFOCLI').validations = {
                        "required": {
                            "message": "Campo Obrigatorio"
                        }
                    };

                }
                else if (!(widget.getField('DSALMOXINFOCLI').value()) && widget.getField('DSALMOXARIFE').value()) {
                    widget.getField('DSALMOXARIFE').validations = {
                        "required": {
                            "message": "Campo Obrigatorio"
                        }
                    };
                    widget.getField('DSALMOXINFOCLI').validations = {};
                    widget.getField('DSALMOXTERCCLI').validations = {
                        "required": {
                            "message": "Campo Obrigatorio"
                        }
                    };

                }
            }


        }
        widget.reload();

    };

    this.new = function(widget) {

        if (widget.name === 'FORNECEDOR') {
            widget.getField('DSALMOXARIFE').validations = {
                "required": {
                    "message": "Campo Obrigatorio"
                }
            };
            widget.getField('DSALMOXTERCFOR').validations = {};
            widget.getField('DSALMOXINFOFOR').validations = {
                "required": {
                    "message": "Campo Obrigatorio"
                }
            };
        }
        else if (widget.name === 'CLIENTE') {
            widget.getField('DSALMOXARIFE').validations = {
                "required": {
                    "message": "Campo Obrigatorio"
                }
            };
            widget.getField('DSALMOXTERCCLI').validations = {};
            widget.getField('DSALMOXINFOCLI').validations = {
                "required": {
                    "message": "Campo Obrigatorio"
                }
            };
        }
        if (widget.getParent().currentRow.CDFILIAL) {
            widget.newRow();
        }
        else {
             MessageService.showMessageByCode('UNIDADE_OBRIGATORIA');
        }
        widget.reload();
    };

    this.save = function(widget) {
        widget.currentRow.CDFILIAL = widget.getParent().currentRow.CDFILIAL;

        if (widget.currentRow.CDLOCALESTOQ === '')
            widget.currentRow.CDLOCALESTOQ = ' ';

        General.saveCrud(widget).then(function(reponse) {
            if (widget.name === 'FORNECEDOR') {
                widget.getField('DSLOCALESTOQ').readOnly = true;
                widget.getField('DSLOCALTERCFOR').readOnly = true;
                widget.getField('DSLOCALINFOFOR').readOnly = true;
            }
            else if (widget.name === 'CLIENTE') {
                widget.getField('DSLOCALESTOQ').readOnly = true;
                widget.getField('DSLOCALTERCCLI').readOnly = true;
                widget.getField('DSLOCALINFOCLI').readOnly = true;

            }

        });
        widget.reload();
    };

    this.cancelar = function(args, widget) {
        if (widget.name === 'FORNECEDOR') {
            widget.getField('DSLOCALESTOQ').readOnly = true;
            widget.getField('DSLOCALTERCFOR').readOnly = true;
            widget.getField('DSLOCALINFOFOR').readOnly = true;

        }
        else if (widget.name === 'CLIENTE') {
            widget.getField('DSLOCALESTOQ').readOnly = true;
            widget.getField('DSLOCALTERCCLI').readOnly = true;
            widget.getField('DSLOCALINFOCLI').readOnly = true;
        }

        General.backOrCancel(args);
        widget.reload();
    };

    this.edit = function(widget) {
        var cdFilial = widget.getParent().currentRow.CDFILIAL;

        Ger33500_AgenteDeTerceirizacao_Service.verificaControle(cdFilial).then(function(response) {

            var controleLocal = response.dataset.paramfilial[0].IDUTILLCESTQ;
            if (widget.name === 'FORNECEDOR') {
                if (widget.getField('DSALMOXARIFE').value()) {
                    if (controleLocal === 'S') {
                        widget.getField('DSLOCALESTOQ').readOnly = false;
                    }
                    widget.getField('DSALMOXARIFE').validations = {
                        "required": {
                            "message": "Campo Obrigatorio"
                        }
                    };

                }
                else {
                    widget.getField('DSALMOXARIFE').validations = {};
                }

                if (widget.getField('DSALMOXTERCFOR').value()) {
                    if (controleLocal === 'S') {
                        widget.getField('DSLOCALTERCFOR').readOnly = false;

                    }
                    widget.getField('DSALMOXTERCFOR').validations = {
                        "required": {
                            "message": "Campo Obrigatorio"
                        }
                    };
                }
                else widget.getField('DSALMOXTERCFOR').validations = {};

                if (widget.getField('DSALMOXINFOFOR').value()) {
                    if (controleLocal === 'S') {
                        widget.getField('DSLOCALINFOFOR').readOnly = false;

                    }
                    widget.getField('DSALMOXINFOFOR').validations = {
                        "required": {
                            "message": "Campo Obrigatorio"
                        }
                    };
                }
                else widget.getField('DSALMOXINFOFOR').validations = {};
            }
            else if (widget.name === 'CLIENTE') {
                if (widget.getField('DSALMOXARIFE').value()) {
                    if (controleLocal === 'S') {
                        widget.getField('DSLOCALESTOQ').readOnly = false;

                    }
                    widget.getField('DSALMOXARIFE').validations = {
                        "required": {
                            "message": "Campo Obrigatorio"
                        }
                    };
                }
                else widget.getField('DSALMOXARIFE').validations = {};

                if (widget.getField('DSALMOXTERCCLI').value()) {
                    if (controleLocal === 'S') {
                        widget.getField('DSLOCALTERCCLI').readOnly = false;

                    }
                    widget.getField('DSALMOXTERCCLI').validations = {
                        "required": {
                            "message": "Campo Obrigatorio"
                        }
                    };
                }
                else widget.getField('DSALMOXTERCCLI').validations = {};

                if (widget.getField('DSALMOXINFOCLI').value()) {
                    if (controleLocal === 'S') {
                        widget.getField('DSLOCALINFOCLI').readOnly = false;

                    }
                    widget.getField('DSALMOXINFOCLI').validations = {
                        "required": {
                            "message": "Campo Obrigatorio"
                        }
                    };

                }
                else widget.getField('DSALMOXINFOCLI').validations = {};
            }

        });


        widget.reload();
        General.edit(widget);
    };

    this.delete = function(widget) {
        widget.currentRow.CDFILIAL = widget.getParent().currentRow.CDFILIAL;
        General.deleteCrud(widget);
        widget.reload();
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Ger33500_AgenteDeTerceirizacao_Controller', Ger33500_AgenteDeTerceirizacao_Controller);
});


// FILE: ./assets/js/controllers/Ger57000_TabelaDoSimplesNacional_Controller.js
function Ger57000_TabelaDoSimplesNacional_Controller(Ger57000_TabelaDoSimplesNacional_Service, ScreenService, General, templateManager, ZHPromise) {

    var errorMessages = {
        "REGISTER_HAS_CHILDREN" : "Exclusão não permitida! O registro possui registros filhos.",
        "ALIQUOTA_SUM"          : "Soma das alíquotas dos impostos difere da alíquota da faixa.",
        "REGISTER_EXISTS"       : "Impossível adicionar. Registro já existe.",
        "REPARTTRIB_INV"        : "Percentual de repartição dos tributos difere de 100%.",
        "REPARTTRIBISS_INV"     : "Percentual de repartição do rateio do ISS foi informado, mas difere de 100%."
    };

    /*Muda, em modo de edição, do grid 'FaixaDeValores' para o grid 'Impostos'
    após uma validação dos campos*/
    this.avancarFaixaDeValores = function (widget) {
        if(widget.isValid()){
            var valida  = Ger57000_TabelaDoSimplesNacional_Service.verificarFaixaDeValores(widget);
            if ( valida === true) {
                this.save(widget);
            } else{ 
                widget.computeDescriptionOrder();
                this.switchWidgetChild(widget, widget.widgets[0]);
                this.controlaStatusWidgets(widget, widget.name, true);
                var impostos      = widget.widgets[0];
                if(impostos.dataSource.data.length > 0) 
                    ScreenService.showMessage(errorMessages[valida], "error");
            }
        }
    };
    
    this.preLoadAutocomplete = function(widget){
      if(widget.dataSource.data.length === 0)
        widget.reload();
    };

    this.back = function () {
         ScreenService.closeSwipe().then(
            function (response)  {
               var widgetAtiva = templateManager.container.getActiveWidget();
               if(widgetAtiva.name ==='Impostos'){
                   if(widgetAtiva.mode ==='list')
                        templateManager.container.getWidget('FaixaDeValores').currentWidget = templateManager.container.getWidget('FaixaDeValores');
                        templateManager.container.getWidget('FaixaDeValores').activate();
                }
                
                if(widgetAtiva.name ==='FaixaDeValores'){
                
                    templateManager.container.getWidget('FaixaDeValores').currentWidget = templateManager.container.getWidget('FaixaDeValores');
                    templateManager.container.getWidget('FaixaDeValores').activate();
                    }
                    
                }
        );
       
        
        
    };

    /*Função chamada por quaisquer grids para cancelar um edicao ou inserção de um registro*/
    this.cancel = function (args) {
        var widget = args.owner.widget;
        var name = widget.name;
        switch (name) {
        case 'FaixaDeValores':
            this.configFaixaDeValores(widget, "cancel");
            //this.switchTemplateImpostos(widget.widgets[0]);
            General.backOrCancel(args);
            widget.widgets[0].reload();
            break;
        case 'Impostos':
            if (widget.mode === "list") {
                if(widget.getParent().currentRow.__is_new){
                    this.configFaixaDeValores(widget.getParent(), "cancel");
                    //this.switchTemplateImpostos(widget);
                    General.backOrCancel(args);
                    widget.getParent().reload();
                    widget.reload();
                }else{
                    widget.list(true);
                    widget.getParent().view();
                    this.configFaixaDeValores(widget.getParent(), "cancel");
                    //this.switchTemplateImpostos(widget);
                    widget.activate();
                    widget.getParent().reload();
                    widget.reload();
                }
            }else{
                General.backOrCancel(args);   
                widget.getParent().reload();
                widget.reload();
            }
            break;
        case 'TabelaDeReferencia':
            General.changeWidgetAcessibility(widget.widgets, true);
            General.backOrCancel(args);
            break;
        default:
            General.backOrCancel(args);
            // code
        }
        
    };

    //Configura faixa de valores de acordo com seu modo.
    this.configFaixaDeValores = function (widget, operation) {
        var impostos = widget.widgets[0];
        var lastIndex = widget.dataSource.getLastIndex();
        if (operation === "open") {
            if (widget.currentRow.__is_new) {
                operation = "new";
            } else {
                operation = "view";
            }
        }
        switch (operation) {
        case 'cancel':
        case 'save':
            General.changeWidgetAcessibility([impostos], true);
            impostos.getAction("save-list").isVisible = false;
            //impostos.getAction("plus").isVisible = false;
            impostos.getAction("cancel-list").isVisible = false;
            impostos.getAction("back-list").isVisible = true;
            break;
        case 'edit':
        case 'new':
            this.updateImposto(widget);
            if (operation === "edit" && widget.currentRow.__index < lastIndex) {
                General.changeWidgetAcessibility([impostos], true);
                this.fieldsMethods(widget, "block", ["VRFINAL"]);
            } else {
                this.fieldsMethods(widget, "unlock", ["VRFINAL"]);
            }
            impostos.getAction("save-list").isVisible = true;
            //impostos.getAction("plus").isVisible = true;
            impostos.getAction("cancel-list").isVisible = true;
            impostos.getAction("back-list").isVisible = false;
            break;
        case 'view':
            this.updateImposto(widget);
            if (widget.currentRow.__index < lastIndex) {
                widget.getAction("delete").isVisible = false;
            } else {
                widget.getAction("delete").isVisible = true;
            }
            break;
        default:
            // code
        }
    };

    //acessa as funções de delatar dos grids
    this.delete = function (widget) {
        var name = widget.name;
        switch (name) {
        case "TabelaDeReferencia":
            ScreenService.confirmMessage(ScreenService.i18n('Ao confirmar a exclusão toda referências para a tabela também será excluída. Confirma a operação?'), 'question', function (success) {
              
                Ger57000_TabelaDoSimplesNacional_Service.deleteTabelaDeReferencia(widget).then(function(success){
                    ScreenService.closeSwipe().then(function(){
                        widget.activate();
                        widget.reload();
                    });
                }, function(error){
                    if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                        ScreenService.showMessage(errorMessages.REGISTER_HAS_CHILDREN);
                    } else {
                        ScreenService.showMessage(error.data.error);
                    }
                });
                
            
            });
            break;
        case "FaixaDeValores":
            ScreenService.confirmMessage(ScreenService.i18n('Ao confirmar a exclusão toda referência para a faixa de valores será excluída. Confirma a operação?'), 'question', function (success) {
                Ger57000_TabelaDoSimplesNacional_Service.deleteFaixaValores(widget).then(function () {
                    ScreenService.closeSwipe().then(function () {
                        widget.activate();
                        widget.reload();
                    });
                }, function (error) {
                    if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                        ScreenService.showMessage(errorMessages.REGISTER_HAS_CHILDREN);
                    } else {
                        ScreenService.showMessage(error.data.error);
                    }
                });
            });
            break;
        case "Impostos":
            this.deleteImpostos(widget);
            //widget.dataSource.data.pop();
            break;
        default:
            General.deleteCrud(widget);
        }
    };

    //Deleta uma linha do grid impostos
    this.deleteImpostos = function (widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
            var index = widget.currentRow.__index;
            widget.dataSource.data.splice(index, 1); //retira a linha do grid
            widget.currentRow = {};
            ScreenService.closeSwipe();	
            this.validaEdicaoIntervaloFaixa();
            if(widget.getParent().mode ==='view'){
                this.configFaixaDeValores(widget.getParent(), 'edit');
                General.edit(widget.getParent());
                widget.list();
                
            }
        }.bind(this));
    };

    //Responsável pela edição de todos os grids
    this.edit = function (widget) {
        var name = widget.name;
        General.edit(widget);
        switch (name) {
        case 'FaixaDeValores':
            this.editFaixaDeValores(widget);
            break;
        case 'TabelaDeReferencia':
            General.changeWidgetAcessibility(widget.widgets, false);
            break;
        default:
        }
    };

    //Realiza o edit no grid Faixa de valores
    this.editFaixaDeValores = function (widget) {
        widget.reload();
        widget.widgets[0].reload();
        widget.getField("VRFINAL").validations = {
            "required": {}
        };
        this.configFaixaDeValores(widget, "edit");
        //this.switchTemplateImpostos(widget.widgets[0]);
    };

    //Responsável por incluir registros novos de todos os grids
    this.new = function (widget) {
        var name = widget.name;
        if(widget.name ==='Impostos'){
            if(!widget.currentRow){
                widget.currentRow = {};
                ScreenService.openSwipe();
            }
        }
        
        widget.newRow();
        if (name !== "TabelaDeReferencia") {
            this.fillFields(widget.getParent(), widget, ["CDSIMPLESNACIONAL"]);
            if (name === "TipoDeOperacao") {
                widget.currentRow.IDTIPOOPERTAB = 'S';
            } else {
                if (name === "FaixaDeValores") {
                    this.newFaixaDeValores(widget);
                }
            }
        } else {
            this.valoresMinMaxData(widget);
            General.changeWidgetAcessibility(widget.widgets, false);
        }
    };
    
    /*
    this.changeDataSourceAfterSelectOpen = function (){
        var widget = templateManager.container.getActiveWidget();
            widget.dataSource.name = 'df#/crud/cfopsimplesnac_cfoppadrao_new';
            widget.reload();
    };
    */
    
    this.controlaStatusWidgets = function(mainWidget, nomeWidgetAtual, status){
        mainWidget.widgets.forEach(function(widg) {
            if(nomeWidgetAtual !== widg.name)
                widg.enabled = status;
        });
    };
    
    //Realiza o new no grid Faixa de valores tabbed (nao utilizado)
    this.newFaixaDeValores = function (widget) {
        this.configFaixaDeValores(widget, "new");
        //this.switchTemplateImpostos(widget.widgets[0]);
        this.controlaStatusWidgets(widget, widget.name, false);
        var lastIndex = widget.dataSource.getLastIndex() - 1;
        var row = widget.currentRow;
        if (lastIndex >= 0) {
            var lastRow = widget.dataSource.data[lastIndex];
            row.VRINICIAL = 0.01 + parseFloat(lastRow.VRFINAL);
        } else {
            row.VRINICIAL = 0.00;
        }
        widget.getField("VRFINAL").validations = {
            "required": {},
            "min": {
                "limit": row.VRINICIAL
            }
        };
    };

    //Recarrega o datasource das widgets filhas da widget passada
    this.reloadWidgetChildrenDatasource = function (widget) {
        widget.widgets.forEach(
            function (widget) {
                widget.reload();
            }
        );
    };

    //Função responsável por salvar os novos dados de um grid.
    this.save = function (widget) {
        var name = widget.name;
        switch (name) {
        case 'TipoDeOperacao':
        case 'CodigoFiscalDeOperacao':
            this.saveCustom(widget);
            break;
        case 'FaixaDeValores':
            this.saveFaixaDeValores(widget);
            break;
        case 'Impostos':
            this.saveImpostos(widget);
            break;
        case 'TabelaDeReferencia':
            this.saveTabelaSimplesNacional(widget);
            break;
        default:
            General.saveCrud(widget);
        }
    };

    this.saveTabelaSimplesNacional = function (widget){
        if( widget.isValid() ){
            General.callRestEngine('df#/saveTabelaSimplesNacional','Row',widget.currentRow ).then(
                function (response)  {
                    widget.activate();
                    if (widget.currentRow.__is_new) {
                        widget.list(true);
                        ScreenService.goBack();
                        widget.reload();
                    } else {
                        widget.computeDescriptionOrder();
                        widget.view();
                    }
                },
                function (callback) {
                    if (callback.data) {
                        ScreenService.showMessage(callback.data.error);
                    }
                }  
            );    
        }
        let widgets = [widget.widgets];
        widgets.forEach(
            function(widgetFilha){
                General.changeWidgetAcessibility(widgetFilha, true);
            }
        )
        ;
    };

    //Funcao para utilizar uma rota modificada
    this.saveCustom = function (widget, dataSet) {
        if (!dataSet)
            dataSet = [];
        if (dataSet.length === 0) {
            dataSet.push(widget.currentRow);
        }
        if (widget.isValid()) {
            Ger57000_TabelaDoSimplesNacional_Service.save(dataSet, widget.name).then(function (success) {
                widget.activate();
                if (widget.currentRow.__is_new) {
                    if (success.dataset && success.dataset.codigos) {
                        widget.currentRow.CDFAIXA = success.dataset.codigos.CDFAIXA;
                    }
                    widget.currentRow.__is_new = false;
                    widget.list(true);
                    widget.reload();
                    
                    ScreenService.goBack().then(
                        function (response)  {
                            /*
                            templateManager.container.getWidget('FaixaDeValores').currentWidget = templateManager.container.getWidget('FaixaDeValores');
                            templateManager.container.getWidget('FaixaDeValores').activate();
                            */
                        }
                    );
                    
                    if (widget.name == "FaixaDeValores") {
                        widget.widgets[0].getAction('save-list').isVisible   = false;
                        widget.widgets[0].getAction('cancel-list').isVisible = false;
                        widget.widgets[0].getAction('back-list').isVisible   = true;
                    }
                } else {
                    widget.computeDescriptionOrder();
                    widget.view();
                    widget.reload();
                    if (widget.name == "FaixaDeValores") {
                        widget.widgets[0].reload();
                        if(templateManager.container.getActiveWidget().name ==='Impostos'){
                            widget.widgets[0].list(true);
                            widget.widgets[0].activate();
                        }
                        widget.widgets[0].getAction('save-list').isVisible   = false;
                        widget.widgets[0].getAction('cancel-list').isVisible = false;
                        widget.widgets[0].getAction('back-list').isVisible   = true;
                    }
                }
                return true;
            }, function (error) {
                var errorMessage = 'Timeout';
                if (error.data) {
                    errorMessage = error.data.error;
                }
                ScreenService.showMessage(errorMessage);
                return false;
            });
        }
        else{
            if(widget.name==='FaixaDeValores')
                 widget.currentWidget = widget;
                widget.activate();
        }
    };

    /*
    Funcao chamada ao finalizar faixa de valores com seus impostos
    -Variaveis-
    widget - recebe a widget 'FaixaDeValores'
    dataSet - array com a faixa de valores e os impostos a serem inseridos
    */
    this.saveFaixaDeValores = function (widget) {
        var dataSet = [];
        
        if(widget.currentRow.__is_new){
            widget.widgets[0].dataSource.data.forEach(
                function (data) {
                    data.__is_new = true;
                    //Formata o valor devido a alguns problemas de formato.
                    data.VRALIQSIMPLES = Ger57000_TabelaDoSimplesNacional_Service.formataValoresDecimais(data.VRALIQSIMPLES);
                });
        }else{
             widget.widgets[0].dataSource.data.forEach(
                function (data) {
                    //Formata o valor devido a alguns problemas de formato.
                    data.VRALIQSIMPLES = Ger57000_TabelaDoSimplesNacional_Service.formataValoresDecimais(data.VRALIQSIMPLES);
                });
        }
        //Formata o valor devido a alguns problemas de formato.                
        widget.currentRow.VRALIQSIMPLES = Ger57000_TabelaDoSimplesNacional_Service.formataValoresDecimais(widget.currentRow.VRALIQSIMPLES);
        dataSet.push(widget.currentRow); //adiciona a linha da faixa de valores
        dataSet = dataSet.concat(widget.widgets[0].dataSource.data); // adiciona os impostos da faixa de valores
        var isValidFaixa = Ger57000_TabelaDoSimplesNacional_Service.verificarFaixaDeValores(widget);
        var impostos      = widget.widgets[0];
        
        if(isValidFaixa === true){
            if(this.saveCustom(widget, dataSet)){
                this.configFaixaDeValores(widget, 'save');
                //this.switchTemplateImpostos(widget.widgets[0]);
                widget.view();
                widget.reload();
                //this.switchWidgetChild(widget, widget);
            }
        }else{
            if(impostos.dataSource.data.length > 0){
                ScreenService.showMessage(errorMessages[isValidFaixa], "error");
            }else{
                ScreenService.showMessage("As informações devem ser cadastradas.");
            }
        }
    };

    /*
    Função que salva a linha editada ou inserida no grid 'Impostos'
    */
    this.saveImpostos = function (widget) {
        if(widget.isValid()){
            var isValidImpostos = Ger57000_TabelaDoSimplesNacional_Service.verificarImpostos(widget);
            if (isValidImpostos === true) {
                this.validaEdicaoIntervaloFaixa();
                widget.currentRow.__is_new = false;
                ScreenService.closeSwipe();
                widget.list();
                widget.activate();
                if(widget.getParent().mode ==='view'){
                    this.configFaixaDeValores(widget.getParent(), 'new');
                    General.edit(widget.getParent());
                    widget.list();
                }
            } else {
                ScreenService.showMessage(errorMessages.REGISTER_EXISTS, "error");
            }    
        }
        
    };

    this.setFloatFieldPrecision = function (widget, fieldName, precision) {
        var value = widget.currentRow[fieldName];
        if (typeof value === 'string') {
            value = value.replace(',', '.');
        }
        if (value && value.length > 0) {
            widget.currentRow[fieldName] = ((parseFloat(value)).toFixed(precision)).toString();
        }
    };

    /*
    Troca o filho da widget que está sendo mostrado na tela
    */
    this.switchWidgetChild = function (widget, widgetChild) {
        widget.currentWidget = widgetChild;
        widgetChild.activate();
    };

    this.switchTemplateImpostos = function (widget) {
        /*
        if (widget.template === "widget/grid.html") {
            widget.template = "widget/master_detail/grid.html";
        } else {
            widget.template = "widget/grid.html";
        }*/
    };

    //Coloca um filtro de comparação em todos os campos do objeto escolhido.
    this.updateFilter = function (object, fieldNames, operator, values) {
        var dataSourceFilter = object.dataSourceFilter;
        for (var i = 0; i < fieldNames.length; i++) {
            dataSourceFilter = dataSourceFilter.concat({
                "name": fieldNames[i],
                "operator": operator,
                "value": values[i]
            });
        }
        object.dataSourceFilter = dataSourceFilter;
    };

    //Atualizar o filtro do grid Impostos
    this.updateImposto = function (widget) {
        var fieldNames = ["CDSIMPLESNACIONAL", "CDFAIXA"];
        var values = [widget.currentRow.CDSIMPLESNACIONAL, widget.currentRow.CDFAIXA];
        var impostos = widget.widgets[0];
        impostos.dataSourceFilter = [impostos.dataSourceFilter[0]];
        this.updateFilter(widget.widgets[0], fieldNames, "=", values);
    };

    //Habilita e filtra todas as widgters filhas.
    this.updateWidgets = function (widget) {
        var fieldName = "CDSIMPLESNACIONAL";
        var widgets = widget.widgets;
        var value = widget.currentRow[fieldName];
        if (value) {
            for (var i = 0; i < widgets.length; i++) {
                widgets[i].dataSourceFilter = [widgets[i].dataSourceFilter[0]]; //reseta o filtro
                this.updateFilter(widgets[i], [fieldName], '=', [value]);
            }
            General.changeWidgetAcessibility(widgets, true);
        }
        this.reloadWidgetChildrenDatasource(widget);
    };

    //Passar a widget que deseja mostrar em um swipe;

    //Volta do modo edit da widget
    this.backWidget = function (widget) {
        ScreenService.closeSwipe();
        widget.getParent().getParent().activate();
    };

    //Coloca a widget em modo edit.
    this.editTabbedWidget = function (widget) {
        widget.edit();
        widget.widgets[0].edit();
        widget.fields.forEach(
            function (field) {
                field.template = "field/text-edit.html";
                widget.currentRow[field.name] = widget.getParent().getParent().currentRow[field.name];
            }
        );
    };

    //Abre um swipe para a criação de um novo registro.
    this.newTabbedWidget = function (widget) {
        widget.edit();
        widget.widgets[0].edit();
        widget.fields.forEach(
            function (field) {
                field.template = "field/text-edit.html";
                widget.currentRow[field.name] = "";
            }
        );
        ScreenService.openSwipe(widget.getParent());
    };

    //Coloca uma widget em modo view.
    this.viewTabbedWidget = function (widget) {
        widget.view();
        widget.widgets[0].view();
        widget.fields.forEach(
            function (field) {
                field.template = "field/span-field.html";
                widget.currentRow[field.name] = widget.getParent().getParent().currentRow[field.name];
            }
        );
    };

    //Funções para campos
    this.fieldsMethods = function (widget, method, fieldNames) {
        switch (method) {
        case 'block':
            this.blockFields(widget, fieldNames);
            break;
        case 'clear':
            this.clearFields(widget, fieldNames);
            break;
        case 'hide':
            this.hideFields(widget, fieldNames);
            break;
        case 'show':
            this.showFields(widget, fieldNames);
            break;
        case 'unlock':
            this.unlockFields(widget, fieldNames);
            break;
        default:
            // code
        }
    };

    this.blockFields = function (widget, fieldNames) {
        for (var i = 0; i < fieldNames.length; i++) {
            widget.getField(fieldNames[i]).readOnly = true;
        }
    };

    this.clearFields = function (widget, fieldNames) {
        var row = widget.currentRow;
        for (var i = 0; i < fieldNames.length; i++) {
            row[fieldNames[i]] = "";
        }
    };

    this.fillFields = function (originWidget, destinyWidget, fieldNames) {
        if (!fieldNames) {
            for (var i = 0; i < destinyWidget.fields.length; i++) {
                fieldNames[i] = destinyWidget.fields[i];
            }
        }
        var currentRowDestiny = destinyWidget.currentRow;
        var currentRowOrigin = originWidget.currentRow;
        for (var j = 0; j < fieldNames.length; j++) {
            if (currentRowDestiny[fieldNames[j]] !== undefined) {
                currentRowDestiny[fieldNames[j]] = currentRowOrigin[fieldNames[j]];
            }
        }
    };

    this.hideFields = function (widget, fieldNames) {
        for (var i = 0; i < fieldNames.length; i++) {
            widget.getField(fieldNames[i]).isVisible = false;
        }
    };

    this.showFields = function (widget, fieldNames) {
        for (var i = 0; i < fieldNames.length; i++) {
            widget.getField(fieldNames[i]).isVisible = true;
        }
    };

    this.unlockFields = function (widget, fieldNames) {
        for (var i = 0; i < fieldNames.length; i++) {
            widget.getField(fieldNames[i]).readOnly = false;
        }
    };
    
    this.valoresMinMaxData = function (widget){
        var dataInicial = widget.getField('DTINIVIGENCIA');
        var dataFinal   = widget.getField('DTFIMVIGENCIA');

        dataInicial.maxDate = dataFinal.value()   || '0';
        dataFinal.minDate   = dataInicial.value() || '0';
    };
    
    this.verificaEdicaoImpostoGrid = function (widget) {
        if(widget.getParent().mode ==='view'){
            this.configFaixaDeValores(widget.getParent(), 'new');
            General.edit(widget.getParent());
        }
        widget.list(true);
        this.validaEdicaoIntervaloFaixa();
    };
    
    this.validaEdicaoIntervaloFaixa = function (){
        var widget = templateManager.container.getWidget('FaixaDeValores');
        widget.getField("VRFINAL").validations = {
            "required": {},
            "min": {
                "limit": widget.currentRow.VRINICIAL + 0.01
            }
        };
    };
    
    this.ativaWidgetImposto = function(){
        templateManager.container.getWidget('Impostos').currentWidget = templateManager.container.getWidget('Impostos');
        templateManager.container.getWidget('Impostos').activate();
        templateManager.container.getWidget('Impostos').currentRow = {
           "CDSIMPLESNACIONAL":"",
           "CDFAIXA":"",
            "CDIMPOSTO":"",
            "NRORG":""    
        };
    };
    

}
Configuration(function (ContextRegister) {
    ContextRegister.register("Ger57000_TabelaDoSimplesNacional_Controller", Ger57000_TabelaDoSimplesNacional_Controller);
});

// FILE: ./assets/js/controllers/Ger80800_RegrasTributarias_Controller.js
var Ger80800_RegrasTributarias_Controller = function(ScreenService, General, templateManager, Ger80900_ParametrizacaoFiscalCentralizada_Service, ZhFilterPreferences) {

  var requestError = function(error){
    var errorMessage = "Ocorreu um erro ao tentar realizar a operação.";
    if (error && error.data) {
      errorMessage = error.data.error;
    }
    ScreenService.showMessage(errorMessage, "ERROR");
  };
	
	this.getCodProd = function(){
	  	var widget = templateManager.container.getWidget("regratributaria");
	  	
			var currentRow = widget.currentRow;
			var dataset = widget.dataSource.data;
			var id_cenario = currentRow.ID_CENARIO;
			if(widget.isValid()) {
				General.callRestEngine("df#/getCodProd", "Row", currentRow).then((response) => {
					var a = 'a';
					console.log(response);
					currentRow.COD_PROD = response.dataset.getCodProd.dados.COD_PROD;

			  // 	currentRow.IDUTUFDEST = args.owner.widget.getParent().currentRow.IDUTUFDEST;
	  		//   currentRow.IDUTUFORIG = args.owner.widget.getParent().currentRow.IDUTUFORIG;
					// if(this.produtoRepetido(args.owner.widget.dataSource.data ,currentRow  ).length > 0 ) {
					// 		 ScreenService.showMessage("Operação Bloqueada! Produto já cadastrado");
					// }

				});
			}
	};
	
	var controlarStatusWidgets = function(widgetPai, nomeWidgetAtual, status){
    widgetPai.widgets.forEach(function(widg){
      if(nomeWidgetAtual !== widg.name)
        widg.enabled = status;
    });

    if(widgetPai.name !== nomeWidgetAtual)
      widgetPai.enabled = status;
  };
	
	this.cancelarRow = function(args){
	  var widget = args.owner.widget;
	  controlarStatusWidgets(widget, widget.name, true);
	  /*delete widget.getField('NMESTAORIG').validations;
	  delete widget.getField('NMPAISORIG').validations;
	  delete widget.getField('NMPAISDEST').validations;
	  delete widget.getField('NMESTADEST').validations;*/
	  General.backOrCancel(args);
	};
	
	this.saveCodigo = function(widget){
		let row = widget.currentRow;
		
		if (widget.isValid()) {
      General.callRestEngine('/sys_material/save', "Row", row).then(function(response){
	      ScreenService.successNotification("Registro salvo com sucesso.");
	      ScreenService.closeSwipe();
      });
    } else{
        ScreenService.showMessage("Preencha os campos obrigatórios corretamente!");
    }
	
  };
 
 

 
 
};

Configuration(function(ContextRegister) {
  ContextRegister.register("Ger80800_RegrasTributarias_Controller", Ger80800_RegrasTributarias_Controller);
});

// FILE: ./assets/js/controllers/Ger80900_ParametrizacaoFiscalCentralizada_Controller.js
var Ger80900_ParametrizacaoFiscalCentralizada_Controller = function(ScreenService, General, templateManager, Ger80900_ParametrizacaoFiscalCentralizada_Service, ZhFilterPreferences) {

  var requestError = function(error){
    var errorMessage = "Ocorreu um erro ao tentar realizar a operação.";
    if (error && error.data) {
      errorMessage = error.data.error;
    }
    ScreenService.showMessage(errorMessage, "ERROR");
  };

  var getRegraFiscalWidget = function(){
    if(templateManager.container)
      return templateManager.container.getWidget("regraFiscal");
  };

  var getImpostoPadraoWidget = function(){
    if(templateManager.container)
      return templateManager.container.getWidget("impostoPadrao");
  };
  

  var removerRowExcluida = function (args){
	  args.owner.widget.reload();
  };

  var controlarStatusWidgets = function(widgetPai, nomeWidgetAtual, status){
    widgetPai.widgets.forEach(function(widg){
      if(nomeWidgetAtual !== widg.name)
        widg.enabled = status;
    });

    if(widgetPai.name !== nomeWidgetAtual)
      widgetPai.enabled = status;
  };

  this.getSequencialMensagem = function(widgetname){
    if(templateManager.container){
      var datasetMensagem = templateManager.container.getWidget(widgetname).dataSource.data;
      var maiorSequencial = Ger80900_ParametrizacaoFiscalCentralizada_Service.getMaiorSequencialDataset(datasetMensagem, 'NRSEQMSG');
      return Ger80900_ParametrizacaoFiscalCentralizada_Service.zerofill(parseInt(maiorSequencial) + 1, 4);
    }
  };

  this.validacoesRegraAfterMoveRow = function(args) {
    controlarVisibilidadeActAtivarRegra(args.owner, args.row.IDATIVREGRAFISC === 'N');
  };

  var controlarVisibilidadeActAtivarRegra = function(parentWidget, status){
    parentWidget.getAction('actAtivarRegra').isVisible = status;
    parentWidget.getAction('actDesativarRegra').isVisible = !status;
  };

  this.controlarCamposAliquotaCFOP = function(args){
    if(args.row.IDCFOPINTEREST === 'E'){
      args.owner.field.widget.getField('IDDIFEALIQ').readOnly = true;
      args.owner.field.widget.getField('IDDIFEALIQ').setValue('N');
      args.owner.field.widget.getField('IDDIFALIQCUSPR').readOnly = true;
      args.owner.field.widget.getField('IDDIFALIQCUSPR').setValue('N');
    }
    else{
      args.owner.field.widget.getField('IDDIFEALIQ').readOnly = false;
      args.owner.field.widget.getField('IDDIFEALIQ').setValue('N');
      args.owner.field.widget.getField('IDDIFALIQCUSPR').readOnly = false;
      args.owner.field.widget.getField('IDDIFALIQCUSPR').setValue('N');
    }
  };

  this.consultarErrosRegra = function(regra){
    return Ger80900_ParametrizacaoFiscalCentralizada_Service.consultarErrosRegra(regra);
  };

  this.ativarRegraFiscal = function(args) {
    if(args.owner.widget.currentRow.IDATIVREGRAFISC === 'N'){
      this.consultarErrosRegra(args.row).then(function(response){
        if(response.dataset.consultarErrosRegra.length > 0){
          exibirPopupErrosRegra(args.owner.widget, response.dataset.consultarErrosRegra);
        }
        else{
          toogleStatusRegra(args.owner.widget, 'S');
          controlarVisibilidadeActAtivarRegra(args.owner.widget, false);
        }
      });
    }
  };

  this.desativarRegraFiscal = function(args){
    desativarRegra(args.owner.widget);
  };

  var exibirPopupErrosRegra = function(widgetRegra, erros){
    var popupErros = widgetRegra.container.getWidget('popupErrosRegraFiscal');
    ScreenService.openPopup(popupErros);
    popupErros.dataSource.data = erros;
  };

  var toogleStatusRegra = function(widgetRegra, status){
    widgetRegra.currentRow.IDATIVREGRAFISC = status;
    widgetRegra.dataSource.name =  "df#/crud/regrafiscal";
    General.saveCrud(widgetRegra);
    widgetRegra.dataSource.name =  "df#/regrafiscal_agrupimposto";
  };

  var desativarRegra = function(widgetRegra) {
    if(widgetRegra.currentRow.IDATIVREGRAFISC === 'S'){
      toogleStatusRegra(widgetRegra, 'N');
      controlarVisibilidadeActAtivarRegra(widgetRegra, true);
    }
  };

  this.abrirVerificaRegraFisc = function(args) {
    ScreenService.openWindow("df#ger80902_verifica_regra_fiscal").then(function() {
      setTimeout(function(){ //(Corrigit activate da ger80902_verifica_regra_fiscal)Verificar a necessidade deste timeout quando convertar a tela para o filtro 3.
        let widget = templateManager.containers["df#ger80902_verifica_regra_fiscal"].getWidget('verificaRegraFiscal');
        widget.getField('CDINSCESTA').dataSourceFilter[0].value = "@General.getFieldValue('CDEMPRESA', 'verificaRegraFiscal', true)@";
        widget.getField('NMTIPOOPER').dataSourceFilter[0].value = "@General.getFieldValue('IDENTRSAIDOP', 'verificaRegraFiscal', true)@";
        widget.getField('NMESTAORIG').dataSourceFilter[0].value = "@General.getFieldValue('CDPAISORIG', 'verificaRegraFiscal', true)@";
        widget.getField('NMMUNICIPIOORIG').dataSourceFilter[0].value = "@General.getFieldValue('CDPAISORIG', 'verificaRegraFiscal', true)@";
        widget.getField('NMMUNICIPIOORIG').dataSourceFilter[1].value = "@General.getFieldValue('SGESTAORIG', 'verificaRegraFiscal', true)@";
        
        
        widget.getField('NMESTADEST').dataSourceFilter[0].value = "@General.getFieldValue('CDPAISDEST', 'verificaRegraFiscal', true)@";
        widget.getField('NMMUNICIPIODEST').dataSourceFilter[0].value = "@General.getFieldValue('CDPAISDEST', 'verificaRegraFiscal', true)@";
        widget.getField('NMMUNICIPIODEST').dataSourceFilter[1].value = "@General.getFieldValue('SGESTADODEST', 'verificaRegraFiscal', true)@";
      }, 1000);
    });
  };
  
  this.abrirConsultoriaFiscal = function(args) {
    ScreenService.openWindow("df#ger80908_consultoria_fiscal");
  };
   
  this.abrirGrupoDeProdutosParaConsultoria = function(args) {
    ScreenService.openWindow("df#ger80909_grupo_produto_consultoria");
  };
  
  this.abrirParametrosGrupoFiscal = function(args) {
  	var  wdgAnterior;
  	localStorage.setItem("wdgAnterior", "Parametrização Fiscal Centralizada");
    ScreenService.openWindow("df#ger10200_parametros_grupo_fiscal");
  };
  
  this.abrirClassificacaoFiscalNcmProd = function(args) {
    ScreenService.openWindow("fis#fis01000_classificacaoFiscalProduto");
  };
  
  this.abrirDadosFiscaisCliente = function(args) {
    ScreenService.openWindow("df#ger80906_dados_fiscais_cliente"); 
  };

	this.filtroCFOP = function(args) {
		var campo = args.owner.getField("CDFISCOPER");
		if(args.owner.parent.parent.currentRow.CDIMPOSTONFE == "00"){
			campo.readOnly = false;
			campo.validations = {"required": true};
		} else {
			campo.readOnly = true;
			campo.validations = {};
		}
	};

  this.abrirDadosFiscaisFornecedor = function(args) {
      ScreenService.openWindow("df#ger80905_dados_fiscais_fornecedor"); 
  };

  this.abrirAgrupamentoImpostos = function(args) {
    ScreenService.openWindow("df#ger81100_agrupamento_impostos");
  };

  this.abrirDadosFiscaisCSTimposto = function(args) {
    ScreenService.openWindow("df#ger80909_dados_fiscais_cst_imposto");
  };

  this.abrirDadosFiscaisImposto = function(args) {
    ScreenService.openWindow("df#ger80908_dados_fiscais_imposto");
  };
  
  

  this.limpaCampoCodigoMensagem = function(args) {
    args.row.CDMENSAGEM = null;
  };

  this.excluirRow = function(args){
		return General.deleteCrud(args.owner.widget).then(function(){
			removerRowExcluida(args);
		});
	};

	this.adicionarRow = function(args){
		var widget = args.owner.widget;
		
		var widgetImposPadrao = templateManager.container.getWidget('impostoPadrao');
		var widgetImposNCM = templateManager.container.getWidget('impostoNCM');
		var widgetImposProduto = templateManager.container.getWidget('impostoProduto');
		
	  controlarStatusWidgets(widget, widget.name, false);
	  
	  if(args.owner.widget.name === 'regraFiscal') {
	      Ger80900_ParametrizacaoFiscalCentralizada_Service.getNewCodeRegra({}).then(function(success) {
	          widget.newRow();
	          widget.currentRow.CDREGRAFISC = success.dataset.getNewCodeRegra.CODIGO;
	          
	      });
	  } else{
	  	widget.newRow();
	  	if(widget.getParent().name == "imposto"){
	  		this.validaMvaECompBC(widget);
	  	}
	  	widget.currentRow.CDIMPOSTO = widget.getParent().currentRow.CDIMPOSTO;
	  } 
	};

	this.editarRow = function(args){
	  controlarStatusWidgets(args.owner.widget, args.owner.widget.name, false);
	  General.edit(args.owner.widget);
	  
	};
	
	this.editarRowImposto = function(args){
		this.editarRow(args);
		this.validaMvaECompBC(args.owner.widget);
		var widget = args.owner.widget;
	};

	this.cancelarRow = function(args){
	  var widget = args.owner.widget;
	  controlarStatusWidgets(widget, widget.name, true);
	  /*delete widget.getField('NMESTAORIG').validations;
	  delete widget.getField('NMPAISORIG').validations;
	  delete widget.getField('NMPAISDEST').validations;
	  delete widget.getField('NMESTADEST').validations;*/
	  General.backOrCancel(args);
	};

	var preencherPrioridade = function(row){
	  if(row.CDTIPOOPER && row.CDINSCESTA)
	    row.NRORDPRIORIDADE = 1;
	  else if(row.CDTIPOOPER && !row.CDINSCESTA)
	    row.NRORDPRIORIDADE = 2;
    else if(!row.CDTIPOOPER && row.CDINSCESTA)
	    row.NRORDPRIORIDADE = 3;
    else if(!row.CDTIPOOPER && !row.CDINSCESTA)
	    row.NRORDPRIORIDADE = 4;
	};

	var validarDataFinal = function(args){
	  if(args.row.DTVALVINCFISC){
  	  var dataInicial = moment(args.owner.widget.getParent().currentRow.DTVALREGRAFISC, 'DD/MM/YYY');
  	  var dataFinal = moment(args.row.DTVALVINCFISC, 'DD/MM/YYY');
  	  return dataFinal.isAfter(dataInicial);
	  }
	  return true;
	};

	this.salvarVinculo = function(args){
		if(validarDataFinal(args)){
			preencherPrioridade(args.row);
			args.owner.widget.dataSource.name =  "df#/crud/vinculofisregra";
			General.callRestEngine("df#/ConsultaRepetida", "Row", args.owner.widget.currentRow).then(success => {
				if(success.length == 0 || success.dataset.ConsultaRepetida.length == 0){
					General.saveCrud(args.owner.widget).then(function(response){
					desativarRegra(args.owner.widget.getParent());
				});
				args.owner.widget.dataSource.name =  "df#/vinculofisregra_empresa_tipooperac";
				} else {
					ScreenService.showMessage("Impossivel adicionar, registro já existe");
				}
			});
		}
		else{
			ScreenService.showMessage("Validade do vínculo fiscal deve ser maior ou igual a validade da regra");
		}
	};

	this.excluirVinculo = function(args){
		args.owner.widget.dataSource.name =  "df#/crud/vinculofisregra";
		General.deleteCrud(args.owner.widget).then(function(){
			args.owner.widget.dataSource.name =  "df#/vinculofisregra_empresa_tipooperac";
			desativarRegra(args.owner.widget.getParent());
			removerRowExcluida(args);
		});
	};

	this.excluirMensagem = function(args){
	  this.excluirRow(args).then(function(response){
	    desativarRegra(args.owner.widget.getParent());
	  });
	};

	this.empresaVinculoOnChange = function(args) {
	  var inscricaoEstadual = args.owner.field.widget.getField('CDINSCESTA');
    if(args.owner.field.widget.dataSource.data.length > 0 && args.owner.field.widget.dataSource.data[0].__is_new === false){
      var primeiroVinculo = args.owner.field.widget.dataSource.data[0];
	    if(primeiroVinculo.CDINSCESTA){
	      inscricaoEstadual.readOnly = !args.owner.field.value();
        inscricaoEstadual.setValue('');
	    }
    }
    else{
      inscricaoEstadual.readOnly = !args.owner.field.value();
      inscricaoEstadual.setValue('');
    }
    inscricaoEstadual.reload();
  };

	this.identrsaidopVinculoOnChange = function(args) {
	  var tipoOperacao = args.owner.field.widget.getField('NMTIPOOPER');
    tipoOperacao.readOnly = !args.owner.field.value();
    tipoOperacao.setValue('');
    if(!!args.owner.field.value())
	    tipoOperacao.validations = {"required": true};
    else
	    delete tipoOperacao.validations;
  };

  this.validacoesVinculoAfterMoveRow = function(args){
    var fieldEntradaSaida = args.owner.getField('IDENTRSAIDOP');
    fieldEntradaSaida.readOnlyOnEdit = !fieldEntradaSaida.value();

    var fieldTipoOperacao = args.owner.getField('NMTIPOOPER');
    fieldTipoOperacao.readOnlyOnEdit = !fieldTipoOperacao.value();

    var fieldInscricaoEstadual = args.owner.getField('CDINSCESTA');
    var fieldempresa= args.owner.getField('NMEMPRESA');
    fieldInscricaoEstadual.readOnly = !fieldInscricaoEstadual.value() && !(fieldempresa.value());
		fieldInscricaoEstadual.reload();
    
  };

  this.actNovoVinculo = function(args){
    var widget = args.owner.widget;
    var fieldEntradaSaida = widget.getField('IDENTRSAIDOP');
    var fieldTipoOperacao = widget.getField('NMTIPOOPER');
    var fieldInscesta = widget.getField('CDINSCESTA');

    if(widget.dataSource.data.length > 0){
      var primeiroVinculo = widget.dataSource.data[0];

      if(primeiroVinculo.IDENTRSAIDOP){
        fieldEntradaSaida.readOnly = false;
        fieldEntradaSaida.validations = {"required": true};
      }
      else{
        fieldEntradaSaida.readOnly = true;
        delete fieldEntradaSaida.validations;
      }

      if(primeiroVinculo.CDINSCESTA){
        fieldInscesta.readOnly = false;
        fieldInscesta.validations = {"required": true};
      }
      else{
        fieldInscesta.readOnly = true;
        delete fieldInscesta.validations;
      }
    }
    else{
      fieldEntradaSaida.readOnly = false;
      fieldInscesta.readOnly = true;
      delete fieldEntradaSaida.validations;
      delete fieldInscesta.validations;
      delete fieldTipoOperacao.validations;
    }
    fieldTipoOperacao.readOnly = true;
    args.owner.widget.newRow();
  };

  this.actNovoCfop = function(args){
    if(args.owner.widget.dataSource.data.length > 0 && args.owner.widget.dataSource.data[0].IDCFOPINTEREST === 'T')
      ScreenService.showMessage("Já existe CFOP cadastrado para todas as operações");
    else
      args.owner.widget.newRow();
  };

  var validarCfopPadrao = function(args){
    var rows = args.owner.widget.dataSource.data.filter(function(row){
      return row.__is_new === false && row.__index !== args.row.__index;
    });

    if(rows.length > 0){
      if(rows[0].IDCFOPINTEREST === 'T')
        return "Já existe CFOP cadastrado para todas as operações";
      else if(args.row.IDCFOPINTEREST === 'T')
        return "Já existe CFOP cadastrado para alguma outra operação";
    }
  };

  var validarCfopNCM = function(args){
    var rows = args.owner.widget.dataSource.data.filter(function(row){
      return row.__is_new === false && row.__index !== args.row.__index && row.CDCLASFISC === args.row.CDCLASFISC;
    });

    if(rows.length > 0){
      if(rows[0].IDCFOPINTEREST === 'T')
        return "Já existe CFOP cadastrado para esse NCM em todas as operações";
      else if(args.row.IDCFOPINTEREST === 'T')
        return "Já existe CFOP cadastrado para esse NCM em alguma outra operação";
    }
  };

  var validarCfopProduto = function(args){
    var rows = args.owner.widget.dataSource.data.filter(function(row){
      return row.__is_new === false && row.__index !== args.row.__index && row.CDPRODUTO === args.row.CDPRODUTO;
    });

    if(rows.length > 0){
      if(rows[0].IDCFOPINTEREST === 'T')
        return "Já existe CFOP cadastrado para esse produto em todas as operações";
      else if(args.row.IDCFOPINTEREST === 'T')
        return "Já existe CFOP cadastrado para esse produto alguma outra operação";
    }
  };

  this.salvarCfopPadrao = function(args){
    var msgValidacao = validarCfopPadrao(args);
    if(msgValidacao)
      ScreenService.showMessage(msgValidacao);
    else{
      args.owner.widget.dataSource.name =  "df#/crud/cfopadraoregfis";
    	General.saveCrud(args.owner.widget).then(function(response){
    	  desativarRegra(args.owner.widget.getParent().getParent());
    		controlarStatusWidgets(args.owner.widget, 'cfopPadrao', true);
    	});
  	  args.owner.widget.dataSource.name =  "df#/cfopadraoregfis_msgcfopadrao";
    }
	};

	var validarMensagem = function(args){
    var rows = args.owner.widget.dataSource.data.filter(function(row){
      return row.__is_new === false && row.__index !== args.row.__index && row.DSMENSREGRAFISC.toUpperCase() === args.row.DSMENSREGRAFISC.toUpperCase();
    });

    if(rows.length > 0){
      return "Mensagem já cadastrada";
    }
  };

  this.salvarMensagem = function(args){
    var msgValidacao = validarMensagem(args);
    if(msgValidacao)
      ScreenService.showMessage(msgValidacao);
    else{
    	General.saveCrud(args.owner.widget).then(function(response){
    	  desativarRegra(args.owner.widget.getParent());
    	});
    }
	};

	this.excluirCfopPadrao = function(args){
		args.owner.widget.dataSource.name =  "df#/crud/cfopadraoregfis";
		General.deleteCrud(args.owner.widget).then(function(){
			args.owner.widget.dataSource.name =  "df#/cfopadraoregfis_msgcfopadrao";
			desativarRegra(args.owner.widget.getParent().getParent());
			removerRowExcluida(args);
		});
	};

  this.salvarCfopNCM = function(args){
    var msgValidacao = validarCfopNCM(args);
    if(msgValidacao)
      ScreenService.showMessage(msgValidacao);
    else{
      args.owner.widget.dataSource.name =  "df#/crud/cfopncmregrafis";
    	General.saveCrud(args.owner.widget).then(function(response){
    	  desativarRegra(args.owner.widget.getParent().getParent());
    		controlarStatusWidgets(args.owner.widget, 'cfopNCM', true);
    	});
  	  args.owner.widget.dataSource.name =  "df#/cfopncmregrafis_msgcfopncm";
    }
	};

	this.excluirCfopNCM = function(args){
		args.owner.widget.dataSource.name =  "df#/crud/cfopncmregrafis";
		General.deleteCrud(args.owner.widget).then(function(){
			args.owner.widget.dataSource.name =  "df#/cfopncmregrafis_msgcfopncm";
			desativarRegra(args.owner.widget.getParent().getParent());
			removerRowExcluida(args);
		});
	};

	this.salvarCfopProduto = function(args){
    var msgValidacao = validarCfopProduto(args);
    if(msgValidacao)
      ScreenService.showMessage(msgValidacao);
    else{
      args.owner.widget.dataSource.name =  "df#/crud/cfopregrafiscal";
    	General.saveCrud(args.owner.widget).then(function(response){
    	  desativarRegra(args.owner.widget.getParent().getParent());
    		controlarStatusWidgets(args.owner.widget, 'cfopProduto', true);
    	});
  	  args.owner.widget.dataSource.name =  "df#/cfopregrafiscal_msgcfopregfisc";
    }
	};

	this.excluirCfopProduto = function(args){
		args.owner.widget.dataSource.name =  "df#/crud/cfopregrafiscal";
		General.deleteCrud(args.owner.widget).then(function(){
			args.owner.widget.dataSource.name =  "df#/cfopregrafiscal_msgcfopregfisc";
			desativarRegra(args.owner.widget.getParent().getParent());
			removerRowExcluida(args);
		});
	};

  this.actNovoGrupoFiscal = function(args){
    if(args.owner.widget.getParent().currentRow.IDUTGRUPOFISCAL === 'S')
      args.owner.widget.newRow();
    else
      ScreenService.showMessage("Inclusão não permitida <br/> Trata-se de uma regra fiscal que não utiliza grupo fiscal");
  };

  this.salvarGrupoFiscal = function(args){
  	args.owner.widget.dataSource.name =  "df#/crud/grupofiscregra";
  	General.saveCrud(args.owner.widget).then(function(response){
  	  desativarRegra(args.owner.widget.getParent());
  	});
	  args.owner.widget.dataSource.name =  "df#/grupofiscregra_grupofiscal";
	};

	this.excluirGrupoFiscal = function(args){
		args.owner.widget.dataSource.name =  "df#/crud/grupofiscregra";
		General.deleteCrud(args.owner.widget).then(function(){
			args.owner.widget.dataSource.name =  "df#/grupofiscregra_grupofiscal";
			desativarRegra(args.owner.widget.getParent());
			removerRowExcluida(args);
		});
	};

	this.salvarRegra = function(args){
		let widget = 	args.owner.widget;
  	widget.currentRow.IDATIVREGRAFISC = 'N';
  	widget.saveRow().then(function(){
			widget.reload();
			widget.view();
  	  controlarStatusWidgets(args.owner.widget, 'regraFiscal', true);
  	  controlarVisibilidadeActAtivarRegra(args.owner.widget, true);
  	  General.callRestEngine("df#/saveAgrupImposto", "Row", args.owner.widget.currentRow).then(() => {
  	  	args.owner.widget.widgets[0].reload();
  	  });
  	});
	};

	this.excluirRegra = function(args){
		General.deleteCrud(args.owner.widget).then(function(){
			removerRowExcluida(args);
		});
	};

	this.salvarImposto = function(args){
		if(args.owner.widget.isValid()) {
	  	args.owner.widget.dataSource.name =  "df#/crud/impregrafiscal";
	  	General.saveCrud(args.owner.widget).then(function(response){
	  	  desativarRegra(args.owner.widget.getParent());
	  	  controlarStatusWidgets(args.owner.widget, 'imposto', true);
	  	});
		  args.owner.widget.dataSource.name =  "df#/impregrafiscal_imposto_impostonfe";
		}
	};

	this.excluirImposto = function(args){
		args.owner.widget.dataSource.name =  "df#/crud/impregrafiscal";
		General.deleteCrud(args.owner.widget).then(function(){
			args.owner.widget.dataSource.name =  "df#/impregrafiscal_imposto_impostonfe";
			desativarRegra(args.owner.widget.getParent());
			removerRowExcluida(args);
		});
	};

	this.salvarImpostoPadrao = function(widget){
	  	widget.dataSource.name =  "df#/crud/dadosfiscregra";
	  	var validaOrigem  = false;
	  	var validaDestino = false;
	  	widget.fieldGroups.forEach((fieldGroup) => {
	  		if(fieldGroup.name == 'origem') {
	  			validaOrigem = fieldGroup.isVisible;
	  		}	
	  		else if(fieldGroup.name == 'destino') {
	  			validaDestino = fieldGroup.isVisible;
	  		} 
	  	});
  	
	  	if(!validaOrigem) {
	  		widget.getField('NMPAISORIG').validations = {};
	  		widget.getField('NMESTAORIG').validations = {};
	  		widget.getField('NMMUNICIPIOORIG').validations = {};
	  	}
	  	else {
	  		if(widget.currentRow.IDTIPOORIGFISC == 'E') {
				widget.getField('NMPAISORIG').validations = {required: true};
		  		widget.getField('NMESTAORIG').validations = {required: true};
		  		widget.getField('NMMUNICIPIOORIG').validations = {required: true};
	  		}
	  	}
	  	if(!validaDestino) {
	  		widget.getField('NMPAISDEST').validations = {};
	  		widget.getField('NMESTADEST').validations = {};
	  		widget.getField('NMMUNICIPIODEST').validations = {};
	  	}
	  	else {
	  		if(widget.currentRow.IDTIPODESTFISC == 'E') {
				widget.getField('NMPAISORIG').validations = {required: true};
		  		widget.getField('NMESTAORIG').validations = {required: true};
		  		widget.getField('NMMUNICIPIOORIG').validations = {required: true};
	  		}
	  	}
	  	
	  	General.saveCrud(widget).then(function(response){
	  	  desativarRegra(widget.getParent().getParent());
	  		controlarStatusWidgets(widget, 'impostoPadrao', true);
	  	});
	  widget.dataSource.name =  "df#/dadosfiscregra_centralizada";
	};
	
	this.validarNatureza = function(args, num){
		var aux = [];
		var widget = args.owner.widget;
		aux.push({name: "NRORG", operator: "=",value: getLocalVar('NRORG')});
		aux.push({name: "CDSITUTRIB", operator: "=",value: widget.currentRow.CDSITUTRIB});
		aux.push({name: "CDNATUREC", operator: "=", value: widget.currentRow.CDNATUREC});
		aux.push({name: "DTINCLUSAO", operator: "=", value: moment().format("DD/MM/YYYY")});
		aux.push({name: "VRALIQUOTA", operator: "=", value: widget.currentRow.VRALIQUOTA});
		General.callRestEngine('df#/crud/dadosfiscregra_validacao', "FilterData", aux)
		.then( (nome)=> {
			let VALIDADE = true; 
			
			if(nome.dataset.dadosfiscregra_validacao[0]){
				if(nome.dataset.dadosfiscregra_validacao[0].VALIDADE == "Natureza da Receita não cadastrada/inválida para o CST"){
					VALIDADE = false;
					ScreenService.showMessage("Natureza da Receita não cadastrada/inválida para o CST");
				}
			}
			
			if(VALIDADE){
				if(num==0)
					this.salvarImpostoPadrao(widget);
				if(num==1)
					this.salvarImpostoNCM(args);
				if(num==2)
					this.salvarImpostoProduto(args);
			}
		});
	};

 /*	this.salvarImpostoPadrao = function(widget){
		widget.dataSource.name = "df#/crud/dadosfiscregra";
		General.saveCrud(widget).then(function(response){
			desativarRegra(widget.getParent().getParent());
			controlarStatusWidgets(widget, 'impostoPadrao', true);
		}).catch( e => e);
		widget.dataSource.name =  "df#/dadosfiscregra_centralizada";

	};*/
	
	this.deletarRegraImposto = function(args){
		ScreenService.confirmMessage('Todas as mensagens cadastradas para esse registro serão excluídas, deseja continuar?', 'question', function (success) {
      Ger80900_ParametrizacaoFiscalCentralizada_Service.deletarRegraImposto(args.row).then(function(resp){
				General.backOrCancel(args);
				removerRowExcluida(args);
			}, requestError);
		});
	};

	this.deletarRegraCfop = function(args){
	  ScreenService.confirmMessage('Todas as mensagens cadastradas para esse registro serão excluídas, deseja continuar?', 'question', function (success) {
       Ger80900_ParametrizacaoFiscalCentralizada_Service.deletarRegraCfop(args.row).then(
	      function(resp){
	        General.backOrCancel(args);
    			removerRowExcluida(args);
	    }, requestError);
    });
	};

	this.salvarImpostoNCM = function(args){
	  		
	  	var validaOrigem  = false;
	  	var validaDestino = false;
	  	args.owner.widget.fieldGroups.forEach((fieldGroup) => {
	  		if(fieldGroup.name == 'origem') {
	  			validaOrigem = fieldGroup.isVisible;
	  		}	
	  		else if(fieldGroup.name == 'destino') {
	  			validaDestino = fieldGroup.isVisible;
	  		}
	  	});
	  	if(!validaOrigem) {
	  		args.owner.widget.getField('NMPAISORIG').validations = {};
	  		args.owner.widget.getField('NMESTAORIG').validations = {};
	  		args.owner.widget.getField('NMMUNICIPIOORIG').validations = {};
	  	}
	  	else {
	  		if(args.owner.widget.currentRow.IDTIPOORIGFISC == 'E') {
				args.owner.widget.getField('NMPAISORIG').validations = {required: true};
		  		args.owner.widget.getField('NMESTAORIG').validations = {required: true};
		  		args.owner.widget.getField('NMMUNICIPIOORIG').validations = {required: true};
	  		}
	  	}
	  	if(!validaDestino) {
	  		args.owner.widget.getField('NMPAISDEST').validations = {};
	  		args.owner.widget.getField('NMESTADEST').validations = {};
	  		args.owner.widget.getField('NMMUNICIPIODEST').validations = {};
	  	}
	  	else {
	  		if(args.owner.widget.currentRow.IDTIPODESTFISC == 'E') {
				args.owner.widget.getField('NMPAISORIG').validations = {required: true};
		  		args.owner.widget.getField('NMESTAORIG').validations = {required: true};
		  		args.owner.widget.getField('NMMUNICIPIOORIG').validations = {required: true};
	  		}
	  	}
	  	if(args.owner.widget.currentRow.__is_new) {
	  		args.owner.widget.dataSource.name =  "df#/crud/dadosfiscncm";	  
	  		General.callRestEngine("df#/getMaxdadosfiscncm", "Row", args.owner.widget.currentRow).then((response) => {
					args.owner.widget.currentRow.NRSEQDADOSFISC = response.dataset.getMaxdadosfiscncm.NRSEQDADOSFISC;
			  	General.saveCrud(args.owner.widget).then(function(response){
			  		desativarRegra(args.owner.widget.getParent().getParent());
			  		controlarStatusWidgets(args.owner.widget, 'impostoNCM', true);
			  		args.owner.widget.dataSource.name =  "df#/dadosfiscncm_centralizada";
			  		args.owner.widget.reload();
			  	}, (error) => {
				  		args.owner.widget.dataSource.name =  "df#/dadosfiscncm_centralizada";			  		
				  		args.owner.widget.reload();
			  	});
		  	});
	  	}
	  	else {
		  		args.owner.widget.dataSource.name =  "df#/crud/dadosfiscncm";	  
	  			General.saveCrud(args.owner.widget).then(function(response){
			  		desativarRegra(args.owner.widget.getParent().getParent());
			  		controlarStatusWidgets(args.owner.widget, 'impostoNCM', true);
			  		args.owner.widget.dataSource.name =  "df#/dadosfiscncm_centralizada";
			  		args.owner.widget.reload();
			  	}, (error) => {
				  		args.owner.widget.dataSource.name =  "df#/dadosfiscncm_centralizada";
				  		args.owner.widget.reload();
			  	});
	  	}
	  
	};

	this.salvarImpostoProduto = function(args){
	  	var validaOrigem  = false;
	  	var validaDestino = false;
	  	args.owner.widget.fieldGroups.forEach((fieldGroup) => {
	  		if(fieldGroup.name == 'origem') {
	  			validaOrigem = fieldGroup.isVisible;
	  		}	
	  		else if(fieldGroup.name == 'destino') {
	  			validaDestino = fieldGroup.isVisible;
	  		}
	  	});
	  	if(!validaOrigem) {
	  		args.owner.widget.getField('NMPAISORIG').validations = {};
	  		args.owner.widget.getField('NMESTAORIG').validations = {};
	  		args.owner.widget.getField('NMMUNICIPIOORIG').validations = {};
	  	}
	  	else {
	  		if(args.owner.widget.currentRow.IDTIPOORIGFISC == 'E') {
				args.owner.widget.getField('NMPAISORIG').validations = {required: true};
		  		args.owner.widget.getField('NMESTAORIG').validations = {required: true};
		  		args.owner.widget.getField('NMMUNICIPIOORIG').validations = {required: true};
	  		}
	  	}
	  	if(!validaDestino) {
	  		args.owner.widget.getField('NMPAISDEST').validations = {};
	  		args.owner.widget.getField('NMESTADEST').validations = {};
	  		args.owner.widget.getField('NMMUNICIPIODEST').validations = {};
	  	}
	  	else {
	  		if(args.owner.widget.currentRow.IDTIPODESTFISC == 'E') {
				args.owner.widget.getField('NMPAISDEST').validations = {required: true};
		  		args.owner.widget.getField('NMESTADEST').validations = {required: true};
		  		args.owner.widget.getField('NMMUNICIPIODEST').validations = {required: true};
	  		}
	  	}
	  	
		if(args.owner.widget.isValid()){
				var currentRow = args.owner.widget.currentRow;
				var dataset = args.owner.widget.dataSource.data;
				if(currentRow.__is_new) {
					General.callRestEngine("df#/getMaxdadosfiscprod", "Row", currentRow).then((response) => {
						currentRow.NRSEQDADOSFISC = response.dataset.getMaxdadosfiscprod.NRSEQDADOSFISC;
				  	args.owner.widget.dataSource.name =  "df#/crud/dadosfiscprod";
				  	currentRow.VRPERREDBC = (currentRow.VRPERREDBC) ? currentRow.VRPERREDBC.replace(/([,])/g,".") : null;
				  	currentRow.VR1PERREDBC = (currentRow.VR1PERREDBC) ? currentRow.VR1PERREDBC.replace(/([,])/g,".") : null;
				  	currentRow.VRINIFAIXA = (currentRow.VRINIFAIXA) ? currentRow.VRINIFAIXA.replace(/([,])/g,".") : null;
				  	currentRow.VRFIMFAIXA = (currentRow.VRFIMFAIXA) ? currentRow.VRFIMFAIXA.replace(/([,])/g,".") : null;
				  	currentRow.VRPARDEDUZ = (currentRow.VRPARDEDUZ) ? currentRow.VRPARDEDUZ.replace(/([,])/g,".") : null;
				  	currentRow.VRPBCSUBTRIB = (currentRow.VRPBCSUBTRIB) ? currentRow.VRPBCSUBTRIB.replace(/([,])/g,".") : null;
				  	currentRow.VRALIQUOTA = (currentRow.VRALIQUOTA) ? currentRow.VRALIQUOTA.replace(/([,])/g,".") : null;
				  	currentRow.VRADICALIQFECP = currentRow.VRADICALIQFECP ? currentRow.VRADICALIQFECP.replace(/([,])/g,".") : null;
				  	currentRow.IDUTUFDEST = args.owner.widget.getParent().currentRow.IDUTUFDEST;
		  		  currentRow.IDUTUFORIG = args.owner.widget.getParent().currentRow.IDUTUFORIG;
						if(this.produtoRepetido(args.owner.widget.dataSource.data ,currentRow  ).length > 0 ) {
								 ScreenService.showMessage("Operação Bloqueada! Produto já cadastrado");
						}
						else {
						  	General.saveCrud(args.owner.widget).then(function(response){
						  	  desativarRegra(args.owner.widget.getParent().getParent());
							  	controlarStatusWidgets(args.owner.widget, 'impostoProduto', true);
						  	});
						  	args.owner.widget.dataSource.name =  "df#/dadosfiscprod_centralizada";
						  	args.owner.widget.reload();
						}

					});
				}
				else {
			  	args.owner.widget.dataSource.name =  "df#/crud/dadosfiscprod";
			  	currentRow.VRPERREDBC = (currentRow.VRPERREDBC) ? currentRow.VRPERREDBC.replace(/([,])/g,".") : null;
			  	currentRow.VR1PERREDBC = (currentRow.VR1PERREDBC) ? currentRow.VR1PERREDBC.replace(/([,])/g,".") : null;
			  	currentRow.VRINIFAIXA = (currentRow.VRINIFAIXA) ? currentRow.VRINIFAIXA.replace(/([,])/g,".") : null;
			  	currentRow.VRFIMFAIXA = (currentRow.VRFIMFAIXA) ? currentRow.VRFIMFAIXA.replace(/([,])/g,".") : null;
			  	currentRow.VRPARDEDUZ = (currentRow.VRPARDEDUZ) ? currentRow.VRPARDEDUZ.replace(/([,])/g,".") : null;
			  	currentRow.VRPBCSUBTRIB = (currentRow.VRPBCSUBTRIB) ? currentRow.VRPBCSUBTRIB.replace(/([,])/g,".") : null;
			  	currentRow.VRALIQUOTA = (currentRow.VRALIQUOTA) ? currentRow.VRALIQUOTA.replace(/([,])/g,".") : null;
			  	currentRow.VRADICALIQFECP = currentRow.VRADICALIQFECP ? currentRow.VRADICALIQFECP.replace(/([,])/g,".") : null;
			  	currentRow.IDUTUFDEST = args.owner.widget.getParent().currentRow.IDUTUFDEST;
	  		  currentRow.IDUTUFORIG = args.owner.widget.getParent().currentRow.IDUTUFORIG;
	  			
	  			if(this.origemDestinoChanged(args.owner.widget)){
	  				
						if(this.produtoRepetido(args.owner.widget.dataSource.data , currentRow  ).length > 0 ) {
								 ScreenService.showMessage("Operação Bloqueada! Produto já cadastrado");
						}	else {
							  	General.saveCrud(args.owner.widget).then(function(response){
							  	  desativarRegra(args.owner.widget.getParent().getParent());
								  	controlarStatusWidgets(args.owner.widget, 'impostoProduto', true);
							  	});
							  	args.owner.widget.dataSource.name =  "df#/dadosfiscprod_centralizada";
							  	args.owner.widget.reload();
							}
						}else {
						  	General.saveCrud(args.owner.widget).then(function(response){
						  	  desativarRegra(args.owner.widget.getParent().getParent());
							  	controlarStatusWidgets(args.owner.widget, 'impostoProduto', true);
						  	});
						  	args.owner.widget.dataSource.name =  "df#/dadosfiscprod_centralizada";
						  	args.owner.widget.reload();
						}
				}

		}
	};

	this.validarDataSourceFilterCST = function(args){
	 // args.owner.widgets.forEach(function(widget){
	 //   if(args.row.CDIMPOSTONFE){
  // 	    widget.getField("CDSITUTRIB").dataSourceFilter = [{
  // 						"name": "CDIMPOSTO",
  // 						"operator": "=",
  // 						"value": "@General.getFieldValue('CDIMPOSTONFE', 'imposto')@"
  // 					}]
  // 	  }
  // 	  else{
  // 		  widget.getField("CDSITUTRIB").dataSourceFilter = [];
  // 	  }
	 // });
	};

	var setFieldGroupVisibleStatus = function(fieldgroups, name, status){
	  var fieldgroup = fieldgroups.find(function(fieldgroup){
      return fieldgroup.name === name;
    });

    fieldgroup.isVisible = status;
    
	};

	var validarCamposBaseCalculo = function(widget, parentCurrentRow){
	  var contador = 0;

	  if(parentCurrentRow.IDUTREDUBC === 'S'){
	    widget.getField('VRPERREDBC').isVisible = true;
	  }
	  else{
	    widget.getField('VRPERREDBC').isVisible = false;
	    contador++;
	  }

	  if(parentCurrentRow.ID1UTREDUBC === 'S'){
	    widget.getField('VR1PERREDBC').isVisible = true;
	  }
	  else{
	    widget.getField('VR1PERREDBC').isVisible = false;
	    contador++;
	  }

	  if(parentCurrentRow.IDUTPARCDED === 'S'){
	    widget.getField('VRPARDEDUZ').isVisible = true;
	  }
	  else{
	    widget.getField('VRPARDEDUZ').isVisible = false;
	    contador++;
	  }

	  if(contador === 3){
      setFieldGroupVisibleStatus(widget.fieldGroups, 'baseCalculo', false);
	  }
	  else{
	    setFieldGroupVisibleStatus(widget.fieldGroups, 'baseCalculo', true);
	  }
	};

	var validarCamposFaixaValores = function(widget, parentCurrentRow){
	  if(parentCurrentRow.IDUTFAIXVLR === 'S'){
	    setFieldGroupVisibleStatus(widget.fieldGroups, 'faixaValores', true);
	  }
	  else{
	    setFieldGroupVisibleStatus(widget.fieldGroups, 'faixaValores', false);
	  }
	};

	var validarCamposDestino = function(widget, parentCurrentRow){
	  if(parentCurrentRow.IDUTUFDEST === 'S'){
	    if(parentCurrentRow.IDUTMUNIORIG === 'D'){
	      widget.getField('NMMUNICIPIODEST').isVisible = true;
	    }else{
	      widget.getField('NMMUNICIPIODEST').isVisible = false;

	    }
	    setFieldGroupVisibleStatus(widget.fieldGroups, 'destino', true);
	  }
	  else{
	    setFieldGroupVisibleStatus(widget.fieldGroups, 'destino', false);
	  }
	};

	var validarCamposOrigem = function(widget, parentCurrentRow){
	  if(parentCurrentRow.IDUTUFORIG === 'S'){
	    if(parentCurrentRow.IDUTMUNIORIG === 'O'){
	      widget.getField('NMMUNICIPIOORIG').isVisible = true;

	    }else{
	      widget.getField('NMMUNICIPIOORIG').isVisible = false;

	    }
	    setFieldGroupVisibleStatus(widget.fieldGroups, 'origem', true);
	  }
	  else{
	    setFieldGroupVisibleStatus(widget.fieldGroups, 'origem', false);
	  }
	};

	var validarCamposSubstTribut = function(widget, parentCurrentRow){
	  if(parentCurrentRow.CDIMPOSTONFE == 80){
	    setFieldGroupVisibleStatus(widget.fieldGroups, 'substTribut', true);
	  }
	  else{
	    setFieldGroupVisibleStatus(widget.fieldGroups, 'substTribut', false);
	  }
	};


	var validarCamposRegraImposto = function(widget, parentCurrentRow){
    	validarCamposBaseCalculo(widget, parentCurrentRow);
		validarCamposFaixaValores(widget, parentCurrentRow);
    	validarCamposDestino(widget, parentCurrentRow);
    	validarCamposOrigem(widget, parentCurrentRow);
    	validarCamposSubstTribut(widget, parentCurrentRow);
	};

	this.nmpaisOrigOnChange = function(widget, manteField){
		widget.getField('NMESTAORIG').readOnly = !widget.currentRow.NMPAISORIG;
		widget.getField('NMESTAORIG').setValue('');
		widget.getField('NMESTAORIG').reload();
		widget.getField('NMMUNICIPIOORIG').reload();
		widget.getField('NMMUNICIPIOORIG').setValue('');
		widget.getField('NMMUNICIPIOORIG').readOnly = true;
	  
	};

	this.nmestadoOrigOnChange = function(widget){
		widget.getField('NMMUNICIPIOORIG').readOnly = !widget.currentRow.NMESTAORIG;
		widget.getField('NMMUNICIPIOORIG').setValue('');
		widget.getField('NMMUNICIPIOORIG').reload();
	 
	};

	this.nmpaisDestOnChange = function(widget){
	  widget.getField('NMESTADEST').readOnly = !widget.currentRow.NMPAISDEST;
	  widget.getField('NMESTADEST').setValue('');
	  widget.getField('NMESTADEST').reload();
	  widget.getField('NMMUNICIPIODEST').reload();
	  widget.getField('NMMUNICIPIODEST').setValue('');
	  widget.getField('NMMUNICIPIODEST').readOnly = true;
	};

	this.nmestadoDestOnChange = function(widget){
	  widget.getField('NMMUNICIPIODEST').readOnly = !widget.currentRow.NMESTADEST;
	  widget.getField('NMMUNICIPIODEST').setValue('');
	  widget.getField('NMMUNICIPIODEST').reload();
	};

	var validarCampoCstObrigatorio = function(widget, parentCurrentRow){
	  if(parentCurrentRow.IDUTCSTREGFISC === 'S')
	    widget.getField('CDSITUTRIB').validations = {"required": true};
   else
	  delete widget.getField('CDSITUTRIB').validations;
	};

	var validarCampoCfopObrigatorio = function(widget, parentCurrentRow){
	  if(parentCurrentRow.IDUTCSTREGFISC === 'S')
	    widget.getField('CDFISCOPER').validations = {"required": true};
    else
	    delete widget.getField('CDFISCOPER').validations;
	};

	this.validacoesRegraImpostoAfterMoveRow = function(args){
	  validarCamposRegraImposto(args.owner, args.owner.getParent().currentRow);
	  validarCampoCstObrigatorio(args.owner, args.owner.getParent().currentRow);
	  validarCampoCfopObrigatorio(args.owner, args.owner.getParent().currentRow);
	  /*this.nmpaisOrigOnChange(args.owner);
	  this.nmestadoOrigOnChange(args.owner);
	  this.nmpaisDestOnChange(args.owner);
	  this.nmestadoDestOnChange(args.owner);*/
	};
	
	/*
	this.setMaskCNPJOrCpfFornecedor = function(widget) {
	    let data = widget.dataSource.data;
	    let inscFormatada;
	    data.forEach((value) => {
	        if(value.NRINSJURFORN.length === 14) inscFormatada = value.NRINSJURFORN.substr(0, 2) + '.' + value.NRINSJURFORN.substr(2, 3) + '.' + value.NRINSJURFORN.substr(5, 3) + '/' + value.NRINSJURFORN.substr(8, 4) + '-' + value.NRINSJURFORN.substr(12, 2);
          else if(value.NRINSJURFORN.length === 11) inscFormatada = value.NRINSJURFORN.substr(0, 3) + '.' + value.NRINSJURFORN.substr(3, 3) + '.' + value.NRINSJURFORN.substr(6, 3) + '-' + value.NRINSJURFORN.substr(9, 2);
          value.NRINSJURFORN = inscFormatada;
	    });
	};
	
	this.setMaskCNPJOrCpfCliente = function(widget) {
	    let data = widget.dataSource.data;
	    let inscFormatada;
	    data.forEach((value) => {
	        if(value.NRINSJURCLIE.length === 14) inscFormatada = value.NRINSJURCLIE.substr(0, 2) + '.' + value.NRINSJURCLIE.substr(2, 3) + '.' + value.NRINSJURCLIE.substr(5, 3) + '/' + value.NRINSJURCLIE.substr(8, 4) + '-' + value.NRINSJURCLIE.substr(12, 2);
          else if(value.NRINSJURCLIE.length === 11) inscFormatada = value.NRINSJURCLIE.substr(0, 3) + '.' + value.NRINSJURCLIE.substr(3, 3) + '.' + value.NRINSJURCLIE.substr(6, 3) + '-' + value.NRINSJURCLIE.substr(9, 2);
          value.NRINSJURCLIE = inscFormatada;
	    });
	};
	*/
	
	this.removeCustomFilter = function(args) {
	    args.e.preventDefault();
	    var widget = args.owner;
	    var filter = [];
	    filter = widget.dataSourceFilter.filter((item) =>  {
	      return (item.name != "CDIMPOSTO");
	    });
	    widget.dataSourceFilter = filter;
	    widget.reload();
	};
	
	this.getFieldValue = function(fieldName, widgetName) {
	    if(templateManager.container) {
	    	if(templateManager.container.name == 'ger80900_parametrizacao_fiscal_centralizada'){
	        var widget = templateManager.container.getWidget(widgetName);
	      	return widget.currentRow[fieldName];
	    	}
	    }
	    else return null;
	};
	
	this.saveWithParent = function(widget, parents) {
		if(Array.isArray(parents)) {
        parents.forEach(function(parent){
            widget.currentRow[parent] = widget.getParent().currentRow[parent];
        });
        return General.widgetSave(widget);
    }
    else throw("\'Parents\' property should be an array, " + typeof parents + " given.");
	};
	
	this.initializeMvaECompBC = function(widget){
		
		var widgetPai = widget.getParent();
		var mva = widget.getField("IDCONSMVAPMC");
		var compBC = widget.getField("IDCONSCOMPBCPMC");
		var adcAliquota = widget.getField("VRADICALIQFECP");
		var enqLegal =  widget.getField("CDENQLEGALIPI");
		var bcICMS =  widget.getField("IDMODBCNFE");
		var bcICMSST =  widget.getField("IDMODBCSTNFE");
		var cest =  widget.getField("CDCEST");
		
		var showIMP80 = widgetPai.currentRow.CDIMPOSTONFE == "80";
		mva.isVisible = showIMP80;
    mva.showOnForm = showIMP80;
    mva.showOnView = showIMP80;
    mva.showOnList = showIMP80;
			
		compBC.isVisible = showIMP80;
		compBC.showOnForm = showIMP80;
    compBC.showOnView = showIMP80;
    compBC.showOnList = showIMP80;
    
		var showAdcAli  =  (widgetPai.currentRow.CDIMPOSTONFE == '00' || widgetPai.currentRow.CDIMPOSTONFE == '80' || widgetPai.currentRow.CDIMPOSTONFE == '99');
		adcAliquota.isVisible  = showAdcAli;
		adcAliquota.showOnForm = showAdcAli;
    adcAliquota.showOnView = showAdcAli;
    adcAliquota.showOnList = showAdcAli;
    
    var showIPI = (widgetPai.currentRow.CDIMPOSTONFE == '10' );
		enqLegal.isVisible = showIPI;
		enqLegal.showOnForm = showIPI;
    enqLegal.showOnView = showIPI;
    enqLegal.showOnList = showIPI;
    
    bcICMS.isVisible = showIPI;
		bcICMS.showOnForm = showIPI;
    bcICMS.showOnView = showIPI;
    bcICMS.showOnList = showIPI;
    
    bcICMSST.isVisible = showIPI;
		bcICMSST.showOnForm = showIPI;
    bcICMSST.showOnView = showIPI;
    bcICMSST.showOnList = showIPI;
    
    var showCEST = (widgetPai.currentRow.CDIMPOSTONFE == '80');
    cest.isVisible = showCEST;
		cest.showOnForm = showCEST;
    cest.showOnView = showCEST;
    cest.showOnList = showCEST;


		widget.reload();
	};
	
	this.validaMvaECompBC = function(widget){
		var mva = widget.getField("IDCONSMVAPMC");
		var compBC = widget.getField("IDCONSCOMPBCPMC");
		
		if(widget.currentRow.IDPMCBASECA == "S"){
			mva.readOnly = false;
			compBC.readOnly = false;
		} else{
			mva.readOnly = true;
			compBC.readOnly = true;
		}
	};
	
	this.origemOnChange =  function(widget) {
		var tipoOrigem =	widget.getField("IDTIPOORIGFISC"); 
		var pais = 	widget.getField("NMPAISORIG"); 
		var estado = 	widget.getField("NMESTAORIG"); 
			if(tipoOrigem.value() === "T"){
				pais.readOnly = true; 
				pais.setValue(null);
				estado.readOnly = true;
				estado.setValue(null);
				widget.currentRow.SGESTAORIG = null;
				widget.currentRow.CDPAISORIG = null;

					if(pais.validations.required)
						delete pais.validations.required;
						
					if(estado.validations.required)
						delete estado.validations.required;
				
			}else{
				
				pais.readOnly = false; 
				estado.readOnly = false;
				pais.validations = {};
				pais.validations.required = true;

				estado.validations = {};
				estado.validations.required = true;
				
			}
			pais.reload();
			estado.reload();
		
	};
	
	this.destinoOnChange =  function(widget) {
		var tipoDestino =	widget.getField("IDTIPODESTFISC"); 
		var pais = 	widget.getField("NMPAISDEST"); 
		var estado = 	widget.getField("NMESTADEST"); 
			if(tipoDestino.value() === "T"){
				
				pais.readOnly = true; 
				pais.setValue(null);
				widget.currentRow.CDPAISDEST = null;
				widget.currentRow.SGESTADEST = null;
				if(pais.validations.required)
					delete pais.validations.required;
				
				estado.readOnly = true;
				estado.setValue(null);
				if(estado.validations.required)
					delete estado.validations.required;
				
			}else{
				
				pais.readOnly = false; 
				estado.readOnly = false;
				pais.validations = {};
				pais.validations.required = true;

				estado.validations = {};
				estado.validations.required = true;
				
			}
			pais.reload();
			estado.reload();
		
	};
	
	this.origemDestinoChanged = function(widget){
		let changed = false; 
		
		changed = widget.backupRow.CDPAISDEST != widget.currentRow.CDPAISDEST  || changed ? true : false;
		changed = widget.backupRow.CDPAISORIG !=  widget.currentRow.CDPAISORIG || changed ? true : false; 
		changed = widget.backupRow.SGESTADEST !=  widget.currentRow.SGESTADEST || changed ? true : false; 
		changed = widget.backupRow.SGESTAORIG !=  widget.currentRow.SGESTAORIG || changed ? true : false; 
		changed = widget.backupRow.IDTIPODESTFISC !=  widget.currentRow.IDTIPODESTFISC || changed ? true : false;
		changed = widget.backupRow.IDTIPOORIGFISC !=  widget.currentRow.IDTIPOORIGFISC || changed ? true : false; 
		
		return changed;
	};
	
	this.produtoRepetido = function(produtos, currentRow){
		
		return produtos
		
		.filter(produto => produto.CDPRODUTO === currentRow.CDPRODUTO 
																			&& produto.__createdLocal == undefined 
																			&& produto.__index != currentRow.__index)
		
		.filter( produto => {
			
				// ORIGEM E DESTINO IGUAIS
				if(currentRow.CDPAISDEST == produto.CDPAISDEST 
					&& currentRow.SGESTADEST == produto.SGESTADEST 
					&& currentRow.CDPAISORIG == produto.CDPAISORIG 
					&& currentRow.SGESTAORIG == produto.SGESTAORIG 
				) return true;
				
				
				// ORIGENS IGUAiS, MAS DESTINOS DIFERENTES 
				if(( currentRow.CDPAISDEST != produto.CDPAISDEST 
					&& currentRow.SGESTADEST != produto.SGESTADEST 
					&& currentRow.CDPAISORIG == produto.CDPAISORIG 
					&& currentRow.SGESTAORIG == produto.SGESTAORIG ) ){
							return (produto.IDTIPODESTFISC == 'T'  &&  currentRow.IDTIPODESTFISC == 'E' )
							|| (produto.IDTIPODESTFISC == 'E'  &&  currentRow.IDTIPODESTFISC == 'T' ) ? true : false;
					}
				
				// ORIGENS DIFERENTES, MAS DESTINOS IGUAIS 
				if( currentRow.CDPAISDEST == produto.CDPAISDEST 
					&& currentRow.SGESTADEST == produto.SGESTADEST 
					&& currentRow.CDPAISORIG != produto.CDPAISORIG 
					&& currentRow.SGESTAORIG != produto.SGESTAORIG ){
						return (produto.IDTIPOORIGFISC == 'T'  &&  currentRow.IDTIPOORIGFISC == 'E' )
							|| (produto.IDTIPOORIGFISC == 'E'  &&  currentRow.IDTIPOORIGFISC == 'T' ) 
							? true : false;
				}
				
				//Origem e destino diferentes
				if( currentRow.CDPAISDEST != produto.CDPAISDEST 
					&& currentRow.SGESTADEST != produto.SGESTADEST 
					&& currentRow.CDPAISORIG != produto.CDPAISORIG 
					&& currentRow.SGESTAORIG != produto.SGESTAORIG ){
						return (produto.IDTIPOORIGFISC == 'T'  &&  currentRow.IDTIPOORIGFISC == 'E' )
							|| (produto.IDTIPOORIGFISC == 'E'  &&  currentRow.IDTIPOORIGFISC == 'T' )
							|| (produto.IDTIPODESTFISC == 'T'  &&  currentRow.IDTIPODESTFISC == 'E' )
							|| (produto.IDTIPODESTFISC == 'E'  &&  currentRow.IDTIPODESTFISC == 'T' )
							? true : false;
				}
			
		});
	
		
	};

	
};

Configuration(function(ContextRegister) {
  ContextRegister.register("Ger80900_ParametrizacaoFiscalCentralizada_Controller", Ger80900_ParametrizacaoFiscalCentralizada_Controller);
});

// FILE: ./assets/js/controllers/Ger80902_VerificaRegraFiscal_Controller.js
var Ger80902_VerificaRegraFiscal_Controller = function(ScreenService, General, templateManager, ZhFilterPreferences) {
  this.nmpaisOrigOnChange = function(widget){
	  widget.getField('NMESTADOORIG').readOnly = !widget.currentRow.NMPAISORIG;
	  widget.getField('NMESTADOORIG').setValue('');
	};

	this.nmpaisDestOnChange = function(widget){
	  widget.getField('NMESTADODEST').readOnly = !widget.currentRow.NMPAISDEST;
	  widget.getField('NMESTADODEST').setValue('');
	};
	
	this.empresaOnChange = function(widget){
	  widget.getField('DSINSCESTA').readOnly = !widget.currentRow.CDEMPRESA;
	  widget.getField('DSINSCESTA').setValue('');
	};
	
	this.controlaFiltroFields = function(args){
		var field = args.owner.field;
		var widget = args.owner.field.widget;
		switch(field.name){
			case 'NMEMPRESA':
			case 'DSINSCESTA':
				widget.getField('DSINSCESTA').dataSourceFilter[0].value = widget.currentRow.CDEMPRESA;
				break;
			case 'IDENTRSAIDOP':
			case 'NMTIPOOPER':
				widget.getField('NMTIPOOPER').dataSourceFilter[0].value = widget.currentRow.IDENTRSAIDOP;
				break;
			case 'NMPAISORIG':
			case 'NMESTADOORIG':
				widget.getField('NMESTADOORIG').dataSourceFilter[0].value = widget.currentRow.CDPAISORIG;
				break;
			case 'NMPAISDEST':
			case 'NMESTADODEST':
				widget.getField('NMESTADODEST').dataSourceFilter[0].value = widget.currentRow.CDPAISDEST;
				break;
		}
	};
  
	this.onChangeImpostoFiltro = function(widget) {
		if(widget.currentRow.P_CDIMPOSTO) {
			let filter = [
				{
					name: "CDIMPOSTO",
					operator: "=",
					value: widget.currentRow.P_CDIMPOSTO
				}	
			];
			General.callRestEngine('df#/crud/imposto', 'FilterData', filter).then((success) => {
				let imposto = success.dataset.imposto[0];
				changeFieldVisibilityAndRequired(widget, 'BASECALC',         imposto.IDUTREDUBC,   'S');
				changeFieldVisibilityAndRequired(widget, 'PARCDEDU',         imposto.IDUTPARCDED,  'S');
				
				changeFieldVisibilityAndRequired(widget, 'NMPAISORIG',       imposto.IDUTUFORIG,   'S');
				changeFieldVisibilityAndRequired(widget, 'NMESTADOORIG',     imposto.IDUTUFORIG,   'S');
				if(imposto.IDUTUFORIG == 'S') changeFieldVisibilityAndRequired(widget, 'NMMUNICIPIOORIG',  imposto.IDUTMUNIORIG, 'O');
				else changeFieldVisibilityAndRequired(widget, 'NMMUNICIPIOORIG',  imposto.IDUTMUNIORIG, false);
				
				changeFieldVisibilityAndRequired(widget, 'NMPAISDEST',       imposto.IDUTUFDEST,   'S');
				changeFieldVisibilityAndRequired(widget, 'NMESTADODEST',     imposto.IDUTUFDEST,   'S');
				if(imposto.IDUTUFDEST == 'S') changeFieldVisibilityAndRequired(widget, 'NMMUNICIPIODEST',  imposto.IDUTMUNIORIG, 'D');
				else changeFieldVisibilityAndRequired(widget, 'NMMUNICIPIOORIG',  imposto.IDUTMUNIORIG, false);
			});
		}
		else {
			changeFieldVisibilityAndRequired(widget, 'BASECALC',         false, true);
			changeFieldVisibilityAndRequired(widget, 'PARCDEDU',         false, true);
			changeFieldVisibilityAndRequired(widget, 'NMPAISDEST',       false, true);
			changeFieldVisibilityAndRequired(widget, 'NMESTADODEST',     false, true);
			changeFieldVisibilityAndRequired(widget, 'NMPAISORIG',       false, true);
			changeFieldVisibilityAndRequired(widget, 'NMESTADOORIG',     false, true);
			changeFieldVisibilityAndRequired(widget, 'NMMUNICIPIOORIG',  false, true);
			changeFieldVisibilityAndRequired(widget, 'NMMUNICIPIODEST',  false, true);
		}
	};
	
	this.ajustaFiltro = function(widget) {
		let row = widget.appliedFilter.FILTER_ROW;
		let dataSourceFilter = [];
		dataSourceFilter.push(getFilterObject('P_CDIMPOSTO',    row.P_CDIMPOSTO));
		dataSourceFilter.push(getFilterObject('P_CDEMPRESA',    row.CDEMPRESA));
		dataSourceFilter.push(getFilterObject('P_CDGRUPOFISC',  (row.CDGRUPOFISC)? row.CDGRUPOFISC  : null));
		dataSourceFilter.push(getFilterObject('P_ENTRSAIDIN',   (row.CDTIPOOPER) ? row.IDENTRSAIDOP : null));
		dataSourceFilter.push(getFilterObject('P_TIPOOPER',     (row.CDTIPOOPER) ? row.CDTIPOOPER   : null));
		dataSourceFilter.push(getFilterObject('P_DTENTRSAID',   row.DTENTRSAID));
		
		dataSourceFilter.push(getFilterObject('P_PAISORIG',     row.NMPAISORIG));
		dataSourceFilter.push(getFilterObject('P_ESTAORIG',     row.NMESTADOORIG));
		dataSourceFilter.push(getFilterObject('P_MUNIORIG',     row.NMMUNICIPIOORIG));
		
		dataSourceFilter.push(getFilterObject('P_PAISDEST',     row.NMPAISDEST));
		dataSourceFilter.push(getFilterObject('P_ESTADEST',     row.NMESTADODEST));
		dataSourceFilter.push(getFilterObject('P_MUNIDEST',     row.NMMUNICIPIODEST));
		
		dataSourceFilter.push(getFilterObject('P_BASECALC',     row.BASECALC));
		dataSourceFilter.push(getFilterObject('P_MUNIDEST',     row.NMMUNICIPIODEST));
		dataSourceFilter.push(getFilterObject('P_IDORIGEMPROD', row.IDORIGEMPROD));

		widget.dataSourceFilter = [];
		widget.dataSourceFilter = dataSourceFilter;	
	};
	
	function getFilterObject(filterName, filterValue) {
		return {	
			"name": filterName,
			"operator": "=",
			"value": filterValue
		};
	}
	
	function changeFieldVisibilityAndRequired(widget, fieldName, value, valueExpect) {
		let field = widget.getField(fieldName);
		field.isVisible = (value == valueExpect);
		if(fieldName !== 'BASECALC' && fieldName !== 'PARCDEDU') {
			field.validations = (value == valueExpect) ? {required: true} : {};
		}
		if(value != valueExpect) field.setValue('');
	}
};

Configuration(function(ContextRegister) {
  ContextRegister.register("Ger80902_VerificaRegraFiscal_Controller", Ger80902_VerificaRegraFiscal_Controller);
});


// FILE: ./assets/js/controllers/Ger80908_Consultoria_Fiscal_Controller.js
var Ger80908_Consultoria_Fiscal_Controller = function(General, ScreenService) {
	
	this.importarRegra = function(widget) {
		let rows = widget.getCheckedRows();
		General.callRestEngine("df#/importaRegraSystax", "DataSet", rows).then((success) => {
			ScreenService.successNotification("Regra Fiscal importada com sucesso.");
		}, (error) => {
			ScreenService.errorNotification("Ocorreu um erro ao importar a regra");
		});
	};
	
	this.retornarCentralziada = function(widget) {
	   ScreenService.openWindow("df#ger80900_parametrizacao_fiscal_centralizada");
	};
	
};

Configuration(function(ContextRegister) {
  ContextRegister.register("Ger80908_Consultoria_Fiscal_Controller", Ger80908_Consultoria_Fiscal_Controller);
});


// FILE: ./assets/js/controllers/Ger80909_Grupo_Produto_Consultoria_Controller.js
var Ger80909_Grupo_Produto_Consultoria_Controller = function(General, ScreenService, ZhFilterPreferences) {
	
	this.retornarCentralizada = function(widget) {
	   ScreenService.openWindow("df#ger80900_parametrizacao_fiscal_centralizada");
	};
	
    /** 
     * Funções Gerais
     */
     this.controlaStatusWidgets = function(widgetPai, nomeWidgetAtual, status, ignoredWidgets) {
        var allWidgets = widgetPai.widgets;

        allWidgets.filter(widget => widget.name !== nomeWidgetAtual).forEach(widget => widget.enabled = status);

        if (widgetPai.name !== nomeWidgetAtual)
            widgetPai.enabled = status;

        if (Array.isArray(ignoredWidgets))
            allWidgets.filter(widget => ignoredWidgets.indexOf(widget.name) !== -1).forEach(widget => widget.enabled = !status);
    };
    
    this.addCenarios = function(widget) {
    	widget.newRow();
    	this.controlaStatusWidgets(widget.getParent(), widget.name, false, ['consultoriaFiscal']);
    	widget.currentRow.CDGRUPPRODCON = widget.getParent().currentRow.CDGRUPPRODCON;
    };
    
    this.addProdutos = function(widget) {
    	widget.newRow();
    	this.controlaStatusWidgets(widget.getParent(), widget.name, false, ['produtos_grupo']);
    	widget.currentRow.CDGRUPPRODCON = widget.getParent().currentRow.CDGRUPPRODCON;
    };
    
    this.addRegra = function(widget) {
    	widget.newRow();
    	this.controlaStatusWidgets(widget.getParent(), widget.name, false, ['sys_regraroup']);
    	widget.currentRow.CDGRUPPRODCON = widget.getParent().currentRow.CDGRUPPRODCON;
    };
	
	this.saveCenario = function(widget) {
		if(widget.isValid()) {
			General.callRestEngine("df#/saveCenarioGrupo", "Row", widget.currentRow).then((success) => {
				ScreenService.successNotification("Cenários relacionados com sucesso.");
				ScreenService.goBack();
				widget.list();
				widget.reload();
				this.controlaStatusWidgets(widget.getParent(), widget.name, true);
			}, (error) => {
				ScreenService.errorNotification(error.error);
			});	
		}
	};
	
	this.saveProdutoGrupoCons = function(widget) {
		if(widget.isValid()) {
			General.callRestEngine("df#/saveProdutoGrupoCons", "Row", widget.currentRow).then((success) => {
				ScreenService.successNotification("Produtos relacionados com sucesso.");
				widget.list();
				widget.reload();
				this.controlaStatusWidgets(widget.getParent(), widget.name, true);
			}, (error) => {
				ScreenService.errorNotification(error.error);
			});	
		}
	};
	
	this.atualizarRegra = function(widget) {
		General.callRestEngine("df#/atualizarRegraGrupo", "DataSet", widget.getCheckedRows()).then((success) => {
			ScreenService.successNotification("Regra Fiscal atualizada com sucesso.");
		}, (error) => {
			ScreenService.errorNotification(error.error);
		});	
	};
	
	this.zerarPonteiroAtualizacao = function() {
		General.callRestEngine("df#/zerarPonteiroAtualizacao", "Row", {}).then((success) => {
			ScreenService.successNotification("Ponteiro de atualização reiniciado.");
		}, (error) => {
			ScreenService.errorNotification(error.error);
		});	
	};
	
	this.openFilterImpostos = function(widget) {
		ZhFilterPreferences.openFilterWidget(widget, []).then(() => {
			console.log(widget);
        });
	};
	
	this.preparaFiltro = function(args) {
		args.e.preventDefault();
    	let currentRow = args.e.preference.FILTER_ROW;	
    	let widget = args.owner;
    	let filter = [];
    	widget.dataSourceFilter.forEach((f) => {
    		if(f.name != "NMPRODUTO") {
    			filter.push(f);
    		}
    	});
    	widget.dataSourceFilter = filter;
    	widget.reload();
	};
};

Configuration(function(ContextRegister) {
  ContextRegister.register("Ger80909_Grupo_Produto_Consultoria_Controller", Ger80909_Grupo_Produto_Consultoria_Controller);
});


// FILE: ./assets/js/controllers/Ger81200_AliquotaValidaCstImposto_Controller.js
function Ger81200_AliquotaValidaCstImposto_Controller(General, ScreenService, templateManager) {

    this.recarregarCampo = function(nomeWidget, nomeCampo){
        var widget = templateManager.container.getWidget(nomeWidget);
		widget.getField(nomeCampo).reload();
    };

    this.controlaCstImposto = function(widget){
    	var campoCst = widget.getField('CDSITUTRIBNFE');
    	campoCst.setValue('');
    	if(widget.currentRow.CDIMPOSTO != null && widget.currentRow.CDIMPOSTO != ''){
    		campoCst.readOnly = false;
    	}else{
    		campoCst.readOnly = true;
    	}
    	campoCst.reload();
    	
    };

    this.setarFiltroCst = function(widget){
        var campoCst = widget.getField('CDSITUTRIBNFE');
        campoCst.dataSourceFilter[0].value = widget.currentRow.CDIMPOSTONFE;
        campoCst.reload();
    };

}
Configuration(function (ContextRegister) {
    ContextRegister.register('Ger81200_AliquotaValidaCstImposto_Controller', Ger81200_AliquotaValidaCstImposto_Controller);
});

// FILE: ./assets/js/controllers/Ger81300_NaturezaBaseCalculoCfop_Controller.js
var Ger81300_NaturezaBaseCalculoCfop_Controller = function(templateManager, ZhFilterPreferences){

	this.openFilter = function(widget, defaultFilter) {
        ZhFilterPreferences.openFilterWidget(widget, defaultFilter).then(response => {
            var filter = templateManager.container.getActiveWidget();
            var clearFilter = filter.actions.filter(action => action.name === 'clear')[0];
            var confirmFilter = filter.actions.filter(action => action.name === 'never')[0];
            
            clearFilter.shortcutKey = 'ctrl+backspace';
            clearFilter.tooltip = 'Limpar Filtro (CTRL + Backspace)';
            
            confirmFilter.shortcutKey = 'ctrl + enter';
            filter.reload();
            
        });
    };

};
Configuration(function (ContextRegister){
	ContextRegister.register('Ger81300_NaturezaBaseCalculoCfop_Controller', Ger81300_NaturezaBaseCalculoCfop_Controller);
});

// FILE: ./assets/js/controllers/Ger82000_CodigoTributacaoMunicipio_Controller.js
function Ger82000_CodigoTributacaoMunicipio_Controller(General, ScreenService, templateManager, ZhFilterPreferences) {
    var inscmuni;

    this.loadAutocompleteData = function(field) {
        if (field.dataSource.data.length === 0)
            field.reload();
    };

    this.alteraReadOnlyFilhos = function(field, desbloqueiaComValor = true) {
        let widget = field.widget;
        field.filhos.forEach((filho) => {
            let filhoField = widget.getField(filho);
            filhoField.readOnly = ((field.value() == "") == desbloqueiaComValor);
            if (((field.value() == "") == desbloqueiaComValor)) widget.currentRow[filho] = "";
            if (filhoField.filhos) this.alteraReadOnlyFilhos(filhoField, true);
            widget.getField(filho).reload();
        });
    };

    this.blockFieldsFilhosAndSetEvents = function(widget, filter = true) {
        let fields = widget.fields;
        fields.forEach((field) => {
            if (field.filhos) {
               field.events.push({
                    "name": "FieldOnChange",
                    "code": "Ger82000_CodigoTributacaoMunicipio_Controller.alteraReadOnlyFilhos(args.owner.field)"
                });
                field.filhos.forEach((filho) => {
                    field = widget.getField(filho);
                    if (!field.filterProperties) widget.filterProperties = {};
                    if (filter) field.filterProperties.readOnly = false;
                    else widget.getField(filho).readOnly = true;
                });
            }
        });
    };

    this.preparaFiltro = function(args, widgetStrype) {
        
        var fieldEmpresa = widgetStrype.getField('Empresa');
        var fieldInscmuni = widgetStrype.getField('InscricaoMunicipal');
        
        args.e.preventDefault();
        var widget = args.owner;
        var row = args.e.preference.FILTER_ROW;
      
        widget.CDEMPRESA = row.CDEMPRESA;
        widget.CDINSCMUNI = row.CDINSCMUNI;
        widget.dataSourceFilter = [{
                "name": "CDEMPRESA",
                "operator": "=",
                "value": row.CDEMPRESA
            },
            {
                "name": "CDINSCMUNI",
                "operator": "=",
                "value": row.CDINSCMUNI
            },
            {
                "name": "CDNVPRODUTO",
                "operator": "=",
                "value": General.getLevelByMask('GER_CDPICTPROD', '1').length
            }
        ];
        inscmuni = row.CDINSCMUNI;
      
        fieldEmpresa.setValue(row.NMEMPRESA);
        
        if (row.NMREPRINSMUN !== null) {
          
            fieldInscmuni.setValue(row.CDINSCMUNI + ' | ' + row.NMREPRINSMUN);
        }
        else {
            fieldInscmuni.setValue(row.CDINSCMUNI);
        }
        widget.reload().then((success) => {
            ScreenService.closePopup();
        });
    };
    
    this.openFilter = function(widget, defaultFilter) {
        ZhFilterPreferences.openFilterWidget(widget, defaultFilter).then(response => {
            var filter = templateManager.container.getActiveWidget();
            var clearFilter = filter.actions.filter(action => action.name === 'clear')[0];
            var confirmFilter = filter.actions.filter(action => action.name === 'never')[0];
            
            clearFilter.shortcutKey = 'ctrl+backspace';
            clearFilter.tooltip = 'Limpar Filtro (CTRL + Backspace)';
            
            confirmFilter.shortcutKey = 'ctrl + enter';
            filter.reload();
            
        });
    };

    this.setDefaultFilter = function(widget) {
        General.callRestEngine("df#/crud/filial", "FilterData", [{ "name": "CDFILIAL", "operator": "=", "value": "FILIAL_LOGADA" }]).then((success) => {
                if (success.dataset.filial[0]) {
                    General.callRestEngine("df#/crud/empresa", "FilterData", [{ "name": "CDEMPRESA", "operator": "=", "value": success.dataset.filial[0].CDEMPRESA }]).then((resul) => {
                            var row = Util.clone(widget.currentRow);
                            row.CDEMPRESA = resul.dataset.empresa[0].CDEMPRESA;
                            row.NMEMPRESA = resul.dataset.empresa[0].NMEMPRESA;
                            row.CDINSCMUNI = resul.dataset.empresa[0].CDINSCMUNI;
                            widget.currentRow.CDEMPRESA = row.CDEMPRESA;
                            this.empresaOnChange(widget);
                            ZhFilterPreferences.setOriginalFilter(widget.id, row);
                            this.openFilter(widget, row);
                            this.atualizaFilter();
                       
                    });
                }
            });
    };
    
    this.atualizaFilter = function() {
        let w = templateManager.container.getWidget("wdg_tributacao");

        w.currentRow.CDEMPRESA = null;
        w.currentRow.CDINSCMUNI = null;
        w.currentRow.NMEMPRESA = null;
        w.currentRow.NMREPRINSMUN = null;
    };

    this.save = function(widget) {
        widget.currentRow.CDEMPRESA = widget.CDEMPRESA;
        widget.currentRow.CDINSCMUNI = widget.CDINSCMUNI;

        if (!widget.currentRow.CDINSCMUNI) {
            ScreenService.showMessage("Não é possível adicionar. Favor selecionar uma inscrição municipal antes de adicionar");
        }
        else if (!widget.currentRow.CDEMPRESA) {
            ScreenService.showMessage("Não é possível adicionar. Favor selecionar uma empresa antes de adicionar");
        }
        else {
            General.saveCrud(widget);
        }
    };

    this.produtoFinalIsValid = function(widget) {
        if (widget.currentRow.CDARVPRODFIN && widget.currentRow.CDARVPRODINI) {
            if (widget.currentRow.CDARVPRODFIN < widget.currentRow.CDARVPRODINI) {
                ScreenService.showMessage(ScreenService.i18n("Produto inicial deve ser menor ou igual ao produto final."));
                widget.currentRow.CDARVPRODFIN = null;
                widget.currentRow.NMPRODUTOFIN = null;
            }
        }
    };

    this.produtoInicialIsValid = function(widget) {
        if (widget.currentRow.CDARVPRODFIN && widget.currentRow.CDARVPRODINI) {
            if (widget.currentRow.CDARVPRODFIN < widget.currentRow.CDARVPRODINI) {
                ScreenService.showMessage(ScreenService.i18n("Produto inicial deve ser menor ou igual ao produto final."));
                widget.currentRow.CDARVPRODINI = null;
                widget.currentRow.NMPRODUTOINI = null;
            }
        }
    };

    this.openPopup = function() {
        var widgetCad = templateManager.container.getWidget('wdg_cadAutomatico');
        widgetCad.currentRow = {};
        widgetCad.getField('NMPRODUTOINI').dataSourceFilter[0].value = General.getLevelByMask("CDPICTPROD", 1).length;
        widgetCad.getField('NMPRODUTOFIN').dataSourceFilter[0].value = General.getLevelByMask("CDPICTPROD", 1).length;
        widgetCad.getField('NMPRODUTOINI').reload();
        widgetCad.getField('NMPRODUTOFIN').reload();
        widgetCad.isVisible = true;
        ScreenService.openPopup(widgetCad);
        widgetCad.activate();
    };

    this.closePopup = function(widget) {
        widget.isVisible = false;
        widget.dataSource.data = [];
        ScreenService.closePopup(widget);
        widget.getParent().activate();
        widget.getParent().unSelectAll();
        widget.getParent().reload();
        ScreenService.closePopup();
    };

    this.realizaCadastroAutomatico = function(widget) {
        if (widget.isValid()) {
            if (!widget.getParent().dataSourceFilter[1].value) {
                ScreenService.showMessage("Inscrição Municipal não selecionada.");
            }
            else if (!widget.getParent().dataSourceFilter[0].value) {
                ScreenService.showMessage("Empresa não selecionada.");
            }
            else {
                var row = widget.currentRow;
                row.CDPICTPRODMASK = General.getLevelByMask("CDPICTPROD", 1).length;
                row.CDINSCMUNI = widget.getParent().dataSourceFilter[1].value;
                row.CDEMPRESA = widget.getParent().dataSourceFilter[0].value;

                General.callRestEngine("df#/cadastroAutomaticoCodTribMuni", "Row", row).then((success) => {
                        this.closePopup(widget);
                    },
                    (error) => {
                        ScreenService.showMessage(error.data.error);
                    });
            }
        }
    };

    this.setInscMuniFilter = function(widget) {
        let cdEmpresa = widget.currentRow.CDEMPRESA;
        let inscMuniField = widget.getField("CDINSCMUNI");

        inscMuniField.dataSourceFilter = [{
            name: "CDEMPRESA",
            operator: "=",
            value: cdEmpresa
        }];
    };

    this.empresaOnChange = function(widget) {
        let currentRow = widget.currentRow;
        let inscMuniField = widget.getField("CDINSCMUNI");

        currentRow.CDINSCMUNI = null;
        currentRow.NMREPRINSMUN = null;
        if (currentRow.CDEMPRESA) {
            this.setInscMuniFilter(widget);
            inscMuniField.readOnly = false;
        }
        else
            inscMuniField.readOnly = true;

        inscMuniField.reload();
    };


}
Configuration(function(ContextRegister) {
    ContextRegister.register('Ger82000_CodigoTributacaoMunicipio_Controller', Ger82000_CodigoTributacaoMunicipio_Controller);
});


// FILE: ./assets/js/controllers/Ger97500_TipoOpercaoCfop_Controller.js
var Ger97500_TipoOpercaoCfop_Controller = function(General,ScreenService, templateManager,Ger97500_TipoOpercaoCfop_Service) {

    this.adicionar = function(args){
        args.owner.widget.newRow();
        this.controlaStatusWidgets(templateManager.container.getWidget('gridCfop'),'gridCfop',false);
    };

    this.editar = function(args){
        General.edit(args.owner.widget);
        this.controlaStatusWidgets(args.owner.widget,'gridCfop',false);
    };

    this.cancelar = function(args){
        General.backOrCancel(args);
        this.controlaStatusWidgets(args.owner.widget,'gridCfop',true);
    };

    this.recarregarField = function(nomeWidget,nomeField){
        var field =  templateManager.container.getWidget(nomeWidget).getField(nomeField);
        field.reload();
    };

    this.verificaTipoOperacao = function(args){
        var campoES = templateManager.container.getWidget('container_tabbed').getField('IDENTRSAIDOP');
        var campoTipoOperacao = templateManager.container.getWidget('container_tabbed').getField('NMTIPOOPER');
        var campoCdTipoOperacao = templateManager.container.getWidget('container_tabbed').getField('CDTIPOOPER');
        campoTipoOperacao.value("");
        campoCdTipoOperacao.value("");
        if(campoES.value()){
            campoTipoOperacao.readOnly=false;
        }else{
            campoTipoOperacao.readOnly=true;
        }
    };

    this.recarregarWidget = function(nomeWidget){
        var campoES = templateManager.container.getWidget('container_tabbed').getField('IDENTRSAIDOP');
        var campoTipoOperacao = templateManager.container.getWidget('container_tabbed').getField('NMTIPOOPER');
        var widgetCfop =  templateManager.container.getWidget('gridCfop');
        //var widgetExcecao =  templateManager.container.getWidget('gridExcecao');

        if(campoES.value() && campoTipoOperacao.value()){
            widgetCfop.getAction('ADD').isVisible=true;
        }else{
            widgetCfop.getAction('ADD').isVisible=false;

        }
        widgetCfop.reload();
        //widgetExcecao.reload();
    };

    this.setFiltroCfop = function(){
        var widgetCfop =  templateManager.container.getWidget('gridCfop');
        var tabbed =  templateManager.container.getWidget('container_tabbed');
        var campoCfop = widgetCfop.getField('CDFISCOPER');
        if(tabbed.currentRow.IDENTRSAIDOP=='E'){
            campoCfop.dataSourceFilter[0].value=['1','2','3'];
        }else{
            campoCfop.dataSourceFilter[0].value=['6','7','8'];
        }
        campoCfop.reload();
    };

    this.deletarsalvarTipoOperacaoCfop = function(args){
        var widget = args.owner.widget;
        ScreenService.confirmMessage('Excluir Registro?', 'question', function (success) {
            Ger97500_TipoOpercaoCfop_Service.deletar(widget.currentRow).then(function () {
                this.bloquearFiltro('LIVRE');
                ScreenService.showMessage('Deletado com sucesso.');
                widget.reload();
                ScreenService.closeSwipe().then(function () {
                    widget.activate();
                });
            }.bind(this), function (error) {
                if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                    ScreenService.showMessage('Exclusão não permitida! O registro possui registros filhos.');
                } else {
                    ScreenService.showMessage(error.data.error);
                }
            });
        }.bind(this));
    };

    this.salvarTipoOperacaoCfop = function(args){
        var widget = args.owner.widget;
        var tabbed =  templateManager.container.getWidget('container_tabbed');
        if(widget.currentRow.__is_new){
            widget.currentRow.CDTIPOOPER = tabbed.currentRow.CDTIPOOPER;
            widget.currentRow.IDENTRSAIDOP = tabbed.currentRow.IDENTRSAIDOP;
        }
        General.saveCrud(widget).then(function(Success) {
            this.controlaStatusWidgets(widget,widget.name,true);
        }.bind(this), function(callback){
            if(callback.data){
                ScreenService.showMessage(callback.data.error);
            }
        });
    };

    this.salvarExecaoProduto = function(args){
        var widget = args.owner.widget;
        var widgetCfop =  templateManager.container.getWidget('gridCfop');
        widget.currentRow.IDENTRSAIDOP = widgetCfop.currentRow.IDENTRSAIDOP;
        widget.currentRow.CDTIPOOPER = widgetCfop.currentRow.CDTIPOOPER;
        widget.currentRow.CDFISCOPER = widgetCfop.currentRow.CDFISCOPER;

        widget.currentRow.CDARVPROD =widget.currentRow.CDARVPROD.replace(/[^0-9]/g,'');
        General.saveCrud(widget).then(function(Success) {

        }.bind(this), function(callback){
            if(callback.data){
                ScreenService.showMessage(callback.data.error);
            }
        });
    };

    this.controlaStatusWidgets = function(mainWidget, nomeWidgetAtual, status){
        mainWidget.widgets.forEach(function(widg) {
            if(nomeWidgetAtual !== widg.name)
                widg.enabled = status;
        });

        if(mainWidget.name !== nomeWidgetAtual)
            mainWidget.enabled = status;
    };

    this.recarrearFiltroWidgetExecao = function(args){
        var widget = args.owner;
        if(! widget.currentRow.__is_new){
            widget.widgets[0].reload();
            this.bloquearFiltro('BLOQ');
        }
    };

    this.bloquearFiltro= function(situacao){
        var tabbed =  templateManager.container.getWidget('container_tabbed');
        var widgetCfop =  templateManager.container.getWidget('gridCfop');

        if(situacao=='BLOQ'){
            tabbed.getField('IDENTRSAIDOP').readOnly=true;
            tabbed.getField('NMTIPOOPER').readOnly=true;
        }else if('LIVRE'){
            if(widgetCfop.active){
                tabbed.getField('IDENTRSAIDOP').readOnly=false;
                tabbed.getField('NMTIPOOPER').readOnly=false;
            }
        }
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Ger97500_TipoOpercaoCfop_Controller", Ger97500_TipoOpercaoCfop_Controller);
});


// FILE: ./assets/js/controllers/Ger98500_TipoGestaoFiscal_Controller.js
function Ger98500_TipoGestaoFiscal_Controller(General, ScreenService, templateManager,Ger98500_TipoGestaoFiscal_Service) {

	this.controlaStatusWidgets = function(mainWidget, nomeWidgetAtual, status){
        mainWidget.widgets.forEach(function(widg) {
            if(nomeWidgetAtual !== widg.name)
                widg.enabled = status;
        });

        if(mainWidget.name !== nomeWidgetAtual)
            mainWidget.enabled = status;
    };

	this.adicionar = function(args){
        args.owner.widget.newRow();
        this.controlaStatusWidgets(templateManager.container.getWidget('gridTipoGestaoFiscal'),'gridTipoGestaoFiscal',false);
    };

    this.editar = function(args){
        General.edit(args.owner.widget);
        this.controlaStatusWidgets(args.owner.widget,'gridTipoGestaoFiscal',false);
    };

    this.cancelar = function(args){
        General.backOrCancel(args);
        this.controlaStatusWidgets(args.owner.widget,'gridTipoGestaoFiscal',true);
    };

    this.recarregarField = function(nomeWidget,nomeField){
        var field =  templateManager.container.getWidget(nomeWidget).getField(nomeField);
        field.reload();
    };

    this.salvarUnidadeTipoGestaoFiscal = function(args){
        var widgetTipoGestaoFiscal = templateManager.container.getWidget('gridTipoGestaoFiscal');
        var widget = args.owner.widget;
        widget.currentRow.CDTPFATURFIS = widgetTipoGestaoFiscal.currentRow.CDTPFATURFIS;
        var dataSource = widget.dataSource.name;
        var filtro = [
            {
                "name": "CDTPFATURFIS",
                "operator": "<>",
                "value": widget.currentRow.CDTPFATURFIS
            },
            {   
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.currentRow.CDFILIAL
            }
        ];
        
        General.callRestEngine(dataSource, 'FilterData', filtro).then(result => {
           if(result.dataset.filtpfaturfis_filial_tpfaturfis.length > 0)
                return ScreenService.showMessage('Filial já relacionada a um tipo de gestão fiscal.');
            else{
                General.saveCrud(widget).then(function(Success) {
                }, function(callback){
                    if(callback.indexOf('ORA-00001') !== -1){
                        ScreenService.showMessage("Impossível adicionar. Registro já existe.");
                    }
                    else {
        				ScreenService.showMessage(callback.data.error);
        			}
                
                });
            }
        });
    
    };

    this.salvarTipoGestaoFiscal = function(args){
        var widget = args.owner.widget;
        
        General.saveCrud(widget).then(function(Success) {
            this.controlaStatusWidgets(args.owner.widget,'gridTipoGestaoFiscal',true);
        }.bind(this), function(callback){
            if(callback.data){
                ScreenService.showMessage(callback.data.error);
            }
        });
    };

    this.deletarTipoGestaoFiscal = function(args){
        var widget = args.owner.widget;

        Ger98500_TipoGestaoFiscal_Service.verificaUnidadeRelacionada(widget.currentRow).then(function(resul) {
            var dataset = resul.dataset.filtpfaturfis_filial_tpfaturfis;
            if(dataset.length>0){
                ScreenService.confirmMessage(('Tipo de gestão fiscal possui unidade relacionada. Deseja exlcuir ?'), 'question', function (success) {
                    Ger98500_TipoGestaoFiscal_Service.deletar(widget.currentRow).then(function (success) {
                        ScreenService.closeSwipe().then(function () {
                            widget.activate();
                            widget.reload();
                        });
                    }, function (error) {
                        if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                            ScreenService.showMessage('Exclusão não  permitida! Tipo de Gestão Fiscal possui Unidade relacionada.');
                        } else {
                            ScreenService.showMessage(error.data.error);
                        }
                    });
                });
            }else{
              General.deleteCrud(widget);
            }
        }.bind(this), function(callback){
            if(callback.data){
                ScreenService.showMessage(callback.data.error);
            }
        });

    };

    this.recarrearFiltroWidgetUnidade = function(args){
        var widget = args.owner;
        if(! widget.currentRow.__is_new){
            widget.widgets[0].reload();
        }
    };

}
Configuration(function (ContextRegister) {
	ContextRegister.register('Ger98500_TipoGestaoFiscal_Controller', Ger98500_TipoGestaoFiscal_Controller);
});

// FILE: ./assets/js/controllers/Ger98600_CodigoDeSituacaoDaOperacaoDoSimplesNacional_Controller.js
function Ger98600_CodigoDeSituacaoDaOperacaoDoSimplesNacional_Controller(Ger98600_CodigoDeSituacaoDaOperacaoDoSimplesNacional_Service, ScreenService,templateManager, ZhFilterPreferences, General){

    var filtered = false;

    this.back = function(widget){
        editPrimaryFields(widget);
        ScreenService.closeSwipe();
    };
    
    this.preLoadAutocomplete = function(widget){
      if(widget.dataSource.data.length === 0)
        widget.reload();
    };

    this.backFastDelete = function(widget){
        ScreenService.closePopup();
        widget.isVisible = false;
        widget.getParent().activate();
        widget.getParent().reload();
    };

    this.cancel = function(args){
        var widget = args.owner.widget;
        blockPrimaryFields(widget);
        General.backOrCancel(args);
    };

    this.delete = function(widget){
        General.deleteCrud(widget);
    };

    this.edit = function(widget){
       // blockPrimaryFields(widget);
        General.edit(widget);
    };

    this.fastDelete = function(widget){
        General.deleteCrudCheckedRows(widget).then(function(){
            ScreenService.closePopup();
            widget.isVisible = false;
            widget.getParent().activate();
            widget.getParent().reload();
        });
    };

    this.filter = function(widget){
        editPrimaryFields(widget);
        ZhFilterPreferences.openFilterWidget(widget);
    };

    this.filterPopup = function(widget){
        var widgetPrincipal = templateManager.container.getWidget('codigos_de_situacao');
        if(widgetPrincipal.appliedFilter!== undefined){
            var filterRow = widgetPrincipal.appliedFilter.FILTER_ROW;
            if(!popupIsEmpty(filterRow)){
                var fieldNames = ["CDEMPRESA","CDINSCESTA","IDENTSAICSOSN","CDTIPOOPER"];
                var values = [filterRow.CDEMPRESA, filterRow.CDINSCESTA, filterRow.IDENTSAICSOSN, filterRow.CDTIPOOPER];
                updateFilter(widget,fieldNames ,"=", values);
                widget.reload();
                templateManager.updateTemplate();
            }
            else{
                ScreenService.showMessage("Nenhum filtro foi realizado");
            }
        }
        else{
            ScreenService.showMessage("Nenhum filtro foi realizado");
        }
    };

    this.new = function(widget){
        widget.newRow();
        preSetFields(widget);
        editPrimaryFields(widget);
    };

    this.openFastDelete = function(args){
        var widget = args.widgets[0];
        widget.dataSource = args.dataSource;
    //    this.filterPopup(widget);
        ScreenService.openPopup(widget);
        widget.isVisible = true;
        templateManager.updateTemplate();
    };

    this.putFields  =  function(){
        filtered = true;
    };

    this.save = function(widget){
        if(widget.isValid()){
            Ger98600_CodigoDeSituacaoDaOperacaoDoSimplesNacional_Service.save(widget.currentRow).then(function(response){
                if(response.dataset.hasOwnProperty('validations')){
                    if(!response.dataset.validations.validate){
                        ScreenService.showMessage("Impossível adicionar. Registro já existe.","E");
                        editPrimaryFields(widget);
                    }
                }
                else{
                    blockPrimaryFields(widget);
                    ScreenService.closeSwipe();
                    widget.reload();
                }
            });
        }
    };

    this.updateInscricaoEstadual = function(widget){
        var codigosWidget = templateManager.container.getWidget('codigos_de_situacao');
        var fieldName = 'CDINSCESTA';
        var filterFieldName = 'CDEMPRESA';
        var valueFieldName = 'CDEMPRESA';
        this.updateField(widget, fieldName, filterFieldName, valueFieldName, codigosWidget);
        widget.currentRow[fieldName] = "";
        templateManager.updateTemplate();
    };

    this.updateTipoOperacao = function(widget){
        var codigosWidget = templateManager.container.getWidget('codigos_de_situacao');
        var fieldName = 'NMTIPOOPER';
        var filterFieldName = 'IDENTRSAIDOP';
        var valueFieldName = 'IDENTSAICSOSN';
        this.updateField(widget, fieldName, filterFieldName, valueFieldName, codigosWidget);
        codigosWidget.getField(fieldName).readOnly = false;
        widget.currentRow[fieldName] = "";
        templateManager.updateTemplate();
    };

    this.updateField = function(firstWidget,fieldName, filterFieldName, valueFieldName){
        var field = firstWidget.getField(fieldName);
        var filterValue = firstWidget.currentRow[valueFieldName];
        firstWidget.currentRow[filterFieldName] = filterValue;
        updateFilter(field,[filterFieldName],"=",[filterValue]);
        if(filterValue && filterValue.length > 0){
            field.readOnly = false;
        }
        else{
            field.readOnly = true;
        }
    };

    function blockPrimaryFields(widget){
        widget.getField('NMRAZSOCEMPR').readOnly = true;
        widget.getField('CDINSCESTA').readOnly = true;
        widget.getField('IDENTSAICSOSN').readOnly = true;
        widget.getField('NMTIPOOPER').readOnly = true;
    }

    function editPrimaryFields(widget){
        widget.getField('NMRAZSOCEMPR').readOnly = false;
        widget.getField('IDENTSAICSOSN').readOnly = false;
        if(widget.currentRow.NMRAZSOCEMPR && widget.currentRow.NMRAZSOCEMPR.length > 0){
            widget.getField('CDINSCESTA').readOnly = false;
        }
        if(widget.currentRow.IDENTSAICSOSN){
            widget.getField('NMTIPOOPER').readOnly = false;
        }
    }
    function popupIsEmpty(row){
        if(!row.CDEMPRESA || row.CDEMPRESA === ""){
            return true;

        }else{
            return false;
        }
    }

    function preSetFields(widget){
        if(filtered){
            widget.currentRow.CDEMPRESA = widget.appliedFilter.FILTER_ROW.CDEMPRESA;
            widget.currentRow.NMRAZSOCEMPR = widget.appliedFilter.FILTER_ROW.NMRAZSOCEMPR;
            widget.getField('CDINSCESTA').dataSourceFilter[0].value = widget.appliedFilter.FILTER_ROW.CDEMPRESA;
            widget.getField('CDINSCESTA').reload().then(function(){
                widget.getField('CDINSCESTA').value(widget.appliedFilter.FILTER_ROW.CDINSCESTA);
            });
            widget.currentRow.CDINSCESTA = widget.appliedFilter.FILTER_ROW.CDINSCESTA;
            widget.currentRow.IDENTSAICSOSN = widget.appliedFilter.FILTER_ROW.IDENTSAICSOSN;
            widget.currentRow.IDENTRSAIDOP = widget.appliedFilter.FILTER_ROW.IDENTRSAIDOP;
            widget.currentRow.NMTIPOOPER = widget.appliedFilter.FILTER_ROW.NMTIPOOPER;
            widget.currentRow.CDTIPOOPER = widget.appliedFilter.FILTER_ROW.CDTIPOOPER;
        }
    }

    function updateFilter(object, fieldNames, operator, values){
        var dataSourceFilter = [];
        for (var i = 0; i< fieldNames.length; i++){
            if(values[i] !== ""){
                dataSourceFilter.push(
                    {
                        "name": fieldNames[i],
                        "operator": operator,
                        "value": values[i]
                    }
                );
            }
        }
        object.dataSourceFilter = dataSourceFilter;
        object.reload();
    }
}
Configuration(function(ContextRegister) {
    ContextRegister.register("Ger98600_CodigoDeSituacaoDaOperacaoDoSimplesNacional_Controller", Ger98600_CodigoDeSituacaoDaOperacaoDoSimplesNacional_Controller);
});

// FILE: ./assets/js/controllers/Ger98700_ControleFiscal_Controller.js
var Ger98700_ControleFiscal_Controller = function(templateManager, ScreenService) {

    // Campos que só devem estar visíveis quando o tipo de operação é de entrada.
    var entradaFields = [
        "IDOBINFCSTICMENT",
        "IDOBINFCSTCFOPEN"
    ];
    // Campos que só devem estar visíveis quando o tipo de operação é de saída.
    var saidaFields = [
        "IDGERICMSTDESPA",
        "NMENCARGOICMST",
        "IDPEINFICMSTDEME",
        "IDGERIPIDESPA",
        "NMIMPOSTOIPI"
    ];
    // Variável de controle de swipe aberto.
    // Utilizado para fechar o Swipe, caso esteja aberto, ao trocar o filtro entrada/saída.
    var openSwipe = false;

    /*
     * Altera o estado readonly dos fields de saída, de acordo com sua marcação (Sim/Não).
     */
    this.changeWidgetsReadonlyDependency = function(widget) {
        if (widget.currentRow.IDENTRSAIDOP == 'S') {
            widget.fields.forEach((function(field) {
                if (field.dependentFields) {
                    this.toggleDependentFieldsReadOnly(field, true);
                }
            }).bind(this));
        }
    };

    /*
     * Função para alterar a visibilidade dos campos conforme o filtro de entrada ou saída.
     */
    this.checkChangeVisibleFields = function(widget) {
        var entradaIsSelected = widget.currentRow.IDENTRSAIDOP == 'E';
        var childWidget = widget.widgets[0];
        entradaFields.forEach(function(fieldName) {
            childWidget.getField(fieldName).isVisible = entradaIsSelected;
        });
        saidaFields.forEach(function(fieldName) {
            childWidget.getField(fieldName).isVisible = !entradaIsSelected;
        });
    };

    /*
     * Função que verifica se o swipe está aberto ao trocar o filtro e recarregar o grid.
     * Caso esteja aberto, fecha-o e altera a variável de controle.
     */
    this.checkOpenSwipe = function() {
        if (openSwipe) {
            ScreenService.goBack();
            this.toggleOpenSwipe();
        }
    };

    /*
     * Função que verifica se todos os filtros da tabbed estão preenchidos para recarregar o grid filho.
     */
    this.checkReloadDependentWidget = function(currentWidget) {
        var childWidget = templateManager.container.getWidget(currentWidget.dependentWidgetName);
        // Se todos os fields tiverem preenchidos na currentRow, retorna true.
        var toReload = currentWidget.widgetDependentFields.every(function(fieldValue) {
            return !!currentWidget.currentRow[fieldValue];
        });
        if (toReload) {
            childWidget.reload();
        }
        // Se houver algum filtro sem valor, limpa o grid.
        else {
            if (childWidget.viewTemplate) {
                childWidget.template = childWidget.viewTemplate;
            }
            childWidget.dataSource.data = [];
            childWidget.currentRow = {};
        }
    };

    /*
     * Função para verificar se um grid será atualizado, evitando requisições desnecessárias.
     * É utilizada no caso do grid Impostos Obrigatórios, para que sejam atualizados somente
     * ao trocar de tipo de operação.
     */
    this.checkReloadOnEnter = function(widget) {
        if (widget.needReloadAfterMoveRow) {
            widget.reload();
            widget.needReloadAfterMoveRow = false;
        }
    };

    /*
     * Função para habilitar/desabilitar campos de parametrização do tipo de operação de saída
     * que não esteja parametrizado para possibilitar essa mudança.
     */
    this.checkToggleEditableWidget = function(widget) {
        var editAction = widget.actions.filter(function(action) {
            return action.name == "editar";
        })[0];
        var isSaida = widget.currentRow.IDENTRSAIDOP == 'S';
        var notEditable = isSaida && widget.currentRow.IDOPERDEVONF != 'S';
        editAction.isVisible = !notEditable;
        if (isSaida) {
            saidaFields.forEach(function(fieldName) {
                widget.getField(fieldName).isVisible = !notEditable;
            });
        }
    };

    /*
     * Limpa o descriptionField de um field tipo select para que o mesmo não permaneça na currentRow.
     */
    this.clearDescriptionFieldValue = function(field) {
        if (!field.value()) {
            field.widget.currentRow[field.descriptionField] = null;
        }
    };

    /*
     * Função para preparar a widget Impostos Obrigatórios para adicionar um imposto.
     * Busca o IDENTRSAIDOP e o CDTIPOOPER do tipo de operação selecionado.
     */
    this.newImpostoObrigatorio = function(widget) {
        widget.newRow();
        widget.currentRow.IDENTRSAIDOP = widget.getParent().currentRow.IDENTRSAIDOP;
        widget.currentRow.CDTIPOOPER = widget.getParent().currentRow.CDTIPOOPER;
    };

    /*
     * Altera a variável de controle para recarregar a widget filha ao trocar de
     * tipo de operação.
     */
    this.setChildWidgetNeedReload = function(widget) {
        widget.widgets[0].needReloadAfterMoveRow = true;
    };

    /*
     * Altera o readOnly de fields que dependem do valor de outros.
     * Um exemplo é o campo Imposto IPI, que requer que o campo Gera IPI
     * como Despesa Acessória tenha o valor "Sim" (tenha valor e não seja "Não").
     * A função recebe o parâmetro 'onEdit', que é setado como true ao utilizar
     * a action "Editar". Isso impede que os campos dependentes percam seu valor.
     */
    this.toggleDependentFieldsReadOnly = function(parentField, onEdit) {
        var widget = parentField.widget;
        var dependentFields = parentField.dependentFields || [];
        var readOnly = !parentField.value() || parentField.value() == 'N';
        dependentFields.forEach(function(fieldName) {
            var dependentField = widget.getField(fieldName);
            dependentField.readOnly = readOnly;
            dependentField.validations = readOnly ? {} : {required: true};
            if (!onEdit) {
                dependentField.setValue(dependentField.defaultValue || null);
            }
        });
    };

    /*
     * Alterna a variável de controle de swipe aberto.
     */
    this.toggleOpenSwipe = function() {
        openSwipe = !openSwipe;
    };

    this.setFloatingControl = function(field) {
        field.floatingControl.defaultSearchField = '';
    };

};

Configuration(function(ContextRegister){
    ContextRegister.register('Ger98700_ControleFiscal_Controller', Ger98700_ControleFiscal_Controller);
});

// FILE: ./assets/js/controllers/Ger99200_encargoNotaFiscalEletronica_Controller.js
var Ger99200_encargoNotaFiscalEletronica_Controller = function (ScreenService, templateManager, General) {

    this.openSwipeRelationships = function(widget) {
            var relacWidget = templateManager.container.getWidget('relacionamentos');
            ScreenService.openSwipe(relacWidget);
            relacWidget.dataSourceFilter[0].value = widget.currentRow.CDENCARGONFE;
            relacWidget.list();
            relacWidget.reload();
        };
    
    this.actionControll = function(widget){
        var widgetRelacionamento = widget.container.getWidget('relacionamentos');
        
        if(widget.name === 'relacionamentos'){
            widgetRelacionamento.getAction('editar').isVisible = true;
            widgetRelacionamento.getAction('DELETE').isVisible = true;
        }else{
            widgetRelacionamento.getAction('editar').isVisible = false;
            widgetRelacionamento.getAction('DELETE').isVisible = false;
        }
    };
    
    this.deleteRelationships = function(widget){
      widget.currentRow.CDENCARGONFE = null;
      General.saveCrud(widget).then(() => {
          ScreenService.goBack();
          widget.list();
          widget.reload();
          widget.activate();
      });
    };
    
    this.saveRelationships = function(widget){
        widget.currentRow.__is_new = false;
        widget.currentRow.CDENCARGONFE = widget.getParent().currentRow.CDENCARGONFE;
        General.saveCrud(widget).then(() => {
            ScreenService.goBack();
            widget.reload();
        });
        
    };

    this.filtraCampo = function(widget){
        if(widget.parent.parent.currentRow.CDENCARGONFE == "03"){
            widget.getField('NMENCARGO').dataSourceFilter[0].value = "D";
        } else {
            widget.getField('NMENCARGO').dataSourceFilter[0].value = "A";
        }
        widget.getField('NMENCARGO').reload();
        widget.reload();
    };

    this.preLoadAutocomplete = function(field){
      if(field.dataSource.data.length === 0)
        field.reload();
    };
    
    
};
Configuration(function (ContextRegister) {
    ContextRegister.register('Ger99200_encargoNotaFiscalEletronica_Controller', Ger99200_encargoNotaFiscalEletronica_Controller);
});

// FILE: ./assets/js/controllers/Grs18000_ParametroExportacaoNfSap_Controller.js
var Grs18000_ParametroExportacaoNfSap_Controller = function(ScreenService, General) {

    this.unidadeOnChange = function(widget) {
        var currentRow = widget.currentRow;
        var unidadeIsSet = !!currentRow.CDFILIAL;
        widget.getField("IDAVISOCREDSAP").readOnly = !unidadeIsSet;
        if (unidadeIsSet) {
            General.callRestEngine("df#/paramfilial_parametro_exportacao/getParameter", "Row", {
                NRORG: currentRow.NRORG,
                CDFILIAL: currentRow.CDFILIAL
            }).then(function(response) {
                currentRow.IDAVISOCREDSAP = response.dataset.parametro[0];
            }, function(error) {
                var errorMessage = 'Timeout';
                if (error.data) {
                    errorMessage = error.data.error;
                }
                ScreenService.showMessage(ScreenService.i18n(errorMessage));
            });
        }
        else {
            currentRow.IDAVISOCREDSAP = null;
        }
    };

    this.changeParameter = function(widget) {
        if (widget.isValid()) {
            var currentRow = widget.currentRow;
            General.callRestEngine("df#/paramfilial_parametro_exportacao/changeParameter", "Row", {
                NRORG: currentRow.NRORG,
                CDFILIAL: currentRow.CDFILIAL,
                IDAVISOCREDSAP: currentRow.IDAVISOCREDSAP || 'N'
            }).then(function() {
                ScreenService.showMessage("Salvo com sucesso!");
            }, function(error) {
                var errorMessage = 'Timeout';
                if (error.data) {
                    errorMessage = error.data.error;
                }
                ScreenService.showMessage(ScreenService.i18n(errorMessage));
            });
        }
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("Grs18000_ParametroExportacaoNfSap_Controller", Grs18000_ParametroExportacaoNfSap_Controller);
});


// FILE: ./assets/js/controllers/Lib98004_EmailsAdicionais_Controller.js
var Lib98004_EmailsAdicionais_Controller = function(General, Lib98004_EmailsAdicionais_Service, templateManager, ScreenService, MessageService, ZhFilterPreferences, ZHPromise, localStorageEngine) {
    
    var getWidgetEmailsAdicionaisDetails = function(){
        return templateManager.container.getWidget('EmailsAdicionaisDetails');
    }; 
    
    var getWidgetEmailsAdicionaisGrid = function(){
        return templateManager.container.getWidget('EmailsAdicionaisGrid');
    };
    
    this.idorigdestOnChange = function(args) {
        alteraVisibilidadeCampoInscricaoFiltro(args);
        limpaCamposOutDataSelectsTpOperacFiltro(args);
        habilitaEmailsAdicionaisGrid();
    };

    
    var habilitaEmailsAdicionaisGrid = function(){
        var detailsWidget = getWidgetEmailsAdicionaisDetails();
        var gridWidget = getWidgetEmailsAdicionaisGrid();
        
        var adicionarAction = gridWidget.getAction('novoEmail');
        
        if(ScreenService.isSwipeOpen()){
            ScreenService.closeSwipe();
        }
        
        if(detailsWidget.isValid()){
            gridWidget.isVisible = true;
            adicionarAction.isVisible = true;
            gridWidget.reload();
        } else{
            gridWidget.isVisible = false;
            adicionarAction.isVisible = false;
        }
    };


    var alteraVisibilidadeCampoInscricaoFiltro = function(args) {
        var filtroWidget = args.owner.field.widget;
        var cliente = filtroWidget.getField("NMRAZSOCCLIE");
        var fornecedor = filtroWidget.getField("NMRAZSOCFORN");
        var filial = filtroWidget.getField("DSFILIRECE");
        var transportadora = filtroWidget.getField("NMRAZSOCTRAN");
        var tipoDestino = args.row.IDORIGEM;

        switch (tipoDestino) {
            case "C":
                validacaoCamposOutTpOperacFilter(cliente, fornecedor, filial, transportadora);
                break;
            case "F":
                validacaoCamposOutTpOperacFilter(fornecedor, cliente, filial, transportadora);
                break;
            case "L":
                validacaoCamposOutTpOperacFilter(filial, fornecedor, cliente, transportadora);
                break;
            case "T":
                validacaoCamposOutTpOperacFilter(transportadora, filial, fornecedor, cliente);
                break;
            default:
                filial.readOnly = true;
                filial.setValue(null);
                cliente.readOnly = true;
                cliente.setValue(null);
                fornecedor.readOnly = true;
                fornecedor.setValue(null);
                transportadora.readOnly = true;
                transportadora.setValue(null);
        }
    };

    var limpaCamposOutDataSelectsTpOperacFiltro = function(args)
    {
        var detailsWidget= args.owner.field.widget;
        var cabecalhoRow = detailsWidget.currentRow;
        var cliente = detailsWidget.getField("NMRAZSOCCLIE");
        var fornecedor = detailsWidget.getField("NMRAZSOCFORN");
        var filial = detailsWidget.getField("DSFILIRECE");
        var transportadora = detailsWidget.getField("NMRAZSOCTRAN");

        filial.value(null);

        cliente.value(null);
        cabecalhoRow.CDCLIENTE = null;

        fornecedor.value(null);
        cabecalhoRow.CDFORNECED = null;
        cabecalhoRow.IDSITUCADA = null;
        cabecalhoRow.IDBLQNFRELAFOAF = null;

        cabecalhoRow.NRINSJURNF = null;
        cabecalhoRow.NMINSJURNF = null;
        cabecalhoRow.IDTPINSJURNF = null;
        
        transportadora.value(null);
    };
    
    var validacaoCamposOutTpOperacFilter = function(princialField) {
        princialField.showOnView = true;
        princialField.showOnForm = true;
        princialField.isVisible = true;
        princialField.readOnly = false;
        princialField.focused = true;
        princialField.validations = {"required": true};
        
        for (var i = 1; i < arguments.length; i++) {
            arguments[i].showOnView = false;
            arguments[i].showOnForm = false;
            arguments[i].isVisible = false;
            arguments[i].readOnly = true;
            arguments[i].validations = {};
        }
    };
    
    this.orgFieldOnChange = function(){
        
        var detailsWidget = templateManager.container.getWidget('EmailsAdicionaisDetails');
        var row = detailsWidget.currentRow;
        var widgetEmail = templateManager.container.getWidget('EmailsAdicionaisGrid');
        widgetEmail.dataSourceFilter = [];
        
        var tipoDestino = row.IDORIGEM;
        
        var filterCodigo = [];
        var filterOrigem = {
            "name": "IDORIGEM",
            "operator": "=",
            "value": tipoDestino,
            "isCustomFilter": true
        };
        
        switch (tipoDestino) {
            case "C":
                filterCodigo = {
                    "name": "CDCLIENTE",
                    "operator": "=",
                    "value": row.CDCLIENTE,
                    "isCustomFilter": true
                };
                break;
            case "F":
                filterCodigo = {
                    "name": "CDFORNECED",
                    
                    "operator": "=",
                    "value": row.CDFORNECED
                };
                break;
            case "L":
                filterCodigo = {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": row.CDFILIRECE
                };
                break;
            case "T":
                filterCodigo = {
                    "name": "NRINSJURTRAN",
                    "operator": "=",
                    "value": row.NRINSJURTRAN
                };
                widgetEmail.dataSourceFilter.push(filterCodigo);
                filterCodigo = {
                    "name": "IDTPIJURTRAN",
                    "operator": "=",
                    "value": row.IDTPIJURTRAN
                };
                break;
        }
        widgetEmail.dataSourceFilter.push(filterCodigo);
        widgetEmail.dataSourceFilter.push(filterOrigem);
        habilitaEmailsAdicionaisGrid();
    };
    
    
    this.adicionarEmail = function(widget){
        widget.newRow();
    };
    
    this.cancelarEmail = function(args){
        General.backOrCancel(args);
    };
    
    this.editarEmail = function(args){
        General.edit(args.owner.widget);
    };
    
    this.excluirEmail = function(args){
        var widgetEmail = templateManager.container.getWidget('EmailsAdicionaisGrid');
        var idEmail = widgetEmail.currentRow.IDEMAIL;
        
        ScreenService.confirmMessage('Deseja excluir o registro?', 'question', function (success) {
            Lib98004_EmailsAdicionais_Service.removeEmail(idEmail).then(function(response){
                ScreenService.closeSwipe();
                widgetEmail.list();
                widgetEmail.reload();
                ScreenService.successNotification(response.messages[0].message);
            });
        });
        
    };
    
    
    this.salvarEmail = function(args){
        var widgetEmail = templateManager.container.getWidget('EmailsAdicionaisGrid');
        var detailsWidget = templateManager.container.getWidget('EmailsAdicionaisDetails');
        var isNew = widgetEmail.currentRow.__is_new;
        var idEmail = null;
        var tipoDestino = detailsWidget.currentRow.IDORIGEM;
        var email = widgetEmail.currentRow.DSEMAIL;
        var cdCliente = null;
        var cdForneced = null;
        var cdFilial = null;
        var idtpijurtran = null;
        var nrinsjurtran = null;
        
        if(!isNew){
            idEmail = widgetEmail.currentRow.IDEMAIL;
        }
        
        switch (tipoDestino) {
        case "C":
            cdCliente = detailsWidget.currentRow.CDCLIENTE;
            break;
        case "F":
            cdForneced = detailsWidget.currentRow.CDFORNECED;
            break;
        case "L":
            cdFilial = detailsWidget.currentRow.CDFILIRECE;
            break;
        case "T":
            idtpijurtran = detailsWidget.currentRow.IDTPIJURTRAN;
            nrinsjurtran = detailsWidget.currentRow.NRINSJURTRAN;
            break;
        }
    
        
        
        if (widgetEmail.isValid()) {
            Lib98004_EmailsAdicionais_Service.saveEmail(tipoDestino, cdCliente, cdFilial, cdForneced, idtpijurtran, nrinsjurtran, email, idEmail).then(function(response){
                ScreenService.successNotification("Email adicionado com sucesso.");
                ScreenService.closeSwipe();
                habilitaEmailsAdicionaisGrid();
            });
        } else{
            ScreenService.showMessage("Preencha os campos obrigatórios corretamente!");
        }
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("Lib98004_EmailsAdicionais_Controller", Lib98004_EmailsAdicionais_Controller);
});

// FILE: ./assets/js/controllers/Produto_x_Cfop_Controller.js
var Produto_x_Cfop_Controller = function(ZhFilterPreferences, ScreenService, templateManager, General) {
    
    this.openFilterWidget = function(args, identrsaidop) {
        ZhFilterPreferences.openFilterWidget(args.owner, {'IDENTRSAIDOP': identrsaidop}).then(response => {
            var filterWidget = templateManager.container.getActiveWidget();
            let clear = filterWidget.actions.filter(action => action.name === "clear")[0];
            clear.shortcutKey = "ctrl+backspace";
            clear.tooltip = "Limpar (ctrl + Backspace)";
        });
    };
    
    this.getValorIdentrsaidop = function() {
        var activeWidget = templateManager.container.getActiveWidget();
        var valorPadrao = "E";
        if(activeWidget.label == 'Filtro') {
            return activeWidget.getField('IDENTRSAIDOP').value();
        } else if(activeWidget.label == 'Tipo de Operação' || activeWidget.label == 'CFOP') {
            return activeWidget.lastActiveWidget.currentRow.IDENTRSAIDOP;
        } else {
            if(activeWidget.appliedFilter) {
                return activeWidget.appliedFilter.FILTER_ROW.IDENTRSAIDOP;
            } else {
                return valorPadrao;
            }
        }
    };
    
    this.setFiltroPadrao = function() {
        var activeWidget = templateManager.container.getActiveWidget();
        var valorPadrao = "E";
        activeWidget.dataSourceFilter.push({name: "IDENTRSAIDOP", operator: "=", value: valorPadrao});
    };
    
    this.corrigeCfop = function() {
        var fieldCfop = templateManager.container.getActiveWidget().getField('CDFISCOPER');
        if(fieldCfop) {
            var novoValor = fieldCfop.value().substr(1);
            fieldCfop.setValue(novoValor);
        }
    };
    
    this.esOnChange = function() {
        var widget = templateManager.container.getActiveWidget();
        widget.getField('CDTIPOOPER').reload();
    };
    
    this.newRow = function(widget) {
        widget.newRow();
    };
    
    this.saveRow = function(widget) {
        General.saveCrud(widget).then(function(Success) {
            if(widget.currentRow.__is_new) {
                widget.currentRow.__is_new = false;
            }
        }.bind(this), function(callback){
            if(callback.indexOf('ORA-00001') !== -1){
                ScreenService.showMessage("Impossível adicionar. Registro já existe.");
            }
            else {
				ScreenService.showMessage(callback);
			}
        });
    };
    
    this.deleteRow = function(widget) {
        General.deleteCrud(widget);
    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Produto_x_Cfop_Controller", Produto_x_Cfop_Controller);
});

// FILE: ./assets/js/controllers/Ger80909_DadosFiscaisCstImposto.js
var Ger80909_DadosFiscaisCstImposto = function(General) {

	this.salvarDadosFiscais = function(args){
		args.owner.widget.dataSource.name =  "df#/crud/paramcst";
		
		//SALVA PARA ENTRADA
		args.owner.widget.currentRow.IDENTRSAIDOP = 'E';
		General.saveCrud(args.owner.widget);
		
		//SALVA PARA SAIDA
		args.owner.widget.currentRow.IDENTRSAIDOP = 'S';
		General.saveCrud(args.owner.widget);
		
		args.owner.widget.dataSource.name =  "df#/paramcst_impostonfe";
	};
	
	this.removeCustomFilter = function(args) {
	    args.e.preventDefault();
	    var widget = args.owner;
	    var filter = [];
	    filter = widget.dataSourceFilter.filter((item) =>  {
	      return !(item.isCustomFilter) && item.name != "CDIMPOSTO";
	    });
	    widget.dataSourceFilter = filter;
	    widget.reload();
	};
};

Configuration(function(ContextRegister) {
  ContextRegister.register("Ger80909_DadosFiscaisCstImposto", Ger80909_DadosFiscaisCstImposto);
});


// FILE: ./assets/js/controllers/Ger81100_AgrupamentoImpostos.js
var Ger81100_AgrupamentoImpostos = function(General, MessageService, ScreenService, Ger81100_AgrupamentoImpostos_Service) {

	this.salvarImpostoAgrupamento = function(args){
		if(args.owner.widget.isValid()) {
			args.owner.widget.dataSource.name =  "df#/crud/itagrupimposto";
			General.saveCrud(args.owner.widget);
			General.changeWidgetAcessibility(args.owner.widget.widgets, true);
			args.owner.widget.dataSource.name =  "df#/itagrupimposto_imposto";
		}
	};
	
	this.excluirImpostoAgrupamento = function(args){
		args.owner.widget.dataSource.name =  "df#/crud/itagrupimposto";
		General.deleteCrud(args.owner.widget).then(function(){
			args.owner.widget.dataSource.name =  "df#/itagrupimposto_imposto";
			args.owner.widget.reload();
			removerRowExcluida(args);
		});
	};
	
	this.salvar = function(args){
		if(args.owner.widget.isValid()) {
			General.saveRow(args,false);
			args.owner.widget.view();
			General.changeWidgetAcessibility(args.owner.widget.widgets, true);
		}
	};

	this.excluirAgrupamento = function(args){
		Ger81100_AgrupamentoImpostos_Service.getRegrasFiscais(args.owner.widget.currentRow.CDGRUPIMPOSTO).then( regras => {
			if(regras.length == 0){
				General.deleteCrud(args.owner.widget);
			} else{
				let txt = "AGR_IMP_REL_RF";
				MessageService.errorNotificationByCode(txt, {"PARAMETRO": regras});
			}
		});
	};
	
	var removerRowExcluida = function (args){
		var rowExcluida = args.owner.widget.dataSource.data.filter(function(row){
      return args.row === row;
    });
    var index = args.owner.widget.dataSource.data.indexOf(rowExcluida[0]);
    args.owner.widget.dataSource.data.splice(index, 1);
	};
};

Configuration(function(ContextRegister) {
  ContextRegister.register("Ger81100_AgrupamentoImpostos", Ger81100_AgrupamentoImpostos);
});


// FILE: ./assets/js/controllers/ger00300_encargo_controller.js
function ger00300_encargo_controller(restEngine, ScreenService, templateManager, ZHPromise, Endereco, General, MessageService) {
    
    var self = this;

    self.DESCONTAR = '03';
    
    self.preparaFiltroEncargo = function(widget){
        var encargo = widget.getField('NMENCARGO');
        
        if (widget.currentRow.CDENCARGONFE == self.DESCONTAR){
            encargo.dataSourceFilter = [{"name": "IDENCARGO", "operator": "=", "value": 'D'}];
        }else if (widget.currentRow.CDENCARGONFE){
            encargo.dataSourceFilter = [{"name": "IDENCARGO", "operator": "=", "value": 'A'}];
        }else {
            encargo.dataSourceFilter = [];
        }
        widget.currentRow.CDENCARGO = '';
        widget.currentRow.NMENCARGO = '';
    };

    self.validateEncargo = function(widget) {
        /*Quando é alterado o Field, é necessario voltar o valor default para o campo, para assim alterá-lo*/
        widget.getField('CDENCARGONFE').reload();
        var encargo = [
            {
                "CDENCARGONFE": "01",
                "NMENCARGONFE": "Frete"
            },
            {
                "CDENCARGONFE": "02",
                "NMENCARGONFE": "Seguro"
            },
            {
                "CDENCARGONFE": "04",
                "NMENCARGONFE": "Outras Despesas"
            },
            {
                "CDENCARGONFE": "05",
                "NMENCARGONFE": "Despesas Aduaneiras"
            },
            {
                "CDENCARGONFE": "03",
                "NMENCARGONFE": "Descontar"
            }
        ];
        
        if (widget.currentRow.IDENCARGO) {
            widget.currentRow.CDENCARGONFE = "";
            widget.currentRow.NMENCARGONFE = "";
            
            if(widget.currentRow.IDENCARGO=='V'){
                widget.getField('CDENCARGONFE').readOnly = true;
                
            } else if (widget.currentRow.IDENCARGO=='A'){
                
                widget.getField('CDENCARGONFE').readOnly = false;
                widget.getField('CDENCARGONFE').dataSource.data = encargo;
                
            } else {
                
                widget.getField('CDENCARGONFE').readOnly = false;
                widget.getField('CDENCARGONFE').dataSource.data = encargo;
                    
            }
        
        } else {
            widget.currentRow.CDENCARGONFE = "";
            widget.currentRow.NMENCARGONFE = "";
            widget.getField('CDENCARGONFE').readOnly = true;
        }
        
        widget.getField('CDENCARGONFE').reload();
        
        
        
        
       
    };
    
    //Insere no campo NMENCARGONFE, o nome relativo ao CDENCARGONFE.
    self.BuscaNmencargonfe = function(){
        
        var widget = templateManager.container.getWidget("encargo");
            
            if(widget.getField('CDENCARGONFE').value() == '01'){
                widget.currentRow.NMENCARGONFE = "Frete";
            } 
            if(widget.getField('CDENCARGONFE').value() == '02'){
                widget.currentRow.NMENCARGONFE = 'Seguro';
            } 
            if(widget.getField('CDENCARGONFE').value() == '04'){
                widget.currentRow.NMENCARGONFE = "Outras Despesas";
            } 
            if(widget.getField('CDENCARGONFE').value() == '05'){
                widget.currentRow.NMENCARGONFE = "Despesas Aduaneiras";
            } 
            if(widget.getField('CDENCARGONFE').value() == '03'){
                widget.currentRow.NMENCARGONFE = "Descontar";
            } 
        
        
        widget.getField('CDENCARGONFE').reload();
        
    };
    
    // Campos incongruentes - Tratamento e Encargo
    self.validateEncargoNFE = function(){
        
        
        var widget = templateManager.container.getWidget("encargo");
        var fieldEncargo =  widget.getField('CDENCARGONFE');
        var fieldTratamento =  widget.getField('IDENCARGO');
        
        if((fieldTratamento.value() == "D") && (["01", "02", "04", "05"].includes(fieldEncargo.widget.currentRow.CDENCARGONFE))
        || (fieldTratamento.value() == "A") && (fieldEncargo.widget.currentRow.CDENCARGONFE == "03")){
            
            MessageService.showMessageByCode("INCONGRUENCIA_TRATAMENTO_ENCARGO");
            fieldEncargo.widget.currentRow.CDENCARGONFE = "";
            fieldEncargo.widget.currentRow.NMENCARGONFE = "";
            
        } 
    };
    
    self.validateAddEncargo = function(){
        var widget = templateManager.container.getWidget('encargo');
        
        self.validateEncargo(widget);
    };

    //UTILITIES
    self.changeWidgetAcessibility = function(widgets, enabled) {
        widgets.forEach(function(widget){
            widget.enabled = enabled;
        });

        templateManager.updateTemplate();
    };

    self.masterDetailCancelForm = function(widget) {
        if(self.formChangeTemplate(widget) && widget.getParent()) {
            var widgets = widget.getParent().widgets;

            self.changeWidgetAcessibility(widgets.concat(widget.getParent()), true);
        } else self.changeWidgetAcessibility(widget.widgets, true);

        if(widget.currentRow.__is_new) {
            var indexOfCurrentRow = widget.dataSource.data.indexOf(widget.currentRow);
            if(indexOfCurrentRow !== -1) {
                widget.dataSource.data.pop();
            }
            widget.list();

            ScreenService.closeSwipe();
        } else {
            Util.extend(widget.currentRow, widget.backupRow);
            widget.view();
        }
    };

    self.masterDetailEdit = function(widget) {
        if(!widget.editing) {
            if(self.formChangeTemplate(widget) && widget.getParent()) {
                var widgets = widget.getParent().widgets;

                self.changeWidgetAcessibility(widgets.filter(function(w) {return w.name != widget.name;}).concat(widget.getParent()), false);
            } else self.changeWidgetAcessibility(widget.widgets, false);

            widget.backupRow = Util.clone(widget.currentRow);
            widget.edit();
        }
    };

    self.masterDetailGoBackToList = function(widget) {
        ScreenService.closeSwipe();
        widget.list();
    };

    self.masterDetailNewRow = function(widget) {
        self.changeWidgetAcessibility(widget.widgets, false);

        widget.newRow();
    };
    
    self.saveEncargo = function(widget) {
        if(widget.isValid()) {
            var route = 'df#/saveEncargo';
            General.callRestEngine(route, 'Row', widget.currentRow)
            .then(function(response){
                ScreenService.closeSwipe();
                widget.reload();
            })
            .catch(function(error){
                ScreenService.showMessage(error.data.error);
            });
        }
        
    };


    self.formChangeTemplate = function(widget) {
        if((widget.template != "widget/form.html")&&(widget.template != "widget/form-view.html")) return false;

        if(!widget.templateToForm) widget.templateToForm = "widget/form.html";
        if(!widget.templateToView) widget.templateToForm = "widget/form-view.html";

        if(widget.template == widget.templateToForm) widget.template = widget.templateToView;
        else if(widget.template == widget.templateToView) widget.template = widget.templateToForm;

        templateManager.updateTemplate();

        return true;
    };
    
    self.encargoNfe = function(filterValue) {
        return restEngine.post('df#/crud/encargonfe', {
            "requestType": "FilterData",
            "filter": [ 
                        {
                            "name": "CDENCARGONFE",
                            "operator": "=",
                            "value": filterValue
                        }
                      ]
        });
	};
	
	self.saveEncargoNfe = function(row) {
        return restEngine.post('df#/crud/encargonfe/save', {
            "requestType": "Row",
            "row": row
        });
	};
	
	self.confereEncargoNfe = function(selectEncargoNfe, widget) {
	    let deferred = ZHPromise.defer();
	    
	    if(selectEncargoNfe.dataset.encargonfe.length === 0){
              widget.currentRow.__is_new = true;
              this.saveEncargoNfe(widget.currentRow).then(() => deferred.resolve());
          }
         else deferred.resolve();
         return deferred;
	};
    
    self.hideFloatingControl = function(field) {
     //  field.selectWidget.floatingControl.refreshAction = false;
    };
    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ger00300_encargo_controller', ger00300_encargo_controller);
});


// FILE: ./assets/js/controllers/zhFavorites.js
var ZeedhiFields = angular.module("ZeedhiFields");

ZeedhiFields.directive('zhNavbarDefaultHeader', function(templateManager, ScreenService, eventAggregator, $compile, ApplicationContext) {
	return {
		restrict: 'C',
		link: function($scope, element) {
		    $scope.favoritesList = [];
		    $scope.lastDeletedItem = undefined;
		    
		    var favoritesText = 'Favoritos';
		    var favoritesAddText = 'Adicionar aos Favoritos';
		    var favoritesRemoveText = 'Remover dos Favoritos';
		    var favoritesUndoText = 'DESFAZER';

			// TRANSLATE - Adicionar essas variaveis a lista da aplicação (words.jon)
		    // var favoritesText = ScreenService.i18n('FAVORITES');
		    // var favoritesAddText = ScreenService.i18n('ADD_TO_FAVORITES');
		    // var favoritesRemoveText = ScreenService.i18n('REMOVE_FROM_FAVORITES');
		    // var favoritesUndoText = ScreenService.i18n('UNDO');
		    
		    var favoritesContainer;
		    
		    var startHollowCode = '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 30 30" style="enable-background:new 0 0 30 30;" xml:space="preserve">'+
				'	<style type="text/css">'+
				'		.st0{fill:#FFFFFF;}'+
				'	</style>'+
				'	<g>'+
				'		<path class="st0" d="M15,4.21l3.51,7.1l7.84,1.14l-5.67,5.53l1.34,7.81L15,22.1l-7.01,3.69l1.34-7.81l-5.67-5.53l7.84-1.14L15,4.21'+
				'			 M15,2.21c-0.76,0-1.46,0.43-1.79,1.11l-3.04,6.16l-6.8,0.99c-0.75,0.11-1.38,0.64-1.61,1.36c-0.24,0.72-0.04,1.52,0.51,2.05'+
				'			l4.92,4.8l-1.16,6.77c-0.13,0.75,0.18,1.51,0.8,1.96c0.35,0.25,0.76,0.38,1.18,0.38c0.32,0,0.64-0.08,0.93-0.23l6.08-3.2l6.08,3.2'+
				'			c0.29,0.15,0.61,0.23,0.93,0.23c0.41,0,0.83-0.13,1.18-0.38c0.62-0.45,0.92-1.21,0.8-1.96l-1.16-6.77l4.92-4.8'+
				'			c0.55-0.53,0.74-1.33,0.51-2.05c-0.24-0.72-0.86-1.25-1.61-1.36l-6.8-0.99l-3.04-6.16C16.46,2.64,15.76,2.21,15,2.21L15,2.21z"/>'+
				'	</g>'+
				'</svg>';
		    var startFilledCode = '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 30 30" style="enable-background:new 0 0 30 30;" xml:space="preserve">'+
				'	<style type="text/css">'+
				'		.st0{fill:#FFFFFF;}'+
				'	</style>'+
				'	<g>'+
				'		<path class="st0" d="M28.25,11.83c-0.24-0.72-0.86-1.25-1.61-1.36l-6.8-0.99l-3.04-6.16C16.46,2.64,15.76,2.21,15,2.21'+
				'			s-1.46,0.43-1.79,1.11l-3.04,6.16l-6.8,0.99c-0.75,0.11-1.38,0.64-1.61,1.36c-0.24,0.72-0.04,1.52,0.51,2.05l4.92,4.8l-1.16,6.77'+
				'			c-0.13,0.75,0.18,1.51,0.8,1.96c0.35,0.25,0.76,0.38,1.18,0.38c0.32,0,0.64-0.08,0.93-0.23l6.08-3.2l6.08,3.2'+
				'			c0.29,0.15,0.61,0.23,0.93,0.23c0.41,0,0.83-0.13,1.18-0.38c0.62-0.45,0.92-1.21,0.8-1.96l-1.16-6.77l4.92-4.8'+
				'			C28.29,13.35,28.48,12.56,28.25,11.83z"/>'+
				'	</g>'+
				'</svg>';
		    
		    function createVisualElements() {
    			var headerContainer = $(element).children()[0];
    			headerContainer.style.width = 'calc(100% - 50px)';
    			
    			favoritesContainer = $('<div class="zh-favorites"></div>').appendTo(element);
    			
    			var favoritesButtonElement = $compile(
    				'<div class="zh-favorites-button" ng-click="toggleList()">'+
    				'  <div class="zh-favorites-button-star" ng-if="!isFavorited">'+
    				'    '+startHollowCode+
    				'  </div>'+
    				'  <div class="zh-favorites-button-star" ng-if="isFavorited">'+
    				'    '+startFilledCode+
    				'  </div>'+
    				'  <div class="zh-favorites-button-text">'+favoritesText+'</div>'+
    				'</div>'
    			)($scope);
    			var favoritesButton = $(favoritesButtonElement).appendTo(favoritesContainer);
    			
    			var favoritesListElement = $compile(
    				'<div class="zh-favorites-list" ng-class="{\'opened\': opened}">'+
    				'  <div class="zh-favorites-list-button" ng-if="!isFavorited" ng-click="addToFavorites()">'+favoritesAddText+'</div>'+
    				'  <div class="zh-favorites-list-button" ng-if="isFavorited" ng-click="removeFromFavorites()">'+favoritesRemoveText+'</div>'+
    				'  <div class="zh-favorites-list-undo" ng-show="lastDeletedItem" ng-click="undo()">'+favoritesUndoText+'</div>'+
    				'  <div class="zh-favorites-list-items" ng-show="favoritesList.length > 0">'+
    				'	 <div class="zh-favorites-list-item" ng-repeat="item in favoritesList track by item.id" ng-click="openItem(item)">'+
    				'      {{ item.label }}'+
    				'      <span class="zh-favorites-list-item-delete" data-zh-icon="trash icon-white no-border" ng-click="deleteItem(item, $index, $event)"></span>'+
    				'    </div>'+
    				'  </div>'+
    				'</div>'
    			)($scope);
    			var favoritesList = $(favoritesListElement).appendTo(favoritesContainer);
		    }
		    
		    function checkIsFavorited(containerId) {
		    	var oldIsFavorited = $scope.isFavorited;
			    $scope.isFavorited = $scope.favoritesList.findIndex((item) => item.id === containerId) !== -1;
			    if (oldIsFavorited !== $scope.isFavorited) {
			    	templateManager.updateTemplate();
			    }
		    }
		    
		    $scope.addToFavorites = function() {
		    	var item = {
		    		id: templateManager.container.id,
		    		name: templateManager.container.fileName.split('.json')[0],
		    		label: templateManager.container.label,
		    	};
		    	$scope.favoritesList.push(item);
		    	$scope.isFavorited = true;
		    	eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item, status: 'add' }});
		    };
		    
		    $scope.removeFromFavorites = function() {
		    	var index = $scope.favoritesList.findIndex((item) => item.id === templateManager.container.id);
		    	if (index > -1) {
			    	var item = $scope.favoritesList.splice(index, 1);
			    	$scope.isFavorited = false;
			    	eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item, status: 'delete' }});
		    	}
		    };
		    
		    $scope.openItem = function(item) {
		    	ApplicationContext.LibUtilities.openWindow(item.name);
	    		$scope.opened = false;
		    };
		    
		    var undoTimeout;
		    
		    $scope.deleteItem = function(item, index, event) {
		    	$scope.favoritesList.splice(index, 1);
		    	$scope.lastDeletedItem = { item, index };

		    	clearTimeout(undoTimeout);
		    	undoTimeout = setTimeout(() => {
		    		$scope.lastDeletedItem = undefined;
		    		templateManager.updateTemplate();
		    	}, 10000);

			    eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item, status: 'delete' }});

		    	if (item.id === templateManager.container.id) {
		    		$scope.isFavorited = false;
		    	}

		    	event.stopImmediatePropagation();
		    };
		    
		    $scope.undo = function() {
		    	if ($scope.lastDeletedItem) {
		    		$scope.favoritesList.splice($scope.lastDeletedItem.index, 0, $scope.lastDeletedItem.item);
		    	}
		    	$scope.lastDeletedItem = undefined;
		    };

		    $scope.toggleList = function() {
		    	$scope.opened = !$scope.opened;
		    };
		    
		    document.addEventListener('mousedown', (evt) => {
		    	if ($(favoritesContainer).find(evt.target).length === 0 && $scope.opened) {
		    		$scope.opened = false;
			    	templateManager.updateTemplate();
		    	}	
		    });
		    
		    eventAggregator.subscribe(eventAggregator.ON_CONTAINER_OPEN, (args) => {
		    	checkIsFavorited(args.container.id);
		    });
		    
		    eventAggregator.subscribe('ZhFavoritesOnLoad', (favorites) => {
		    	
		    	$scope.favoritesList = favorites;
			    if (templateManager.container) {
			    	checkIsFavorited(templateManager.container.id);
			    }
		    });

		    createVisualElements();
		}
	};
});

// FILE: ./assets/modules/entrada/assets/js/controllers/Dfe07900_Controle_de_Arquivos_Xml_Controller.js
var Dfe07900_Controle_de_Arquivos_Xml_Controller = function(ScreenService, templateManager, General, metaDataFactory, restEngine) {
    
    var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
    
    this.cancelar = function(widget) {
        if (widget.currentRow.__is_new) {
            widget.dataSource.data.pop();
            ScreenService.closeSwipe();
            widget.list();
        }
    };
    
    this.salvar = function(widget){
        var row = Util.clone(widget.currentRow, ignoredKeys);
        if (widget.isValid()) {
            General.callRestEngine('df_entrada#/novoDirArqXML', "Row", row).then(function() {
				widget.activate();
				if (widget.currentRow.__is_new) {
					widget.list();  
					ScreenService.goBack();
					widget.reload();
				} else {
					widget.view();
				}
			}.bind(this), this.errorRequest);
        }
    };
    
    this.onEnterOnce = function(widget) {
        General.callRestEngine('df_entrada#/getArquivosXML', "FilterData", widget.dataSourceFilter).then((response) => {
            
            widget.dataSource.data = response.dataset.arquivos_xml;
            General.callRestEngine('df_entrada#/getIsSaas', "FilterData", []).then( success => {
                if(success.dataset.isSaas[0]) {
                    widget.getParent().getField('DSDIRIMPARQXML').isVisible = false;
                    widget.getAction("adicionarDiretorio").isVisible = false;
                }
                else {
                    if(response.dataset.arquivos_xml[0].RAIZ){
                        widget.getParent().currentRow.DSDIRIMPARQXML = response.dataset.arquivos_xml[0].RAIZ;
                    } else {
                        ScreenService.showMessage("Diretório não parametrizado, favor adicionar um diretório base na tela \"Parametros de Documentos Fiscais de Entrada\"");
                    }
                }
                widget.getParent().redraw();
                widget.reload();
            });
        }).catch(this.errorRequest);  
    };
    
    this.openWidgetUpload = function(widget){
        var widgetUpload = templateManager.container.getWidget('widgetUploadXML');
        widgetUpload.currentRow = { __is_new: true };
        widgetUpload.isVisible  = true;
        ScreenService.openSwipe(widgetUpload).then(function(){
            widgetUpload.newRow();
        });
        
    };
    
    this.closeUploadSwipe = function(widget){
        ScreenService.closeSwipe();
        templateManager.container.getWidget('arvoreArquivosNfe').activate();
        widget.isVisible = false;
        
    };
    
    this.upload = function(widget){
        
        if(!widget.isValid()){
            return;
        }
        let row = widget.currentRow;
        let widgetReload = templateManager.container.getWidget('arvoreArquivosNfe');
        row.DIR = (widget.getParent().currentRow.DIR_FULL) ? widget.getParent().currentRow.DIR_FULL : widget.getParent().getParent().currentRow.DSDIRIMPARQXML;
        if(row.DIR || row.isSaas){
            General.callRestEngine('df_entrada#/uploadArqXML', "Row", row).then(function() {
                ScreenService.closeSwipe();
                widgetReload.resizeGridColumns(true);
                widgetReload.reload();
            }.bind(this), this.errorRequest);
        } else{
            ScreenService.closeSwipe();
            ScreenService.showMessage("Diretório não definido, não é possivel fazer o upload");
        }
        
    };
    
    this.download = function(widget){
        var row = Util.clone(widget.currentRow, ignoredKeys);
        let params = "/downloadArqXML?requestType=Row&row[DSARQUIVO]=" + row.DOWNLOAD;
        
        window.open(
            window.serviceUrls.filter(
                (x) => { return x.package == 'df'; }
            )[0].baseUrl + params
        );
        
    };
    
    this.excluir = function(widget){
        var row = Util.clone(widget.currentRow, ignoredKeys);
        General.callRestEngine('df_entrada#/excluirArqXML', "Row", row).then(function() {
            ScreenService.closeSwipe().then(function() {
                widget.list();
                widget.reload();
            });
        }.bind(this), this.errorRequest);
    };
    
    this.errorRequest = function(error) {
        if (error.data) {
            ScreenService.showMessage(error.data.error);
        } else {
            ScreenService.showMessage(error);
        }
        ScreenService.hideLoader();
    };
    
    this.novoDiretorio = function(widget){
        widget.newRow();
    };
};
Configuration(function(ContextRegister) {
    ContextRegister.register('Dfe07900_Controle_de_Arquivos_Xml_Controller', Dfe07900_Controle_de_Arquivos_Xml_Controller);
});

// FILE: ./assets/modules/entrada/assets/js/controllers/Dfe11000_LancamentoEntrada_Controller.js
var Dfe11000_LancamentoEntrada_Controller = function(ScreenService, templateManager, Df11000_Lancamento_Service, General, Dfe11000_LancamentoEntrada_Service, ValidacoesEstoque_Service, ZhFilterPreferences, ZHPromise, metaDataFactory, eventEngine, MessageService, Lib00206_InfComplProduto_Controller) {

    var self = this;
    
    var atualizaEstoque;

    /* ************************* Lancamento Entrada - Gerais ********************************* */

    const MESSAGES = {
        TIPO_OPERACAO_BLOQ_FORNECEDOR: {
            type: 'ERROR'
        },
        CHAVE_ACESSO_INVALIDA: {
            type: 'ALERT'
        },
        CHAVE_ACESSO_UTILIZADA: {
            type: 'ALERT'
        },
        PRODUTO_JA_RELACIONADO: {
            type: 'ALERT'
        },
        EXISTE_PRODUTOS_NAO_RELACIONADO: {
            type: 'ERROR'
        },
        NF_ORIGEM_NECESSARIA: {
            type: 'ERROR'
        },
        NF_ORIGEM_NAO_NECESSARIA: {
            type: 'ALERT'
        },
        NF_ORIGEM_NAO_CADASTRADA: {
            type: 'ERROR'
        },
        NFS_ORIGEM_NAO_CADASTRADAS: {
            type: 'ERROR'
        },
        AF_OBRIGATORIA: {
            type: 'INFORMATION'
        },
        AF_OBRIGATORIA_E_NENHUMA_ENCONTRADA: {
            type: 'ALERT'
        },
        AF_OBRIGATORIA_E_NENHUMA_SELECIONADA: {
            type: 'ERROR'
        },
        AF_OBRIGATORIA_E_INSUFICIENTE: {
            type: 'ALERT'
        },
        AFS_OBRIGATORIAS_E_INSUFICIENTES: {
            type: 'ALERT'
        },
        PERMITE_LANCAMENTO_MANUAL: {
            type: 'INFORMATION'
        }
    };
    
    this.preLoadAutocomplete = function(widget){
      if(widget.dataSource.data.length === 0)
        widget.reload();
    };

    const showMessageDfe = messageName => {
        const data = MESSAGES[messageName];

        return MessageService.showMessageByCode(messageName, null, data);
    };
    

    var requestError = function(error) {
        if (typeof error === 'string') {
            return error in MESSAGES ? showMessageDfe(error) : ScreenService.showMessage(error, 'ERROR');
        }

        if (Array.isArray(error)) {
            var erros = error.join(', ');
            return ScreenService.showMessage(erros, 'ERROR');
        }

        if (typeof error === 'object' && !!error.data) {
            return error.data.error in MESSAGES ?
                showMessageDfe(error.data.error) : ScreenService.showMessage(error.data.error, 'ERROR');
        }

        return ScreenService.showMessage('Tempo de execução se esgotou.<br>Por favor, tente novamente!', 'ERROR');
    };
    
    var getItensWidget = function(){
        return templateManager.container.getWidget("itens");
    };
    
    
    //Função que ajusta casas decimais de valor unitario de acordo com o parametro
    //Desativada por enquanto até futura definição
    this.ajustaCasasDecimais = function(){
        var itemsWidget = getItensWidget();
        var fieldItem = itemsWidget.getField('VRUNITITEM');
        
        /*General.callRestEngine('df#/crud/paramgeral', 'FilterData', []).then(function(success) {
            if(success.dataset.paramgeral[0]){
                var paramgeral = success.dataset.paramgeral[0];
                fieldItem.mask.params.precision = paramgeral.NRDECITOTANF;
            }
        });*/
    };


    const prepareAndCleanTaxes = tax => {
        tax.__is_new = true;

        tax.VRCONTIMPOIT = 0;
        tax.VRREDBCIMPOIT = 0;
        tax.VRBCALCIMPOIT = 0;
        tax.VRPDEDIMPOIT = 0;

        return tax;
    };

    const alteraTabelasImpostos = ({CDFILIAL, NRLANCTONF}) => Df11000_Lancamento_Service.getDataSourceItensImpostos()
        .then(taxes => taxes.map(prepareAndCleanTaxes))
        .then(Df11000_Lancamento_Service.salvaImpostosTmpItem)
        .then(() => Dfe11000_LancamentoEntrada_Service.limpaTrabsLancamento(CDFILIAL, NRLANCTONF));
        

    const alteraStatusNotafiscal = ({currentRow}) => {
        let deferred = ZHPromise.defer();

        if (currentRow.IDSTATUS !== 'D') {
            currentRow.IDSTATUS = 'D';

            validaEdicaoNota();

            Df11000_Lancamento_Service.salvaCabecalho(currentRow).then().catch(requestError);
            alteraTabelasImpostos(currentRow).then(deferred.resolve);
        } else
            deferred.resolve();

        return deferred.promise;
    };

    //Evento chamado ao sair da tela de lançamento de nota - deprecated on list
    this.voltarListaLancamentosNotas = function(args) {
        removeDataSourcesConsultaLancEntradaActionEvent();
        General.backOrCancel(args);
        args.owner.widget.reload();
    };

    const criaDetalhesLancamentoWidgetAfterMoveRow = consultaLancamentoRow =>
        Df11000_Lancamento_Service.criaDetalhesLancamento(consultaLancamentoRow)
            .then(lancamentoSaida => {
                templateManager.container.getWidget('itens').dataSource.data = lancamentoSaida.ITENS;
                templateManager.container.getWidget('encargos').dataSource.data = lancamentoSaida.ENCARGO;
                templateManager.container.getWidget('vencimentos').dataSource.data = lancamentoSaida.VENCIMENTO;
                templateManager.container.getWidget('dados_adicionais').dataSource.data = lancamentoSaida.DADOSADICIONAIS;
            });

    var validaEdicaoNota = function() {
        var lancamentoNotaWidget = getCabecalhoWidget();
        var statusNota = lancamentoNotaWidget.currentRow.IDSTATUS || 'D';
        var statusNotaTransmissao = lancamentoNotaWidget.currentRow.IDSTATUSNFE;
        
        var notaOl = lancamentoNotaWidget.currentRow.DSNOTAFISORIGOL != null;
        var notaCanceladaOuAprovada = (statusNota === 'A' || statusNota === 'C');
        var notaTransmissao = (statusNotaTransmissao === 'T' || statusNotaTransmissao === 'A' || statusNotaTransmissao === 'C' || statusNotaTransmissao === 'D');
        var permiteEditarNota = (!notaCanceladaOuAprovada && !notaTransmissao && (statusNota === 'D' || statusNota === 'P' || statusNota === 'L') && !notaOl);
        var allActions = lancamentoNotaWidget.getActions();

        var invisibleActions = allActions.filter(action => {
            var widgetName = action.widget.name;
            var actionName = action.name.toLowerCase();
            var isActionVoltar = (actionName.indexOf('voltar') === -1);
            var novaNota = (actionName.indexOf('novo') !== -1 && widgetName === 'cabecalho');
            var controleLancRetroativo = (actionName.indexOf('controlelancretro') !== -1 && widgetName === 'cabecalho');
            var zeedhiWidgets = (widgetName !== 'filterPopup' && widgetName !== 'stripe');
            return (isActionVoltar && zeedhiWidgets && !novaNota && !controleLancRetroativo);
        });

        invisibleActions.forEach(action => {
            action.isVisible = permiteEditarNota;

            if (action.name.toLowerCase().indexOf('fecharnota') !== -1 && statusNota === 'P') {
                action.isVisible = false;
            }
            
            if(statusNota === 'L' 
                && action.name.toLowerCase().indexOf('fecharnota') === -1 
                && action.name !== 'actCancelarCabecalho'){
                action.isVisible = false;
            }
            
            if(action.name.indexOf('actApagarCabecalho') !== -1) {
                action.isVisible = !notaOl;
            } 
            
            if(action.name.indexOf('DOWNLOAD') !== -1) {
                action.isVisible = true;
            }
            
        });

        var encargoInlineEdit = lancamentoNotaWidget.container.getWidget('encargos').getField('VRITEMENCA');
        if (permiteEditarNota) {
            encargoInlineEdit.inlineEditable = 'number';
        } else {
            delete encargoInlineEdit.inlineEditable;
        }
        
        lancamentoNotaWidget.redraw();
    };
    
    this.vencimentoWidgetAfterLoadDataSource = function(){
        var vencimentoWidget = templateManager.container.getWidget('vencimentos');
        
        if(vencimentoWidget.dataSource.data.length == 0){
            vencimentoWidget.dataSource.reload();
        }
    };

    this.validacoesNotaAfterMoveRow = function(consultaLancamentoArgs, widgetCabecalho) {
        var cabecalhoWidget = widgetCabecalho ? widgetCabecalho : consultaLancamentoArgs.owner;
        var consultaLancamentoRow = widgetCabecalho ? widgetCabecalho.currentRow : consultaLancamentoArgs.row;
        removeDataSourcesConsultaLancEntradaActionEvent();

        if ('NRLANCTONF' in consultaLancamentoRow)
            criaDetalhesLancamentoWidgetAfterMoveRow(consultaLancamentoRow);

        validaEdicaoNota();
        alteraVisibilidadeCampoInscricao();

        validaCampoModDocFiscal(cabecalhoWidget, consultaLancamentoRow);

        var cbutilizaafField = cabecalhoWidget.getField('CBUTILIZAAF');
        cbutilizaafField.readOnly = (!consultaLancamentoRow.CDTIPOOPER || !consultaLancamentoRow.CDFORNECED);
        
        this.habilitaTabProdutosOuNotas(cabecalhoWidget);
        
        cabecalhoWidget.getField('NMTIPOOPER').reload().then(() => {
            let widget = templateManager.container.getActiveWidget();
            if(!widget.editing) { //True = Visualizar nota.
                this.controlVisibilityLocalPrestServ(widget);
            }
        });
        
    };

    this.fecharNota = function(args) {
        var row, forcaFechamento;
        
        var cabecalhoWidget = templateManager.container.getWidget('cabecalho');
        
        cabecalhoWidget.reload();
        
        if(args) {
            if(args.owner.getParent().name == 'cabecalho') var popup = true;
            row = args.row;
            forcaFechamento = false;
        } else{
            row = cabecalhoWidget.currentRow;
            forcaFechamento = true;
        }
        
        if(row.IDSTATUS === 'L'){
            ScreenService.openPopup(templateManager.container.getWidget("confirmaDataLancto"));
        } else {
           ScreenService.confirmMessage('Deseja realizar o fechamento da nota fiscal?', 'question',
                (success) =>  {
                    Df11000_Lancamento_Service.fecharNota(cabecalhoWidget.currentRow, forcaFechamento)
                    .then(processaFechamento)
                    .catch((error) => erroFechamento(error, popup))
                    .then(() => reloadWidgetCabecalho())
                    .then(() => {
                        if(popup && templateManager.container.getActiveWidget().name != 'errosAvisos') {
                            ScreenService.closePopup();
                        }
                    });
                },
        
                function(error) {
                     return ScreenService.errorNotification(error, 'ERROR');
                }
            ); 
        }
    };
    
    this.setDefaultValuesConfDtLncto = function(widget){
        if(widget.getParent().currentRow.DTENTRSAID)
            widget.currentRow.DTENTRSAID = widget.getParent().currentRow.DTENTRSAID;  
    };
    
    this.fecharNotaLiberada = function(widget){
        var cabecalhoWidget = templateManager.container.getWidget('cabecalho');
        
        widget.getParent().currentRow.DTENTRSAID = widget.currentRow.DTENTRSAID;
        
        Df11000_Lancamento_Service.fecharNota(cabecalhoWidget.currentRow, false)
        .then(processaFechamento)
        .catch((error) => erroFechamento(error, true))
        .then(() => reloadWidgetCabecalho())
        .then(() => {
            if(templateManager.container.getActiveWidget().name != 'errosAvisos') {
                ScreenService.closePopup();
            }
        });
    };
     
    var reloadWidgetCabecalho = function() {
        templateManager.container.getWidget('cabecalho').reload();
    };

    const processaFechamento = (response) => {
        var dataset = response.dataset;
        var fechamento = dataset.fecharNota;
        const currentRow = getCabecalhoWidget().currentRow;

        currentRow.IDSTATUS = fechamento.STATUS;
        currentRow.VRNOTAFISC = fechamento.CABECALHO.VRNOTAFISC;
        
        if(fechamento.ERROS_APROVACAO[0]) {
            ScreenService.successNotification('Nota fechada com sucesso! Porém, não foi possível efetuar a aprovação da nota fiscal.');
        }
        else {
            ScreenService.successNotification('Nota fechada com sucesso!');
        }
        ScreenService.closeSwipe(0);
        Df11000_Lancamento_Service.geraRateio();
    };

    const erroFechamento = (error, popup) => { 
        if(popup) ScreenService.closePopup();
        var errosAvisos = templateManager.container.getWidget('errosAvisos');
        var bloqueia   = false;
        var consiste   = false;
        var errosAf    = ["AV01", "AV11", "AV12", "AV13", "AV15", "AV16"];
        errosAvisos.dataSource.data = error.data.dataset.errosFechamento;
        error.data.dataset.errosFechamento.forEach((erro) => {
            if(erro.IDERROAVISO == 'B') bloqueia = true;
            if(errosAf.indexOf(erro.CDERROAVISO) != -1) consiste = true;
        });
        errosAvisos.isVisible = true;
        errosAvisos.getAction('confirmarFechamento').isVisible = !bloqueia;
        errosAvisos.getAction('consistir_af_nf').isVisible = consiste;
        ScreenService.openPopup(errosAvisos);
        
        ScreenService.showMessage(error.data.error);
        this.errosAvisosAtualizaItens(templateManager.container.getWidget("errosAvisos"));
        return error;
    };
    
    this.errosAvisosAtualizaItens = function (widget) {
        var errosAvisos = widget.dataSource.data;
        var widgetProdutos = widget.getParent().widgets.filter (widget => widget.name === "itens")[0];
        widgetProdutos.reload().then ( () => {
            //Verifica erros que não são gravados no campo IDERROSITEM
            let produtos = widgetProdutos.dataSource.data;
            errosAvisos.forEach (erroAviso => {
                var produto;
                if (erroAviso.NRSEQITPROD) {
                    produto = produtos.filter(produto => produto.NRSEQITPROD === erroAviso.NRSEQITPROD)[0];
                }
                else if (erroAviso.ITEMERRO) {
                    let cdItem = erroAviso.ITEMERRO.split(" - ")[0];
                    produto = produtos.filter(produto => produto.CDPRODUTO === cdItem)[0];
                }
                else {
                    return;
                }
                if (erroAviso.IDERROAVISO === "B") {
                    produto.IDERROSITEM = "1";
                }
                else if (erroAviso.IDERROAVISO === "C") {
                    produto.IDAVISOSITEM = "1";
                }
            });
        });
    };

    var controlaStatusWidgets = function(widgetPai, nomeWidgetAtual, status, ignoredWidgets) {
        var allWidgets = widgetPai.widgets;

        allWidgets.filter(widget => widget.name !== nomeWidgetAtual).forEach(widget => widget.enabled = status);

        if (widgetPai.name !== nomeWidgetAtual)
            widgetPai.enabled = status;

        if (Array.isArray(ignoredWidgets))
            allWidgets.filter(widget => ignoredWidgets.indexOf(widget.name) !== -1).forEach(widget => widget.enabled = !status);
    };

    var removeEdicaoLinhasWidget = function(registrosAlterados) {
        registrosAlterados.forEach(registro => {
            registro.__is_new = false;
            Object.keys(registro).filter(key => !key.indexOf('changed') ? delete registro[key] : !key.indexOf('changed'));
        });
    };
    
    this.validacoesWidgetOnEnterOnce = function(consultaLancamentoArgs) {
        var cabecalhoWidget = consultaLancamentoArgs.owner;
        if(sessionStorage.getItem('TABLE_NAME') == 'rastreioContabil'){
                var filter = [
                        {
                            "name": "CDEMPRESA",
                            "operator": "=",
                            "value": sessionStorage.getItem('CDEMPRESA')
                        },
                        {
                            "name": "CDFILIRECE",
                            "operator": "=",
                            "value": sessionStorage.getItem('CDFILIRECE')
                        },
                        {
                            "name": "NRNOTAFISC",
                            "operator": "=",
                            "value": sessionStorage.getItem('NRNOTAFISC')
                        },
                        {
                            "name": "DTENTRSAID",
                            "operator": "BETWEEN",
                            "value": [sessionStorage.getItem('DTENTRSAID'), sessionStorage.getItem('DTENTRSAID')]
                        }
                ];
                cabecalhoWidget.dataSourceFilter = filter;
                cabecalhoWidget.reload().then ( () => {
                    cabecalhoWidget.moveToFirst();
                });
                sessionStorage.removeItem('TABLE_NAME');
                sessionStorage.removeItem('CDEMPRESA');
                sessionStorage.removeItem('CDFILIRECE');
                sessionStorage.removeItem('NRNOTAFISC');
                sessionStorage.removeItem('DTENTRSAID');
                return;
        }
        setDefaultFilialFilter(cabecalhoWidget).then(defaultFilter => {
            ZhFilterPreferences.setOriginalFilter(cabecalhoWidget.id, defaultFilter);
            ZhFilterPreferences.openFilterWidget(cabecalhoWidget, defaultFilter).then( () =>
                {
                    var filterWidget = templateManager.container.getActiveWidget();
                    filterWidget.getField('NMFILIAL').reload();
                    filterWidget.getField("IDSTATUS").setValue("");
                    filterWidget.getField("IDORIGDESTNF").setValue("");
                    filterWidget.actions.filter(action => action.name === "clear")[0].shortcutKey = "ctrl+backspace";
                });
            removeDataSourcesConsultaLancEntradaActionEvent();
        });
        
    };
    
    var removeDataSourcesConsultaLancEntradaActionEvent = function() {
        Df11000_Lancamento_Service.removeDataSourceLancamentoNota();
    };
    
    
    const setDefaultFilialFilter = (widget, index) => {
                var promise = ZHPromise.defer();
        let defaultFilter = Util.clone(widget.currentRow);
        defaultFilter.DTENTRSAID = undefined;

        General.callRestEngine('df#/filioper_lancamento_notafiscal', 'FilterData', 
                                [{"name": "CDOPERADOR", "operator": "=", "value": "OPERADOR_LOGADO"},
                                {"name": "CDFILIAL"  , "operator": "=", "value": "FILIAL_LOGADA"}]
            ).then(function(filialLogada) {
                let filialDefaultValue, cdFilialOut = null;
                let filialFieldOutData = widget.getField('NMFILIAL').outData;
                
                //Utiliza o primeiro dataset retornado. Necessário pois o dataset pode ser filioper_lancamento_notafiscal ou perfacessfili_lancamento_notafiscal
                for (let datasetName in filialLogada.dataset) {
                    if (filialLogada.dataset.hasOwnProperty(datasetName)) {
                        filialDefaultValue = filialLogada.dataset[datasetName][0];
                	    break;
                    }
                }
                if(filialDefaultValue) cdFilialOut = filialDefaultValue.CDFILIAL;
                
                Df11000_Lancamento_Service.buscaDataProcessamento(cdFilialOut).then(([processamento]) => {
                    if(processamento) {
                        const dtProcessFormat = moment(processamento.DTPROCESSA, 'DD/MM/YYYY').format('DD/MM/YYYY');
                        defaultFilter.DTENTRSAID = {start: dtProcessFormat, period: undefined, end: dtProcessFormat};
                    }
                    promise.resolve(defaultFilter);
                });
                setCurrentRowWithOutData(defaultFilter, filialDefaultValue, filialFieldOutData);
        });
        return promise;
        
    };
    
    const setCurrentRowWithOutDataFilial = (CDFILIAL, {dataSourceFilter, dataSource: {name: route}, outData}, currentRow, entryDateField = false) => {
        const DEFAULT_FILTER = {name: 'CDFILIAL', operator: '=', value: CDFILIAL},
            DATASOURCE_NAME_REGEXP = /(?:[A-z_]+#)?\/?(?:crud\/)?(.*)/; //matches the last part of the route, ignoring '/', 'serviceUrl#' and 'crud/'

        const filial = filter => filter.name == 'CDFILIAL',
            remove = fn => obj => !fn(obj),
            getDataSource = RegExp.prototype.exec.bind(DATASOURCE_NAME_REGEXP);

        const [, dataSourceName] = getDataSource(route), //[, var] to skip the match and get the capture group
            filter = [
                DEFAULT_FILTER,
                ...(dataSourceFilter && dataSourceFilter.length?
                    dataSourceFilter.some(filial)?
                        dataSourceFilter.filter(remove(filial))
                        : dataSourceFilter
                    : [])
            ];

        if (entryDateField)
                return Df11000_Lancamento_Service.buscaDataProcessamento(CDFILIAL)
                .then(([processing]) => {
                    const DTPROCESSA = moment(processing.DTPROCESSA, 'DD/MM/YYYY').format('DD/MM/YYYY');
                    currentRow.DTENTRSAID = {start: DTPROCESSA, period: undefined, end: DTPROCESSA};
                    return General.callRestEngine(route, 'FilterData', filter)
                        .then(response =>response.dataset[dataSourceName])
                        .then(([filialRow]) => setCurrentRowWithOutData(currentRow, filialRow, outData) || filialRow);
                });
        else {
            return General.callRestEngine(route, 'FilterData', filter)
            .then(response =>response.dataset[dataSourceName])
            .then(([filialRow]) => setCurrentRowWithOutData(currentRow, filialRow, outData) || filialRow);
        }
        
    };
    
    this.getDataProcessamento = function(filialSelecionada){
        var promise = ZHPromise.defer();
        Df11000_Lancamento_Service.buscaDataProcessamento(filialSelecionada).then(function(processamento){
            var dtProcessFormat = moment(processamento[0].DTPROCESSA, 'DD/MM/YYYY').format('DD/MM/YYYY');
            promise.resolve(dtProcessFormat);
        });
        return promise;
    };
    
    var setCurrentRowWithOutData = function(currentRow, valuesToSetOnCurrentRow, outDataValue) {
        for (var key in outDataValue) {
            currentRow[outDataValue[key]] = valuesToSetOnCurrentRow[key];
        }
    };

    var limpaWidgetAutoForne = function(autoForneWidget, cabecalhoWidget) {
        autoForneWidget.unSelectAll();
        delete cabecalhoWidget.currentRow.AF;
        cabecalhoWidget.currentRow.CBUTILIZAAF = null;
    };

    var getCabecalhoWidget = function() {
        return templateManager.container.getWidget('cabecalho');
    };

    var removeOutDataAndRequired = function(currentRow) {
        for (var i = 1; i < arguments.length; i++) {
            var field = arguments[i];
            setCampoNaoObrigatorio(field);
            limpaCamposOutData(field, currentRow);
        }
    };

    var setCampoObrigatorio = function(field, readOnly) {
        field.validations = {required: true};
        field.readOnly = !!readOnly;
    };

    var setCampoNaoObrigatorio = function(field) {
        delete field.validations;
        field.readOnly = true;
    };

    var limpaCamposOutData = function(field, currentRow) {
        var outData = field.outData;
        for (var value in outData) {
            delete currentRow[outData[value]];
        }
    };

    var buscaOperadorLogado = function() {
        return Df11000_Lancamento_Service.buscaOperadorLogado();
    };


    var clearRowWithOutData = function(currentRow, outDataValue) {
        for (var key in outDataValue) {
            currentRow[outDataValue[key]] = null;
        }
    };

    /* ************************* Lancamento Entrada - Filtro  *********************************** */
    this.getFieldValueFilter = function(filedName, widgetName) {
        var container =  templateManager.container;
        if(container) {
            var filterWidget;
            var filterBforeApply = templateManager.container.getActiveWidget();
            if(filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
              filterWidget = filterBforeApply;
            else {
                var f = container.getWidget(widgetName);
                if(f) {
                     filterWidget = container.getWidget(widgetName).appliedFilter;
                }
               
            }
            
            if(filterWidget) {
                var valueField;
                if(filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
                    valueField = filterWidget.currentRow[filedName];
                else
                    valueField = filterWidget.FILTER_ROW[filedName];
                    
                return valueField || null;
            }
            else return null;
        }
        else return null;
    };

    var validacaoCamposOutTpOperacFilter = function(principalField) {
        principalField.showOnView = true;
        principalField.showOnForm = true;
        principalField.isVisible = true;
        principalField.readOnly = false;
        principalField.reload();

        for (var i = 1; i < arguments.length; i++) {
            arguments[i].showOnView = false;
            arguments[i].showOnForm = false;
            arguments[i].isVisible = false;
            arguments[i].readOnly = true;
        }
    };

    var alteraVisibilidadeCampoInscricaoFiltro = function(filterArgs) {
        var filtroWidget = filterArgs.owner.field.widget;
        var currentRow = filtroWidget.currentRow;
        var cliente = filtroWidget.getField('NMRAZSOCCLIE');
        var fornecedor = filtroWidget.getField('NMRAZSOCFORN');
        var filial = filtroWidget.getField('DSFILIEMIT');
        var tipoDestino = filterArgs.row.IDORIGDESTNF;

        removeOutDataAndRequired(currentRow, cliente, fornecedor, filial);

        switch (tipoDestino) {
            case 'C':
                validacaoCamposOutTpOperacFilter(cliente, fornecedor, filial);
                break;
            case 'F':
                validacaoCamposOutTpOperacFilter(fornecedor, cliente, filial);
                break;
            case 'L':
                validacaoCamposOutTpOperacFilter(filial, fornecedor, cliente);
                break;
            default:
                cliente.readOnly = true;
                cliente.reload();
                fornecedor.readOnly = true;
                fornecedor.reload();
                filial.readOnly = true;
                filial.reload();
                break;
        }
    };

    /**
         * @desc - Altera o campo de remetente dependendo do valor que usuário escolher
        * @emit - No evento OnChange do campo IDORIGDESTNF do filtro
        * @param {[object]} filterArgs
    */
    this.idorigdestnfOnChangeFiltro = function (filterArgs) {
        alteraVisibilidadeCampoInscricaoFiltro(filterArgs);
    };

    /**
         * @desc - Limpa o "dataSourceFilter", pois o framework está concatenando a row do filter com o dataSourceFilter,
        *         o que gerava objetos repetidos.
        * @emit - No evento FilterBeforeApply
        * @param object filterArgs
    */
    this.criaDataSourceFilterBeforeApply = function(filterArgs) {
        var widgetPrincipal = filterArgs.owner;
        widgetPrincipal.dataSourceFilter = [];
    };

    /**
         * @desc - Adicionar os paremetros seguindo os "parameters bag" da query.
        * @emit - No evento FilterAfterApply
        * @param Object filterArgs
    */


    /* ************************* Lancamento Entrada - Cabeçalho ********************************* */

    const setCamposDefaultValueCabecalho = cabecalhoWidget => {
        var deferred = ZHPromise.defer();
        var {currentRow} = cabecalhoWidget;

        Df11000_Lancamento_Service.getDefaultNovaNota().then((defaultNota) => {
            currentRow = Object.assign(currentRow, defaultNota[0]);
            currentRow.__is_new = true;
            this.defaultValueDestinatario();
            unidadeOnChangeNovaNota(cabecalhoWidget).then(() => {
                cabecalhoWidget.activate();
                deferred.resolve();
            });
        });

        return deferred;
    };
    
    this.defaultValueDestinatario = function() {
        var data,
            dataSource,
            currentRow  =  getCabecalhoWidget().currentRow,
            filterRow = getCabecalhoWidget().appliedFilter.FILTER_ROW,
            dataSourceFilter =  [{ "name": "CDOPERADOR", "operator": "=", "value": "OPERADOR_LOGADO"}];
        
        General.callRestEngine("df#/filioper_lancamento_notafiscal", "FilterData", dataSourceFilter).then((filialLogada) => {
            
            for (let datasetName in filialLogada.dataset) {
                if (filialLogada.dataset.hasOwnProperty(datasetName)) {
                    dataSource = filialLogada.dataset[datasetName];
            	    break;
                }
            }
            
            if (dataSource) {
                data = dataSource.filter(data => data.CDFILIAL == filterRow.CDFILIRECE)[0];

                currentRow.DSENDEDEST       = data.DSENDEFILI;
                currentRow.NMBAIRDEST       = data.NMBAIRFILI;
                currentRow.NRCEPDEST        = data.NRCEPFILI;
                currentRow.NRTELEDEST       = data.NRTELEFILI;
                currentRow.IEDEST           = data.IEDEST;
                currentRow.SGESTADODEST     = data.SGESTADO ;
                currentRow.NMMUNICIPIODEST  = data.NMMUNICIPIO;
            }

        });
        

    };

    this.notaActionEvent = ({owner: {widget}}) =>
        Df11000_Lancamento_Service.removeDataSourceLancamentoNota()
            .then(() => {
                ScreenService.showLoader();
                Df11000_Lancamento_Service.inicializaLancamento();
                return Df11000_Lancamento_Service.setDataSourceLancamentoNota();
            }).then(() => {
                widget.newRow();
                widget.getField('DSLINKANEXO').isVisible = true;
                this.modeloDeDocumentoOnChange('', widget);
                widget.getField('CBUTILIZAAF').validations = {};
                widget.getField("CDSERIEENTR").reload();
                this.requireCTE(widget);
                controlaStatusWidgets(widget, widget.name, false, ['relacionamentoItens']);
                widget.container.getWidget('relacionamentoItens').isVisible = false;
                setCamposDefaultValueCabecalho(widget)
                .then(() => {
                    var widgetCabecalho = templateManager.container.getWidget('cabecalho');
                    widgetCabecalho.redraw();
                });
                applyAccessKeyAccessibilityRules(widget, false);
                ScreenService.hideLoader();
            });

    this.excluirNotaEntradaActionEvent = cabecalhoArgs =>
        ScreenService.confirmMessage(
            ScreenService.i18n('Excluir Registro?'),
            'question',
            () => Dfe11000_LancamentoEntrada_Service.excluirNotaEntrada(cabecalhoArgs.row)
                .then(() => {
                    General.backOrCancel(cabecalhoArgs);
                    const {data} = cabecalhoArgs.owner.widget.dataSource;

                    const index = data.findIndex(notafiscal => cabecalhoArgs.row.NRLANCTONF === notafiscal.NRLANCTONF);

                    if (index > 0)
                        data.splice(index, 1);

                    cabecalhoArgs.owner.widget.reload();
                    ScreenService.successNotification('Nota Fiscal excluída com sucesso');
                })
                .catch(requestError)
        );
    
    this.editarNotaEntrada = function(args){
        let widget = args.owner.widget,
        currentRow = Util.clone(widget.currentRow);
        
        controlaStatusWidgets(widget, widget.name, false);
        applyZeroFillNumeroNota(widget);
        
        widget.getField('DSLINKANEXO').isVisible = false;
        
        this.getDataProcessamento(currentRow.CDFILIAL).then( dtProcessamento => {
            currentRow.DTPROCESSA = moment(dtProcessamento, 'DD/MM/YYYY');
            currentRow.DTENTRSAID = moment(currentRow.DTENTRSAID, "DD/MM/YYYY hh:mm:ss");
            if(currentRow.DTPROCESSA.valueOf() > currentRow.DTENTRSAID.valueOf()){
                Dfe11000_LancamentoEntrada_Service.tipoEdicao(currentRow, dtProcessamento).then( response => {
                    if(response == 'N'){
                        this.habilitaEdicaoNotaEntrada(widget);
                        this.validaEdicaoCabecalho(null, widget);
                        this.controlVisibilityLocalPrestServ(widget);
                    } else if(response == 'S'){
                        this.openLancRetro(args);
                    } else if(response == 'B'){
                        ScreenService.showMessage("Edição bloqueada");
                    }
                });  
            } else{
                this.habilitaEdicaoNotaEntrada(widget);
                this.validaEdicaoCabecalho(null, widget);
                this.controlVisibilityLocalPrestServ(widget);
            }
              
        });
        
         
    };
    
    this.validaEdicaoCabecalho = function(widget){
        this.modeloDeDocumentoOnChange(null, widget);
    };
    
    this.habilitaEdicaoNotaEntrada = function(widget){
        General.edit(widget);
        widget.getField('CBUTILIZAAF').validations = {};
    };

    var validaCampoModDocFiscal = function(lancamentoCabecalhoWidget, lancamentoCabecalhoRow) {
        var descModFisc = lancamentoCabecalhoWidget.getField('DSMODDOCFISC');
        descModFisc.readOnly = !lancamentoCabecalhoRow.CDTIPOOPER;
        descModFisc.reload();

        var tipoOperac = lancamentoCabecalhoRow.CDTIPOOPER;
        if (tipoOperac === '09' || tipoOperac === '10')
            delete descModFisc.validations;
        else
            descModFisc.validations = {required: true};
    };

    var validaCampoEmpresa = function(lancamentoCabecalhoArgs) {
        var empresa = getCabecalhoWidget().getField('NMEMPRESA');
        empresa.readOnly = !lancamentoCabecalhoArgs.row.CDFILIEMIT;
    };

    var validaCampoSerie = function(lancamentoCabecalhoRow, limpaCampo = false) {
        var serie = getCabecalhoWidget().getField('CDSERIEENTR');
        serie.readOnly = !lancamentoCabecalhoRow.NMEMPRESA;

        if (limpaCampo)
            serie.value(null);

        var tipoOperac = lancamentoCabecalhoRow.CDTIPOOPER;
        var operacoesSerieNaoObrig = ['09', '10', '12', '13'];
        var tipoOperacObrigaSerie = operacoesSerieNaoObrig.indexOf(tipoOperac) !== -1;

        if (tipoOperacObrigaSerie) {
            delete serie.validations;
        } else {
            serie.validations = {required: true};
        }
    };

    
    this.dateOnChange = function(dateField){
        var dataAtual =  dateField.value();
        var widget = dateField.widget;
        var status = dateField.widget.getField('IDSTATUS');
        
        var today = moment().format('DD/MM/YYYY');
        var nowFullTime = moment().format('DD/MM/YYYY');
        if(moment(dateField.value(), 'DD/MM/YYYY').format('DD/MM/YYYY') === today)
            dateField.value(nowFullTime);

    };


    var validaCampoNumero = function(lancamentoCabecalhoRow) {
        var numero = getCabecalhoWidget().getField('NRNOTAFISC');

        var tipoOperac = lancamentoCabecalhoRow.CDTIPOOPER;
        var operacoesNumeroNaoObrig = ['08', '09', '10', '12', '13'];

        var tipoOperacObrigaNumero = operacoesNumeroNaoObrig.indexOf(tipoOperac) !== -1;

        if (tipoOperacObrigaNumero || lancamentoCabecalhoRow.IDUTILEMISNF === 'S')
            delete numero.validations;
        else {
            if(!numero.validations) {
                numero.validations =  { "required": {"message": "Obrigatório"},
                                        "maxLength": { "length": 20}};
            }
        }
    };

    var limpaCamposOutDataSelectsTpOperac = function() {
        var cabecalhoWidget = getCabecalhoWidget();
        var cabecalhoRow = cabecalhoWidget.currentRow;
        var cliente = cabecalhoWidget.getField('NMRAZSOCCLIE');
        var fornecedor = cabecalhoWidget.getField('NMRAZSOCFORN');
        var filial = cabecalhoWidget.getField('DSFILIEMIT');

        filial.value(null);

        cliente.value(null);
        cabecalhoRow.CDCLIENTE = null;

        fornecedor.value(null);
        cabecalhoRow.CDFORNECED = null;
        cabecalhoRow.IDSITUCADA = null;
        cabecalhoRow.IDBLQNFRELAFOAF = null;

        cabecalhoRow.NRINSJURNF = null;
        cabecalhoRow.NMRAZSOCFORN = null;
        cabecalhoRow.IDTPIJURNF = null;
    };

    var validacaoCamposOutTpOperac = function(princialField) {
        setCampoObrigatorio(princialField);
        princialField.showOnView = true;
        princialField.showOnForm = true;
        princialField.isVisible = true;
        princialField.reload();

        for (var i = 1; i < arguments.length; i++) {
            setCampoNaoObrigatorio(arguments[i]);
            arguments[i].showOnView = false;
            arguments[i].showOnForm = false;
            arguments[i].isVisible = false;
        }
    };

    const reloadCampoTipoOperacao = widget => {
        var deferred = ZHPromise.defer();
        const field = widget.getField('NMTIPOOPER');
        field.reload().then(() => {
                var data = field.dataSource.data;
                if (!data || !data.length)
                    return false;

                const {currentRow} = field.widget;
                const defaultRow = data.find(operacao => operacao.CDTIPOOPER === '01');
                if(defaultRow)
                    setCurrentRowWithOutData(currentRow, defaultRow, field.outData);
                else
                    field.value(null);
                field.change();
                deferred.resolve();
            });
        return deferred;
    };
    
    const unidadeOnChangeNovaNota = cabecalhoWidget => {
        var deferred = ZHPromise.defer();
        var autoForneWidget = cabecalhoWidget.container.getWidget('autorizacaoFornecimento');
        limpaWidgetAutoForne(autoForneWidget, cabecalhoWidget);
        applyZeroFillNumeroNota(cabecalhoWidget);
        reloadCampoTipoOperacao(cabecalhoWidget).then(() => {
            deferred.resolve();
        });
        return deferred;
    };

    const validaFornecedorBloqTpOperac = widget => {
        var row = widget.currentRow;
        if(row.IDUTBLOQOPAF === 'S')
            Dfe11000_LancamentoEntrada_Service.validaFornecedorBloqTpOperac(row.CDTIPOOPER, row.NRINSJURNF)
                .then(({dataset: {parambloqnfafor_fornecedorparamsbloq: data}}) => {
                    const BLOCK = (data.length === 1);
                    row.ERROS_LANCAMENTO = {
                        fornecedorBloqTipoOperac: BLOCK
                    };

                    if (BLOCK){
                        showMessageDfe('TIPO_OPERACAO_BLOQ_FORNECEDOR');
                        row.CDTIPOOPER = null;
                        row.NMTIPOOPER = null;
                        widget.getField('NMTIPOOPER').change();
                    }
                });
        else
            return false;
    };

    var limpaCampoChaveAcessoPeloTipoOperac = function(cabecalhoWidget) {
        var currentRowCabecalho = cabecalhoWidget.currentRow;
        var fieldChaveAcesso = cabecalhoWidget.getField('NRACESSONFE');
        var chaveAcessoValue = fieldChaveAcesso.value();
        var tipoOpeacaoValue = currentRowCabecalho.CDTIPOOPER;

        fieldChaveAcesso.readOnly = !tipoOpeacaoValue;

        if (chaveAcessoValue) {
            limpaOutDataChaveAcess(currentRowCabecalho);
            applyAccessKeyAccessibilityRules(cabecalhoWidget, false);
        }
    };

    /**
         * @desc - Chama as validações responsáveis por alterar a visibilidade dos
        *         campos de "remetente" verificando se é "Cliente", "Fornecedor" ou "Filial"
        *       - Validar se a "chave de acesso" e "af" é obrigatório
        *       - Validar se o número da nota "NRNOTAFISC" é obrigatório
        *       - Validar se o "modelo de documento" é obrigatório
        *       - Validar se a "série" é obrigatória
        * @emit - No evento de onChange do campo "Operação" - "NMTIPOOPER"
        * @param object cabecalhoArgs
    */
    this.tipoOperacaoOnChange = function(cabecalhoArgs) {
        var cabecalhoCurrentRow = cabecalhoArgs.row;
        var cabecalhoWidget = cabecalhoArgs.owner.field.widget;

        cabecalhoCurrentRow.IDORIGDESTNF = cabecalhoCurrentRow.IDORIGDESTOP;

        limpaCampoChaveAcessoPeloTipoOperac(cabecalhoWidget);
        alteraVisibilidadeCampoInscricao();
        validaCampoModDocFiscal(cabecalhoWidget, cabecalhoCurrentRow);
        validaCampoSerie(cabecalhoCurrentRow);
        validaCampoNumero(cabecalhoCurrentRow);
        limpaCamposOutDataSelectsTpOperac(cabecalhoWidget);
        validaCampoAF(cabecalhoWidget);
        validaChaveAcessoObrigatoria(cabecalhoWidget);
        validaCampoNotaOrigem(cabecalhoWidget, cabecalhoCurrentRow);
        this.controlVisibilityLocalPrestServ(cabecalhoWidget);
    };

    /**
        * @desc - Chama a validação da chave de acesso obrigatória
        * @emit - No evento de onChange do campo "Modelo de Documentos" - "DSMODDOCFISC"
        * @param object cabecalhoArgs
    */
    this.modeloDeDocumentoOnChange = function(cabecalhoArgs, cabecalhoWidget = '') {
        cabecalhoWidget = (cabecalhoWidget) ? cabecalhoWidget : templateManager.container.getWidget('cabecalho');
        
        if(cabecalhoWidget.currentRow.IDOBRIGANEXO == 'S')
            cabecalhoWidget.getField('DSLINKANEXO').validations.required = true;
        else
            cabecalhoWidget.getField('DSLINKANEXO').validations = {};
        validaChaveAcessoObrigatoria(cabecalhoWidget);
        this.requireCTE(cabecalhoWidget);
    };

    this.requireCTE = function(widget){
        //Verifica se é necessário os campos de CTE
        let requireCTE =  ["57", "63", "67"].includes(widget.currentRow.CDMODDOCFISC);
        
        if(requireCTE){
            widget.getField("CDIBGEORIG").validations.required = true;
            widget.getField("CDIBGEDEST").validations.required = true;
            widget.getField("CDTIPOCTE").validations.required  = true;
        } else{
            widget.getField("CDIBGEORIG").validations = {};
            widget.getField("CDIBGEDEST").validations = {};
            widget.getField("CDTIPOCTE").validations  = {};
        }
    };

    this.unidadeFieldOnChange = function(cabecalhoWidget) {
        var autoForneWidget = cabecalhoWidget.container.getWidget('autorizacaoFornecimento');
        limpaWidgetAutoForne(autoForneWidget, cabecalhoWidget);
        applyZeroFillNumeroNota(cabecalhoWidget);
        reloadCampoTipoOperacao(cabecalhoWidget);
    };
    
    this.empresaFieldOnChange = function(field) {
        var widget = field.widget;
        var fieldFilial =  widget.getField('NMFILIAL');
        widget.currentRow.CDFILIAL = '';
        widget.currentRow.NMFILIAL = '';
        if (field.value()) {
           fieldFilial.readOnly = false;
        }
        else {
            fieldFilial.readOnly = true;
        }
        fieldFilial.setValue("");
        widget.getField('NMFILIAL').reload();
    };

    this.cdfiliemitOnChange = function(lancamentoCabecalhoArgs) {
        validaCampoEmpresa(lancamentoCabecalhoArgs);
        validaCampoSerie(lancamentoCabecalhoArgs.row);
    };

    this.cancelaEdicaoCabecalho = function(cabecalhoArgs) {
        var cabecalhoWidget = cabecalhoArgs.owner.widget;
        General.backOrCancel(cabecalhoArgs).then(() => {
            controlaStatusWidgets(cabecalhoWidget, cabecalhoWidget.name, true);
            alteraVisibilidadeCampoInscricao();
            
            cabecalhoWidget.getField('DSLINKANEXO').isVisible = false;
            
            cabecalhoWidget.reload();
        });
    };

    this.remetenteFieldOnChange = function(cabecalhoArgs) {
        var remetenteField = cabecalhoArgs.owner.field;
        var cabecalhoWidget = remetenteField.widget;

        if (remetenteField.name == 'NMRAZSOCFORN')
            this.validaFornecedorOnChange(cabecalhoArgs);
        
        var dsinsjurnf;
        switch (remetenteField.name) {
            case "NMRAZSOCCLIE": dsinsjurnf = cabecalhoWidget.currentRow.NMRAZSOCCLIE; break;
            case "NMRAZSOCFORN" : dsinsjurnf = cabecalhoWidget.currentRow.NMRAZSOCFORN; break;
            case "DSFILIEMIT": dsinsjurnf = cabecalhoWidget.currentRow.DSFILIEMIT; break;
        }
        
        cabecalhoWidget.currentRow.DSINSJURNF = dsinsjurnf;

        validaCampoNotaOrigem(cabecalhoWidget, cabecalhoWidget.currentRow);
        validaChaveAcessoObrigatoria(cabecalhoWidget);
        validaCampoAF(cabecalhoWidget);
        // processaModelFieldNotaOrigem(cabecalhoWidget.getField('DSNOTAFISCORIG'), cabecalhoWidget.currentRow);
    };
    
    var alteraVisibilidadeCampoNotaOrigem = function(cabecalhoWidget) {
        var validacoes = processaValidacoesNotaOrigem(cabecalhoWidget.currentRow);
        if(validacoes.usaNotaOrigem) {
            if(validacoes.usaListaNotaOrigem) {
                cabecalhoWidget.getField('DSNOTAFISCORIG').isVisible = false; 
                cabecalhoWidget.getField('PKNOTAFISCORIG').isVisible = true; 
            } else {
                cabecalhoWidget.getField('PKNOTAFISCORIG').isVisible = false; 
                cabecalhoWidget.getField('DSNOTAFISCORIG').isVisible = true; 
            }    
        } else {
            cabecalhoWidget.getField('PKNOTAFISCORIG').isVisible = false; 
            cabecalhoWidget.getField('DSNOTAFISCORIG').isVisible = false; 
        }
    };
    
    this.cabecalhoWidgetAfterMoveRow = function(cabecalhoWidget) {
        alteraVisibilidadeCampoNotaOrigem(cabecalhoWidget);
    };
    
    var getNotaOrigemField = function(cabecalhoWidget) {
        var notaOrigemField;
        alteraVisibilidadeCampoNotaOrigem(cabecalhoWidget);
        notaOrigemField = cabecalhoWidget.getField('DSNOTAFISCORIG').isVisible? cabecalhoWidget.getField('DSNOTAFISCORIG') : cabecalhoWidget.getField('PKNOTAFISCORIG');
        
        return notaOrigemField;
        
    };

    //POR ENQUANTO SÓ ESTÁ PREPARADO PARA NOTAS DE DEVOLUÇÃO, DESCONSIDERANDO AS DE COMPLEMENTO E A DE DEVOLUÇÃO INDUSTRIAL
    var validaCampoNotaOrigem = function(cabecalhoWidget, cabecalhoCurrentRow) {
        var notaOrigemField = getNotaOrigemField(cabecalhoWidget);

        limpaCamposOutData(notaOrigemField, cabecalhoCurrentRow);
        processaViewFieldNotaOrigem(notaOrigemField, cabecalhoCurrentRow);

        processaModelFieldNotaOrigem(notaOrigemField, cabecalhoCurrentRow);
    };

    var processaValidacoesNotaOrigem = function(cabecalhoCurrentRow) {
        var validacoesNotaOrigem = {};

        validacoesNotaOrigem.usaNotaOrigem = cabecalhoCurrentRow.IDOPERDEVONF == 'S' &&
                                                !!cabecalhoCurrentRow.NRINSJURNF;

        validacoesNotaOrigem.notaOrigemObrigatoria = validacoesNotaOrigem.usaNotaOrigem &&
                                                        cabecalhoCurrentRow.IDNFORIGOBRI == 'S';

        validacoesNotaOrigem.usaListaNotaOrigem = validacoesNotaOrigem.usaNotaOrigem &&
                                                    cabecalhoCurrentRow.IDUTLLISTORI == 'S';

        validacoesNotaOrigem.limitaNotaOrigem = validacoesNotaOrigem.usaNotaOrigem &&
                                                cabecalhoCurrentRow.DSOPERNFORIG !== null &&
                                                cabecalhoCurrentRow.DSOPERNFORIG !== undefined &&
                                                cabecalhoCurrentRow.DSOPERNFORIG.length > 4;

        validacoesNotaOrigem.notaOrigemEntrada = validacoesNotaOrigem.limitaNotaOrigem &&
                                                    cabecalhoCurrentRow.DSOPERNFORIG.indexOf('E') !== -1;

        validacoesNotaOrigem.notaOrigemSaida = validacoesNotaOrigem.limitaNotaOrigem &&
                                                cabecalhoCurrentRow.DSOPERNFORIG.indexOf('S') !== -1;

        validacoesNotaOrigem.estorno = cabecalhoCurrentRow.IDNFESTCANC != 'N';

        var DESTINATARIO = cabecalhoCurrentRow.IDORIGDESTOP || cabecalhoCurrentRow.IDORIGDESTNF;

        validacoesNotaOrigem.destinatario = {};
        validacoesNotaOrigem.destinatario.cliente    = DESTINATARIO == 'C';
        validacoesNotaOrigem.destinatario.fornecedor = DESTINATARIO == 'F';
        validacoesNotaOrigem.destinatario.filial     = DESTINATARIO == 'L';

        validacoesNotaOrigem.complemento = cabecalhoCurrentRow.CDTIPOOPER == 3; //REGRA PODE MUDAR!

        validacoesNotaOrigem.devolucao = validacoesNotaOrigem.usaNotaOrigem &&
                                        !validacoesNotaOrigem.complemento;

        return validacoesNotaOrigem;
    };

    var processaViewFieldNotaOrigem = function(notaOrigemField, cabecalhoCurrentRow) {
        var validacoes = processaValidacoesNotaOrigem(cabecalhoCurrentRow);

        if (validacoes.usaNotaOrigem) {
            notaOrigemField.isVisible = true;
        }
        else {
            notaOrigemField.isVisible = false;
        }

        if (validacoes.notaOrigemObrigatoria)
            notaOrigemField.validations = {required: true};
        else
            delete notaOrigemField.validations;
        
        templateManager.container.getActiveWidget().reload();
    };

    var processaModelFieldNotaOrigem = function(notaOrigemField, cabecalhoCurrentRow) {
        var validacoes = processaValidacoesNotaOrigem(cabecalhoCurrentRow),
            valoresFiltro = processaValoresFiltroNotaOrigem(cabecalhoCurrentRow),
            filtros;

        if (validacoes.estorno) {
            filtros = {};
            filtros.CDFILIAL    = 'CDFILIAL';
            filtros.IDTPIJURNF  = 'IDTPIJURNF';
            filtros.NRINSJURNF  = 'NRINSJURNF';
            filtros.CDFILIRECE  = 'CDFILIRECE';
            filtros.NOTAFISC    = 'NOTAFISC';
            filtros.TNRNOTAFISC = 'TNRNOTAFISC';
            filtros.CDSERIE     = 'CDSERIE';
            filtros.TSERIE      = 'TSERIE';
            filtros.CDTIPOOPEN  = 'CDTIPOOPEN';
            filtros.CDTIPOOPSA  = 'CDTIPOOPSA';
            filtros.CDFILIEMIT  = 'CDFILIRECE'; //atenção!

            notaOrigemField.dataSource.name = 'df_entrada#/crud/doc_origem_estorno_lancamento_entrada';
        } else {
            filtros = [];
            filtros.push('CDFILIAL', 'IDTPIJURNF', 'NRINSJURNF', 'CDFILIRECE', 'RELACDOORI');

            if (validacoes.notaOrigemEntrada) {
                filtros.push('CDTIPOOPEN', 'CDTIPOOPSA', 'CDTIPOOPER');

                if (validacoes.notaOrigemSaida)
                    notaOrigemField.dataSource.name = 'df_entrada#/crud/doc_origem_entrsaid_lancamento_entrada';
                else
                    notaOrigemField.dataSource.name = 'df_entrada#/crud/doc_origem_entrada_lancamento_entrada';
            } else {
                if (validacoes.notaOrigemSaida) {
                    filtros.push('CDTIPOOPSA');

                    notaOrigemField.dataSource.name = 'df_entrada#/crud/doc_origem_saida_lancamento_entrada';
                } else {
                    filtros.push('CDTIPOOPER');

                    notaOrigemField.dataSource.name = 'df_entrada#/crud/doc_origem_todos_lancamento_entrada';
                }
            }
        }

        notaOrigemField.dataSourceFilter = processaFiltro(filtros, valoresFiltro); //array/object, object => object array
    };

    var processaFiltro = function(nomes, valores) {
        if (typeof nomes != 'object')
            throw {data: {error: 'Tipo de variável inválido para a função processaFiltro!'}};
        else if (Array.isArray(nomes))
            return nomes.map(nome => ({name: nome, operator: '=', value: valores[nome]}));
        else return Object.keys(nomes).map(key => ({name: key, operator: '=', value: valores[nomes[key]]}));
    };

    var processaValoresFiltroNotaOrigem = function(cabecalhoCurrentRow) {
        var validacoes = processaValidacoesNotaOrigem(cabecalhoCurrentRow),
            valoresFiltro = {};

        valoresFiltro.CDSERIE = '';
        valoresFiltro.TSERIE = 'T';
        // valoresFiltro.NRNOTAFISC = 'L';
        valoresFiltro.TNRNOTAFISC = 'T';

        valoresFiltro.CDTIPOOPER = cabecalhoCurrentRow.CDTIPOOPER;
        valoresFiltro.RELACDOORI = cabecalhoCurrentRow.IDDCORIGRELAC;

        valoresFiltro.CDTIPOOPEN = processaTiposOrigemPermitidosPorOperacao(cabecalhoCurrentRow, 'E');
        valoresFiltro.CDTIPOOPSA = processaTiposOrigemPermitidosPorOperacao(cabecalhoCurrentRow, 'S');

        if (validacoes.destinatario.cliente || validacoes.destinatario.fornecedor) {
            valoresFiltro.CDFILIAL = cabecalhoCurrentRow.CDFILIRECE;
            valoresFiltro.IDTPIJURNF = cabecalhoCurrentRow.IDTPIJURNF;
            valoresFiltro.NRINSJURNF = cabecalhoCurrentRow.NRINSJURNF;
            valoresFiltro.CDFILIRECE = 'T';
        } else {
            valoresFiltro.IDTPIJURNF = '';
            valoresFiltro.NRINSJURNF = '';

            if (validacoes.estorno) {
                valoresFiltro.CDFILIAL = cabecalhoCurrentRow.CDFILIRECE;
                valoresFiltro.CDFILIRECE = cabecalhoCurrentRow.CDFILIEMIT;
            } else {
                valoresFiltro.CDFILIAL = cabecalhoCurrentRow.CDFILIEMIT;
                valoresFiltro.CDFILIRECE = cabecalhoCurrentRow.CDFILIRECE;
            }
        }

        return valoresFiltro;
    };
    
    //Função para adicionar sequencialmente as informações complementares dos Produtos.
    this.nextProdutoLib = function() {
        var widget = templateManager.container.getWidget("itens");
        widget.currentRow.NRLANCTONF = widget.getParent().currentRow.NRLANCTONF;
        
        var validacoes = processaValidacoesNotaOrigem(templateManager.container.getWidget("cabecalho").currentRow);
        
        var row2 = {
            IDENTRSAIDOP:widget.getParent().currentRow.IDENTRSAIDOP,
            CDTIPOOPER:widget.getParent().currentRow.CDTIPOOPER
        };
        
        General.callRestEngine("df_entrada#/getIDATUESTTPOP", "Row", row2).then((success) => {
          atualizaEstoque = success.dataset.atualizaEstoque.IDATUESTTPOP;
           
        });
        
        if(validacoes.devolucao) 
            widget.paramInfoCompl.DocuDevo = true;
        
        if(!widget.currentRow.__is_new) //salva mudança de almoxarifado em itens já importados.
            Df11000_Lancamento_Service.salvaItem(widget.getParent().currentRow, widget.currentRow);
            
            if(widget.itemsInfCompl){  //confere se existe mais produtos na lista de produtos do datasource para ser adicional local de estoque
                for(var i=0 ; i < widget.dataSource.data.length  ; i++){ //atualiza datasourceLocal da widget para setar no localstorage no final do processo
                    if(widget.dataSource.data[i].NRSEQITPROD === widget.currentRow.NRSEQITPROD)
                        widget.dataSource.data[i] = widget.currentRow;
                        Df11000_Lancamento_Service.setDataSourceItem(widget.dataSource.data)
                        .then(
                            Df11000_Lancamento_Service.atualizaLancamento()
                    );
                }    
                    widget.itemsInfCompl.shift();
                if(widget.itemsInfCompl[0]) {
                    var row = widget.itemsInfCompl[0];
                    widget.currentRow = row;
                    setValoresEmissaoProdutoOnChange(row);
                    
                    Dfe11000_LancamentoEntrada_Service.getFornImportRastXml().then((success) => {
                            let fornImportRastXml = (success) ? success.IDIMPRASTXML : 'N';
                            
                            if(fornImportRastXml === 'S') Lib00206_InfComplProduto_Controller.infComplementares(widget, row, true, atualizaEstoque);
                            else                          Lib00206_InfComplProduto_Controller.infComplementares(widget, row, false, atualizaEstoque);
                            
                    });
                }
                else {
                    Df11000_Lancamento_Service.setDataSourceItem(widget.dataSource.data)
                    .then(function(success){
                        Df11000_Lancamento_Service.atualizaLancamento();
                    });
                }
            }
            
    };

    var processaTiposOrigemPermitidosPorOperacao = function(cabecalhoCurrentRow, operacao) {
        var validacoes = processaValidacoesNotaOrigem(cabecalhoCurrentRow);

        if (validacoes.limitaNotaOrigem)
            return cabecalhoCurrentRow.DSOPERNFORIG.split('_')
                .filter(piece => piece.length == 4 && piece[0] == operacao)
                .reduce((prevValue, currValue) => prevValue += currValue + '_', '_');
        else
            return '_';
    };

    var alteraVisibilidadeCampoInscricao = function() {
        var cabecalhoWidget = getCabecalhoWidget();
        var cabecalhoCurrentRow = cabecalhoWidget.currentRow;
        var cliente = cabecalhoWidget.getField('NMRAZSOCCLIE');
        var fornecedor = cabecalhoWidget.getField('NMRAZSOCFORN');
        var filial = cabecalhoWidget.getField('DSFILIEMIT');
        var tipoDestino = cabecalhoCurrentRow.IDORIGDESTOP || cabecalhoCurrentRow.IDORIGDESTNF;

        switch (tipoDestino) {
            case 'C':
                validacaoCamposOutTpOperac(cliente, fornecedor, filial);
                break;
            case 'F':
                validacaoCamposOutTpOperac(fornecedor, cliente, filial);
                break;
            case 'L':
                validacaoCamposOutTpOperac(filial, fornecedor, cliente);
                break;
            default:
                filial.readOnly = true;
                cliente.readOnly = true;
                fornecedor.readOnly = true;
        }
    };

    var validaChaveAcessoObrigatoria = function(cabecalhoWidget) {
        var cabecalhoRow = cabecalhoWidget.currentRow,
            chaveAcessoField = cabecalhoWidget.getField('NRACESSONFE'),
            chaveAcessoObrigatoria = cabecalhoRow.IDMODDOCNFE === 'S' && cabecalhoRow.IDORIGDESTNF === 'F' && (cabecalhoRow.IDOBRCHAVACES === 'S') && cabecalhoRow.IDTIPOOPERELE === 'S';
        
        if(chaveAcessoObrigatoria == false && cabecalhoRow.CDFORNECED) {
            var filter = [
                {
                    "name": "CDFORNECED",
                    "operator": "=",
                    "value": cabecalhoRow.CDFORNECED
                }
            ];
            General.callRestEngine("df#/crud/paramforne", "FilterData", filter).then((sucesso) => {
                if(sucesso.dataset.paramforne[0]) {
                    var paramforne = sucesso.dataset.paramforne[0];
                    chaveAcessoObrigatoria = cabecalhoRow.IDMODDOCNFE === 'S' && cabecalhoRow.IDORIGDESTNF === 'F' && (paramforne.IDOBRCHAVACESFO === 'S') && cabecalhoRow.IDTIPOOPERELE === 'S';
                    
                    if (chaveAcessoObrigatoria)
                        chaveAcessoField.validations = {required: true};
                    else
                        delete chaveAcessoField.validations; 
                    
                    Dfe11000_LancamentoEntrada_Service.setFornImportRastXml(paramforne.IDIMPRASTXML);
                } 
            });
        }
        else {
            if (chaveAcessoObrigatoria)
                chaveAcessoField.validations = {required: true};
            else
                delete chaveAcessoField.validations; 
        }
        
    };

    var validaCampoAF = function(cabecalhoWidget, importingXML) {
        var cabecalhoRow = cabecalhoWidget.currentRow;
        var cbutilizaafField = cabecalhoWidget.getField('CBUTILIZAAF');

        cbutilizaafField.readOnly = (!cabecalhoRow.CDTIPOOPER || !cabecalhoRow.CDFORNECED);

        Dfe11000_LancamentoEntrada_Service.validaAF(cabecalhoRow).then(function(response){
            var obrigaAF = response.dataset.validaAF[0];
            if(obrigaAF)
                cbutilizaafField.validations = {required: true};
            else{
                if (isAFRequired(cabecalhoRow, importingXML))
                    cbutilizaafField.validations = {required: true};
                else{
                    cbutilizaafField.readOnly = false;
                    delete cbutilizaafField.validations;
            
                }    
            }
        });
        
        
    };
    
    
    var isAFRequired = function(cabecalhoRow, importingXML){
        if(isNFCompra(cabecalhoRow))
            return AFRequiredForSupplier(cabecalhoRow) || cabecalhoRow.IDBLOQAF == 'S';
        else{
            General.callRestEngine('df#/crud/paramgeral', 'FilterData', []).then(function(success) {
                if(success.dataset.paramgeral[0]){
                    var paramgeral = success.dataset.paramgeral[0];
                    if(paramgeral.IDUTBLOQOPAFPG  == 'S')
                        return Dfe11000_LancamentoEntrada_Service.isAFTipoOper(cabecalhoRow.CDTIPOOPER, cabecalhoRow.CDFILIRECE).then(result => {
                            if(result == true)
                                return importOnlyAFProducts(cabecalhoRow) && importingXML;
                            else
                                return false;
                        });
                    else
                        return false;
                }
                return false;
            });
            
        }

    };
    
    this.venctosSemDataSourceFilter = (widget) => {
       if(widget.dataSourceFilter.length) {
           if(widget.dataSourceFilter[0].isCustomFilter) {
               widget.dataSourceFilter = [];
               widget.dataSource.reload();
           }
       }
   };
    
    //const isAFRequired = (cabecalhoRow, importingXML) =>  isNFCompra(cabecalhoRow)? cabecalhoRow.IDBLOQAF == 'S' || AFRequiredForSupplier(cabecalhoRow) : importingXML && importOnlyAFProducts(cabecalhoRow);

    const isNFCompra = ({CDTIPOOPER, IDORIGDESTNF, IDNFPRESTSERV = 'N', IDNFESTCANC = 'N', IDNFRETIND = 'N'}) => (CDTIPOOPER == '01');

    const AFRequiredForSupplier = ({CDTIPOOPER, IDBLOQCMPSAFFIL = 'N', IDBLOQCMPSAFFOR = 'N'}) => !!(IDBLOQCMPSAFFIL === 'S' && IDBLOQCMPSAFFOR === 'S' && CDTIPOOPER === '01');

    const importOnlyAFProducts = row => row.IDPERIMPSPRAF === 'S';
    
    var isAFTipoOper = function(cabecalhoRow){
            return Dfe11000_LancamentoEntrada_Service.isAFTipoOper(cabecalhoRow.CDTIPOOPER, cabecalhoRow.CDFILIAL);
    };
    
    /**
         * @desc - Adiciona valores obrigatorios na current row da nota.
        * @emit - Chamada na funcao responsável pelo salvamento da nota fiscal(salvaCabeçalhoLancamentoOnClick)
        * @param object cabecalhoRow
        * @param object paramsNewCode
    */
    var addValoresObrigatoriosCurrentRowCabebalho = function(cabecalhoRow, paramsNewCode) {
        cabecalhoRow.IDBLOQUEADO  = 'N';
        cabecalhoRow.IDDOCFISCAL  = 'S';
        cabecalhoRow.IDDOCFINANC  = 'S';

        cabecalhoRow.VRPRODUTOS   = 0;

        cabecalhoRow.IDENTRSAIDOP = 'E';
        cabecalhoRow.IDSISTEMA    = 'DFE';
        cabecalhoRow.IDSTATUS     = 'D';
        
        if(cabecalhoRow.IDNFGERADAVIAXML !== 'S')
            cabecalhoRow.IDNFGERADAVIAXML = (!!cabecalhoRow.NRACESSONFE && !cabecalhoRow.MANUAL_POSTING) ? 'S' : 'N';

        cabecalhoRow.CDTIPOOPEN = processaTiposOrigemPermitidosPorOperacao(cabecalhoRow, 'E');
        cabecalhoRow.CDTIPOOPSA = processaTiposOrigemPermitidosPorOperacao(cabecalhoRow, 'S');

        if (paramsNewCode && typeof paramsNewCode === 'object') {
            var keysParamsNewCode = Object.keys(paramsNewCode);
            keysParamsNewCode.forEach(value => {
                cabecalhoRow[value] = paramsNewCode[value];
            });
        }
    };

    var realizaSalvamentoCabecalhoNotaFiscal = function(cabecalhoWidget, datasourceProduto = null) {
        var paramsNewCode = cabecalhoWidget.paramsNewCode,
            cabecalhoRow = cabecalhoWidget.currentRow;

        var fornecedBloqTipoOperac = ('ERROS_LANCAMENTO' in cabecalhoRow && !!cabecalhoRow.ERROS_LANCAMENTO.fornecedorBloqTipoOperac);

        if (fornecedBloqTipoOperac) {
            return showMessageDfe('TIPO_OPERACAO_BLOQ_FORNECEDOR');
        }
        
        addValoresObrigatoriosCurrentRowCabebalho(cabecalhoRow, paramsNewCode);
        
        validacaoRelacionamentoItens(cabecalhoWidget).then(() => {

            if(cabecalhoWidget.currentRow.__is_new == false) {
                General.callRestEngine("df_entrada#/af_indice", "Row", cabecalhoRow).then((response) => {
                    if(response.dataset.af_indice.utilizaIndice) {
                        ScreenService.confirmMessage('O valor do índice financeiro pode ser diferente para a nova data de emissão. Deseja atualizar os valores dos produtos importados de uma AF com o novo índice?', 'question',
                            (success) =>  {
                                cabecalhoRow.ATUALIZAINDICE = true;
                                this.salvaCabecalho(cabecalhoWidget, cabecalhoRow);
                            },
                            (error) => {
                                cabecalhoRow.ATUALIZAINDICE = false;
                                this.salvaCabecalho(cabecalhoWidget, cabecalhoRow);
                            }
                        ); 
                    }
                    else {
                        cabecalhoRow.ATUALIZAINDICE = false;
                        this.salvaCabecalho(cabecalhoWidget, cabecalhoRow);
                    }
                });
            }
            else {
                cabecalhoRow.ATUALIZAINDICE = false;
                this.salvaCabecalho(cabecalhoWidget, cabecalhoRow, datasourceProduto);
            }
            
             this.habilitaTabProdutosOuNotas(cabecalhoWidget);
            
        });
        
    }.bind(this);
    
    this.salvaCabecalho = (cabecalhoWidget, cabecalhoRow, datasourceProduto = null) => {
        const novaNota = cabecalhoRow.__is_new;
        const usaAf    = (cabecalhoRow.AF) ? true : false;
        General.callRestEngine("df_entrada#/salvarNotaEntrada", "Row", cabecalhoRow).then((cabecalho) => {
            var columnsToReturn = cabecalho.dataset.columnsToReturn[0];
            cabecalhoRow.NRLANCTONF = columnsToReturn.NRLANCTONF;
            cabecalhoRow.CDFILIAL = columnsToReturn.CDFILIAL;

            cabecalhoWidget.container.getWidget('relacionamentoItens').isVisible = false;

            controlaStatusWidgets(cabecalhoWidget, cabecalhoWidget.name, true);
            

            criaDetalhesLancamentoWidgetAfterMoveRow(cabecalhoRow)
            .then(() => {
                if(novaNota) {
                    if(cabecalhoRow.NRACESSONFE || usaAf) {
                        cabecalhoWidget.paramInfoCompl.showLote = true;
                        this.persisteLocalEstoque(cabecalhoWidget, cabecalhoRow, datasourceProduto);
                    }
                }
                else {
                    cabecalhoWidget.activate();
                }
            });
            
            if(cabecalhoWidget.isValid() && cabecalhoWidget.currentRow.DSLINKANEXO != null){
                General.callRestEngine('df_entrada#/salvar_Anexo', "Row", cabecalhoRow);
            }
            
            if (!!cabecalhoRow.__is_new && (!!cabecalhoRow.NRACESSONFE || 'AF' in cabecalhoRow)) {
                delete cabecalhoRow.AF;
            }
            cabecalhoWidget.container.getWidget('impostosItem').reload();
            cabecalhoWidget.currentWidget = cabecalhoWidget;
            cabecalhoWidget.view();
            validaEdicaoNota();
            cabecalhoRow.__is_new = false; 
        }).catch((error) => {
            MessageService.showMessageByCode(error.data.error).catch (response => {
                ScreenService.showMessage(error.data.error); 
            });
            //ScreenService.showMessage(error.data.error);
        });
    };
    
    //Ao importar XML confere se utiliza local de estoque e almoxarifado para chamar Lib de dados complementares do produto
    this.persisteLocalEstoque = (cabecalhoWidget, cabecalhoRow, datasourceProduto = null) => {
        var widgetProduto = cabecalhoWidget.container.getWidget('itens');
        var validacoes = processaValidacoesNotaOrigem(cabecalhoRow);
        
        if(validacoes.devolucao) 
            widgetProduto.paramInfoCompl.DocuDevo = true;
            
        var row = {
            IDENTRSAIDOP:widgetProduto.getParent().currentRow.IDENTRSAIDOP,
            CDTIPOOPER:widgetProduto.getParent().currentRow.CDTIPOOPER
        };
        
        General.callRestEngine("df_entrada#/getIDATUESTTPOP", "Row", row).then((success) => {
          atualizaEstoque = success.dataset.atualizaEstoque.IDATUESTTPOP;
           
        });

        
        General.callRestEngine("df#/crud/paramfilial", "FilterData", [{name: "CDFILIAL", operator: "=", value: cabecalhoRow.CDFILIRECE}]).then((success) => {
            if(success.dataset.paramfilial[0].IDUTILALMOX == 'S' && success.dataset.paramfilial[0].IDCTRLESTQ == 'S') {
    
                widgetProduto.reload();
                var data = cabecalhoWidget.container.getWidget('itens').dataSource.data;
                // data[0]["NMPRODUTO"] = datasourceProduto.NMPRODUTOFORNXML;
                // data[0]["SGUNIDADE"] = datasourceProduto.SGUNIDADE;
                // data[0]["DSPRODEMIS"] = datasourceProduto.NMPRODUTOFORNXML;
                // data[0]["CDPRODEMIS"] = datasourceProduto.CDPRODUTOXML;
                // data[0]["DSPRODEMIS"] = datasourceProduto.NMPRODUTOFORNXML;
                cabecalhoWidget.openTab('itens');
                widgetProduto.activate();
                data.forEach(function(row){
                    row.CDFILIMOVI = cabecalhoRow.CDFILIRECE;
                });
                widgetProduto.itemsInfCompl = data; 
                widgetProduto.currentRow = data[0];
                var row = data[0];
                if(row){
                    Dfe11000_LancamentoEntrada_Service.getFornImportRastXml().then((success) => {
                        let fornImportRastXml = (success) ? success.IDIMPRASTXML : 'N';
                        
                        if(fornImportRastXml === 'S') Lib00206_InfComplProduto_Controller.infComplementares(widgetProduto, row, true, atualizaEstoque);
                        else                          Lib00206_InfComplProduto_Controller.infComplementares(widgetProduto, row, false, atualizaEstoque);
                        
                    });
                }
                else
                    Df11000_Lancamento_Service.setDataSourceItem(widgetProduto.dataSource.data)
                    .then(Df11000_Lancamento_Service.atualizaLancamento());
            } 
            else {
                cabecalhoWidget.openTab('itens');

                widgetProduto.reload();
                widgetProduto.activate();
            }
        });
    };
    
    this.chamaDadosAdicionais = (args) => {
        var widgetProduto =  templateManager.container.getWidget("itens");
        var row = args.row;
        var widgetCabecalho = widgetProduto.getParent();
        var validacoes = processaValidacoesNotaOrigem(widgetCabecalho.currentRow);
        row.CDFILIMOVI = widgetCabecalho.currentRow.CDFILIRECE;
        
        var rows = {
            IDENTRSAIDOP:widgetProduto.getParent().currentRow.IDENTRSAIDOP,
            CDTIPOOPER:widgetProduto.getParent().currentRow.CDTIPOOPER
        };
        
        General.callRestEngine("df_entrada#/getIDATUESTTPOP", "Row", rows).then((success) => {
          atualizaEstoque = success.dataset.atualizaEstoque.IDATUESTTPOP;
           
        });
        
        if(validacoes.devolucao) 
            widgetProduto.paramInfoCompl.DocuDevo = true;
        
        Dfe11000_LancamentoEntrada_Service.getFornImportRastXml().then((success) => {
            let fornImportRastXml = (success) ? success.IDIMPRASTXML : 'N';
            
            if(fornImportRastXml === 'S') Lib00206_InfComplProduto_Controller.infComplementares(widgetProduto, row, true, atualizaEstoque);
            else                          Lib00206_InfComplProduto_Controller.infComplementares(widgetProduto, row, false, atualizaEstoque);
            
        });
    };

    this.persisteImpostos = (cabecalhoWidget, cabecalhoRow) => {
        if (!cabecalhoRow.__is_new) {
            return calculaImpostoItemProduto(cabecalhoRow, cabecalhoWidget.container.getWidget('itens').dataSource.data[0]);
        }
    };

    this.realizaImportacaoItensNotaOrigem = function(cabecalhoWidget, cabecalhoRow) {
        cabecalhoRow = Util.clone(cabecalhoRow, ["__rightActions", "__leftActions"]);
        var deferred = ZHPromise.defer(),
            itensWidget = cabecalhoWidget.container.getWidget('itens'),
            validacoes = processaValidacoesNotaOrigem(cabecalhoRow);

        if (validacoes.usaNotaOrigem) {
            //to-do: implementar importação de notas de complemento e estorno
            if (validacoes.devolucao) {
                Dfe11000_LancamentoEntrada_Service.realizaImportacaoItensNotaOrigem(cabecalhoRow)
                    .then(itens => {
                        itens = itens.dataset.itens_notas_origem_entrada;
                        /*var isInvalid = itens.some(function(item) {
                            return !item.QTITEMPROD || !item.VRUNITITEM || !item.VRTOTAITEM;
                        });

                        if (isInvalid) {
                            return ScreenService.showMessage('Campos obrigatórios não preenchidos.');
                        }*/

                        itens.forEach(item => {
                            item.__is_new = true;
                            delete(item.__rightActions); 
                            delete(item.__leftActions);
                        });

                        itensWidget.dataSource.data = itens;

                        return Dfe11000_LancamentoEntrada_Service.salvaMultiItens(itens, [], cabecalhoRow);
                    })
                    .then(impostosProdutos => {
                        var itens = itensWidget.dataSource.data,
                            impostos = impostosProdutos.dataset.impostosTemp;

                        impostos.forEach(imposto => {
                            imposto = Object.keys(imposto).map(key => imposto[key]);
                            imposto.forEach(impos => {
                                impos.CDIMPOSTO = impos.CDIMPOTEMP;
                                impos.__is_new = false;

                                delete impos.CDIMPOTEMP;
                            });
                            Df11000_Lancamento_Service.setDataSourceItensImpostos(imposto);
                        });

                        itens.forEach(item => {
                            item.__is_new = false;
                            delete item.changed;
                        });

                        Df11000_Lancamento_Service.setDataSourceItem(itens).then(() => {
                            Df11000_Lancamento_Service.atualizaLancamento().then(itensWidget.reload());
                        });

                        alteraStatusNotafiscal(cabecalhoWidget).then(() => controlaStatusWidgets(cabecalhoWidget, cabecalhoWidget.name, true));

                        itensWidget.list();
                        cabecalhoWidget.view();
                    }).catch(requestError);
            } else
                deferred.resolve();
        } else
            deferred.resolve();

        return deferred;
    };

    /**
         * @desc - Função responsável pelo salvamento da notafiscal, importação de af e chave de acesso
        * @emit - No evento de onClick da action salvar
        * @param object argsCabecalho
    */
    this.salvaCabecalhoLancamentoOnClick = function(argsCabecalho) {
        validaDataPeriodoContabil().then( response => {
            if(response){
                var cabecalhoWidget = argsCabecalho.owner.widget;
            
                Dfe11000_LancamentoEntrada_Service.getFilial().then(function(result){
                    if(!cabecalhoWidget.currentRow.CDFILIAL)
                        cabecalhoWidget.currentRow.CDFILIAL = result.dataset.filial[0].CDFILIAL;
                
                    if (!cabecalhoWidget.isValid()) {
                        return argsCabecalho.e.preventDefault();
                    }
                    
                    alteraStatusNotafiscal(cabecalhoWidget);
                    return realizaSalvamentoCabecalhoNotaFiscal(cabecalhoWidget);
                });
            } else {
                return MessageService.showMessageByCode('DT_ENTRADA_FORA_PERIODO_CONTABIL');
            }
        });
    };
    
    var validaDataPeriodoContabil = function(){
        var promise = ZHPromise.defer();
        var widget = templateManager.container.getActiveWidget();
        var dtentrsaid = widget.currentRow.DTENTRSAID,
            empresa = widget.currentRow.CDEMPRESA,
            nmempresa = widget.currentRow.NMEMPRESA;
        
        if(dtentrsaid && empresa) {
            let filter = [
                {
                    "name": "CDEMPRESA",
                    "operator": "=",
                    "value": empresa
                }
            ];
                
            General.callRestEngine("df#/crud/contopectbl", "FilterData", filter).then(function(success) {
                if(success.dataset.contopectbl.length) {
                    if(success.dataset.contopectbl[0].DTINIINTCONTDFE && success.dataset.contopectbl[0].DTFININTCONTDFE) {
                        var dataEntrada = formataData(dtentrsaid);
                        var dataIni = formataData(success.dataset.contopectbl[0].DTINIINTCONTDFE.substr(0, 10));
                        var dataFin = formataData(success.dataset.contopectbl[0].DTFININTCONTDFE.substr(0, 10));
            
                        var periodoContabilValido;
                
                        if(dataEntrada >= dataIni && dataEntrada <= dataFin) {
                            periodoContabilValido = true;
                            promise.resolve(periodoContabilValido);
                        } else {
                            periodoContabilValido = false;
                            promise.resolve(periodoContabilValido);
                        }
                    }
                } else {
                    MessageService.showMessageByCode('PARAMETRIZACAO_CONTABIL_NAO_REALIZADA', {"CDEMPRESA": empresa, "NMEMPRESA": nmempresa});
                }
            }).catch( error => {
                MessageService.errorNotificationByCode(ScreenService.showMessage(error));
            });
            
            return promise;
        }
    };

    
    var formataData = function(dataString){
        var arrData = dataString.split('/');
        var dataFormatada = arrData[1] + '-' + arrData[0] + '-' + arrData[2];
        var dataFinal = new Date(dataFormatada);
        
        return dataFinal;
    };

    this.validaFornecedorOnChange = function(cabecalhoArgs) {
        var cabecalhoRow = cabecalhoArgs.row;
        var cabecalhoWidget = cabecalhoArgs.owner.field.widget;
        var autoForneWidget = cabecalhoWidget.container.getWidget('autorizacaoFornecimento');
        limpaWidgetAutoForne(autoForneWidget, cabecalhoWidget);
        validaFornecedorBloqTpOperac(cabecalhoWidget);
        validaCampoAF(cabecalhoWidget);
    };

    var criaParametrosImportacaoAf = function(cabecalhoWidget, importacaoAfWidget) {
        var cabecalhoRow = cabecalhoWidget.currentRow;
        var filial = cabecalhoRow.CDFILIRECE;
        var fornecedor = cabecalhoRow.CDFORNECED;
        var dtminvazia = cabecalhoRow.DTMINVAZIA;
        var dtmaxvazia = cabecalhoRow.DTMAXVAZIA;
        var dtprocini = cabecalhoRow.DTPROCINI;
        var dtprocfin = cabecalhoRow.DTPROCFIN;

        var dataSourceFilterImportacaoAF = [
            {
                name: 'P_CDFILIAL',
                operator: '=',
                value: filial
            },
            {
                name: 'P_CDFORNECED',
                operator: '=',
                value: fornecedor
            },
            {
                name: 'P_DTPROCINI',
                operator: '=',
                value: dtprocini
            },
            {
                name: 'P_DTMINVAZIA',
                operator: '=',
                value: dtminvazia
            },
            {
                name: 'P_DTPROCFIN',
                operator: '=',
                value: dtprocfin
            },
            {
                name: 'P_DTMAXVAZIA',
                operator: '=',
                value: dtmaxvazia
            }
        ];

        importacaoAfWidget.dataSourceFilter = dataSourceFilterImportacaoAF;
        importacaoAfWidget.reload();
    };

    this.abreImportacaoAF = function(field) {
        var cabecalhoWidget = field.widget;
        delete cabecalhoWidget.currentRow.AF;

        var importaAf = field.value();

        if (importaAf === 'S') {
            var autoFornWidget = cabecalhoWidget.container.getWidget('autorizacaoFornecimento');
            autoFornWidget.selectField = field;

            ScreenService.openPopup(autoFornWidget);
            criaParametrosImportacaoAf(cabecalhoWidget, autoFornWidget);
        }
    };

    this.checarAFs = AFWidget => (!AFWidget.dataSource.data.length && (AFWidget.selectField.validations && AFWidget.selectField.validations.required))? showMessageDfe('AF_OBRIGATORIA_E_NENHUMA_ENCONTRADA') : false;

    this.actConfirmarAfsSelecionadas = function(autoForneArgs) {
        var autoForneWidget = autoForneArgs.owner.widget;
        var afsSelecionadas = autoForneWidget.getCheckedRows();
        var cabecalhoWidget = autoForneArgs.owner.widget.container.getWidget('cabecalho');
        var cabecalhoRow = cabecalhoWidget.currentRow;

        if (afsSelecionadas.length) {
            cabecalhoRow.AF = afsSelecionadas;

            var geraNfIgualXml = cabecalhoRow.IDGERNFIGUALXML === 'N';
            if ((!('NRACESSONFE' in cabecalhoRow) || !cabecalhoRow.NRACESSONFE)) {
                ScreenService.showLoader();
                
                var cdfililancs = afsSelecionadas.map(af => af.CDFILLANC).join('_');
                var nrautofrons = afsSelecionadas.map(af => af.NRAUTOFORN).join('_');

                var filter = [
                    {
                        "name": "CDFILLANC",
                        "operator": "=",
                        "value": cdfililancs
                    },
                    {
                        "name": "NRAUTOFORN",
                        "operator": "=",
                        "value": nrautofrons
                    },
                    {
                        "name": "CDFILDEST",
                        "operator": "=",
                        "value": afsSelecionadas[0].CDFILDEST
                    },
                    {
                        "name": "NRORG",
                        "operator": "=",
                        "value": getLocalVar('NRORG')
                    }
                ];
                

               var afEstrangeira = afsSelecionadas.filter(af =>  af.CDINDICE != null);
                
                if(afEstrangeira.length === 0){
                    General.callRestEngine('df_entrada#/crud/totalNf_af', 'FilterData', filter).then(result => {
                        if(result.dataset.totalNf_af && result.dataset.totalNf_af.length > 0 && result.dataset.totalNf_af[0].VRNOTAFISC){
                             var valorNota = result.dataset.totalNf_af[0].VRNOTAFISC;
                                cabecalhoWidget.currentRow.VRNOTAFISC = valorNota;
                                ScreenService.closePopup();
                        } else {
                            ScreenService.showMessage('Erro ao buscar o valor total da nota.');
                        }
                        ScreenService.hideLoader();
                    }, this.genericalRequestError);
                }else{
                    cabecalhoWidget.currentRow.VRNOTAFISC = null;
                    ScreenService.closePopup();
                    ScreenService.hideLoader();
                }
            } else {
                ScreenService.closePopup();
            }
        } else {
            limpaWidgetAutoForne(autoForneWidget, cabecalhoWidget);
            ScreenService.closePopup();
        }
    };

    this.genericalRequestError = (error) => {
        if (error.data.error) {
            ScreenService.showMessage(error.data.error);
        } else {
            ScreenService.showMessage(error);
        }
        ScreenService.hideLoader();
    };

    this.actCancelarAfsSelecionadas = function(autoForneArgs) {
        var cabecalhoWidget = autoForneArgs.owner.widget.container.getWidget('cabecalho');
        var autoFornWidget = autoForneArgs.owner.widget;
        limpaWidgetAutoForne(autoFornWidget, cabecalhoWidget);
        ScreenService.closePopup();
    };

    var applyZeroFillNumeroNota = function(cabecalhoWidget) {
        var filter =  [{"name": "CDFILIAL", "operator": "=", "value": cabecalhoWidget.currentRow.CDFILIRECE}],
            nrNotaFiscField = cabecalhoWidget.getField('NRNOTAFISC'),
            nrTamNumNfUt,
            quantCaracteres;
        
        General.callRestEngine("df#/crud/paramfilial", "FilterData", filter).then((success) => {
            if(success.dataset.paramfilial[0]) 
                nrTamNumNfUt = success.dataset.paramfilial[0].NRTAMNUMNFUT;
            
            quantCaracteres = nrTamNumNfUt ? nrTamNumNfUt : 9;
            nrNotaFiscField.mask.params.maxlength = quantCaracteres;
            nrNotaFiscField.validations.maxLength.length = nrTamNumNfUt ? nrTamNumNfUt : 20;

            nrNotaFiscField.reload();
        });
    };

    var limpaOutDataChaveAcess = function(currentRow, outDataForneced = false) {
        currentRow.NRNOTAFISC  = null;
        currentRow.VRNOTAFISC  = 0;
        currentRow.NRACESSONFE = null;
        currentRow.CDSERIEENTR = null;
        currentRow.DSSERIEENTR = null;
        currentRow.DTEMISSAO   = null;

        currentRow.CDMODDOCFISC = null;
        currentRow.DSMODDOCFISC = null;
        currentRow.IDMODDOCNFE  = null;
        currentRow.IDOBRCHAVACES = 'N';
        
        if (outDataForneced)
            clearRowWithOutData(currentRow, outDataForneced);
    };

    const applyAccessKeyAccessibilityRules = (cabecalhoWidget, block) => {
        const dependentFields = [
                'NRNOTAFISC',  'CDSERIEENTR', 'NMRAZSOCCLIE',
                'NMRAZSOCFORN', 'DSFILIEMIT', 'DTEMISSAO', 'DSMODDOCFISC'
            ], applyRule = name => cabecalhoWidget.getField(name).readOnly = block;
        
        
        
        dependentFields.forEach(applyRule);
        
        if(cabecalhoWidget.currentRow.IDGERNFIGUALXML == 'S') {
            cabecalhoWidget.getField('VRNOTAFISC').readOnly = block;
        } 
        else {
            cabecalhoWidget.getField('VRNOTAFISC').readOnly = false;
        }
    };

    const applyInvalidAccessKeyRules = headerWidget => {
        const getWidget = headerWidget.container.getWidget,
            {currentRow} = headerWidget;

        if(currentRow.MANUAL_POSTING) delete currentRow.MANUAL_POSTING;

        limpaOutDataChaveAcess(currentRow, headerWidget.getField('NMRAZSOCFORN').outData);
        applyAccessKeyAccessibilityRules(headerWidget, false);
        limpaWidgetAutoForne(getWidget('autorizacaoFornecimento'), headerWidget);

        getWidget('relacionamentoItens').isVisible = false;
        return null;
    };
    
    const doNothing = (widget, vXML) =>{
        let a = widget;
        let b = vXML;
    };

    const processXMLData = (widget, XML) => {
        //Verificar se cdfilial foi digitada
        Dfe11000_LancamentoEntrada_Service.validaInscEstadual(widget.currentRow).then( () => {
            if (!XML.UTILIZADA) {
                const {currentRow} = widget;
        
                applyAccessKeyAccessibilityRules(widget, true);
                widget.container.getWidget('relacionamentoItens').isVisible = true;
        
                const neededData = [
                        'VRNOTAFISC', 'CDSERIEENTR', 'CDSERIEENTR', 'NRNOTAFISC',
                        'DTEMISSAO', 'CDMODDOCFISC', 'DSMODDOCFISC', 'IDMODDOCNFE',
                        'NRPROTOCOLONFE', 'DTHRPROTOCOLNFE', 'CDCLIENTE'
                    ], copyData = name => currentRow[name] = XML[name];
        
                neededData.forEach(copyData);
                
                if(XML.NRINSJURTRANXML) {
                    var transportadora = [
                        {
                            "name": "P_NRINSJURTRAN",
                            "operator": "=",
                            "value": XML.NRINSJURTRANXML
                        }  
                    ];
                    General.callRestEngine("df#/crud/transport_municipio", "FilterData", transportadora).then((success) => {
                       if(success.dataset.transport_municipio[0]) {
                            currentRow.NRINSJURTRAN    = XML.NRINSJURTRANXML;
                            currentRow.NMRAZSOCTRAN    = success.dataset.transport_municipio[0].NMRAZSOCTRAN;
                            currentRow.NRINSJURTRAN    = success.dataset.transport_municipio[0].NRINSJURTRAN;
                            currentRow.DSENDETRAN      = success.dataset.transport_municipio[0].DSENDETRAN;
                            currentRow.NMMUNICIPIOTRAN = success.dataset.transport_municipio[0].NMMUNICIPIO;
                            currentRow.SGESTADOTRAN    = success.dataset.transport_municipio[0].SGESTADO;
                            currentRow.NRINSESTTRAN    = success.dataset.transport_municipio[0].NRINSESTTRAN;
                            currentRow.IDTPIJURTRAN    = success.dataset.transport_municipio[0].IDTPIJURTRAN;
                       } 
                    });
                }
                
                currentRow.CDIBGEDEST    = XML.CDIBGEDEST;
                currentRow.CDIBGEORIG    = XML.CDIBGEORIG;
                currentRow.CDTIPOCTE     = XML.CDTIPOCTE;
                currentRow.NRPLACAVEIC   = XML.NRPLACAVEICXML;
                currentRow.SGUFVEIC      = XML.SGUFVEICXML;
                currentRow.IDPGTOFRETE   = XML.IDPGTOFRETEXML;
                currentRow.DSQTDEVOLU    = XML.DSQTDEVOLUXML;
                currentRow.DSESPECVOLU   = XML.DSESPECVOLUXML;
                currentRow.DSMARCAVOLU   = XML.DSMARCAVOLUXML;
                currentRow.DSNUMEVOLU    = XML.DSNUMEVOLUXML;
                currentRow.VRPBRUTOVOLU  = XML.VRPBRUTOVOLUXML;
                currentRow.VRPLIQUVOLU   = XML.VRPLIQUVOLUXML;
                currentRow.CDCLIENTE     = XML.CDCLIENTE || '';
                currentRow.DTEMISSAO     = XML.DTEMISSAO;
                currentRow.IDOBRCHAVACES = XML.IDOBRCHAVACESFO;
                
                currentRow.NRNOTAFISC = currentRow.NRNOTAFISC.padStart(currentRow.NRTAMNUMNFUTDFE, '0');
                currentRow.NRACESSONFE = XML.NRACESSONFE;
        
                setCurrentRowWithOutData(currentRow, XML, widget.getField('NMRAZSOCFORN').outData);
        
                if (isAFRequired(currentRow, true) && importOnlyAFProducts(currentRow)) {
                    if(!currentRow.AF) {
                        showAFMessageAndOpenPopup(widget);
                    }
                }
        
                validaCampoAF(widget, true);
                validaChaveAcessoObrigatoria(widget);
                this.modeloDeDocumentoOnChange('', widget);
            } else if(XML.IDSTATUS == 'L'){
                widget.view();
                General.changeWidgetAcessibility(widget.widgets, true);
                
                let filter = Util.clone(widget.dataSourceFilter);
                filter = filter.filter( filtro => (filtro.name == "CDFILIRECE" || filtro.name == "CDEMPRESA"));
                filter.push(
                    {
                        "name": "NRACESSONFE",
                        "operator": "=",
                        "value": XML.NRACESSONFE
                    }
                );
                General.callRestEngine("df_entrada#/crud/notafiscal_entrada", "FilterData", filter).then(success => {
                    if(success.dataset.notafiscal_entrada[0]){
                        widget.currentRow = success.dataset.notafiscal_entrada[0];
                        setCurrentRowWithOutData(widget.currentRow, XML, widget.getField('NMRAZSOCFORN').outData);
                        this.validacoesNotaAfterMoveRow(undefined, widget);
                    }
                });
                this.modeloDeDocumentoOnChange('', widget);
            } else{
                widget.currentRow.NRACESSONFE = '';
                return showMessageDfe('CHAVE_ACESSO_UTILIZADA');
            }
        }).catch( error => {
            widget.currentRow.NRACESSONFE = '';
            requestError(error);
        });
    };
    
    const applyManualPostingWithAccessKeyRules = widget => {
        widget.currentRow.MANUAL_POSTING = true;
        limpaOutDataChaveAcess(widget.currentRow, widget.getField('NMRAZSOCFORN').outData);
        applyAccessKeyAccessibilityRules(widget, false);
        //limpaWidgetAutoForne(widget.container.getWidget('autorizacaoFornecimento'), widget);
    };

    const suggestManualIncomingPosting = widget => MessageService.confirmMessageByCode(
        'PERMITE_LANCAMENTO_MANUAL', null,
        {
            type: MESSAGES.PERMITE_LANCAMENTO_MANUAL.type,
            success: applyManualPostingWithAccessKeyRules.bind(this, widget),
            fail: applyInvalidAccessKeyRules.bind(this, widget)
        }
    );

    const invalidAccessKey = widget => applyInvalidAccessKeyRules(widget) || showMessageDfe('CHAVE_ACESSO_INVALIDA');

    const XMLNotFound = widget =>  {
        var filter = [
            {
                "name": "CDFORNECED",
                "operator": "=",
                "value": 'J' + widget.currentRow.NRACESSONFE.substr(6, 14)
            }  
        ];
        General.callRestEngine("df#/crud/paramforne_fornecedor", "FilterData", filter).then((fornecedor) => {
            if(fornecedor.dataset.paramforne_fornecedor[0]) {
                if(fornecedor.dataset.paramforne_fornecedor[0].IDOBRIMPXMLNF != 'S') {
                    suggestManualIncomingPosting(widget);
                }
                else {
                    invalidAccessKey(widget);
                }
            }
            else {
                invalidAccessKey(widget);
            }
        });
    };
    
    const fetchIncomingByAccessKey = (widget, ACCESS_KEY) => Dfe11000_LancamentoEntrada_Service.buscaChaveAcesso(ACCESS_KEY)
        .then(response => response.dataset.intg_notafiscal_lancamento)
        .then(([XML = false]) => (XML? processXMLData : XMLNotFound)(widget, XML));
        
    const fetchAccessKeyByOtherFields = (widget, params) => Dfe11000_LancamentoEntrada_Service.buscaChaveAcessoPorCampos(params)
        .then(response => response.dataset.intg_notafiscal_lancamento_campos)

        .then(([XML = false]) => {
            if(XML) {
                widget.currentRow.NRACESSONFE = XML.NRACESSONFE;
                processXMLData(widget, XML);
            }
            else {
                doNothing(widget, XML);
            }
        });
        
    this.accessKeyOnBlur = ({owner: {field}}) => {
        const ACCESS_KEY = field.value() || false;
        let {widget} = field;
        let ACCESS_KEY_NOMASK = '';
        
        if (ACCESS_KEY.length >= 44) {
            ACCESS_KEY_NOMASK = ACCESS_KEY.replace(/[^\d]+/g, "");
            
            if(!field.mask) {
                field.mask = {
                    type: 'fix',
                            params: {
                                mask: '9999.9999.9999.9999.9999.9999.9999.9999.9999.9999.9999'
                            }
                };
                field.widget.redraw();
            }
        }
        else {
            ACCESS_KEY_NOMASK = ACCESS_KEY;
            if(field.mask) {
                delete field.mask;
                field.widget.redraw();
            }
        }
        
        if (ACCESS_KEY){
            let filter_nf = [
                {
                    "name": "NRACESSONFE",
                    "operator": "=",
                    "value": ACCESS_KEY_NOMASK
                }
            ];
            
            General.callRestEngine("df#/crud/intg_notafiscal", "FilterData", filter_nf).then(function(notafiscal){
                if (notafiscal.dataset.intg_notafiscal[0]){
                    let cdforneced = notafiscal.dataset.intg_notafiscal[0].CDFORNECED;
                    
                    let filter_end = [
                        {
                            "name": "CDFORNECED",
                            "operator": "=",
                            "value": cdforneced
                        }
                    ];
                    
                    General.callRestEngine("df#/crud/endeforn", "FilterData", filter_end).then(function(endeforneced){
                        if(endeforneced.dataset.endeforn[0]){
                            return fetchIncomingByAccessKey(widget, ACCESS_KEY_NOMASK);
                        } else {
                            return MessageService.showMessageByCode('FORNECEDOR_SEM_ENDERECO');
                        }
                    }, function(callback) {
                        return MessageService.showMessageByCode('FALHA_CONSULTA_BANCO');
                    });
                    
                } else {
                    return MessageService.showMessageByCode('CHAVE_ACESSO_INVALIDA');
                }
            }, function(callback) {
                return MessageService.showMessageByCode('FALHA_CONSULTA_BANCO');
            });
        } else {
            return applyInvalidAccessKeyRules(widget);
        }
    };

    const showAFMessageAndOpenPopup = widget => {
        const data = MESSAGES.AF_OBRIGATORIA,
            field = widget.getField('CBUTILIZAAF'),
            openAFPopup = () => {
                widget.currentRow.CBUTILIZAAF = 'S';
                widget.currentRow.DSUTILIZAAF = 'Realizar importação Af';

                field.value('S');
                this.abreImportacaoAF(field);
            };

        MessageService.showMessageByCode('AF_OBRIGATORIA', null, data).then(openAFPopup);
    };

    /* ************************* Lancamento Entrada - Itens ********************************* */

    var somaValorTotalItens = function(itensWidget) {
        var dataSourceItem = itensWidget.dataSource.data,
            valorTotal = Df11000_Lancamento_Service.somaTotalItens(dataSourceItem);
            
        return valorTotal;
    };

    const validaEdicaoImportacaoXml =lancamentoItemWidget => {
        const parentWidgetCurrentRow = lancamentoItemWidget.getParent().currentRow,
            importadoViaXml = parentWidgetCurrentRow.IDNFGERADAVIAXML === 'S',
            importacaoIdenticaXml = parentWidgetCurrentRow.IDGERNFIGUALXML === 'S',
            changeEventQuantidade = importadoViaXml ? 'calculaValorUnitario(true);' : 'quantidadeFieldOnChange(args);';
        
        lancamentoItemWidget.getField('QTITEMPROD').events[0].code = `Dfe11000_LancamentoEntrada_Controller.${changeEventQuantidade}`;

        ['VRUNITITEM', 'VRTOTAITEM', 'VRDESCITEM'].forEach(name =>
            lancamentoItemWidget.getField(name).readOnlyOnEdit = importadoViaXml);
    };

    this.actNovoItemOnClick = function(lancamentoItemArgs) {
        lancamentoItemArgs.owner.widget.newRow();
        
        Df11000_Lancamento_Service.getProxSeqItens().then(nrseqitprod => {
            lancamentoItemArgs.owner.widget.currentRow.NRSEQITPROD = nrseqitprod;
            controlaStatusWidgets(lancamentoItemArgs.owner.widget, lancamentoItemArgs.owner.widget.name, false);
        });
    };

    this.actEditarItemOnClick = function(lancamentoItemArgs) {
        var lancamentoItemWidget = lancamentoItemArgs.owner.widget;
        var lancamentoItemRow = lancamentoItemArgs.row;
        
        if(lancamentoItemRow.hasOwnProperty('__rightActions') || lancamentoItemRow.hasOwnProperty('__leftActions')){
            delete lancamentoItemRow.__rightActions;
            delete lancamentoItemRow.__leftActions;
        }
        General.edit(lancamentoItemWidget);
        validaEdicaoImportacaoXml(lancamentoItemWidget);
        validaEdicaoItemNotaOrigem(lancamentoItemWidget);
        controlaStatusWidgets(lancamentoItemWidget, lancamentoItemWidget.name, false);
    };

    var validaEdicaoItemNotaOrigem = function(itensWidget) {
        var qtFieldOrig = itensWidget.getField('QTITEMORIG'),
            qtField = itensWidget.getField('QTITEMPROD');

        if (itensWidget.getParent().currentRow.IDOPERDEVONF == 'S') {
            qtFieldOrig.isVisible = true;
            qtField.validations.max = {value: itensWidget.currentRow.QTITEMORIG};
        } else {
            qtFieldOrig.isVisible = false;
            delete qtField.validations.max;
        }
    };

    this.actCancelarItemOnClick = function(lancamentoItemArgs)  {
        var lancamentoItemWidget = lancamentoItemArgs.owner.widget;
        
        controlaStatusWidgets(lancamentoItemWidget.getParent(), lancamentoItemWidget.name, true);
        backStatusModify();

        if (lancamentoItemWidget.inlineEditing || lancamentoItemWidget.getField('QTITEMPROD').inlineEditable) {
            lancamentoItemWidget.list();
            fieldItensInlineEditable(lancamentoItemWidget, false);

            //TODO alterar para remover os itens manualmente e executar o calculo do valor total
            lancamentoItemWidget.reload();

            delete lancamentoItemWidget.inlineEditing;
            return lancamentoItemWidget.rollbackInlineEdit();
        }

        return General.backOrCancel(lancamentoItemArgs);
    };

    var calculaImpostoItemProduto = function(widgetParentRow, lancamentoItemRow) {
        if(lancamentoItemRow) {
            
            if(lancamentoItemRow.hasOwnProperty('__rightActions') || lancamentoItemRow.hasOwnProperty('__leftActions')){
                delete lancamentoItemRow.__rightActions;
                delete lancamentoItemRow.__leftActions;
            }
            return Dfe11000_LancamentoEntrada_Service.calculaImpostoItemProduto(widgetParentRow, lancamentoItemRow).then(function(calculoImpostoProduto) {
                var impostosTemp = calculoImpostoProduto.dataset.calculoImpostoProduto;
    
                impostosTemp = Object.keys(impostosTemp).map(key => impostosTemp[key]);
    
                impostosTemp.forEach(imposto => {
                    imposto.CDIMPOSTO = imposto.CDIMPOTEMP;
                    imposto.VRALIQUTMP = imposto.VRALIQUTMP;
    
                    imposto.CDFILIAL = this.CDFILIAL;
                    imposto.NRLANCTONF = this.NRLANCTONF;
                    imposto.NRSEQITPROD = this.NRSEQITPROD;
                    imposto.__is_new = false;
    
                    delete imposto.CDIMPOTEMP;
                });
    
                Df11000_Lancamento_Service.setDataSourceItensImpostos(impostosTemp);
            }.bind(lancamentoItemRow));
        }
    };

    this.actIncluirItemOnClick = function(lancamentoItemArgs) {
        var lancamentoItemWidget = lancamentoItemArgs.owner.widget;
        var cabecalhoRow = lancamentoItemWidget.getParent().currentRow;
        var validacoes = processaValidacoesNotaOrigem(templateManager.container.getWidget("cabecalho").currentRow);
        
        if(validacoes.devolucao) 
            this.chamaDadosAdicionais(lancamentoItemArgs);
        
        if (lancamentoItemWidget.getField('QTITEMPROD').inlineEditable) {
            return this.actSalvarMultiItemOnClick(lancamentoItemArgs); //Widget em modo de edição de múltiplos itens.
        }
        if (!lancamentoItemWidget.isValid())
            return lancamentoItemWidget.e.preventDefault();

        var lancamentoItemRow = lancamentoItemArgs.row;
        var widgetParentRow = lancamentoItemArgs.owner.widget.getParent().currentRow;
        const values = getCorrectsValues(lancamentoItemWidget);
        if(lancamentoItemRow.hasOwnProperty('__rightActions') || lancamentoItemRow.hasOwnProperty('__leftActions')){
            delete lancamentoItemRow.__rightActions;
            delete lancamentoItemRow.__leftActions;
        }

        lancamentoItemRow.NRLANCTONF = lancamentoItemRow.NRLANCTONF || widgetParentRow.NRLANCTONF;
        lancamentoItemRow.CDFILIAL = lancamentoItemRow.CDFILIAL || widgetParentRow.CDFILIAL;
        lancamentoItemRow.CDFILIMOVI = widgetParentRow.CDFILIRECE || lancamentoItemRow.CDFILIAL;
        lancamentoItemRow.QTITEMPROD = values.QTITEMPROD;

        lancamentoItemRow.VRUNITITEM = values.VRUNITITEM;
        lancamentoItemRow.VRTOTAITEM = values.VRTOTAITEM;
        lancamentoItemRow.VRTOTAITEM = values.VRTOTAITEM;

        Dfe11000_LancamentoEntrada_Service.validaParamProduto(lancamentoItemRow.CDFILIAL, lancamentoItemRow.CDARVPROD).then(function(response){
            var idbloqnfsaf = response.dataset.validaParamProduto[0].IDBLOQNFSAF == 'S' ? true : false;
            if(idbloqnfsaf){
                ScreenService.showMessage("Esse produto não está vinculado a uma AF.");
            } else{
                Df11000_Lancamento_Service.salvaItem(widgetParentRow, lancamentoItemRow).then(() => {
                    lancamentoItemRow.__generate_taxes = lancamentoItemRow.__is_new;
                    alteraStatusNotafiscal(lancamentoItemWidget.getParent()).then(() => {
                        calculaImpostoItemProduto(widgetParentRow, lancamentoItemRow);
                        templateManager.container.getWidget("impostosItem").fields[2].inlineEditable = "number";
                        templateManager.container.getWidget("impostosItem").fields[3].inlineEditable = "number";
                    });
                    if(lancamentoItemRow.__is_new == false && lancamentoItemRow.NRAUTOFORN) {
                        lancamentoItemRow.OLDQTITEMPROD = lancamentoItemWidget.backupRow.QTITEMPROD;
                        Dfe11000_LancamentoEntrada_Service.atualizaItemAf(lancamentoItemRow);
                    }
        
                    lancamentoItemRow.__is_new = false;
        
                    
                    Df11000_Lancamento_Service.setDataSourceItem(lancamentoItemRow)
                        .then(Df11000_Lancamento_Service.atualizaLancamento)
                        .then(() => {
                            Df11000_Lancamento_Service.setDataSourceLancamentoNota();
                            validaEdicaoItemNotaOrigem(lancamentoItemWidget);
                            lancamentoItemWidget.view();
                            controlaStatusWidgets(lancamentoItemWidget, lancamentoItemWidget.name, true);
                            backStatusModify();
                        });
                }, error => {
                    if (Array.isArray(error)) {
                        var stringCamposObrig = error.join(', ');
                        var mensagem = 'Os campos %1 deve(m) ser maior que zero.'.replace('%1', stringCamposObrig);
                        return ScreenService.showMessage(mensagem);
                    } else if (typeof error === 'string')
                        return ScreenService.showMessage(error);
                    return ScreenService.showMessage(error.data.error);
                });
            }
        });

        
    };

    this.actSalvarMultiItemOnClick = function(lancamentoItemArgs) {
        var lancamentoItemWidget = lancamentoItemArgs.owner.widget;
        var itens = lancamentoItemWidget.dataSource.data;
        var parentWidget = lancamentoItemWidget.getParent();
        var currentRowParentWidget = Util.clone(parentWidget.currentRow, ["__rightActions", "__leftActions"]);

        var isInvalid = itens.some(item => !item.QTITEMPROD || !item.VRUNITITEM || !item.VRTOTAITEM);

        if (isInvalid) {
            return ScreenService.showMessage('Campos obrigatórios não preenchidos.');
        }

        var novosItens = itens.filter(item => item.__is_new);
        novosItens.forEach(item => {
            delete(item.__rightActions); 
            delete(item.__leftActions);
        });
        var itensAlterados = itens.filter(item => item.changed && !item.__is_new);
        itensAlterados.forEach(item => {
            delete(item.__rightActions); 
            delete(item.__leftActions);
        });

        //TODO validar campos obrigatórios igual a 0
        Dfe11000_LancamentoEntrada_Service.salvaMultiItens(novosItens, itensAlterados, currentRowParentWidget)
            .then(calculoImpostosProdutos => {
                var impostos = calculoImpostosProdutos.dataset.impostosTemp;

                impostos.forEach(imposto => {
                    imposto = Object.keys(imposto).map(key => imposto[key]);
                    imposto.forEach(impos => {
                        impos.CDIMPOSTO = impos.CDIMPOTEMP;
                        impos.__is_new = false;

                        delete impos.CDIMPOTEMP;
                    });
                    Df11000_Lancamento_Service.setDataSourceItensImpostos(imposto);
                });

                itens.forEach(item => {
                    item.__is_new = false;
                    //@todo remover delete item.changed quando desmarcaLinhasEditadas for implementada
                    delete item.changed_QTITEMPROD;
                    delete item.changed_VRUNITITEM;
                    delete item.changed_VRTOTAITEM;
                    delete item.changed;
                });

                Df11000_Lancamento_Service.setDataSourceItem(itens).then(function () {
                        setTimeout(Df11000_Lancamento_Service.atualizaLancamento, 500);
                    });

                alteraStatusNotafiscal(parentWidget).then(() => {
                    controlaStatusWidgets(parentWidget, lancamentoItemWidget.name, true, lancamentoItemWidget.widgets);
                    controlaStatusWidgets(lancamentoItemWidget, lancamentoItemWidget.name, true);
                    fieldItensInlineEditable(lancamentoItemWidget, false);
                    delete lancamentoItemWidget.inlineEditing;
                });
                lancamentoItemWidget.list();
            })
            .catch(requestError);
    };

    this.actExcluirItemOnClick = function(lancamentoItemArgs) {
        ScreenService.confirmMessage(
            ScreenService.i18n('Excluir Registro?'),
            'question',
            () => {
                const {owner: {widget}, row} = lancamentoItemArgs;

                Df11000_Lancamento_Service.excluiItem(row)
                    .then(() => alteraStatusNotafiscal(widget.getParent()))
                    .then(() => Df11000_Lancamento_Service.excluiItemDataSource(row.NRSEQITPROD))
                    .then(Df11000_Lancamento_Service.atualizaLancamento)
                    .then(() => {
                        if(row.NRAUTOFORN) {
                            Dfe11000_LancamentoEntrada_Service.excluirItemAf(row);
                        }
                        Df11000_Lancamento_Service.setDataSourceLancamentoNota();
                        General.backOrCancel(lancamentoItemArgs);
                        widget.reload();
                        widget.activate();
                    });
            });
    };

    this.nivelCompletoProduto = function() {
        return General.getLevelByMask('GER_CDPICTPROD','1').length;
    };

    var setValoresEmissaoProdutoOnChange = function(lancamentoItemRow) {
        lancamentoItemRow.CDPRODEMIS = lancamentoItemRow.CDARVPROD;
        lancamentoItemRow.DSPRODEMIS = lancamentoItemRow.NMPRODUTO;
    };
    
    var informacoesComplementaresProduto = function(lancamentoItemArgs){
        var widgetParentRow = lancamentoItemArgs.owner.owner.getParent().getParent().currentRow;
        lancamentoItemArgs.row.CDFILIMOVI = widgetParentRow.CDFILIRECE;
        lancamentoItemArgs.row.CDFILIAL = widgetParentRow.CDFILIAL;
        
        lancamentoItemArgs.row.IDENTRSAIDOP = widgetParentRow.IDENTRSAIDOP;
        lancamentoItemArgs.row.CDTIPOOPER = widgetParentRow.CDTIPOOPER;
        
        var row = {
            IDENTRSAIDOP:lancamentoItemArgs.row.IDENTRSAIDOP,
            CDTIPOOPER:lancamentoItemArgs.row.CDTIPOOPER
        };
        
        General.callRestEngine("df_entrada#/getIDATUESTTPOP", "Row", row).then((success) => {
          atualizaEstoque = success.dataset.atualizaEstoque.IDATUESTTPOP;
           Lib00206_InfComplProduto_Controller.infComplementares(lancamentoItemArgs.owner.field.widget, lancamentoItemArgs.row, false, atualizaEstoque);
           
        });
        
    };
    
    this.validacoesProdutoOnChange = function(lancamentoItemArgs) {
        setValoresEmissaoProdutoOnChange(lancamentoItemArgs.row);
        informacoesComplementaresProduto(lancamentoItemArgs);
    };

    var calculaValorTotal = function(lancamentoItemArgs) {
        const widget = lancamentoItemArgs.owner.field.widget;
        var values = {};

        if (widget.inlineEditing) {
            values.QTITEMPROD = lancamentoItemArgs.row.QTITEMPROD;
            values.VRUNITITEM = lancamentoItemArgs.row.VRUNITITEM;
        } else {
            values = getCorrectsValues(widget);
        }
        
        lancamentoItemArgs.row.VRTOTAITEM = Df11000_Lancamento_Service.calculaValorTotal(
            values.QTITEMPROD, values.VRUNITITEM
        );
    };

    this.valorUnitarioFieldOnChange = function(lancamentoItemArgs) {
        lancamentoItemArgs.owner.field.modify = true;
        lancamentoItemArgs.owner.field.widget.getField('VRTOTAITEM').modify = true;
        calculaValorTotal(lancamentoItemArgs);
    };

    this.quantidadeFieldOnChange = function(lancamentoItemArgs) {
        lancamentoItemArgs.owner.field.modify = true;
        lancamentoItemArgs.owner.field.widget.getField('VRTOTAITEM').modify = true;
        calculaValorTotal(lancamentoItemArgs);
    };

    this.calculaValorUnitario = function(chageField) {
        const widget = templateManager.container.getWidget('itens');

        var values = {};
        var lancamentoItemCurrentRow = widget.currentRow;

        if (chageField) {
            widget.getField('QTITEMPROD').modify = true;
            widget.getField('VRUNITITEM').modify = true;
        }

        if (widget.inlineEditing) {
            values.QTITEMPROD = lancamentoItemCurrentRow.QTITEMPROD;
            values.VRTOTAITEM = lancamentoItemCurrentRow.VRTOTAITEM;
        } else {
            values = getCorrectsValues(widget);
        }

        lancamentoItemCurrentRow.VRUNITITEM = Df11000_Lancamento_Service.calculaValorUnitario(
            values.QTITEMPROD, values.VRTOTAITEM
        );
    };

    this.valorTotalFieldOnChange = function(lancamentoItemArgs) {
        lancamentoItemArgs.owner.field.modify = true;
        lancamentoItemArgs.owner.field.widget.getField('VRUNITITEM').modify = true;
        this.calculaValorUnitario(false);
        calculaValorTotal(lancamentoItemArgs);
    };
    
    //
    this.valorTotalFieldOnChangeNew = function(lancamentoItemArgs) {
        lancamentoItemArgs.owner.field.modify = true;
        lancamentoItemArgs.owner.field.widget.getField('VRTOTAITEM').modify = true;
        lancamentoItemArgs.owner.field.widget.getField('VRUNITITEM').modify = true;
        
        lancamentoItemArgs.row.VRTOTAITEM = calculaValorTotalNew(lancamentoItemArgs.owner.field.widget.getField('QTITEMPROD').value(), lancamentoItemArgs.owner.field.widget.getField('VRUNITITEM').value());
        
        lancamentoItemArgs.row.VRUNITITEM = calculaValorUnitarioNew(lancamentoItemArgs.owner.field.widget.getField('QTITEMPROD').value(), lancamentoItemArgs.owner.field.widget.getField('VRTOTAITEM').value());
        //this.calculaValorUnitario(false);
    };
    
    this.valorUnitarioFieldOnChangeNew = function(lancamentoItemArgs) {
        lancamentoItemArgs.owner.field.modify = true;
        lancamentoItemArgs.owner.field.widget.getField('VRUNITITEM').modify = true;
        
        lancamentoItemArgs.row.VRTOTAITEM = calculaValorTotalNew(lancamentoItemArgs.owner.field.widget.getField('QTITEMPROD').value(), lancamentoItemArgs.owner.field.widget.getField('VRUNITITEM').value());
        
        //this.calculaValorUnitario(false);
    };
    
    this.quantidadeFieldOnChangeNew = function(lancamentoItemArgs) {
        lancamentoItemArgs.owner.field.modify = true;
        lancamentoItemArgs.owner.field.widget.getField('QTITEMPROD').modify = true;
        
        lancamentoItemArgs.row.VRTOTAITEM = calculaValorTotalNew(lancamentoItemArgs.owner.field.widget.getField('QTITEMPROD').value(), lancamentoItemArgs.owner.field.widget.getField('VRUNITITEM').value());
        //this.calculaValorUnitarioNew(false);
    };
    
    function calculaValorTotalNew (quantidade, valorUnitario) {
        quantidade = removeMaskString(quantidade);
        valorUnitario = removeMaskString(valorUnitario);

        return quantidade * valorUnitario;
    }

    var removeMaskString = function(field) {
        return typeof(field) === "string" ? parseFloat(field.replace(",", ".")) : field;
        //field = (typeof(field) === "string" && field.indexOf('.') === -1) ? $.currency.unmaskString(field) : field;
        //field = Math.abs(field);
        //return field;
    };

    var calculaValorUnitarioNew = function(quantidade, valorTotal) {
        // Atribui valor 0 caso a quantidade/valorTotal passados sejam valores
        // inválidos, como null ou undefined.
        quantidade = removeMaskString(quantidade) || 0;
        valorTotal = removeMaskString(valorTotal) || 0;
        var calculo = 0;

        if (quantidade === 0)
            return calculo;

        calculo = valorTotal / quantidade;
        return calculo;
    };
    
    //

    this.validateActionsWidgetOnEnter = function(lancamentoItemArgs) {
        var lancamentoItemWidget = lancamentoItemArgs.owner,
            parentCurrentRow = lancamentoItemWidget.getParent().currentRow,
            importadoViaXml = parentCurrentRow.IDNFGERADAVIAXML === 'S',
            importacaoIdenticaXml = parentCurrentRow.IDGERNFIGUALXML === 'S',
            notaIgualXml = importacaoIdenticaXml && importadoViaXml,
            statusNota = parentCurrentRow.IDSTATUS,
            disableActionNotaIgualXml = !notaIgualXml && (statusNota === 'D' || statusNota === 'P'),
            disableActionNotaDevolucao = (parentCurrentRow.IDOPERDEVONF == 'N' || parentCurrentRow.CDTIPOOPER == 3),
            disableActionEdicao = (parentCurrentRow.IDSTATUSNFE === 'R' || parentCurrentRow.IDSTATUSNFE === 'P' || parentCurrentRow.IDSTATUSNFE === undefined) && (statusNota === 'D' || statusNota === 'P'),
            isTipo10 = (parentCurrentRow.CDTIPOOPER != '10'),
            tipooper = parentCurrentRow.CDTIPOOPER,
            itnotafisc = parentCurrentRow.itnotafisc;

        lancamentoItemWidget.getAction('actNovoItem').isVisible = (disableActionNotaIgualXml && disableActionNotaDevolucao);
        lancamentoItemWidget.getAction('actExcluirItem').isVisible = (disableActionNotaIgualXml );
        lancamentoItemWidget.getAction('actEditarItem').isVisible = (disableActionEdicao);

        var saveAction = lancamentoItemWidget.getAction('save');
        if (saveAction) {
            saveAction.isVisible = false;
        }

        fieldItensInlineEditable(lancamentoItemWidget, false);
        
        lancamentoItemWidget.container.getWidget('impostosItem').enabled = isTipo10;
        
    };

    this.after = function(lancamentoItemArgs) {
        var produtoField = lancamentoItemArgs.owner.field;
        var produtoFieldWidget = produtoField.selectWidget;
        produtoFieldWidget.unSelectAll();

        if (!produtoFieldWidget.getAction('confirmarProdutos')) {
            produtoFieldWidget.showCheckbox = true;

            var actionConfirmarProdutos = {
                name: 'confirmarProdutos',
                label: 'Confirmar',
                showAsAction: 'never_list',
                isVisible: true,
                showOnList: true,
                events: [{
                    name: 'ActionEvent',
                    code: 'Dfe11000_LancamentoEntrada_Controller.confirmarSelecaoProdutos(args);',
                }]
            };
            actionConfirmarProdutos = metaDataFactory.actionFactory(actionConfirmarProdutos);
            actionConfirmarProdutos.widget = produtoFieldWidget;
            produtoFieldWidget.actions.push(actionConfirmarProdutos);

            eventEngine.unbindEvent(produtoFieldWidget, 'WidgetAfterMoveRow', 0);
            eventEngine.bindEvent(produtoFieldWidget, 'WidgetAfterMoveRow', 'args.owner.toggle(args.owner.currentRow);');

            templateManager.updateTemplate();
        }
    };

    var fieldItensInlineEditable = function(itensWidget, isInlineEditable) {
        if (isInlineEditable) {
            itensWidget.getField('QTITEMPROD').inlineEditable = 'number';
            itensWidget.getField('VRUNITITEM').inlineEditable = 'number';
            itensWidget.getField('VRTOTAITEM').inlineEditable = 'number';
            itensWidget.getField('VRDESCITEM').inlineEditable = 'number';
        } else {
            delete itensWidget.getField('QTITEMPROD').inlineEditable;
            delete itensWidget.getField('VRUNITITEM').inlineEditable;
            delete itensWidget.getField('VRTOTAITEM').inlineEditable;
            delete itensWidget.getField('VRDESCITEM').inlineEditable;
        }
        templateManager.updateTemplate();
    };

    this.confirmarSelecaoProdutos = function(args) {
        var fieldProdutoWidget = args.owner.widget;
        var fieldProduto = fieldProdutoWidget.selectField;
        var itensWidget = fieldProduto.widget;
        var selectedRows = fieldProdutoWidget.getCheckedRows();
        var multiItens = selectedRows.length !== 1;
        var ignoredPropertyes = ['__rightActions', '__leftActions'];

        itensWidget.insertMultipleLines = true;
        //itensWidget.getAction('actSalvarMultiItem').isVisible = multiItens;
        //itensWidget.getAction('actSalvarItem').isVisible = !multiItens;

        if (!multiItens) {
            args.owner = fieldProdutoWidget;
            args.row = args.selectedRow;
            fieldProdutoWidget.setSelected(args);
            itensWidget.insertMultipleLines = false;
            fieldItensInlineEditable(itensWidget, false);
            return args;
        }

        var parentWidget = itensWidget.getParent();
        var emptyRow = Util.clone(itensWidget.currentRow, ignoredPropertyes);
        var indexNumberRow = emptyRow.__index;
        var seqItens = emptyRow.NRSEQITPROD;
        
        itensWidget.dataSource.data.splice(itensWidget.currentRow.__index, 1);

        selectedRows.forEach(selectedItem => {
            var newRegister = Util.clone(emptyRow, ignoredPropertyes);

            setCurrentRowWithOutData(newRegister, selectedItem, fieldProduto.outData);
            setValoresEmissaoProdutoOnChange(newRegister);

            newRegister.__index = ++indexNumberRow;
            newRegister.NRSEQITPROD = '0000'.substring(0, 4 - seqItens.length) + seqItens;

            newRegister.NRLANCTONF = newRegister.NRLANCTONF || parentWidget.currentRow.NRLANCTONF;
            newRegister.CDFILIAL   = newRegister.CDFILIAL || parentWidget.currentRow.CDFILIAL;
            newRegister.CDFILIMOVI = newRegister.CDFILIAL;
            newRegister.__rightActions = itensWidget.currentRow.__rightActions;
            newRegister.__leftActions = itensWidget.currentRow.__leftActions;
            
            itensWidget.dataSource.data.push(newRegister);
            seqItens = (++seqItens).toString();
        });

        //itensWidget.inlineEditing = true; aparentemente não tem necessidade de setar editing como true, incluisive isto causa erros ao adicionar as rows.
        fieldItensInlineEditable(itensWidget, true);

        ScreenService.closePopup();
        ScreenService.closeSwipe();

        controlaStatusWidgets(itensWidget.getParent(), itensWidget.name, false, itensWidget.widgets);

        itensWidget.edit();
        templateManager.updateTemplate();
    };

    const getCorrectsValues = widget => {
        const currentRow = widget.currentRow;
        const auxRow = widget.backupRow;

        const qtIsModify = widget.getField('QTITEMPROD').modify;
        const vrUniIsModify = widget.getField('VRUNITITEM').modify;
        const totUniIsModify = widget.getField('VRTOTAITEM').modify;
        const descUniIsModify = widget.getField('VRDESCITEM').modify;

        const back = {
            'QTITEMPROD': qtIsModify || currentRow.__is_new ? currentRow.QTITEMPROD : auxRow.QTITEMPROD,
            'VRUNITITEM': vrUniIsModify || currentRow.__is_new ? currentRow.VRUNITITEM : auxRow.VRUNITITEM,
            'VRTOTAITEM': totUniIsModify || currentRow.__is_new ? currentRow.VRTOTAITEM : auxRow.VRTOTAITEM,
            'VRDESCITEM': descUniIsModify || currentRow.__is_new ? currentRow.VRDESCITEM : auxRow.VRDESCITEM
        };

        return back;
    };

    const backStatusModify = () => {
        const widget = templateManager.container.getWidget('itens');
        widget.getField('QTITEMPROD').modify = false;
        widget.getField('VRUNITITEM').modify = false;
        widget.getField('VRTOTAITEM').modify = false;
        widget.getField('VRDESCITEM').modify = false;
    };

    /* *********************** Lancamento Entrada - Imposto Item ****************************** */
    this.actSalvaImpostosItemWidgetOnSaveInlineEdit = function(impostosItemArgs) {
        var impostosItemWidget = impostosItemArgs.owner;
        const data = impostosItemWidget.dataSource.data;
        var statusNota = impostosItemArgs.owner.getParent().getParent().currentRow.IDSTATUS;
        const camposAlterados = data.filter(row => (!!('changed_VRIMPOSTOIT' in row) || !!('changed_VRALIQIMPOIT' in row)));
        
        camposAlterados.forEach((row) => {
            if((row.changed_VRIMPOSTOIT || row.changed_VRIMPOSTOIT === 0)  && !row.changed_VRALIQIMPOIT) {
                row.VRALIQIMPOIT = (row.VRIMPOSTOIT / impostosItemWidget.getParent().currentRow.VRTOTAITEM) * 100;
            }
            else if((row.changed_VRALIQIMPOIT || row.changed_VRALIQIMPOIT === 0)&& !row.changed_VRIMPOSTOIT) {
                row.VRIMPOSTOIT = (row.VRALIQIMPOIT * impostosItemWidget.getParent().currentRow.VRTOTAITEM) / 100;   
            }
        });
        
        const salvaImpostosTmpItem = () =>
            Df11000_Lancamento_Service.salvaImpostosTmpItem(camposAlterados)
                .then(() => {
                    Df11000_Lancamento_Service.setDataSourceItensImpostos(data)
                    .then(()=> Df11000_Lancamento_Service.atualizaLancamento());
                    removeEdicaoLinhasWidget(camposAlterados);
                    impostosItemWidget.list();
                })
                .catch(requestError);
                
        const salvaImpostosItem = () =>
            Df11000_Lancamento_Service.salvaImpostosItem(camposAlterados)
                .then(() => {
                    Df11000_Lancamento_Service.setDataSourceItensImpostos(data)
                    .then(()=> Df11000_Lancamento_Service.atualizaLancamento());
                    removeEdicaoLinhasWidget(camposAlterados);
                    impostosItemWidget.list();
                })
                .catch(requestError);

        if (statusNota !== 'D')
            salvaImpostosItem();
        else
            salvaImpostosTmpItem();

    };

    /* *********************** Lancamento Entrada - Imposto ********************************** */

    this.recalculoImpostosNota = function(impostosArgs) {
        var impostosWidget = impostosArgs.owner;

        Df11000_Lancamento_Service.calculaTotalImpostos().then(() => {
            impostosWidget.redraw();
            impostosWidget.reload();
            });
    };

    this.setIdvalorinfo = function(row) {
        if (!row.VRIMPOSTOIT || row.VRIMPOSTOIT == 0) {
            row.IDVALORINFO = 'N';
        } else {
            row.IDVALORINFO = 'S';
        }
    };

    /* ************************* Lancamento Entrada - Encargos ********************************* */

    this.encargoOnChange = function(encargosArgs) {
        var encargoWidget = encargosArgs.owner.field.widget;
        controlaStatusWidgets(encargoWidget.getParent(), encargoWidget.name, false);
    };

    this.setDtsEncargoOnEnter = function(encargosArgs) {
        var widgetEncargo = encargosArgs.owner;
        if (!widgetEncargo.dataSource.data.length) {
            Df11000_Lancamento_Service.buscaEncargosPadrao().then(encargos => {
                encargos.forEach(encargo => {
                    encargo.VRITEMENCA = 0;
                    encargo.NRLANCTONF = widgetEncargo.getParent().currentRow.NRLANCTONF;
                    encargo.CDFILIAL = widgetEncargo.getParent().currentRow.CDFILIAL;
                    encargo.VRENMOEDESTR = '';
                    encargo.IDALTERADO = 'N';
                    encargo.__is_new = true;
                });
                Df11000_Lancamento_Service.setDataSourceEncargo(encargos).then(() => {
                    widgetEncargo.currentRow = encargos;
                    widgetEncargo.reload();
                });
            });
        }
    };

    this.salvaEncargoWidgetOnSaveInlineEdit = function(encargosArgs) {
        if (encargosArgs.owner.isValid()) {
            var encargoWidget = encargosArgs.owner;
            var itensWidget = encargosArgs.owner.container.getWidget('itens');
            var dataSourceEncargo = encargoWidget.dataSource.data;
            var parentWidget = encargosArgs.owner.getParent();

            var totalItens = somaValorTotalItens(itensWidget);
            var validaTotalEncargos = Df11000_Lancamento_Service.validaTotalEncargos(dataSourceEncargo, totalItens);

            if (!validaTotalEncargos)
                return requestError('Valor de encargo a descontar maior que o valor total dos itens.');

            //TODO remover esta linha e coloca-la no general
            var encargosSalvar;
            if (encargosArgs.row.__is_new) {
                encargosSalvar = dataSourceEncargo;
            } else {
                encargosSalvar = encargosArgs.dataSource.filter(encargo => !!encargo.changed);
            }

            Df11000_Lancamento_Service.salvaEncargo(encargosSalvar).then(() => {
                alteraStatusNotafiscal(parentWidget);
                encargosSalvar.forEach(encargo => {
                    encargo.__is_new = false;
                    //@todo remover delete encargo.changed quando desmarcaLinhasEditadas for implementada
                    delete encargo.changed_VRITEMENCA;
                    delete encargo.changed;
                });

                Df11000_Lancamento_Service.setDataSourceEncargo(encargosArgs.dataSource)
                    .then(Df11000_Lancamento_Service.atualizaLancamento)
                    .then(() => {
                        controlaStatusWidgets(encargoWidget.getParent(), encargoWidget.name, true);
                        //@todo desmarcaLinhasEditadas
                        //GerUtil.desmarcaLinhasEditadas(lancamentoEncargoWidget, CANCELAR);
                        Df11000_Lancamento_Service.setDataSourceLancamentoNota();
                        //encargosArgs.owner.view();
                        encargosArgs.owner.list();
                        encargosArgs.owner.reload();
                    });
            }, requestError);
        }
    };

    this.cancelaEncargoWidgetOnCancelInlineEdit = function(encargosArgs) {
        // TODO criar função para desmarcar linhas editadas
        // GerUtil.desmarcaLinhasEditadas(lancamentoEncargoWidget, CANCELAR);
        // delete encargo["changed_VRITEMENCA"];
        // delete encargo.changed;
        var encargoWidget = encargosArgs.owner;
        encargosArgs.owner.dataSource.data.forEach(encargo => {
            encargo.VRITEMENCA = (encargo.changed) ? encargo.changed_VRITEMENCA : encargo.VRITEMENCA;
            delete encargo.changed_VRITEMENCA;
            delete encargo.changed;
        });
        controlaStatusWidgets(encargoWidget.getParent(), encargoWidget.name, true);
        encargoWidget.view();
    };

    /* ************************* Lancamento Entrada - Vencimentos ********************************* */

    this.actNovoVencimentoActionEvent = function(vencimentoArgs) {
        ScreenService.showLoader();
        var vencimentoWidget = vencimentoArgs.owner.widget;
        var widgetParentRow = vencimentoWidget.getParent().currentRow;

        
        var vencimentoCurrentRow = vencimentoWidget.currentRow;

        var dataSourceVencimento = vencimentoWidget.dataSource.data;
        var valorNotaFiscal = widgetParentRow.VRNOTAFISC;
        
        var promises = [];
        
        promises.push(Df11000_Lancamento_Service.somaTotalImpostosRetidos().then(valorImpostosRetidos => {
            var valorVencimento = Df11000_Lancamento_Service.calculaValorVencimento(dataSourceVencimento, valorNotaFiscal, valorImpostosRetidos);
            vencimentoWidget.getField('VRATUAPAG').defaultValue     = valorVencimento;
        }));
        promises.push(Df11000_Lancamento_Service.getProxSeqVencimento().then(nrseqvenpag => {
            vencimentoWidget.getField('NRSEQVENPAG').defaultValue   = nrseqvenpag;
        }));
        
        ZHPromise.all(promises).then((success) => {
            vencimentoWidget.newRow();
            ScreenService.hideLoader();
        });
    };

    this.editarWidget = function(args){
        General.edit(args.owner.widget);
        args.owner.widget.getField('DSLINKANEXO').isVisible = false;
        args.owner.widget.getField('DSLINKANEXO').validations = {};
        args.owner.widget.getField('CBUTILIZAAF').validations = {};
        controlaStatusWidgets(args.owner.widget, args.owner.widget.name, false);
        validaCampoModDocFiscal(args.owner.widget, args.row);
    };

    this.actIncluirVencimentoActionEvent = vencimentoArgs => {
        if (vencimentoArgs.owner.widget.isValid()) {
            var vencimentoWidget = vencimentoArgs.owner.widget;
            var dataSourceVencimento = vencimentoWidget.dataSource.data;
            var widgetParent = vencimentoArgs.owner.widget.getParent();
            var widgetParentRow = widgetParent.currentRow;
            var valorNotaFiscal = widgetParentRow.VRNOTAFISC;
            var vencimentoRow = vencimentoArgs.row;
            var dataEmissao = widgetParentRow.DTEMISSAO;
            var dataVencimento = vencimentoRow.DTATUAVENPAG;
            var erroDataVencimento = Dfe11000_LancamentoEntrada_Service.validaDataVencimento(dataEmissao, dataVencimento);

            if (erroDataVencimento)
                return ScreenService.showMessage(erroDataVencimento);
                
            if (vencimentoRow.VRATUAPAG === "0.00" || vencimentoRow.VRATUAPAG === 0.00)
                return ScreenService.showMessage("Valor deve ser maior que zero.");
                
            
            Df11000_Lancamento_Service.somaTotalImpostosRetidos().then(valorImpostosRetidos => {
                var valorVencimento = Df11000_Lancamento_Service.validaValorVencimento(dataSourceVencimento, valorNotaFiscal, valorImpostosRetidos);
                if(valorVencimento < 0) {
                    return ScreenService.showMessage('Total de vencimentos maior que total da nota!');
                }
                    
                vencimentoRow.NRPARCELAPAG = '00';
                vencimentoRow.NRLANCTONF = widgetParentRow.NRLANCTONF;
                vencimentoRow.CDFILIAL = widgetParentRow.CDFILIAL;
                vencimentoRow.CDEMPRESA = widgetParentRow.CDEMPRESA;
                vencimentoRow.NRORG = widgetParentRow.NRORG;
                vencimentoRow.VRORIGPAG = vencimentoRow.VRATUAPAG;
                vencimentoRow.DTORIGVENPAG = vencimentoRow.DTATUAVENPAG;
                
                
    
                Dfe11000_LancamentoEntrada_Service.salvaVencimento(vencimentoRow).then(() => {
                    vencimentoRow.__is_new = false;
                    alteraStatusNotafiscal(widgetParent);
    
                    Df11000_Lancamento_Service.setDataSourceVencimento(vencimentoRow)
                        .then(Df11000_Lancamento_Service.atualizaLancamento)
                        .then(Df11000_Lancamento_Service.setDataSourceLancamentoNota)
                        .then(ScreenService.goBack.bind(ScreenService))
                        .catch(requestError);
                }, requestError);
            });

            
        }
    };

    this.actExcluirVencimentoOnClick = lancamentoVencimentoArgs => {
        ScreenService.confirmMessage(
            ScreenService.i18n('Excluir Registro?'),
            'question',
            () => {
                Dfe11000_LancamentoEntrada_Service.excluiVencimento(lancamentoVencimentoArgs.row).then(() => {
                    alteraStatusNotafiscal(lancamentoVencimentoArgs.owner.widget.getParent());

                    Df11000_Lancamento_Service.excluiVencimentoDataSource(lancamentoVencimentoArgs.row.NRSEQVENPAG)
                        .then(Df11000_Lancamento_Service.atualizaLancamento)
                        .then(Df11000_Lancamento_Service.setDataSourceLancamentoNota)
                        .then(() => {
                            General.backOrCancel(lancamentoVencimentoArgs);
                            lancamentoVencimentoArgs.owner.widget.reload();
                        })
                        .catch(requestError);
                }, requestError);
            }
        );
    };

    this.qtDiasPadVencimentoFieldOnChange = function(vencimentoArgs) {
        var qtdias = vencimentoArgs.owner.field.value();
        var vencimentoWidget = vencimentoArgs.owner.field.widget;
        var vencimentoRow = vencimentoWidget.currentRow;
        var cabecalhoRow = vencimentoWidget.getParent().currentRow;
        var dtemissao = cabecalhoRow.DTEMISSAO;

        vencimentoRow.DTATUAVENPAG = Df11000_Lancamento_Service.calculaDataVencimento(qtdias, dtemissao);
    };

    this.dtVencimentoFieldOnChange = function(vencimentoArgs) {
        var vencimentoWidget = vencimentoArgs.owner.field.widget;
        var vencimentoRow = vencimentoWidget.currentRow;
        var cabecalhoRow = vencimentoWidget.getParent().currentRow;
        var dtEmissao = cabecalhoRow.DTEMISSAO;
        var dtVencimento = vencimentoRow.DTATUAVENPAG;

        //.toString() necessario pois ao validar a widget o 0 é considerado como boolean, ai . . .
        vencimentoRow.QTDIASVENPAG = Df11000_Lancamento_Service.calculaNumeroDiasVencimento(dtVencimento, dtEmissao).toString();
    };

    /* ************************* Lancamento Entrada - Dados Adic ***************************** */

    this.actNovoDadosAdicionaisOnClick = function(dadosAdicArgs) {
        dadosAdicArgs.owner.widget.newRow();
        Df11000_Lancamento_Service.getProxSeqDadosAdicionais().then(nrdadoadic => {
            dadosAdicArgs.owner.widget.currentRow.NRDADOADIC = nrdadoadic;
        });
    };


    this.limpaCampoCodigoDadosAdic = function(dadosAdicArgs) {
        dadosAdicArgs.row.CDMENSAGEM = null;
    };

    this.salvaDadosAdicActionEvent = function(dadosAdicArgs) {
        if (dadosAdicArgs.owner.widget.isValid()) {
            var dadosAdicRow = dadosAdicArgs.row;
            var widgetparent = dadosAdicArgs.owner.widget.getParent();
            var widgetParentRow = widgetparent.currentRow;

            dadosAdicRow.NRLANCTONF = widgetParentRow.NRLANCTONF;
            dadosAdicRow.CDFILIAL = widgetParentRow.CDFILIAL;

            Df11000_Lancamento_Service.salvaDadosAdicionais(dadosAdicRow).then(() => {
                alteraStatusNotafiscal(widgetparent);
                dadosAdicRow.__is_new = false;

                Df11000_Lancamento_Service.setDataSourceDadosAdicionais(dadosAdicRow)
                    .then(Df11000_Lancamento_Service.atualizaLancamento)
                    .then(Df11000_Lancamento_Service.setDataSourceLancamentoNota)
                    .then(dadosAdicArgs.owner.widget.view)
                    .catch(requestError);
            });
        }
    };

    this.deletaDadosAdic = function(dadosAdicArgs) {
        ScreenService.confirmMessage(
            ScreenService.i18n('Excluir Registro?'),
            'question',
            () => {
                var dadosAdicRow = dadosAdicArgs.row;
                var dadosAdicWidget = dadosAdicArgs.owner.widget;
                Df11000_Lancamento_Service.excluiDadosAdicionais(dadosAdicRow)
                    .then(() => alteraStatusNotafiscal(dadosAdicWidget.getParent()))
                    .then(() => Df11000_Lancamento_Service.excluiDadosAdicionaisDataSource(dadosAdicRow.NRDADOADIC))
                    .then(Df11000_Lancamento_Service.atualizaLancamento)
                    .then(Df11000_Lancamento_Service.setDataSourceLancamentoNota)
                    .then(() => {
                        General.backOrCancel(dadosAdicArgs);
                        dadosAdicWidget.reload();
                    });
            }
        );
    };

    /* ************************* Lancamento Entrada - Relacionamento Itens ********************************* */
    this.actEditarRelacionamentoItensOnClick = function(lancamentoRelacionamentoItensArgs) {
        var lancamentoRelacItensWidget = lancamentoRelacionamentoItensArgs.owner.widget,
            parentWidget = lancamentoRelacItensWidget.getParent(),
            parentCurrentRow = parentWidget.currentRow,
            produtoImportarField = lancamentoRelacItensWidget.getField('CDARVPRODFORNXML');

        
        this.produtosImportarFieldBeforeSelectOpen(lancamentoRelacItensWidget.getField('CDARVPRODFORNXML'));

        General.edit(lancamentoRelacItensWidget);
    };

    this.actVoltarRelacionamentoItensList = function(lancamentoRelacionamentoItensArgs) {
        General.backOrCancel(lancamentoRelacionamentoItensArgs);
    };

    this.actExcluirRelacionamentoItem = function(lancamentoRelacionamentoItensArgs) {
        var lancamentoRelacItensWidget = lancamentoRelacionamentoItensArgs.owner.widget,
            currentRow = lancamentoRelacItensWidget.currentRow,
            cdprodutoxml = currentRow.CDPRODUTOINTG,
            cdforneced = lancamentoRelacItensWidget.getParent().currentRow.CDFORNECED,
            cdfilial = lancamentoRelacItensWidget.getParent().currentRow.CDFILIRECE,
            cdproduto  = currentRow. CDPRODFORNXML;

        ScreenService.confirmMessage(
            'Deseja remover o relacionamento?',
            'question',
            () => {
                Dfe11000_LancamentoEntrada_Service.removerRelacionamentoItemXml(cdprodutoxml, cdforneced, cdfilial, cdproduto)
                    .then(() => {
                        currentRow.CDFORNECED = null;
                        currentRow.CDARVPRODFORNXML = null;
                        currentRow.NMPRODUTOFORNXML = null;
                        currentRow.CDPRODFORNXML = null;
                        lancamentoRelacItensWidget.list();
                        lancamentoRelacItensWidget.reload();
                        lancamentoRelacItensWidget.view();
                    })
                    .catch(requestError);
            }
        );
    };

    this.actCancelarRelacionamentoItens = function(lancamentoRelacionamentoItensArgs) {
        General.backOrCancel(lancamentoRelacionamentoItensArgs);
    };

    this.actSalvarRelacionamentoItens = function(lancamentoRelacItensWidget) {
        if(lancamentoRelacItensWidget.isValid()) {
            let {currentRow} = lancamentoRelacItensWidget;
    
            let parentRow = lancamentoRelacItensWidget.getParent().currentRow; 
    
            const {CDPRODFORNXML, CDPRODUTOINTG} = currentRow;
    
            if (!parentRow.CDFORNECED) {
                return ScreenService.showMessage('Fornecedor não encontrado.');
            }
    
            currentRow.CDFORNECED   = parentRow.CDFORNECED;
            currentRow.CDPRODUTOXML = parentRow.CDPRODUTOINTG;
            currentRow.CDFILIAL     = parentRow.CDFILIRECE;
            currentRow.CDPRODUTOXML_BACKUP_ROW = lancamentoRelacItensWidget.backupRow.CDPRODFORNXML;
    
            if (isAFRequired(parentRow, true) && importOnlyAFProducts(parentRow)) {
                const produtoRelacionado = lancamentoRelacItensWidget.dataSource.data
                    .findIndex(item =>
                        (item.CDPRODFORNXML === CDPRODFORNXML) &&
                        (item.CDPRODUTOINTG !==  CDPRODUTOINTG));
    
                if (produtoRelacionado !== -1)
                    return showMessageDfe('PRODUTO_JA_RELACIONADO');
            }
    
            return Dfe11000_LancamentoEntrada_Service.salvaRelacionamentoItensXml(currentRow)
                .then(() =>  {
                    lancamentoRelacItensWidget.list();
                    lancamentoRelacItensWidget.reload();
                    lancamentoRelacItensWidget.view();
                })
                .catch(requestError);
        }
    };

    this.actProdutosAfRelacionamentoItensOnClick = function(lancamentoRelacionamentoItensArgs) {
        var lancamentoRelacItensWidget = lancamentoRelacionamentoItensArgs.owner.widget,
            itensProgAfWidget = lancamentoRelacItensWidget.container.getWidget('itensProgAf');

        itensProgAfWidget.reload();
        ScreenService.openPopup(itensProgAfWidget);
    };

    var createDataSourceFilterRelacionamentoItens = function(lancamentoRelacItensWidget, currentRowParentWidget) {

        lancamentoRelacItensWidget.dataSourceFilter = [];
        var dataSourceFilter = lancamentoRelacItensWidget.dataSourceFilter,
            filterDefault = {
                'operator': '='
            };

        if (currentRowParentWidget.NRACESSONFE) {
            filterDefault.name = 'NRACESSONFE';
            filterDefault.value = currentRowParentWidget.NRACESSONFE;
            dataSourceFilter.push(Util.clone(filterDefault));

            filterDefault.name = 'CDFORNECED';
            filterDefault.value = currentRowParentWidget.CDFORNECED;
            dataSourceFilter.push(Util.clone(filterDefault));
            
            filterDefault.name = 'CDFILIAL';
            filterDefault.value = currentRowParentWidget.CDFILIRECE;
            dataSourceFilter.push(Util.clone(filterDefault));
        }
    };

    var createDataSourceFilterItensProgAf = function(itensProgAfWidget, currentRowParentWidget) {

        itensProgAfWidget.dataSourceFilter = [];

        var dataSourceFilter = itensProgAfWidget.dataSourceFilter,
            filterDefault = {
                'operator': '='
            };

        if ('AF' in currentRowParentWidget) {
            filterDefault.name = 'AF';
            filterDefault.value = currentRowParentWidget.AF;
            dataSourceFilter.push(Util.clone(filterDefault));
        }
    };

    const getItensNaoRelacionados = (itensPrincipais, itensSec, compare) => itensPrincipais.filter(itemXml => !itensSec.some(compare(itemXml)));

    var reloadRelacionamentoItens = function(itensProgAfWidget, lancamentoRelacItensWidget, cabecalhoRow) {
        var reloadAndCleanItensXml = function reloadAndCleanItensXml() {
            var itensAF = [];
            return itensProgAfWidget.reload()
                .then(itensProgAf => {
                    itensAF = itensProgAf.dataset.grupo_itens_afs;

                    return lancamentoRelacItensWidget.reload();
                })
                .then(relacItensXml => {

                    var itensXml = ( relacItensXml.dataset.produtos_xml ) ? relacItensXml.dataset.produtos_xml : relacItensXml.dataset.produtos_xml_filial,
                        compare = function compare(item) {
                            return function(itemAf) {
                                return item.CDPRODFORNXML === itemAf.CDPRODUTO;
                            };
                        };

                    var itensNaoRelacionados = getItensNaoRelacionados(itensXml, itensAF, compare);

                    if (itensAF.length && itensNaoRelacionados.length) {
                        let message = 'AF_OBRIGATORIA_E_INSUFICIENTE';

                        showMessageDfe(message);
                    }

                    itensNaoRelacionados.forEach(item => {
                        item.NMPRODUTOFORNXML = null;
                        item.CDARVPRODFORNXML = null;
                        item.CDFORNECED = null;
                        item.CDPRODFORNXML = null;
                        item.CDPRODUTOXML = null;
                        item.NMPRODUTOFORNXML = null;
                    });

                    return itensXml;
                })
                .catch(error => {
                    itensProgAfWidget.dataSource.data = [];
                    lancamentoRelacItensWidget.dataSource.data = [];
                    requestError(error);
                })
            ;
        };

        var onlyReloadWidgets = function onlyReloadWidgets() {

            if (lancamentoPossuiAf) {
                itensProgAfWidget.reload();
            }

            return lancamentoRelacItensWidget.reload().then(relacItensXml => {
                var itensXml = relacItensXml.dataset.produtos_xml;
                return itensXml;
            });
        };

        var parentCurrentRow = lancamentoRelacItensWidget.getParent().currentRow,
            useOnlyAFProducts = isAFRequired(parentCurrentRow) && importOnlyAFProducts(parentCurrentRow),
            lancamentoPossuiAf = ('AF' in parentCurrentRow);

        return useOnlyAFProducts ? reloadAndCleanItensXml() : onlyReloadWidgets();
    };

    var createDataSourceRelacionamentoItens = function(lancamentoRelacItensWidget, itensProgAfWidget, cabecalhoCurrentRow) {
        createDataSourceFilterItensProgAf(itensProgAfWidget, cabecalhoCurrentRow);
        createDataSourceFilterRelacionamentoItens(lancamentoRelacItensWidget, cabecalhoCurrentRow);
    };
    
    this.relacionamentoItensWidgetOnMoveRow = function(lancamentoRelacionamentoItensArgs){
          let lancamentoRelacItensWidget = lancamentoRelacionamentoItensArgs.owner,
              parentCurrentRow = lancamentoRelacItensWidget.getParent().currentRow,
              permiteAlterar = (parentCurrentRow.IDBLOQALTPROXML !== 'S' && parentCurrentRow.IDBLOQALTPRODREL !== 'S');
          
        lancamentoRelacItensWidget.getAction('actEditarRelacionamentoItens').isVisible = permiteAlterar;
        lancamentoRelacItensWidget.getAction('actExcluirRelacionamentoItens').isVisible = permiteAlterar;
          
    };

    this.relacionamentoItensWidgetOnEnter = function(lancamentoRelacionamentoItensArgs) {
        var lancamentoRelacItensWidget = lancamentoRelacionamentoItensArgs.owner,
            parentWidget = lancamentoRelacItensWidget.getParent(),
            parentCurrentRow = parentWidget.currentRow,
            afsSelecionadas = ('AF' in parentCurrentRow),
            visibleActionCabecalhoEdit = parentWidget.mode !== 'edit',
            itensProgAfWidget = lancamentoRelacItensWidget.container.getWidget('itensProgAf');

        lancamentoRelacItensWidget.getAction('actVoltarRelacionamentoItensList').isVisible = visibleActionCabecalhoEdit;
        lancamentoRelacItensWidget.getAction('actProdutosAfRelacionamentoItens').isVisible = afsSelecionadas;

        createDataSourceRelacionamentoItens(lancamentoRelacItensWidget, itensProgAfWidget, parentCurrentRow);

        reloadRelacionamentoItens(itensProgAfWidget, lancamentoRelacItensWidget, lancamentoRelacItensWidget.getParent().currentRow)
            .then(() => {
                if (isAFRequired(parentCurrentRow) && importOnlyAFProducts(parentCurrentRow) && !afsSelecionadas)
                    showMessageDfe('AF_OBRIGATORIA_E_NENHUMA_SELECIONADA');
            });
            
    };

    var criaDataSourceCdarvprodfornxmlTodosProdutos = function(cdarvprodfornxmlField, eFilialOl, cdforneced) {
        var dataSource = cdarvprodfornxmlField.dataSource;
        cdarvprodfornxmlField.dataSourceFilter = [];
        var cabecalho = templateManager.container.getWidget('cabecalho');
        dataSource.name = eFilialOl ? 'df_entrada#/produtos_relacionados_ol' : 'df#/produtoImportarLancamento';

        if (!eFilialOl) {
            cdarvprodfornxmlField.dataSourceFilter = [
                {
                    'name': 'CDFILIAL',
                    'operator': '=',
                    'value': cabecalho.currentRow.CDFILIRECE
                },
                {
                    'name': 'IDPRODATIVPR',
                    'operator': '=',
                    'value': 'S'
                },
                {
                    'name': 'IDPRODATIV',
                    'operator': '=',
                    'value': 'A'
                },
                {
                    'name': 'CDNVPRODUTO',
                    'operator': '=',
                    'value': self.nivelCompletoProduto()
                },
                {
                    'name': 'CDFORNECED',
                    'operator': '=',
                    'value': cdforneced
                },
                {
                    'name': 'NRAUTOFORN',
                    'operator': '=',
                    'value': (cabecalho.currentRow.AF) ? self.buildInstrAf(cabecalho.currentRow.AF) : ''
                },
                {
                    "name": "DTPROGENTAF",
                    "operator": "=",
                    "value": (cabecalho.currentRow.AF) ? cabecalho.currentRow.AF[0].DTPROGENTAF : null
                } 
                
            ];
        }
        else {
            cdarvprodfornxmlField.dataSourceFilter = [
                {
                    'name': 'CDFILIAL',
                    'operator': '=',
                    'value': cabecalho.currentRow.CDFILIRECE
                },
                {
                    'name': 'CDFORNECED',
                    'operator': '=',
                    'value': cdforneced
                },
                {
                    'name': 'NRAUTOFORN',
                    'operator': '=',
                    'value': (cabecalho.currentRow.AF) ? self.buildInstrAf(cabecalho.currentRow.AF) : ''
                }
            ];
        }
    };
    
    this.buildInstrAf = function(AF) {
        let instrAF = '_';
        if(Array.isArray(AF)) {
            AF.forEach((af) => {
                instrAF += af.NRAUTOFORN + '_'; 
            });
        }  
        return instrAF;
    };

    this.produtosImportarFieldBeforeSelectOpen = function(field) {
        const {widget} = field, AFWidget = widget.container.getWidget('itensProgAf');

        const isFilialOL   = widget.getParent().currentRow.IDFUNCAOFILI  === 'O',
            AFProductsOnly = isAFRequired(widget.getParent().currentRow) && importOnlyAFProducts(widget.getParent().currentRow);

        criaDataSourceCdarvprodfornxmlTodosProdutos(field, isFilialOL, widget.getParent().currentRow.CDFORNECED);

        field.dataSourceFilter = (AFProductsOnly)?
            getProdutosImportarFilterByAF(widget, AFWidget, field.dataSourceFilter)
            : getProdutosImportarFilterAllProducts(field.dataSourceFilter);

        field.reload();
    };

    const getProdutosImportarFilterByAF = (widget, AFWidget, previousFilter) => {
        const AFItems = AFWidget.dataSource.data.map(item => item.CDPRODUTO),
            XMLItems = widget.dataSource.data.map(item => item.CDPRODFORNXML);
            
        let products = [];
        var item = [];

        AFItems.forEach((itemAf) => {
            item = [];
            item = XMLItems.filter(( xml ) => {
                return itemAf == xml; 
            });
            
            if( item.length == 0 ) {
                products.push(itemAf);
            }
        });

        const productFilter = previousFilter.find(filter => filter.name == 'CDPRODUTO');

        if (productFilter) {
            productFilter.operator = 'IN';
            productFilter.value = products;
        } else
            previousFilter.push({name: 'CDPRODUTO', operator: 'IN', value: products});

        return previousFilter;
    };

    const getProdutosImportarFilterAllProducts = previousFilter => previousFilter.filter(filter => filter.name !== 'CDPRODUTO');

    var validacaoRelacionamentoItens = function(cabecalhoWidget) {
        var lancamentoRelacItensWidget = cabecalhoWidget.container.getWidget('relacionamentoItens'),
            itensProgAfWidget   = cabecalhoWidget.container.getWidget('itensProgAf'),
            currentRowCabecalho = cabecalhoWidget.currentRow,
            possuiChaveAcesso   = !!currentRowCabecalho.NRACESSONFE,
            deferred = ZHPromise.defer(),
            isNew = currentRowCabecalho.__is_new;
            
        createDataSourceRelacionamentoItens(lancamentoRelacItensWidget, itensProgAfWidget, currentRowCabecalho);
        
        if (possuiChaveAcesso && isNew) {
            var reload = reloadRelacionamentoItens(itensProgAfWidget, lancamentoRelacItensWidget);
            reload.then(itensRelacionados => {
                const produtoNaoRelacionado = Array.isArray(itensRelacionados)
                    && itensRelacionados.some(item => !item.CDPRODUTOXML);

                if (produtoNaoRelacionado) {
                    deferred.reject('EXISTE_PRODUTOS_NAO_RELACIONADO');
                    lancamentoRelacItensWidget.activate();
                } else {
                    deferred.resolve();
                }
            });
        } else {
            deferred.resolve();
        }
        
        return deferred.promise;
    };

    this.actConfirmarNotaRelacionamentoItens = function(lancamentoRelacionamentoItensArgs) {
        var lancamentoRelacItensWidget = lancamentoRelacionamentoItensArgs.owner.widget,
            cabecalhoWidget = lancamentoRelacItensWidget.getParent();
        var dataSource = lancamentoRelacItensWidget.dataSource.data;
        
        if(validaProdutosRelacionados(dataSource)){
            return showMessageDfe("EXISTE_PRODUTOS_NAO_RELACIONADO");
        } else{
            var datasourceProduto = dataSource[0];
            realizaSalvamentoCabecalhoNotaFiscal(cabecalhoWidget, datasourceProduto);
        }
    };
    
    var validaProdutosRelacionados = function(dataSource){
        var produtoNaoRelacionado = false;
        dataSource.forEach((produto) => {
           if(produto.CDARVPRODFORNXML === '' || produto.CDARVPRODFORNXML === null) produtoNaoRelacionado = true;
        });
        
        return produtoNaoRelacionado;
    };
    
    this.getCurrentField = function(field) {
        var cabecalhoWidget = getCabecalhoWidget();
        if (cabecalhoWidget)
             return !cabecalhoWidget.currentRow[field] ? null : cabecalhoWidget.currentRow[field];
        return null;
    };
    
    this.getFieldFilter = function(fieldName, args) {
        if(args) console.log(args.owner);  
    };
    
    this.bloqueiaEdicaoImpostos = function(cabecalhoArgs){
        var rowNota = cabecalhoArgs.row;
        var impostosItens = cabecalhoArgs.owner.container.getWidget('impostosItem');
        
        if(rowNota.IDSTATUS === 'D' || rowNota.IDSTATUS === 'P'){
            impostosItens.fields.forEach( row => {
                if(row.name === 'VRALIQIMPOIT' || row.name === 'VRIMPOSTOIT')
                     row.inlineEditable = "number";
            });
        }else{
            impostosItens.fields.forEach(row => {
                delete(row.inlineEditable);
            });
        }
    };
    
    this.setDefaultNovaNota = function (args) {
        var row = {CDFILIAL: args.e.preference.FILTER_ROW.CDFILIRECE};
        General.callRestEngine("df_entrada#/getDefaultNovaNota", "Row", row).then((success) => {
            Df11000_Lancamento_Service.setDefaultNovaNota(success.dataset.defaultNovaNota);
        });
    };
    
    this.generateReportConsistencia = function() {
        var row = templateManager.container.getWidget('cabecalho').currentRow;
        var rowReport = {
            CDFILIAL: row.CDFILIAL,
            CHAVE: [row.CDFILIAL+row.NRLANCTONF],
            DTINTERVALO: {start: row.DTENTRSAID.substr(0, 10), end: row.DTENTRSAID.substr(0, 10)},
            L_CDFILIRECE: '_'+row.CDFILIRECE+'_',
            L_CHAVE: '_'+row.CDFILIAL+row.NRLANCTONF+'_',
            NMFILIAL: row.NMFILIAL, 
            NRNOTAFISC: [row.NRNOTAFISC],
            P_DTFINAL: row.DTENTRSAID.substr(0, 10),
            P_DTINICIAL: row.DTENTRSAID.substr(0, 10),
            T_CDFILIAL: "L",
            NMRELATORIO: 'DFE16000',
            __report_name: "DFE16000"
        };
        
        General.callRestEngine("df_entrada#/notafiscal_consistencias_af_nf", "Row", rowReport).then( (response) => {
            window.open(response.messages[0].message);
        });
    };
    
    this.searchForAccessKey = function(widget){
        let valfDTEMISSAO    = widget.getField('DTEMISSAO').value();
        let valfCDSERIEENTR  = widget.getField('CDSERIEENTR').value();
        let valfNRNOTAFISC   = widget.getField('NRNOTAFISC').value();
        let valfNMRAZSOCFORN = widget.getField('NMRAZSOCFORN').value();
        let nracessonfe      = widget.getField('NRACESSONFE').value();
        if( valfDTEMISSAO && 
           valfCDSERIEENTR && valfNRNOTAFISC && valfNMRAZSOCFORN && !nracessonfe){
                let params = {};
                params.DTEMISSAO    = valfDTEMISSAO;  
                params.CDSERIEENTR  = widget.currentRow.CDSERIEENTR;  
                params.NRNOTAFISC   = valfNRNOTAFISC;  
                params.NRINSJURNF   = widget.currentRow.NRINSJURNF;
                fetchAccessKeyByOtherFields(widget, params);
                widget.getField('NRACESSONFE').value(); //ScreenService.showMessage('SUCESSO!'); 
        }
    };

    this.abrirPopupBuscarAf = function(widget) {
        var buscarAf = templateManager.container.getWidget('NOVA_AF');
        ScreenService.openPopup(buscarAf).then(() => {
            buscarAf.activate();
            buscarAf.isVisible = true;
        });
    };
    
    this.buscarAfPorCodigo = function(widget) {
        if(widget.isValid()) {
            let cabecalho   = templateManager.container.getWidget('cabecalho');
            let autoforne   = templateManager.container.getWidget('autorizacaoFornecimento');
            let cabecalhoRow = cabecalho.currentRow;
            let dtminvazia  = cabecalhoRow.DTMINVAZIA;
            let dtmaxvazia  = cabecalhoRow.DTMAXVAZIA;
            let dtprocini   = cabecalhoRow.DTPROCINI;
            let dtprocfin   = cabecalhoRow.DTPROCFIN;
            let filter = [
                {
                    "name": "P_CDFILIAL",
                    "operator": "=",
                    "value": cabecalho.currentRow.CDFILIRECE
                },
                {
                    "name": "P_CDFORNECED",
                    "operator": "=",
                    "value": "T"
                },
                {
                    "name": "P_DTPROCINI",
                    "operator": "=",
                    "value": dtprocini
                },
                  {
                    "name": "P_DTMINVAZIA",
                    "operator": "=",
                    "value": dtminvazia
                },
                {
                    "name": "P_DTPROCFIN",
                    "operator": "=",
                    "value": dtprocfin
                },
                {
                    "name": "P_DTMAXVAZIA",
                    "operator": "=",
                    "value": dtmaxvazia
                },
                {
                    "name": "NRAUTOFORN",
                    "operator": "=",
                    "value": widget.currentRow.NRAUTOFORN
                }
                
            ];
            
            General.callRestEngine("df_entrada#/autoforne_itemautf", "FilterData", filter).then((success) => {
                if(success.dataset.autoforne_itemautf[0]) {
                    //Tira os elementos repetidos
                    autoforne.dataSource.data = autoforne.dataSource.data.filter( data => {
                        return data.NRAUTOFORN !== success.dataset.autoforne_itemautf[0].NRAUTOFORN || data.NRINSJURFORN !== success.dataset.autoforne_itemautf[0].NRINSJURFORN;
                    });
                    autoforne.dataSource.data.unshift(success.dataset.autoforne_itemautf[0]);
                    ScreenService.closePopup(widget);
                    ScreenService.successNotification("Autorização de Fornecedor encontrada.");

                }
                else {
                    ScreenService.errorNotification("Autorização de Fornecimento não encontrada.");
                }
            }, (error) => {
                ScreenService.errorNotification("Ocorreu um erro ao buscar a autorização de fornecimento");
            });  
        }
    };
    
    this.closePopupBuscarAf = function(widget) {
        ScreenService.closePopup(widget);
    };
    

    /***************************** Anexos *********************************************/
    
    this.inicializaAnexo = function(args){
        var filter = [
            {
                "name": "P_CDFILIAL",
                "operator": "=",
                "value": args.owner.getParent().currentRow.CDFILIAL
            },
            {
                "name": "P_NRLANCTONF",
                "operator": "=",
                "value": args.owner.getParent().currentRow.NRLANCTONF
            }
            
        ];
        
        General.callRestEngine("df#/crud/anexo_obrigatorio", "FilterData", filter).then((success) => {
            if(success.dataset.anexo_obrigatorio[0].IDBLOQANEXO) {
                if(success.dataset.anexo_obrigatorio[0].IDBLOQANEXO == 0) {
                    args.owner.getAction("actNovoAnexo").isVisible = false;
                    args.owner.getAction("actDeleteAnexo").isVisible = false;
                }
                else if(success.dataset.anexo_obrigatorio[0].IDBLOQANEXO == 1 && args.owner.getParent().currentRow.IDSTATUS == 'A') {
                    args.owner.getAction("actNovoAnexo").isVisible = false;     
                    args.owner.getAction("actDeleteAnexo").isVisible = false;
                }                
                else if(success.dataset.anexo_obrigatorio[0].IDBLOQANEXO == 2 && (args.owner.getParent().currentRow.IDBLOQNFFIN == 'L')) {
                    args.owner.getAction("actNovoAnexo").isVisible = false;     
                    args.owner.getAction("actDeleteAnexo").isVisible = false;
                }
                else {
                    args.owner.getAction("actNovoAnexo").isVisible = true;    
                    args.owner.getAction("actDeleteAnexo").isVisible = true;
                }
            }
        });
        args.owner.reload();
    };
    
    
    this.downloadArqNF = function(widget){
        var row = widget.currentRow;

        window.open(row.link);
    };
    
    this.salvarAnexo = function(widget){
        if(widget.isValid()){
            var row = widget.currentRow;
            var widgetReload = widget.parent.parent;
            row.CDFILIAL = widgetReload.currentRow.CDFILIAL;
            row.NRLANCTONF = widgetReload.currentRow.NRLANCTONF;
            row = Util.clone(widget.currentRow, ['__rightActions', '__leftActions']);
            General.callRestEngine('df_entrada#/salvar_Anexo', "Row", row).then(function() {
                widget.reload();
                ScreenService.closeSwipe();
                widgetReload.resizeGridColumns(true);
            }.bind(this), this.errorRequest);
        }
    };
    
    const atualizarVrItemProduto = function (widget) {
       let row = Util.clone(widget.currentRow);
        General.callRestEngine("df_entrada#/atualizaVrItemProduto", "Row", row).then((success) => {
             ScreenService.showMessage("Valores dos produtos atualizados com sucesso");  
        }, (error) => {
                ScreenService.showMessage(error.data.error);
            });  
    };
    
    /* ************************** Lancamento Entrada - Controle de Lançamento Retroativo ************************** */
    
    this.cancelaContrLancRetro = function(args){
        ScreenService.closePopup(args.owner.widget);  
    };
    
    this.openContrLancRetro = function(args){
        ScreenService.openPopup(templateManager.container.getWidget("controleLancRetroativo"));  
    };
    
    this.gerarSenha = function(args){
        let widget = args.owner.widget;
        
        if(widget.isValid()){
            Dfe11000_LancamentoEntrada_Service.geraSenhaLancRetroativo(widget.currentRow).then( response => {
                widget.currentRow.SENHAGERADA = response.dataset.senha.SENHAGERADA;
            });
        }
    };

    /* ************************** Lancamento Entrada - Lançamento Retroativo ************************** */
    
    this.openLancRetro = function(args){
        let widgetLancRetro = templateManager.container.getWidget("lanctoRetroativo"),
        rowLancamento = args.owner.widget.currentRow;
        
        ScreenService.openPopup(widgetLancRetro);
        widgetLancRetro.currentRow.CDFILIAL = rowLancamento.CDFILIAL;
        widgetLancRetro.currentRow.NRLANCTO = rowLancamento.NRLANCTONF;
    };
    
    this.cancelaLancRetro = function(args){
        ScreenService.closePopup(args.owner.widget);  
    };
    
    this.validaSenha = function(args){
        let widget = args.owner.widget,
        currentRow = widget.currentRow;
        if(widget.isValid()){
            Dfe11000_LancamentoEntrada_Service.geraSenhaLancRetroativo(currentRow).then( response => {
                if(response.dataset.senha.SENHAGERADA == currentRow.SENHALANCTO){
                    ScreenService.closePopup(widget);
                    this.validaEdicaoCabecalho(widget);
                    this.habilitaEdicaoNotaEntrada(templateManager.container.getActiveWidget());
                } else{
                    ScreenService.showMessage("Senha inválida");
                }
            });
        }
    };
    
    /*	
    * Tipo de Operação 11	
    **/	
    this.habilitaTabProdutosOuNotas = function(widget) {	
        templateManager.container.getWidget('entrada_semnf').isVisible = (widget.currentRow.CDTIPOOPER == '11');	
        templateManager.container.getWidget('entrada_semnf').showAsTab = (widget.currentRow.CDTIPOOPER == '11');	
    }; 	
    	
    this.actNovaEntradaSemNota = function(widget) {	
        let entradaSemNota = templateManager.container.getWidget('popup_entradaSemNf');	
        entradaSemNota.activate();	
        entradaSemNota.isVisible = true;	
        ScreenService.openPopup(entradaSemNota).then(() => {	
            entradaSemNota.reload();	
        });	
    };	
    	
    this.actClosePopupEntradaSemnota = function(widget) {	
        let entradaSemNota = templateManager.container.getWidget('popup_entradaSemNf');	
        ScreenService.closePopup();	
        entradaSemNota.isVisible = false;	
        templateManager.container.getWidget('entrada_semnf').activate();	
        	
    };	
    	
    	
    this.validaActionsEntradaSemNota = function(widget) {	
        let idStatus = widget.getParent().currentRow.IDSTATUS;	
        if(idStatus != 'A') {	
            if(idStatus == 'P') widget.getAction('actFecharNotaSNF').isVisible = true;	
            else widget.getAction('actFecharNotaSNF').isVisible = false;	
            	
            widget.getAction('actNovoEntradaSNF').isVisible = true;	
            widget.getAction('actFecharNotaSNF').isVisible = true;	
        }	
        else {	
            widget.getAction('actFecharNotaSNF').isVisible = false;	
            widget.getAction('actNovoEntradaSNF').isVisible = false;            	
        }	
    };	
    	
    	
    this.actSalvaEntradaSemNota = function(widget) {	
        var itens = widget.getCheckedRows();	
        var cabecalho = templateManager.container.getWidget('cabecalho').currentRow;
        var rows = {
                cabecalho : cabecalho,
                itens : itens
        };
        General.callRestEngine("df#/salvarNotaOperacaoOnze", "DataSet", rows).then(() => {	
            this.actClosePopupEntradaSemnota();	
            templateManager.container.getWidget('entrada_semnf').reload();	
            removeDataSourcesConsultaLancEntradaActionEvent();	
            criaDetalhesLancamentoWidgetAfterMoveRow(cabecalho);
            ScreenService.successNotification('Notas Fiscais relacionadas com sucesso.');	
            alteraStatusNotafiscal(templateManager.container.getWidget('cabecalho'));	
        }, (error) => {	
            ScreenService.errorRequest("Ocorreu um erro ao relacionar as notas.");	
        });	
    };	
    	
    this.actExcluirEntradaSemNota = function(widget) {	
        var row = widget.currentRow;	
        General.callRestEngine("df#/excluirRelacionamentoOpOnze", "Row", row).then(() => {	
            this.actClosePopupEntradaSemnota();	
            templateManager.container.getWidget('entrada_semnf').reload();	
            removeDataSourcesConsultaLancEntradaActionEvent();	
            criaDetalhesLancamentoWidgetAfterMoveRow(row);	
            ScreenService.successNotification('Exclusão realizada com sucesso.');	
        });	
    };
    
    this.sgEstadoNotaOnChange = function(widget) {
        widget.getField('NMMUNICIPIONOTA').readOnly = widget.currentRow.SGESTADONOTA ? false : true;
        widget.currentRow.CDMUNICIPIO = null;
        widget.currentRow.CDMUNICIPIONOTA = null;
        widget.currentRow.NMMUNICIPIONOTA = null;
        widget.currentRow.SGESTADO = widget.currentRow.SGESTADONOTA;
        if (widget.currentRow.SGESTADONOTA) {
    		widget.getField('NMMUNICIPIONOTA').dataSourceFilter.forEach(data => {
    		    if(data.name == "SGESTADO") data.value = widget.currentRow.SGESTADONOTA;
    		});
        }
    };
    
    this.nmMunicipioNotaOnChange = function(widget) {
        widget.currentRow.CDMUNICIPIO = widget.currentRow.CDMUNICIPIONOTA;
   };
    
    this.controlVisibilityLocalPrestServ = function(widget) {
        let rowCDTIPOOPER = widget.currentRow.CDTIPOOPER;
        let fieldSGESTADONOTA = widget.getField('SGESTADONOTA');
        let fieldNMMUNICIPIONOTA = widget.getField('NMMUNICIPIONOTA');
        let dsNMTIPOOPER = widget.getField('NMTIPOOPER').dataSource.data;
        let utilizaLocalPrestServ = false;
        
        for(let data in dsNMTIPOOPER) {
            if(rowCDTIPOOPER === dsNMTIPOOPER[data].CDTIPOOPER) {
                if(dsNMTIPOOPER[data].IDUTLOCPRESTSERV === 'S') {
                    utilizaLocalPrestServ = true;
                }
                break;
            }
        }
        if(widget.editing) { //True = Add e edit nota.
            if(utilizaLocalPrestServ) {
                if(widget.currentRow.__is_new) {
                    widget.currentRow.CDMUNICIPIONOTA = null;
                    widget.currentRow.NMMUNICIPIONOTA = null;
                    widget.currentRow.SGESTADONOTA = null;
                    widget.currentRow.NMESTADONOTA = null;
                } else {
                    if (widget.currentRow.SGESTADONOTA) {
                        widget.getField('NMMUNICIPIONOTA').dataSourceFilter.forEach(data => {
                		    if(data.name == "SGESTADO") data.value = widget.currentRow.SGESTADONOTA;
                		});
                    }
                }
                widget.fieldGroups.forEach(group => {
                    if(group.name === 'prestacaoServico') {
                        group.isVisible = true;
                        group.opened = true;
                    }
                });
                widget.currentRow.SGESTADO = widget.currentRow.SGESTADONOTA;
                widget.currentRow.CDMUNICIPIO = widget.currentRow.CDMUNICIPIONOTA;
                fieldSGESTADONOTA.isVisible = true;
                fieldSGESTADONOTA.readOnly = false;
                fieldSGESTADONOTA.validations.required = true;
                fieldNMMUNICIPIONOTA.isVisible = true;
                fieldNMMUNICIPIONOTA.readOnly = widget.currentRow.SGESTADONOTA ? false : true;
                fieldNMMUNICIPIONOTA.validations.required = true;
            } else {
                widget.fieldGroups.forEach(group => {
                    if(group.name === 'prestacaoServico') {
                        group.isVisible = false;
                        group.opened = false;
                    }
                });
                fieldSGESTADONOTA.isVisible = false;
                fieldSGESTADONOTA.readOnly = true;
                fieldSGESTADONOTA.validations.required = false;
                fieldNMMUNICIPIONOTA.isVisible = false;
                fieldNMMUNICIPIONOTA.readOnly = true;
                fieldNMMUNICIPIONOTA.validations.required = false;
            }
        } else { //False = Visualizar nota.
            if(utilizaLocalPrestServ) {
                widget.fieldGroups.forEach(group => {
                    if(group.name === 'prestacaoServico') {
                        group.isVisible = true;
                    }
                });
                fieldSGESTADONOTA.isVisible = true;
                fieldNMMUNICIPIONOTA.isVisible = true;
            } else {
                widget.fieldGroups.forEach(group => {
                    if(group.name === 'prestacaoServico') {
                        group.isVisible = false;
                    }
                });
                fieldSGESTADONOTA.isVisible = false;
                fieldNMMUNICIPIONOTA.isVisible = false;
            }
        }
   };
   
   this.openWindowDocumentosComposicaoCustos = function() {
        MessageService.showMessageByCode('ROTINA_EM_IMPLEMENTACAO').then( function() {
            ScreenService.openWindow('df_entrada#dfe18500_documentos_para_composicao_de_custos');
        });
   };

};
    
Configuration(function(ContextRegister) {
    ContextRegister.register('Dfe11000_LancamentoEntrada_Controller', Dfe11000_LancamentoEntrada_Controller);
});

// FILE: ./assets/modules/entrada/assets/js/controllers/Dfe18500_DocumentosParaComposicaoDeCustos_Controller.js
var Dfe18500_DocumentosParaComposicaoDeCustos_Controller = function(General, ScreenService, templateManager, ZhFilterPreferences, ZHPromise, metaDataFactory, eventEngine, MessageService) {
    
    var widgetCabecalhoEntrada;
    
    this.fieldFilterOnChange = function() {
        var widget = templateManager.container.getActiveWidget();
        
        widget.reload();
    };

    this.setCabecalhoWidgetEntrada = function(widget) {
        widgetCabecalhoEntrada =  widget;
    };
    
    this.getCabecalhoWidgetEntrada = function() {
        return widgetCabecalhoEntrada;
    };
    
    this.getFieldValueNotaFiscalOrigem = function(name) {
        if (widgetCabecalhoEntrada && widgetCabecalhoEntrada.currentRow) 
			return widgetCabecalhoEntrada.currentRow[name];
		else 
			return "";
		
    };
    
    this.salvaWidgetOnSaveInlineEdit = function(args) {
        General.callRestEngine("df_entrada#/salvaRateioIt", "Row",  args.row).then((response) => {
            ScreenService.showMenssage("Salvo!");
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('Dfe18500_DocumentosParaComposicaoDeCustos_Controller', Dfe18500_DocumentosParaComposicaoDeCustos_Controller);
});

// FILE: ./assets/modules/entrada/assets/js/controllers/Dfe31000_ParametrosDocumentosFiscaisEntrada_Controller.js
var  Dfe31000_ParametrosDocumentosFiscaisEntrada_Controller = function(ScreenService, templateManager,General){

      this.verficaUtilizaAlmoxarifado = function(args){
        var gridParametrosFilial =  templateManager.container.getWidget('parametrosFilial');
        if(gridParametrosFilial.currentRow.IDUTILALLONF=='S'){
            if(gridParametrosFilial.currentRow.IDUTILALMOX=='N'){
                ScreenService.showMessage('Unidade Não Utiliza Almoxarifado');
                gridParametrosFilial.getField('IDUTILALLONF').value('N');
            }
        }
    };

    this.verificaInsercaoFilial = function(args){
        var tabbedParametrosFilial =  templateManager.container.getWidget('tabbedParametrosFilial');
        var gridParametrosFilial =  templateManager.container.getWidget('parametrosFilial');
        var filial =tabbedParametrosFilial.getField('CDFILIAL');
        if(filial.value()){
            var dataParametros = gridParametrosFilial.dataSource.data;
            //se não exister o parametros para filial cadastrado insere uma nova linha
            if(dataParametros.length==0){
                gridParametrosFilial.currentRow={};
                gridParametrosFilial.currentRow.CDFILIAL=filial.value();
                gridParametrosFilial.currentRow.__is_new=true;
                gridParametrosFilial.dataSource.name = "df#/crud/paramfilial";
                gridParametrosFilial.saveRow().then(function() {
                    gridParametrosFilial.dataSource.name = "df_entrada#/paramfilial_entrada";
                    gridParametrosFilial.reload();
                }.bind(this), function(callback){
                    if(callback.data){
                        ScreenService.showMessage(callback.data.error);
                    }
                    gridParametrosFilial.dataSource.name = "df_entrada#/paramfilial_entrada";
                });

            }
        }
    };

    this.filtroParametrosFilial =  function(){
        var tabbedParametrosFilial =  templateManager.container.getWidget('tabbedParametrosFilial');
        var gridParametrosFilial =  templateManager.container.getWidget('parametrosFilial');
        var filial = tabbedParametrosFilial.getField('CDFILIAL').value();
        if(filial){
            gridParametrosFilial.getAction('editar').isVisible = true;
            gridParametrosFilial.reload();
        }else{
            gridParametrosFilial.getAction('editar').isVisible = false;
            for (let registro in gridParametrosFilial.currentRow) {
        		gridParametrosFilial.currentRow[registro] = '';
        	}
        }
    };

    this.verificaInsercaoEmpresa = function(args){
        var tabbedParametrosEmpresa =  templateManager.container.getWidget('tabbedParametrosEmpresa');
        var gridParametrosEmpresa =  templateManager.container.getWidget('parametrosEmpresa');
        var empresa =tabbedParametrosEmpresa.getField('CDEMPRESA');
        if(empresa.value()){
            var dataParametros = gridParametrosEmpresa.dataSource.data;
            //se não exister o parametros para empresa cadastrado insere uma nova linha
            if(dataParametros.length==0){
                gridParametrosEmpresa.currentRow={};
                gridParametrosEmpresa.currentRow.CDEMPRESA=empresa.value();
                gridParametrosEmpresa.currentRow.__is_new=true;
                gridParametrosEmpresa.dataSource.name = "df#/crud/paramempresa";
                gridParametrosEmpresa.saveRow().then(function() {
                    gridParametrosEmpresa.dataSource.name = "df_entrada#/paramempresa_entrada";
                    gridParametrosEmpresa.reload();
                }.bind(this), function(callback){
                    if(callback.data){
                        ScreenService.showMessage(callback.data.error);
                    }
                    gridParametrosEmpresa.dataSource.name = "df_entrada#/paramempresa_entrada";
                });

            }
        }
    };

    this.campoParametrosEmpresa = function(){
        var gridParametrosEmpresa =  templateManager.container.getWidget('parametrosEmpresa');
        var idt = gridParametrosEmpresa.getField('IDABATNFDNFOEN').value();
        var tipoAjusteField = gridParametrosEmpresa.getField('CDAJUCTREC');

        if(idt=="S"){
            tipoAjusteField.readOnly = false;
            tipoAjusteField.validations.required = true;
            tipoAjusteField.dataInitialized = false;
            tipoAjusteField.value("");
            gridParametrosEmpresa.getField('DSAJUCTREC').value("");
        }else{
            delete(tipoAjusteField.validations.required );
            tipoAjusteField.readOnly = true;
            tipoAjusteField.value("");
            gridParametrosEmpresa.getField('DSAJUCTREC').value("");
        }
    };

    this.filtroParametrosEmpresa =  function(){
        var tabbedParametrosEmpresa =  templateManager.container.getWidget('tabbedParametrosEmpresa');
        var gridParametrosEmpresa =  templateManager.container.getWidget('parametrosEmpresa');
        var empresa = tabbedParametrosEmpresa.getField('CDEMPRESA').value();
        if(empresa){
            gridParametrosEmpresa.getAction('editar').isVisible = true;
            gridParametrosEmpresa.reload();
        }else{
            gridParametrosEmpresa.getAction('editar').isVisible = false;
            for (let registro in gridParametrosEmpresa.currentRow) {
        		gridParametrosEmpresa.currentRow[registro] = '';
        	}
        }
    };


    this.editar = function(widget){
        if(widget.name=="parametrosGerais"){
            this.situacaoCampoLancamentoManual();
        }
        if(widget.name=='parametrosFilial'){
                var tabbedParametrosFilial =  templateManager.container.getWidget('tabbedParametrosFilial');
                tabbedParametrosFilial.getField('NMFILIAL').readOnly=true;
        }
        if(widget.name=='parametrosEmpresa'){
                var tabbedParametrosEmpresa =  templateManager.container.getWidget('tabbedParametrosEmpresa');
                tabbedParametrosEmpresa.getField('NMEMPRESA').readOnly=true;
        }
        widget.template = widget.templateToForm;
        General.edit(widget);
    };

    this.cancelar = function(widget){
     
		for (var i in widget.backupRow) {
			widget.currentRow[i] = widget.backupRow[i];
		}
		widget.template = widget.templateToView;
		widget.view();
		widget.activate();
	
        if(widget.name=='parametrosFilial'){
            var tabbedParametrosFilial =  templateManager.container.getWidget('tabbedParametrosFilial');
            tabbedParametrosFilial.getField('NMFILIAL').readOnly = false;
        }
        if(widget.name=='parametrosEmpresa'){
            var tabbedParametrosEmpresa =  templateManager.container.getWidget('tabbedParametrosEmpresa');
            tabbedParametrosEmpresa.getField('NMEMPRESA').readOnly = false;
        }

    };

    this.WidgetOnEnter = function(args){
        args.owner.currentRow = args.owner.dataSource.data[0];
    };

    this.salvarParametrosGerais = function(args){
        var Widget = args.owner.widget;
        if(Widget.currentRow.IDBLAPRLCTMAPBL != "S"){
            Widget.currentRow.IDBLAPRLCTMAPBL = "N";
        }
        if(Widget.currentRow.VRPETOLEVRAF>1000){
            ScreenService.showMessage("Percentual de Tolerância de Valores deve ser Menor que 1000%");
        }else if(Widget.currentRow.VRPETOLEQTAF>1000) {
            ScreenService.showMessage("Percentual de Tolerância de Quantidade deve ser Menor que 1000%");
        }
        else{
            General.saveCrud(Widget).then(function() {
                this.controlaStatusWidgets( templateManager.container.getWidget('container_tabbed'),'parametrosGerais',true);
                var currentRow = Widget.currentRow;
                currentRow.__is_new = false;
                currentRow.changed = false;
                Widget.template = Widget.templateToView;
                Widget.view();
                ScreenService.closeSwipe();
                Widget.reload();
                Widget.list();
            }.bind(this), function(callback){
                if(callback.data){
                    ScreenService.showMessage(callback.data.error);
                }

            });
        }
    };

    this.salvarParametrosFilial = function(args){
        var Widget = args.owner.widget;
        if(Widget.currentRow.NRTAMNUMNFUTDFE == null || Widget.currentRow.NRTAMNUMNFUTDFE == 0 || Widget.currentRow.NRTAMNUMNFUTDFE >  20){
            ScreenService.showMessage("Tamanho no Nº da Nota Fiscal deve estar entre 1 e 20");
        }
        else{
            this.concatenaCampoDsparfilfis(Widget.currentRow);
            General.saveCrud(Widget).then(function() {
                this.controlaStatusWidgets( templateManager.container.getWidget('container_tabbed'),'tabbedParametrosFilial', true);
                var tabbedParametrosFilial =  templateManager.container.getWidget('tabbedParametrosFilial');
                tabbedParametrosFilial.getField('NMFILIAL').readOnly = false;
                var currentRow = Widget.currentRow;
                currentRow.__is_new = false;
                currentRow.changed = false;
                Widget.template = Widget.templateToView;
                Widget.view();
                Widget.reload();
                Widget.list();
            }.bind(this), function(callback){
                if(callback.data){
                    ScreenService.showMessage(callback.data.error);
                }

            });
        }
    };

    this.concatenaCampoDsparfilfis = function(row){

        var campoCompleto = row.DSPARFILFIS;
        if(! row.DSPARFILFIS1){
            row.DSPARFILFIS1 =' ';
        }
        if(! row.DSPARFILFIS2){
            row.DSPARFILFIS2 =' ';
        }


        if(!campoCompleto){
            row.DSPARFILFIS = row.DSPARFILFIS1 + row.DSPARFILFIS2;
        }
        else{
            if(campoCompleto.length==2){
                row.DSPARFILFIS = row.DSPARFILFIS1 + row.DSPARFILFIS2;
            }else{
                var partefinal = campoCompleto.substr(2);
                row.DSPARFILFIS = row.DSPARFILFIS1+ row.DSPARFILFIS2 + partefinal;
            }

        }
    };
    
    this.enterInlineEditTipoOperacao = function(widget) {
        this.controlaStatusWidgets(widget.parent, "tipoOperacao", false);
    };
    
    this.exitInlineEditTipoOperacao = function(widget) {
        this.controlaStatusWidgets(widget.parent, "tipoOperacao", true);
    };
    
    this.cancelTipoOperacao = function(widget) {
        this.exitInlineEditTipoOperacao(widget);
        widget.reload();
        widget.list();
    };
    
    this.salvarTipoOperacao = function (widget) {
        var itens = widget.dataSource.data;
        var itensModificados = itens.filter(item => item.changed_IDUTILOPER || item.changed_IDCONSFISCPROD);
        
        itensModificados.forEach (item => {
            item.changed = true;
        });
        
        widget.dataSource.save().then(function () {
            itensModificados.forEach (item => {
                //@todo remover delete item.changed quando desmarcaLinhasEditadas for implementada
                delete item.changed_IDUTILOPER;
                delete item.changed_IDCONSFISCPROD;
                delete item.changed;
            });
            widget.activate();
            widget.list(true);
            ScreenService.successNotification("Parâmetros salvos com sucesso.");
        });
        this.exitInlineEditTipoOperacao(widget);
    };

    this.salvarParametrosEmpresa = function(args){
        var Widget = args.owner.widget;
        General.saveCrud(Widget).then(function() {
            this.controlaStatusWidgets( templateManager.container.getWidget('container_tabbed'),'tabbedParametrosEmpresa',true);
            var tabbedParametrosEmpresa =  templateManager.container.getWidget('tabbedParametrosEmpresa');
            tabbedParametrosEmpresa.getField('NMEMPRESA').readOnly=false;
            var currentRow = Widget.currentRow;
            currentRow.__is_new = false;
            currentRow.changed = false;
            Widget.template = Widget.templateToView;
            Widget.view();
            Widget.reload();
            Widget.list();
        }.bind(this), function(callback){
            if(callback.data){
                ScreenService.showMessage(callback.data.error);
            }

        });

    };

    this.salvarNFe = function(args){
        var Widget = args.owner.widget;
        General.saveCrud(Widget).then(function() {
            this.controlaStatusWidgets( templateManager.container.getWidget('container_tabbed'),'NFe',true);
            var currentRow = Widget.currentRow;
            currentRow.__is_new = false;
            currentRow.changed = false;
            Widget.template = Widget.templateToView;
            Widget.view();
            ScreenService.closeSwipe();
            Widget.reload();
            Widget.list();
        }.bind(this), function(callback){
            if(callback.data){
                ScreenService.showMessage(callback.data.error);
            }

        });
    };
    
    this.salvarFiscalizacao = function(args){
        var Widget = args.owner.widget;
        General.saveCrud(Widget).then(function() {
            this.controlaStatusWidgets( templateManager.container.getWidget('container_tabbed'),'fiscalizacaoProdutos',true);
            var currentRow = Widget.currentRow;
            currentRow.__is_new = false;
            currentRow.changed = false;
            Widget.template = Widget.templateToView;
            Widget.view();
            ScreenService.closeSwipe();
            Widget.reload();
            Widget.list();
        }.bind(this), function(callback){
            if(callback.data){
                ScreenService.showMessage(callback.data.error);
            }

        });
    };

    this.situacaoCampoLancamentoManual = function(){
        var gridParametrosGerais =  templateManager.container.getWidget('parametrosGerais');
        var campoImportaXML= gridParametrosGerais.getField('IDTPIMPNFE');
        var campoLancamento =gridParametrosGerais.getField('IDPELANIMPXML');
        if(campoImportaXML.value()=="F"){
            campoLancamento.value("N");
            campoLancamento.readOnly=true;
        }else {
            campoLancamento.readOnly=false;
        }
    };

    this.controlaStatusWidgets = function(mainWidget, nomeWidgetAtual, status){
        mainWidget.widgets.forEach(function(widg) {
            if(nomeWidgetAtual !== widg.name)
                widg.enabled = status;
        });

        if(mainWidget.name !== nomeWidgetAtual)
            mainWidget.enabled = status;
    };

    this.recarregarWidget = function(nomeWidget){
        var widget =  templateManager.container.getWidget(nomeWidget);
        widget.reload();
    };
    
    this.onChangeTolerValPerm = function(widget){
        var val = widget.getField("VRPETOLEVRAF"); 
        if(val.value() == null || val.value() == ""){
            val.validations = null;
            widget.currentRow.$error.VRPETOLEVRAF = [];
        } else{
            val.validations = {
                max: {
                    value: 1000,
                    message: "Apenas valores entre 0 e 1000"
                }
            };
        }
        val.reload();
    };
    
    this.onChangeTolerQtdPerm = function(widget){
        var qtd = widget.getField("VRPETOLEQTAF"); 
        if(qtd.value() == null || qtd.value() == ""){
            qtd.validations = null;
            widget.currentRow.$error.VRPETOLEQTAF = [];
        } else{
            qtd.validations = {
                max: {
                    value: 1000,
                    message: "Apenas valores entre 0 e 1000"
                }
            };
        }
        qtd.reload();
    };
    
    this.inicializaTipoAjuste = function (field) {
        if (!field.dataInitialized) {
            field.reload().then ( () => {
                field.dataInitialized = true;
                if (!field.dataSource.data.length) {
                    ScreenService.showMessage('Não há Tipo de Ajuste cadastrado');
                }
            }); 
        }
        else if (!field.dataSource.data.length) {
            ScreenService.showMessage("Não há Tipo de Ajuste cadastrado");
        }
    };
    
    this.bloquearCampoDiretorio = function(widget) {
        var nrorg = widget.dataSource.data[0].NRORG;
        if(nrorg != 1) {
            widget.currentRow.DSDIRIMPARQXML = '/tmp/';
            widget.getField('DSDIRIMPARQXML').readOnly = true; 
        }
        
    };
    
};

Configuration(function (ContextRegister){
    ContextRegister.register('Dfe31000_ParametrosDocumentosFiscaisEntrada_Controller', Dfe31000_ParametrosDocumentosFiscaisEntrada_Controller);
});

// FILE: ./assets/modules/entrada/assets/js/controllers/Dfe36000_EntradasTransferenciaDoOperadorLogistico_Controller.js
var Dfe36000_EntradasTransferenciaDoOperadorLogistico_Controller = function(Dfe36000_EntradasTransferenciaDoOperadorLogistico_Service, templateManager, ScreenService, General, ZhFilterPreferences){
    
    this.widgetOnEnterOnce = function(widget) {
        Dfe36000_EntradasTransferenciaDoOperadorLogistico_Service.getFilial().then((success) => {
            var row = Util.clone(widget.currentRow);
            if(success.dataset.filial[0].IDFUNCAOFILI == 'O') {
                row.CDOPERLOG = success.dataset.filial[0].CDFILIAL;
                row.NMOPERLOG = success.dataset.filial[0].NMFILIAL;
            }
            else {
                row.CDFILDEST = success.dataset.filial[0].CDFILIAL;
                row.NMFILDEST = success.dataset.filial[0].NMFILIAL;
            }
            var dataAtual = formatarData(new Date());
            row.DTENTRSAID = {start : dataAtual, end : dataAtual};
            
            ZhFilterPreferences.setOriginalFilter(widget.id, row);
            ZhFilterPreferences.openFilterWidget(widget, row);
        },
        (error) => {
            ZhFilterPreferences.openFilterWidget(widget);
            console.log(error);
        });
        
    };
    
    this.preparaFiltro = function(widget) {
        if(widget.appliedFilter) {
            Dfe36000_EntradasTransferenciaDoOperadorLogistico_Service.getParamGeral().then((success) => {
                let currentRow = widget.appliedFilter.FILTER_ROW;
                widget.dataSourceFilter = Dfe36000_EntradasTransferenciaDoOperadorLogistico_Service.buildFilter(["CDOPERLOG", "CDFILDEST", "CDSERIE", "NRNOTAFISC"], currentRow);
                widget.dataSourceFilter.push({name: "P_DTINI", operator: "=", value: currentRow.DTENTRSAID.start});
                widget.dataSourceFilter.push({name: "P_DTFIN", operator: "=", value: currentRow.DTENTRSAID.end});
                widget.dataSourceFilter.push({name: "P_CDOPERADOR", operator: "=", value: getLocalVar('CDOPERADOR')});
                widget.dataSourceFilter.push({name: "P_BLOQLANCNFOP", operator: "=", value: success.dataset.paramgeral[0].IDBLOQLANCNFOPFIL});
                widget.reload();
            });
        }
    };
    
    /*  Função para bloquear os fields filhos ligados a um field pai.
        Caso desbloqueiaComValor = true o field filho será habilitado 
        se o field pai possuir valor. No caso de valueDesbloqueia != null
        o valor para desbloquear o filho deve ser igual ao valor de valueDesbloqueia.
        
        @param field field 
    */
    this.alteraReadOnlyFilhos = function(field) {
        let widget = field.widget;
        field.filhos.forEach((filho) => {
            var desbloqueiaComValor = (filho.desbloqueiaComValor) ? filho.desbloqueiaComValor : true;
            var valueDesbloqueia    = (filho.valueDesbloqueia)    ? filho.valueDesbloqueia    : null;
            
            let filhoField = widget.getField(filho.name);
            if(valueDesbloqueia == null) {
                filhoField.readOnly =  ((field.value() == "") == desbloqueiaComValor);
                if(((field.value() == "") == desbloqueiaComValor)) widget.currentRow[filho.name] = "";
                if(filhoField.filhos) this.alteraReadOnlyFilhos(filhoField, true);
                widget.getField(filho.name).reload();
            }
            else {
                var habilita = (field.value() == valueDesbloqueia);
                if(habilita) {
                    if(filhoField.readOnly) {
                        if(filho.clearFields) {
                            filho.clearFields.forEach((clearField) => {
                                widget.currentRow[clearField] = "";
                            });
                        }
                        widget.currentRow[filho.name] = "";
                    }
                    
                    filhoField.readOnly = false;
                }
                else {
                    widget.currentRow[filho.name] = "";
                    filhoField.readOnly = true;
                }
                if(filhoField.filhos) this.alteraReadOnlyFilhos(filhoField, true);
                widget.getField(filho.name).reload();
            }
        });
    };

    /*
        Percorre uma widget procurando por fields com a propiedade "filhos".
        Para os fields com a propiedade fields a função adiciona o evento 
        de alteraReadOnlyFilhos.
        
        @param widget widget
        @param boolean filter
    */
    this.blockFieldsFilhosAndSetEvents = function(args, filter = true) {
        var widget = args.owner;
        let fields = widget.fields;
        fields.forEach((field) => {
            if(field.filhos) {
                field.events.push(
                    {
                        "name": "FieldOnChange",
                        "code": "Dfe36000_EntradasTransferenciaDoOperadorLogistico_Controller.alteraReadOnlyFilhos(args.owner.field)"
                    }
                );
                
                // field.filhos.forEach((filho) => {
                //     if(filter) widget.appliedFilter.getField(filho).readOnly = true; 
                //     else  widget.getField(filho).readOnly = true; 
                // });
            }
        });
    };
    
    this.openPopup = function() {
	    var widget = templateManager.container.getWidget('entrada_transfOpLogistico');
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.appliedFilter.FILTER_ROW.CDFILDEST
            },
            {
                "name": "CDFILIOPELOG",
                "operator": "=",
                "value": widget.appliedFilter.FILTER_ROW.CDOPERLOG
            },
    	    {
        		"name": "ROWNUM",
        		"operator": "=",
        		"value": "1"
    	    }
        ];
        General.callRestEngine("df#/crud/paramprodsup", "FilterData", filter).then((success) => {
            var popUp = templateManager.container.getWidget('dadosGeracao');
            var nmTipoOper  = popUp.getField('NMTIPOOPER'); 
            if(success.dataset.paramprodsup[0]) {
                nmTipoOper.isVisible = false;
                delete(nmTipoOper.validations);
            } 
            else {
                nmTipoOper.isVisible = true;
                nmTipoOper.validations = {
                    "required": {
                        "message": "Obrigatório"
                    }  
                };
            }
            popUp.isVisible = true;
            ScreenService.openPopup(popUp);
            popUp.activate();
        });
    };
    
    this.closePopup = function(widget) {
        widget.isVisible = false;
        widget.dataSource.data = [];
        ScreenService.closePopup(widget);
        widget.getParent().activate();
        widget.getParent().unSelectAll();
        widget.getParent().reload();
        ScreenService.closePopup();
    };
    
    this.gerarEntrada = function(widget) {
        var widgetTransf = templateManager.container.getWidget('entrada_transfOpLogistico');
        var checkedRows = widgetTransf.getCheckedRows();
        checkedRows.forEach((row) => {
            row = Object.assign(row, widget.currentRow);
        });
        General.callRestEngine("df_entrada#/gerarEntradaOpl", "Row", checkedRows).then((success) => {
            this.closePopup(widget);
            ScreenService.successNotification(success.messages[0].message);
            if(success.dataset.errosGeracao[0]) this.openPopupErros(success.dataset.errosGeracao);
        });
    };
    
    this.openPopupErros = function(errosGeracao) {
        var widgetErros = templateManager.container.getWidget('errosGeracao');
        widgetErros.dataSource.data = errosGeracao;
        widgetErros.isVisible = true;
        ScreenService.openPopup(widgetErros);
        widgetErros.activate();
    };
    
    this.setDefaultFilter = function(widget) {
        Dfe36000_EntradasTransferenciaDoOperadorLogistico_Service.getFilial().then((success) => {
            if(success.dataset.filial[0].IDFUNCAOFILI == 'O') {
                widget.appliedFilter.FILTER_ROW.CDOPERLOG = success.dataset.filial[0].CDFILIAL;
                widget.appliedFilter.FILTER_ROW.NMOPERLOG = success.dataset.filial[0].NMFILIAL;
            }
            else {
                widget.appliedFilter.FILTER_ROW.CDFILIAL = success.dataset.filial[0].CDFILIAL;
                widget.appliedFilter.FILTER_ROW.NMFILIAL = success.dataset.filial[0].NMFILIAL;
            }
            var dataAtual = formatarData(new Date());
            widget.appliedFilter.FILTER_ROW.DTENTRSAID = {start : dataAtual, end : dataAtual};
        },
        (error) => {
            console.log(error);
        });
    };
    
    this.getFieldValueFilter = function(widget) {
        console.log(widget);  
    };
    
    function formatarData(data) {
        var d = new Date(data),
        mes = '' + (d.getMonth() + 1),
        dia = '' + d.getDate(),
        ano = d.getFullYear();
    
        if (mes.length < 2) mes = '0' + mes;
        if (dia.length < 2) dia = '0' + dia;
    
        return [dia, mes, ano].join('/');
    }
    
    this.getFieldValueFilter = function(fieldName) {
        if(templateManager.container) return templateManager.container.getActiveWidget().currentRow[fieldName];
    };
};

Configuration(function (ContextRegister){
    ContextRegister.register('Dfe36000_EntradasTransferenciaDoOperadorLogistico_Controller', Dfe36000_EntradasTransferenciaDoOperadorLogistico_Controller);
});


// FILE: ./assets/modules/entrada/assets/js/controllers/Dfe43000_ParametrosPorFornecedor_Controller.js
var Dfe43000_ParametrosPorFornecedor_Controller = function(ScreenService, MessageService, templateManager, General, ZHPromise) {
    
    var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
    
    var hasDuplicateKeys = function(widget) {
        var codFornecedor = widget.currentRow.CDFORNECED;
        var arr = widget.dataSource.data.filter(function(data) {
            return (data.CDFORNECED === codFornecedor);
        });
        return (arr.length > 1);
    };
    
    this.saveParametrizacao = function(widget) {
        if (widget.isValid()) {
            if (!hasDuplicateKeys(widget)) {
                General.saveCrud(widget);
                MessageService.showMessageByCode("REGISTRO_SALVO");
            }
        }
    };

    this.validateParametersExistance = function(widget) {
        if ((widget.dataSource.data.length === 0) && (widget.dataSourceFilter.length > 0)) {
            widget.newRow();
            General.justInteger('#NRDIASBLOQNFFOR');
            popupWidget = widget.container.getWidget("filterPopup");
            widget.currentRow.CDFORNECED = popupWidget.currentRow.CDFORNECED;
            widget.currentRow.NMRAZSOCFORN = popupWidget.currentRow.NMRAZSOCFORN;
        }
    };

    this.onChangeBloqAlt = function(field) {
        if(field.value() == 'S') {
            field.widget.getField('DSEMAILITPEN').readOnly = false;
        }
        else {
            field.widget.currentRow.DSEMAILITPEN = '';
            field.widget.getField('DSEMAILITPEN').readOnly = true;
        }
        field.reload();
    };
    
    this.edit = function(widget) {
        if(widget.currentRow.IDBLOQALTPRODREL == 'S') {
            widget.getField('DSEMAILITPEN').readOnly = false;
        }
        else {
            widget.getField('DSEMAILITPEN').readOnly = true;
        }
        General.edit(widget);
    };

    this.openPopupCadastroAutomatico = function(args){
        var widget = args.owner.widget;
        var widgetPopup = templateManager.container.getWidget('CADASTROAUTOMATICOPARAMFORN');
        widgetPopup.isVisible = true;
        widgetPopup.activate();
        widgetPopup.edit();
        ScreenService.openPopup(widgetPopup);
    };
    
    this.saveCadastroAutomatico = function(widget){
        ignoredKeys.push("changed", "$error","NMRAZSOCFORN", "NRINSJURFORN");
        if(widget.isValid()) {
            var row = Util.clone(widget.currentRow, ignoredKeys);
            if(row.CDFORNECED.length >= 1000){
                var fornecedores = [];
                while(row.CDFORNECED.length >= 1000){
                    fornecedores.push(row.CDFORNECED.splice(-1000));
                }
                fornecedores.push(row.CDFORNECED);
                
                //Executa um apos o outro
                let chain = ZHPromise.when();
                fornecedores.forEach(fornecedor => {
                    const cadastraFornecedores = fornecedor =>  {
                        row.CDFORNECED = fornecedor;
                        return General.callRestEngine('df_entrada#/cadastroAutomaticoParamForn', 'Row', row);
                    };
                    chain = chain.then(() => cadastraFornecedores(fornecedor));
                });
                
                chain.then(() => {
                    var widgetActivate = templateManager.container.getWidget('parametrosFornecedor');
                    var widgetPopup = templateManager.container.getWidget('CADASTROAUTOMATICOPARAMFORN');
                    
                    widgetPopup.isVisible = false;
                    widgetPopup.currentRow = {};
                    ScreenService.closePopup(widgetPopup);
                    ScreenService.showMessage('Operação realizada com sucesso');
                    widgetActivate.activate();
                    widgetActivate.reload();
                });
    
            }else {
               General.callRestEngine('df_entrada#/cadastroAutomaticoParamForn', 'Row', row).then(function(success){
                    var widgetActivate = templateManager.container.getWidget('parametrosFornecedor');
                    var widgetPopup = templateManager.container.getWidget('CADASTROAUTOMATICOPARAMFORN');
                    
                    widgetPopup.isVisible = false;
                    widgetPopup.currentRow = {};
                    ScreenService.closePopup(widgetPopup);
                    ScreenService.showMessage('Operação realizada com sucesso');
                    widgetActivate.activate();
                    widgetActivate.reload();
    
                },
                function(callback){
                        if(callback.data){
                            ScreenService.showMessage(callback.data.error);
                        }
                    }
                ); 
            }
        }
    };
    
    this.cancelCadastroAutomatico = function() {
        var widget = templateManager.container.getWidget('CADASTROAUTOMATICOPARAMFORN');
        widget.currentRow = {};
        widget.isVisible = false;
        var widgetActivate = templateManager.container.getWidget('parametrosFornecedor');
        ScreenService.closePopup(widget);
        widgetActivate.activate();
        widgetActivate.reload();
    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfe43000_ParametrosPorFornecedor_Controller", Dfe43000_ParametrosPorFornecedor_Controller);
});

// FILE: ./assets/modules/entrada/assets/js/controllers/Dfe47000_LiberacaoParaFinanceiro_Controller.js
var Dfe47000_LiberacaoParaFinanceiro_Controller = function(General, ScreenService, MessageService, ZhFilterPreferences, templateManager) {
    
    var messages = {};
    
    this.setFilterOnEnterOnce = function(args) {
        let widget = args.owner;
        if (sessionStorage.getItem('CDEMPRESA') &&  sessionStorage.getItem('NMEMPRESA') && sessionStorage.getItem('DTEMISSAO')){
            this.blockFieldsFilhosAndSetEvents(widget, true, undefined , false);
            this.filtroPainelDeControle(args);
            sessionStorage.clear();
        }else{
            General.callRestEngine("df_entrada#/getFilter_Liberacao", "FilterData", []).then((success) => {
                var filterDefault = success.dataset.filter_liberacao;
                var row = Util.clone(widget.currentRow);
                for(var key in filterDefault) {
                    row[key] = filterDefault[key];
                }
                this.blockFieldsFilhosAndSetEvents(widget, true, row , true );
            },
            (error) => {
                this.blockFieldsFilhosAndSetEvents(widget, true, true );
            });
        }
    };
    
    this.preparaFiltro = function(args) {
        args.e.preventDefault();
        var widget = args.owner;
        if(widget.appliedFilter) {
            args.e.filters.forEach((f) => {
                if(f.name == "NMTIPOOPER") f.value = widget.appliedFilter.FILTER_ROW.CDTIPOOPER;
                else if(f.name == "DSSERIEENTR") f.value = widget.appliedFilter.FILTER_ROW.CDSERIEENTR;
                else if(f.name == "DSBLOQNFFIN") f.value = widget.appliedFilter.FILTER_ROW.IDBLOQNFFIN;
            });      
        }
        widget.reload();
    };
    
    this.inicializaAnexoLiberaLanc = function(args){
        var filter = [
            {
                "name": "P_CDFILIAL",
                "operator": "=",
                "value": args.owner.getParent().currentRow.CDFILIAL
            },
            {
                "name": "P_NRLANCTONF",
                "operator": "=",
                "value": args.owner.getParent().currentRow.NRLANCTONF
            }
            
        ];

        General.callRestEngine("df#/crud/anexo_obrigatorio", "FilterData", filter).then((success) => {
            if(success.dataset.anexo_obrigatorio[0].IDBLOQANEXO) {
                if(success.dataset.anexo_obrigatorio[0].IDBLOQANEXO == 0) {
                    args.owner.getAction("actNovoAnexo").isVisible = false;
                    args.owner.getAction("actDeleteAnexo").isVisible = false;
                }
                else if(success.dataset.anexo_obrigatorio[0].IDBLOQANEXO == 1 && args.owner.getParent().currentRow.IDSTATUS == 'A') {
                    args.owner.getAction("actNovoAnexo").isVisible = false;     
                    args.owner.getAction("actDeleteAnexo").isVisible = false;
                }                
                else if(success.dataset.anexo_obrigatorio[0].IDBLOQANEXO == 2 && (args.owner.getParent().currentRow.IDBLOQNFFIN == 'L')) {
                    args.owner.getAction("actNovoAnexo").isVisible = false;     
                    args.owner.getAction("actDeleteAnexo").isVisible = false;
                }
                else {
                    args.owner.getAction("actNovoAnexo").isVisible = true;    
                    args.owner.getAction("actDeleteAnexo").isVisible = true;
                }
            }
        });
        args.owner.reload();
    };
    
    this.alteraReadOnlyFilhos = function(field) {
        let widget = field.widget;
        field.filhos.forEach((filho) => {
            var desbloqueiaComValor = (filho.desbloqueiaComValor) ? filho.desbloqueiaComValor : true;
            var valueDesbloqueia    = (filho.valueDesbloqueia)    ? filho.valueDesbloqueia    : null;
            var isRequired          = (filho.isRequired)          ? filho.isRequired          : false;
            
            let filhoField = widget.getField(filho.name);
            
            
            if(valueDesbloqueia == null) {
                filhoField.readOnly =  ((field.value() == "") == desbloqueiaComValor);
                if(((field.value() == "") == desbloqueiaComValor)) widget.currentRow[filho.name] = "";
                if(filhoField.filhos) this.alteraReadOnlyFilhos(filhoField, true);
                widget.getField(filho.name).reload();
            }
            else {
                var habilita = (field.value() == valueDesbloqueia);
                if(habilita) {
                    if(filhoField.readOnly) {
                        if(filho.clearFields) {
                            filho.clearFields.forEach((clearField) => {
                                widget.currentRow[clearField] = "";
                            });
                        }
                        widget.currentRow[filho.name] = "";
                    }
                    
                    filhoField.readOnly = false;
                    
                    if(isRequired) {
                        if(filhoField.validations) {
                            filhoField.validations.required = true;
                        }
                        else {
                            filhoField.validations = {required : true};
                        }
                    }
                }
                else {
                    var filhoOutData = filhoField.outData;
                    for(var fieldOut in filhoOutData) {
                        widget.currentRow[fieldOut] = "";
                    } 
                    widget.currentRow[filho.name] = "";
                    filhoField.readOnly = true;
                    if(isRequired) {
                        if(filhoField.validations) {
                            if(filhoField.validations.required) delete filhoField.validations.required;
                        }
                    }
                }
                if(filhoField.filhos) this.alteraReadOnlyFilhos(filhoField, true);
            }
        });
    };

    /*
        Percorre uma widget procurando por fields com a propiedade "filhos".
        Para os fields com a propiedade fields a função adiciona o evento 
        de alteraReadOnlyFilhos.
        
        @param widget widget
        @param boolean filter
    */
    this.blockFieldsFilhosAndSetEvents = function(widget, filter, row, filtrarPeloDashboard) {
        let fields = widget.fields;
        fields.forEach((field) => {
            if(field.filhos) {
                field.events.push(
                    {
                        "name": "FieldOnChange",
                        "code": "Dfe47000_LiberacaoParaFinanceiro_Controller.alteraReadOnlyFilhos(args.owner.field)"
                    }
                );
                
                field.filhos.forEach((filho) => {
                    if(filter) {
                        if(!widget.getField(filho.name).filterProperties) 
                            widget.getField(filho.name).filterProperties = {};
                        
                        widget.getField(filho.name).filterProperties.readOnly = true; 
                    }
                    else  widget.getField(filho.name).readOnly = true; 
                });
            }
        });
        if(row && filtrarPeloDashboard) {
            ZhFilterPreferences.setOriginalFilter(widget.id, row);
            ZhFilterPreferences.openFilterWidget(widget, row);
        }
        else if (filtrarPeloDashboard){ ZhFilterPreferences.openFilterWidget(widget);}
    };
    
    /*
    * Processos
    **/
    
    this.setAprovacaoLiberacao = function(widget, mode) {
        var rows = widget.getCheckedRows();
        if(rows)  {
            if(mode == 'L'){
                General.callRestEngine("df_entrada#/liberarTitulos", "DataSet", rows).then((result) => {
                var titulosLiberados = "TITULOS_LIBERADOS";
                ScreenService.closePopup();
                if(result.dataset.erros_aprovacao[0]) {
                    if(rows.length != result.dataset.erros_aprovacao.length) {
                        ScreenService.successNotification(result.messages[0].message);
                    }
                    this.openPopupErros(result.dataset.erros_aprovacao);
                }
                else {
                    if(result.messages[0].message == "Nenhum dos titulos selecionados pode ser liberado."){
                        ScreenService.showMessage("Nenhum dos titulos selecionados pode ser liberado.");
                    }else{
                    MessageService.successNotificationByCode(titulosLiberados, null, {"NUMTITULOS": result.messages[0].message});
                    }
                }
                widget.reload();
            }, (error) => ScreenService.errorNotification(error));

            }else{
                General.callRestEngine("df_entrada#/bloquearTitulos", "DataSet", rows).then((result) => {
                var titulosBloqueados = "TITULOS_BLOQUEADOS";
                ScreenService.closePopup();
                if(result.dataset.erros_aprovacao[0]) {
                    if(rows.length != result.dataset.erros_aprovacao.length) {
                        ScreenService.successNotification(result.messages[0].message);
                    }
                    this.openPopupErros(result.dataset.erros_aprovacao);
                }
                else {
                    if(result.messages[0].message == "Nenhum dos titulos selecionados pode ser bloqueado."){
                        ScreenService.showMessage("Nenhum dos titulos selecionados pode ser bloqueado.");
                    }else{
                        MessageService.successNotificationByCode(titulosBloqueados, null,{"NUMTITULOS": result.messages[0].message});                        
                    }
                }
                widget.reload();
            }, (error) => ScreenService.errorNotification(error));

            }
        }
        else {
            ScreenService.showMessage("Nenhum titulo foi selecionado.");
        }
    };
    
    this.openPopupErros = function(erros) {
        var widgetErros = templateManager.container.getWidget('WidgetErros');
        widgetErros.dataSource.data = erros;
        widgetErros.isVisible = true;
        ScreenService.openPopup(widgetErros);
        widgetErros.activate();
    };
    
    this.filtroPainelDeControle = function(args) {
        let CDEMPRESA =  sessionStorage.getItem('CDEMPRESA');
        let DTENTRSAID =  JSON.parse(sessionStorage.getItem('DTEMISSAO'));
        let LiberacaoParaFinanceiro = args.owner;
        
        let dataSourceFilter = [
            {"name":"CDEMPRESA","value":CDEMPRESA ,"operator":"="},
            {"name":"DTENTRSAID","value":[ DTENTRSAID.start, DTENTRSAID.end],"operator":"BETWEEN"},
            {"name":"TCDTIPOOPER","value":"T","operator":"="},
            {"name":"TCDFILIORIG","value":"T","operator":"="},
            {"name":"TCDSERIEENTR","value":"T","operator":"="},
            {"name":"ERRO","value":"T","operator":"="},
            {"name":"IDBLOQNFFIN","value":"T","operator":"="}
            ];
        LiberacaoParaFinanceiro.dataSourceFilter = dataSourceFilter;
        LiberacaoParaFinanceiro.reload().then(() => {
            LiberacaoParaFinanceiro.activate();
        });
        };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfe47000_LiberacaoParaFinanceiro_Controller", Dfe47000_LiberacaoParaFinanceiro_Controller);
});

// FILE: ./assets/modules/entrada/assets/js/controllers/Dfe51000_ImportacaoConsultaNotaFiscal_Controller.js
var Dfe51000_ImportacaoConsultaNotaFiscal_Controller = function(Dfe51000_ImportacaoConsultaNotaFiscal_Service, MessageService, templateManager, ScreenService, General, ZhFilterPreferences){
    
    this.mensagens = {
        xmlUtilizado: "Arquivo XML já utilizado",
        relacSucesso: "Relacionamento feito com sucesso!",
        nfOk:         "Nota fiscal gerada com sucesso.",
        nfOkIncons:   "Nota fiscal gerada com inconsistência(s).",
        nfIncons:     "Operação não realizada! Verifique as inconsistências",
        nfNumero:     "Nº de Nota Fiscal Inicial deve ser menor ou igual ao Nº da Nota Fiscal Final informado.",
        afDesvinc:    "AF desvinculada com sucesso."
    };
    
    /*  Função para bloquear os fields filhos ligados a um field pai.
        Caso desbloqueiaComValor = true o field filho será habilitado 
        se o field pai possuir valor. No caso de valueDesbloqueia != null
        o valor para desbloquear o filho deve ser igual ao valor de valueDesbloqueia.
        
        @param field field 
    */
    this.alteraReadOnlyFilhos = function(field) {
        let widget = field.widget;
        field.filhos.forEach((filho) => {
            var desbloqueiaComValor = (filho.desbloqueiaComValor) ? filho.desbloqueiaComValor : true;
            var valueDesbloqueia    = (filho.valueDesbloqueia)    ? filho.valueDesbloqueia    : null;
            var isRequired          = (filho.isRequired)          ? filho.isRequired          : false;
            
            let filhoField = widget.getField(filho.name);
            
            
            if(valueDesbloqueia == null) {
                filhoField.readOnly =  ((field.value() == "") == desbloqueiaComValor);
                if(((field.value() == "") == desbloqueiaComValor)) widget.currentRow[filho.name] = "";
                if(filhoField.filhos) this.alteraReadOnlyFilhos(filhoField, true);
                widget.getField(filho.name).reload();
            }
            else {
                var habilita = (field.value() == valueDesbloqueia);
                if(habilita) {
                    if(filhoField.readOnly) {
                        if(filho.clearFields) {
                            filho.clearFields.forEach((clearField) => {
                                widget.currentRow[clearField] = "";
                            });
                        }
                        widget.currentRow[filho.name] = "";
                    }
                    
                    filhoField.readOnly = false;
                    
                    if(isRequired) {
                        if(filhoField.validations) {
                            filhoField.validations.required = true;
                        }
                        else {
                            filhoField.validations = {required : true};
                        }
                    }
                    widget.getField(filho.name).reload();
                }
                else {
                    var filhoOutData = filhoField.outData;
                    for(var fieldOut in filhoOutData) {
                        widget.currentRow[fieldOut] = "";
                    } 
                    widget.currentRow[filho.name] = "";
                    filhoField.readOnly = true;
                    if(isRequired) {
                        if(filhoField.validations) {
                            if(filhoField.validations.required) delete filhoField.validations.required;
                        }
                    }
                }
                if(filhoField.filhos) this.alteraReadOnlyFilhos(filhoField, true);
                widget.getField(filho.name).reload();
            }
        });
    };

    /*
        Percorre uma widget procurando por fields com a propiedade "filhos".
        Para os fields com a propiedade fields a função adiciona o evento 
        de alteraReadOnlyFilhos.
        
        @param widget widget
        @param boolean filter
    */
    this.blockFieldsFilhosAndSetEvents = function(widget, filter = true) {
        let fields = widget.fields;
        fields.forEach((field) => {
            if(field.filhos) {
                field.events.push(
                    {
                        "name": "FieldOnChange",
                        "code": "Dfe51000_ImportacaoConsultaNotaFiscal_Controller.alteraReadOnlyFilhos(args.owner.field)"
                    }
                );
                
                field.filhos.forEach((filho) => {
                    if(filter) {
                        if(!widget.getField(filho.name).filterProperties) 
                            widget.getField(filho.name).filterProperties = {};
                        
                        widget.getField(filho.name).filterProperties.readOnly = false; 
                    }
                    else  widget.getField(filho.name).readOnly = false; 
                    
                });
            }
        });
        this.openFilterWidget(widget);
    };
    
    this.alteraNomeBotaoBloqueia = function (widget){
        
        let wdg_historico = templateManager.container.getWidget("wdg_historico");
            
            wdg_historico.reload().then( () => {
                var num = wdg_historico.dataSource.data.length-1;
                var idStatus = wdg_historico.dataSource.data[num].IDSTATUS;
                var idStatusNfe = widget.currentRow.IDSTATUSNFE;
            
                if((idStatus == 'B' && idStatusNfe == 'N') || (idStatus == 'B' && idStatusNfe == 'B') ) {
                    widget.actions.filter(a => a.name === "statusNFE" )[0].label = "Desbloquear NF";
                    widget.actions.filter(a => a.name === "statusNFE" )[0].isVisible = true;
                } else if((idStatus == 'N' && idStatusNfe == 'N') || (idStatus == 'N' && idStatusNfe == 'B')) {
                    widget.actions.filter(a => a.name === "statusNFE" )[0].label = "Bloquear NF";
                    widget.actions.filter(a => a.name === "statusNFE" )[0].isVisible = true;
                } else {
                    widget.actions.filter(a => a.name === "statusNFE" )[0].isVisible = false;
                }
                widget.activate();
               
            });
            widget.reload();
    };    


    this.preparaFiltro = function(widget, args) {
        args.e.preventDefault();
        let currentRow = args.e.preference.FILTER_ROW;

        if(widget.isValid()) {
            widget.dataSourceFilter = [];
            Dfe51000_ImportacaoConsultaNotaFiscal_Service.buildFilter(["CDPAIS", "SGESTADO", "CDFILIAL", "CDFORNECED", "IDSTATUSNFE", "CDINSCESTA", "NRNOTAFISCINI", "NRNOTAFISCFIN", "NRNOTA", "NRACESSONFE", "IDDOCFISCELE"], currentRow, true)
            .then((filter) => {
                widget.dataSourceFilter = filter;
                if(currentRow.DTEMISSAO) {
                    if(currentRow.DTEMISSAO.start && currentRow.DTEMISSAO.end) {
                        widget.dataSourceFilter.push({name: "P_DTINICIAL", operator: "=", value: currentRow.DTEMISSAO.start});
                        widget.dataSourceFilter.push({name: "P_DTFINAL", operator: "=", value: currentRow.DTEMISSAO.end});
                        widget.dataSourceFilter.push({name: "P_DATA", operator: "=", value: ''});
                    }
                    else {
                        widget.dataSourceFilter.push({name: "P_DTINICIAL", operator: "=", value: '30/12/1989'});
                        widget.dataSourceFilter.push({name: "P_DTFINAL", operator: "=", value: '30/12/1989'});
                        widget.dataSourceFilter.push({name: "P_DATA", operator: "=", value: 'T'});
                    }
                }
                else {
                    widget.dataSourceFilter.push({name: "P_DTINICIAL", operator: "=", value: '30/12/1989'});
                    widget.dataSourceFilter.push({name: "P_DTFINAL", operator: "=", value: '30/12/1989'});
                    widget.dataSourceFilter.push({name: "P_DATA", operator: "=", value: 'T'});
                }
                widget.reload().then(() => {
                    widget.activate();
                });
                ScreenService.closePopup();
            });
            
        }

    };
    
    this.validaNumeroOnChange = function(widget, field) {
        if(widget.currentRow.NRNOTAFISCINI && widget.currentRow.NRNOTAFISCFIN) {
            if(parseInt(widget.currentRow.NRNOTAFISCINI) > parseInt(widget.currentRow.NRNOTAFISCFIN)) {
               ScreenService.showMessage(this.mensagens.nfNumero);
                field.setValue(''); 
            }
        }
    };
    
    function openPopup (popupName) {
        var widgetPopup = templateManager.container.getWidget(popupName);
        widgetPopup.isVisible = true;
        ScreenService.openPopup(widgetPopup);
        widgetPopup.activate();
        widgetPopup.reload();
    }
    
    this.closePopup = function(widget) {
        widget.isVisible = false;
        widget.dataSource.data = [];
        ScreenService.closePopup(widget);
        widget.parent.activate();
        widget.parent.unSelectAll();
        widget.parent.reload();
        ScreenService.closePopup();
    };
    
    this.gerarDanfe = function(widget){
        let row = widget.currentRow;
        
        Dfe51000_ImportacaoConsultaNotaFiscal_Service.gerarDanfe(row).then(
            resp => {
                window.open(window.serviceUrls.filter(function(x){return x.package == 'df';})[0].baseUrl+"/downloadArquivo?requestType=Row&row[DSARQUIVO]=" + resp.dataset.danfe[0]);
            }
        ).catch(
            error => {
                let msg = error.data ? error.data.error : error.error;
                
                ScreenService.showMessage(msg);
            }
        );  
    };
    
    this.afterMoveRow = function(widget){
        let hasNrAcessoNFe = !!widget.currentRow.NRACESSONFE;
        
        widget.getAction("gerarDanfe").isVisible = hasNrAcessoNFe;
        this.exibeNfeCte(widget);
    };
    
     this.openWidgetUpload = function(widget){
        var widgetUpload = templateManager.container.getWidget('widgetUploadXML');
        widgetUpload.currentRow = { __is_new: true };
        widgetUpload.isVisible  = true;
        ScreenService.openSwipe(widgetUpload).then(function(){
            widgetUpload.newRow();
        });
        
    };
    
      this.closeUploadSwipe = function(widget){
        ScreenService.closeSwipe();
        templateManager.container.getWidget('wdg_romaneio').activate();
        widget.isVisible = false;
        
    };
    
    this.upload = function(widget){
        if(!widget.isValid()){
            return;
        }
        let row = widget.currentRow;
        row.ARQ = widget.currentRow.ARQUIVO;
        General.callRestEngine('df_entrada#/uploadNotasFiscais', "Row", row.ARQ)
        .then(response => {
            if(response.messages[0].message){
                var msg = "SUCESSO_IMPORTAR";
                var sucessos = response.messages[0].message;
                
                General.callRestEngine('df_entrada#/enviaEmailRelacionamentoPendente', "Row", row).then(function(){
                    MessageService.showMessageByCode(msg,{"NUMSUCESS": sucessos });
                    ScreenService.closeSwipe();
                });
            } else {
                var msge = "ERRO_IMPORTAR";
                var erros = response.error[0];
                MessageService.showMessageByCode(msge, {"NUMERROR": erros});
                ScreenService.closeSwipe();    
            }
            
        },
        (error) => {
            ScreenService.showMessage(error.data.error.replace("Exception: ", ""));
        });
        
    };
    
    
    this.openFilterWidget = function(widget) {
        var filter = Util.clone(widget.currentRow, ["__rightActions", "__leftActions"]);
        filter.TCDINSCESTA   = 'L';
        filter.TCDFILIAL     = 'L';
        filter.TNRINSJURFORN = 'T';
        filter.NRACESSONFE   = '';
        
        var row        = { CDFILIAL: getLocalVar('CDFILIAL')};
        widget.getField("NMRAZSOCFORNFILTER").filterProperties.readOnly = true;
        widget.getField("NMRAZSOCFORNFILTER").filterProperties.validations = {};
        General.callRestEngine("df_entrada#/defaultFilterImportacao", "Row", row).then((defaultFilter) =>  {
            defaultFilter = defaultFilter.dataset.defaultFilter;

            for(var key in defaultFilter) {
                if(key == 'DTEMISSAO') {
                    filter.DTEMISSAO = {
                        start:  defaultFilter[key],
                        period: undefined,
                        end:    defaultFilter[key]
                    };
                }
                else {
                     if (key == 'CDINSCESTA') {
                        filter.CDINSCESTAFILTER = [defaultFilter[key]];
                     }
                    else if (key == 'NMRAZSOCFORN') {
                        filter.NMRAZSOCFORNFILTER = [defaultFilter[key]];
                     }
                    filter[key] = [defaultFilter[key]];
                }
            
            }
            ZhFilterPreferences.setOriginalFilter(widget.id, filter);
            ZhFilterPreferences.openFilterWidget(widget, filter).then(response => {
                var filterWidget = templateManager.container.getActiveWidget();
                let clear =  filterWidget.actions.filter(action => action.name === "clear")[0];
                clear.shortcutKey = "ctrl+backspace";
                clear.tooltip = "Limpar (ctrl + Backspace)";
            });
        },
        (error) => {
            ZhFilterPreferences.openFilterWidget(widget).then(response => {
                var filterWidget = templateManager.container.getActiveWidget();
                let clear =  filterWidget.actions.filter(action => action.name === "clear")[0];
                clear.shortcutKey = "ctrl+backspace";
                clear.tooltip = "Limpar (ctrl + Backspace)";
            });
        });
        
    };

    this.getUnidadeValue = function(widget){
        var widgetAtiva = templateManager.container.getActiveWidget();
        var row = widgetAtiva.currentRow;
        var dados = {"CDFILIAL": row.CDFILIAL};
        General.callRestEngine("df_entrada#/getInscEstadual", "Row", dados).then((response) => {
            if(response.dataset.getInscEstadual[0]) {
                widgetAtiva.getField('CDINSCESTAFILTER').dataSourceFilter = [
                    {
                        "name": "CDINSCESTA",
                        "operator": "=",
                        "value": response.dataset.getInscEstadual[0]
                    }  
                ];
            }
            else {
                widgetAtiva.getField('CDINSCESTAFILTER').dataSourceFilter = [];
            }
            widgetAtiva.getField('CDINSCESTAFILTER').reload();
        });
    };
    this.openRelacionarAf = function(widget) {
        if(widget.currentRow.IDSTATUSNFE == 'N') {
            openPopup("wdg_relacionarAf");
        }
        else {
            ScreenService.showMessage(this.mensagens.xmlUtilizado);
        }
    };
    
    this.relacionarAf = function(widget) {
        var row = {"NRAUTOFORN": widget.currentRow.NRAUTOFORN, "NRACESSONFE": widget.parent.parent.currentRow.NRACESSONFE};
        General.callRestEngine("df_entrada#/updateNumeroAfXml", "Row", row).then(() => {
            this.closePopup(widget);    
            ScreenService.showMessage(this.mensagens.relacSucesso);
        });
        
    };
    
    this.verificaGeraNf = function(widget) {
       General.callRestEngine("df#/crud/paramgeral", "FilterData", [] ).then((success) => {
            if(success.dataset.paramgeral[0].IDOBTPOPLANCSIMP) openPopup('wdg_tipooperacao');
            else {
                var nracessonfe = widget.currentRow.NRACESSONFE;
                var tipooperac  = widget.currentRow.CDTIPOOPER;
                General.callRestEngine("df_entrada#/gerarNfXml", "Row", {"NRACESSONFE": nracessonfe, "CDTIPOOPER": tipooperac} ).then(() => {
                    var nfGerada = success.dataset.gerarNfXml[0];
                    if(nfGerada.IDSTATUSNFE == 'U' && nfGerada.IDSTATUS == 'L') ScreenService.showMessage(this.mensagens.nfOk);
                    else if(nfGerada.IDSTATUSNFE == 'U' && nfGerada.IDSTATUS == 'I') ScreenService.showMessage(this.mensagens.nfOkIncons);
                    else  ScreenService.showMessage(this.mensagens.nfIncons);
                });
            }
        });
        
    };
    
    this.gerarNf = function(widget, opDiferenteNota = false) {
        if(widget.isValid()) {
            var nracessonfe = widget.parent.currentRow.NRACESSONFE;
            var tipooperac  = widget.currentRow.CDTIPOOPER;
            General.callRestEngine("df_entrada#/gerarNfXml", "Row", {"NRACESSONFE": nracessonfe, "CDTIPOOPER": tipooperac} ).then((success) => {
               
                var nfGerada = success.dataset.gerarNfXml[0];
                if(nfGerada.IDSTATUSNFE == 'U' && nfGerada.IDSTATUS == 'L') ScreenService.showMessage(this.mensagens.nfOk);
                else if(nfGerada.IDSTATUSNFE == 'U' && nfGerada.IDSTATUS == 'I') ScreenService.showMessage(this.mensagens.nfOkIncons);
                else  ScreenService.showMessage(this.mensagens.nfIncons);
                
                this.closePopup(widget);
                
            });
        }
    };
    
    this.openGerarRelatorio = function() {
        var wdgRelatorio = templateManager.container.getWidget('wdg_relatorio');
        wdgRelatorio.currentRow = {IDREPORT: "DFE51000", DSREPORT: "Importação/Consulta de Notas Fiscais (Arquivo XML)"};
        openPopup("wdg_relatorio");
    };
    
    this.onChangeRelatorio = function(widget) {
        if(widget.currentRow.IDREPORT == "DFE51000") {
            widget.getField("NMARQUIVO").isVisible          = false;
            widget.getField("NRACESSONFE").isVisible        = true;
            widget.getField("NMRAZSOCFORN").isVisible       = true;
            widget.getField("DSSERIEENTR").isVisible        = true;
            widget.getField("NRNOTAFISC").isVisible         = true;
            widget.getField("DSINSCONSISTENCIA").isVisible  = false;
            widget.reload();
        }  
        else if(widget.currentRow.IDREPORT == "DFE51001") {
            widget.getField("NMARQUIVO").isVisible          = true;
            widget.getField("NRACESSONFE").isVisible        = true;
            widget.getField("NMRAZSOCFORN").isVisible       = true;
            widget.getField("DSSERIEENTR").isVisible        = true;
            widget.getField("NRNOTAFISC").isVisible         = true;
            widget.getField("DSINSCONSISTENCIA").isVisible  = true;
            widget.reload();
        }
        else {
            widget.getField("NMARQUIVO").isVisible          = false;
            widget.getField("NRACESSONFE").isVisible        = true;
            widget.getField("NMRAZSOCFORN").isVisible       = false;
            widget.getField("DSSERIEENTR").isVisible        = false;
            widget.getField("NRNOTAFISC").isVisible         = false;
            widget.getField("DSINSCONSISTENCIA").isVisible  = true;
            widget.reload();
        }
    };
    
    this.gerarRelatorio = function(widget) {
        widget.currentRow.NRORG = getLocalVar("NRORG").toString();
        General.callRestEngine("df_entrada#/geraRelatorioImpCons", "Row", widget.currentRow).then(function(success) {
            if(success.messages[0].message) window.open(success.messages[0].message);
        });  
    };
    
    this.importarNotas = function(widget) {
        General.callRestEngine("df_entrada#/importarNotasFiscais", "Row", {})
            .then((success) => {
                if(success.messages){
                    var msgSucess = "NFE_IMPORTADAS";
                    var numSucess = success.messages[0].message;
                    MessageService.successNotificationByCode(msgSucess, null,{"NUMSUCESS": numSucess});    
                }
                if(success.notifications){
                    var msgError = "NFE_ERROS";
                    var numError = success.notifications[0].message;
                    MessageService.errorNotificationByCode(msgError,{"NUMERROR": numError});
                }
                ScreenService.closePopup();
                widget.reload();
            },
            (error) => {
                ScreenService.showMessage(error.data.error.replace("Exception: ", ""));
                ScreenService.closePopup();
            });
    };
    
    this.importarCTe = function(widget){
        General.callRestEngine("df_entrada#/importarCTe", "Row", {}).then((success) => {
            ScreenService.showMessage(success.messages[0].message);
            ScreenService.closePopup();
            widget.reload();
        },
        (error) => {
            ScreenService.showMessage(error.data.error.replace("Exception :", ""));
            ScreenService.closePopup();
        });
    };
    
    this.exibeNfeCte = function(widget) {
        templateManager.container.getWidget('wdg_nfeCte').showAsTab = (widget.currentRow.IDDOCFISCELE == 1);
    };
    
    this.saveIntg = function(widget){
        widget.currentRow.NRACESSONFE = widget.currentRow.NRACESSONFEIC;
        General.callRestEngine("df#/crud/intg_notafiscal/save", "Row", widget.currentRow).then((success) => {
            ScreenService.successNotification("Registro salvo com sucesso");
            ScreenService.closeSwipe();
        },
        (error) => {
            ScreenService.showMessage(error.data.error);
        });
    };
    this.openPopupSincronizacao = function() {
        var wdgSinc = templateManager.container.getWidget('WidgetImportacaoNFSe');
        wdgSinc.currentRow.FORMAPESQUISA    = "0";
        wdgSinc.isVisible = true;
        ScreenService.openPopup(wdgSinc);
    };
    
    this.importarNFSe = function(widget){
        if(widget.isValid()){
        General.callRestEngine("df_entrada#/importarNFSe", "Row",widget.currentRow)
            .then((success) => {
                if(success.messages){
                    var msgSucess = "NFSE_IMPORTADAS";
                    var numSucess = success.messages[0].message;
                    MessageService.showMessageByCode(msgSucess, {"NUMSUCESS": numSucess});
                } 
                else if(success.notifications){
                    var msgError = "NFSE_ERRROS";
                    var numError = success.notifications[0].message;
                    MessageService.showMessageByCode(msgError, {"NUMERROR": numError});
                } 
                ScreenService.closePopup();
                widget.reload();
            },
            (error) => {
                ScreenService.showMessage(error.data.error.replace("Exception: ", ""));
                ScreenService.closePopup();
            });
        }
    };
    
    this.empresaOnChange = function(widget) {
        let cdempresa = widget.currentRow.CDEMPRESA;
        let campo = widget.getField("CDINSCMUNI");
        
        if (cdempresa != null && cdempresa != ""){
            campo.readOnly = false;
            campo.setValue(null);
            campo.dataSourceFilter[0].value = cdempresa;
            //campo.clearValue();
            campo.reload();
        }else {
            campo.readOnly = true;
            campo.setValue(null);
            campo.reload();
        }

    };
    
    this.closePopupSincronizacao = function() {
        var wdgSinc = templateManager.container.getWidget('WidgetSincronizacao');
        ScreenService.closePopup();
        wdgSinc.isVisible = false;
    };
    
    function getLocalDate(){
        var dNow = new Date();
        var localdate = dNow.getDate() + '/' + (dNow.getMonth()+1) + '/' + dNow.getFullYear() + ' ' + dNow.getHours() + ':' + dNow.getMinutes() +':' + dNow.getSeconds();
        return localdate;
    }
    
    this.alteraStatusNfeXml = function (widget) {
        let nrAcessoNfe = widget.currentRow.NRACESSONFE;
        let idStatusNfe = widget.currentRow.IDSTATUSNFE;
        widget.currentRow.CDOPERADOR = getLocalVar('CDOPERADOR'); 
        let cdOperador =   widget.currentRow.CDOPERADOR;
        let wdg_historico = templateManager.container.getWidget("wdg_historico");
        let wdg_romaneio =  templateManager.container.getWidget("wdg_romaneio");
        var num = wdg_historico.dataSource.data.length-1;
        let idStatus = wdg_historico.dataSource.data[num].IDSTATUS;
        var dtHistor = getLocalDate();

        General.callRestEngine("df_entrada#/statusNfeXml", "Row", {"NRACESSONFE": nrAcessoNfe, "IDSTATUSNFE": idStatusNfe, "CDOPERADOR":cdOperador, "DTHISTORICO": dtHistor, "IDSTATUS": idStatus}).then(() => {
            ScreenService.successNotification("Alteração realizada com sucesso");
            ScreenService.closeSwipe(); 
            ScreenService.closePopup();       
            wdg_romaneio.reload();
        },
        (error) => {
            ScreenService.showMessage(error.data.error);
            ScreenService.closePopup();       
        });
    };
    
    this.importarFacturacionEletronica = function(widget) {
         General.callRestEngine("df_entrada#/importarFacturacionElectronica", "Row", {}).then((success) => {
            if(success.messages){
                var msgSucess = "FE_IMPORTADAS";
                var numSucess = success.messages[0].message;
                MessageService.sucessNotificationByCode(msgSucess, {"NUMSUCESS": numSucess});
            } 
            if(success.notifications){
                var msgError = "FE_ERROS";
                var numError = success.notifications[0].message;
                MessageService.errorNotificationByCode(msgError, {"NUMERROR": numError});
            } 
            ScreenService.closePopup();
            widget.reload();
        },
        (error) => {
            ScreenService.showMessage(error.data.error.replace("Exception:", ""));
            ScreenService.closePopup();
        });
    };

    
    this.verificaDesvinculaAfNotaFiscal = function () {
        var wdg_notafiscal = templateManager.container.getWidget('wdg_romaneio');
        
        if(wdg_notafiscal.currentRow.DESVINCULANF == 0) {
            wdg_notafiscal.actions.filter(a => a.name === "desvincularAf" )[0].isVisible = false;
            wdg_notafiscal.reload();
        } else {
            wdg_notafiscal.actions.filter(a => a.name === "desvincularAf" )[0].isVisible = true;
            wdg_notafiscal.reload();
        }
    };
    
    this.verificaDesvinculaAfProduto = function () {
        var wdg_produtos = templateManager.container.getWidget('wdg_produtos');
        var wdg_notafiscal = templateManager.container.getWidget('wdg_notafiscal');
        
        if(wdg_produtos.currentRow.NRAUTOFORN === null) {
            wdg_produtos.actions.filter(a => a.name === "desvincularAf" )[0].isVisible = false;
            wdg_notafiscal.reload();
        } else {
            wdg_produtos.actions.filter(a => a.name === "desvincularAf" )[0].isVisible = true;
            wdg_notafiscal.reload();
        }
    };
    
    this.desvinculaAfNotafiscal = function(){
        var wdg_romaneio = templateManager.container.getWidget('wdg_romaneio');
        var statusNF = wdg_romaneio.currentRow.IDSTATUSNFE;
        var verificacoesAf = this.verificaDesvinculaAfNotaFiscal;
        var refreshWidgetRomaneio = this.refreshWidgetRomaneio;
        
        if (statusNF === 'U') {
            ScreenService.showMessage(this.mensagens.xmlUtilizado);
            return;
        }
        
        var row = {"NRACESSONFE": wdg_romaneio.currentRow.NRACESSONFE};
         
        General.callRestEngine("df_entrada#/updateDesvinculaAfNotafiscal", "Row", row).then(() => {
            ScreenService.closePopup();
            ScreenService.showMessage(this.mensagens.afDesvinc);
            refreshWidgetRomaneio(wdg_romaneio).then( () => {
                verificacoesAf();
            });
        });
    };
    
    this.desvinculaAfProduto = function(){
        var wdg_romaneio = templateManager.container.getWidget('wdg_romaneio');
        var wdg_produtos = templateManager.container.getWidget('wdg_produtos');
        var statusNF = wdg_romaneio.currentRow.IDSTATUSNFE;
        var verificacoesAf = this.verificaDesvinculaAfProduto;
        
        if (statusNF === 'U') {
            ScreenService.showMessage(this.mensagens.xmlUtilizado);
            return;
        }
        
        var row = {"NRACESSONFE": wdg_produtos.currentRow.NRACESSONFE, "NRSEQITPROD": wdg_produtos.currentRow.NRSEQITPROD,"NRAUTOFORN": wdg_produtos.currentRow.NRAUTOFORN};
        
        General.callRestEngine("df_entrada#/updateDesvinculaAfProduto", "Row", row).then(() => {
            ScreenService.showMessage(this.mensagens.afDesvinc);
            wdg_produtos.reload().then( () => {
                verificacoesAf();
            });
        });
    };
    
    /*  Realiza o refresh da widget de romaneio. 
        Motivo: Ao se realizar o refresh pelo metodo widget.refresh(), 
        somente são passados alguns parametros, o que ocasiona em uma 
        resposta com dataset vazio.
        
        @param widget widgetRomaneio
    */
     
    this.refreshWidgetRomaneio = function (widget) {
      return General.callRestEngine("df_entrada#/intgnotafiscal_fornecedor", "FilterData", widget.dataSourceFilter).then((response) => {
            widget.dataSource.data = response.dataset.intgnotafiscal_fornecedor;
            widget.currentRow = response.dataset.intgnotafiscal_fornecedor.filter(row => {
                return row.NRACESSONFE ===  widget.currentRow.NRACESSONFE;
            })[0];
      });  
    };

    this.alteraValorFilialSemNome = function (widget) {

       widget.getField('NMFILIAL').dataSource.data.forEach(filial => filial.NMFILIAL = filial.NMFILIAL === null ? filial.CDFILIAL : filial.NMFILIAL);
       widget.reload();
    };
   

};
Configuration(function (ContextRegister){
    ContextRegister.register('Dfe51000_ImportacaoConsultaNotaFiscal_Controller', Dfe51000_ImportacaoConsultaNotaFiscal_Controller);
});

// FILE: ./assets/modules/entrada/assets/js/controllers/Dfe52000_RelacionamentoProdutoXProduto_Controller.js
var Dfe52000_RelacionamentoProdutoXProduto_Controller = function(ScreenService, MessageService, General, templateManager, ZhFilterPreferences, ZhReports) {

    var cdforneced;
    var nrinsjurforn;
    var nmrazsocforn;
    var nmrazsocfornfil;
    var nmunidade;
    var cdunidade;
    var FRANQUEADORA;

    this.filialDefault = function(args) {
        General.callRestEngine("est#/crud/filial", "FilterData", [{ name: "CDFILIAL", operator: "=", value: "FILIAL_LOGADA" }]).then((response) => {
            args.getField("NMFILIAL").widget.currentRow.NMFILIAL = response.dataset.filial[0].NMFILIAL;
            args.getField("NMFILIAL").widget.currentRow.CDFILIAL = response.dataset.filial[0].CDFILIAL;

        }, (error) => {
            let a = 1;
        });

    };

    this.novoRelacionamento = function(widget) {
        General.callRestEngine("df#/filial_franqueadora", "FilterData", []).then((success) => {
            let franquia = success.dataset.filial_franqueadora.EXISTE_FRANQUEADORA;
            let filial_franqueadora = success.dataset.filial_franqueadora.FRANQUEADORA;
            if (franquia && !filial_franqueadora) {
                MessageService.showMessageByCode('PERMISSAO_RELACIONAMENTO_GERAL');
            }
            else {
                widget.newRow();
                if (widget.appliedFilter) {
                    widget.currentRow.CDFORNECED = widget.appliedFilter.FILTER_ROW.CDFORNECED;
                    widget.currentRow.NMRAZSOCFORN = widget.appliedFilter.FILTER_ROW.NMRAZSOCFORN;
                }

            }
        });
    };

    this.novoRelacionamentoFilial = function(widget) {
        widget.newRow();
        if (widget.appliedFilter) {
            widget.currentRow.CDFORNECED = widget.appliedFilter.FILTER_ROW.CDFORNECED;
            widget.currentRow.NMRAZSOCFORN = widget.appliedFilter.FILTER_ROW.NMRAZSOCFORN;
            widget.currentRow.CDFILIAL = widget.appliedFilter.FILTER_ROW.CDFILIAL;
            widget.currentRow.NMFILIAL = widget.appliedFilter.FILTER_ROW.NMFILIAL;
            widget.currentRow.NRINSJURFORN = widget.appliedFilter.FILTER_ROW.NRINSJURFORN;
        }
    };
    
   this.verificaFranqueadora = function(widget) {
        General.callRestEngine("df#/filial_franqueadora", "FilterData", []).then((success) => {
            let franquia = success.dataset.filial_franqueadora.EXISTE_FRANQUEADORA;
            let filial_franqueadora = success.dataset.filial_franqueadora.FRANQUEADORA;
            if (franquia && !filial_franqueadora) {
                FRANQUEADORA = 'N';
                widget.styles[0].condition = 'true';
            }
            else {
                FRANQUEADORA = 'S';
                widget.styles[0].condition = 'false';
            }
            widget.reload();
        }); 
    };

    this.validaFranqueadora = function(widget) {
        if (FRANQUEADORA == 'N') {
            widget.getAction('edit').readOnly = true;
            widget.getAction('excluir').readOnly = true;
            widget.getAction('replicar').readOnly = true;
        }
        else {
            widget.getAction('edit').readOnly = false;
            widget.getAction('excluir').readOnly = false;
            widget.getAction('replicar').readOnly = false;

        }
        this.setOldCdprodutoxml(widget);
        this.setOldCdproduto(widget);
    };

    this.validacoesWidgetOnEnterOnce = function(filtroArgs) {

        var cabecalhoWidget = filtroArgs.owner;

        const defaultFilter = {
            "NMRAZSOCFORN": "",
            "CDFILIAL": "",
            "NRORG": ""
        };

        General.callRestEngine("est#/crud/filial", "FilterData", [{ name: "CDFILIAL", operator: "=", value: "FILIAL_LOGADA" }]).then((response) => {
            defaultFilter.CDFILIAL = response.dataset.filial[0].CDFILIAL;
            defaultFilter.NMFILIAL = response.dataset.filial[0].NMFILIAL;
            ZhFilterPreferences.setOriginalFilter(cabecalhoWidget.id, defaultFilter);
            this.openFilter(cabecalhoWidget, defaultFilter);
        }, (error) => {
            this.openFilter(cabecalhoWidget, true);

        });

    };


    this.setGlobalVariables = function(currentRow) {
        if (!currentRow) return;
        
        cdforneced = currentRow.CDFORNECED;
        nrinsjurforn = currentRow.NRINSJURFORN;
        if (templateManager.container.getWidget('dfe52000_unidade_grid').isLastActiveWidget()) {
            nmrazsocfornfil = currentRow.NMRAZSOCFORN;
            nmunidade = currentRow.NMFILIAL;
            cdunidade = currentRow.CDFILIAL;
        }
        else {
            nmrazsocforn = currentRow.NMRAZSOCFORN;
        }
    };

    this.setFornecedIfIsNew = function(widget) {
        widget.currentRow.CDFORNECED = cdforneced;
        widget.currentRow.NRINSJURFORN = nrinsjurforn;
        widget.currentRow.NMRAZSOCFORN = nmrazsocforn;
    };

    this.setOldCdprodutoxml = function(widget) {
        if (!widget.currentRow.__is_new) {
            widget.currentRow.OLDCDPRODUTOXML = widget.currentRow.CDPRODUTOXML;
        }
    };
    
    this.setOldCdproduto = function(widget) {
        if (!widget.currentRow.__is_new) {
            widget.currentRow.OLDCDPRODUTO = widget.currentRow.CDPRODUTO;
        }
    };

    this.atualizaNomeFornecedor = function(widget) {
        var fieldFornecedor = widget.getField('Fornecedor');
        var fieldUnidade = widget.getField('Unidade');
        
        var unidadeWidget = templateManager.container.getWidget('dfe52000_unidade_grid');
        var geralWidget = templateManager.container.getWidget('dfe52000_grid');
        
        if (unidadeWidget.isLastActiveWidget()) {
            if (unidadeWidget.appliedFilter) {
                fieldFornecedor.setValue(nrinsjurforn + ' | ' + nmrazsocfornfil);
                fieldUnidade.setValue(cdunidade + ' | ' + nmunidade);
            }
            else {
                fieldFornecedor.setValue('');
                fieldUnidade.setValue('');
            }
            fieldUnidade.isVisible = true;
            fieldFornecedor.class = "8";
        }
        else {
            if (geralWidget.appliedFilter) {
                fieldFornecedor.setValue(nrinsjurforn + ' | ' + nmrazsocforn);
            }
            else {
                fieldFornecedor.setValue('');
            }
            fieldUnidade.isVisible = false;
            fieldFornecedor.class = "12";
        }
    };

    this.getFieldValueFilter = function(fieldName, widgetName) {
        var container = templateManager.container;
        if (container) {
            var filterWidget;
            var filterBforeApply = templateManager.container.getActiveWidget();
            if (filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
                filterWidget = filterBforeApply;
            else {
                var f = container.getWidget(widgetName);
                if (f) {
                    filterWidget = container.getWidget(widgetName).appliedFilter;
                }

            }

            if (filterWidget) {
                var valueField;
                if (filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
                    valueField = filterWidget.currentRow[fieldName];
                else
                    valueField = filterWidget.FILTER_ROW[fieldName];

                return valueField || null;
            }
            else return null;
        }
        else return null;
    };

    this.generateReport = function(sourceWidget, format) {
        var labelWidget = sourceWidget.label;

        sourceWidget.label = "Relacionamento Produto X Produto - " + labelWidget;
        let customData = {};
        ZhReports.generateReport(customData, sourceWidget, this._previewCallback, function() {}, format).
        then (function () {
            sourceWidget.label = labelWidget;
        });
    };

    this._previewCallback = function(data) {
        var popup_window = window.open(data.path, '_blank');
        if (!popup_window) {
            ScreenService.showMessage(ScreenService.i18n('Pop-up Blocker is enabled! Please add this site to your exception list.'));
        }
    };
    
    this.excluirRelacionamento = function(widget) {
        
        MessageService.confirmMessageByCode("REGISTRO_EXCLUIR").then( function () {
            widget.removeRow().then(function() {
                ScreenService.closeSwipe();
                widget.reload();
            });
        });
    };
    
    this.save = function(widget) {
        var cdprodutoxml = widget.currentRow.CDPRODUTOXML;
        
        if(validaCodigoProdutoXML(widget) && widget.currentRow.__is_new) {
            MessageService.showMessageByCode('EXISTE_CDPRODUTOXML', {"CDPRODUTOXML": cdprodutoxml});
        } else {
            General.saveCrud(widget).then(() => {
                MessageService.successNotificationByCode('REGISTRO_SALVO');
            });
        }
    };
    
    var validaCodigoProdutoXML = function(widget) {
        var data = widget.dataSource.data;
        var cdprodutoxml = widget.currentRow.CDPRODUTOXML;
        var valido;
        
        data.forEach(function(element) {
            if(element.CDPRODUTOXML === cdprodutoxml)
                valido = true;
        });
        return valido;
    };
    
    this.adicionarReplicacaoRelacionamentoProduto = function(wdgReplicacao, widgetPai) {

        General.callRestEngine("df#/filial_franqueadora", "FilterData", []).then((success) => {
            let franquia = success.dataset.filial_franqueadora.EXISTE_FRANQUEADORA;
            let filial_franqueadora = success.dataset.filial_franqueadora.FRANQUEADORA;
            if (franquia && !filial_franqueadora) {
                MessageService.showMessageByCode('PERMISSAO_RELACIONAMENTO_GERAL').then((confirm) => {
                    ScreenService.closeSwipe();
                    this.closePopupReplicacaoRelacionamento(wdgReplicacao);
                });
                
            } else {
                wdgReplicacao.newRow();
                this.getCamposOrigemReplicacao(wdgReplicacao, widgetPai);       
            }
        });

    };
    
    this.adicionarReplicacaoRelacionamentoProdutoFilial = function(wdgReplicacao, widgetPai) {
        wdgReplicacao.newRow();
        this.getCamposOrigemReplicacao(wdgReplicacao, widgetPai);                                                                         
    };


    this.openPopupReplicacaoRelacionamento = function(wdgReplicacao, widgetPai) {
        this.getCamposOrigemReplicacao(wdgReplicacao, widgetPai);
        if(widgetPai.name === 'dfe52000_grid') 
            this.adicionarReplicacaoRelacionamentoProduto(wdgReplicacao, widgetPai);
        else 
            this.adicionarReplicacaoRelacionamentoProdutoFilial(wdgReplicacao, widgetPai);
        
        ScreenService.openPopup(wdgReplicacao);
    
    };
    
    this.getCamposOrigemReplicacao = function(wdgReplicacao, widgetPai) {
        wdgReplicacao.currentRow.NMRAZSOCFORNORIG = widgetPai.currentRow.NMRAZSOCFORN;
        wdgReplicacao.currentRow.NMPRODUTO = widgetPai.currentRow.NMPRODUTO;
        wdgReplicacao.currentRow.VRFATCONVOL =  widgetPai.currentRow.VRFATCONVOL;
        wdgReplicacao.currentRow.CDARVPROD = widgetPai.currentRow.CDARVPROD;
        wdgReplicacao.currentRow.CDPRODUTOXML = widgetPai.currentRow.CDPRODUTOXML;
        wdgReplicacao.currentRow.CDPRODUTO = widgetPai.currentRow.CDPRODUTO;
        wdgReplicacao.currentRow.NMPRODUTOXML = widgetPai.currentRow.NMPRODUTOXML;
        
        if(wdgReplicacao.name === 'replicacaoRelacionamentoProdutoFilial') {
            wdgReplicacao.currentRow.CDFILIAL = widgetPai.currentRow.CDFILIAL;
        }
        
    };
    
    this.closePopupReplicacaoRelacionamento = function(wdgReplicacao) {
        ScreenService.closePopup(wdgReplicacao);
        wdgReplicacao.isVisible = false;
    };
    
    this.alteraReadOnlyFornecedor = function(widget) {
        if(widget.name === 'dfe52000_grid') {
            widget.getField('NMRAZSOCFORN').readOnly = true;
        } else if(widget.name === 'dfe52000_unidade_grid'){
            widget.getField('NMRAZSOCFORN').readOnly = true;
            widget.getField('NMFILIAL').readOnly = true;
        } else {
            widget.getField('NMRAZSOCFORN').readOnly = false;
        }
        widget.reload();
    };
    
    this.alteraReadOnlyFornecedorReplicacao = function(widget) {

       var tnrInsJurForn   = widget.currentRow.TNRINSJURFORN;
        var fieldFornecedor = widget.getField("NMRAZSOCFORNDEST");


        if(tnrInsJurForn !== undefined) {
            if( tnrInsJurForn == 'T') {
                fieldFornecedor.clearValue();
                fieldFornecedor.readOnly = true;
            
                if(fieldFornecedor.validations) {
                    if(fieldFornecedor.validations.required) delete fieldFornecedor.validations.required;
                }
                
            } else { 
                fieldFornecedor.readOnly = false;
                
                if(fieldFornecedor.validations) 
                    fieldFornecedor.validations.required = true;
                else 
                    fieldFornecedor.validations = {required : {message: "Obrigatório"}};
                
            }
        } else {
            widget.currentRow.TNRINSJURFORN = 'L';
            this.alteraReadOnlyFornecedorReplicacao(widget);
        }
        
        this.validaCampoFornecedorDestino(widget);
        fieldFornecedor.reload();
    };
    
    this.openFilter = function(widget, defaultFilter) {
        ZhFilterPreferences.openFilterWidget(widget, defaultFilter).then(response => {
            var filter = templateManager.container.getActiveWidget();
            var clearFilter = filter.actions.filter(action => action.name === 'clear')[0];
            var confirmFilter = filter.actions.filter(action => action.name === 'never')[0];
            
            clearFilter.shortcutKey = 'ctrl+backspace';
            clearFilter.tooltip = 'Limpar Filtro (CTRL + Backspace)';
            
            confirmFilter.shortcutKey = 'ctrl + enter';
            filter.reload();
            
        });
    };
    
    this.getFieldFocused = function(widget) {
        return widget.fields.filter(field => field.focused === true)[0];   
    };
    
    this.habilitarFocused = function(field, status) {
        if(status == true) {
            field.focused = true;
        } else {
            delete field.focused;
        }
    };
    
    this.saveReplic = function(widget) {
        if(widget.currentRow.NMRAZSOCFORNDEST !== 'undefined' && widget.currentRow.NMRAZSOCFORNDEST !== "") {
            var cdFornecedores = widget.currentRow.CDFORNECED;
            var cdFornecedorigem = widget.lastActiveWidget.appliedFilter.FILTER_ROW.CDFORNECED;
            var row;
            
            if(widget.lastActiveWidget.label == "Geral"){
                row = { CDFORNECEDORES: cdFornecedores, CDFORNECEDORIGEM: cdFornecedorigem};
                
                General.callRestEngine("df#/realizaReplicacao", "Row", row).then((success) =>  {
                    MessageService.successNotificationByCode('REPLICACAO_REALIZADA');
                },
                (error) => {
                    MessageService.errorNotificationByCode("ERRO_REPLICACAO");
                });
                this.closePopupReplicacaoRelacionamento(widget);
            } else {
                var cdFilial = widget.lastActiveWidget.appliedFilter.FILTER_ROW.CDFILIAL;
                row = { CDFORNECEDORES: cdFornecedores, CDFORNECEDORIGEM: cdFornecedorigem, CDFILIAL: cdFilial};
                
                General.callRestEngine("df#/realizaReplicacaoFilial", "Row", row).then((success) =>  {
                    MessageService.successNotificationByCode('REPLICACAO_REALIZADA');
                },
                (error) => {
                    MessageService.errorNotificationByCode("ERRO_REPLICACAO");
                });
                this.closePopupReplicacaoRelacionamento(widget);
            }
        } else {
            MessageService.showMessageByCode('FORNECEDOR_DESTINO_OBRIGATORIO');
        }
    };
    
    this.bloqueiaAcao = function(action, status) {
        if(status) 
            action.aclReadOnly = true;
        else 
            action.aclReadOnly = false;

    };
    
    this.validaCampoFornecedorDestino = function(widget) {
        var fonecedDestino  = widget.currentRow.NMRAZSOCFORNDEST;
        var tnrInsJurForn   = widget.currentRow.TNRINSJURFORN;
        var action          =     widget.actions.filter(action => action.name === 'salvar')[0];
        
        if( fonecedDestino === "" && tnrInsJurForn == 'L')
            this.bloqueiaAcao(action, true);
        else 
            this.bloqueiaAcao(action, false);

    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfe52000_RelacionamentoProdutoXProduto_Controller", Dfe52000_RelacionamentoProdutoXProduto_Controller);
});


// FILE: ./assets/modules/entrada/assets/js/controllers/Dfe58000_ManifestacaoDestinatario_Controller.js
var dfe58000_ManifestacaoDestinatario_Controller = function(General, templateManager, ScreenService, Dfe58000_ManifestacaoDestinatario_Service, ZhFilterPreferences) {
    

    
    var messages = {
        confirmaDesconhecimento: 'Atenção! Após confirmar a operação não será mais possível importar o XML da mesma. Deseja confirmar a operação?',
        nenhumaNota: 'Não é possivel realizar a operação para a(s) nota(s) selecionadas.',
        confirmarOperacao: 'Atenção! Deseja confirmar a operação para todas as notas marcadas?'
        
    };
    
    var CDEMPRESA;
    var CDINSCESTA;
    
    this.setDefaultFilter = function(args) {
        var widget     = args.owner;
        var currentRow = widget.currentRow;
        var row        = { CDFILIAL: getLocalVar('CDFILIAL')};
        General.callRestEngine("df_entrada#/defaultFilterManif", "Row", row).then((defaultFilter) =>  {
            defaultFilter = defaultFilter.dataset.defaultFilter;
            var filter = Util.clone(currentRow, ["__rightActions", "__leftActions"]);
            for(var key in defaultFilter) {
                if(key == 'DTEMISSAO') {
                    filter.DTEMISSAO = {
                        start:  defaultFilter[key],
                        period: undefined,
                        end:    defaultFilter[key]
                    };
                }
                else {
                    filter[key] = defaultFilter[key];
                }
            }
            widget.getField('NRINSJURIEST').readOnly    = false;
            widget.getField('CDINSCESTA').readOnly      = false;
            ZhFilterPreferences.setOriginalFilter(widget.id, filter);
            ZhFilterPreferences.openFilterWidget(widget, filter);
            this.coloreLegenda(widget);
        },
        (error) => {
            ZhFilterPreferences.openFilterWidget(widget);
            this.coloreLegenda(widget);
        });
    };
    
    var requestError = function(error){
        var errorMessage = "Ocorreu um erro ao tentar realizar a operação.";
        if (error && error.data) {
            errorMessage = error.data.error;
        }
        ScreenService.showMessage(errorMessage, "ERROR");
    };
    
    this.coloreLegenda = function(widgetGrid){
        var widget = widgetGrid.container.getWidget('legendaStripe');
        widget.getField('LEGENDAGRAY').element[0].style.color = 'gray';
        widget.getField('LEGENDARED').element[0].style.color = 'red';
        widget.getField('LEGENDAGREEN').element[0].style.color = 'green';
        widget.getField('LEGENDABLUE').element[0].style.color = 'blue';
    };

    var validaCampoCnpj = function(manifestacaoArgs) {
        var cnpj = manifestacaoArgs.owner.field.widget.getField("NRINSJURIEST");
        cnpj.readOnly = !manifestacaoArgs.row.CDEMPRESA;
        cnpj.setValue('');
        validaCampoInscricaoEstadual(manifestacaoArgs);
    };

    var validaCampoInscricaoEstadual = function(manifestacaoArgs) {
        var inscricaoEstadual = manifestacaoArgs.owner.field.widget.getField("CDINSCESTA");
        inscricaoEstadual.readOnly = !manifestacaoArgs.row.NRINSJURIEST;
        inscricaoEstadual.setValue('');
    };

    this.empresaOnChange = function(manifestacaoArgs) {
        validaCampoCnpj(manifestacaoArgs);
    };

    this.cnpjOnChange = function(manifestacaoArgs) {
        validaCampoInscricaoEstadual(manifestacaoArgs);
    };

    var getManifestacaoWidget = function(){
        if(templateManager.container)
            return templateManager.container.getWidget("manifestacao");
    };

    this.getCurrentField = function(field) {
        var manifestacaoWidget = getManifestacaoWidget();
        if (manifestacaoWidget)
            return !manifestacaoWidget.currentRow[field] ? null : manifestacaoWidget.currentRow[field];
    };

    var getFilterPopupCurrentRow = function(args){
        return args.e.preference.FILTER_ROW;
    };

    this.getEmpresaFiltrada = function(){
        var manifestacaoWidget = getManifestacaoWidget();
        if(manifestacaoWidget){
            return this.CDEMPRESA;
        }
    };
    
    this.setFilterFields = function(widget, field){
        field.dataSourceFilter.forEach(function(filter){
            if(widget.name == 'zhFilterPreferencesSelectConditions')
                filter.value = widget.currentRow[filter.name] || 'TD';
            else
                filter.value = widget.appliedFilter.FILTER_ROW[filter.name] || 'TD';
        });
        field.reload();
    };

    this.getFieldFromFilterPopup = function(field){
        var manifestacaoWidget = getManifestacaoWidget();
        if(manifestacaoWidget){
            var widgetAtiva = manifestacaoWidget.getParent().getActiveWidget();
            if(widgetAtiva.name == 'zhFilterPreferencesSelectConditions')
                return widgetAtiva.currentRow[field] || 'TD';
            else
                return manifestacaoWidget.appliedFilter.FILTER_ROW[field] || 'TD';
        }
    };

    this.idorigdestOnChange = function(filterArgs) {
        alteraVisibilidadeCampoInscricaoFiltro(filterArgs);
        limpaCamposOutDataSelectsTpOperacFiltro(filterArgs);
    };

    var alteraVisibilidadeCampoInscricaoFiltro = function(filterArgs) {
        var filtroWidget = filterArgs.owner.field.widget;
        var cliente = filtroWidget.getField("NMRAZSOCCLIE");
        var fornecedor = filtroWidget.getField("NMRAZSOCFORN");
        var filial = filtroWidget.getField("DSFILIEMIT");
        var tipoDestino = filterArgs.row.IDORIGDESTNF;

        switch (tipoDestino) {
            case "C":
                validacaoCamposOutTpOperacFilter(cliente, fornecedor, filial);
                break;
            case "F":
                validacaoCamposOutTpOperacFilter(fornecedor, cliente, filial);
                break;
            case "L":
                validacaoCamposOutTpOperacFilter(filial, fornecedor, cliente);
                break;
            default:
                filial.readOnly = true;
                cliente.readOnly = true;
                fornecedor.readOnly = true;
        }
    };

    var limpaCamposOutDataSelectsTpOperacFiltro = function(filterArgs)
    {
        var filtroWidget = filterArgs.owner.field.widget;
        var cabecalhoRow = filtroWidget.currentRow;
        var cliente = filtroWidget.getField("NMRAZSOCCLIE");
        var fornecedor = filtroWidget.getField("NMRAZSOCFORN");
        var filial = filtroWidget.getField("DSFILIEMIT");

        filial.value(null);

        cliente.value(null);
        cabecalhoRow.CDCLIENTE = null;

        fornecedor.value(null);
        cabecalhoRow.CDFORNECED = null;
        cabecalhoRow.IDSITUCADA = null;
        cabecalhoRow.IDBLQNFRELAFOAF = null;

        cabecalhoRow.NRINSJURNF = null;
        cabecalhoRow.NMINSJURNF = null;
        cabecalhoRow.IDTPINSJURNF = null;
    };

    var validacaoCamposOutTpOperacFilter = function(princialField) {
        princialField.showOnView = true;
        princialField.showOnForm = true;
        princialField.isVisible = true;
        princialField.readOnly = false;

        for (var i = 1; i < arguments.length; i++) {
            arguments[i].showOnView = false;
            arguments[i].showOnForm = false;
            arguments[i].isVisible = false;
            arguments[i].readOnly = true;
        }
    };

    this.criaDataSourceGridFilterBeforeApply = function(args) {
        args.e.preventDefault();
        var manifestacaoWidget = args.owner;

        
        if(manifestacaoWidget.isValid()) {
            var dataSourceFilter = manifestacaoWidget.dataSourceFilter;
            var currentRow       = args.e.preference.FILTER_ROW;
            this.CDEMPRESA          = currentRow.CDEMPRESA;
            this.CDINSCESTA         = currentRow.CDINSCESTA;
            
            dataSourceFilter = dataSourceFilter.filter(function(value) {
                return value.name !== 'P_DTEMISSAOINI' && value.name !== 'P_DTEMISSAOFIN' && (value.isCustomFilter !== true); 
            });

            var filterDefault = {
                'operator': '='
            };
            
            var datasFiltro = [{name: "DTEMISSAO"}];
            
            dataSourceFilter.forEach((data) => {
                if(data.name != 'DTEMISSAO' && data.name != 'P_IDENTRSAIDNF' && data.name != 'P_IDENTRSAIDOP') {
                    if(data.name == "P_NRINSJUREMIT") {
                        data.value = currentRow.NRINSJURNF || 'TD';
                    }
                    else {
                        data.value = currentRow[data.name.replace("P_", "")] || 'TD';
                    }
                }
            });
            datasFiltro.forEach(function(data) {
                filterDefault.name = 'P_' + data.name + 'INI';
                filterDefault.value = currentRow.DTEMISSAO.start;
                dataSourceFilter.push(Util.clone(filterDefault));
                filterDefault.name = 'P_' + data.name + 'FIN';
                filterDefault.value = currentRow.DTEMISSAO.end;
                dataSourceFilter.push(Util.clone(filterDefault));
    
                if(dataSourceFilter.indexOf(data) > 0)
                    dataSourceFilter.splice(dataSourceFilter.indexOf(data), 1);
            });
            manifestacaoWidget.dataSourceFilter = dataSourceFilter;
            manifestacaoWidget.reload().then(() => {
                manifestacaoWidget.activate();
            });
            ScreenService.closePopup();
        }
    };

    this.abrirPopupJustificativa = function(args){
        var notas = args.owner.widget.getCheckedRows().filter(function(notafiscal){
                    return !notafiscal.NRLANCTONF && notafiscal.IDSTATUSMANIF !== "3";
                });

        if(notas.length > 0){
            var popupJustificativa = args.owner.widget.container.getWidget('popupJustificativa');
            ScreenService.openPopup(popupJustificativa);
        }
        else{
            ScreenService.showMessage(messages.nenhumaNota);
        }
    };

    this.montaConfig = function(args){
        Dfe58000_ManifestacaoDestinatario_Service.montaConfigTransmissao(getFilterPopupCurrentRow(args)).then(function(resp){
        }, requestError);
        args.e.preference.CUSTOM_FILTER_ROW = {};
    };

    this.manifestarCiencia = function(args){
        var notas = args.owner.widget.getCheckedRows().filter(function(notafiscal){
            return (notafiscal.IDSTATUSMANIF === "0" || notafiscal.IDSTATUSMANIF === "5") && (notafiscal.IDSTATUSNFE != 2 && notafiscal.IDSTATUSNFE != 3);
        });
        
        var evento = '210210';
        var justificativa = '';

        if(notas.length > 0){
            ScreenService.confirmMessage(ScreenService.i18n(messages.confirmarOperacao), 'question', function (success) {
   
                Dfe58000_ManifestacaoDestinatario_Service.manifestarNfe(notas, evento, justificativa).then(function(callback){
                    if(callback[0].SUCESSO) ScreenService.showMessage(callback[0].SUCESSO);
                    else if(callback[0].ERRO) this.openPopupErros(callback[0].ERRO);
                    args.owner.widget.reload();
                }.bind(this), function(error) {
                    ScreenService.showMessage(error.data.error);
                    args.owner.widget.reload();
                });
            }.bind(this));
        }
        else{
            ScreenService.showMessage(messages.nenhumaNota);
        }
    };
    
    this.openPopupErros = function(erros) {
        var widgetErros = templateManager.container.getWidget('WidgetErros');
        widgetErros.dataSource.data = erros;
        widgetErros.isVisible = true;
        ScreenService.openPopup(widgetErros);
        widgetErros.activate();
    };
    
    this.closeErros = function(widget) {
        widget.isVisible = false;
        widget.dataSource.data = [];
        ScreenService.closePopup(widget);
        widget.getParent().activate();
        widget.getParent().unSelectAll();
        widget.getParent().reload();
        ScreenService.closePopup();
    };

    this.manifestarConfirmacao = function(args){
        var notas = args.owner.widget.getCheckedRows().filter(function(notafiscal){
                    return (notafiscal.IDSTATUSMANIF !== "1") && (notafiscal.IDSTATUSNFE != 2 && notafiscal.IDSTATUSNFE != 3);
                });
        var evento = '210200';
        var justificativa = '';

        if(notas.length > 0){
            ScreenService.confirmMessage(ScreenService.i18n(messages.confirmarOperacao), 'question', function (success) {
                Dfe58000_ManifestacaoDestinatario_Service.manifestarNfe(notas, evento, justificativa).then(function(callback){
                    if(callback[0].SUCESSO) ScreenService.showMessage(callback[0].SUCESSO);
                    else if(callback[0].ERRO) this.openPopupErros(callback[0].ERRO);
                    args.owner.widget.reload();
                }.bind(this), function(error) {
                    ScreenService.showMessage(error.data.error);
                    args.owner.widget.reload();
                });
            }.bind(this));
        }
        else{
            ScreenService.showMessage(messages.nenhumaNota);
        }
    };

    this.manifestarDesconhecimento = function(args){
        var notas = args.owner.widget.getCheckedRows().filter(function(notafiscal){
            return (!notafiscal.NRLANCTONF && notafiscal.IDSTATUSMANIF !== "2") && (notafiscal.IDSTATUSNFE != 2 && notafiscal.IDSTATUSNFE != 3);
        });
        
        var evento = '210220';
        var justificativa = '';

        if(notas.length > 0){	
            ScreenService.confirmMessage(ScreenService.i18n(messages.confirmaDesconhecimento), 'question', function (success) {
                Dfe58000_ManifestacaoDestinatario_Service.manifestarNfe(notas, evento, justificativa).then(function(callback){
                    if(callback[0].SUCESSO) ScreenService.showMessage(callback[0].SUCESSO);
                    else if(callback[0].ERRO) this.openPopupErros(callback[0].ERRO);
                    args.owner.widget.reload();
                }.bind(this), function(error) {
                    ScreenService.showMessage(error.data.error);
                    args.owner.widget.reload();
                });
            }.bind(this));
        }
        else{
            ScreenService.showMessage(messages.nenhumaNota);
        }
    };

    this.manifestarNaoRealizada = function(args){
        var notas = args.owner.widget.getParent().getCheckedRows().filter(function(notafiscal){
                    return (!notafiscal.NRLANCTONF && notafiscal.IDSTATUSMANIF !== "3") && (notafiscal.IDSTATUSNFE != 2 && notafiscal.IDSTATUSNFE != 3);
                });
        var evento = '210240';
        var justificativa = args.owner.widget.currentRow.JUSTIFICATIVA;

        if(!justificativa || justificativa.trim().length < 15){
            ScreenService.showMessage('Justificativa deve conter no mínimo 15 caracteres', 'ERROR');
        }
        else{
            if(notas.length != 0) {
                ScreenService.confirmMessage(ScreenService.i18n(messages.confirmaDesconhecimento), 'question', success => {
                    Dfe58000_ManifestacaoDestinatario_Service.manifestarNfe(notas, evento, justificativa).then( callback => {
                        ScreenService.closePopup();
                        if(callback[0].SUCESSO) ScreenService.showMessage(callback[0].SUCESSO);
                        else if(callback[0].ERRO) this.openPopupErros(callback[0].ERRO);
                        args.owner.widget.getParent().reload();
                    }, error => {
                        ScreenService.showMessage(error.data.error);
                        args.owner.widget.getParent().reload();
                    });
                });
            }
            else {
                ScreenService.showMessage(messages.nenhumaNota);
                ScreenService.closePopup();
                args.owner.widget.getParent().reload();
            }
            
        }
    };

    this.gerarXML = function(args){
        var notasMarcadas = args.owner.widget.getCheckedRows();
        var notas = notasMarcadas.filter(function(notafiscal){
            return notafiscal.IDSTATUSMANIF == "1" || notafiscal.IDSTATUSMANIF == "4";
        });
        
        if(notas.length > 0){

            Dfe58000_ManifestacaoDestinatario_Service.gerarXML(notas).then(function(response){
                 window.open(window.serviceUrls.filter(function(x){return x.package == 'df';})[0].baseUrl+"/downloadArquivo?requestType=Row&row[DSARQUIVO]=" + response.dataset.gerarXmlNfeManifestacao[0]);
            }, function(error) {
                ScreenService.showMessage(error.data.error);
            });
        }
        else{
            ScreenService.showMessage(messages.nenhumaNota);
        }
    };

    this.sincronizarNFe = function(widget){
        var manifesacao = templateManager.container.getWidget('manifestacao');
        widget.currentRow.CDEMPRESA  = this.CDEMPRESA;
        widget.currentRow.CDINSCESTA = this.CDINSCESTA;
        Dfe58000_ManifestacaoDestinatario_Service.sincronizarNFe(widget.currentRow).then((callback) => {
            this.closePopupSincronizacao();
            ScreenService.successNotification("Sincronização Realizada com Sucesso!");
            widget.getParent().reload();
            this.openPopupResumo(callback);
        }, (error) => {
            ScreenService.errorNotification(error.data.error);
        });
    };
    
    this.openPopupResumo = function(sincronizacao) {
        var wdgResumo = templateManager.container.getWidget('RESUMOSINC');
        wdgResumo.isVisible = true;
        if(sincronizacao.dataset.sincronizarNfeManifestacao[0]) {
            wdgResumo.currentRow = sincronizacao.dataset.sincronizarNfeManifestacao[0].DADOS;
            wdgResumo.widgets[0].dataSourceFilter[0].value = sincronizacao.dataset.sincronizarNfeManifestacao[0].CHAVES; 
            wdgResumo.widgets[0].dataSourceFilter[0].operator = "IN";
            wdgResumo.widgets[0].reload();
            ScreenService.openPopup(wdgResumo);
        }
        
        
    };
    
    this.closePopupResumo = function() {
        var wdgSinc = templateManager.container.getWidget('RESUMOSINC');
        ScreenService.closePopup();
        wdgSinc.isVisible = false;
    };
    
    this.openPopupSincronizacao = function() {
        var wdgSinc = templateManager.container.getWidget('WidgetSincronizacao');
        wdgSinc.currentRow.FORMAPESQUISA    = "0";
        wdgSinc.isVisible = true;
        ScreenService.openPopup(wdgSinc);
    };
    
    this.closePopupSincronizacao = function() {
        var wdgSinc = templateManager.container.getWidget('WidgetSincronizacao');
        ScreenService.closePopup();
        wdgSinc.isVisible = false;
    };
    
    this.closePopup = function(widget) {
        widget.isVisible = false;
        widget.dataSource.data = [];
        ScreenService.closePopup(widget);
        widget.getParent().activate();
        widget.getParent().unSelectAll();
        widget.getParent().reload();
        ScreenService.closePopup();
    };
    
    this.openConsultaNsuNfe = function(widget) {
        var widgetNsu = templateManager.container.getWidget('popupConsultaNsu');
        widgetNsu.isVisible = true;
        
        Dfe58000_ManifestacaoDestinatario_Service.openConsultaNsuNfe(widget, widgetNsu);
    };
    
    this.closePopupNsuNfe = function(widget) {
        widget.isVisible = false;
        widget.widgets[0].dataSource.data = [];
        ScreenService.closePopup(widget);
        widget.getParent().activate();
        widget.getParent().unSelectAll();
        widget.getParent().reload();
    };
    
    this.onChangeDtRecbtoNsu = function(widget) {
        
        widget.widgets[0].dataSourceFilter[2].value = widget.currentRow.DTRECBTO.start;
        widget.widgets[0].dataSourceFilter[3].value = widget.currentRow.DTRECBTO.end;
        
        widget.widgets[0].reload();
    };

};
    

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfe58000_ManifestacaoDestinatario_Controller", dfe58000_ManifestacaoDestinatario_Controller);
});

// FILE: ./assets/modules/entrada/assets/js/controllers/Dfe60000_ManifestacaoDestinatario_Controller.js
var Dfe60000_ManifestacaoDestinatario_Controller = function(ScreenService, General) {
    var self = this;
    
    var popupOpen = false;
    var isVazio;
    
    self.save = function(widget) {
        if (!popupOpen) {
            if(isVazio) {
                widget.currentRow.__is_new = true;
                General.widgetSave(widget);
                isVazio = false;
            } else {
                General.widgetSave(widget);
            }
        }
    };

    self.validaDataFinalmanifestacao = function(field){
        var prazoMaximo = field.widget.getField('NRDIASCONCLUSAO');
        var aviso = field.widget.getField('NRDIASAVCONCL');
        var bloqueio = field.widget.getField('NRDIASBLCONCL');
        var element = field.element;
        if (field.name == 'NRDIASAVCONCL' && !prazoMaximo.value()){
            popupOpen = true;
            ScreenService.showMessage('Prazo máximo para manifestação conclusiva deve ser informado.').then(function(){
                field.widget.getField('NRDIASCONCLUSAO').focus();
                popupOpen = false;
            });
        }else {
            if (aviso.value() > prazoMaximo.value() || bloqueio.value() > prazoMaximo.value()) {
                popupOpen = true;
                ScreenService.showMessage('Número de dias anteriores à data final de manifestação conclusiva deve ser menor que o prazo máximo.').then(function () {
                    field.setValue(field.widget.backupRow[field.name]);
                    element.focus();
                    popupOpen = false;
                });
            }
            else {
                field.widget.backupRow[field.name] = field.value();
            }
        }
    };

    self.justInteger = function (campos){
        campos.forEach(function(campo){
            General.justInteger(campo);
        });
    };

    self.validaDiasDownload = function (field){
        var element = field.element;
        if (field.value() > 30){
            var msg = "A quantidade de dias para download automático deve ser menor que 31 dias.";
            popupOpen = true;
            ScreenService.showMessage(msg).then (() => {
               element.focus();
               popupOpen = false;
            });
            field.widget.currentRow.NRDIASDOWNLOD = field.widget.backupRow.NRDIASDOWNLOD;
        }else{
            field.widget.backupRow.NRDIASDOWNLOD = field.widget.currentRow.NRDIASDOWNLOD;
        }
    };

    self.validaDiaFechamento = function(field){
        var element = field.element;
        if (field.value() > 30){
            var msg = "Dia para fechamento deve ser menor que 31 dias.";
            popupOpen = true;
            ScreenService.showMessage(msg).then (() => {
               element.focus();
               popupOpen = false;
            });
            field.widget.currentRow.NRDIAFECHMANIF = field.widget.backupRow.NRDIAFECHMANIF;
        }else{
            field.widget.backupRow.NRDIAFECHMANIF = field.widget.currentRow.NRDIAFECHMANIF;
        }
    };
    
    self.desabilitaValues = function(field, values) {
        var fieldsToDisable;
        var fieldsToEnable;
        if (field.dataSource.rawdata[0].IDCIENCIAEMISS != null) {
            //Field idcienciaemiss
            fieldsToDisable = field.dataSource.rawdata.filter(value => {
            return values.includes(value.IDCIENCIAEMISS);
            });
            
            fieldsToEnable = field.dataSource.rawdata.filter(value => {
                return !values.includes(value.IDCIENCIAEMISS);
            });
        }
        else {
            //Field idconfirmaemiss
            fieldsToDisable = field.dataSource.rawdata.filter(value => {
            return values.includes(value.IDCONFIRMAEMISS);
            });
            
            fieldsToEnable = field.dataSource.rawdata.filter(value => {
                return !values.includes(value.IDCONFIRMAEMISS);
            });
        }
        
        fieldsToDisable.forEach(field => {
            field.ENABLE = false;
        });
        
        fieldsToEnable.forEach(field => {
            field.ENABLE = true;
        });
    };

    self.validaCamposEmissaoConfirmacao = function (widget) {
        var idcienciaemiss = widget.getField('IDCIENCIAEMISS');
        var idconfirmaemiss = widget.getField('IDCONFIRMAEMISS');
        
        /*0: Não Realizar Automaticamente
          1: Na Importação de XML
          2: No Lançamento de Entrada
          3: Na Aprovação da Nota
          4: Na Liberação para Financeiro
        */
        
        switch (idcienciaemiss.value()) {
            case "0" : self.desabilitaValues(idconfirmaemiss, []); break;
            case "1" : self.desabilitaValues(idconfirmaemiss, ["1"]); break;
            case "2" : self.desabilitaValues(idconfirmaemiss, ["1", "2"]); break;
            case "3" : self.desabilitaValues(idconfirmaemiss, ["1", "2", "3"]); break;
            case "4" : self.desabilitaValues(idconfirmaemiss, ["1", "2", "3", "4"]); break;
        }
        
        switch (idconfirmaemiss.value()) {
            case "0" : self.desabilitaValues(idcienciaemiss, []); break;
            case "1" : self.desabilitaValues(idcienciaemiss, ["1", "2", "3", "4"]); break;
            case "2" : self.desabilitaValues(idcienciaemiss, ["2", "3", "4"]); break;
            case "3" : self.desabilitaValues(idcienciaemiss, ["3", "4"]); break;
            case "4" : self.desabilitaValues(idcienciaemiss, ["4"]); break;
        }
        
        self.alteraVisibilidadeCienciaConfirmacao(idcienciaemiss, idconfirmaemiss);
    };

    self.alteraVisibilidadeCienciaConfirmacao = function(idcienciaemiss, idconfirmaemiss){
        idcienciaemiss.dataSource.data = idcienciaemiss.dataSource.rawdata.filter(function(data){
            if (data.ENABLE){
                return data;
            }
        });

        idconfirmaemiss.dataSource.data = idconfirmaemiss.dataSource.rawdata.filter(function(data){
            if (data.ENABLE){
                return data;
            }
        });
        
        idconfirmaemiss.reload();
        idcienciaemiss.reload();
        idcienciaemiss.widget.redraw();
    };

    self.setCurrentRowForm = function (widget){
        if (widget.dataSource.data.length) {
            widget.currentRow = widget.dataSource.data[0];
        }
        widget.mode = 'view';
    };

    self.unMask = function(str) {
        if(typeof str === 'string' && str.indexOf('R$') != -1)
            str = $.currency.unmaskString(str, {"thousands": "."});
        else if(typeof str === 'string' && str.indexOf('%') != -1)
            str = parseFloat(str.replace(',', '.'));
        else str = str.replace(/[\D]+/g,'');
        return str;
    };
    
    self.inicializaNmParManifest = function(widget) {
        var field = widget.getField('NRPARMANIFEST');
        field.applyDefaultValue();
    };
    
    self.verificaDataSourceVazio = function(widget){
        isVazio = false;
        if(widget.dataSource.data.length == 0) {
            isVazio = true;
        }
    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfe60000_ManifestacaoDestinatario_Controller", Dfe60000_ManifestacaoDestinatario_Controller);
});

// FILE: ./assets/modules/entrada/assets/js/controllers/Dfe64000_IntegracaoCfopXmlComTipoDeOperacao_Controller.js
var Dfe64000_IntegracaoCfopXmlComTipoDeOperacao_Controller = function(ScreenService, templateManager, ZhFilterPreferences, Dfe64000_IntegracaoCfopXmlComTipoDeOperacao_Service) {
    
    this.novoRelacionamento = function (widget) {
        widget.newRow();
        widget.currentRow.CDTIPOOPER = widget.appliedFilter.FILTER_ROW.CDTIPOOPER;
        widget.currentRow.NMTIPOOPER = widget.appliedFilter.FILTER_ROW.NMTIPOOPER;
    };
    
    this.novoRelacionamentoFilial = function (widget) {
        widget.newRow();
        widget.currentRow.CDTIPOOPER = widget.appliedFilter.FILTER_ROW.CDTIPOOPER;
        widget.currentRow.NMTIPOOPER = widget.appliedFilter.FILTER_ROW.NMTIPOOPER;
        widget.currentRow.CDFILIAL = widget.appliedFilter.FILTER_ROW.CDFILIAL;
        widget.currentRow.NMFILIAL = widget.appliedFilter.FILTER_ROW.NMFILIAL;
    };
    
    this.filterAfterApply = function (widget) {
        
        var stripeWidget = templateManager.container.getWidget("stripe");
        
        var campoFiltroTipoOper = stripeWidget.getField("NMTIPOOPERFILTRO");
        var campoFiltroUnidade = stripeWidget.getField("NMFILIALFILTRO");
        
        if (widget.appliedFilter === undefined) {
            campoFiltroTipoOper.setValue('');
            campoFiltroUnidade.setValue('');
            return;
        }
        
        var CDtipoOper = widget.appliedFilter.FILTER_ROW.CDTIPOOPER;
        var NMtipoOper = widget.appliedFilter.FILTER_ROW.NMTIPOOPER;
        
        campoFiltroTipoOper.setValue(CDtipoOper + " | " + NMtipoOper);
        
        if (widget.name === "Dfe64000_grid_filial") {
            var CDfilial   = widget.appliedFilter.FILTER_ROW.CDFILIAL;
            var NMfilial   = widget.appliedFilter.FILTER_ROW.NMFILIAL;

            campoFiltroUnidade.setValue(CDfilial + " | " + NMfilial);
        }
    };
    
    this.setDefaultFilialFilter = function(widget) {
        let popupFilter = Util.clone(widget.currentRow);
        
        Dfe64000_IntegracaoCfopXmlComTipoDeOperacao_Service.getFilialLogada().then( (filialLogada) =>  {
            let geralWidget = templateManager.container.getWidget("Dfe64000_grid_geral");
            let filial;
            
            //Utiliza o primeiro dataset retornado. Necessário pois o dataset pode ser filioper_lancamento_notafiscal ou perfacessfili_lancamento_notafiscal
            for (let datasetName in filialLogada.dataset) {
                if (filialLogada.dataset.hasOwnProperty(datasetName)) {
                    filial = filialLogada.dataset[datasetName][0];
            	    break;
                }
            }
            if (filial) {
                popupFilter.CDFILIAL = filial.CDFILIAL;
                popupFilter.NMFILIAL = filial.NMFILIAL;
            }
            else {
                popupFilter.CDFILIAL = '';
                popupFilter.NMFILIAL = '';
            }
            if (geralWidget.appliedFilter) {
                popupFilter.NMTIPOOPER = geralWidget.appliedFilter.FILTER_ROW.NMTIPOOPER;
                popupFilter.CDTIPOOPER = geralWidget.appliedFilter.FILTER_ROW.CDTIPOOPER;
            }
            ZhFilterPreferences.setOriginalFilter(widget.id, popupFilter);
            ZhFilterPreferences.openFilterWidget(widget, popupFilter);
        }); 
    };
    
    this.changeTabs = function (widget) {
        
        var stripeWidget = templateManager.container.getWidget("stripe");
        var campoFiltroUnidade = stripeWidget.getField("NMFILIALFILTRO");
        
        if (widget.name === "Dfe64000_grid_geral") {
            campoFiltroUnidade.isVisible = false;
        }
        else {
            campoFiltroUnidade.isVisible = true;
        }
        
        this.filterAfterApply(widget);
        
    };
    
    var requestError = function(error){
        var errorMessage = "Ocorreu um erro ao tentar realizar a operação.";
        if (error && error.data) {
          errorMessage = error.data.error;
        }
        if (error.type !== "click") {
            ScreenService.showMessage(errorMessage, "ERROR"); 
        }
    };
    
    this.salvar = function(widget){
        Dfe64000_IntegracaoCfopXmlComTipoDeOperacao_Service.salvar(widget);
    };
    
    this.deletar = function(widget){
        ScreenService.confirmMessage('Excluir Registro?', 'question', function (success) {
            Dfe64000_IntegracaoCfopXmlComTipoDeOperacao_Service.deletar(widget);}, requestError);
    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfe64000_IntegracaoCfopXmlComTipoDeOperacao_Controller", Dfe64000_IntegracaoCfopXmlComTipoDeOperacao_Controller);
});

// FILE: ./assets/modules/entrada/assets/js/controllers/Ger97500_Tipo_De_Operacao_x_Cfop_Controller.js
var Ger97500_Tipo_De_Operacao_x_Cfop_Controller = function(ScreenService, ZhFilterPreferences, templateManager, General, MessageService) {
    var self = this;

    var getMainWidget = function() {
        if (templateManager.container)
            return templateManager.container.widgets[0];
    };

    self.getFieldValue = function(field) {
        var mainWidget = getMainWidget();
        if (mainWidget) {
            var widgetAtiva = mainWidget.getParent().getActiveWidget();
            if (widgetAtiva.label == 'Filtro'){
                return widgetAtiva.currentRow[field];
            } else if (widgetAtiva.label == 'Tipo de Operação' && widgetAtiva.lastActiveWidget.label == 'Filtro'){
                return widgetAtiva.lastActiveWidget.currentRow[field];
            } else{
                return mainWidget.appliedFilter.FILTER_ROW[field];
            }
        }
    };

    self.justInteger = function(campos) {
        campos.forEach(function(campo) {
            General.justInteger(campo);
        });
    };
    
    this.deleteTipoOperCfop = function (parentWidget){
        MessageService.confirmMessageByCode('REGISTRO_EXCLUIR').then(sucess => {
            parentWidget.removeRow().then(function () {
                ScreenService.closeSwipe();
                MessageService.successNotificationByCode('REGISTRO_EXCLUIDO');
                parentWidget.reload();
                parentWidget.activate();
            }, function (error) {
                if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                    MessageService.showMessageByCode('ERRO_EXCLUSAO_NAO_PERMITIDA');
                } else {
                    ScreenService.showMessage(error.data.error);
                }
            });
            
        });
    };
    
    this.controlaStatusWidgets = function(widgetPai, widgetAtual, status, ignoredWidgets) {
        var allWidgets = widgetPai.widgets;

        allWidgets.filter(widget => widget.name !== widgetAtual.name).forEach(widget => widget.enabled = status);

        if (widgetPai.name != widgetAtual.name)
            widgetPai.enabled = status;

        if (Array.isArray(ignoredWidgets))
            allWidgets.filter(widget => ignoredWidgets.indexOf(widget.name) !== -1).forEach(widget => widget.enabled = !status);
    };
    
    this.esOnChange = function (args){
        let wid = args.owner.owner.widget;
        let codTipoOp = wid.getField("CDTIPOOPER");
        codTipoOp.setValue(null);
        codTipoOp.reload();
    };
    
    this.corrigeFiltro = function (args) {
        let filtroES = args.owner.widget.appliedFilter.FILTER_ROW.IDENTRSAIDOP;
        args.owner.widget.currentRow.IDENTRSAIDOP = filtroES;
    };
    
    this.openFilter = function(args, identrsaidop){
        ZhFilterPreferences.openFilterWidget(args.owner, {'IDENTRSAIDOP': identrsaidop}).then(response => {
            var filterWidget = templateManager.container.getActiveWidget();
            let clear = filterWidget.actions.filter(action => action.name === "clear")[0];
            clear.shortcutKey = "ctrl+backspace";
            clear.tooltip = "Limpar (ctrl + Backspace)";
            let close = filterWidget.actions.filter(action => action.name === "close")[0];
            close.isVisible = false;
        });
    };
    
    this.salvaCfop = function(widget) {
        var gridExcecao = templateManager.container.getWidget('gridExcecaoEstoque');
        if(widget.isValid()){
            widget.saveRow().then(function(Success) {
                if(widget.currentRow.__is_new) {
                    widget.view();
                    widget.reload();
                    widget.currentRow.__is_new = false;
                    gridExcecao.enabled = true;
                    gridExcecao.activate();
                } else {
                    widget.view();
                    gridExcecao.enabled = true;
                }
            }.bind(this), function(callback){
                if(callback.data){
                    ScreenService.showMessage(callback.data.error);
                }
            });
        }
    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Ger97500_Tipo_De_Operacao_x_Cfop_Controller", Ger97500_Tipo_De_Operacao_x_Cfop_Controller);
});


// FILE: ./assets/modules/entrada/assets/js/controllers/dfe45000_parametros_bloquear_tipo_operacao_exige_af_controller.js
function dfe45000_parametros_bloquear_tipo_operacao_exige_af_controller(restEngine, ScreenService, templateManager, ZHPromise, General, Endereco) {
    var self = this;

    self.fornecedorApplyFilter = function(filterCode, widgetToApply) {
        if(filterCode){
            restEngine.post("df_entrada#/parambloqnfafor_fornecedorparamsbloq", {
                "requestType": "FilterData",
                "filter": [
                    {
                        "name": "P_NRINSJURFORN",
                        "operator": "=",
                        "value": filterCode
                    }
                ]
            }).then(function(response) {
                widgetToApply.dataSource.data = response.dataset.parambloqnfafor_fornecedorparamsbloq;
            });
        } else {
            widgetToApply.dataSource.data = [];
            ScreenService.showMessage('Fornecedor deve ser informado!');
        }
    };

    self.fornecedorSave = function(widget) {
        widget.currentRow.changed = true;

        restEngine.post("df_entrada#/parambloqnfafor_fornecedorparamsbloq/save", {
            "requestType": "DataSet",
            "dataset": [
                widget.currentRow
            ]
        }).then(function(response) {
            widget.currentRow.changed = false;
            widget.currentRow.__is_new = false;

            widget.view();
            General.changeWidgetAcessibility(widget.widgets, true);

            widget.reload();
        });
    };
    
    self.unidadeSave = function(widget) {
        widget.currentRow.changed = true;

        restEngine.post("df_entrada#/parambloqnfafil_unidadeparamsbloq/save", {
            "requestType": "DataSet",
            "dataset": [
                widget.currentRow
            ]
        }).then(function(response) {
            widget.currentRow.changed = false;
            widget.currentRow.__is_new = false;

            widget.view();
            General.changeWidgetAcessibility(widget.widgets, true);

            widget.reload();
        });
    };

    self.unidadeApplyFilter = function(filterCode, widgetToApply) {
        if(filterCode){
            restEngine.post("df_entrada#/parambloqnfafil_unidadeparamsbloq", {
                "requestType": "FilterData",
                "filter": [
                    {
                        "name": "P_CDFILIAL",
                        "operator": "=",
                        "value": filterCode
                    }
                ]
            }).then(function(response) {
                widgetToApply.dataSource.data = response.dataset.parambloqnfafil_unidadeparamsbloq;
            });
        } else {
            widgetToApply.dataSource.data = [];
            ScreenService.showMessage('Unidade deve ser informada!');
        }
    };

    self.unidadeVerifyRow = function(widget) {
        if(widget.currentRow.CDTIPOOPER == '01' || widget.currentRow.IDORIGEM != 'F') 
            widget.getField('IDEXIGEAF').readOnly = true;
        else 
            widget.getField('IDEXIGEAF').readOnly = false;
    };
    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('dfe45000_parametros_bloquear_tipo_operacao_exige_af_controller', dfe45000_parametros_bloquear_tipo_operacao_exige_af_controller);
});

// FILE: ./assets/modules/saida/assets/js/controllers/Dfe43200_ParametrosDocumentosFiscaisSaida_Controller.js
var  Dfe43200_ParametrosDocumentosFiscaisSaida_Controller = function(Dfe43200_ParametrosDocumentosFiscaisSaida_Service, ScreenService, templateManager,General){

    this.editar = function(widget){
        if(widget.name=='parametrosFilial'){
                var tabbedParametrosFilial =  templateManager.container.getWidget('tabbedParametrosFilial');
                tabbedParametrosFilial.getField('NMFILIAL').readOnly=true;
        }
        if(widget.name=='parametrosEmpresa'){
                var tabbedParametrosEmpresa =  templateManager.container.getWidget('tabbedParametrosEmpresa');
                tabbedParametrosEmpresa.getField('NMEMPRESA').readOnly=true;
        }
        widget.template = widget.templateToForm;
        General.edit(widget);
    };

    this.cancelar = function(widget){
        for (var i in widget.backupRow) {
			widget.currentRow[i] = widget.backupRow[i];
		}
		widget.template = widget.templateToView;
		widget.view();
		widget.activate();
		
        if(widget.name=='parametrosFilial'){
                var tabbedParametrosFilial =  templateManager.container.getWidget('tabbedParametrosFilial');
                tabbedParametrosFilial.getField('NMFILIAL').readOnly=false;
        }
        if(widget.name=='parametrosEmpresa'){
                var tabbedParametrosEmpresa =  templateManager.container.getWidget('tabbedParametrosEmpresa');
                tabbedParametrosEmpresa.getField('NMEMPRESA').readOnly=false;
        }
    };

    this.controlaStatusWidgets = function(mainWidget, nomeWidgetAtual, status){
        mainWidget.widgets.forEach(function(widg) {
            if(nomeWidgetAtual !== widg.name)
                widg.enabled = status;
        });

        if(mainWidget.name !== nomeWidgetAtual)
            mainWidget.enabled = status;
    };

    this.WidgetOnEnter = function(widget){
        widget.currentRow = widget.dataSource.data[0];
        if(widget.name=='parametrosGerais'){
            this.setarCampoEncargo(widget);
        }
    };
    
    this.enterInlineEditTipoOperacao = function(widget) {
        this.controlaStatusWidgets(widget.parent, "tipoOperacao", false);
    };
    
    this.exitInlineEditTipoOperacao = function(widget) {
        this.controlaStatusWidgets(widget.parent, "tipoOperacao", true);
    };
    
    this.cancelTipoOperacao = function(widget) {
        this.exitInlineEditTipoOperacao(widget);
        widget.reload();
        widget.list();
    };
    
    this.salvarTipoOperacao = function (widget) {
        var itens = widget.dataSource.data;
        var itensModificados = itens.filter(item => item.changed_IDUTILOPER);
        
        itensModificados.forEach (item => {
            item.changed = true;
        });
        
        widget.dataSource.save().then(function () {
            itensModificados.forEach (item => {
                //@todo remover delete item.changed quando desmarcaLinhasEditadas for implementada
                delete item.changed_IDUTILOPER;
                delete item.changed;
            });
            widget.activate();
            widget.list(true);
            ScreenService.successNotification("Parâmetros salvos com sucesso.");
        });
        
        this.exitInlineEditTipoOperacao(widget);
    };

    this.salvarParametrosGerais = function(args){
        var widget = args.owner.widget;
        if(widget.currentRow.NRDECITOTANF>2) {
            ScreenService.showMessage("Número de casas decimais de totalização deve estar entre 0 e 2.");
        }
        else if(widget.currentRow.NRDUPLINFS>100000){
            ScreenService.showMessage("Intervalo entre número de lançamentos para duplicações de notas fiscais deve estar ente 0 e 100000.");
        }
        else{
            if(widget.currentRow.CDENCDESCBONDEV==" "){
                widget.currentRow.CDENCDESCBONDEV=null;
            }
            General.saveCrud(widget).then(function() {
                this.controlaStatusWidgets( templateManager.container.getWidget('container_tabbed'),'parametrosGerais',true);
                var currentRow = widget.currentRow;
                currentRow.__is_new = false;
                currentRow.changed = false;
                widget.template = widget.templateToView;
                widget.view();
                ScreenService.closeSwipe();
                widget.reload();
                widget.list();
            }.bind(this), function(callback){
                ScreenService.showMessage(callback.data.error);

            });
        }
    };

    this.setarCampoEncargo = function(widget){
        var cdEncargo = widget.dataSource.data[0].CDENCDESCBONDEV;
        var currentRow = widget.currentRow;
        if (cdEncargo){
            General.callRestEngine( "df#/crud/encargo", "FilterData", 
            [{name: "CDENCARGO", operator: "=", value: cdEncargo}]).then( (encargo) => {
                currentRow.NMENCARGO = encargo.dataset.encargo[0].NMENCARGO;
             });
        }
    };

    this.filtroParametrosFilial =  function(){
        var tabbedParametrosFilial =  templateManager.container.getWidget('tabbedParametrosFilial');
        var gridParametrosFilial =  templateManager.container.getWidget('parametrosFilial');
        var filial = tabbedParametrosFilial.getField('NMFILIAL').value();
        if(filial){
            gridParametrosFilial.getAction('editar').isVisible = true;
            gridParametrosFilial.reload();
        }else{
            gridParametrosFilial.getAction('editar').isVisible = false;
            for (let registro in gridParametrosFilial.currentRow) {
        		gridParametrosFilial.currentRow[registro] = '';
        	}
        }
        
    };

    this.verificaInsercaoFilial = function(){
        var tabbedParametrosFilial =  templateManager.container.getWidget('tabbedParametrosFilial');
        var gridParametrosFilial =  templateManager.container.getWidget('parametrosFilial');
        var filial =tabbedParametrosFilial.getField('CDFILIAL');
        if(filial.value()){
            var dataParametros = gridParametrosFilial.dataSource.data;
            //se não exister o parametros para filial cadastrado insere uma nova linha
            if(dataParametros.length==0){
                gridParametrosFilial.currentRow={};
                gridParametrosFilial.currentRow.CDFILIAL=filial.value();
                gridParametrosFilial.currentRow.__is_new=true;
                gridParametrosFilial.dataSource.name = "df#/crud/paramfilial";
                gridParametrosFilial.saveRow().then(function() {
                    gridParametrosFilial.dataSource.name = "df_saida#/paramfilial_saida";
                    gridParametrosFilial.reload();
                }.bind(this), function(callback){
                    if(callback.data){
                        ScreenService.showMessage(callback.data.error);
                    }
                    gridParametrosFilial.dataSource.name = "df_saida#/paramfilial_saida";
                });

            }
        }
    };

    this.salvarParametrosFilial = function(args){
        var widget = args.owner.widget;
        if(widget.currentRow.NRTAMNUMNFUT>20){
            ScreenService.showMessage("Tamanho no Nº da Nota Fiscal deve estar entre 1 e 20");
        }
        else{
            widget.currentRow.DSPARFILFIS =  Dfe43200_ParametrosDocumentosFiscaisSaida_Service.concatenaCampoParametrosFilial(widget.currentRow);
            Dfe43200_ParametrosDocumentosFiscaisSaida_Service.salvarParametrosFilial(widget.currentRow).then(function() {
                this.controlaStatusWidgets( templateManager.container.getWidget('container_tabbed'),'tabbedParametrosFilial',true);
                var tabbedParametrosFilial =  templateManager.container.getWidget('tabbedParametrosFilial');
                tabbedParametrosFilial.getField('NMFILIAL').readOnly=false;
                var currentRow = widget.currentRow;
                currentRow.__is_new = false;
                currentRow.changed = false;
                widget.template = widget.templateToView;
                widget.view();
                ScreenService.closeSwipe();
                widget.reload();
                widget.list();
            }.bind(this), function(callback){
                ScreenService.showMessage(callback.data.error);

            });
        }
    };

    this.filtroParametrosEmpresa =  function(){
        var tabbedParametrosEmpresa =  templateManager.container.getWidget('tabbedParametrosEmpresa');
        var gridParametrosEmpresa =  templateManager.container.getWidget('parametrosEmpresa');
        var empresa = tabbedParametrosEmpresa.getField('CDEMPRESA').value();
        if(empresa){
            gridParametrosEmpresa.getAction('editar').isVisible = true;
            gridParametrosEmpresa.reload();
        }else{
            gridParametrosEmpresa.getAction('editar').isVisible = false;
            for (let registro in gridParametrosEmpresa.currentRow) {
        		gridParametrosEmpresa.currentRow[registro] = '';
        	}
        }
    };

    this.verificaInsercaoEmpresa = function(args){

        var tabbedParametrosEmpresa =  templateManager.container.getWidget('tabbedParametrosEmpresa');
        var gridParametrosEmpresa =  templateManager.container.getWidget('parametrosEmpresa');
        var empresa =tabbedParametrosEmpresa.getField('CDEMPRESA');
        if(empresa.value()){
            var dataParametros = gridParametrosEmpresa.dataSource.data;
            //se não exister o parametros para empresa cadastrado insere uma nova linha
            if(dataParametros.length==0){
                gridParametrosEmpresa.currentRow={};
                gridParametrosEmpresa.currentRow.CDEMPRESA=empresa.value();
                gridParametrosEmpresa.currentRow.__is_new=true;
                gridParametrosEmpresa.dataSource.name = "df#/crud/paramempresa";
                gridParametrosEmpresa.saveRow().then(function() {
                    gridParametrosEmpresa.dataSource.name = "df_saida#/paramempresa_saida";
                    gridParametrosEmpresa.reload();
                }.bind(this), function(callback){
                    if(callback.data){
                        ScreenService.showMessage(callback.data.error);
                    }
                    gridParametrosEmpresa.dataSource.name = "df_saida#/paramempresa_saida";
                });

            }
        }
    };

    this.salvarParametrosEmpresa = function(args){
        var widget = args.owner.widget;
        General.saveCrud(widget).then(function() {
            this.controlaStatusWidgets( templateManager.container.getWidget('container_tabbed'),'tabbedParametrosEmpresa',true);
            var tabbedParametrosEmpresa =  templateManager.container.getWidget('tabbedParametrosEmpresa');
            tabbedParametrosEmpresa.getField('NMEMPRESA').readOnly=false;
            var currentRow = widget.currentRow;
            currentRow.__is_new = false;
            currentRow.changed = false;
            widget.template = widget.templateToView;
            widget.view();
            widget.reload();
            widget.list();
        }.bind(this), function(callback){
            if(callback.data !== undefined)
                ScreenService.showMessage(callback.data.error);
            else
                ScreenService.showMessage(callback);

        });
    };

    this.campoParametrosEmpresa = function(){
        var gridParametrosEmpresa =  templateManager.container.getWidget('parametrosEmpresa');
        var idt = gridParametrosEmpresa.getField('IDABATNFDNFOSA').value();

        if(idt=="S"){
            gridParametrosEmpresa.getField('CDAJUCTPAG').readOnly = false;
            gridParametrosEmpresa.getField('CDAJUCTPAG').validations.required = true;
            gridParametrosEmpresa.getField('CDAJUCTPAG').reload();

        }else{
            delete(gridParametrosEmpresa.getField('CDAJUCTPAG').validations.required );
            gridParametrosEmpresa.getField('CDAJUCTPAG').readOnly = true;
            gridParametrosEmpresa.getField('CDAJUCTPAG').value("");
            gridParametrosEmpresa.getField('DSAJUCTPAG').value("");
            gridParametrosEmpresa.getField('CDAJUCTPAG').reload();
        }
    };

    /*
    this.salvarOperadorLogistico = function(args){
        var Widget = args.owner.widget;
        General.saveCrud(Widget).then(function() {
            this.controlaStatusWidgets( templateManager.container.getWidget('container_tabbed'),'gridOperadorLogistico',true);
            var currentRow = Widget.currentRow;
            currentRow.__is_new = false;
            currentRow.changed = false;
            Widget.template = Widget.templateToView;
            Widget.view();
            Widget.reload();
            Widget.list();
        }.bind(this), function(callback){
            ScreenService.showMessage(callback.data.error);

        });
    };
    */
    this.recarregarWidget = function(nomeWidget){
        var widget =  templateManager.container.getWidget(nomeWidget);
        widget.reload();
    };

    this.salvar = function(args){
        var widget = args.owner.widget;
        General.saveCrud(widget).then(function() {
            this.controlaStatusWidgets( templateManager.container.getWidget('container_tabbed'),widget.name,true);
            var currentRow = widget.currentRow;
            currentRow.__is_new = false;
            currentRow.changed = false;
            widget.template = widget.templateToView;
            widget.view();
            widget.reload();
            widget.list();
        }.bind(this), function(callback){
            ScreenService.showMessage(callback.data.error);

        });
    };
    /*
    this.verficaUtilizaAlmoxarifado = function(args){
        var gridParametrosFilial =  templateManager.container.getWidget('parametrosFilial');
        if(gridParametrosFilial.currentRow.IDUTILALLONF=='S'){
            if(gridParametrosFilial.currentRow.IDUTILALMOX=='N'){
                ScreenService.showMessage('Unidade Não Utiliza Almoxarifado');
                gridParametrosFilial.getField('IDUTILALLONF').value('N');
            }
        }

    };

    this.situacaoCampoLancamentoManual = function(){
        var gridParametrosGerais =  templateManager.container.getWidget('parametrosGerais');
        var campoImportaXML= gridParametrosGerais.getField('IDTPIMPNFE');
        var campoLancamento =gridParametrosGerais.getField('IDPELANIMPXML');
        if(campoImportaXML.value()=="F"){
            campoLancamento.value("N");
            campoLancamento.readOnly=true;
        }else {
            campoLancamento.readOnly=false;
        }
    };


*/
};


Configuration(function (ContextRegister){
    ContextRegister.register('Dfe43200_ParametrosDocumentosFiscaisSaida_Controller', Dfe43200_ParametrosDocumentosFiscaisSaida_Controller);
});

// FILE: ./assets/modules/saida/assets/js/controllers/Dfs11000_LancamentoSaida_Controller.js
var dfs11000_LancamentoSaida_Controller = function(ScreenService, MessageService, templateManager, Df11000_Lancamento_Service, Dfs11000_LancamentoSaida_Service, General, ValidacoesEstoque_Service, ZhFilterPreferences, ErrosReplicacaoDocumentos_Utilities, Lib00206_InfComplProduto_Controller, localStorageEngine, ZHPromise){
    
    var CDFILIEMIT;
    var atualizaEstoque;
    
    var getCabecalhoWidget = function(){
        if(templateManager.container)
            return templateManager.container.getWidget("cabecalho");
    };

    var getItensWidget = function(){
        return templateManager.container.getWidget("itens");
    };
    
    var getImpostosItensWidget = function(){
        return templateManager.container.getWidget("impostosItem");
    };
    
    var requestError = function(error){
        var errorMessage = "Ocorreu um erro ao tentar realizar a operação.";
        if (error && error.data) {
            errorMessage = error.data.error;
        }
        ScreenService.showMessage(errorMessage, "ERROR");
    };
    
    //Função que ajusta casas decimais de valor unitario de acordo com o parametro
    //Desativada por enquanto até futura definição
    this.ajustaCasasDecimais = function(){
        var itemsWidget = getItensWidget();
        var fieldItem = itemsWidget.getField('VRUNITITEM');
        
        /*General.callRestEngine('df#/crud/paramgeral', 'FilterData', []).then(function(success) {
            if(success.dataset.paramgeral[0]){
                var paramgeral = success.dataset.paramgeral[0];
                fieldItem.mask.params.precision = paramgeral.NRDECITOTANF;
            }
        });*/
    };
    
    this.actVoltar = function(args) {
        var widget = args.owner.widget;
        General.backOrCancel(args);
        widget.reload();
    };
    
    this.actNovaNota = function(args) {
        Df11000_Lancamento_Service.removeDataSourceLancamentoNota();
        Df11000_Lancamento_Service.inicializaLancamento();
        Df11000_Lancamento_Service.setDataSourceLancamentoNota().then(function() {
            defaultValueIndicadorPresenca(args.owner.widget);
            alteraVisibilidadeCampoNaturezaOper(args.owner.widget, false);
            validaCampoModDocFiscal(args.owner.widget, []);
            //validaCampoEmpresa(args);
            validaCampoSerie(args.row, true);
            args.owner.widget.newRow();
            controlaStatusWidgets(args.owner.widget, args.owner.widget.name, false);
            buscaItensNotaOrigem = true;
            validaNumeroNF(args.owner.widget);
            setCamposDefaultValueCabecalho(args)
            .then(() => {
                    var widgetCabecalho = templateManager.container.getWidget('cabecalho');
                    widgetCabecalho.redraw();
                    //widgetCabecalho.reload();
                });
        });
    };

    var reloadWidgetsFilhas = function(widgetPai) {
        widgetPai.widgets.forEach(function(widgetFilha) {
            widgetFilha.reload();
        });
    };

    this.encargoOnChange = function(args){
        var field = args.owner.field;
        
        if (!field.value()){
            field.setValue(0);
        }
        controlaStatusWidgets(args.owner.field.widget.getParent(), args.owner.field.widget.name, false);
    };

    this.unidadeDestinatarioOnChange = function(widget) {
        var currentRow = widget.currentRow;
        
        if (currentRow.CDFILIRECE == currentRow.CDFILIEMIT) {
            ScreenService.showMessage("Unidade de origem deve ser diferente da unidade de destino.");
            currentRow.DSFILIRECE = null;
            currentRow.CDFILIRECE = null;
            currentRow.CDEMPRESARECE = null;
        }
        else if (currentRow.CDEMPRESARECE != currentRow.CDEMPRESA) {
            ScreenService.showMessage("Unidade de destino deve possuir a mesma empresa da nota fiscal.");
            currentRow.DSFILIRECE = null;
            currentRow.CDFILIRECE = null;
            currentRow.CDEMPRESARECE = null;
        }
        widget.reload();
    };

    this.destinatarioOnChange = function(field) {
        var currentRow = field.widget.currentRow;
        
        currentRow.DSINSJURNF = currentRow[field.name];
    };

    this.qtdiasvenrecOnChange = function(field) {
        var currentRow = field.widget.currentRow;
        
        if (currentRow.QTDIASVENREC) {
            var parentCurrentRow = field.widget.getParent().currentRow;
            currentRow.DTATUAVENREC = moment(parentCurrentRow.DTENTRSAID, "DD/MM/YYYY").add(currentRow.QTDIASVENREC, "days").format("DD/MM/YYYY");
        }
    };

    this.dtatuavenrecOnChange = function(field) {
        var currentRow = field.widget.currentRow;
        
        if (currentRow.DTATUAVENREC) {
            var parentCurrentRow = field.widget.getParent().currentRow;
            var dataNota = moment(parentCurrentRow.DTENTRSAID, "DD/MM/YYYY");
            var dataInserida = moment(currentRow.DTATUAVENREC, "DD/MM/YYYY");
            currentRow.QTDIASVENREC = dataInserida.diff(dataNota, "days").toString();
        }
    };

    var controlaStatusWidgets = function(widgetPai, nomeWidgetAtual, status){
        widgetPai.widgets.forEach(function(widg){
            if(nomeWidgetAtual !== widg.name)
                widg.enabled = status;
        });

        if(widgetPai.name !== nomeWidgetAtual)
            widgetPai.enabled = status;
    };

    this.criaDetalhesLancamentoWidgetAfterMoveRow = function(consultaLancamentoArgs) {
        //buscaItensNotaOrigem = true;
        return Df11000_Lancamento_Service.criaDetalhesLancamento(consultaLancamentoArgs.row).then(function(lancamentoSaida) {
            templateManager.container.getWidget("itens").dataSource.data = lancamentoSaida.ITENS;
            templateManager.container.getWidget("impostos").dataSource.data = lancamentoSaida.IMPOSTO;
            templateManager.container.getWidget("encargos").dataSource.data = lancamentoSaida.ENCARGO;
            templateManager.container.getWidget("vencimentos").dataSource.data = lancamentoSaida.VENCIMENTO;
            templateManager.container.getWidget("dados_adicionais").dataSource.data = lancamentoSaida.DADOSADICIONAIS;
            return true;
        });
    };

    this.removeDataSourcesConsultaLancSaidaActionEvent = function() {
        Df11000_Lancamento_Service.removeDataSourceLancamentoNota();
    };
    
    /* var alteraVisibilidadeRemetenteDestinatario = function(){
    //     var cabecalhoWidget = getCabecalhoWidget();
    //     var allFields = cabecalhoWidget.fields;
    //     var remetenteDestinatario = allFields.filter(function(field){
    //         return field.fieldGroup === "destinatario" || field.fieldGroup === "remetente";
    //     });
    //     remetenteDestinatario.forEach(function(field){
    //         field.showOnView = cabecalhoWidget.currentRow.IDSTATUS !== "D";
    //     });
    // };*/

    var validaEdicaoNota = function() {
        var cabecalhoWidget = getCabecalhoWidget();
        var statusNota = cabecalhoWidget.currentRow.IDSTATUS || 'D';
        var statusNotaTransmissao = cabecalhoWidget.currentRow.IDSTATUSNFE;
        var notaCanceladaOuAprvada = (statusNota === 'A' || statusNota === 'C');
        var notaTransmissao = (statusNotaTransmissao === 'T' || statusNotaTransmissao === 'A' || statusNotaTransmissao === 'C' || statusNotaTransmissao === 'D');
        var permiteEditarNota = (!notaCanceladaOuAprvada && !notaTransmissao && (statusNota === 'D' || statusNota === 'P'));
        var allActions = cabecalhoWidget.getActions();

        var invisibleActions   = allActions.filter(function(action) {
            var widgetName     = action.widget.name;
            var actionName     = action.name.toLowerCase();
            var isActionVoltar = (actionName.indexOf('voltar') === -1);
            var novaNota       = (actionName.indexOf('novo') !== -1 && widgetName === 'cabecalho');
            var zeedhiWidgets  = (widgetName !== 'filterPopup' && widgetName !== 'stripe');
            return (isActionVoltar && zeedhiWidgets && !novaNota);
        });
        invisibleActions.forEach(function(action) {
            if(action.name !== 'REPLICARDOCUMENTO' && action.name !== 'INVISIBLEACTION' && action.name != "COPIARNOTA") action.isVisible = permiteEditarNota;
            else action.isVisible = (statusNota !== 'D');
            var encargoInlineEdit = cabecalhoWidget.container.getWidget('encargos').getField('VRITEMENCA');
            if(!!permiteEditarNota)
                encargoInlineEdit.inlineEditable = "number";
            else
                delete encargoInlineEdit.inlineEditable;

            if (action.name.toLowerCase().indexOf('fecharnota') !== -1 && statusNota === 'P')
                action.isVisible = false;
        });
    };

    var validaEdicaoItensServico = function(cabecalhoWidget){
        if(cabecalhoWidget.currentRow.IDORIGDESTNF === "C"){
            var itensWidget = cabecalhoWidget.container.getWidget("itens");
            var item = itensWidget.dataSource.data[0];
            if(cabecalhoWidget.appliedFilter.FILTER_ROW.IDBLOQITNFAPUR === "S" &&
                itensWidget.dataSource.data.length !== 0 &&
                (item.CDSERVFILI || item.CDSERVFATU)){

                itensWidget.getAction("novoItem").isVisible = false;
                itensWidget.getAction("novoServico").isVisible = false;
            }
            else{
                itensWidget.getAction("novoItem").isVisible = true;
                itensWidget.getAction("novoServico").isVisible = true;
            }
        }
    };

    var alteraVisibilidadeBotaoIncluirServico = function(widget){
        if(widget.currentRow.IDORIGDESTNF === "C"){
            Dfs11000_LancamentoSaida_Service.retornaClienteFilial(
                widget.currentRow.CDFILIEMIT,
                widget.currentRow.CDCLIENTE).then(function(clienteFilial){
                    if(clienteFilial.length !== 0)
                        widget.container.getWidget("itens").getAction("novoServico").isVisible = true;
                    else
                        widget.container.getWidget("itens").getAction("novoServico").isVisible = false;
                });
        }
        else{
            widget.container.getWidget("itens").getAction("novoServico").isVisible = false;
        }
    };

    var alteraVisibilidadeActionsProduto = function() {

        var situacaoNFe  = getCabecalhoWidget().currentRow.IDSTATUSNFE; 
        var situacaoNF   = getCabecalhoWidget().currentRow.IDSTATUS; 
        var adicionar    = getItensWidget().actions.filter(action => action.name === 'novoItem')[0];
        var excluirItem  = getItensWidget().actions.filter(action => action.name === 'excluirItem')[0];
    
        if(situacaoNFe !== "undefined") {
            if((situacaoNFe == "R" || situacaoNFe == "P" || situacaoNF == "D" || situacaoNF == "P") && (getCabecalhoWidget().currentRow.IDOPERDEVONF != "S")) {
                adicionar.isVisible = true;
                excluirItem.isVisible = true;
            } else {
                adicionar.isVisible = false;
                excluirItem.isVisible = false;
            }
        }     
        
    };
    
    var validaNotaServico = function(cabecalhoWidget){
        validaEdicaoItensServico(cabecalhoWidget);
        alteraVisibilidadeBotaoIncluirServico(cabecalhoWidget);
    };

    this.validacoesNotaAfterMoveRow = function(args) {
        let widget = args.owner;
        
        Df11000_Lancamento_Service.removeDataSourceLancamentoNota();
        if(args.row.__is_new === false) {
            this.criaDetalhesLancamentoWidgetAfterMoveRow(args).then(function(){
                validaNotaServico(args.owner);
            });
        }
        alteraVisibilidadeCampoNaturezaOper(widget);
        validaNumeroNF(widget);
        validaEdicaoNota();
        // alteraVisibilidadeRemetenteDestinatario();
        alteraVisibilidadeCampoInscricao();
        //validaCamposDocumentoOrigem(getCabecalhoWidget(), args.row);
        preparaCamposDocumentoOrigem(args);
        
        widget.getField('NMTIPOOPER').reload().then(() => {
            let widget = templateManager.container.getActiveWidget();
            if(!widget.editing) { //True = Visualizar nota.
                this.controlVisibilityLocalPrestServ(widget);
            }
        });
        this.validacoesFieldGroupIntermediador(args.owner);

    };

    this.excluirNotaActionEvent = function(consultaLancamentoArgs) {
        ScreenService.confirmMessage('Deseja excluir a nota fiscal?', 'question', function (success) {
            var filter = [
                {
                    "name": "IDDOCREPLIC",
                    "operator": "=",
                    "value": consultaLancamentoArgs.row.CDFILIAL.concat(consultaLancamentoArgs.row.NRLANCTONF)
                }    
            ];
            General.callRestEngine('df_saida#/crud/notaReplicada_tipooperac', 'FilterData', filter).then(function(success) {
                if(success.dataset.notaReplicada_tipooperac[0]) this.openReplicacaoExclusao(consultaLancamentoArgs);
                else this.excluirNota(consultaLancamentoArgs);
            }.bind(this));
        }.bind(this));
    };
    
    this.excluirNota = function(consultaLancamentoArgs) {
        Dfs11000_LancamentoSaida_Service.excluirNota(consultaLancamentoArgs.row).then(function() {
            General.backOrCancel(consultaLancamentoArgs);
            var rowExcluida = consultaLancamentoArgs.owner.widget.dataSource.data.filter(function(notafiscal){
                return consultaLancamentoArgs.row.NRLANCTONF === notafiscal.NRLANCTONF;
            });
            var index = consultaLancamentoArgs.owner.widget.dataSource.data.indexOf(rowExcluida[0]);
            consultaLancamentoArgs.owner.widget.dataSource.data.splice(index, 1);
            consultaLancamentoArgs.owner.widget.reload();
            ScreenService.showMessage('Nota Fiscal excluída com sucesso');
        }, function (error) {
            requestError(error);
        });
    };
    
    var alteraStatusNotafiscal = function(widget){
        if(widget.getParent().currentRow.IDSTATUS !== 'D'){
            widget.getParent().currentRow.IDSTATUS = 'D';
            Df11000_Lancamento_Service.salvaCabecalho(widget.getParent().currentRow).then(function(){}, requestError);
            // alteraVisibilidadeRemetenteDestinatario();
            validaEdicaoNota();
        }
    };
    
   var confereFilialLogada = function(lancamentoCabecalhoArgs){
        var deferred = ZHPromise.defer();
        if(!lancamentoCabecalhoArgs.row.hasOwnProperty('CDFILIAL')){
          Dfs11000_LancamentoSaida_Service.getFilial().then(function(result){
              lancamentoCabecalhoArgs.row.CDFILIAL = result.dataset.filial[0].CDFILIAL;
              deferred.resolve();
          });
        }
        else{
            deferred.resolve();
        }
        return deferred;
    };

    this.salvaLancamentoCabecalho = function(lancamentoCabecalhoArgs) {
        
     confereFilialLogada(lancamentoCabecalhoArgs).then(function(success){

        if (lancamentoCabecalhoArgs.owner.widget.isValid()) {
            if (!lancamentoCabecalhoArgs.row.NRINSJURCLIE && !lancamentoCabecalhoArgs.row.CDFILIRECE && !lancamentoCabecalhoArgs.row.DSINSJURNF)
                return ScreenService.showMessage("Destinatário da Nota Fiscal não informado.");

            if (lancamentoCabecalhoArgs.row.CDFILIRECE) {
                if (lancamentoCabecalhoArgs.row.CDFILIRECE == lancamentoCabecalhoArgs.row.CDFILIEMIT) {
                    lancamentoCabecalhoArgs.row.DSFILIRECE = null;
                    lancamentoCabecalhoArgs.row.CDFILIRECE = null;
                    lancamentoCabecalhoArgs.row.CDEMPRESARECE = null;
                    return ScreenService.showMessage("Unidade de origem deve ser diferente da unidade de destino.");
                }
                else if (lancamentoCabecalhoArgs.row.CDEMPRESARECE != lancamentoCabecalhoArgs.row.CDEMPRESA) {
                    lancamentoCabecalhoArgs.row.DSFILIRECE = null;
                    lancamentoCabecalhoArgs.row.CDFILIRECE = null;
                    lancamentoCabecalhoArgs.row.CDEMPRESARECE = null;
                    return ScreenService.showMessage("Unidade de destino deve possuir a mesma empresa da nota fiscal.");
                }
            }

            ValidacoesEstoque_Service.validaDataLancamento(lancamentoCabecalhoArgs.row.CDFILIEMIT, lancamentoCabecalhoArgs.row.DTENTRSAID, "SAIDA", "DFS").then(function(){
                if(lancamentoCabecalhoArgs.row.__is_new){
                    //lancamentoCabecalhoArgs.row.__new_code_pk = lancamentoCabecalhoArgs.owner.widget.paramsNewCode.__new_code_pk+lancamentoCabecalhoArgs.row.CDFILIAL;
                    lancamentoCabecalhoArgs.row.__new_code_pk = lancamentoCabecalhoArgs.owner.widget.paramsNewCode.__new_code_pk;
                    lancamentoCabecalhoArgs.row.__using_unit = lancamentoCabecalhoArgs.owner.widget.paramsNewCode.__using_unit;
                    lancamentoCabecalhoArgs.row.__table_pk = lancamentoCabecalhoArgs.owner.widget.paramsNewCode.__table_pk;
                    lancamentoCabecalhoArgs.row.__new_code_length = lancamentoCabecalhoArgs.owner.widget.paramsNewCode.__new_code_length || 5;
                    lancamentoCabecalhoArgs.row.__columns_to_return = lancamentoCabecalhoArgs.owner.widget.paramsNewCode.__columns_to_return;
                    lancamentoCabecalhoArgs.row.VRPRODUTOS = 0;
                    lancamentoCabecalhoArgs.row.VRNOTAFISC = 0;
                    lancamentoCabecalhoArgs.row.IDENTRSAIDOP = "S";
                    lancamentoCabecalhoArgs.row.IDBLOQUEADO = "N";
                    lancamentoCabecalhoArgs.row.IDSISTEMA = "DFS";
                    lancamentoCabecalhoArgs.row.IDDOCFINANC = "S";
                }

                lancamentoCabecalhoArgs.row.IDSTATUS     = "D";
                lancamentoCabecalhoArgs.row.CDINSCESTA   = lancamentoCabecalhoArgs.row.IEREM;
                lancamentoCabecalhoArgs.row.NRSEQCAPADOC = "";
                lancamentoCabecalhoArgs.row.DTCOMPFATUR  = moment().format("DD/MM/YYYY LTS");
                
                Df11000_Lancamento_Service.salvaCabecalho(lancamentoCabecalhoArgs.row).then(function(callback){
                    var rowDADOSERVPRESnew = false;

                    if(lancamentoCabecalhoArgs.row.__is_new){
                        if(lancamentoCabecalhoArgs.row.IDUTILNFSE === 'S') rowDADOSERVPRESnew = true;
                        lancamentoCabecalhoArgs.row.CDFILIAL = callback.dataset.columnsToReturn[0].CDFILIAL;
                        lancamentoCabecalhoArgs.row.NRLANCTONF = callback.dataset.columnsToReturn[0].NRLANCTONF;
                        lancamentoCabecalhoArgs.row.NRORG = callback.dataset.columnsToReturn[0].NRORG;

                        //OPERAÇÕES DE DEVOLUÇÃO E COMPLEMENTO 
                        if((lancamentoCabecalhoArgs.row.IDOPERDEVONF === 'S' || lancamentoCabecalhoArgs.row.CDTIPOOPER == '03' ) && lancamentoCabecalhoArgs.row.__is_new === true){
                            lancamentoCabecalhoArgs.row.CDFILIALIT = lancamentoCabecalhoArgs.row.CDFILINOTAORIG;
                            lancamentoCabecalhoArgs.row.NRLANCTONFIT = lancamentoCabecalhoArgs.row.NRLANCNOTAORIG;
                            lancamentoCabecalhoArgs.row.NRSEQUITNOTA = "001";

                            var itemnotafisc = {
                                "CDFILIAL": lancamentoCabecalhoArgs.row.CDFILIAL,
                                "NRLANCTONF": lancamentoCabecalhoArgs.row.NRLANCTONF,
                                "CDFILIALIT": lancamentoCabecalhoArgs.row.CDFILINOTAORIG,
                                "NRLANCTONFIT": lancamentoCabecalhoArgs.row.NRLANCNOTAORIG,
                                "NRSEQUITNOTA": "001",
                                "__is_new": true
                            };
                            Df11000_Lancamento_Service.salvaItemnotafisc(itemnotafisc).then(function(){}, requestError);
                        }
                        //@todo remover __is_new quando widget.edit() funcionar
                        lancamentoCabecalhoArgs.row.__is_new = false;
                        reloadWidgetsFilhas(lancamentoCabecalhoArgs.owner.widget);
                    }
                    Df11000_Lancamento_Service.setDataSourceCabecalho(lancamentoCabecalhoArgs.row);
                    Df11000_Lancamento_Service.setDataSourceLancamentoNota();
                    lancamentoCabecalhoArgs.owner.widget.view();
                    validaEdicaoNota();
                    habilitaCamposItemNotaOrigem(getItensWidget(), lancamentoCabecalhoArgs.row);
                    alteraVisibilidadeBotaoIncluirServico(lancamentoCabecalhoArgs.owner.widget);
                    // alteraVisibilidadeRemetenteDestinatario();
                    controlaStatusWidgets(lancamentoCabecalhoArgs.owner.widget, lancamentoCabecalhoArgs.owner.widget.name, true);
                    
                    if(lancamentoCabecalhoArgs.row.IDUTILNFSE === 'S') {
                        let rowDADOSERVPRES = setRowDADOSERVPRES(lancamentoCabecalhoArgs.row);
                        
                        if(rowDADOSERVPRESnew) {
                            rowDADOSERVPRES.__is_new         = true;
                            General.callRestEngine('df#/crud/dadoservpres/save', "Row", rowDADOSERVPRES);
                        }
                        else {
                            let filter = [{"name":"CDFILIAL"  , "operator":"=", "value": rowDADOSERVPRES.CDFILIAL},
                                          {"name":"NRLANCTONF", "operator":"=", "value": rowDADOSERVPRES.NRLANCTONF}];
                                          
                            General.callRestEngine('df#/crud/dadoservpres', 'FilterData', filter)
                                .then((sucess) => {
                                    rowDADOSERVPRES.__is_new = sucess.dataset.dadoservpres.length ? false : true;
                                    General.callRestEngine('df#/crud/dadoservpres/save', "Row", rowDADOSERVPRES);
                                });
                        }
                    }
                }, requestError);
            }, requestError);
        }
     });
    };
    
    var setRowDADOSERVPRES = function(lancamentoCabecalhoArgsRow){
        let rowDADOSERVPRES = {};
        rowDADOSERVPRES.CDFILIAL             = lancamentoCabecalhoArgsRow.CDFILIAL;
        rowDADOSERVPRES.NRLANCTONF           = lancamentoCabecalhoArgsRow.NRLANCTONF;
        rowDADOSERVPRES.IDNATUOPER           = lancamentoCabecalhoArgsRow.IDNATUOPER;
        rowDADOSERVPRES.CDPAISPRESSERV       = lancamentoCabecalhoArgsRow.CDPAISNOTA;
        rowDADOSERVPRES.SGESTADOPRESSERV     = lancamentoCabecalhoArgsRow.SGESTADONOTA;
        rowDADOSERVPRES.CDMUNICIPRESSERV     = lancamentoCabecalhoArgsRow.CDMUNICIPIONOTA;
        rowDADOSERVPRES.NRORG                = lancamentoCabecalhoArgsRow.NRORG;
        rowDADOSERVPRES.IDATIVO              = 'S';
        return rowDADOSERVPRES;
    };

    this.actEditarCabecalhoOnClick = function(detalhesCabecalhoArgs) {
        General.edit(detalhesCabecalhoArgs.owner.widget);
        controlaStatusWidgets(detalhesCabecalhoArgs.owner.widget, detalhesCabecalhoArgs.owner.widget.name, false);
        validaCampoModDocFiscal(detalhesCabecalhoArgs.owner.widget, detalhesCabecalhoArgs.row);
        alteraVisibilidadeCampoNaturezaOper(detalhesCabecalhoArgs.owner.widget);
        changeDataSourceNotaOrigem(detalhesCabecalhoArgs.owner.widget);
        this.controlVisibilityLocalPrestServ(detalhesCabecalhoArgs.owner.widget);
        
    };

    this.getCurrentField = function(field) {
        var cabecalhoWidget = getCabecalhoWidget();
        if (cabecalhoWidget)
             return !cabecalhoWidget.currentRow[field] ? null : cabecalhoWidget.currentRow[field];
        return null;
    };

    this.getSequencialItemSelecionado = function(args){
        return '0001';
    };

    var validaCampoModDocFiscal = function(lancamentoCabecalhoWidget, lancamentoCabecalhoRow) {
         var descModFisc = lancamentoCabecalhoWidget.getField("DSMODDOCFISC");
         descModFisc.readOnly = !lancamentoCabecalhoRow.CDTIPOOPER;

         var tipoOperac = lancamentoCabecalhoRow.CDTIPOOPER;
         if (tipoOperac === "09" || tipoOperac === "10")
             delete descModFisc.validations;
         else
             descModFisc.validations = {"required": true};
    };

    var validaCampoEmpresa = function(lancamentoCabecalhoArgs) {
        var empresa = getCabecalhoWidget().getField("NMEMPRESA");
        empresa.readOnly = !lancamentoCabecalhoArgs.row.CDFILIEMIT;
    };

    var validaCampoSerie = function(lancamentoCabecalhoRow, limpaCampo) {
        var serie = getCabecalhoWidget().getField("DSSERIE");
        serie.readOnly = !lancamentoCabecalhoRow.NMEMPRESA;

        if (!!limpaCampo)
            serie.value(null);

        var tipoOperac = lancamentoCabecalhoRow.CDTIPOOPER;
        if (lancamentoCabecalhoRow.IDSERIEAUTO === 'S' && (tipoOperac === "09" || tipoOperac === "10"))
            delete serie.validations;
        else
            serie.validations = {"required": true};
    };

    var limpaCamposOutDataSelectsTpOperac = function(widget)    {
         widget =  !widget ? getCabecalhoWidget() : widget; 
        var cabecalhoWidget = getCabecalhoWidget();
        var cabecalhoRow = widget.currentRow;
        var cliente = widget.getField("NMRAZSOCCLIE");
        var fornecedor = widget.getField("NMRAZSOCFORN");
        var filial = widget.getField("DSFILIRECE");

        filial.value(null); 
         
        cliente.value(null);
        cabecalhoRow.CDCLIENTE = null;

        fornecedor.value(null);
        cabecalhoRow.CDFORNECED = null;
        cabecalhoRow.IDSITUCADA = null;
        cabecalhoRow.IDBLQNFRELAFOAF = null;

        cabecalhoRow.NRINSJURNF = null;
        cabecalhoRow.NMINSJURNF = null;
        cabecalhoRow.IDTPIJURNF = null;
    };

    var setCampoObrigatorio = function(field, readOnly = false) {
        field.isVisible = true;
        field.validations = {"required": true};
        field.readOnly = readOnly;
    };

    var setCampoNaoObrigatorio = function(field) {
        field.isVisible = false;
        delete field.validations;
        field.readOnly = true;
    };

    var validacaoCamposOutTpOperac = function(princialField) {
        setCampoObrigatorio(princialField);
        princialField.showOnView = true;
        princialField.showOnForm = true;
        princialField.isVisible = true;

        for (var i = 1; i < arguments.length; i++) {
            setCampoNaoObrigatorio(arguments[i]);
            arguments[i].showOnView = false;
            arguments[i].showOnForm = false;
            arguments[i].isVisible = false;
        }
    };

    var preparaCamposDocumentoOrigem = function(args){
        habilitaCamposCabecalhoNotaOrigem(args.owner, args.row);
        habilitaCamposItemNotaOrigem(getItensWidget(), args.row);
    };

    var validaCamposDocumentoOrigem = function(lancamentoCabecalhoWidget, lancamentoCabecalhoRow) {
        var serieOrigem = lancamentoCabecalhoWidget.getField("CDSERIEORIGEM");
        var notaOrigem = lancamentoCabecalhoWidget.getField("DSNOTAORIGEM");
        var tipoDestino = lancamentoCabecalhoRow.IDORIGDESTOP || lancamentoCabecalhoRow.IDORIGDESTNF;
        var destinatario;

        switch (tipoDestino) {
            case "C":
                destinatario = lancamentoCabecalhoWidget.getField("NMRAZSOCCLIE");
                break;
            case "F":
                destinatario = lancamentoCabecalhoWidget.getField("NMRAZSOCFORN");
                break;
            case "L":
                destinatario = lancamentoCabecalhoWidget.getField("DSFILIRECE");
                break;
        }

        //serieOrigem.value(null);
        //notaOrigem.value(null);

        //lancamentoCabecalhoRow.CDFILINOTAORIG = null;
        //lancamentoCabecalhoRow.NRNOTAFISCORIGEM = null;
        //lancamentoCabecalhoRow.NRLANCNOTAORIG = null;

        habilitaCamposItemNotaOrigem(getItensWidget(), lancamentoCabecalhoRow);

        if ((lancamentoCabecalhoRow.IDNFORIGOBRI == "S" && lancamentoCabecalhoRow.IDOPERDEVONF == "S" ) || lancamentoCabecalhoRow.CDTIPOOPER == "03") {
            setCampoObrigatorio(serieOrigem, !destinatario.value());
            setCampoObrigatorio(notaOrigem, !destinatario.value());
            return true;
        }
        
        setCampoNaoObrigatorio(serieOrigem);
        setCampoNaoObrigatorio(notaOrigem);
        return false;
    };

    this.idorigdestOnChange = function(filterArgs) {
        alteraVisibilidadeCampoInscricaoFiltro(filterArgs);
        limpaCamposOutDataSelectsTpOperacFiltro(filterArgs);
    };

    var alteraVisibilidadeCampoInscricaoFiltro = function(filterArgs) {
        var filtroWidget = filterArgs.owner.field.widget;
        var cliente = filtroWidget.getField("NMRAZSOCCLIE");
        var fornecedor = filtroWidget.getField("NMRAZSOCFORN");
        var filial = filtroWidget.getField("DSFILIRECE");
        var tipoDestino = filterArgs.row.IDORIGDESTNF;

        switch (tipoDestino) {
            case "C":
                validacaoCamposOutTpOperacFilter(cliente, fornecedor, filial);
                break;
            case "F":
                validacaoCamposOutTpOperacFilter(fornecedor, cliente, filial);
                break;
            case "L":
                validacaoCamposOutTpOperacFilter(filial, fornecedor, cliente);
                break;
            default:
                filial.readOnly = true;
                cliente.readOnly = true;
                fornecedor.readOnly = true;
                filtroWidget.currentRow.DSFILIRECE = null;
                filtroWidget.currentRow.NMRAZSOCCLIE = null;
                filtroWidget.currentRow.NMRAZSOCFORN = null;
        }
    };

    var validacaoCamposOutTpOperacFilter = function(princialField) {
        princialField.showOnView = true;
        princialField.showOnForm = true;
        princialField.isVisible = true;
        princialField.readOnly = false;

        for (var i = 1; i < arguments.length; i++) {
            arguments[i].showOnView = false;
            arguments[i].showOnForm = false;
            arguments[i].isVisible = false;
            arguments[i].readOnly = true;
        }
    };

    this.tipoOperacaoOnChange = function(cabecalhoArgs) {
        var hasFieldValueFilial = !!cabecalhoArgs.owner.field.value();
        var cabecalhoRow = cabecalhoArgs.row;
        var cabecalhoWidget = cabecalhoArgs.owner.field.widget;

        cabecalhoRow.IDORIGDESTNF = cabecalhoRow.IDORIGDESTOP;

        alteraVisibilidadeCampoInscricao();
        alteraVisibilidadeCampoNaturezaOper(cabecalhoWidget, true);

        validaCampoModDocFiscal(cabecalhoWidget, cabecalhoRow);
        validaCamposDocumentoOrigem(cabecalhoWidget, cabecalhoRow);
        validaCampoSerie(cabecalhoRow);
        validaNumeroNF(cabecalhoWidget);
        limpaCamposOutDataSelectsTpOperac();
        this.controlVisibilityLocalPrestServ(cabecalhoWidget);
        changeDataSourceNotaOrigem(cabecalhoWidget);
        
        if(cabecalhoWidget.currentRow.__is_new)
            defaultValueIndicadorPresenca(cabecalhoWidget);
    };

    var limpaCamposOutDataSelectsTpOperacFiltro = function(filterArgs)    {
        var filtroWidget = filterArgs.owner.field.widget;
        var cabecalhoRow = filtroWidget.currentRow;
        var cliente = filtroWidget.getField("NMRAZSOCCLIE");
        var fornecedor = filtroWidget.getField("NMRAZSOCFORN");
        var filial = filtroWidget.getField("DSFILIRECE");

        filial.value(null);

        cliente.value(null);
        cabecalhoRow.CDCLIENTE = null;

        fornecedor.value(null);
        cabecalhoRow.CDFORNECED = null;
        cabecalhoRow.IDSITUCADA = null;
        cabecalhoRow.IDBLQNFRELAFOAF = null;

        cabecalhoRow.NRINSJURNF = null;
        cabecalhoRow.NMINSJURNF = null;
        cabecalhoRow.IDTPIJURNF = null;
    };

    this.cdfiliemitOnChange = function(lancamentoCabecalhoArgs) {
        lancamentoCabecalhoArgs.owner.field.widget.getField('NMRAZSOCCLIE').reload();
        CDFILIEMIT = lancamentoCabecalhoArgs.row.CDFILIEMIT;
        //validaCampoEmpresa(lancamentoCabecalhoArgs);
        validaCampoSerie(lancamentoCabecalhoArgs.row);
        applyZeroFillNumeroNota(lancamentoCabecalhoArgs.owner.field.widget);
    };

    this.empresaOnChange = function(lancamentoCabecalhoArgs) {
         validaCampoSerie(lancamentoCabecalhoArgs.row, true);
         validaCamposDocumentoOrigem(getCabecalhoWidget(), lancamentoCabecalhoArgs.row);
    };

    this.fornecedorOnChange = function(lancamentoCabecalhoArgs) {
         validaCamposDocumentoOrigem(getCabecalhoWidget(), lancamentoCabecalhoArgs.row);
    };
    
    this.validacoesWidgetOnEnterOnce = function(consultaLancamentoArgs) {
        var cabecalhoWidget = consultaLancamentoArgs.owner;
        setDefaultFilialFilter(cabecalhoWidget).then(defaultFilter => {
            ZhFilterPreferences.setOriginalFilter(cabecalhoWidget.id, defaultFilter);
            ZhFilterPreferences.openFilterWidget(cabecalhoWidget, defaultFilter).then (response => {
                var filterWidget = templateManager.container.getActiveWidget();
                filterWidget.actions.filter(action => action.name === "clear")[0].shortcutKey = "ctrl+backspace";
            });
            CDFILIEMIT = defaultFilter.CDFILIEMIT;
            cabecalhoWidget.getField('NMTIPOOPER').reload();
        });
    };
    
 
    var setDefaultFilialFilter = function(cabecalhoWidget) {
        var promise = ZHPromise.defer();
        let defaultFilter = Util.clone(cabecalhoWidget.currentRow);
        defaultFilter.DTENTRSAID = undefined;

        General.callRestEngine('df#/filioper_lancamento_notafiscal', 'FilterData', [{"name": "CDOPERADOR", "operator": "=", "value": "OPERADOR_LOGADO"},
                                                                                    {"name": "CDFILIAL"  , "operator": "=", "value": "FILIAL_LOGADA"}]
            ).then(function(filialLogada) {
                let filialDefaultValue, cdFilialOut = null;
                let filialFieldOutData = cabecalhoWidget.getField('NMFILIAL').outData;
                
                //Utiliza o primeiro dataset retornado. Necessário pois o dataset pode ser filioper_lancamento_notafiscal ou perfacessfili_lancamento_notafiscal
                for (let datasetName in filialLogada.dataset) {
                    if (filialLogada.dataset.hasOwnProperty(datasetName)) {
                        filialDefaultValue = filialLogada.dataset[datasetName][0];
                	    break;
                    }
                }
                if(filialDefaultValue) cdFilialOut = filialDefaultValue.CDFILIAL;
                
                Df11000_Lancamento_Service.buscaDataProcessamento(cdFilialOut).then(([processamento]) => {
                    if(processamento) {
                        const dtProcessFormat = moment(processamento.DTPROCESSA, 'DD/MM/YYYY').format('DD/MM/YYYY');
                        defaultFilter.DTENTRSAID = {start: dtProcessFormat, period: undefined, end: dtProcessFormat};
                    }
                    promise.resolve(defaultFilter);
                });
                setCurrentRowWithOutData(defaultFilter, filialDefaultValue, filialFieldOutData);
        });
        return promise;
    };
    
    /*var setCurrentRowWithOutDataFilial = function(filialOut, filialField, currentRow, dtEntrField) {
        var promise = ZHPromise.defer(); 
        General.callRestEngine('df#/crud/filioper_lancamento_notafiscal', 'FilterData', [{"name": "CDFILIAL", "operator": "=", "value": filialOut}]).then(function(success){
            var defaultValue = success.dataset.filial[0]; 
            var filialOutData = filialField.outData;
            Df11000_Lancamento_Service.buscaDataProcessamento(filialOut).then(([processamento]) => {
                const dtProcessFormat = moment(processamento.DTPROCESSA, 'DD/MM/YYYY').format('DD/MM/YYYY');
                currentRow.DTENTRSAID = {start: dtProcessFormat, period: undefined, end: dtProcessFormat};
                promise.resolve();
            });
            setCurrentRowWithOutData(currentRow, defaultValue, filialOutData);
        });
            
        return promise;
    };*/
    
    this.getDataProcessamento = function(filialSelecionada){
        var promise = ZHPromise.defer();
        Df11000_Lancamento_Service.buscaDataProcessamento(filialSelecionada).then(function(processamento){
            var dtProcessFormat = moment(processamento[0].DTPROCESSA, 'DD/MM/YYYY').format('DD/MM/YYYY');
            promise.resolve(dtProcessFormat);
        });
        return promise;
    };
    
    var setCurrentRowWithOutData = function(currentRow, valuesToSetOnCurrentRow, outDataValue) {
        for (var key in outDataValue) {
            currentRow[outDataValue[key]] = valuesToSetOnCurrentRow[key];
        }
    };
    
    
    
    const reloadSerie = widget => {
        const field = widget.getField('DSSERIE');
        field.reload();
    };
    
    const reloadCampoTipoOperacao = widget => {
        var deferred = ZHPromise.defer();
        const field = widget.getField('NMTIPOOPER');
        field.reload().then(() => {
                var data = field.dataSource.data;
                if (!data || !data.length)
                    return false;

                const {currentRow} = field.widget;
                const defaultRow = data.find(operacao => operacao.CDTIPOOPER === '01');
                if(defaultRow)
                    setCurrentRowWithOutData(currentRow, defaultRow, field.outData);
                else
                    field.value(null);
                field.change();
                deferred.resolve();
            });
        return deferred;
    };
    
    var setCamposDefaultValueCabecalho = function(lancamentoCabecalhoArgs) {
        var deferred = ZHPromise.defer();
        Df11000_Lancamento_Service.getDefaultNovaNotaSaida().then((defaultNota) => {
            lancamentoCabecalhoArgs.owner.widget.currentRow.NMFILIAL = defaultNota[0].NMFILIAL;
            lancamentoCabecalhoArgs.owner.widget.currentRow.CDFILIEMIT = defaultNota[0].CDFILIEMIT;
            lancamentoCabecalhoArgs.owner.widget.currentRow.NMEMPRESA = defaultNota[0].NMEMPRESA;
            lancamentoCabecalhoArgs.owner.widget.currentRow.CDEMPRESA = defaultNota[0].CDEMPRESA;
            lancamentoCabecalhoArgs.owner.widget.currentRow.CDINSCESTA = defaultNota[0].CDINSCESTA;
            lancamentoCabecalhoArgs.owner.widget.currentRow.CDINSCMUNI = defaultNota[0].CDINSCMUNI;
            lancamentoCabecalhoArgs.owner.widget.currentRow.CDSERIE = defaultNota[0].CDSERIE;
            lancamentoCabecalhoArgs.owner.widget.currentRow.DSSERIE = defaultNota[0].DSSERIE;
            lancamentoCabecalhoArgs.owner.widget.currentRow.OPERADOR = defaultNota[0].CDOPERADOR;
            lancamentoCabecalhoArgs.owner.widget.currentRow.IEREM = defaultNota[0].CDINSCESTA;
            lancamentoCabecalhoArgs.owner.widget.currentRow.SGESTADOREM = defaultNota[0].SGESTADO;
            lancamentoCabecalhoArgs.owner.widget.currentRow.NMMUNICIPIOREM = defaultNota[0].NMMUNICIPIO;
            lancamentoCabecalhoArgs.owner.widget.currentRow.DSENDEREM = defaultNota[0].DSENDEFILI;
            lancamentoCabecalhoArgs.owner.widget.currentRow.NMBAIRREM = defaultNota[0].NMBAIRFILI;
            lancamentoCabecalhoArgs.owner.widget.currentRow.NRCEPREM = defaultNota[0].NRCEPFILI;
            lancamentoCabecalhoArgs.owner.widget.currentRow.NRTELEREM = defaultNota[0].NRTELEFILI;
            lancamentoCabecalhoArgs.owner.widget.currentRow.CDOPERGERA = defaultNota[0].CDOPERADOR;
            lancamentoCabecalhoArgs.owner.widget.currentRow.DTENTRSAID = defaultNota[0].DTENTRSAID;
            
            reloadCampoTipoOperacao(lancamentoCabecalhoArgs.owner.widget);
            reloadSerie(lancamentoCabecalhoArgs.owner.widget);

            
            validaCampoSerie(lancamentoCabecalhoArgs.row);
            deferred.resolve();
        });
        return deferred;
    };

/*    var setCamposDefaultValueCabecalho = function(lancamentoCabecalhoArgs) {
        var deferred = ZHPromise.defer();
        Df11000_Lancamento_Service.buscaOperadorLogado().then(function(operador){
            var defaultValue = lancamentoCabecalhoArgs.owner.widget.getField('NMFILIAL').dataSource.data.filter(function(data) {
                //@todo filial logada
                return data.CDFILIAL == operador[0].CDFILIALSELECIONADA;
            });
            defaultValue = defaultValue[0];
            lancamentoCabecalhoArgs.owner.widget.currentRow.NMFILIAL = defaultValue.NMFILIAL;
            lancamentoCabecalhoArgs.owner.widget.currentRow.CDFILIEMIT = defaultValue.CDFILIAL;
            lancamentoCabecalhoArgs.owner.widget.currentRow.NMEMPRESA = defaultValue.NMEMPRESA;
            lancamentoCabecalhoArgs.owner.widget.currentRow.CDEMPRESA = defaultValue.CDEMPRESA;
            lancamentoCabecalhoArgs.owner.widget.currentRow.CDINSCESTA = defaultValue.CDINSCESTA;
            lancamentoCabecalhoArgs.owner.widget.currentRow.CDINSCMUNI = defaultValue.CDINSCMUNI;
            lancamentoCabecalhoArgs.owner.widget.currentRow.CDSERIE = defaultValue.CDSERIE;
            lancamentoCabecalhoArgs.owner.widget.currentRow.DSSERIE = defaultValue.DSSERIE;
            lancamentoCabecalhoArgs.owner.widget.currentRow.OPERADOR = defaultValue.CDOPERADOR;
            lancamentoCabecalhoArgs.owner.widget.currentRow.IEREM = defaultValue.CDINSCESTA;
            lancamentoCabecalhoArgs.owner.widget.currentRow.SGESTADOREM = defaultValue.SGESTADO;
            lancamentoCabecalhoArgs.owner.widget.currentRow.NMMUNICIPIOREM = defaultValue.NMMUNICIPIO;
            lancamentoCabecalhoArgs.owner.widget.currentRow.DSENDEREM = defaultValue.DSENDEFILI;
            lancamentoCabecalhoArgs.owner.widget.currentRow.NMBAIRREM = defaultValue.NMBAIRFILI;
            lancamentoCabecalhoArgs.owner.widget.currentRow.NRCEPREM = defaultValue.NRCEPFILI;
            lancamentoCabecalhoArgs.owner.widget.currentRow.NRTELEREM = defaultValue.NRTELEFILI;

            Df11000_Lancamento_Service.buscaDataProcessamento(operador[0].CDFILIALSELECIONADA).then(function(processamento){
                var dtProcess = processamento[0].DTPROCESSA;
                var dtProcessFormat = moment(dtProcess, 'DD/MM/YYYY').format('DD/MM/YYYY');
                                var today = moment().format('DD/MM/YYYY');
                                var nowFullTime = moment().format('DD/MM/YYYY');
                                if(dtProcessFormat === today){
                                  dtProcessFormat = nowFullTime;
                                }else{
                                    dtProcessFormat = moment(dtProcess, 'DD/MM/YYYY').format('DD/MM/YYYY');
                                }
                lancamentoCabecalhoArgs.owner.widget.currentRow.DTENTRSAID = dtProcessFormat;
            });

            validaCampoSerie(lancamentoCabecalhoArgs.row);
            deferred.resolve();
        });
        return deferred;
    };*/

    this.cancelaEdicaoCabecalho = function(args){
        General.backOrCancel(args).then(function(){
            alteraVisibilidadeCampoInscricao();
            controlaStatusWidgets(args.owner.widget, args.owner.widget.name, true);
        });
        args.owner.widget.reload();
    };
    
    this.dateOnChange = function(dateField){
        var today = moment().format('DD/MM/YYYY');
        var nowFullTime = moment().format('DD/MM/YYYY');
        if(moment(dateField.value(), 'DD/MM/YYYY').format('DD/MM/YYYY') === today)
            dateField.value(nowFullTime);
    };

    var alteraVisibilidadeCampoInscricao = function(widget) {
        widget =  !widget ? getCabecalhoWidget() : widget  ; 
        var cabecalhoWidget = getCabecalhoWidget();
        var cabecalhoCurrentRow = widget.currentRow;
        var cliente = widget.getField("NMRAZSOCCLIE");
        var fornecedor = widget.getField("NMRAZSOCFORN");
        var filial = widget.getField("DSFILIRECE");
        var tipoDestino = cabecalhoCurrentRow.IDORIGDESTOP || cabecalhoCurrentRow.IDORIGDESTNF;

        switch (tipoDestino) {
            case "C":
                validacaoCamposOutTpOperac(cliente, fornecedor, filial);
                break;
            case "F":
                validacaoCamposOutTpOperac(fornecedor, cliente, filial);
                break;
            case "L":
                validacaoCamposOutTpOperac(filial, fornecedor, cliente);
                break;
            default:
                filial.readOnly = true;
                cliente.readOnly = true;
                fornecedor.readOnly = true;
        }
    };

    this.actNovoDadosAdicionaisOnClick = function(args) {
        args.owner.widget.newRow();
        Df11000_Lancamento_Service.getProxSeqDadosAdicionais().then(function(nrdadoadic){
           args.owner.widget.currentRow.NRDADOADIC = nrdadoadic;
        });
    };

    this.actEditarDadosAdicionaisOnClick = function(detalhesDadosAdicionaisWidget) {
        General.edit(detalhesDadosAdicionaisWidget);
    };

    this.limpaCampoCodigoDadosAdic = function(dadosAdicArgs) {
        dadosAdicArgs.row.CDMENSAGEM = null;
    };

    this.actIncluirDadosAdicionaisOnClick = function(lancamentoDadosAdicionaisArgs) {
         if (lancamentoDadosAdicionaisArgs.owner.widget.isValid()) {
            lancamentoDadosAdicionaisArgs.row.NRLANCTONF = lancamentoDadosAdicionaisArgs.owner.widget.getParent().currentRow.NRLANCTONF;
            lancamentoDadosAdicionaisArgs.row.CDFILIAL = lancamentoDadosAdicionaisArgs.owner.widget.getParent().currentRow.CDFILIAL;

            Df11000_Lancamento_Service.salvaDadosAdicionais(lancamentoDadosAdicionaisArgs.row).then(function() {
                alteraStatusNotafiscal(lancamentoDadosAdicionaisArgs.owner.widget);
                var closeSwipe = lancamentoDadosAdicionaisArgs.row.__is_new === true;
                //@todo remover __is_new quando widget.edit() funcionar
                lancamentoDadosAdicionaisArgs.row.__is_new = false;
                Df11000_Lancamento_Service.setDataSourceDadosAdicionais(lancamentoDadosAdicionaisArgs.row).then(function(){
                    Df11000_Lancamento_Service.atualizaLancamento().then(function(){
                        Df11000_Lancamento_Service.setDataSourceLancamentoNota();
                        General.backOrCancel(lancamentoDadosAdicionaisArgs);
                        if(closeSwipe)
                            ScreenService.closeSwipe();
                        lancamentoDadosAdicionaisArgs.owner.widget.reload();
                    });
                });
            }, requestError);
         }
    };

    this.actExcluirDadosAdicionaisOnClick = function(lancamentoDadosAdicionaisArgs) {
        ScreenService.confirmMessage('Deseja excluir a mensagem?', 'question', function (success) {
            Df11000_Lancamento_Service.excluiDadosAdicionais(lancamentoDadosAdicionaisArgs.row).then(function () {
                alteraStatusNotafiscal(lancamentoDadosAdicionaisArgs.owner.widget);
                Df11000_Lancamento_Service.excluiDadosAdicionaisDataSource(lancamentoDadosAdicionaisArgs.row.NRDADOADIC).then(function () {
                    Df11000_Lancamento_Service.atualizaLancamento().then(function () {
                        Df11000_Lancamento_Service.setDataSourceLancamentoNota();
                        General.backOrCancel(lancamentoDadosAdicionaisArgs);
                        lancamentoDadosAdicionaisArgs.owner.widget.reload();
                    });
                });
            }, requestError);
        });
    };

    this.setDtsEncargoOnEnter = function(lancamentoEncargoArgs) {
        if((lancamentoEncargoArgs.owner.getParent().currentRow.IDSTATUS === "D") && (!lancamentoEncargoArgs.owner.dataSource.data || !lancamentoEncargoArgs.owner.dataSource.data.length)){
            Df11000_Lancamento_Service.buscaEncargosPadrao().then(function(encargos){
                encargos.forEach(function(encargo){
                    encargo.VRITEMENCA = 0;
                    encargo.NRLANCTONF = lancamentoEncargoArgs.owner.getParent().currentRow.NRLANCTONF;
                    encargo.CDFILIAL = lancamentoEncargoArgs.owner.getParent().currentRow.CDFILIAL;
                    encargo.VRENMOEDESTR = "";
                    encargo.IDALTERADO = "N";
                    encargo.__is_new = true;
                });
                Df11000_Lancamento_Service.setDataSourceEncargo(encargos).then(function(){
                    lancamentoEncargoArgs.owner.currentRow = encargos;
                    lancamentoEncargoArgs.owner.reload();
                });
            }, requestError);
        }
    };

    this.salvaEncargoInlineEdit = function(lancamentoEncargoArgs) {
        var encargosSalvar;
         if (lancamentoEncargoArgs.owner.isValid()) {
         //    GerUtil.desmarcaLinhasEditadas(lancamentoEncargoOwner, SALVAR);
             if(lancamentoEncargoArgs.row.__is_new === true){
                 encargosSalvar = lancamentoEncargoArgs.owner.dataSource.data;
             }
             else{
                 encargosSalvar = lancamentoEncargoArgs.dataSource.filter(function(encargo){
                    return encargo.changed === true;
                });
             }

            Df11000_Lancamento_Service.salvaEncargo(encargosSalvar).then(function(){
                alteraStatusNotafiscal(lancamentoEncargoArgs.owner);
                encargosSalvar.forEach(function(encargo){
                    encargo.__is_new = false;
                    //@todo remover delete encargo.changed quando desmarcaLinhasEditadas for implementada
                    delete encargo.changed_VRITEMENCA;
                    delete encargo.changed;
                });
                Df11000_Lancamento_Service.setDataSourceEncargo(lancamentoEncargoArgs.dataSource).then(function(){
                    Df11000_Lancamento_Service.atualizaLancamento().then(function(){
                        //@todo desmarcaLinhasEditadas
                        //GerUtil.desmarcaLinhasEditadas(lancamentoEncargoWidget, CANCELAR);
                        Df11000_Lancamento_Service.setDataSourceLancamentoNota();
                        lancamentoEncargoArgs.owner.view();
                        lancamentoEncargoArgs.owner.reload();
                        controlaStatusWidgets(lancamentoEncargoArgs.owner.getParent(), lancamentoEncargoArgs.owner.name, true);
                    });
                });
            }, requestError);
         }
    };

    this.cancelaEncargoInlineEdit = function(lancamentoEncargoWidget) {
         //GerUtil.desmarcaLinhasEditadas(lancamentoEncargoWidget, CANCELAR);
         //delete encargo["changed_VRITEMENCA"];
         //delete encargo.changed;
        lancamentoEncargoWidget.dataSource.data.forEach(function(encargo){
            delete encargo.changed_VRITEMENCA;
            delete encargo.changed;
        });
        lancamentoEncargoWidget.view();
        controlaStatusWidgets(lancamentoEncargoWidget.getParent(), lancamentoEncargoWidget.name, true);
    };

    /* this.validaEncargoWizardBeforeNextStage = function(lancamentoSaidaEncargoArgs) {
    //     /* Remover todas acões relacionadas a variavel "nextStage" quando o processo de
    //      * "WizardBeforeNextStage, WizardAfterNextStage" se tornar síncrono
    //      * /
    //     if (!nextStage) {
    //         lancamentoSaidaEncargoArgs.e.preventDefault();

    //         LancamentoNFSaidaEncargoService.validaEncargoMaiorQuantItens().then(function() {
    //             nextStage = true;
    //             wizardEngine.next().then(function() {
    //                 nextStage = false;
    //             });
    //         }, function(error) {
    //             ScreenService.showMessage(error);
    //         });
    //     }
    // }; */
    
    const modificaDatasourceFilterProdutoServico = action => {
        const field = action.widget.getField('NMPRODUTO');

        field.dataSourceFilter = field.dataSourceFilter.filter(filter => filter.name != 'CDCLIENTE');

        if(action.name === 'novoServico'){
            field.dataSource.name = 'df_saida#/crud/servclie_servico_produto';

            const cliente = action.widget.getParent().currentRow.CDCLIENTE;
            const filterCdcliente = {
                name: 'CDCLIENTE',
                operator: '=',
                value: cliente
            };

            field.dataSourceFilter = [...field.dataSourceFilter, filterCdcliente];
            field.fields[0].isVisible = false;
            field.fields[1].isVisible = false;
            field.fields[2].isVisible = true;
            field.fields[3].isVisible = true;
            
            field.displayedFields       =  ["CDSERVFILI", "NMSERVICO"];
            field.searchableFields      =  ["CDSERVFILI", "NMSERVICO"];
            delete field.outData.NMPRODUTO;
            field.outData.NMSERVICO     = ['NMPRODUTO', 'NMSERVICO'];
            field.descriptionField      = 'NMSERVICO';
        } else if(action.name === 'novoItem') {
            this.verificaDataSourceProduto(field);
            field.fields[0].isVisible = true;
            field.fields[1].isVisible = true;
            field.fields[2].isVisible = false;
            field.fields[3].isVisible = false;
            field.displayedFields       =  ["CDARVPROD", "NMPRODUTO"];
            field.searchableFields =  ["CDARVPROD", "NMPRODUTO"];
            delete field.outData.NMPRODUTO;
            field.outData.NMPRODUTO     = 'NMPRODUTO';
            field.descriptionField  = 'NMPRODUTO';
        }

        field.dataSource.data = [];
        field.reload();

        return field.dataSource.name;
    };

    var verificaTipoItemProdutoServico = function(widget){
        if(widget.getParent().currentRow.IDOPERDEVONF !== "S"){
            if(widget.getParent().appliedFilter.FILTER_ROW.IDBLOQITNFAPUR !== "S"){
                if(widget.getField("NMPRODUTO").dataSource.name === "df#/produtoLancamento"){
                    widget.getAction("novoItem").isVisible = true;
                    widget.getAction("novoServico").isVisible = false;

                }
                else{
                    widget.getAction("novoItem").isVisible = false;
                    widget.getAction("novoServico").isVisible = true;
                }
            }
            else{
                widget.getAction("novoItem").isVisible = true;
                widget.getAction("novoServico").isVisible = false;
            }
        }
    };

    this.actNovoItemOnClick = function(args) {
        var itemWidget = args.owner.widget;
        var parentWidget = itemWidget.getParent();
        var dataSourceName = modificaDatasourceFilterProdutoServico(args.owner);

        var filter = [
            {
                'name': 'CDFILIAL', 
                'operator': '=', 
                'value': args.owner.widget.getParent().currentRow.CDFILIEMIT
            }
        ];
        
        General.callRestEngine('df#/crud/paramfilial', 'FilterData', filter).then(function(params){
            if((params.dataset.paramfilial[0].IDPERPROINAT == 'N') || (params.dataset.paramfilial[0].IDPERPROINAT == null)){
                var idproativpr = 
                {
                    "name": "IDPRODATIVPR",
                    "operator": "=",
                    "value": "S"
                };
                var idproatv = 
                {
                    "name": "IDPRODATIV",
                    "operator": "=",
                    "value": "A" 
                };
                //templateManager.container.getWidget('itens').getField('NMPRODUTO').dataSourceFilter.push(idproativpr);
                //templateManager.container.getWidget('itens').getField('NMPRODUTO').dataSourceFilter.push(idproatv);
            }/* else{
                var dataSourceFilter = templateManager.container.getWidget('itens').getField('NMPRODUTO').dataSourceFilter;
                if(dataSourceFilter.length > 2){
                    dataSourceFilter.pop();
                    dataSourceFilter.pop();
                }
            }*/
        });
        
        if(dataSourceName === "df_saida#/crud/servclie_servico_produto") {
            var controlaMaxItens = parentWidget.currentRow.IDCONMAXITEM;
            var numMaxServicos = parentWidget.currentRow.NRMAXITNF;
            if(controlaMaxItens != "N" && args.owner.widget.dataSource.data.length >= numMaxServicos)
                return ScreenService.showMessage("Foi alcançada a quantidade máxima de serviços no lançamento");
            itemWidget.label = 'Serviços';
        }

        itemWidget.newRow();
        Df11000_Lancamento_Service.getProxSeqItens().then(function(nrseqitprod){
            itemWidget.currentRow.NRSEQITPROD = nrseqitprod;
            controlaStatusWidgets(itemWidget, itemWidget.name, false);
        });
        General.justInteger('#NRSEQPEDPROD');

    };
    
    this.vencimentoWidgetAfterLoadDataSource = function(){
        var vencimentoWidget = templateManager.container.getWidget('vencimentos');
        
        if(vencimentoWidget.dataSource.data.length == 0){
            vencimentoWidget.dataSource.reload();
        }
    };

    var habilitaCamposCabecalhoNotaOrigem = function(cabecalhoWidget, lancamentoCabecalhoRow){
        var verificaUtilizaNotaOrigem = lancamentoCabecalhoRow.IDOPERDEVONF === "S";
        var serieOrigem = cabecalhoWidget.getField("CDSERIEORIGEM");
        var notaOrigem = cabecalhoWidget.getField("DSNOTAORIGEM");

        if(verificaUtilizaNotaOrigem){
            serieOrigem.required = true;
            notaOrigem.required = true;
            serieOrigem.isVisible = true;
            serieOrigem.readOnly = true;
            notaOrigem.isVisible = true;
        }
        else
        {
            delete serieOrigem.required;
            delete notaOrigem.required;
            serieOrigem.isVisible = false;
            notaOrigem.isVisible = false;
        }
    };

    var habilitaCamposItemNotaOrigem = function(lancamentoItensWidget, lancamentoCabecalhoRow) {
        var isNFComplemento = lancamentoCabecalhoRow.CDTIPOOPER == "03";
        var verificaUtilizaNotaOrigem = lancamentoCabecalhoRow.IDOPERDEVONF === "S" && !isNFComplemento;

        lancamentoItensWidget.getField('QTITEMORIG').isVisible = verificaUtilizaNotaOrigem;
        lancamentoItensWidget.getField('VRUNITITEM').readOnly = verificaUtilizaNotaOrigem;
        lancamentoItensWidget.getField('VRDESCITEM').readOnly = verificaUtilizaNotaOrigem;
        lancamentoItensWidget.getField('NMPRODUTO').readOnly = verificaUtilizaNotaOrigem;
        lancamentoItensWidget.getField('CDPRODEMIS').readOnly = verificaUtilizaNotaOrigem;
        lancamentoItensWidget.getField('DSPRODEMIS').readOnly = verificaUtilizaNotaOrigem;
        lancamentoItensWidget.getField('VRTOTAITEM').readOnly = verificaUtilizaNotaOrigem;
        lancamentoItensWidget.getField('DSINFADICPROD').readOnly = verificaUtilizaNotaOrigem;

        if(lancamentoCabecalhoRow.IDSTATUS !== "A" && lancamentoCabecalhoRow.IDSTATUS !== "C"){
            lancamentoItensWidget.getAction('novoItem').isVisible = !verificaUtilizaNotaOrigem;
            lancamentoItensWidget.getAction('excluirItem').isVisible = !verificaUtilizaNotaOrigem;
        }

        //if (verificaUtilizaNotaOrigem) {
        //    lancamentoItensWidget.getField('QTITEMPRODORIG').class = 4;
        //    lancamentoItensWidget.getField('QTITEMPROD').class = 4;
        //    lancamentoItensWidget.getField('VRUNITITEM').class = 4;
        //} else {
        //    lancamentoItensWidget.getField('QTITEMPRODORIG').class = 12;
        //    lancamentoItensWidget.getField('QTITEMPROD').class = 6;
        //    lancamentoItensWidget.getField('VRUNITITEM').class = 6;
        //}
    };

    this.actCancelarEdicaoItem = function(args){
        General.backOrCancel(args);
        var widget = args.owner.widget;
        widget.reload();
        let servicos = false; 
        widget.label = 'Produtos';
        controlaStatusWidgets(args.owner.widget, args.owner.widget.name, true);
    };

    this.actIncluirItemOnClick = function(lancamentoItemArgs) {
        lancamentoItemArgs.owner.widget.label = 'Produtos';
         if (lancamentoItemArgs.owner.widget.isValid()) {
             lancamentoItemArgs.row.NRLANCTONF = lancamentoItemArgs.owner.widget.getParent().currentRow.NRLANCTONF;
             lancamentoItemArgs.row.CDFILIAL = lancamentoItemArgs.owner.widget.getParent().currentRow.CDFILIAL;
             lancamentoItemArgs.row.IDATUVRESTOQ = 'S';
             lancamentoItemArgs.row.VRCONTITEM = '0';
             lancamentoItemArgs.row.IDCONFORITEM = 'S';
             lancamentoItemArgs.row.VRBRUTITEM = '0';
             if(!lancamentoItemArgs.row.VRDESCITEM){
                 lancamentoItemArgs.row.VRDESCITEM = '0';
             }

             if(Df11000_Lancamento_Service.verificaUtilizaNotaOrigem(lancamentoItemArgs.owner.widget.getParent().currentRow) && lancamentoItemArgs.row.QTITEMPROD === "0" && lancamentoItemArgs.row.__is_new === false){
                 Df11000_Lancamento_Service.excluiItem(lancamentoItemArgs.row).then(function() {
                     lancamentoItemArgs.row.__is_new = true;
                     Df11000_Lancamento_Service.setDataSourceItem(lancamentoItemArgs.row).then(function () {
                         Df11000_Lancamento_Service.atualizaLancamento().then(function () {
                             Df11000_Lancamento_Service.setDataSourceLancamentoNota();
                             lancamentoItemArgs.owner.widget.view();
                             //General.backOrCancel(lancamentoItemArgs);
                             lancamentoItemArgs.owner.widget.reload();
                             controlaStatusWidgets(lancamentoItemArgs.owner.widget, lancamentoItemArgs.owner.widget.name, true);
                         });
                     });
                 }, requestError);
             }
             else{
                 Df11000_Lancamento_Service.salvaItem(lancamentoItemArgs.owner.widget.getParent().currentRow, lancamentoItemArgs.row).then(function(){
                     alteraStatusNotafiscal(lancamentoItemArgs.owner.widget);
                     //lancamentoItemArgs.owner.widget.getParent().currentRow.IDSTATUS = 'D';
                     var closeSwipe = lancamentoItemArgs.row.__is_new === true;
                     //@todo remover __is_new quando widget.edit() funcionar
                     lancamentoItemArgs.row.__is_new = false;
                     verificaTipoItemProdutoServico(lancamentoItemArgs.owner.widget);
                     Df11000_Lancamento_Service.setDataSourceItem(lancamentoItemArgs.row).then(function(){
                         Df11000_Lancamento_Service.atualizaLancamento().then(function(){
                             Df11000_Lancamento_Service.setDataSourceLancamentoNota();
                             lancamentoItemArgs.owner.widget.view();
                             //General.backOrCancel(lancamentoItemArgs);
                             if(closeSwipe)
                                 ScreenService.closeSwipe();
                             lancamentoItemArgs.owner.widget.reload();
                             controlaStatusWidgets(lancamentoItemArgs.owner.widget, lancamentoItemArgs.owner.widget.name, true);
                         });
                     });
                 }, function(error) {
                    if (Array.isArray(error)) {
                        var stringCamposObrig = error.join(', ');
                        var mensagem = "CAMPOS_MAIORES_ZERO";
                        MessageService.showMessageByCode(mensagem, {"PARAMETRO": stringCamposObrig});
                    } else if (typeof error === 'string')
                        ScreenService.showMessage(error);
                    if(error.data)
                        ScreenService.showMessage(error.data.error);
                });
             }
         }
    };

    this.actEditarItemOnClick = function(detalhesItemWidget) {
        if(Df11000_Lancamento_Service.verificaUtilizaNotaOrigem(detalhesItemWidget.getParent().currentRow)){
            Df11000_Lancamento_Service.buscaQuantidadeDevolvida(detalhesItemWidget.getParent().currentRow, detalhesItemWidget).then(function(data) {
                detalhesItemWidget.currentRow.QTITEMDEVO = data.length !== 0 ? data[0].QTITEMDEVO : "0";
                detalhesItemWidget.currentRow.QTITEMPRODBEFOREEDIT = detalhesItemWidget.currentRow.QTITEMPROD;
                General.edit(detalhesItemWidget);
            }, function(error) {
                if (!!error)
                    ScreenService.showMessage('Ocorreu um erro ao buscar a quantidade de itens disponíveis.' + error.data.error);
            });
        }
        else{
            General.edit(detalhesItemWidget);
        }
        General.justInteger('#NRSEQPEDPROD');
    };

    this.actExcluirItemOnClick = function(lancamentoItemArgs) {
        ScreenService.confirmMessage('Deseja excluir o item?', 'question', function (success) {
            Df11000_Lancamento_Service.excluiItem(lancamentoItemArgs.row).then(function(){
                alteraStatusNotafiscal(lancamentoItemArgs.owner.widget);
                Df11000_Lancamento_Service.excluiItemDataSource(lancamentoItemArgs.row.NRSEQITPROD).then(function(){
                    Df11000_Lancamento_Service.atualizaLancamento().then(function(){
                        Df11000_Lancamento_Service.setDataSourceLancamentoNota();
                        General.backOrCancel(lancamentoItemArgs);
                        lancamentoItemArgs.owner.widget.reload().then(function(){
                            verificaTipoItemProdutoServico(lancamentoItemArgs.owner.widget);
                        });
                    });
                });
            }, requestError);
        });
    };

    var setValoresEmissaoProdutoOnChange = function(lancamentoItemArgs) {
        if(lancamentoItemArgs.row.CDSERVFILI) {
            General.callRestEngine("df#/crud/paramgeral", "FilterData", []).then(function(success) {
                var resul = success.dataset.paramgeral;
                if(resul[0].IDUTPROPEMIS == 'S') {
                    lancamentoItemArgs.row.CDPRODEMIS = lancamentoItemArgs.row.CDARVPROD;
                    lancamentoItemArgs.row.DSPRODEMIS = lancamentoItemArgs.row.NMPRODUTO;
                }
                else {
                    lancamentoItemArgs.row.CDPRODEMIS = lancamentoItemArgs.row.CDSERVFILI;
                    if(lancamentoItemArgs.row.DSEMISSERCLI) {
                        lancamentoItemArgs.row.DSPRODEMIS = lancamentoItemArgs.row.DSEMISSERCLI;
                    }
                    else {
                        lancamentoItemArgs.row.DSPRODEMIS = lancamentoItemArgs.row.NMSERVICO;
                    }
                }
            });
        }
        else {
            lancamentoItemArgs.row.CDPRODEMIS = lancamentoItemArgs.row.CDARVPROD;
            lancamentoItemArgs.row.DSPRODEMIS = lancamentoItemArgs.row.NMPRODUTO;
        }
    };
    
    var consultaCustoQuantidadeProdutoOnChange = function(lancamentoItemArgs) {
         var filial = lancamentoItemArgs.owner.field.widget.getParent().currentRow.CDFILIEMIT;
         var dtLancamento = lancamentoItemArgs.owner.field.widget.getParent().currentRow.DTENTRSAID;
         var produto = lancamentoItemArgs.row.CDPRODUTO;
         var operacao = lancamentoItemArgs.owner.field.widget.getParent().currentRow.CDTIPOOPER;

         lancamentoItemArgs.row.VRUNITITEM = "";
         lancamentoItemArgs.row.QTITEMPROD = "";
         lancamentoItemArgs.row.VRTOTAITEM = "";

        if (produto) {
            if(lancamentoItemArgs.row.CDSERVFILI) {
                lancamentoItemArgs.row.VRUNITITEM = lancamentoItemArgs.row.VRSERVFATU;
            }
            else {
                ValidacoesEstoque_Service.consultaCustoQuantidadeProduto(filial, produto, dtLancamento, null , null, null, null, null , operacao).then(function(custoQuant) {
                    custoQuant = custoQuant.dataset.consultaCustoQuantidadeProduto.VRCUSTOPROD;
                    lancamentoItemArgs.row.VRUNITITEM = typeof(custoQuant) === "string" ? parseFloat(custoQuant) : custoQuant;
                }, requestError);
            }
        }
    };

    var informacoesComplementaresProduto = function(lancamentoItemArgs){
        var itensWidget = lancamentoItemArgs.owner.field.widget;
        
        var filter = [
            { "name": "CDTIPOOPER", "operator": "=", "value": getCabecalhoWidget().currentRow.CDTIPOOPER},
            {"name": "IDENTRSAIDOP", "operator": "=", "value":  "S"}
        ];
         
        lancamentoItemArgs.row.CDFILIMOVI = getCabecalhoWidget().currentRow.CDFILIEMIT;
      
        General.callRestEngine("df#/crud/tipooperac", "FilterData", filter).then((success) => {
            
            atualizaEstoque = success.dataset.tipooperac[0].IDATUESTTPOP;
            itensWidget.paramInfoCompl.DocuDevo =  getCabecalhoWidget().currentRow.IDOPERDEVONF == 'S';
            itensWidget.paramInfoCompl.AtuEstoq = atualizaEstoque == 'S';
        
            Lib00206_InfComplProduto_Controller.infComplementares(itensWidget, lancamentoItemArgs.row, false, atualizaEstoque);
        });

    };

    this.validacoesProdutoOnChange = function(lancamentoItemArgs) {
        if(getCabecalhoWidget().currentRow.IDNFAJUSTE != 'S' && getCabecalhoWidget().currentRow.CDTIPOOPER != '03') 
            informacoesComplementaresProduto(lancamentoItemArgs);
        setValoresEmissaoProdutoOnChange(lancamentoItemArgs);
        consultaCustoQuantidadeProdutoOnChange(lancamentoItemArgs);
    };

    this.afterCancelInfoCompl = function(args){
        args.owner.widget = args.owner;
        this.actCancelarEdicaoItem(args);
    };

    var calculaValorTotal = function(lancamentoItemArgs) {
        if(Df11000_Lancamento_Service.verificaUtilizaNotaOrigem(getCabecalhoWidget().currentRow) && lancamentoItemArgs.row.QTITEMPROD === "0"){
            lancamentoItemArgs.row.VRTOTAITEM = lancamentoItemArgs.row.VRUNITITEM;
        }
        else{
            lancamentoItemArgs.row.VRTOTAITEM = Df11000_Lancamento_Service.calculaValorTotal(
                lancamentoItemArgs.row.QTITEMPROD, lancamentoItemArgs.row.VRUNITITEM
            );
        }
    };

    this.valorUnitarioFieldOnChange = function(lancamentoItemArgs) {
         lancamentoItemArgs.row.VRUNITITEM =  typeof(lancamentoItemArgs.row.VRUNITITEM) === "string" ? parseFloat(lancamentoItemArgs.row.VRUNITITEM.replace(/[.]+/g, '').replace("," , ".")): lancamentoItemArgs.row.VRUNITITEM;
         calculaValorTotal(lancamentoItemArgs);
    };

    this.quantidadeFieldOnChange = function(lancamentoItemArgs) {
        /* Se QTESTOQDIA não exister na row significa que a "Informações Complementares de Produto" não foi aberta
        porntanto, trat-ase de um produto que não controla estoque ou a operação está sendo feita com uma filial
        que não controla estoque, logo a validação de quantidade não deve ser feita. */
        if(lancamentoItemArgs.row.QTESTOQDIA && parseFloat(lancamentoItemArgs.row.QTITEMPROD) > lancamentoItemArgs.row.QTESTOQDIA) {
            MessageService.showMessageByCode("QUANTIDADE_ESTOQUE",{"QTESTOQDIA": parseFloat(lancamentoItemArgs.row.QTESTOQDIA), "SGUNIDADE": parseFloat(lancamentoItemArgs.row.SGUNIDADE)});
            lancamentoItemArgs.row.QTITEMPROD = 0.00;
        }
        else{
           
            lancamentoItemArgs.row.QTITEMPROD =  typeof(lancamentoItemArgs.row.QTITEMPROD) === "string" ? parseFloat(lancamentoItemArgs.row.QTITEMPROD.replace(/[.]+/g, '').replace("," , ".")): lancamentoItemArgs.row.QTITEMPROD;
            calculaValorTotal(lancamentoItemArgs);
        } 
    };

    this.valorTotalFieldOnChange = function(lancamentoItemArgs) {
         lancamentoItemArgs.row.VRUNITITEM = Df11000_Lancamento_Service.calculaValorUnitario(
             lancamentoItemArgs.row.QTITEMPROD, lancamentoItemArgs.row.VRTOTAITEM
         );
    };

    var buscaItensNotaOrigem = false;

    this.setDtsItensNotaOrigemOnEnter = function(lancamentoItensArgs) {

        if((buscaItensNotaOrigem && Df11000_Lancamento_Service.verificaUtilizaNotaOrigem(lancamentoItensArgs.owner.getParent().currentRow))){
            Df11000_Lancamento_Service.buscaItensNotaOrigem(lancamentoItensArgs.owner.getParent().currentRow).then(function(itens) {
                itens.forEach(function(item){
                    var itemJaSalvo = lancamentoItensArgs.owner.dataSource.data.filter(function(data){
                        if(data.NRSEQITPROD === item.NRSEQITPROD)
                            data.QTITEMORIG = item.QTITEMORIG;
                        return data.NRSEQITPROD === item.NRSEQITPROD;
                    });
                    if(itemJaSalvo.length > 0){
                        var index = itens.indexOf(item);
                        itens.splice(index, 1);
                    }
                    else{
                        item.__is_new = true;
                        lancamentoItensArgs.owner.dataSource.data.push(item);
                    }
                });
                buscaItensNotaOrigem = false;
                if(lancamentoItensArgs.owner.getParent().currentRow.IDSTATUS === 'D') {
                    Df11000_Lancamento_Service.setDataSourceItem(itens).then(() => {
                        Df11000_Lancamento_Service.atualizaLancamento().then(lancamentoItensArgs.owner.reload());
                    });
                }
                
            }, function(error) {
                lancamentoItensArgs.owner.reload();
            });
        }
        
    };

    this.actNovoVencimentoOnClick = function(args) {
        args.owner.widget.newRow();
        Dfs11000_LancamentoSaida_Service.getProxSeqVencimento().then(function(nrseqvenrec){
            args.owner.widget.currentRow.NRSEQVENREC = nrseqvenrec;
        });
        setMinMaxDateVencimento(args.owner.widget);
    };

    this.actEditarVencimentoOnClick = function(detalhesVencimentoWidget) {
        setMinMaxDateVencimento(detalhesVencimentoWidget);
        General.edit(detalhesVencimentoWidget);
    };

    var setMinMaxDateVencimento = function(widget) {
        var parentCurrentRow = widget.getParent().currentRow;
        var parentDate = moment(parentCurrentRow.DTENTRSAID, "DD/MM/YYYY");
        var dateField = widget.getField("DTATUAVENREC");
        dateField.minDate = parentDate.format("DD/MM/YYYY");
        dateField.maxDate = parentDate.add(60, "days").format("DD/MM/YYYY");
    };

    this.actIncluirVencimentoOnClick = function(lancamentoVencimentoArgs) {
         if (lancamentoVencimentoArgs.owner.widget.isValid()) {
            var dataLancamento = lancamentoVencimentoArgs.owner.widget.getParent().currentRow.DTENTRSAID;
            var dataVencimento = lancamentoVencimentoArgs.row.DTATUAVENREC;
            var dataProcessamento = getLocalVar('DATA_PROCESSAMENTO');
            var erroDataVencimento = Df11000_Lancamento_Service.validaDataVencimento(dataLancamento, dataVencimento, dataProcessamento);

            if (erroDataVencimento)
                return ScreenService.showMessage(erroDataVencimento);

            lancamentoVencimentoArgs.row.NRLANCTONF = lancamentoVencimentoArgs.owner.widget.getParent().currentRow.NRLANCTONF;
            lancamentoVencimentoArgs.row.CDFILIAL = lancamentoVencimentoArgs.owner.widget.getParent().currentRow.CDFILIAL;
            lancamentoVencimentoArgs.row.NRPARCELAREC = '00';
            lancamentoVencimentoArgs.row.VRORIGREC = lancamentoVencimentoArgs.row.VRATUAREC;
            lancamentoVencimentoArgs.row.DTORIGVENREC = lancamentoVencimentoArgs.row.DTATUAVENREC;

            Dfs11000_LancamentoSaida_Service.salvaVencimento(lancamentoVencimentoArgs.row).then(function() {
                alteraStatusNotafiscal(lancamentoVencimentoArgs.owner.widget);
                var closeSwipe = lancamentoVencimentoArgs.row.__is_new === true;
                //@todo remover __is_new quando widget.edit() funcionar
                lancamentoVencimentoArgs.row.__is_new = false;
                Df11000_Lancamento_Service.setDataSourceVencimento(lancamentoVencimentoArgs.row).then(function(){
                    Df11000_Lancamento_Service.atualizaLancamento().then(function(){
                        Df11000_Lancamento_Service.setDataSourceLancamentoNota();
                        lancamentoVencimentoArgs.owner.widget.view();
                        if(closeSwipe)
                            ScreenService.closeSwipe();
                        lancamentoVencimentoArgs.owner.widget.reload();
                    });
                });
            }, requestError);
         }
    };

    this.actExcluirVencimentoOnClick = function(lancamentoVencimentoArgs) {
        ScreenService.confirmMessage('Deseja excluir o vencimento?', 'question', function (success) {
            Dfs11000_LancamentoSaida_Service.excluiVencimento(lancamentoVencimentoArgs.row).then(function () {
                alteraStatusNotafiscal(lancamentoVencimentoArgs.owner.widget);
                Df11000_Lancamento_Service.excluiVencimentoDataSource(lancamentoVencimentoArgs.row.NRSEQVENREC).then(function () {
                    Df11000_Lancamento_Service.atualizaLancamento().then(function () {
                        Df11000_Lancamento_Service.setDataSourceLancamentoNota();
                        General.backOrCancel(lancamentoVencimentoArgs);
                        lancamentoVencimentoArgs.owner.widget.reload();
                    });
                });
            }, requestError);
        });
    };

    this.validaUtilizaVencimentoWidgetOnEnter = function(vencimentoSaidaArgs) {
        if(vencimentoSaidaArgs.owner.getParent().currentRow.IDSTATUS !== "A" || vencimentoSaidaArgs.owner.getParent().currentRow.IDSTATUS !== "C"){
            //@todo isVisible || readonly
            vencimentoSaidaArgs.owner.getAction('actNovoVencimento').isVisible = (vencimentoSaidaArgs.owner.getParent().currentRow.IDUTILVENC === 'S');
            validaEdicaoNota();
        }
    };

    this.fecharNota = function(args) {
        var row, forcaFechamento;
        
        var cabecalhoWidget = templateManager.container.getWidget('cabecalho');
        
        cabecalhoWidget.reload();
        
        if(args) {
            if(args.owner.getParent().name == 'cabecalho') var popup = true;
            row = args.row;
            forcaFechamento = false;
        } else{
            row = cabecalhoWidget.currentRow;
            forcaFechamento = true;
        }
        
        ScreenService.confirmMessage('Deseja fechar a nota fiscal?', 'question', function (success) {
            
            Df11000_Lancamento_Service.fecharNota(getCabecalhoWidget().currentRow, forcaFechamento).then(function(callback) {
                Df11000_Lancamento_Service.geraRateio();
                if (callback) {
                    if(callback.dataset.fecharNota.ERROS_REPLICACAO){
                        ErrosReplicacaoDocumentos_Utilities.errosReplicacao(args.owner.widget, callback.dataset.fecharNota.ERROS_REPLICACAO);
                        ScreenService.showMessage("Ocorreram erros ao tentar realizar a replicação da nota fiscal.");
                    }
                    else{
                        ScreenService.goBack();
                        var parentWidget = getCabecalhoWidget();
                        parentWidget.currentRow.IDSTATUS = 'P';
                        parentWidget.currentRow.VRNOTAFISC = callback.dataset.fecharNota.CABECALHO.VRNOTAFISC;
                        parentWidget.reload();
                        ScreenService.successNotification("Nota fechada com sucesso!");
                        if(popup) ScreenService.closePopup();
                    
                    }
                }
                else {
                    ScreenService.showMessage("TIMEOUT <br> Ocorreu algum erro interno no servidor. Por favor realize a operação novamente.");
                }
            }, 
            (error) => {
                if(popup) ScreenService.closePopup();
                var errosAvisos = templateManager.container.getWidget('errosAvisos');
                var bloqueia   = false;
                errosAvisos.dataSource.data = error.data.dataset.errosFechamento;
                error.data.dataset.errosFechamento.forEach((erro) => {
                    if(erro.IDERROAVISO == 'B') bloqueia = true;
                });
                errosAvisos.isVisible = true;
                errosAvisos.getAction('confirmarFechamento').isVisible = !bloqueia;
                ScreenService.openPopup(errosAvisos);
                
                ScreenService.showMessage(error.data.error);
            });

        });

            
    };

    this.nivelCompletoProduto = function(){
        return General.getLevelByMask('GER_CDPICTPROD','1').length;
    };
    
   /* this.getFieldFromFilterPopup = function(field, preference){
        
        var container =  templateManager.container;
        if(container) {
            var cabecalhoWidget = getCabecalhoWidget();
            if(cabecalhoWidget && cabecalhoWidget.appliedFilter)
                return cabecalhoWidget.appliedFilter.FILTER_ROW.field || 'TD';
        }
    };*/
    
    this.getFiltroCDFILIEMIT = function () {
        return CDFILIEMIT;
    };
    
    
    this.getFieldFromFilterPopup = function(filedName, widgetName) {
        var container =  templateManager.container;
        if(container) {
            var filterWidget;
            var filterBforeApply = templateManager.container.getActiveWidget();
            if(filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
              filterWidget = filterBforeApply;
            else {
                var f = container.getWidget(widgetName);
                if(f) {
                     filterWidget = container.getWidget(widgetName).appliedFilter;
                }
               
            }
            
            if(filterWidget) {
                var valueField;
                if(filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
                    valueField = filterWidget.currentRow[filedName];
                else
                    valueField = filterWidget.FILTER_ROW[filedName];
                    
                return valueField || 'TD';
            }
            else return null;
        }
        else return null;
    };

    this.criaDataSourceFilterAfterApply = function(args) {
        var cabecalhoWidget = args.owner;
        var dataSourceFilter = cabecalhoWidget.dataSourceFilter;
        var currentRowPopup = cabecalhoWidget.appliedFilter.FILTER_ROW;
        var camposData = ['P_DTENTRSAIDINI', 'P_DTENTRSAIDFIM', 'P_DTEMISSAOINI', 'P_DTEMISSAOFIM'];
        var datasRemover = dataSourceFilter.filter(function(field){
            return camposData.indexOf(field.name) >= 0;
        });
        if(datasRemover && datasRemover.length >= 0){
            datasRemover.forEach(function(data){
                dataSourceFilter.splice(dataSourceFilter.indexOf(data), 1);
            });
        }
        var datasFiltro = dataSourceFilter.filter(function(value) {
            return value.name === "DTENTRSAID" || value.name === 'DTEMISSAO';
        });
        if(datasFiltro.length === 1){
            var dataEmissao = {
                "name": "DTEMISSAO",
                "value": {
                    0: "TD", 1: "TD"
                }
            };
            datasFiltro.push(dataEmissao);
        }
        var filterDefault = {
            'operator': '='
        };
        datasFiltro.forEach(function(data) {
            filterDefault.name = 'P_' + data.name + 'INI';
            filterDefault.value = (data.value[0] != 'TD') ? data.value[0] : '';
            dataSourceFilter.push(Util.clone(filterDefault));
            filterDefault.name = 'P_' + data.name + 'FIM';
            filterDefault.value = (data.value[1] != 'TD') ? data.value[1] : '';
            dataSourceFilter.push(Util.clone(filterDefault));
            if(data.name == 'DTEMISSAO') {
                if(data.value[0] == 'TD') {
                    filterDefault.name = 'P_TDTEMISSAO';
                    filterDefault.value = 'TD';
                    dataSourceFilter.push(Util.clone(filterDefault));
                }
                else {
                    filterDefault.name = 'P_TDTEMISSAO';
                    filterDefault.value = '';
                    dataSourceFilter.push(Util.clone(filterDefault));
                }
            }
            if(dataSourceFilter.indexOf(data) > 0)
                dataSourceFilter.splice(dataSourceFilter.indexOf(data), 1);
        });
        
        cabecalhoWidget.dataSourceFilter = cabecalhoWidget.dataSourceFilter.filter(function(field){return field.name[1] == '_';});
        let filterCDTIPOOPER = this.getFieldFromFilterPopup('CDTIPOOPER');
        if(filterCDTIPOOPER.length && filterCDTIPOOPER != 'TD')
            cabecalhoWidget.dataSourceFilter.push({operator: "IN", name: "CDTIPOOPER", value: "@Dfs11000_LancamentoSaida_Controller.getFieldFromFilterPopup('CDTIPOOPER')@"});
        
        cabecalhoWidget.reload();
    };
    
     //----------------------------------- DFS11300 Replicação de Documentos ---------------------------------------------------//
    
    this.openReplicacaoExclusao = function(args) {
        var widgetPopup = templateManager.container.getWidget('EXCLUSAONOTAREPLICADA');
        widgetPopup.cabecalhoArgs = args;
        widgetPopup.dataSourceFilter[0].value = args.row.CDFILIAL.concat(args.row.NRLANCTONF);
        widgetPopup.isVisible = true;
        widgetPopup.activate();
        widgetPopup.edit();
        ScreenService.openPopup(widgetPopup);
        widgetPopup.reload();
    };
    
    this.closeExclusao = function(widgetPopup) {
        var widget = templateManager.container.getWidget('cabecalho');
        ScreenService.closePopup(widgetPopup);
        widget.list();
        widget.activate();
        widget.reload();
    };
    
    /**
     * Valida a widget e abre a widget
     * de replicação.
     * 
     * @param {object} widget widget de 
     * uma nota fiscal.
    */
    this.openReplicacao = function(parentWidget) {
        if(parentWidget.currentRow.CDFILIRECE && parentWidget.currentRow.CDFILIEMIT) {
            //var length = parentWidget.widgets.length;
            var widget = templateManager.container.getWidget('REPLICACAODOCUMENTOS');
            ScreenService.closePopup();
            //var widget = parentWidget.widgets[length-2];
            var widgetFilha = widget.widgets[0];
            
            changeWidgetAcessibility([...parentWidget.widgets, widget], false);
            
            widget.activate();
            widget.showAsTab = true;
            parentWidget.openTab('REPLICACAODOCUMENTOS');
            
            
            widget.fields[1].reload();
            widget.reload();
            widget.edit();
            
            
            widgetFilha.edit();
            widgetFilha.actions[0].isVisible = true;
            widgetFilha.actions[1].isVisible = true;
            widgetFilha.actions[4].isVisible = false;
            widgetFilha.actions[5].isVisible = false;
            widgetFilha.reload();
        }
        else {
            ScreenService.showMessage("Replicação indisponivel para esta nota.");
        }
    };
    
    /**
     * Altera a acessibildiade da widget.
     * 
     * @param {array} widgets widgets
     * do cabecalho.
    */
	function changeWidgetAcessibility(widgets, enabled) {
		widgets.forEach(function (widget) {
			if(widget.name !== 'REPLICACAODOCUMENTOS') widget.enabled = enabled;
			else widget.enabled = !enabled;
		});
        widgets[0].getParent().enabled = enabled;
		templateManager.updateTemplate();
	}
    
    this.produtosOnEnter = function(widget, tabName, args, preference) {

        // close replicacao
        if(widget.showAsTab == true) {
            widget.showAsTab = false;
            widget.reload();
            widget.currentRow = {};
            widget.widgets[0].currentRow = {};
            widget.widgets[0].reload();
            widget.getParent().activate();
            widget.getParent().reload().then(() => {
                this.setDtsItensNotaOrigemOnEnter(args, preference);
            });
            changeWidgetAcessibility([...widget.getParent().widgets, widget], true);
            if(widget.getParent().openTab) widget.getParent().openTab(tabName);
        }
        else {
            this.setDtsItensNotaOrigemOnEnter(args, preference);
        }
        
        //muda a visibilidade da ação de adicionar item de acordo com o status NFE
        alteraVisibilidadeActionsProduto();
    };
    
    /**
     * Cancela uma replicação.
     * 
     * @param {object} widget widget
     * de replicação.
    */
    this.closeReplicacao = function(widget, tabName) {
        if(widget.showAsTab == true) {
            widget.showAsTab = false;
            widget.reload();
            widget.currentRow = {};
            widget.widgets[0].currentRow = {};
            widget.widgets[0].reload();
            widget.getParent().activate();
            widget.getParent().reload();
            changeWidgetAcessibility([...widget.getParent().widgets, widget], true);
            if(widget.getParent().openTab) widget.getParent().openTab(tabName);
        } 
    };
    
    /**
     * Replica o documento.
     * 
     * @param {object} widget popup da replicação de 
     * documentos.
    */
    this.replicardocumento = function(widget) {
        var notafiscal = widget.getParent().getParent().currentRow;
        var erro = false;
        widget.dataSource.data.forEach(function(data) {
            if((data.CDFILIORIGRD !== notafiscal.CDFILIEMIT || data.CDFILIDESTRD !== notafiscal.CDFILIRECE) && data.IDENTRSAIDOP === 'E') erro = true;
        });
        if(erro) ScreenService.showMessage("Para toda entrada a ser gerada deve ser informada uma saída.");
        else {
            var dataset = [
                notafiscal,
                widget.getParent().currentRow,
                widget.dataSource.data
            ];
            Dfs11000_LancamentoSaida_Service.replicarDocumento(dataset).then(function(callback) {
                if(callback.dataset.replicacaoManual.ERROS_REPLICACAO){
                    
                    ErrosReplicacaoDocumentos_Utilities.errosReplicacao(widget, callback.dataset.replicacaoManual.ERROS_REPLICACAO);
                    ScreenService.showMessage("Ocorreram erros ao tentar realizar a replicação da nota fiscal.");
                }
                else {
                    ScreenService.showMessage("Replicação de documentos realizada com sucesso.");
                    this.closeReplicacao(widget.getParent(), 'cabecalho');
                }
            }.bind(this));
        }
    };
    
    /**
     * Carrega a widget de documentos e
     * altera o funcionamento para localStorage.
     * 
     * @param {object} widget widget de 
     * documentos gerados.
    */
    this.reloadDocumentos = function(widgetPai) {
        var widget = widgetPai.widgets[0];
        widget.dataSource.rest = true;
        widget.dataSource.localStorage = false;
        widget.reload().then(function(success) {
            localStorageEngine.setDataSource(widget.dataSource.name, widget.dataSource.data);
            widget.dataSource.rest = false;
            widget.dataSource.localStorage = true;
        });
    };
    
    /**
     * Abre a edição de documento.
     * E altera o readOnly/field dos campos.
     * 
     * @param {object} widget widget de 
     * documentos gerados.
    */
    this.editDocReplic = function(widget) {
        widget.actions[0].isVisible = false;
        widget.actions[1].isVisible = false;
        widget.actions[4].isVisible = true;
        widget.actions[5].isVisible = true;
        General.edit(widget);
        var operacao = widget.getField('NMTIPOOPER');
        this.readOnlyAlmoxDocumento(widget);
        
        if(widget.currentRow.IDENTRSAIDOP) {
            this.changeFieldSerie(widget.getField('CDSERIE'), widget.currentRow.IDENTRSAIDOP);
            var cdfilial = (widget.currentRow.IDENTRSAIDOP == 'S') ? 'CDFILIORIGRD' : 'CDFILIDESTRD';
            var almoxarifado = widget.getField('DSALMOXARIFE');
            var localizacao  = widget.getField('DSLOCALESTOQ');
            widget.getField('CDSERIE').readOnly = false;
            operacao.readOnly = false;
            almoxarifado.dataSourceFilter[0].value = "@General.getFieldValue('"+cdfilial+"','DOCUMENTOSGERADOS')@";
            almoxarifado.outData.CDALMOXARIFE = (widget.currentRow.IDENTRSAIDOP == 'S') ? ["CDALMOXARIFE", "CDALMOORIGRD"] : ["CDALMOXARIFE", "CDALMODESTRD"];
            localizacao.dataSourceFilter[0].value = "@General.getFieldValue('"+cdfilial+"','DOCUMENTOSGERADOS')@";
        }
    };
    
    /**
     * Salva a edição de um documento
     * no localStorage.
     * 
     * @param {object} widget widget de 
     * documentos gerados.
    */
    this.saveDoc = function(widget) {
        var data = widget.dataSource.data;
        data[widget.currentRow.__index] = widget.currentRow;
        localStorageEngine.setDataSource(widget.dataSource.name, widget.dataSource.data);
        widget.redraw();
        ScreenService.goBack();
        widget.edit();
        widget.actions[0].isVisible = true;
        widget.actions[1].isVisible = true;
        widget.actions[4].isVisible = false;
        widget.actions[5].isVisible = false;
    };
    
    /**
     * Cancela a edição de um documento.
     * 
     * @param {object} widget widget de 
     * documentos gerados.
    */
    this.cancelEditDocReplic = function(widget) {
        widget.currentRow = widget.backupRow;
        ScreenService.goBack();
        ScreenService.goBack();
        widget.edit();
        widget.actions[0].isVisible = true;
        widget.actions[1].isVisible = true;
        widget.actions[4].isVisible = false;
        widget.actions[5].isVisible = false;
    };
    
    /**
     * Verifica se filial de origem e destino são iguais.
     * Altera o readOnly dos campos ligados
     * a filial(DSALMOXARIFE, CDSERIE).
     * 
     * @param {object} field field de filial (destino ou origem)
     * da widget documentos gerados.
    */
    this.onChangeFilialDocGerado = function(field) {
        var widget  = field.widget;
        var cdfilial = (field.name == 'NMFILIORIG') ? 'CDFILIORIGRD' : 'CDFILIDESTRD';
        var cdfilial2 = (field.name == 'NMFILIORIG') ? 'CDFILIDESTRD' : 'CDFILIORIGRD';
        var filialSerie = (widget.currentRow.IDENTRSAIDOP == 'S') ? 'CDFILIORIGRD' : 'CDFILIDESTRD';
        if(widget.currentRow[cdfilial] && widget.currentRow[cdfilial2]) {
            if(widget.currentRow[cdfilial] === widget.currentRow[cdfilial2]) {
                widget.currentRow[cdfilial] = '';
                widget.currentRow[field.name] = '';
                ScreenService.showMessage('Filial de destino e origem não podem ser iguais');
            }
            else {
                if(widget.currentRow.IDENTRSAIDOP === 'S') this.setFilterSerieSaida(widget.fields[5], widget.currentRow[cdfilial]);
            }
        }
        else if(widget.currentRow.IDENTRSAIDOP === 'S' && cdfilial === 'CDFILIORIGRD' && widget.currentRow[cdfilial]) {
            this.setFilterSerieSaida(widget.fields[3], widget.currentRow[cdfilial]);
        }
        if(widget.currentRow.IDENTRSAIDOP && widget.currentRow[filialSerie]) widget.getField('CDSERIE').readOnly = false;
        else widget.getField('CDSERIE').readOnly = true;
        this.readOnlyAlmoxDocumento(widget);
    };
    
    /**
     * Altera o readOnly e filtro dos campos ligados
     * ao tipo de operação(DSALMOXARIFE, DSLOCALESTOQ, NMTIPOOPER, CDSERIE).
     * 
     * @param {object} field field de IDENTRSAIDOP (destino ou origem)
     * da widget documentos gerados.
    */
    this.onChangeIdEntrSaidaOp = function(field) {
        var widget = field.widget;
        var cdfilial = (field.value() == 'S') ? 'CDFILIORIGRD' : 'CDFILIDESTRD';
        var operacao = widget.getField('NMTIPOOPER');
        var almoxarifado = widget.getField('DSALMOXARIFE');
        var localizacao  = widget.getField('DSLOCALESTOQ');
        if(!field.value()) {
            operacao.readOnly = true;
            almoxarifado.readOnly = true;
            localizacao.readOnly = true;
        }
        else {
            this.changeFieldSerie(widget.getField('CDSERIE'), field.value());
            if(widget.currentRow[cdfilial]) {
                widget.fields[3].readOnly = false;
                widget.currentRow.CDSERIE = '';
                if(field.value() === 'S') this.setFilterSerieSaida(widget.fields[3], widget.currentRow[cdfilial]);
                if(widget.currentRow.IDENTRSAIDOP && widget.currentRow[cdfilial]) widget.getField('CDSERIE').readOnly = false;
                else widget.getField('CDSERIE').readOnly = true;
                almoxarifado.dataSourceFilter[0].value = "@General.getFieldValue('"+cdfilial+"','DOCUMENTOSGERADOS')@";
                almoxarifado.outData.CDALMOXARIFE = (field.value() == 'S') ? ["CDALMOXARIFE", "CDALMOORIGRD"] : ["CDALMOXARIFE", "CDALMODESTRD"];
                localizacao.dataSourceFilter[0].value = "@General.getFieldValue('"+cdfilial+"','DOCUMENTOSGERADOS')@";
            }
            this.readOnlyAlmoxDocumento(widget);
            operacao.readOnly = false;
        }
        operacao.reload();
        almoxarifado.setValue('');
        widget.currentRow.CDALMOXARIFE = '';
        operacao.setValue('');
        widget.currentRow.CDTIPOOPER   = '';
        localizacao.setValue('');
        widget.currentRow.CDLOCALESTOQ = '';
    };
    
    /**
     * Altera o readOnly e do campo de almoxarifado('DSALMOXARIFE')
     * de acordo com a filial e o tipo de operação('E' ou 'S').
     * 
     * @param {object} widget  widget documentos gerados.
    */
    
    this.readOnlyAlmoxDocumento = function(widget) {
        var almoxarifado = widget.getField('DSALMOXARIFE');
        if(widget.currentRow.IDENTRSAIDOP) {
            var cdfilial = (widget.currentRow.IDENTRSAIDOP == 'S') ? 'CDFILIORIGRD' : 'CDFILIDESTRD';
            almoxarifado.dataSourceFilter[0].value = "@General.getFieldValue('"+cdfilial+"','DOCUMENTOSGERADOS')@";
            if(widget.currentRow[cdfilial]) {
                var filter = [
                    {
                        "name":"CDFILIAL",
                        "operator":"=",
                        "value": widget.currentRow[cdfilial]
                    }
                ];
                General.callRestEngine('df#/crud/paramfilial', 'FilterData', filter).then(function(success){
                    if(success.dataset.paramfilial[0].IDUTILALMOX === 'S') {
                        almoxarifado.readonly = false;
                        almoxarifado.reload();
                        this.readOnlyLocalDocumento(widget);
                    } 
                }.bind(this));
            }
            else almoxarifado.readOnly = true;
        }  
        else almoxarifado.readOnly = true;
    };
    
    /**
     * Altera o readOnly e do campo de localização('DSLOCALESTOQ') 
     * de acordo com a filial e o tipo de operação('E' ou 'S').
     * 
     * @param {object} widget  widget documentos gerados.
    */
    this.readOnlyLocalDocumento = function(widget) {
        var local = widget.getField('DSLOCALESTOQ');
        if(widget.currentRow.IDENTRSAIDOP) {
            var cdfilial = (widget.currentRow.IDENTRSAIDOP == 'S') ? 'CDFILIORIGRD' : 'CDFILIDESTRD';
            if(widget.currentRow[cdfilial] && widget.currentRow.CDALMOXARIFE) {
                local.dataSourceFilter[0].value = "@General.getFieldValue('"+cdfilial+"','DOCUMENTOSGERADOS')@";
                var filter = [
                    {
                        "name":"CDFILIAL",
                        "operator":"=",
                        "value": widget.currentRow[cdfilial]
                    }
                ];
                General.callRestEngine('df#/crud/paramfilial', 'FilterData', filter).then(function(success){
                    if(success.dataset.paramfilial[0].IDUTILLCESTQ === 'S') {
                        local.reload();
                        local.readonly = false;
                    } 
                });
            }
            else local.readOnly = true;
        }  
        else local.readOnly = true;
    };
    
    /**
     * Altera o filtro do field de serie com os valores 
     * referentes a filial. - Somente para IDENTRSAIDOP = 'S' -
     * 
     * @param {object} fieldSerie field 'CDSERIE' da
     * widget documentos gerados.
     * @param {string} filial código da filial
    */
    this.setFilterSerieSaida = function(fieldSerie, filial) {
        var filter = [
            {
                "name":"CDFILIAL",
                "operator":"=",
                "value": filial
            }
        ];
        General.callRestEngine('df#/crud/filial', 'FilterData', filter).then(function(success){
            var rowFilial = success.dataset.filial;
            fieldSerie.dataSourceFilter[0].value =  rowFilial[0].CDEMPRESA;
            fieldSerie.dataSourceFilter[1].value =  rowFilial[0].CDINSCESTA;
            fieldSerie.dataSourceFilter[2].value =  rowFilial[0].CDINSCMUNI;
            fieldSerie.reload();
        });
    };
    
    /**
     * Altera o filed de 'CDSERIE' de acordo com 
     * IDENTRSAIDOP.
     * 
     * @param {object} fieldSerie field 'CDSERIE' da
     * widget documentos gerados.
     * @param {string} IDENTRSAIDOP 'E' para entrada
     * 'S' para saída
    */
    this.changeFieldSerie = function(field, IDENTRSAIDOP) {
        if(IDENTRSAIDOP === 'E') {
            field.valueField = "CDSERIEENTR";
            field.descriptionField = "CDSERIEENTR";
            field.outData = {"CDSERIEENTR": "CDSERIE"};
            field.dataSource.name = "df#/crud/serieentr";
            field.order = "CDSERIEENTR";
            field.dataSourceFilter = [];
            field.fields[0].isVisible = true;
            field.fields[1].isVisible = true;
            field.fields[2].isVisible = false;
            field.fields[3].isVisible = false;
            if(field.selectWidget)  {
                field.selectWidget.fields[0].isVisible = true;
                field.selectWidget.fields[1].isVisible = true;
                field.selectWidget.fields[2].isVisible = false;
                field.selectWidget.fields[3].isVisible = false;
                field.selectWidget.redraw();
            }
        }
        else if(IDENTRSAIDOP === 'S') {
            field.valueField = "CDSERIE";
            field.descriptionField = "CDSERIE";
            field.outData = {"CDSERIE": "CDSERIE"};
            field.dataSource.name = "df#/crud/serienf";
            field.order = "CDSERIE";
            field.fields[0].isVisible = false;
            field.fields[1].isVisible = false;
            field.fields[2].isVisible = true;
            field.fields[3].isVisible = true;
            if(field.selectWidget)  {
                field.selectWidget.outData = {"CDSERIE": "CDSERIE"};
                field.selectWidget.fields[0].isVisible = false;
                field.selectWidget.fields[1].isVisible = false;
                field.selectWidget.fields[2].isVisible = true;
                field.selectWidget.fields[3].isVisible = true;
                field.selectWidget.redraw();
            }
            field.dataSourceFilter = [
                {
                    "name": "CDEMPRESA",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "CDINSCESTA",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "CDINSCMUNI",
                    "operator": "=",
                    "value": ""
                }
            ];
        }
        field.reload();
    };
    
    //Função para consultar posição de estoque do produto escolhido para setar validação de quantidade maximo para o lançamento
    //OBS: verificar possibilidade de refatorar a busca da datadeprocessamento e busca-la de outro local para evitar requisição ao back.
    this.atualizaMaxQuantidade = function(widget) {
        var mensagem;
        var filial = widget.currentRow.CDFILIMOVI;
        var result = 0; 
        if(widget.currentRow.QTESTOQDIA === undefined){ // QTESTOQDIA será undefined caso unidade não controle almoxarifado ou local de estoque.

            widget.currentRow.IDQUANTID = 'N';
            if(!widget.currentRow.NRLOTEESTQ) widget.currentRow.NRLOTEESTQ = ' ';
                
            if(!widget.currentRow.NRSUBLOTE) widget.currentRow.NRSUBLOTE = ' ';
                
            if(!widget.currentRow.DTLANCMOVI) widget.currentRow.DTLANCMOVI = widget.getParent().currentRow.DTENTRSAID;
                
            if(!widget.currentRow.CDALMOXARIFE) widget.currentRow.CDALMOXARIFE = ' ';
                
            if(!widget.currentRow.CDLOCALESTOQ) widget.currentRow.CDLOCALESTOQ = ' ';

            
            ValidacoesEstoque_Service.verificaEstoqueProduto(widget.currentRow).then(function(success){
                if(success.dataset.lote_infCompl_semAlmox.length > 0) {
                    result = success.dataset.lote_infCompl_semAlmox[0].QTESTOQDIA; 
                }
                mensagem = "Posição Estoque: ".concat(result);
                widget.getField('QTITEMPROD').validations.max = {"value": result, "message": mensagem};
                General.callRestEngine("df#/crud/paramfilial", "FilterData", [{"name": "CDFILIAL", "operator": "=", "value": widget.currentRow.CDFILIAL}]).then((retorno) => {
                    if(widget.currentRow.IDCNTRESTOQ == 'N' || widget.getParent().currentRow.IDATUESTTPOP == 'N')
                        delete(widget.getField('QTITEMPROD').validations.max);  
                    else if(retorno.dataset.paramfilial[0].IDPERMESTNEG == 'S') 
                        delete(widget.getField('QTITEMPROD').validations.max);  
                    else if(widget.currentRow.CDSERVFILI)
                        delete(widget.getField('QTITEMPROD').validations.max);  
                });
            });    
            
        }else{
            mensagem = "Posição Estoque: ".concat(widget.currentRow.QTESTOQDIA);
            widget.getField('QTITEMPROD').validations.max = {"value": widget.currentRow.QTESTOQDIA, "message": mensagem};
            if(widget.currentRow.IDCNTRESTOQ == 'N' || widget.getParent().currentRow.IDATUESTTPOP == 'N')
                delete(widget.getField('QTITEMPROD').validations.max);
        }    
        
    };
    
    this.openPopupCopiarNF = function() {
        var wdgCopia = templateManager.container.getWidget('WidgetCopiarNF');
        var currentRowCabecalho = templateManager.container.getWidget('cabecalho').currentRow;
        if(currentRowCabecalho.IDSTATUS != 'D'){
            wdgCopia.currentRow = Util.clone(currentRowCabecalho);
            wdgCopia.getField("NMRAZSOCCLIE").dataSourceFilter[0].value = wdgCopia.currentRow.CDFILIEMIT;
            wdgCopia.currentRow.DTLANCAMENNF = moment().format('DD/MM/YYYY');
            wdgCopia.currentRow.LOTEORIG  = 'S';
            
            ScreenService.openPopup(wdgCopia);
            alteraVisibilidadeCampoInscricao(wdgCopia);
        }else{
            ScreenService.showMessage('Não é possível copiar notas em digitação');
        }
    };
    
    this.closePopupCopiaNF = function() {
        var wdgCopia = templateManager.container.getWidget('WidgetCopiarNF');
        ScreenService.closePopup(wdgCopia);
        wdgCopia.isVisible = false;
    };
    
    this.copiarNF = function(widget){
        if (widget.isValid()){
            let row = widget.currentRow;
            Dfs11000_LancamentoSaida_Service.copiarNF(row).then(function(success){
                this.closePopupCopiaNF();
                ScreenService.closePopup();
                ScreenService.closeSwipe();
                templateManager.container.getWidget('cabecalho').reload();
                ScreenService.showMessage(success.dataset.copiarNF.success);
                ScreenService.closePopup();
            }.bind(this)
            ).catch(function(error) {
                 ScreenService.closePopup();
                    var errosCopiar = templateManager.container.getWidget('errosCopiar');
                    if(error.data.dataset){
                        errosCopiar.dataSource.data = error.data.dataset.errosCopiarNF;
                        errosCopiar.isVisible = true;
                        ScreenService.openPopup(errosCopiar);
                    }
                    
                    ScreenService.showMessage(error.data.error);
            });
        }
    };
    
    this.popUpOperacaoOnChange = function(widget){
        alteraVisibilidadeCampoInscricao(widget);
        limpaCamposOutDataSelectsTpOperac(widget);
    };
    
    this.beginDataUpdate = function(widCabecalho) {
        if(!widCabecalho.dataSource.data.length) ScreenService.showMessage("É necessário filtrar dados para atualizar.");
        else {
            let widDataUpdate = getDataUpdateWidget();
            let tabDataUpdate = templateManager.container.getWidget('tabDataUpdate');
            let newDtSaida = tabDataUpdate.getField('NEWDTENTRSAID');
            
            this.openPopupDataUpdate(tabDataUpdate);
            if(newDtSaida.value()) newDtSaida.setValue();
            
            widDataUpdate.dataSourceFilter = widCabecalho.dataSourceFilter;
            widDataUpdate.reload();
        }
    };
    
    this.abrirImportacaoDeSolicitacaoDeCompra = function(args) {
        var widget = args.owner.widget;
        var widgetPopup = templateManager.container.getWidget('importacaoSolicitacaoCompra');
        widgetPopup.isVisible = true;
        ScreenService.openPopup(widgetPopup);
        widgetPopup.activate();
        
        MessageService.showMessageByCode('FUNCIONALIDADE_EM_DESENVOLVIMENTO');
        
        this.setValueFieldImportacaoDeSolicitacaoDeCompra(widget);
    };
    
    this.setValueFieldImportacaoDeSolicitacaoDeCompra = function(widget){
        const widgetPopup = templateManager.container.getWidget('importacaoSolicitacaoCompra');
        Df11000_Lancamento_Service.getDefaultNovaNotaSaida().then((success) => {
           
            //Insere valor no campo nmfilial
            widgetPopup.getField('NMFILIAL').reload();
            widgetPopup.getField("NMFILIAL").value(success[0].CDFILIEMIT);
            
            //Insere valor no campo nmempresa de acordo com o nmfilial
            widgetPopup.getField('NMEMPRESA').reload();
            widgetPopup.getField("NMEMPRESA").value(success[0].CDEMPRESA);
            
            
            //Insere valor no campo nmempresadestino, que precisa ser igual ao nmempresa
            widgetPopup.getField('NMEMPRESADEST').reload();
            widgetPopup.getField("NMEMPRESADEST").value(success[0].CDEMPRESA);
            
        });
        
        
        
        // É atribuido aos campos 'nmfilial' e 'nmempresa', os valores relativos ao filtrado.
        
        
        
    };
    
    
    this.closeImportacao = function(args){
        // Fecha a widget de importação
        ScreenService.closePopup('importacaoSolicitacaoCompra');
    };
    this.saveImportacao = function(args){
        // Salva a widget de importação
        MessageService.showMessageByCode('FUNCIONALIDADE_EM_DESENVOLVIMENTO');
        
    };
    this.setDefaultNovaNota = function (args) {
        var row = {CDFILIAL: args.e.preference.FILTER_ROW.CDFILIEMIT, IDENTRSAIDOP: 'S'};
        General.callRestEngine("df#/getDefaultLancamentoNota", "Row", row).then((success) => {
            
            Df11000_Lancamento_Service.setDefaultNovaNotaSaida(success.dataset.defaultNovaNota);
        });
    };
    
    this.testdestinoOrigem = function() {
        const widget = templateManager.container.getWidget('importacaoSolicitacaoCompra');
        const fieldUndDest = widget.currentRow.NMFILIDEST;
        const fieldUndOrg = widget.currentRow.NMEMPRESA;
        
        if(fieldUndDest == fieldUndOrg) {
            widget.currentRow.NMFILIDEST = "";
            MessageService.showMessageByCode('CORRELACAO_ENTRE_UND_ENT_SAI');
            
        }
        
        
    };
    this.testDtEntregaSaida = function() {
        const widget = templateManager.container.getWidget('importacaoSolicitacaoCompra');
        const fieldDtEntrega = widget.currentRow.DTENTREGA;
        const fieldDtSaida = widget.currentRow.DTENTRSAID;
        
        if(fieldDtSaida == "") {
            widget.currentRow.DTENTRSAID = fieldDtEntrega;
        }
        
    };
    
    
    this.openPopupDataUpdate = function(widTabDataUpdate) {
        widTabDataUpdate.isVisible = true;
        ScreenService.openPopup(widTabDataUpdate);
        widTabDataUpdate.activate();
    };
    
    this.saveDataUpdate = function(tabDataUpdate, widDataUpdate) {
        if(tabDataUpdate.isValid()) {
            if(!widDataUpdate.hasCheckedRows()) ScreenService.showMessage("Selecione uma nota para atualizar.");
            else {
                let self = this;
                ScreenService.confirmMessage('Confirma a atualização e o fechamento da(s) nota(s) ?', 'question', function (success) {
                    let rowsUpdate = widDataUpdate.getCheckedRows();
                    let NEWDTENTRSAID = tabDataUpdate.getField('NEWDTENTRSAID').value();
                    Dfs11000_LancamentoSaida_Service.saveDataUpdate(rowsUpdate, NEWDTENTRSAID).then (function (success) {
                        widDataUpdate.reload().then(() => {
                            if(widDataUpdate.dataSource.data.length) widDataUpdate.unSelectAll();
                            else self.closePopupDataUpdate(tabDataUpdate);
                        });
                        ScreenService.showMessage(success.messages[0].message);
                    }, requestError);
                });
            }
        }
    };
    
    this.closePopupDataUpdate = function(widTabDataUpdate) {
        let widDataUpdate = getDataUpdateWidget();
        
        widDataUpdate.unSelectAll();
        widDataUpdate.dataSource.data = [];
        
        widTabDataUpdate.isVisible = false;
        ScreenService.closePopup(widTabDataUpdate);
        widTabDataUpdate.getParent().activate();
        widTabDataUpdate.getParent().reload();
    };
    
    let getDataUpdateWidget = function() {
        return templateManager.container.getWidget('wdg_DataUpdate');
    };
    
    this.getFieldSerieFromFilterPopup = function(fieldSerie) {
        fieldSerie.dataSourceFilter.forEach(data => {
            if(data.name == "CDEMPRESA") data.value = this.getFieldFromFilterPopup('CDEMPRESA');
            if(data.name == "CDINSCESTA") data.value = this.getFieldFromFilterPopup('IEREM');
            if(data.name == "CDINSCMUNI") data.value = this.getFieldFromFilterPopup('CDINSCMUNI');
        });
    };
    
    this.nmPaisNotaOnChange = function(widget) {
        widget.getField('SGESTADONOTA').readOnly = widget.currentRow.CDPAISNOTA ? false : true;
        widget.getField('NMMUNICIPIONOTA').readOnly = true;
        widget.currentRow.CDMUNICIPIO = null;
        widget.currentRow.CDMUNICIPIONOTA = null;
        widget.currentRow.NMMUNICIPIONOTA = null;
        widget.currentRow.SGESTADO = null;
        widget.currentRow.SGESTADONOTA = null;
        widget.currentRow.NMESTADONOTA = null;
        if (widget.currentRow.CDPAISNOTA) {
    		widget.getField('SGESTADONOTA').dataSourceFilter.forEach(data => {
    		    if(data.name == "CDPAIS") data.value = widget.currentRow.CDPAISNOTA;
    		});
    		widget.getField('NMMUNICIPIONOTA').dataSourceFilter.forEach(data => {
    		    if(data.name == "CDPAIS") data.value = widget.currentRow.CDPAISNOTA;
    		});
        }
    };
    
    this.sgEstadoNotaOnChange = function(widget) {
        widget.getField('NMMUNICIPIONOTA').readOnly = widget.currentRow.SGESTADONOTA ? false : true;
        widget.currentRow.CDMUNICIPIO = null;
        widget.currentRow.CDMUNICIPIONOTA = null;
        widget.currentRow.NMMUNICIPIONOTA = null;
		widget.currentRow.SGESTADO = widget.currentRow.SGESTADONOTA;
		if (widget.currentRow.SGESTADONOTA) {
    		widget.getField('NMMUNICIPIONOTA').dataSourceFilter.forEach(data => {
    		    if(data.name == "SGESTADO") data.value = widget.currentRow.SGESTADONOTA;
    		});
		}
    };
    
    this.nmMunicipioNotaOnChange = function(widget) {
        widget.currentRow.CDMUNICIPIO = widget.currentRow.CDMUNICIPIONOTA;
   };
    
    this.controlVisibilityLocalPrestServ = function(widget) {
        let fieldSGESTADONOTA = widget.getField('SGESTADONOTA');
        let fieldNMMUNICIPIONOTA = widget.getField('NMMUNICIPIONOTA');
        let fieldNMPAISNOTA = widget.getField('NMPAISNOTA');
        let fieldIDNATUOPER = widget.getField('IDNATUOPER');
        let utilizaNFSe = widget.currentRow.IDUTILNFSE === 'S' ? true : false;
        
        if(widget.editing) { //True = Add e edit nota.
            if(utilizaNFSe) {
                if(widget.currentRow.__is_new) {
                    widget.currentRow.IDNATUOPER = null;
                    widget.currentRow.CDPAISNOTA = null;
                    widget.currentRow.NMPAISNOTA = null;
                    widget.currentRow.CDMUNICIPIONOTA = null;
                    widget.currentRow.NMMUNICIPIONOTA = null;
                    widget.currentRow.SGESTADONOTA = null;
                    widget.currentRow.NMESTADONOTA = null;
                } else {
                    if (widget.currentRow.CDPAISNOTA) {
                        widget.getField('SGESTADONOTA').dataSourceFilter.forEach(data => {
                		    if(data.name == "CDPAIS") data.value = widget.currentRow.CDPAISNOTA;
                		});
                		widget.getField('NMMUNICIPIONOTA').dataSourceFilter.forEach(data => {
                		    if(data.name == "CDPAIS") data.value = widget.currentRow.CDPAISNOTA;
                		});
                    }
                    if (widget.currentRow.SGESTADONOTA) {
                        widget.getField('NMMUNICIPIONOTA').dataSourceFilter.forEach(data => {
                		    if(data.name == "SGESTADO") data.value = widget.currentRow.SGESTADONOTA;
                		});
                    }
                }
                widget.fieldGroups.forEach(group => {
                    if(group.name === 'prestacaoServico') {
                        group.isVisible = true;
                        group.opened = true;
                    }
                });
                widget.currentRow.SGESTADO = widget.currentRow.SGESTADONOTA;
                widget.currentRow.CDMUNICIPIO = widget.currentRow.CDMUNICIPIONOTA;
                
                fieldIDNATUOPER.isVisible = true;
                fieldIDNATUOPER.readOnly = false;
                fieldIDNATUOPER.validations.required = true;
                
                fieldNMPAISNOTA.isVisible = true;
                fieldNMPAISNOTA.readOnly = false;
                fieldNMPAISNOTA.validations.required = true;
                
                fieldSGESTADONOTA.isVisible = true;
                fieldSGESTADONOTA.readOnly = widget.currentRow.CDPAISNOTA ? false : true;
                fieldSGESTADONOTA.validations.required = true;
                
                fieldNMMUNICIPIONOTA.isVisible = true;
                fieldNMMUNICIPIONOTA.readOnly = widget.currentRow.SGESTADONOTA ? false : true;
                fieldNMMUNICIPIONOTA.validations.required = true;
            } else {
                widget.fieldGroups.forEach(group => {
                    if(group.name === 'prestacaoServico') {
                        group.isVisible = false;
                        group.opened = false;
                    }
                });
                fieldIDNATUOPER.isVisible = false;
                fieldIDNATUOPER.readOnly = true;
                fieldIDNATUOPER.validations.required = false;
                
                fieldNMPAISNOTA.isVisible = false;
                fieldNMPAISNOTA.readOnly = true;
                fieldNMPAISNOTA.validations.required = false;
                
                fieldSGESTADONOTA.isVisible = false;
                fieldSGESTADONOTA.readOnly = true;
                fieldSGESTADONOTA.validations.required = false;
                
                fieldNMMUNICIPIONOTA.isVisible = false;
                fieldNMMUNICIPIONOTA.readOnly = true;
                fieldNMMUNICIPIONOTA.validations.required = false;
            }
        } else { //False = Visualizar nota.
            if(utilizaNFSe) {
                widget.fieldGroups.forEach(group => {
                    if(group.name === 'prestacaoServico') {
                        group.isVisible = true;
                    }
                });
                fieldIDNATUOPER.isVisible = true;
                fieldNMPAISNOTA.isVisible = true;
                fieldSGESTADONOTA.isVisible = true;
                fieldNMMUNICIPIONOTA.isVisible = true;
            } else {
                widget.fieldGroups.forEach(group => {
                    if(group.name === 'prestacaoServico') {
                        group.isVisible = false;
                    }
                });
                fieldIDNATUOPER.isVisible = false;
                fieldNMPAISNOTA.isVisible = false;
                fieldSGESTADONOTA.isVisible = false;
                fieldNMMUNICIPIONOTA.isVisible = false;
            }
        }
    };
   
    this.reloadNumeroNotaFiscal = function() {
        let currentRow = getCabecalhoWidget().currentRow;
        let data = getCabecalhoWidget().dataSource.data;
        data.forEach(function(row) {
            if(row.NRLANCTONF == currentRow.NRLANCTONF) {
                if(getCabecalhoWidget().inViewMode()) {
                    currentRow.NRNOTAFISC = row.NRNOTAFISC;
                    getCabecalhoWidget().reload();
                }
            }
        });
    };
    
    var validaNumeroNF = function(cabecalhoWidget) {
        var NRNOTAFISC = cabecalhoWidget.getField("NRNOTAFISC");
        var currentRow = cabecalhoWidget.currentRow;
        
        if(currentRow.IDPERNUMENF == "S")
            NRNOTAFISC.readOnly = false;
        else 
            NRNOTAFISC.readOnly = true;    
    };
    
    var applyZeroFillNumeroNota = function(cabecalhoWidget) {
        var filter =  [{"name": "CDFILIAL", "operator": "=", "value": cabecalhoWidget.currentRow.CDFILIEMIT}],
            nrNotaFiscField = cabecalhoWidget.getField('NRNOTAFISC'),
            nrTamNumNfUt,
            quantCaracteres;
        
        General.callRestEngine("df#/crud/paramfilial", "FilterData", filter).then((success) => {
            if(success.dataset.paramfilial[0]) 
                nrTamNumNfUt = success.dataset.paramfilial[0].NRTAMNUMNFUT;
            
            quantCaracteres = nrTamNumNfUt ? nrTamNumNfUt : 9;
            nrNotaFiscField.mask.params.maxlength = quantCaracteres;
            nrNotaFiscField.validations.maxLength.length = nrTamNumNfUt ? nrTamNumNfUt : 20;
            nrNotaFiscField.reload();
        });
    };
    
    var alteraVisibilidadeCampoNaturezaOper = function(cabecalhoWidget, tipoOperacaoOnChange) {
        var IDNFAJUSTE   = cabecalhoWidget.currentRow.IDNFAJUSTE,
            CDTIPOOPER   = cabecalhoWidget.currentRow.CDTIPOOPER,
            DSCDFISOPENF = cabecalhoWidget.getField("DSCDFISOPENF"),
            IDNATUOPER   = cabecalhoWidget.getField("IDNATUOPER"),
            utilizaNFSe  = cabecalhoWidget.currentRow.IDUTILNFSE === 'S' ? true : false; 
          
        if(!utilizaNFSe) {    
            if(IDNFAJUSTE == "S" || CDTIPOOPER == "03") {
                DSCDFISOPENF.readOnly = false;
                DSCDFISOPENF.isVisible = true;
                DSCDFISOPENF.validations.required = true;
                if(cabecalhoWidget.currentRow.__is_new) {
                    DSCDFISOPENF.clearValue();
                }
            } else {
            	if (tipoOperacaoOnChange) DSCDFISOPENF.clearValue();
                DSCDFISOPENF.readOnly = true;
                DSCDFISOPENF.isVisible = true;
                delete DSCDFISOPENF.validations.required;
            }
        } else {
            DSCDFISOPENF.isVisible = false;
            delete DSCDFISOPENF.validations.required;
        }
    };
    
    var changeDataSourceNotaOrigem = function(cabecalhoWidget) {
        var notaOrigemField = cabecalhoWidget.getField('DSNOTAORIGEM'),
            tipooperac  = cabecalhoWidget.currentRow.CDTIPOOPER,
            idNfAjuste  = cabecalhoWidget.currentRow.IDNFAJUSTE,
            filter      = [ {name: "P_FILIAL", operator: "=", value: cabecalhoWidget.currentRow.CDFILIEMIT},
                            {name: "P_CDFILIORIGEM", operator: "=", value: "T"},
                            {name: "P_IDTPIJURNF", operator: "=", value: cabecalhoWidget.currentRow.IDTPIJURNF},
                            {name: "P_NRINSJURNF", operator: "=", value:  cabecalhoWidget.currentRow.NRINSJURNF},
                            {name: "P_NOTAFISC", operator: "=", value: "T"},
                            {name: "P_CDTIPOOPER", operator: "=", value: ""},
                            {name: "P_SERIE", operator: "=", value: "T"}],
            dataSourceName = (idNfAjuste == 'S' || tipooperac == '03') ? 'df#/crud/notafiscal_origem_saida' : 'df#/crud/notafiscal_origem_entrada';

        if(dataSourceName) {
            General.callRestEngine(dataSourceName, 'FilterData', filter).then(function(sucess) {
                notaOrigemField.dataSource.name = dataSourceName;
                notaOrigemField.dataSource.data = sucess.dataset.dataSourceName;
                notaOrigemField.reload();
            });
        }
    };
    
    this.validacoesFieldGroupIntermediador = function(cabecalhoWidget) {
        var intermedGroup = cabecalhoWidget.fieldGroups.filter(group => group.name === 'intermediador')[0];
        var intermediador = cabecalhoWidget.currentRow.IDINTERMED;
        var cnpjIntermed = cabecalhoWidget.getField('NRCNPJINTERMED');
        var nmCadIntermed = cabecalhoWidget.getField('NMCADINTERMED');

        if(intermediador == '1') {
           cnpjIntermed.validations  = { "required":  true};
           nmCadIntermed.validations = { "required":  true,
                                         "maxLength": {"length": 60},
                                         "minLength": {"length": 2}};

            nmCadIntermed.readOnly = false;
            cnpjIntermed.readOnly = false;

            intermedGroup.opened = true;
        }
        else {
            nmCadIntermed.readOnly = true;
            cnpjIntermed.readOnly = true;

            nmCadIntermed.clearValue();
            cnpjIntermed.clearValue();

            intermedGroup.opened = false;

            delete nmCadIntermed.validations;
            delete cnpjIntermed.validations;
        }
        nmCadIntermed.reload();
        cnpjIntermed.reload();
    };
    
    var defaultValueIndicadorPresenca = function(cabecalhoWidget) {
        if(cabecalhoWidget.currentRow.IDNFAJUSTE == 'S' || cabecalhoWidget.currentRow.CDTIPOOPER == '03')
            cabecalhoWidget.getField('IDPRESENCIAL').setValue('0');
        else
            cabecalhoWidget.getField('IDPRESENCIAL').setValue('9');
        
        cabecalhoWidget.getField('IDPRESENCIAL').reload();
    };
    
    this.actSalvaImpostosItemWidgetOnSaveInlineEdit = function(impostosItemArgs) {
        var impostosItemWidget =  templateManager.container.getWidget("impostosItem");
        const data = impostosItemWidget.dataSource.data;
        var statusNota = getCabecalhoWidget().currentRow.IDSTATUS;
        const camposAlterados = data.filter(row =>  !!('changed_CDIMPOSTO' in row)  ||    
                                                    !!('changed_VRBASECALCIT' in row)  ||
                                                    !!('changed_VRPERREDBCIT' in row)  ||
                                                    !!('changed_VRPERMAJBCIT' in row)  ||
                                                    !!('changed_VRADICALIQFECP' in row)  ||
                                                    !!('changed_CDFISCALIT' in row)  ||
                                                    !!('changed_CDSITUTRIBITNFE' in row)  ||
                                                    !!('changed_IDMODBCITNFE' in row)  ||
                                                    !!('changed_IDMODBCSTITNFE' in row)  ||
                                                    !!('changed_CDCEST' in row)  ||
                                                    !!('changed_CDBENEFFISCAL' in row)  ||
                                                    !!('changed_VRIMPOSTODESON' in row)  ||
                                                    !!('changed_IDMOTDESONERAIT' in row)  ||
                                                    !!('changed_CDENQLEGALIPI' in row) ||
                                                    !!('changed_VRIMPOSTOIT' in row) ||
                                                    !!('changed_VRALIQITEM' in row) ||
                                                    !!('changed_SGIMPOSTO' in row) ||
                                                    !!('changed_IDCONSDESONVRNF' in row));
         
        if(validaCamposImpostoItemEditInline()) {       
            const salvaImpostosItem = () =>
                Df11000_Lancamento_Service.salvaImpostosItem(camposAlterados)
                .then(() => {
                    removeEdicaoLinhasWidget(camposAlterados);
                    impostosItemWidget.list();
                })
                .catch(requestError);
            
            if (statusNota !== 'D') {
                alteraStatusNotafiscal(impostosItemWidget);
                impostosItemWidget.getParent().reload();
            }
            
            salvaImpostosItem();
        }


    };
    
    var removeEdicaoLinhasWidget = function(registrosAlterados) {
        registrosAlterados.forEach(registro => {
            Object.keys(registro).filter(key => !key.indexOf('changed') ? delete registro[key] : !key.indexOf('changed'));
        });
    };
    
    this.actSalvarImposto = function() {
        var impostosItemWidget = getImpostosItensWidget();
        var currentRow = impostosItemWidget.currentRow;
        var statusNota = getCabecalhoWidget().currentRow.IDSTATUS;
        var impostos = [{
            "CDBENEFFISCAL": currentRow.CDBENEFFISCAL,
            "CDCEST": currentRow.CDCEST,
            "CDENQLEGALIPI": currentRow.CDENQLEGALIPI,
            "CDFILIAL":  getCabecalhoWidget().currentRow.CDFILIAL,
            "CDFISCALIT": currentRow.CDFISCALIT ? currentRow.CDFISCALIT : 1,
            "CDFISCOPERIT": "",
            "CDIMPOSTO": currentRow.CDIMPOSTO,
            "CDREGRAIT": "",
            "CDSITUTRIBIT": getItensWidget().currentRow.CDSITUTRIBPR,
            "CDSITUTRIBITNFE": currentRow.CDSITUTRIBITNFE,
            "IDCONSDESONVRNF":  currentRow.IDCONSDESONVRNF?  currentRow.IDCONSDESONVRNF : "N",
            "IDMODBCITNFE":  currentRow.IDMODBCITNFE,
            "IDMODBCSTITNFE": currentRow.IDMODBCSTITNFE,
            "IDMOTDESONERAIT": currentRow.IDMOTDESONERAIT ? currentRow.IDMOTDESONERAIT : "9",
            "NRITPROIMPOS": '1',
            "NRLANCTONF": getCabecalhoWidget().currentRow.NRLANCTONF,
            "NRSEQITPROD": getItensWidget().currentRow.NRSEQITPROD,
            "VRADICALIQFECP": currentRow.VRADICALIQFECP ? currentRow.VRADICALIQFECP : 0,
            "VRADICFECP": currentRow.VRADICFECP ? currentRow.VRADICFECP : 0,
            "VRALIQITEM": currentRow.VRALIQITEM ? currentRow.VRALIQITEM : 0,
            "VRBASECALCIT": currentRow.VRBASECALCIT ? currentRow.VRBASECALCIT : 0,
            "VRCONTITIMP": currentRow.VRCONTITIMP ? currentRow.VRCONTITIMP : 0,
            "VRIMPOSTODESON": currentRow.VRIMPOSTODESON ? currentRow.VRIMPOSTODESON : 0,
            "VRIMPOSTOIT": currentRow.VRIMPOSTOIT ? currentRow.VRIMPOSTOIT : 0,
            "VRPARDEDIT": currentRow.VRPARDEDIT ? currentRow.VRPARDEDIT : 0,
            "VRPERMAJBCIT": currentRow.VRPERMAJBCIT ? currentRow.VRPERMAJBCIT : 0,
            "VRPERREDBCIT": currentRow.VRPERREDBCIT ? currentRow.VRPERREDBCIT : 0
        }];
        
        if(validaPreenchimentoObrigatorioImpostos() && !(validaImpostoItemCadastrado(impostos[0]))) {
            const salvaImpostosItem = () =>
            General.callRestEngine("df#/salvarImpostoItem", "Row", impostos)
            .then(() => {
                this.reloadDataSourceImpostoItem();
                ScreenService.closeSwipe();
            })
            .catch(requestError);
            if (statusNota !== 'D') {
                alteraStatusNotafiscal(impostosItemWidget);
                impostosItemWidget.getParent().reload();
            }
            salvaImpostosItem();
        }
    };
    
    this.verificaDataSourceProduto = function(field) {
        var cdTipoOper  = getCabecalhoWidget().currentRow.CDTIPOOPER;
        var filter = [];
        
        if(cdTipoOper == '03') {
            field.dataSource.name = 'df#/itemproduto_notafiscal_origem';
            filter = [
                { "name": "P_NRLANCTONF", "operator": "=", "value": getCabecalhoWidget().currentRow.NRLANCTONF},
                { "name": "P_CDFILIAL",   "operator": "=", "value": getCabecalhoWidget().currentRow.CDFILIAL}, 
                { "name": "P_NRORG",      "operator": "=", "value": getCabecalhoWidget().currentRow.NRORG}
            ];
        } else {
            field.dataSource.name = 'df#/produtoLancamento';
            filter = [
                {"name": "CDFILIAL", "operator": "=", "value": getCabecalhoWidget().currentRow.CDFILIEMIT},
                {"name": "CDNVPRODUTO", "operator": "=", "value": this.nivelCompletoProduto()}
            ];
        }
        
        field.dataSourceFilter = filter;
        field.reload();
    };
    
    var alteraEdicaoCamposImpostos = function(impostosItemWidget) {
        var currentRow = getCabecalhoWidget().currentRow;
        var permiteEditarImposto = currentRow.CDTIPOOPER == '03' || currentRow.IDNFAJUSTE == 'S';
        
        impostosItemWidget.fields.forEach((field) => {
            if(permiteEditarImposto) {
                if(field.name != 'CDIMPOSTO' && field.name != 'SGIMPOSTO') {
                    if(field.name == 'IDMODBCITNFE' || field.name == 'IDCONSDESONVRNF' || field.name == 'IDMOTDESONERAIT' || field.name == 'IDMODBCSTITNFE' || field.name == 'CDFISCALIT')
                        field.inlineEditable = "select-native";
                    else if(field.name == 'IDMODBCITNFE')
                        field.inlineEditable = permiteEditarImposto;
                    else 
                        field.inlineEditable = "number";
                }
            } else { 
                if(field.inlineEditable)
                    delete field.inlineEditable;
            }
        });
        
    };
    
    var visibilidadeActionsWidgetImposto = function(impostosItemWidget) {
        var currentRow = getCabecalhoWidget().currentRow;
        var isVisible =  currentRow.CDTIPOOPER == '03' || currentRow.IDNFAJUSTE == 'S';
        impostosItemWidget.getAction('adicionarImposto').isVisible = isVisible;
        impostosItemWidget.getAction('excluirImposto').isVisible = isVisible;
    };
    
    var validaPreenchimentoObrigatorioImpostos = function() {
        var impostosItemWidget =  templateManager.container.getWidget("impostosItem");
        var fields = impostosItemWidget.fields;
        var camposInvalidos = [];
        
        fields.forEach(field => {
            if(field.name == 'VRALIQITEM' || field.name == 'SGIMPOSTO' || field.name == 'VRIMPOSTOIT' || field.name == 'CDFISCALIT'|| field.name == 'CDSITUTRIBITNFE') {
                if(!field.value())
                    camposInvalidos.push(field.label);
            }
        });
        
        if(camposInvalidos.length > 0) {
            MessageService.showMessageByCode('CAMPOS_OBRIGATORIOS', {"PARAMETRO": camposInvalidos});
            return false;
        } 
            
        return true; 
        
    
    };
    
    var validaObrigatoriedadeFields = function() {
        var currentRow = getCabecalhoWidget().currentRow;
        var cstnfe = getImpostosItensWidget().getField('CDSITUTRIBITNFE');
        var cdfiscalit = getImpostosItensWidget().getField('CDFISCALIT');

        if(currentRow.CDTIPOOPER == '03' || currentRow.IDNFAJUSTE == 'S') {
            setCampoObrigatorio(cstnfe);
            setCampoObrigatorio(cdfiscalit);
        } else {
            delete cstnfe.validations;
            cstnfe.readOnly = true;
            delete cdfiscalit.validations;
            cdfiscalit.readOnly = true;
        }
    };
    
    this.validacoesImpostosWidgetOnEnter = function(itemImpostosWidget) {
        validaObrigatoriedadeFields();
        alteraEdicaoCamposImpostos(itemImpostosWidget);
        visibilidadeActionsWidgetImposto(itemImpostosWidget);
        
        this.reloadDataSourceImpostoItem();
    };
    
    this.deleteItemProImpos = function(impostosItemArgs) {
        var impostosItemWidget = impostosItemArgs.owner.widget;
        var excluir = impostosItemWidget.currentRow;
        var statusNota = getCabecalhoWidget().currentRow.IDSTATUS;
         
        MessageService.confirmMessageByCode("REGISTRO_EXCLUIR").then(sucess => {
            General.callRestEngine("df#/deleteImpostoItem", "Row", excluir).then(function() {
                MessageService.showMessageByCode("REGISTRO_EXCLUIDO").then(function() {
                    ScreenService.closeSwipe();
                    if (statusNota !== 'D') {
                        alteraStatusNotafiscal(impostosItemWidget);
                        this.reloadDataSourceImpostoItem();
                        impostosItemWidget.getParent().reload();
                    }
                }).catch((error) => {
                    ScreenService.showMessage(error.data.error);
                });
            });

        });
    };
    
    this.validacoesItensOnEnter = function() {
        var currentRow = getCabecalhoWidget().currentRow;
        var cst = getItensWidget().getField('CDSITUTRIBPR');
        var cfop = getItensWidget().getField('CDFISOPEITEM');

        if(currentRow.CDTIPOOPER == '03' || currentRow.IDNFAJUSTE == 'S') {
            setCampoObrigatorio(cst);
            setCampoObrigatorio(cfop);
        } else {
            delete cst.validations;
            cst.readOnly = true;
            delete cfop.validations;
            cfop.readOnly = true;
        }

    };
    
    this.validaCNPJ = function (field) {
        if (!field.value()) {
            return;
        }
        if (!General.validateCNPJ(field.value())) {
            MessageService.showMessageByCode('CNPJ_INVALIDO');
            field.setValue("");
        }
    };
    
    var validaImpostoItemCadastrado = function(imposto) {
        var data = getImpostosItensWidget().dataSource.data.filter(data => data.CDIMPOSTO === imposto.CDIMPOSTO);
        if(data.length > 1)
            MessageService.showMessageByCode('IMPOSTO_JA_CADASTRADO');
            
        return data.length > 1;
    };
    
    var validaCamposImpostoItemEditInline = function() {
        var impostosItemWidget =  templateManager.container.getWidget("impostosItem");
        var rowsAlteradas = impostosItemWidget.dataSource.data.filter(row => row.changed);
        var camposInvalidos = [];
        var isAliquotaEmpty = false;
        var isValorEmpty = false;
        var isCstNfeEmpty = false;
        
        rowsAlteradas.forEach(row => {
            if(!row.VRALIQITEM) 
                isAliquotaEmpty = true;
            if(!row.VRIMPOSTOIT)  
                isValorEmpty = true;
            if(!row.CDSITUTRIBITNFE) 
                isCstNfeEmpty = true;
        });
        
        if(isAliquotaEmpty) 
            camposInvalidos.push('Alíquota');
        if(isValorEmpty)  
            camposInvalidos.push('Valor');
        if(isCstNfeEmpty) 
            camposInvalidos.push('CST NFe');
    
        if(camposInvalidos.length > 0) {
            MessageService.showMessageByCode('CAMPOS_OBRIGATORIOS', {"PARAMETRO": camposInvalidos});
            return false;
        } 
            
        return true; 
        
    };

    this.reloadDataSourceImpostoItem = function() {
        var filter = [{"name": "NRSEQITPROD", "operator": "=", "value": getItensWidget().currentRow.NRSEQITPROD},
                      {"name": "NRLANCTONF", "operator": "=", "value": getCabecalhoWidget().currentRow.NRLANCTONF},
                      {"name": "CDFILIAL", "operator": "=", "value": getCabecalhoWidget().currentRow.CDFILIAL}];
                      
        General.callRestEngine("df#/crud/itemproimpos_notafiscal_itemproduto_imposto", "FilterData", filter).then((success) => {
            getImpostosItensWidget().dataSource.data = success.dataset.itemproimpos_notafiscal_itemproduto_imposto;
            
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfs11000_LancamentoSaida_Controller", dfs11000_LancamentoSaida_Controller);
});


// FILE: ./assets/modules/saida/assets/js/controllers/Dfs49000_TransmissaoNfe_Controller.js
var dfs49000_TransmissaoNfe_Controller = function(General, Dfs49000_TransmissaoNfe_Service, templateManager, ScreenService, MessageService, ZhFilterPreferences, ZHPromise, localStorageEngine) {
    var requestError = function(error){
        var errorMessage = "Ocorreu um erro ao tentar realizar a operação.";
        if (error && error.data) {
            errorMessage = error.data.error;
        }
        ScreenService.showMessage(errorMessage, "ERROR");
    };

    var validaCampoEmpresa = function(transmissaoNfeArgs) {
        var empresa = transmissaoNfeArgs.owner.field.widget.getField("NMEMPRESA");
        empresa.readOnly = !transmissaoNfeArgs.row.NMFILIAL;
    };
    
    this.validaCNPJ = function (field) {
        if (!field.value()) {
            return;
        }
        if (!General.validateCNPJ(field.value())) {
            ScreenService.showMessage("CNPJ informado inválido.");
            field.setValue("");
        }
    };

    this.unidadeOnChange = function(transmissaoNfeArgs) {
        validaCampoEmpresa(transmissaoNfeArgs);
        this.empresaUtlizaNFE(transmissaoNfeArgs);
    };

    var getTransmissaoNFeWidget = function(){
        if(templateManager.container)
            return templateManager.container.getWidget("transmissaoNFe");
    };

    this.getCurrentField = function(field) {
        if(templateManager.container) {
            if (templateManager.container.getActiveWidget().name === "zhFilterPreferencesSelectConditions" ){
                return !templateManager.container.getActiveWidget().currentRow[field] ? null : templateManager.container.getActiveWidget().currentRow[field];
                
            }else{
                var transmissaoNFeWidget = getTransmissaoNFeWidget();
                if (transmissaoNFeWidget)
                return !transmissaoNFeWidget.currentRow[field] ? null : transmissaoNFeWidget.currentRow[field];
                
            }
        }
        
    };

    this.getWidgetProdutorRural = function(){
        return templateManager.container.getWidget('produtorRural');
    };

    var getFilterPopupCurrentRow = function(args){
        return args.owner.appliedFilter.FILTER_ROW;
    };

    this.getEmpresaFiltrada = function(){
        var transmissaoNFeWidget = getTransmissaoNFeWidget();
        if(transmissaoNFeWidget){
            return transmissaoNFeWidget.appliedFilter.FILTER_ROW.CDEMPRESA;
        }
    };
        
    this.validacoesWidgetOnEnterOnce = function(args) { 
        var widget = args.owner;
        setDefaultFilialFilter(widget).then(defaultFilter =>{
            ZhFilterPreferences.setOriginalFilter(widget.id, defaultFilter);
            ZhFilterPreferences.openFilterWidget(widget, defaultFilter).then (response => {
                var filterWidget = templateManager.container.getActiveWidget();
                filterWidget.actions.filter(action => action.name === "clear")[0].shortcutKey = "ctrl+backspace";
            });
        });
    };

    var setDefaultFilialFilter = function(widget) {
        var promise = ZHPromise.defer(); 
        var defaultFilter = Util.clone(widget.currentRow);
        defaultFilter.DTENTRSAID = undefined;

        Dfs49000_TransmissaoNfe_Service.buscaOperadorLogado().then(function(operador) {
            operador = operador[0];
            var filialSelecionada = operador.CDFILIALSELECIONADA;
            
            
            // Verifica se há parametro de CDFILIAL de PEDIDO para realizar busca sobre este CDFILIAL
            var filialPed = checkCDFilialOutModule();
            if(filialPed) filialSelecionada = filialPed;
            
            
            var filialField = widget.getField('NMFILIAL');
            var dtEntrField = widget.getField('DTENTRSAID');
            widget.currentRow.CDFILIAL = filialSelecionada;
            setCurrentRowWithOutDataFilial(filialSelecionada, filialField, defaultFilter, dtEntrField).then(callback =>{
                promise.resolve(defaultFilter);
            });
        });
        return promise;
    };
    
    // Função que capta o CDFILIAL Do módulo de Pedido
    var checkCDFilialOutModule = function() {
        
        return Dfs49000_TransmissaoNfe_Service.getLocalVarsFilterOutModule().CDFILIAL;
    };
    
    // Função que retorna Data do módulo de Pedido
    var checkDtEntrSaidOutModule = function() {
        
        return Dfs49000_TransmissaoNfe_Service.getLocalVarsFilterOutModule().DTENTRSAID;
        
    };
    
    // Função que apaga as variaveis de PEDIDO que ficam na cache
    var clearLocalVarsOutModule = function() {
        Dfs49000_TransmissaoNfe_Service.clearLocalVarsOutModule();
    };
    
    var setCurrentRowWithOutDataFilial = function(filialOut, filialField, currentRow, dtEntrField) {
        var promise = ZHPromise.defer();  
        General.callRestEngine('df#/filioper_lancamento_notafiscal', 'FilterData', [{"name": "CDOPERADOR", "operator": "=", "value": "OPERADOR_LOGADO"},
                                                                                    {"name": "CDFILIAL"  , "operator": "=", "value": filialOut}]
        ).then(function(filialLogada) {
            let filialDefaultValue;
            let filialOutData = filialField.outData;
            
            //Utiliza o primeiro dataset retornado. Necessário pois o dataset pode ser filioper_lancamento_notafiscal ou perfacessfili_lancamento_notafiscal
            for (let datasetName in filialLogada.dataset) {
                if (filialLogada.dataset.hasOwnProperty(datasetName)) {
                    filialDefaultValue = filialLogada.dataset[datasetName][0];
            	    break;
                }
            }
            Dfs49000_TransmissaoNfe_Service.buscaDataProcessamento(filialOut).then(([processamento]) => {
                const dtProcessFormat = moment(processamento.DTPROCESSA, 'DD/MM/YYYY').format('DD/MM/YYYY');
                currentRow.DTENTRSAID = {start: dtProcessFormat, period: undefined, end: dtProcessFormat};
               
                // Verifica se há parametro de Data Saida a partir do módulo de Pedidos
                var dtEntrSaid = checkDtEntrSaidOutModule();
                if(dtEntrSaid.start) currentRow.DTENTRSAID = dtEntrSaid;
               
                // Adiciona os outros parmetros advindo do Módulo de Pedidos
                setCurrentRowParamsOutModule(currentRow, Dfs49000_TransmissaoNfe_Service.getLocalVarsFilterOutModule() );
               
                // Limpa dados de Pedido na cache
                clearLocalVarsOutModule();
               
                promise.resolve();
            });
            currentRow.IDENTRSAIDOP = 'S';
            setCurrentRowWithOutData(currentRow, filialDefaultValue, filialOutData);
        });
        return promise;
    };

    // Seta os valores de PEDIDO para o filtro de DF, caso houver falta de algum parametro, os valores não são inseridos
    var setCurrentRowParamsOutModule = function(currentRow, valuesToSetOnCurrentRow) {
        
        var exec = true;
        
        for (var i in valuesToSetOnCurrentRow) {
            
            if(!valuesToSetOnCurrentRow[i]) {
                exec = false;
            }
            
        }
        
        if(exec) {
            
            for (var j in valuesToSetOnCurrentRow) { 
                
                currentRow[j] = valuesToSetOnCurrentRow[j];
    
            }
            
        }
        
    };

    var setCurrentRowWithOutData = function(currentRow, valuesToSetOnCurrentRow, outDataValue) {
        for (var key in outDataValue) {
            currentRow[outDataValue[key]] = valuesToSetOnCurrentRow[key];
        }
    };


    this.getFieldFromFilterPopup = function(field, args){
        var transmissaoNFeWidget = getTransmissaoNFeWidget();
        if(transmissaoNFeWidget){
            if(transmissaoNFeWidget.appliedFilter)
                return transmissaoNFeWidget.appliedFilter.FILTER_ROW[field] || 'TD';
            else
                return 'TD';
        }
    };

    this.criaDataSourceGridFilterAfterApply = function(args) {
        
        args.e.preventDefault();
        
        var transmissaoWidget = args.owner;
        var dataSourceFilter = transmissaoWidget.dataSourceFilter;
        var currentRowPopup = transmissaoWidget.appliedFilter.FILTER_ROW;
        var camposData = ['P_DTENTRSAIDINI', 'P_DTENTRSAIDFIM', 'P_DTEMISSAOINI', 'P_DTEMISSAOFIM'];
        var datasRemover = dataSourceFilter.filter(function(field){
            return camposData.indexOf(field.name) >= 0;
        });
        if(datasRemover && datasRemover.length >= 0){
            datasRemover.forEach(function(data){
                dataSourceFilter.splice(dataSourceFilter.indexOf(data), 1);
            });
        }
        var datasFiltro = dataSourceFilter.filter(function(value) {
            return value.name === "DTENTRSAID" || value.name === 'DTEMISSAO';
        });
        if(datasFiltro.length === 1){
            var dataEmissao = {
                "name": "DTEMISSAO",
                "value": {
                    0: "TD", 1: "TD"
                }
            };
            datasFiltro.push(dataEmissao);
        }
        var filterDefault = {
            'operator': '='
        };
        datasFiltro.forEach(function(data) {
            filterDefault.name = 'P_' + data.name + 'INI';
            filterDefault.value = (data.value[0] != 'TD') ? data.value[0] : '';
            dataSourceFilter.push(Util.clone(filterDefault));
            filterDefault.name = 'P_' + data.name + 'FIM';
            filterDefault.value = (data.value[1] != 'TD') ? data.value[1] : '';
            dataSourceFilter.push(Util.clone(filterDefault));
            if(data.name == 'DTEMISSAO') {
                if(data.value[0] == 'TD') {
                    filterDefault.name = 'P_TDTEMISSAO';
                    filterDefault.value = 'TD';
                    dataSourceFilter.push(Util.clone(filterDefault));
                }
                else {
                    filterDefault.name = 'P_TDTEMISSAO';
                    filterDefault.value = '';
                    dataSourceFilter.push(Util.clone(filterDefault));
                }
            }
            if(dataSourceFilter.indexOf(data) > 0)
                dataSourceFilter.splice(dataSourceFilter.indexOf(data), 1);
        });
        transmissaoWidget.dataSourceFilter = transmissaoWidget.dataSourceFilter.filter(function(field){return field.name[1] == '_' || field.name=="IDSTATUSNFE" ;});
        transmissaoWidget.reload();
    };
    
    this.atualizaFiltro = function(widget) {
      widget.dataSourceFilter[1].operator = '<';
    };

    this.montaDataSourceFilter = function(args){
        args.owner.dataSourceFilter = args.owner.dataSourceFilter.filter(function(field){return field.name[1] == '_';});
    };

    this.preencheDadosCancelamento = function(args){
        args.owner.currentRow = args.owner.getParent().currentRow;
    };

    this.validacoesNotaTransmissaoAfterMoveRow = function(args) {
        desabilitaActionsEmail(args.owner);
        desabilitaTabProdutorRural(args.owner);
        desabilitaActionCancelarTransmissao(args.owner, args.owner.currentRow.IDSTATUSNFE);
        desabilitaActionCartaCorrecao(args.owner, args.owner.currentRow.IDSTATUSNFE);
        desabilitaActionAlterarDataEmissao(args.owner, args.owner.currentRow.IDSTATUSNFE);
        desabilitaActionDownloadDanfe(args.owner, args.owner.currentRow.IDSTATUSNFE);
        var cartaCorrecao = templateManager.container.getWidget('swipeCartaCorrecao');
        cartaCorrecao.showAsTab = false;
        var popupLote = templateManager.container.getWidget('popupAtualizarLote');
        popupLote.showAsTab = false;
    };

    this.idorigdestOnChange = function(filterArgs) {
        alteraVisibilidadeCampoInscricaoFiltro(filterArgs);
        limpaCamposOutDataSelectsTpOperacFiltro(filterArgs);
    };

    this.getPaisFiltroEstado = function(){
        var widgetProdutorRural = this.getWidgetProdutorRural();
        return widgetProdutorRural.currentRow.CDPAIS;
    };

    var alteraVisibilidadeCampoInscricaoFiltro = function(filterArgs) {
        var filtroWidget = filterArgs.owner.field.widget;
        var cliente = filtroWidget.getField("NMRAZSOCCLIE");
        var fornecedor = filtroWidget.getField("NMRAZSOCFORN");
        var filial = filtroWidget.getField("DSFILIRECE");
        var tipoDestino = filterArgs.row.IDORIGDESTNF;

        switch (tipoDestino) {
            case "C":
                validacaoCamposOutTpOperacFilter(cliente, fornecedor, filial);
                break;
            case "F":
                validacaoCamposOutTpOperacFilter(fornecedor, cliente, filial);
                break;
            case "L":
                validacaoCamposOutTpOperacFilter(filial, fornecedor, cliente);
                break;
            default:
                filial.readOnly = true;
                cliente.readOnly = true;
                fornecedor.readOnly = true;
        }
    };

    var limpaCamposOutDataSelectsTpOperacFiltro = function(filterArgs)
    {
        var filtroWidget = filterArgs.owner.field.widget;
        var cabecalhoRow = filtroWidget.currentRow;
        var cliente = filtroWidget.getField("NMRAZSOCCLIE");
        var fornecedor = filtroWidget.getField("NMRAZSOCFORN");
        var filial = filtroWidget.getField("DSFILIRECE");

        filial.value(null);

        cliente.value(null);
        cabecalhoRow.CDCLIENTE = null;

        fornecedor.value(null);
        cabecalhoRow.CDFORNECED = null;
        cabecalhoRow.IDSITUCADA = null;
        cabecalhoRow.IDBLQNFRELAFOAF = null;

        cabecalhoRow.NRINSJURNF = null;
        cabecalhoRow.NMINSJURNF = null;
        cabecalhoRow.IDTPINSJURNF = null;
    };

    var validacaoCamposOutTpOperacFilter = function(princialField) {
        princialField.showOnView = true;
        princialField.showOnForm = true;
        princialField.isVisible = true;
        princialField.readOnly = false;

        for (var i = 1; i < arguments.length; i++) {
            arguments[i].showOnView = false;
            arguments[i].showOnForm = false;
            arguments[i].isVisible = false;
            arguments[i].readOnly = true;
        }
    };

    var desabilitaActionsEmail = function(widget){
        var actionEmailNFSelecionada = widget.getAction('emailNFSelecionada');
        var actionEmailTransp = widget.getAction('emailTransp');
        var actionEmailCanc = widget.getAction('emailCanc');
        var actionEmailCancTransp = widget.getAction('emailCancTransp');

        switch (widget.currentRow.IDSTATUSNFE) {
            case "A":
                actionEmailNFSelecionada.isVisible = true;
                if(widget.currentRow.NRINSJURTRAN){
                    actionEmailTransp.isVisible = true;
                }
                else{
                    actionEmailTransp.isVisible = false;
                }
                actionEmailCanc.isVisible = false;
                actionEmailCancTransp.isVisible = false;
                break;
            case "C":
                actionEmailNFSelecionada.isVisible = false;
                actionEmailTransp.isVisible = false;
                actionEmailCanc.isVisible = true;
                if(widget.currentRow.NRINSJURTRAN){
                    actionEmailCancTransp.isVisible = true;
                }
                else{
                    actionEmailCancTransp.isVisible = false;
                }
                break;
            default:
                actionEmailNFSelecionada.isVisible = false;
                actionEmailTransp.isVisible = false;
                actionEmailCanc.isVisible = false;
                actionEmailCancTransp.isVisible = false;
        }
    };

    var desabilitaActionCancelarTransmissao = function(widgetPai, status){
        var widgetCancelamento = templateManager.container.getWidget('popupCancelar');
        var actionCancelarTransmissao = widgetPai.getAction('cancelarTransmissao');
        var actionConfirmarCancelamento = widgetCancelamento.getAction('confirmarCancelamento');

        if(status === 'A'){
            actionCancelarTransmissao.isVisible = true;
            actionConfirmarCancelamento.isVisible = true;
            widgetCancelamento.getField('DSMOTIVOCANCNFE').readOnly = false;
        }
        else{
            actionCancelarTransmissao.isVisible = false;
            actionConfirmarCancelamento.isVisible = false;
            widgetCancelamento.getField('DSMOTIVOCANCNFE').readOnly = true;
        }
    };

    var desabilitaActionCartaCorrecao = function(widgetPai, status){
        var actionCartaCorrecao = widgetPai.getAction('novaCartaCorrecao');

        if(status === 'A'){
            actionCartaCorrecao.isVisible = true;
        }
        else{
            actionCartaCorrecao.isVisible = false;
        }
    };

    var desabilitaActionAlterarDataEmissao = function(widgetPai, status){
        var actAlterarDataEmissao = widgetPai.getAction('actAlterarDataEmissao');

        if(status === 'R'){
            actAlterarDataEmissao.isVisible = true;
        }
        else{
            actAlterarDataEmissao.isVisible = false;
        }
    };
    
    var desabilitaActionDownloadDanfe = function(widgetPai, status){
        var actAlterarDataEmissao = widgetPai.getAction('actVisualizarDanfe');

        if(status === 'A'){
            actAlterarDataEmissao.isVisible = true;
        }
        else{
            actAlterarDataEmissao.isVisible = false;
        }
    };

    var desabilitaTabProdutorRural = function(widget){
        var widgetProdutorRural = widget.container.getWidget('produtorRural');

        if(widget.currentRow.IDENTRSAIDOP === "S"){
            widgetProdutorRural.isVisible = false;
        }
        else{
            switch (widget.currentRow.IDSTATUSNFE) {
                case "P":
                case "R":
                    widgetProdutorRural.isVisible = true;
                    break;
                default:
                    widgetProdutorRural.isVisible = false;
            }
        }
    };

    this.actAlterarDataEmissaoOnClick = function(transmissaoArgs) {
        General.edit(transmissaoArgs.owner.widget);
    };

    this.cancelaEdicaoDataEmissao = function(args){
        General.backOrCancel(args);
    };

    this.alteraDataEmissao = function(args){
        Dfs49000_TransmissaoNfe_Service.alteraDataEmissao(args.row).then(function(callback){
            args.owner.widget.view();
        });
    };

    this.inutNF = function(args){
        var widget = args.owner.widget;
        if(widget.appliedFilter.FILTER_ROW.CDFILIAL){
            sessionStorage.setItem('CDFILIAL',widget.appliedFilter.FILTER_ROW.CDFILIAL);
            sessionStorage.setItem('NMFILIAL',widget.appliedFilter.FILTER_ROW.NMFILIAL);
            sessionStorage.setItem('NRINSJURIEST', widget.appliedFilter.FILTER_ROW.NRINSJURIESTINUTNF);
        }
        if(widget.appliedFilter.FILTER_ROW.CDEMPRESA){
            sessionStorage.setItem('CDEMPRESA',widget.appliedFilter.FILTER_ROW.CDEMPRESA);
            sessionStorage.setItem('NMRAZSOCEMPR',widget.appliedFilter.FILTER_ROW.NMEMPRESA);
        }
        if(widget.appliedFilter.FILTER_ROW.CDINSCESTA)
            sessionStorage.setItem('CDINSCESTA',widget.appliedFilter.FILTER_ROW.CDINSCESTA);
        if(widget.appliedFilter.FILTER_ROW.CDINSCMUNI)
           sessionStorage.setItem('CDINSCMUNI',widget.appliedFilter.FILTER_ROW.CDINSCMUNI);
        ScreenService.openWindow('df_saida#dfs51000_inutilizacao_nota_fiscal_eletronica');
    };

    this.abrirPopupAtualizarLote = function(args){
        var popupAtualizarLote = args.owner.widget.container.getWidget('popupAtualizarLote');
        popupAtualizarLote.editing      = true;
        popupAtualizarLote.showAsTab    = true;
        if(args.owner.name == 'atualizarLote') {
            ScreenService.closePopup();
        }
        ScreenService.openSwipe(popupAtualizarLote).then(() => {
            popupAtualizarLote.reload();
            popupAtualizarLote.activate();
        });
    };

    this.abrirPopupCancelarTransmissao = function(args){
        if(moment().isBefore(moment(args.owner.widget.currentRow.DTEMISSAO, 'DD/MM/YYYY HH:mm:ss').add(1, 'day')))
        {
            var popupCancelar = args.owner.widget.container.getWidget('popupCancelar');
            ScreenService.openPopup(popupCancelar);
        }
        else{
            if(args.owner.widget.appliedFilter.FILTER_ROW.IDCANCEXTMPNFE !== 'S'){
                ScreenService.showMessage('Operação bloqueada! Não é permitido o cancelamento extemporâneo de Nota Fiscal Eletrônica.');
            }
            else{
                var mensagem = "Atenção! Esta Nota Fiscal Eletrônica está sendo cancelada num prazo superior à 24 horas a partir da sua autorização." +
                               " Conforme legislação estadual em vigor esta operação poderá estar sujeito a multa ou não ser permitida." +
                               " Deseja confirmar a operação?";
        		ScreenService.confirmMessage(mensagem, 'question', function (success) {
                    var popupCancelar = args.owner.widget.container.getWidget('popupCancelar');
                    ScreenService.openPopup(popupCancelar);
                });

            }
        }
    };
    
    this.viewNotasLote = function(args) {
        var popupNotaLote = templateManager.container.getWidget('wdg_notaLote');
        popupNotaLote.editing = true;
        popupNotaLote.label = 'Notas Fiscais do Lote ' + args.owner.widget.currentRow.NRLOTENFE;
        ScreenService.openPopup(popupNotaLote).then(function(){
            popupNotaLote.reload();
        });
    };

    this.abrirSwipeCartaCorrecao = function(args){
        General.callRestEngine("df#/verificaTipoOperFili", "Row", args.row).then((success) => {
            if(success.dataset.liberado[0]) {
                var cartaCorrecao = args.owner.widget.container.getWidget('swipeCartaCorrecao');
                cartaCorrecao.showAsTab = true;
                cartaCorrecao.isVisible = true;
                cartaCorrecao.getParent().openTab('swipeCartaCorrecao');
                preencheDadosCartaCorrecao(cartaCorrecao); 
            }
            else {
                ScreenService.showMessage("Não é possivel enviar carta de correção para esta nota. Tipo de operação não permitido para a unidade.");
            }
        });
        
        /*
        ScreenService.openSwipe(swipeCartaCorrecao).then(function(callback){
            preencheDadosCartaCorrecao(swipeCartaCorrecao);
            // swipeCartaCorrecao.view();
        });*/
    };

    this.montaConfig = function(args){
        Dfs49000_TransmissaoNfe_Service.montaConfigTransmissao(getFilterPopupCurrentRow(args)).then(function(resp){
        }, requestError);
    };

    this.verificarStatusServico = function(args){
        var rows = args.owner.widget.getCheckedRows();
        
        var notasMarcadas = rows.filter((nota) => {
              return nota.IDSTATUSNFE == 'P' || nota.IDSTATUSNFE == 'R';
        });

        if(notasMarcadas.length > 0) {
            let self = this;
            
            Dfs49000_TransmissaoNfe_Service.statusServico(notasMarcadas).then(function(status){
                if(Object.keys(status[0])[0] === "STATUS"){
                    var popupTransmissao = args.owner.widget.container.getWidget('popupTransmissao');
                    popupTransmissao.getField("MENSAGEM").setValue(status[0].STATUS);
                    ScreenService.openPopup(popupTransmissao);
                }
                if(Object.keys(status[0])[0] === "SUCCESS"){
                    ScreenService.showMessage('Nota(s) transmitida(s) com sucesso!');
                    atualizaStatusNota(notasMarcadas, status[0].SUCCESS);
                    args.owner.widget.reload();
                    args.owner.widget.unSelectAll();
                }
                else if(Object.keys(status[0])[0] === "ERRORS"){
                    if(typeof status[0].ERRORS === 'string')
                        ScreenService.showMessage(status[0].ERRORS, "ERROR");
                    else{
                        let popupErros = args.owner.widget.container.getWidget('popupErros');
                        let erros = []; 
                        
                        self.showFieldsPopupErros(popupErros.fields, false);
                        for(let i in status[0].ERRORS) {
                            erros[i] = { DSERRO: status[0].ERRORS[i] };
                        }
                        popupErros.dataSource.data = erros;
                        popupErros.reload();
                        ScreenService.openPopup(popupErros);
                    }
                }
                else{
                    ScreenService.showMessage(status[0].STATUS);
                }
            });
        }
        else {
            ScreenService.showMessage("Todas as notas selecionadas já foram transmitidas.");
        }
    };

    this.recarregarCampo = function(nomeWidget,nomeCampo){
        var widget = templateManager.container.getWidget(nomeWidget);
        widget.getField(nomeCampo).reload();
    };

    this.transmiteNotasSelecionadas = function(args, tipo){
        var rows = args.owner.field.widget.getCheckedRows();
        
        var notasMarcadas = rows.filter((nota) => {
              return nota.IDSTATUSNFE == 'P' || nota.IDSTATUSNFE == 'R';
        });
        if(notasMarcadas.length > 0) {
            let self = this;
            
            Dfs49000_TransmissaoNfe_Service.transmiteNotas(notasMarcadas, tipo).then(function(transmissao){
                if(Object.keys(transmissao[0])[0] === "SUCCESS"){
                    
                    atualizaStatusNota(notasMarcadas, transmissao[0].SUCCESS);
                }
                else if(Object.keys(transmissao[0])[0] === "ERRORS"){
                    if(typeof transmissao[0].ERRORS === 'string')
                        ScreenService.showMessage(transmissao[0].ERRORS, "ERROR");
                    else{
                        let popupErros = args.owner.field.widget.container.getWidget('popupErros');
                        let erros = []; 
                        
                        self.showFieldsPopupErros(popupErros.fields, false);
                        for(let i in transmissao[0].ERRORS) {
                            erros[i] = { DSERRO: transmissao[0].ERRORS[i] };
                        }
                        popupErros.dataSource.data = erros;
                        popupErros.reload();
                        ScreenService.openPopup(popupErros);
                    }
                }
                else{
                    ScreenService.showMessage(transmissao[0]);
                }
            });
        }
        else {
            ScreenService.showMessage("Todas as notas selecionadas já foram transmitidas.");
        }
    };

    this.atualizaLotesSelecionados = function(args){
        let self = this;
        let lotesMarcados = args.owner.widget.getCheckedRows();
        let lotesMarcadosEnvio     = lotesMarcados.map((row) => {
            return Util.clone(row, ["__rightActions", "__leftActions"]);
        });
        
        Dfs49000_TransmissaoNfe_Service.atualizaLote(lotesMarcadosEnvio).then(function(respLote) {
            if(Object.keys(respLote[0])[0] === "SUCCESS") {
                ScreenService.showMessage('Lote(s) atualizado(s) com sucesso!');
                args.owner.widget.unSelectAll();
                templateManager.container.getWidget('transmissaoNFe').activate(); 
                ScreenService.closeSwipe();
                args.owner.widget.getParent().reload();
                args.owner.widget.getParent().activate();
                
                if(respLote[0].NOTAS_SUCCESS.length) {
                    Dfs49000_TransmissaoNfe_Service.getConfig().then((config) => {
                        if(config.aprovacaoAutomatica == 'S') {
                            Dfs49000_TransmissaoNfe_Service.aprovarNotasTransmissao(respLote[0].NOTAS_SUCCESS).then((successAprovacao) => {
                                if(Object.keys(successAprovacao).length == 1 ) {
                                    if(Object.keys(successAprovacao[0])[0] === 'SUCESSO_APROVACAO' ) {
                                        MessageService.successNotificationByCode('SUCESSO_APROVACAO_LOTE');
                                    }
                                } else if(successAprovacao.aprovarNotasTransmissao) {
                                    if(Object.keys(successAprovacao.aprovarNotasTransmissao)[0] === 'ERROS_APROVACAO' ) {
                                        let erros   = successAprovacao.aprovarNotasTransmissao.ERROS_APROVACAO,
                                            dsErros = Dfs49000_TransmissaoNfe_Service.errosNotasTransmissao(erros);
                                        self.showPopupErros(dsErros);
                                    } else {
                                        MessageService.showMessageByCode('ERROR_APROVACAOEMAIL_LOTE');
                                        console.error(successAprovacao);
                                    }
                                } else {
                                    MessageService.showMessageByCode('ERROR_APROVACAOEMAIL_LOTE');
                                    console.error(successAprovacao);
                                }
                            }, (error) => {
                                MessageService.showMessageByCode('ERROR_APROVACAOEMAIL_LOTE');
                                console.error(error);
                            });
                        }
                        if(config.emailAutomatico == 'S') {
                            Dfs49000_TransmissaoNfe_Service.emailAutomaticoTransmissao(respLote[0].NOTAS_SUCCESS, config).then((successEmail) => {
                                if(Object.keys(successEmail[0])[0] === 'SUCESSO_EMAIL' ) {
                                        MessageService.successNotificationByCode('SUCESSO_EMAIL_LOTE');
                                } else if(Object.keys(successEmail[0])[0] === 'ERROS_EMAIL' ) {
                                    let erros   = successEmail[0].ERROS_EMAIL,
                                        dsErros = Dfs49000_TransmissaoNfe_Service.errosNotasTransmissao(erros);
                                    self.showPopupErros(dsErros);
                                }
                                else {
                                    MessageService.showMessageByCode('ERROR_APROVACAOEMAIL_LOTE');
                                    console.error(successEmail);
                                }
                            }, (error) => {
                                MessageService.showMessageByCode('ERROR_APROVACAOEMAIL_LOTE');
                                console.error(error);
                            });
                        }
                    });
                }
            }
            else if(Object.keys(respLote[0])[0] === "ERRORS") {
                args.owner.widget.unSelectAll();
                ScreenService.showMessage(respLote[0].ERRORS, "ERROR");
            }
            else {
                args.owner.widget.unSelectAll();
                ScreenService.successNotification(respLote[0]);
            }
        }, (error) => {
            ScreenService.errorNotification("Ocorreu um erro ao atualizar o lote. Tente novamente mais tarde.");
            console.error(error);
        });
    };
    
    this.showPopupErros = (dsErros) => {
        let popupErros = templateManager.container.getWidget('popupErros'),
            atualWidget = templateManager.container.getActiveWidget();
        
        if(popupErros.name != atualWidget.name) {
            this.showFieldsPopupErros(popupErros.fields, true);
            popupErros.dataSource.data = dsErros;
            ScreenService.openPopup(popupErros);
            MessageService.errorNotificationByCode('ERROR_APROVACAOEMAIL_LOTE');
        } else {
            let newDsErros = popupErros.dataSource.data.concat(dsErros);
            popupErros.dataSource.data = newDsErros;
        }
        popupErros.reload();
    };
    
    this.goBackPopupErros = (args) => {
        args.owner.widget.dataSource.data = [];
        args.owner.widget.container.getWidget('transmissaoNFe').activate();
        ScreenService.closePopup();
    };
    
    this.showFieldsPopupErros = (fields, exibe) => {
        fields.forEach(field => {
            if(field.name != 'DSERRO') field.isVisible = exibe;
        });
    };

    this.cancelarTransmissao = function(args){
        var motivoCancelamento = args.owner.widget.currentRow.DSMOTIVOCANCNFE;
        if(!motivoCancelamento || motivoCancelamento.trim().length < 15)
            ScreenService.showMessage('Motivo deve conter no mínimo 15 caracteres', 'ERROR');
        else{
            Dfs49000_TransmissaoNfe_Service.cancelarTransmissao(args.row).then((resp) => {
                if(Object.keys(resp[0])[0] === "SUCCESS"){
                    args.owner.widget.currentRow.IDSTATUSNFE = 'C';
                    args.owner.widget.currentRow.DTHRPROTOCOLNFE = resp[0].SUCCESS.DTHRPROTOCOLNFE;
                    args.owner.widget.currentRow.DSOBSSTATUSNFE = resp[0].SUCCESS.DSOBSSTATUSNFE;
                    ScreenService.successNotification("Nota cancelada com sucesso.");
                    ScreenService.closePopup(args.owner.widget);
                    ScreenService.goBack();
                    args.owner.widget.getParent().reload();
                    
                    Dfs49000_TransmissaoNfe_Service.getConfig().then(function(config){
                        if(config.emailAutomatico === 'S'){
                            args.owner.widget.currentRow.CDFILIAL = resp[0].SUCCESS.CDFILIAL;
                            args.owner.widget.currentRow.NRLANCTONF = resp[0].SUCCESS.NRLANCTONF;
                            Dfs49000_TransmissaoNfe_Service.enviarEmailNF(args.owner.widget.currentRow, 'C').then(function(resp){
                                if(Object.keys(resp[0])[0] === "SUCCESS"){ 
                                    args.owner.widget.currentRow.IDEMAILNFE = 'E';
                                }
                                else if(Object.keys(resp[0])[0] === "ERRORS"){
                                    MessageService.showMessageByCode('EMAIL_NAO_ENVIADO_NF');
                                }
                            });
                        }
                    });
                }
                else if(Object.keys(resp[0])[0] === "ERRORS"){
                    ScreenService.showMessage(resp[0].ERRORS, "ERROR");
                }
                else{
                    ScreenService.showMessage(resp[0]);
                }
            });
        }
    };

    this.visualizarDanfe = function(args){
        var nota = args.owner.widget.currentRow;

        if(nota.IDSTATUSNFE == 'A'){
            Dfs49000_TransmissaoNfe_Service.visualizarDanfe([nota]).then(function(resp){
                resp.forEach(function(fileUri) {
                    this.downloadArquivo(fileUri);
                    //window.open(window.location.origin+fileUri);
                }.bind(this));
            }.bind(this), function(error) {
                ScreenService.showMessage(error.data.error);
            }.bind(this));
        }
        else{
            ScreenService.showMessage("Esta nota não foi aceita.");
        }
    };
    
     this.downloadArquivo = function(nmArquivo){
        window.open(window.serviceUrls.filter(function(x){return x.package == 'df';})[0].baseUrl+"/downloadArquivo?requestType=Row&row[DSARQUIVO]=" + nmArquivo);
    };

    // this.visualizarNota = function(args){
    //     var filter = [{
    //                 "name": "NRNOTAFISC",
    //                 "operator": "=",
    //                 "value": "000000034"
    //             },
    //             {
    //                 "name": "IDENTRSAIDOP",
    //                 "operator": "=",
    //                 "value": "S"
    //             },
    //             {
    //                 "name": "CDFILIEMIT",
    //                 "operator": "=",
    //                 "value": "0001"
    //             },
    //             {
    //                 "name": "DTENTRSAID",
    //                 "operator": "BETWEEN",
    //                 "value": ["05/09/2016", "09/09/2016"]
    //             }];
    //     ScreenService.openWindow('df_saida#dfs11000_lancamento_saida', filter);
    // }

    this.emissaoBoleto = function(widget){
        sessionStorage.setItem('WIDGET_NAME', widget.name);
        sessionStorage.setItem('CDEMPRESA', widget.currentRow.CDEMPRESA);
        sessionStorage.setItem('NMEMPRESA', widget.currentRow.NMEMPRESA);
        sessionStorage.setItem('NRNOTAFISC', widget.currentRow.NRNOTAFISC);
		
        this.openWindow("fin_contaReceber#rec00001_titulos_a_receber");
    };
    
    this.openWindow = function(windowName, newWindow) {
        Dfs49000_TransmissaoNfe_Service.requestModulesMetadata().then(function (modules) {
            let subModule = windowName.split('#')[0];
            let module = subModule.split('_')[0];
            let url = window.location.origin + '/' + modules[module].baseUrl + "/#/" + windowName;
            ScreenService.openLink(url, false);
        });
    };

    this.downloadDanfeNotasSelecionadas = function(args){
        var notasMarcadas = args.owner.widget.getCheckedRows();
        var notas = notasMarcadas.filter(function(notafiscal){
            return notafiscal.IDSTATUSNFE === "A";
        });
        if(notas.length > 0){
            Dfs49000_TransmissaoNfe_Service.downloadDanfe(notas).then(function(fileUri){
                window.open(window.location.origin+fileUri);
            }, function(error) {
                ScreenService.showMessage(error.data.error);
            });
        }
        else{
            ScreenService.showMessage("Nenhuma nota aceita foi marcada");
        }
    };

    this.downloadXMLNotasSelecionadas = function(args){
        var notasMarcadas = args.owner.widget.getCheckedRows();
        var notas = notasMarcadas.filter(function(notafiscal){
            return notafiscal.IDSTATUSNFE === "A";
        });
        
        if(notas.length > 0){
            Dfs49000_TransmissaoNfe_Service.downloadXML(notas).then(function(fileUri){
                window.open(window.location.origin+fileUri);
            }, function(error) {
                ScreenService.showMessage(error.data.error);
            });
        }
        else{
            ScreenService.showMessage("Nenhuma nota aceita foi marcada");
        }
    };

    this.enviarEmailPendenteNFsAceitas = function(args){
        var notasFiltradas= args.dataSource.data;
        var notas = notasFiltradas.filter(function(notafiscal){
            return notafiscal.IDSTATUSNFE === "A" && notafiscal.IDEMAILNFE === "P";
        });
        if(notas.length > 0){
            Dfs49000_TransmissaoNfe_Service.enviarEmailPendenteNFsAceitas(notas).then(function(resp){
                if(Object.keys(resp[0])[0] === "SUCCESS"){
                    ScreenService.closePopup(args.owner.widget);
                    args.owner.widget.activate();
                    args.owner.widget.currentRow.IDEMAILNFE = 'E';
                    ScreenService.showMessage(resp[0].SUCCESS);
                }
                else if(Object.keys(resp[0])[0] === "ERRORS"){
                    ScreenService.showMessage(resp[0].ERRORS, "ERROR");
                }
                else{
                    ScreenService.showMessage(resp[0]);
                }
            });
        }
        else{
            ScreenService.showMessage("Nenhuma nota aceita com e-mail pendente");
        }
    };

    this.enviarEmailNF = function(args, tipo){
        Dfs49000_TransmissaoNfe_Service.enviarEmailNF(args.owner.widget.currentRow, tipo).then(function(resp){
            if(Object.keys(resp[0])[0] === "SUCCESS"){
                ScreenService.closePopup(args.owner.widget);
                args.owner.widget.activate();
                args.owner.widget.currentRow.IDEMAILNFE = 'E';
                ScreenService.showMessage(resp[0].SUCCESS);
            }
            else if(Object.keys(resp[0])[0] === "ERRORS"){
                ScreenService.showMessage(resp[0].ERRORS, "ERROR");
            }
            else{
                ScreenService.showMessage(resp[0]);
            }
        });
    };

    this.dataAtualMenos15segundos = function(){
        return moment().add(1, 'hours').format("DD/MM/YYYY HH:mm:ss");
    };

    var atualizaStatusNota = function(notasMarcadas, transmissao){
        notasMarcadas.forEach(function(nota){
            nota.DTEMISSAO = this[nota.NRLANCTONF].DTEMISSAO;
            nota.NRACESSONFE = this[nota.NRLANCTONF].NRACESSONFE;
            nota.NRNOTAFISC = this[nota.NRLANCTONF].NRNOTAFISC;
            nota.NRNOTAFISCALELE = this[nota.NRLANCTONF].NRNOTAFISCALELE;
            nota.NRPROTOCOLONFE = this[nota.NRLANCTONF].NRPROTOCOLONFE;
            nota.IDSTATUSNFE = this[nota.NRLANCTONF].IDSTATUSNFE;
        }, transmissao);
    };

    this.salvaProdutorRural = function(args) {
        var widgetProdutorRural = args.owner.widget;
        if (widgetProdutorRural.isValid()) {
            var rowTransmissao = widgetProdutorRural.getParent().currentRow;
            args.row.CDFILIAL = rowTransmissao.CDFILIAL;
            args.row.NRLANCTONF = rowTransmissao.NRLANCTONF;
            args.row.NRACESSONFE = widgetProdutorRural.currentRow.NRACESSONFE.split('.').join("");
            if(args.row.__is_new)
                args.row.NRSEQPRODRURAL = Dfs49000_TransmissaoNfe_Service.getProxSeqProdRural(widgetProdutorRural.dataSource.data);
            if(args.row.NRINSJURIEST.length === 11)
                args.row.IDTPINSCJUR = 'F';
            else if(args.row.NRINSJURIEST.length === 14)
                args.row.IDTPINSCJUR = 'J';
            else
                return requestError({data: {error:'Inscrição informada inválida'}});
            args.row.NRLANCTONF = rowTransmissao.NRLANCTONF;
            Dfs49000_TransmissaoNfe_Service.salvaProdutorRural(args.row).then(function(callback){
                widgetProdutorRural.currentRow.__is_new = false;
                widgetProdutorRural.view();
            }, requestError);
        }
    };

    this.editarProdRural = function(args){
        General.edit(args.owner.widget);
    };

    this.excluirProdutorRural = function(args) {
        Dfs49000_TransmissaoNfe_Service.excluirProdutorRural(args.row).then(function(callback){
            General.backOrCancel(args);
            var rowExcluida = args.owner.widget.dataSource.data.filter(function(prodRural){
                return args.row.NRLANCTONF === prodRural.NRLANCTONF;
            });
            var index = args.owner.widget.dataSource.data.indexOf(rowExcluida[0]);
            args.owner.widget.dataSource.data.splice(index, 1);
            ScreenService.showMessage('Produtor Rural excluido com sucesso');
        }, requestError);
    };

    this.novoProdRural = function(args){
        args.owner.widget.newRow();
    };

    // var preencheDadosCce = true;

    var preencheDadosCartaCorrecao = function(widgetCartaCorrecao){
        widgetCartaCorrecao.list();
        widgetCartaCorrecao.reload().then(function(callback){
            widgetCartaCorrecao.currentRow = widgetCartaCorrecao.dataSource.data[0] || [];
            widgetCartaCorrecao.view();
        });
    };

    var padNrseqcce = function(nrseqcce){
        var str = "" + nrseqcce;
        var pad = "00";
        return pad.substring(0, pad.length - str.length) + str;
    };

    this.editarCartaCorrecao = function(args){
        var widgetCartaCorrecao = args.owner.widget;
        widgetCartaCorrecao.currentRow.NRSEQCCE = padNrseqcce(parseInt(widgetCartaCorrecao.currentRow.NRSEQCCE || 0) + 1);
        widgetCartaCorrecao.currentRow.NRPROTOCOLCCE = "";
        widgetCartaCorrecao.currentRow.DTHRPROTOCOLCCE = "";
        widgetCartaCorrecao.currentRow.DSOBSSTATUSCCE = "";
        widgetCartaCorrecao.currentRow.DSCORRECAONFE    = "";
        widgetCartaCorrecao.template = 'widget/form.html';
        widgetCartaCorrecao.templateToForm = 'widget/form.html';
        widgetCartaCorrecao.edit();
        widgetCartaCorrecao.isVisible = true;
    };

    this.actCancelarEdicaoCce = function(args){
        var widgetCartaCorrecao = args.owner.widget;
        General.backOrCancel(args);
		widgetCartaCorrecao.showAsTab = false;
        widgetCartaCorrecao.isVisible = false;
        widgetCartaCorrecao.currentRow.NRSEQCCE = widgetCartaCorrecao.currentRow.NRSEQCCE - 1; 
        widgetCartaCorrecao.template = 'widget/form-view.html';
        widgetCartaCorrecao.templateToForm = 'widget/form-view.html';
        ScreenService.closeSwipe();
        templateManager.container.getWidget('transmissaoNFe').activate();
    };
    
     this.actVoltarCce = function(args){
        var widgetCartaCorrecao = args.owner.widget;
		widgetCartaCorrecao.showAsTab = false;
        widgetCartaCorrecao.isVisible = false;
    };

    this.confirmarCartaCorrecao = function(args){
		var widgetCartaCorrecao = args.owner.widget;
		if(!widgetCartaCorrecao.currentRow.DSCORRECAONFE || (widgetCartaCorrecao.currentRow.DSCORRECAONFE.length < 15 || widgetCartaCorrecao.currentRow.DSCORRECAONFE.length > 1000))
		    return ScreenService.showMessage("Descrição da correção deve ter no mínimo 15 e no máximo 1000 caracteres");

		var mensagem = "A Carta de Correção é disciplinada pelo § 1º-A do art. 7º do Convênio S/N," +
                           " de 15 de dezembro de 1970 e pode ser utilizada para regularização de erro ocorrido na emissão de documento fiscal," +
                           " desde que o erro não esteja relacionado com:" +
                           " I - as variáveis que determinam o valor do imposto tais como: base de cálculo, alíquota, diferença de preço, quantidade, valor da operação ou da prestação;" +
                           " II - a correção de dados cadastrais que implique mudança do remetente ou do destinatário;" +
                           " III - a data de emissão ou de saída." +
                           " Confirma a Condição de Uso da Carta de Correção?";
		ScreenService.confirmMessage(mensagem, 'question', function (success) {
            Dfs49000_TransmissaoNfe_Service.enviarCartaCorrecao(getTransmissaoNFeWidget().currentRow, widgetCartaCorrecao.currentRow.DSCORRECAONFE, widgetCartaCorrecao.currentRow.NRSEQCCE).then(function(resp){
                if(resp) {
                    if(Object.keys(resp[0])[0] === "SUCCESS") {
                        ScreenService.successNotification("Carta de Correção enviada com sucesso.");
                        Dfs49000_TransmissaoNfe_Service.getConfig().then(function(config){
                            if(config.emailAutomatico === 'S'){
                                Dfs49000_TransmissaoNfe_Service.enviarEmailCC(getTransmissaoNFeWidget().currentRow, widgetCartaCorrecao.currentRow.NRSEQCCE, "CCe").then(function(resp){
                                    if(Object.keys(resp[0])[0] === "ERRORS") {
                                        MessageService.showMessageByCode('EMAIL_NAO_ENVIADO_CC');
                                    }
                                });
                            }
                        });
                    } else {
                        ScreenService.showMessage(resp[0].ERRORS);
                    }
                } else {
                    MessageService.showMessageByCode('ERROR_ENVIOCCE');
                    console.error(resp);
                }
                preencheDadosCartaCorrecao(widgetCartaCorrecao);
                widgetCartaCorrecao.template = 'widget/form-view.html';
                widgetCartaCorrecao.templateToForm = 'widget/form-view.html';
                widgetCartaCorrecao.reload();
            }, function(error) {
                MessageService.showMessageByCode('ERROR_ENVIOCCE');
                console.error(error);
            });
        });
    };
    
    this.reenvioMailCartaCorrecao = (widgetCartaCorrecao) => {
        MessageService.confirmMessageByCode('REENVIO_MAILCCE').then(confirm => {
            Dfs49000_TransmissaoNfe_Service.enviarEmailCC(getTransmissaoNFeWidget().currentRow, widgetCartaCorrecao.currentRow.NRSEQCCE, "CCe").then(function(resp){
                if(Object.keys(resp[0])[0] === "SUCCESS") {
                    ScreenService.successNotification(resp[0].SUCCESS);
                }
                else if(Object.keys(resp[0])[0] === "ERRORS") {
                    if(typeof resp[0].ERRORS === 'string') ScreenService.showMessage(resp[0].ERRORS, "ERROR");
                    else MessageService.showMessageByCode('ERROR_ENVIOCCE');
                }
                else MessageService.showMessageByCode('ERROR_ENVIOCCE');
            }, (error) => {
                MessageService.showMessageByCode('ERROR_ENVIOCCE');
                console.error(error);
            });
        });
    };
    
    this.cartaCorrecaoLoad = (widgetCartaCorrecao) => {
        let datasource = widgetCartaCorrecao.dataSource.data;
        
        if(datasource.length) widgetCartaCorrecao.getAction('reenvioMailCartaCorrecao').isVisible = true;
        else                  widgetCartaCorrecao.getAction('reenvioMailCartaCorrecao').isVisible = false;
    };
    
    this.popupAtualizarLoteClose = function(widget) {
        templateManager.container.getWidget('transmissaoNFe').activate(); 
        ScreenService.closeSwipe();  
        widget.showAsTab = false;
    };
    
    this.consultaSituacaoNF = function(widget) {
        if(widget.currentRow.IDSTATUSNFE != 'R' && widget.currentRow.IDSTATUSNFE != 'T') {
            MessageService.showMessageByCode('BLOCK_CONSULTA_SITUACAONF');
        }
        else if(widget.currentRow.IDSTATUSNFE == 'T') {
            let self = this;
            
            General.callRestEngine('df#/crud/nota_lote', 'FilterData', [{"name": "P_CDEMPLOTENFE", "operator": "=", "value": widget.currentRow.CDEMPRESA},
                                                                        {"name": "CDFILIAL"      , "operator": "=", "value": widget.currentRow.CDFILIAL},
                                                                        {"name": "NRLANCTONF"    , "operator": "=", "value": widget.currentRow.NRLANCTONF},
                                                                        {"name": "NRORG"         , "operator": "=", "value": widget.currentRow.NRORG}]
            ).then((result) => {
                let nota = result.dataset.nota_lote;
                
                if(nota.length) {
                    if(!nota[0].NRRECIBONFE) {
                        self.realizaConsultaSituacaoNF(widget);
                    } else {
                        MessageService.showMessageByCode('BLOCK_CONSULTA_SITUACAONF');
                    }
                } else {
                    MessageService.showMessageByCode('BLOCK_CONSULTA_SITUACAONF');
                }
            });
        } else {
            this.realizaConsultaSituacaoNF(widget);
        }
    }; 
    
    this.realizaConsultaSituacaoNF = (widget) => {
        let self = this;
        
        Dfs49000_TransmissaoNfe_Service.getConfig().then((config) => {
            Dfs49000_TransmissaoNfe_Service.consultaSituacaoNF(widget.currentRow, config).then((respConsulta) => {
                ScreenService.goBack();
                ScreenService.closePopup();
                widget.reload();
                
                if(Object.keys(respConsulta[0])[0] === 'SUCCESS') {
                    MessageService.successNotificationByCode("CONSULTA_SITUACAONF");
                    
                    if(Object.keys(respConsulta[0])[1] === 'NOTAS_SUCCESS') {
                        if(respConsulta[0].NOTAS_SUCCESS.length) {
                            if(config.aprovacaoAutomatica == 'S') {
                                Dfs49000_TransmissaoNfe_Service.aprovarNotasTransmissao(respConsulta[0].NOTAS_SUCCESS).then((successAprovacao) => {
                                    if(Object.keys(successAprovacao).length == 1 ) {
                                        if(Object.keys(successAprovacao[0])[0] === 'SUCESSO_APROVACAO' ) {
                                            MessageService.successNotificationByCode('SUCESSO_APROVACAO_LOTE');
                                        }
                                    } else if(successAprovacao.aprovarNotasTransmissao) {
                                        if(Object.keys(successAprovacao.aprovarNotasTransmissao)[0] === 'ERROS_APROVACAO') {
                                            let erros   = successAprovacao.aprovarNotasTransmissao.ERROS_APROVACAO,
                                                dsErros = Dfs49000_TransmissaoNfe_Service.errosNotasTransmissao(erros);
                                            self.showPopupErros(dsErros);
                                        } else {
                                            MessageService.showMessageByCode('ERROR_APROVACAOEMAIL_LOTE');
                                            console.error(successAprovacao);
                                        }
                                    } else {
                                        MessageService.showMessageByCode('ERROR_APROVACAOEMAIL_LOTE');
                                        console.error(successAprovacao);
                                    }
                                }, (error) => {
                                    MessageService.showMessageByCode('ERROR_APROVACAOEMAIL_LOTE');
                                    console.error(error);
                                });
                            }
                            if(config.emailAutomatico == 'S') {
                                Dfs49000_TransmissaoNfe_Service.emailAutomaticoTransmissao(respConsulta[0].NOTAS_SUCCESS, config).then((successEmail) => {
                                    if(Object.keys(successEmail[0])[0] === 'SUCESSO_EMAIL' ) {
                                        MessageService.successNotificationByCode('SUCESSO_EMAIL_LOTE');
                                    } else if(Object.keys(successEmail[0])[0] === 'ERROS_EMAIL' ) {
                                        let erros   = successEmail[0].ERROS_EMAIL,
                                            dsErros = Dfs49000_TransmissaoNfe_Service.errosNotasTransmissao(erros);
                                        self.showPopupErros(dsErros);
                                    }
                                    else {
                                        MessageService.showMessageByCode('ERROR_APROVACAOEMAIL_LOTE');
                                        console.error(successEmail);
                                    }
                                }, (error) => {
                                    MessageService.showMessageByCode('ERROR_APROVACAOEMAIL_LOTE');
                                    console.error(error);
                                });
                            }
                        }
                    }
                } else if(Object.keys(respConsulta[0])[0] === 'ERRORS') {
                    if(typeof respConsulta[0].ERRORS === 'string') {
                        ScreenService.showMessage(respConsulta[0].ERRORS, "ERROR");
                    } else {
                        MessageService.showMessageByCode("ERROR_CONSULTA_SITUACAONF");
                        console.error(respConsulta);
                    }
                } else {
                    MessageService.showMessageByCode("ERROR_CONSULTA_SITUACAONF");
                    console.error(respConsulta);
                }
            }, (error) => {
                MessageService.showMessageByCode("ERROR_CONSULTA_SITUACAONF");
                console.error(error);
            });
        });
    };
    
    this.gerarXml = function(widget) {
        var erro = false;

        widget.getCheckedRows().map((nota) => {
            if(nota.IDSTATUSNFE != 'C' && nota.IDSTATUSNFE != 'A') erro = true; 
        });
        
        if(erro) {
           ScreenService.showMessage("Só é possivel gerar o XML de notas com Status 'Aceita' ou 'Cancelada'");
        }
        else {
            General.callRestEngine("df#/gerarXmlNfe", "DataSet", widget.getCheckedRows()).then((response) => {
                window.open(window.serviceUrls.filter(function(x){return x.package == 'df';})[0].baseUrl+"/downloadArquivo?requestType=Row&row[DSARQUIVO]="+response.messages[0].message);
            });
        }
    };
    
    this.empresaUtlizaNFE = function(transmissaoNfeArgs){
        let widget =  transmissaoNfeArgs.owner.field.widget;
        let nmfilial = widget.getField("NMFILIAL");
        var cdempresa = this.getCurrentField('CDEMPRESA');
        General.callRestEngine("df_saida#/crud/paramempresa_saida", "FilterData", [{"name": "P_CDEMPRESA","operator": "=", "value": cdempresa}])
        .then((response) => {
            let utilizanfe = response.dataset.paramempresa_saida[0].IDEMPUTILIZANFE === 'S' ?  true : false;
            if (!utilizanfe){
                ScreenService.showMessage("Empresa informada não utiliza nota fiscal eletrônica");
                nmfilial.setValue("");
                nmfilial.reload();
            }
        });
    };
        
    this.fixZeroFill  = function (field) {
        var value = field.value();
        
        if (!value) {
            return;
        }
        
        //Remove pontos e vírgulas
        var newValue = value.replace('.', '').replace(',', '');
        
        //Realiza o zeroFill novamente
        var currentLength = newValue.toString().length;
        newValue = "0".repeat(field.maxlength - currentLength) + newValue;
        
        field.setValue(newValue);
    };
    
    this.checkInscricaoContaCorrente = function(field) {
        var inscricao = General.inscricaoFormat(field.value());
        
        if(inscricao.valid) {
            field.setValue(inscricao.value);
        } else {
            if(inscricao.tipo.cod == 'J') inscricao.tipo.msg = 'CNPJ'+" \""+inscricao.value+"\" inválido!";
            else if(inscricao.tipo.cod == 'F') inscricao.tipo.msg = 'CPF'+" \""+inscricao.value+"\" inválido!";
            else if(inscricao.tipo.cod == 'I') inscricao.tipo.msg = inscricao.tipo.desc+" \""+inscricao.value+"\" inválida!";
            
            if(inscricao.tipo.msg) ScreenService.showMessage(inscricao.tipo.msg, "warning");
        }
    };
    
    
    this.produtosOnEnter = function (widget) {
        var widgetTransmissao = widget.getParent();
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widgetTransmissao.currentRow.CDFILIAL
            },
            {
                "name": "NRLANCTONF",
                "operator": "=",
                "value": widgetTransmissao.currentRow.NRLANCTONF
            }
        ];
        widget.dataSourceFilter = filter;
        General.callRestEngine(widget.dataSource.name, "FilterData", widget.dataSourceFilter).then (response => {
            localStorageEngine.setDataSource(widget.dataSource.name, response.dataset.lancamento_nota_itens).then ( () => {
                widget.dataSource.reload();
            });
        });
    };
    
    this.produtosAfterMoveRow = function (widget) {
        var declaracaoWidget = widget.widgets.filter(widget => widget.name === 'declaracaoImportacao')[0];
        var adicoesWidget = widget.widgets.filter(widget => widget.name === 'adicoes')[0];
        this.declaracaoImportacaoOnEnter(declaracaoWidget).then (existeDeclaracao => {
            if (existeDeclaracao) {
                adicoesWidget.enabled = true;
            }
            else {
                adicoesWidget.enabled = false;
            }
            declaracaoWidget.activate();
        });
    };
    
    this.setCamposDeclaracaoImportacao = function (widget) {
        var fieldDSPAISADQUIRE =  widget.getField("DSPAISADQUIRE");
        var fieldDSPAISLOCEMBIMP = widget.getField("DSPAISLOCEMBIMP");
        var fieldNMESTADOADQUIRE = widget.getField("NMESTADOADQUIRE");
        var fieldNMUFLOCEMBIMP = widget.getField("NMUFLOCEMBIMP");
        var value;
        
        fieldNMESTADOADQUIRE.readOnly = false;
        fieldNMUFLOCEMBIMP.readOnly = false;
        
        fieldDSPAISADQUIRE.reload().then ( () => {
            value = fieldDSPAISADQUIRE.dataSource.data.filter(data => data.CDPAIS === widget.currentRow.CDPAISADQUIRE)[0];
            if (!value) {
                fieldDSPAISADQUIRE.setValue("");
                fieldNMESTADOADQUIRE.setValue("");
                fieldNMESTADOADQUIRE.readOnly = true;
                return;
            }
            fieldDSPAISADQUIRE.setValue(value.NMPAIS);
            fieldNMESTADOADQUIRE.dataSource.reload().then ( () => {
                value = fieldNMESTADOADQUIRE.dataSource.data.filter(data => data.SGESTADO === widget.currentRow.SGESTADOADQUIRE)[0];
                if (!value) {
                    fieldNMESTADOADQUIRE.setValue("");
                    return;
                }
                fieldNMESTADOADQUIRE.setValue(value.NMESTADO);
            });
        });
        
        fieldDSPAISLOCEMBIMP.reload().then ( () => {
            value = fieldDSPAISLOCEMBIMP.dataSource.data.filter(data => data.CDPAIS === widget.currentRow.CDPAISLOCEMBIMP)[0];
            if (!value) {
                fieldDSPAISLOCEMBIMP.setValue("");
                fieldNMUFLOCEMBIMP.setValue("");
                fieldNMUFLOCEMBIMP.readOnly = true;
                return;
            }
            fieldDSPAISLOCEMBIMP.setValue(value.NMPAIS);
            fieldNMUFLOCEMBIMP.dataSource.reload().then ( () => {
                value = fieldNMUFLOCEMBIMP.dataSource.data.filter(data => data.SGESTADO === widget.currentRow.SGUFLOCEMBIMP)[0];
                if (!value) {
                   fieldNMUFLOCEMBIMP.setValue("");
                   return;
                }
                fieldNMUFLOCEMBIMP.setValue(value.NMESTADO);
            });
        });
    };
    
    this.declaracaoImportacaoLoadData = function (widget) {
        if (widget.dataSource.data[0]) {
            delete widget.dataSource.data[0];
        }
        for (let key in widget.currentRow) {
            widget.currentRow[key] = "";
        }
        var promise = ZHPromise.defer(); 
        General.callRestEngine(widget.dataSource.name, "FilterData", widget.dataSourceFilter).then (response => {
            let dataset = response.dataset.decimpnfe[0];
            if (dataset) {
                localStorageEngine.setDataSource(widget.dataSource.name, dataset).then ( () => {
                    widget.dataSource.reload();
                    widget.currentRow = dataset;
                    widget.currentRow.__is_new = false;
                    this.setCamposDeclaracaoImportacao(widget);
                    promise.resolve(true);
                });
            }
            else {
                 localStorageEngine.remove(widget.dataSource.name).then (() => {
                     widget.getField("NMESTADOADQUIRE").readOnly = true;
                     widget.getField("NMUFLOCEMBIMP").readOnly = true;
                     widget.currentRow.__is_new = true;
                     widget.dataSourceFilter.forEach (filter => {
                         widget.currentRow[filter.name] = filter.value;
                     });
                     promise.resolve(false);
                 });
            }
        });
        return promise;
    };
    
    this.declaracaoImportacaoOnEnter = function (widget) {
        let idStatusNFErow = widget.getParent().getParent().currentRow.IDSTATUSNFE;
        var widgetProdutos = widget.getParent();
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widgetProdutos.currentRow.CDFILIAL
            },
            {
                "name": "NRLANCTONF",
                "operator": "=",
                "value": widgetProdutos.currentRow.NRLANCTONF
            },
            {
                "name": "NRSEQITPROD",
                "operator": "=",
                "value": widgetProdutos.currentRow.NRSEQITPROD
            }
        ];
        widget.dataSourceFilter = filter;
        
        if(idStatusNFErow == 'P' || idStatusNFErow == 'R')  widget.getAction('editar').isVisible = true;
        else                                                widget.getAction('editar').isVisible = false;
        
        return this.declaracaoImportacaoLoadData(widget);
    };
    
    this.paisOnChange = function (field) {
        var widget = field.widget;
        var pais = field.value();
        switch (field.name) {
            case "DSPAISADQUIRE": 
                widget.currentRow.SGESTADOADQUIRE = "";
                widget.currentRow.NMESTADOADQUIRE = "";
                if (pais) {
                    widget.getField("NMESTADOADQUIRE").readOnly = false;
                }
                else {
                    widget.getField("NMESTADOADQUIRE").readOnly = true;
                }
                break;
            case "DSPAISLOCEMBIMP": 
                widget.currentRow.SGUFLOCEMBIMP = "";
                widget.currentRow.NMUFLOCEMBIMP = "";
                if (pais) {
                    widget.getField("NMUFLOCEMBIMP").readOnly = false;
                }
                else {
                    widget.getField("NMUFLOCEMBIMP").readOnly = true;
                }
                break;
        }
    };
    
    this.saveDeclaracaoImportacao = function (widget) {
        if (!widget.isValid()) {
            return;
        }
        let isNewDI = widget.currentRow.__is_new ? true : false;
        var adicoesWidget = widget.getParent().widgets.filter(widget => widget.name === 'adicoes')[0];
        General.callRestEngine(widget.dataSource.name+'/save', "Row", widget.currentRow).then( () => {
            this.declaracaoImportacaoLoadData(widget);
            this.setCamposDeclaracaoImportacao(widget);
            widget.list();
            adicoesWidget.enabled = true;
            if(isNewDI) {
                adicoesWidget.activate().then( () => {
                    adicoesWidget.reload().then( () => {
                        adicoesWidget.dataSource.data = [];
                        this.adicoesNewRow(adicoesWidget, true);
                    });
                });
            }
        });
    };
    
    this.cancelarDeclaracaoImportacao = function (widget) {
        widget.list();
        this.declaracaoImportacaoLoadData(widget);
    };
    
    this.adicoesOnEnter = function (widget) {
        let idStatusNFErow = widget.getParent().getParent().currentRow.IDSTATUSNFE;
        var widgetProdutos = widget.getParent();
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widgetProdutos.currentRow.CDFILIAL
            },
            {
                "name": "NRLANCTONF",
                "operator": "=",
                "value": widgetProdutos.currentRow.NRLANCTONF
            },
            {
                "name": "NRSEQITPROD",
                "operator": "=",
                "value": widgetProdutos.currentRow.NRSEQITPROD
            }
        ];
        widget.dataSourceFilter = filter;
        
        if(idStatusNFErow == 'P' || idStatusNFErow == 'R') {
            widget.getAction('editar').isVisible = true;
            widget.getAction('novoServico').isVisible = true;
        }
        else {
            widget.getAction('editar').isVisible = false;
            widget.getAction('novoServico').isVisible = false;
        }
        
        General.callRestEngine(widget.dataSource.name, "FilterData", widget.dataSourceFilter).then (response => {
            let idStatusNFErow = widget.getParent().getParent().currentRow.IDSTATUSNFE;
            
            if(response.dataset.adicdecimpnfe.length <= 1 )          widget.getAction('excluir').isVisible = false;
            else if(idStatusNFErow == 'P' || idStatusNFErow == 'R')  widget.getAction('excluir').isVisible = true;
            else                                                     widget.getAction('excluir').isVisible = false;
            
            localStorageEngine.setDataSource(widget.dataSource.name, response.dataset.adicdecimpnfe).then ( () => {
                widget.dataSource.reload();
            });
        });
    };
    
    this.adicoesNewRow = function (widget, ocultaActCancel) {
        let NRSEQADICDEC = 0;
        
        widget.newRow();
        for (let field in widget.dataSourceFilter) {
            widget.currentRow[widget.dataSourceFilter[field].name] = widget.dataSourceFilter[field].value;
        }
        for (let index in widget.dataSource.data) {
            if (widget.dataSource.data[index].NRSEQADICDEC > NRSEQADICDEC) {
                NRSEQADICDEC = widget.dataSource.data[index].NRSEQADICDEC;
            }
        }
        NRSEQADICDEC++;
        widget.currentRow.NRSEQADICDEC = NRSEQADICDEC;
        
        if(ocultaActCancel) widget.getAction('cancelar').isVisible = false;
        else                widget.getAction('cancelar').isVisible = true;
    };
    
    this.saveAdicoes = function (widget) {
        if (!widget.isValid()) {
            return;
        }
        General.callRestEngine(widget.dataSource.name+'/save', "Row", widget.currentRow).then( () => {
            this.adicoesOnEnter(widget);
            ScreenService.successNotification("Registro salvo com sucesso!");
            widget.view();
            ScreenService.goBack();
        });
    };
    
    this.cancelarAdicoes = function (widget) {
        this.adicoesOnEnter(widget);
        widget.list();
        ScreenService.goBack();
    };
    
    this.excluirAdicoes = function (widget) {
        var self = this;
        ScreenService.confirmMessage('Deseja excluir o registro?', 'question', function (success) {
            General.callRestEngine(widget.dataSource.name+'/remove', "Row", widget.currentRow).then( () => {
                self.adicoesOnEnter(widget);
                ScreenService.successNotification("Registro excluído com sucesso!");
                widget.view();
                ScreenService.goBack();
            });
        }, requestError());
    };
    
    this.codExportMaskOnBlur = (field) => {
        field.setValue(field.value().replace(/[^\d]+/g, ""));
    };
    
    this.entSaidFilterOnChange = (wid) => {
        wid.getField('DTENTRSAID').label = wid.currentRow.IDENTRSAIDOP == 'E' ? 'Data Entrada' : 'Data Saída';
        wid.reload();
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfs49000_TransmissaoNfe_Controller", dfs49000_TransmissaoNfe_Controller);
});

// FILE: ./assets/modules/saida/assets/js/controllers/Dfs51000_InutilizacaoNotaFiscalEletronica_Controller.js
function Dfs51000_InutilizacaoNotaFiscalEletronica_Controller (ZhFilterPreferences, Dfs49000_TransmissaoNfe_Service, Dfs51000_InutilizacaoNotaFiscalEletronica_Service, ScreenService, templateManager, General, ZHPromise) {

    this.filtrar = function(args){
        var widget = args.owner;
        var filtroBforeApply = templateManager.container.getActiveWidget();
        var filtro;
          
        filtro = (filtroBforeApply.name == 'zhFilterPreferencesSelectConditions') ? filtroBforeApply.currentRow : widget.appliedFilter;
        this.montaConfig(args);
        
        //args.e.preventDefault();
    };
    
    this.transmissaoNFE = function(args){
        var widget = args.owner;
        this.setDefaultInutNFEFilter(widget).then(defaultFilter =>{
            ZhFilterPreferences.setOriginalFilter(widget.id, defaultFilter);
            ZhFilterPreferences.openFilterWidget(widget, defaultFilter);
            widget.currentRow=Util.clone(defaultFilter);
            this.controlaCampoUnidade(widget);
        });
    };
    
    this.setDefaultInutNFEFilter = function(widget) {
        var promise = ZHPromise.defer();
        var defaultFilter = Util.clone(widget.currentRow);
        defaultFilter.CDFILIAL=undefined;
        defaultFilter.CDEMPRESA=undefined;
        defaultFilter.NMRAZSOCEMPR=undefined;
        defaultFilter.NMFILIAL=undefined;
        
        if(sessionStorage.length>0){
           defaultFilter.CDFILIAL = (sessionStorage.getItem('CDFILIAL'));
           sessionStorage.removeItem('CDFILIAL');
           defaultFilter.NMFILIAL = (sessionStorage.getItem('NMFILIAL'));
           sessionStorage.removeItem('NMFILIAL');
           defaultFilter.NMRAZSOCEMPR = (sessionStorage.getItem('NMRAZSOCEMPR'));
           sessionStorage.removeItem('NMRAZSOCEMPR');
           defaultFilter.CDEMPRESA = (sessionStorage.getItem('CDEMPRESA'));
           sessionStorage.removeItem('CDEMPRESA');
           defaultFilter.CDINSCMUNI = (sessionStorage.getItem('CDINSCMUNI'));
           sessionStorage.removeItem('CDINSCMUNI');
           defaultFilter.CDINSCESTA = (sessionStorage.getItem('CDINSCESTA'));
           sessionStorage.removeItem('CDINSCESTA');
           defaultFilter.NRINSJURIEST = (sessionStorage.getItem('NRINSJURIEST'));
           sessionStorage.removeItem('NRINSJURIEST');
        }
        promise.resolve(defaultFilter);
        return promise;
    };
    
    /**
     * Seta Data Souce Stripe
     * @param widget
     */
    this.setStripeData = function(widget) {
        var currentRow = widget.appliedFilter.FILTER_ROW;
        var stripeWidget = templateManager.container.getWidget("stripe");

        stripeWidget.currentRow = {};
        stripeWidget.currentRow.NMFILIAL   = currentRow.CDFILIAL + ' - ' + currentRow.NMFILIAL;
        stripeWidget.currentRow.NMRAZSOCEMPR   = currentRow.CDEMPRESA + ' - ' + currentRow.NMRAZSOCEMPR;
        stripeWidget.currentRow.CDINSCESTA   = currentRow.CDINSCESTA;
        stripeWidget.currentRow.CDINSCMUNI   = currentRow.CDINSCMUNI;
    };

    this.recarregarCampo = function(nomeWidget,nomeCampo){
        var widget = templateManager.container.getWidget(nomeWidget);
        widget.getField(nomeCampo).reload();
    };

    this.getValueCampo = function(nomeCampo){
        var widget = templateManager.container.getWidget('gridNotasFiscais');
        //widget.getField(nomeCampo).value();
        return widget.currentRow.CDFILIAL ? widget.currentRow.CDFILIAL : null;
    };

    this.controlaCampoUnidade = function(widget){
        var campoSerie = widget.getField('DSSERIE');
        var empresa = widget.getField('NMRAZSOCEMPR');
        if(widget.currentRow.NMFILIAL){
            campoSerie.readOnly = false;
            empresa.readOnly = false;
            empresa.dataSourceFilter[0].value=widget.currentRow.CDFILIAL;
        }else{
            campoSerie.readOnly = true;
            empresa.readOnly = true;
        }
        widget.getField('NMFILIAL').reload();
        empresa.reload();
        campoSerie.reload();
        widget.currentRow.CDSERIE = '';
        widget.currentRow.NMRAZSOCEMPR = '';
        widget.currentRow.DSSERIE = '';
        widget.currentRow.CDSERIENFE = '';
        this.setFiltroSerieNF(widget);
    };
    
    this.setFiltroSerieNF = function(widget){
        widget.getField('DSSERIE').dataSourceFilter.forEach(function(filtro){
            switch(filtro.name){
                case 'CDEMPRESA': filtro.value = widget.currentRow.CDEMPRESA;
                    break;
                case 'CDINSCESTA': filtro.value = widget.currentRow.CDINSCESTA;
                    break;
                case 'CDINSCMUNI': filtro.value = widget.currentRow.CDINSCMUNI;
                    break;
            }
        });
    };

    this.inutilizarNf = function (widget){
        if(widget.getCheckedRows().length >0){
            var popup = templateManager.container.getWidget('widgetInutulizacao');
            if( this.validaInutilizacao(widget) ){
                popup.currentRow.JUSTIFICATIVA = null;
                ScreenService.openPopup(popup);
            }
        }else{
            ScreenService.showMessage('Faixa de numeração deve ser informada.');
        }
    };

    this.confirmarInutilizacao = function (args){
        var widget = args.owner.widget;
        var widgetNf = templateManager.container.getWidget('gridNotasFiscais');
        if(widget.isValid()){
            if(widget.currentRow.JUSTIFICATIVA.length >= 15){
                Dfs51000_InutilizacaoNotaFiscalEletronica_Service.aplicaJustificativa(widget.currentRow.JUSTIFICATIVA, widgetNf.getCheckedRows(),widgetNf.appliedFilter);
                Dfs51000_InutilizacaoNotaFiscalEletronica_Service.confirmarInutilizacao(widgetNf.getCheckedRows()).then(function(Success) {
                    ScreenService.closePopup(0);
                    widgetNf.reload();
                }.bind(this), function(callback){
                    if(callback.data){
                        ScreenService.showMessage(callback.data.error);
                    }
                });
            }else{
                ScreenService.showMessage('Justificativa deve conter no mínimo 15 caracteres.');
            }
        }
    };

    this.confirmarAgendamento = function (args){
        var widget = args.owner.widget;
        var widgetNf = templateManager.container.getWidget('gridNotasFiscais');
        if(widget.isValid()){
            ScreenService.confirmMessage('A consulta de numeração para inutilização pode demorar alguns minutos. Deseja continuar essa operação?', 'question', function (success) {
                Dfs51000_InutilizacaoNotaFiscalEletronica_Service.confirmarAgendamento(widget.currentRow)
                    .then(function(Success) {
                        ScreenService.closePopup(0);
                        widgetNf.reload();
                    }.bind(this), function(callback){
                        if(callback.data){
                            ScreenService.showMessage(callback.data.error);
                        }
                    }
                );
            });
        }
    };

    this.montaConfig = function(args){

        Dfs49000_TransmissaoNfe_Service.montaConfigTransmissao(args.owner.appliedFilter.FILTER_ROW).then((
            function(resp){
                var widgetNf = templateManager.container.getWidget('gridNotasFiscais');
                widgetNf.getAction('agendamento').isVisible = true;
            }).bind(this),
            function(error){
                // QUANDO OCORRER ALGUM PROBLEMA COM O CERTIFICADO NÃO PODE MOSTRAR OS DADOS
                var widgetNf = templateManager.container.getWidget('gridNotasFiscais');
                widgetNf.dataSource.data = null;
                var errorMessage = "Ocorreu um erro ao tentar realizar a operação.";
                if (error && error.data) {
                    errorMessage = error.data.error;
                }
                ScreenService.showMessage(errorMessage, "ERROR");
            }
        );
    };


    /*this.openFilter = function(widget){
        var filter = [{name: "CDFILIAL", "operator": "=", value: "FILIAL_LOGADA"}];
        General.callRestEngine("df#/crud/filial", "FilterData", filter).then((success) => {
            widget.currentRow.CDFILIAL      = success.dataset.filial[0].CDFILIAL;
            widget.currentRow.NMFILIAL      = success.dataset.filial[0].NMFILIAL;
            widget.currentRow.CDEMPRESA     = success.dataset.filial[0].CDEMPRESA;
            widget.currentRow.CDINSCESTA    = success.dataset.filial[0].CDINSCESTA;
            widget.currentRow.CDINSCMUNI    = success.dataset.filial[0].CDINSCMUNI;
            var filter = [{name: "CDEMPRESA", "operator": "=", value: widget.currentRow.CDEMPRESA}];
            this.controlaCampoUnidade(widget);
            widget.getField("DSSERIE").filterProperties.readOnly = false;
            General.callRestEngine("df#/crud/empresa_filial_filioper", "FilterData").then((success) => {
                widget.currentRow.NMRAZSOCEMPR = success.dataset.empresa[0].NMRAZSOCEMPR;
                var row = Util.clone(widget.currentRow);
                
                ZhFilterPreferences.setOriginalFilter(widget.id, row);
                ZhFilterPreferences.openFilterWidget(widget, row).then( () =>{
                    this.setFiltroSerieNF(templateManager.container.getActiveWidget());
                });
            },
            () => ZhFilterPreferences.openFilterWidget(widget));
           
        },
        () => ZhFilterPreferences.openFilterWidget(widget));
        
    };*/


    this.agendar = function (widget){
        var popup = templateManager.container.getWidget('widgetAgendamento');
        this.setarValoresAgendamento(popup);
        ScreenService.openPopup(popup);
    };

    this.setarValoresAgendamento = function(popup){
        let filtro = popup.getParent().appliedFilter;
        
        popup.currentRow.NMRAZSOCEMPR = filtro.FILTER_ROW.NMRAZSOCEMPR;
        popup.currentRow.CDEMPRESA    = filtro.FILTER_ROW.CDEMPRESA;
        popup.currentRow.NMFILIAL     = filtro.FILTER_ROW.NMFILIAL;
        popup.currentRow.CDFILIAL     = filtro.FILTER_ROW.CDFILIAL;
        popup.currentRow.CDINSCESTA   = filtro.FILTER_ROW.CDINSCESTA;
        popup.currentRow.CDINSCMUNI   = filtro.FILTER_ROW.CDINSCMUNI;
        popup.currentRow.CDSERIENFE   = filtro.FILTER_ROW.CDSERIENFE;
        popup.currentRow.NRINSJURIEST = filtro.FILTER_ROW.NRINSJURIEST;

        Dfs51000_InutilizacaoNotaFiscalEletronica_Service.getDataSourceConfigTransmissao()
            .then(function(config) {
                Dfs51000_InutilizacaoNotaFiscalEletronica_Service.buscaAgendamento(popup.currentRow, config)
                    .then(function(Success) {
                        popup.currentRow.DTHRULTIMACONSULTA =  Success.dataset.data.ultimaConsulta;
                        popup.currentRow.DTHRPROXIMACONSULTA =  Success.dataset.data.proximaConsulta;
                        popup.currentRow.IDSITUAGENDA =  Success.dataset.data.situacaoAgendamento;
                        popup.redraw();
                    }.bind(this),function(callback){
                        if(callback.data){
                            ScreenService.showMessage(callback.data.error);
                        }
                    }
                );
            }.bind(this), function(callback){
                if(callback.data){
                    ScreenService.showMessage(callback.data.error);
                }
            }
        );
    };


    // fazer a validação para não deixa inutilizar uma serie ja homologada
    this.validaInutilizacao = function(widget){
        var rows = widget.getCheckedRows();
        var rowsValidadas= [];
        var msg = [];
        rows.forEach(function(row) {
            if(row.IDSTATUS=='A'){
                row.__isSelected = false;
                var numeracao ="'"+row.INTERVALOINI+"'"+' - '+"'"+row.INTERVALOFIN+"'"+'<br/>';
                msg.push(numeracao);
            }else{
                rowsValidadas.push(row);
            }
        });
        widget.dataSource.updateCheckedRows();
        if(msg.length>0){
            var mensagem ='';
            msg.forEach(function(msg) {
                mensagem = mensagem +msg;
            });
            ScreenService.showMessage('A(s) seguinte(s) faixa(s) de numeração já foi(ram) inutilizada(s):'+'<br/>'+mensagem);
        }
        if(rowsValidadas.length==0){
           return false;
        }else{
            return true;
        }
    };

}

Configuration(function (ContextRegister) {
    ContextRegister.register("Dfs51000_InutilizacaoNotaFiscalEletronica_Controller", Dfs51000_InutilizacaoNotaFiscalEletronica_Controller);
});


// FILE: ./assets/modules/saida/assets/js/controllers/Dfs61000_Consultalote_Controller.js
function Dfs61000_Consultalote_Controller(ScreenService, templateManager, restEngine, ZHPromise) {

    var self = this;

    this.setarFiltro = function() {
        var widget = templateManager.container.getWidget("wdgListaLoteRps");
        widget.activate();
        var filterRow = templateManager.containers["df_saida#dfs61000_transmissao_nfse"].widgets[0].appliedFilter.FILTER_ROW;
        widget.dataSourceFilter[0].value = filterRow.CDEMPRESA;
        widget.dataSourceFilter[1].value = filterRow.CDINSCESTA;
        widget.dataSourceFilter[2].value = filterRow.CDINSCMUNI;
        widget.dataSource.data = null;
    };

    this.consultarLotes = function(rows, transmissao, widget) {
        if(rows) {
            var valida = true;
            var lotesErro = null;
            rows.forEach(function(linha) {
                if (linha.IDSTATUSLOTERPS == 4) {
                    valida = false;
                    return;
                }
            });
    
            if (valida == false) {
                ScreenService.showMessage("Um ou mais lotes marcados já foram processados.");
            }
            else {
                var options = {
                    "requestType": "DataSet",
                    "dataset": rows
                };
    
                restEngine.post("df_saida#/consultarLotes", options,
                    function (success) {
                        if(success.dataset)
                            if(success.dataset.lotesErro) lotesErro = success.dataset.lotesErro;
                        if (success.messages != undefined) {
                            ScreenService.showMessage(success.messages[0].message);
                            if (!transmissao) widget.reload();
                            if (success.dataset && success.dataset.lotesAceitos[0]) {
                                var lotes = success.dataset.lotesAceitos;
                                lotes[0].notaserro = [];
                                self.aprovarLotesAceitos(success.dataset.lotesAceitos, success.messages[0].message).then(function(erro){
                                    if(erro && !erro.erro) {
                                        if(erro) {
                                            let nrNotasErros = erro.map((e) => e.NRNOTAFISC);
                                            lotes[0].notaserro = [...lotes[0].notaserro, ...nrNotasErros];
                                            self.gerarRelatorio(lotes, transmissao).then(function(success) {
                                                if(transmissao) ScreenService.restoreDefaultLoadingMessage();
                                                if(lotesErro)
                                                    this.openPopupErros([...erro, ...lotesErro]);
                                                widget.reload();
                                            }.bind(this));
                                            if(transmissao) ScreenService.restoreDefaultLoadingMessage();
                                        }
                                        this.openPopupErros(erro);
                                    }
                                    else if(success.dataset && success.dataset.lotesErro) {
                                        this.openPopupErros(success.dataset.lotesErro);
                                    }
                                    else {
                                        if(transmissao) ScreenService.restoreDefaultLoadingMessage();
                                        self.gerarRelatorio(success.dataset.lotesAceitos, transmissao).then(function(success){
                                            widget.reload();
                                        });
                                    }
                                }.bind(this));
                            }
                           else {
                                if(success.dataset) 
                                    if(success.dataset.lotesErro) this.openPopupErros(success.dataset.lotesErro);
                                if(transmissao) ScreenService.restoreDefaultLoadingMessage();
                                ScreenService.hideLoader();
                            }
                        }
                        else {
                            if(success.dataset) {
                                if(success.dataset.lotesErro) this.openPopupErros(success.dataset.lotesErro);
                            }
                            if(transmissao) {
                                ScreenService.restoreDefaultLoadingMessage();
                            }
                            ScreenService.hideLoader();
                        }
                    }.bind(this), undefined,
                    function (error) {
                        if(transmissao) {
                            ScreenService.restoreDefaultLoadingMessage();
                            widget.reload();
                        }
                        ScreenService.hideLoader();
                        ScreenService.showMessage("Ocorreu um erro inesperado.");
                        console.log(error);
                    }
                );
            }
        }
        else {
            if(transmissao) ScreenService.restoreDefaultLoadingMessage();
            if(!transmissao) ScreenService.showMessage('Nenhum lote foi selecionado.');
        }
    };

    this.voltarParaTransmissao = function() {
        ScreenService.goBack();
       templateManager.containers["df_saida#dfs61000_transmissao_nfse"].getWidget("wdgTransmissaoNfse").reload();
    };

    this.gerarRelatorio = function(lotes, transmissao) {
        var promise = ZHPromise.defer();

        var params = {
            "requestType": "DataSet",
            "dataset": lotes
        };
        restEngine.post("df_saida#/enviarEspelhoNfse", params/*,
        null, null, null, null, 3000000*/).then(function(response) {
            promise.resolve();
        }).catch(function(error) {
            ScreenService.showMessage(error.data.error);
        });
        if(transmissao) ScreenService.hideLoader();
        return promise;
    };

    this.aprovarLotesAceitos = function(lotes, message) {
        var params = {
            "requestType": "DataSet",
            "dataset": lotes
        };
        return restEngine.post("df_saida#/aprovarLotesAceitos", params).then(function(response) {
            if(response.dataset) {
                if(response.dataset.naoAprovaAutomatico) {
                    return {erro: true};
                }
                //message = message + '<br/><br/>Não foi possível realizar a aprovação automática das notas:';
                else if(response.dataset.errosAprovacao) {
                    if(response.dataset.errosAprovacao[0]) return response.dataset.errosAprovacao;
                }
                else if(response.data) {
                    return {erro: true};
                }
            }
        }).catch(function(error) {
            ScreenService.showMessage(error.data.error);
        });
    };
    
    this.openPopupErros = function(erros) {
        if(erros[0]) {
            var widgetErros = templateManager.container.getWidget('WidgetErros');
            widgetErros.dataSource.data = erros;
            widgetErros.isVisible = true;
            ScreenService.openPopup(widgetErros);
            widgetErros.activate();
        }
    };
    
    this.closeErrosAprovacao = function(widget) {
        widget.isVisible = false;
        widget.dataSource.data = [];
        ScreenService.closePopup(widget);
        widget.getParent().activate();
        widget.getParent().unSelectAll();
        widget.getParent().reload();
    };

}

Configuration(function (ContextRegister) {
    ContextRegister.register("Dfs61000_Consultalote_Controller", Dfs61000_Consultalote_Controller);
});


// FILE: ./assets/modules/saida/assets/js/controllers/Dfs61000_ErrosTransmissaoNfse_Controller.js
function Dfs61000_ErrosTransmissaoNfse_Controller(ScreenService, templateManager, restEngine) {
    
    this.setCampoErro = function() {
        var xml = getLocalVar("TRANSMISSAO_ERRO_XML");
        var widget = templateManager.container.getWidget("wdgErroTransmissaoNfse");
        widget.currentRow.XML = xml;
        widget.currentRow.error = getLocalVar("TRANSMISSAO_ERRO");
    };
    
    this.voltar = function() {
        setLocalVar("TRANSMISSAO_ERRO_XML", "");
        ScreenService.goBack();
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Dfs61000_ErrosTransmissaoNfse_Controller", Dfs61000_ErrosTransmissaoNfse_Controller);
});


// FILE: ./assets/modules/saida/assets/js/controllers/Dfs61000_SubstituicaoNfse_Controller.js
function Dfs61000_SubstituicaoNfse_Controller(General, ScreenService, templateManager, restEngine) {

    var self = this;

    this.actNovoSubstituicao = function(args) {
        var widget = args.owner.widget;
        widget.newRow();
        widget.template = "/widget/form.html";
        widget.currentRow.__is_new = true;
        
        var row = templateManager.containers["df_saida#dfs61000_transmissao_nfse"].getWidget("wdgTransmissaoNfse").currentRow;
        
        widget.currentRow.P_CDFILIAL        = row.CDFILIAL;
        widget.currentRow.P_NRLANCTONF      = row.NRLANCTONF;
        widget.currentRow.P_CDINSCESTA      = row.CDINSCESTA;
        widget.currentRow.P_CDINSCMUNI      = row.CDINSCMUNI;
        widget.currentRow.P_NRINSJURIEST    = row.NRINSJURIEST;
        
        //this.defineParametrosBuscaNfse();
    };
    
    this.actEditaSubstituicao = function(args) {
        General.widgetEdit(args.owner.widget);
        args.owner.widget.template = args.owner.widget.templateToForm;
    };

    this.actCancelaSubstituicao = function(args){
        //General.widgetEdit(args.owner.widget);
        args.owner.widget.list();
        args.owner.widget.template = "/widget/form-view.html";
        args.owner.widget.reload();
    };
    
    this.validaActionVisivel = function(){
        var widget = templateManager.container.getWidget("wdgSubstituicaoNfse");
        var dados = widget.dataSource.data;
        
        if(dados.length > 0){
            widget.currentRow = dados[0];
            
            widget.getAction('novo').isVisible      = false;
            widget.getAction('editar').isVisible    = true;
            widget.getAction('excluir').isVisible    = true;
        }else{
            widget.currentRow = {};
            
            widget.getAction('novo').isVisible      = true;
            widget.getAction('editar').isVisible    = false;
            widget.getAction('excluir').isVisible    = false;
        }
    };
    
    this.voltarParaTransmissao = function() {
        ScreenService.goBack();
        templateManager.containers["df_saida#dfs61000_transmissao_nfse"].getWidget("wdgTransmissaoNfse").reload();
    };

    this.defineParametros = function() {
        var row = templateManager.containers["df_saida#dfs61000_transmissao_nfse"].getWidget("wdgTransmissaoNfse").currentRow;
        var widget = templateManager.containers["df_saida#dfs61000_substituicao_nfse"].getWidget("wdgSubstituicaoNfse");
        
        if(widget.currentRow.__is_new !== true){
            widget.dataSourceFilter[0].value = row.CDFILIAL;
            widget.dataSourceFilter[1].value = row.NRLANCTONF;
            widget.reload([],false);
        }

    };
    
    this.defineParametrosBuscaNfse = function() {
        var row = templateManager.containers["df_saida#dfs61000_transmissao_nfse"].getWidget("wdgTransmissaoNfse").currentRow;
        var widget = templateManager.containers["df_saida#dfs61000_substituicao_nfse"].getWidget("wdgSubstituicaoNfse");
        
        //if(widget.dataSource.data.length > 0){
            widget.currentRow.P_CDFILIAL        = row.CDFILIAL;
            widget.currentRow.P_NRLANCTONF      = row.NRLANCTONF;
            widget.currentRow.P_CDINSCESTA      = row.CDINSCESTA;
            widget.currentRow.P_CDINSCMUNI      = row.CDINSCMUNI;
            widget.currentRow.P_NRINSJURIEST    = row.NRINSJURIEST; 
        //}
    };
    
    this.setFiliLancOnChange = function(args){
        var widget = args.owner.owner.widget;
        widget.currentRow.CDFILIAL = widget.currentRow.P_CDFILIAL;
        widget.currentRow.NRLANCTONF = widget.currentRow.P_NRLANCTONF;
    };
    
    this.actSalvarSubstituicao = function(args){
        General.saveRow(args);
        args.owner.widget.list();
        args.owner.widget.template = "/widget/form-view.html";
        args.owner.widget.reload();
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Dfs61000_SubstituicaoNfse_Controller", Dfs61000_SubstituicaoNfse_Controller);
});

// FILE: ./assets/modules/saida/assets/js/controllers/Dfs61000_TransmissaoNfse_Controller.js
function Dfs61000_TransmissaoNfse_Controller(Dfs61000_TransmissaoNfse_Service, MessageService, ScreenService, templateManager, restEngine, ZHPromise, General, Dfs61000_Consultalote_Controller, ZhFilterPreferences) {
    
    this.setFiltroOnEnterOnce = function(widget) {
        
        var filter = [{"name": "CDFILIAL", "operator": "=", "value": 'FILIAL_LOGADA'}];
        
        General.callRestEngine("df#/crud/filial", "FilterData", filter).then((success) => {
            var filial = success.dataset.filial[0];
            widget.currentRow.CDFILIEMIT = [filial.CDFILIAL];
            widget.currentRow.NMFILIEMIT = [filial.NMFILIAL];
            widget.currentRow.CDEMPRESA  = [filial.CDEMPRESA];
            widget.currentRow.CDINSCESTA = [filial.CDINSCESTA];
            widget.currentRow.CDINSCMUNI = [filial.CDINSCMUNI];
            this.filtraFilhosEmpresa(widget);
            this.setData(widget);
            templateManager.container.getWidget('wdgTransmissaoNfse').dataSourceFilter[0].value = widget.currentRow.CDEMPRESA;
            var row = Util.clone(widget.currentRow);
            ZhFilterPreferences.setOriginalFilter(widget.id, row);
            ZhFilterPreferences.openFilterWidget(widget, row);
            widget.currentRow = {};
        },
        (error) => {
            ZhFilterPreferences.openFilterWidget(widget);
        });
    };

    this.setData = function(widget) {
        var date = new Date();
        var primeiroDia = new Date(date.getFullYear(), date.getMonth(), 1);
        var ultimoDia = new Date(date.getFullYear(), date.getMonth() + 1, 0);
        primeiroDia = formatarData(primeiroDia);
        ultimoDia = formatarData(ultimoDia);
        widget.currentRow.DTENTRSAID = {start : primeiroDia, end : ultimoDia};
    };
    
    function formatarData(data) {
        var d = new Date(data),
        mes = '' + (d.getMonth() + 1),
        dia = '' + d.getDate(),
        ano = d.getFullYear();
    
        if (mes.length < 2) mes = '0' + mes;
        if (dia.length < 2) dia = '0' + dia;
    
        return [dia, mes, ano].join('/');
    }

    this.enviarLoteRps = function(widget) {
        let checkedRows      = widget.getCheckedRows();
        let valida    = false;
        let erros     = [];
        let sucessos  = [];
        let erro;
        let xmlErros  = '';
        let rowsEnviarLote = [];
        let rowProcessada = false;
        let rowsConsultarLote = [];

        checkedRows.forEach(function(checkedRow) {
            if (checkedRow.IDTRANSMISRPS != 4 && checkedRow.IDTRANSMISRPS != 5) {
                if(checkedRow.IDTRANSMISRPS != 2) {
                    rowsEnviarLote.push(checkedRow);
                }
                rowsConsultarLote.push(checkedRow);
                valida = true;
            }
            else rowProcessada = true;
        });
        
        if (valida != false) {
            Dfs61000_TransmissaoNfse_Service.montarLotes(rowsEnviarLote, 50).then((success) => {
                let lotes = success, chain = ZHPromise.when();
                lotes.forEach(lote => {
                    const enviaTrataLote = lote => Dfs61000_TransmissaoNfse_Service.enviarLoteRps(lote).then(
                        callback => console.log(callback),
                        error => {
                            if (error.data.messages) {
                                erro = error.data.error;
                                xmlErros += '\n'+error.data.messages[0].message;
                            }
                            else if (error.data.error && error.data.error !== 'Não foi possivel enviar o lote. Corriga os erros e tente novamente.') {
                                erros += error.data.error + ' ';
                            }
                            else {
                                erros = [...erros, ...error.data.dataset.erros];
                            }
                        }
                    );
                    chain = chain.then(() => enviaTrataLote(lote));
                });
                chain.then(() => {
                    console.log(sucessos);
                    let existeErro = false;
                    if (typeof erros === 'string') {
                        ScreenService.showMessage(erros);
                        ScreenService.hideLoader();
                        existeErro = true;
                    }
                    if(!Util.isEmptyOrBlank(erros)) {
                        this.openPopupErros(erros);
                        widget.reload();
                        existeErro = true;
                    }
                    if(!Util.isEmptyOrBlank(xmlErros)) {
                        existeErro = true;
                        ScreenService.confirmMessage("Ocorreu um erro ao enviar o(s) lote(s). <br>Deseja ver o XML de envio?", "warning", function(){
                            widget.unSelectAll();
                            setLocalVar("TRANSMISSAO_ERRO_XML", xmlErros);
                            setLocalVar("TRANSMISSAO_ERRO", erro);
                            ScreenService.openWindow("df_saida#dfs61000_erros_transmissao_nfse");
                        }, null);
                        widget.reload();
                    }
                    else if(!existeErro) {
                        widget.reload().then(() => {
                            let mensagem = (rowProcessada) ? "Processo realizado com sucesso. Notas já enviadas foram ignoradas." : "Processo realizado com sucesso.";
                            let startTime = moment.now();
                            ScreenService.confirmMessage(ScreenService.i18n(mensagem.concat('Deseja consultar o(s) lote(s)?')), 'question', (success) => {
                    		    let endTime = moment.now();
                    		    let time;
                    		    if(endTime - startTime <= 12000) time = 12000 - (endTime - startTime);
                    		    else time = 1;
                    		    if(!rowsEnviarLote[0]) time = 1;
                    			ScreenService.changeLoadingMessage("Consultando o lote");
                    			ScreenService.showLoader();
                    			this.callConsultaLote(rowsConsultarLote, time, widget);
                    		});
                        });
                    }
                    widget.unSelectAll();
                });
            });
        }  
        else {
            ScreenService.showMessage("Todas as notas marcadas já foram enviadas.");
        }
    };
    
    this.callConsultaLote = function(rows, timeOut, widget) {
        setTimeout(function() {
            General.callRestEngine('df_saida#/getLoteNotas', 'DataSet', [rows]).then(function(response) {
                Dfs61000_Consultalote_Controller.consultarLotes(response.dataset.getLoteNotas, true, widget);
                widget.reload();
            },
            function(error) {
                ScreenService.hideLoader();
                widget.reload();
            });
        }, timeOut);
    };


    this.abrirWidgetCancelamento = function(widget) {
        var rows = widget.getCheckedRows();
        var valida = true;

        rows.forEach(function (linha) {
            if (linha.IDTRANSMISRPS != 4) {
                valida = false;
                return;
            }
        });

        if (valida == false) {
            ScreenService.showMessage("Uma das notas marcadas ainda não foi processada.");
        }
        else {
            var options = {
                "requestType": "Row",
                "row": {
                    "CDEMPRESA": widget.appliedFilter.FILTER_ROW.CDEMPRESA,
                    "CDINSCMUNI": widget.appliedFilter.FILTER_ROW.CDINSCMUNI
                }
            };
            
            restEngine.post("df_saida#/getEstadoMunicipioDeInscricaoMunicipal", options, function(success) {
                if (success.dataset) {
                    widget.widgets[0].isVisible = true;
                    ScreenService.openSwipe(widget.widgets[0]);
                    var dataSource = null;
                    var fieldMotivo = null;
                    
                    if (success.dataset.data.SGESTADO === "MG" && (success.dataset.data.CDMUNICIPIO === "4123" || success.dataset.data.CDMUNICIPIO === "4371")) {
                        dataSource = {
                            "data": [
                                {"VALUE": "1", "DESCRIPTION": "Erro na emissão"},
                                {"VALUE": "2", "DESCRIPTION": "Serviço não concluído"}
                            ]
                        };
                        
                        fieldMotivo = widget.widgets[0].getField("MOTIVO");
                        fieldMotivo.template = "field/select-native.html";
                        fieldMotivo.dataSource = dataSource;
                        fieldMotivo.valueField = "VALUE";
                        fieldMotivo.descriptionField = "DESCRIPTION";
                    }
                    else if (success.dataset.data.SGESTADO === "PR" && success.dataset.data.CDMUNICIPIO === "7777") {
                        dataSource = {
                            "data": [
                                {"VALUE": "1", "DESCRIPTION": "Erro na emissão"},
                                {"VALUE": "2", "DESCRIPTION": "Serviço não prestado"},
                                {"VALUE": "4", "DESCRIPTION": "Duplicidade da nota"}
                            ]
                        };
                        
                        fieldMotivo = widget.widgets[0].getField("MOTIVO");
                        fieldMotivo.template = "field/select-native.html";
                        fieldMotivo.dataSource = dataSource;
                        fieldMotivo.valueField = "VALUE";
                        fieldMotivo.descriptionField = "DESCRIPTION";
                    }
                    else if (success.dataset.data.SGESTADO === "SP" && success.dataset.data.CDMUNICIPIO === "6291") {
                        fieldMotivo = widget.widgets[0].getField("MOTIVO");
                        fieldMotivo.template = "field/text-edit.html";
                        fieldMotivo.dataSource = null;
                        fieldMotivo.valueField = null;
                        fieldMotivo.descriptionField = null;
                    }
                }
                else {
                    ScreenService.showMessage("Funções de NFS-e não habilitada para esse município.");
                }
                
            }, undefined, function(error) {
                console.log(error);
            });
        }
    };

    this.nfseAprovadas = function(rows){
        let nfse = [];
        rows.forEach(
            function(linha) {
                if(linha.IDSTATUS == "A"){
                    nfse.push(linha.NRNOTAFISC);
                }
            }
        );
        return nfse;
    };

    this.cancelarNotas = function(widget) {
        if (widget.isValid()) {
            var motivo = widget.currentRow.MOTIVO;
            var wdgTransmissao = templateManager.container.getWidget("wdgTransmissaoNfse");
            var rows = wdgTransmissao.getCheckedRows();
            var nfseAprovadas = this.nfseAprovadas(widget.getParent().getCheckedRows());
            
            if(nfseAprovadas.length == 0 || nfseAprovadas == null){
                
                rows.forEach(function(linha){
                    linha.MOTIVO = motivo;
                });
                
                var options = {
                    "requestType": "DataSet",
                    "dataset": rows
                };
                restEngine.post("df_saida#/cancelarNotas", options,
                    function (success) {
                        if (success.messages != undefined) {
                            ScreenService.showMessage(success.messages[0].message);
                            ScreenService.closeSwipe();
                            widget.currentRow.MOTIVO = null;
                            widget.isVisible = false;
                            wdgTransmissao.reload();
                        }
                        else {
                            ScreenService.showMessage(success);
                            console.log(success);
                        }
                    }, undefined,
                    function (error) {
                        ScreenService.showMessage("Ocorreu um erro inesperado.");
                        console.log(error);
                    }
                );
            } else{
                var textoMsg = "NAO_FOI_POSSIVEL_EFETUAR";
                var notas;
                nfseAprovadas.forEach(
                    function(numero){
                        notas += numero + ", ";
                    }
                );
                MessageService.showMessageByCode(textoMsg, {"NOTAS": notas.slice(0, -2)});
                ScreenService.closeSwipe();
            }
        }
    };

    this.consultaLotes = function() {
        ScreenService.openWindow("df_saida#dfs61000_consulta_lote");
    };

    this.filtraFilhosEmpresa = function(widget) {
        if(widget.currentRow.CDFILIEMIT.length != 0) {
            General.callRestEngine("df#/crud/filial", "FilterData", [{name: "CDFILIAL", operator: "IN", value: widget.currentRow.CDFILIEMIT}])
                .then((sucesso) => {
                    var filiais = sucesso.dataset.filial;
        
                    var cdempresa = filiais[0].CDEMPRESA;
                    var mostraMensagem = true;
                    var erroEmpresa = false;
                    if(typeof empresas != "string") {
                        filiais.forEach((filial) => {
                            if(filial.CDEMPRESA !== cdempresa) erroEmpresa = true; 
                        });
                    }
                    else if(widget.currentRow.NMFILIEMIT.length == 0) {
                        erroEmpresa = true;
                        mostraMensagem = false;
                    }
                    
                    if(erroEmpresa == true) {
                        if(mostraMensagem) ScreenService.showMessage("Unidades devem estar relacionadas a mesma empresa!");
                        widget.currentRow.CDFILIEMIT = [];
                        widget.currentRow.NMFILIEMIT = [];
                        widget.currentRow.CDEMPRESA  = [];
                        widget.currentRow.CDINSCESTA = [];
                        widget.currentRow.CDINSCMUNI = [];
                        widget.currentRow.CDSERIE    = '';
                    }
                    else {
                        widget.currentRow.CDEMPRESA  = filiais[0].CDEMPRESA;
                        widget.currentRow.CDINSCESTA = filiais[0].CDINSCESTA;
                        widget.currentRow.CDINSCMUNI = filiais[0].CDINSCMUNI;
                        
                        
                        var cdinscesta = filiais[0].CDINSCESTA;
                        var cdinscmuni = filiais[0].CDINSCMUNI;
                
                        var campo = widget.getField("CDINSCESTA");
                        campo.dataSourceFilter[0].value = cdempresa;
                        //campo.clearValue();
                        campo.reload();
                
                        campo = widget.getField("CDINSCMUNI");
                        campo.dataSourceFilter[0].value = cdempresa;
                        //campo.clearValue();
                        campo.reload();
                
                        campo = widget.getField("CDSERIE");
                        campo.dataSourceFilter[0].value = cdempresa;
                        campo.dataSourceFilter[1].value = cdinscesta;
                        campo.dataSourceFilter[2].value = cdinscmuni;
                        campo.clearValue();
                        campo.reload();
                    }
            });
        }
        else {
            widget.currentRow.CDFILIEMIT = [];
            widget.currentRow.NMFILIEMIT = [];
            widget.currentRow.CDEMPRESA  = [];
            widget.currentRow.CDINSCESTA = [];
            widget.currentRow.CDINSCMUNI = [];
            widget.currentRow.CDSERIE    = '';
        }
        
        
    };
    
    this.setFiltroEmpresa = function(args) {
        var widget = args.owner;
        args.e.preventDefault();
        var dataSourceFilter = [];
        dataSourceFilter = widget.dataSourceFilter.filter((data) =>  {
            if(data.name != 'NMFILIEMIT' && data.name != 'CDFILIEMIT' && data.value !== null) return data;
            
        });
        var cdfiliemit = widget.appliedFilter.FILTER_ROW.CDFILIEMIT;
        dataSourceFilter.push({name: "CDFILIEMIT", operator: "IN", value: cdfiliemit});
        widget.dataSource.clearCheckedRows();
        var filterCurrentRow = widget.appliedFilter.FILTER_ROW;
        widget.dataSourceFilter[0].value = filterCurrentRow.CDEMPRESA;
        widget.reload();
    };

    this.filtraFilhosInscricaoEstadual = function(widget) {
        var cdinscesta = widget.currentRow.CDINSCESTA;
        var campo = widget.getField("CDSERIE");
        campo.dataSourceFilter[1].value = cdinscesta;
        campo.clearValue();
        campo.reload();
    };

    this.filtraFilhosInscricaoMunicipal = function(widget) {
        var cdinscmuni = widget.currentRow.CDINSCMUNI;
        var campo = widget.getField("CDSERIE");
        campo.dataSourceFilter[2].value = cdinscmuni;
        campo.clearValue();
        campo.reload();
    };
    
    this.enviarEmail = function(widget) {
        var rows = widget.getCheckedRows();
        
        var valida = true;
    
        rows.forEach(function(linha) {
            if(!linha.NRSEQLOTERPS || linha.IDTRANSMISRPS !== '4') {
                valida = false;
            }
        });
        
        if(!valida)ScreenService.showMessage('Uma ou mais notas selecionadas não foram processadas ou foram processadas com erro!');
        else {
            var options = {
                    "requestType": "DataSet",
                    "dataset": rows
            };
            restEngine.post("df_saida#/enviaEmailNota", options,
                function(success) {
                    if (success.messages && !success.errors) {
                        ScreenService.successNotification('E-mail enviado com sucesso.');
                        widget.unSelectAll();
                        widget.reload();
                        ScreenService.closePopup();
                    }
                    else  if(success.errors) {
                        ScreenService.errorNotification("Ocorreu um erro ao realizar o envio do e-mail. Contate o administrador do sistema.");
                    }
                    else {
                        ScreenService.showMessage(success);
                        widget.unSelectAll();
                    }
                }, undefined,
                function(error) {
                    if(typeof error.data.error[0] !== 'string') this.openPopupErros(error.data.error);
                    else ScreenService.showMessage(error.data.error);
                }.bind(this)
            );
        }
    };
    
    this.openSubstituicao = function() {
        ScreenService.openWindow("df_saida#dfs61000_substituicao_nfse");  
    };
    
    this.openPopupErros = function(erros) {
        var widgetErros = templateManager.container.getWidget('WidgetErros');
        widgetErros.dataSource.data = erros;
        widgetErros.isVisible = true;
        ScreenService.openPopup(widgetErros);
        widgetErros.activate();
    };
    
    this.closeErros = function(widget) {
        widget.isVisible = false;
        widget.dataSource.data = [];
        ScreenService.closePopup(widget);
        widget.getParent().activate();
        widget.getParent().unSelectAll();
        widget.getParent().reload();
        ScreenService.closePopup();
    };
    
    this.openPopupNomeZipXML = function(widget) {
        var widgetNomeZip = templateManager.container.getWidget('WidgetNomeZip');
        widgetNomeZip.isVisible = true;
        ScreenService.openPopup(widgetNomeZip);
        widgetNomeZip.activate();
    };
    
    this.downloadXML = function(widget, row) {
        var widgetNomeZip = templateManager.container.getWidget('WidgetNomeZip');
        var rows = widget.getCheckedRows();
        if(row.NOMEZIP) {
            if(validaNomeArquivo(row.NOMEZIP)) {
                rows[0].nomezip = row.NOMEZIP;
                var valida = true;
        
                rows.forEach(function(linha) {
                    if(!linha.NRSEQLOTERPS || (linha.IDTRANSMISRPS !== '4' && linha.IDTRANSMISRPS !== '2')) {
                        valida = false;
                    }
                });
                
                if(valida === false) ScreenService.showMessage('Uma ou mais notas selecionadas não foram processadas.');
                else {
                    var options = {
                            "requestType": "DataSet",
                            "dataset": rows
                    };
                    restEngine.post("df_saida#/downloadXMLNfse", options,
                        function(success) {
                            this.downloadArquivo(success.dataset.nomeArquivo).then(function(){
                                widgetNomeZip.isVisible  = false;
                                widgetNomeZip.currentRow = {};
                                widget.activate();
                                widget.unSelectAll();
                                widget.reload();
                                ScreenService.goBack();
                                ScreenService.closePopup();
                                ScreenService.closePopup();
                            });
                            
                        }.bind(this), undefined,
                        function(error) {
                            if(typeof error.data.error[0] !== 'string') this.openPopupErros(error.data.error);
                            else ScreenService.showMessage(error.data.error);
                        }.bind(this)
                    );
                }
            }
            else ScreenService.showMessage('Nome do arquivo não pode conter os caracteres "<", "%", "|", "\\", "<", ">", "?", ":"');
        }
        else ScreenService.showMessage('Escolha um nome para o arquivo.');
    };
    
    this.downloadArquivo = function(fileName, message) {
        // Inicializa uma promise.
        var deferred = ZHPromise.defer();
        window.open(window.serviceUrls.filter(function(x){return x.package == 'df';})[0].baseUrl+"/downloadArquivo?requestType=Row&row[DSARQUIVO]="+fileName);

        // Finaliza a promise com sucesso.
        deferred.resolve();
       
        return deferred;
    };
    
    this.visualizarEspelhoNFSe = function(args){
        var nota = args.owner.widget.currentRow;

        if(nota.IDSTATUS == 'A'){
            Dfs61000_TransmissaoNfse_Service.visualizarNFSe(nota).then(function(result){
                var fileUri = result.dataset.arquivo.DIRETORIO;
                this.downloadArquivo(fileUri);
                
            }.bind(this), function(error) {
                ScreenService.showMessage(error.data.error);
            }.bind(this));
        }
        else{
            ScreenService.showMessage("Esta nota não foi aprovada.");
        }
    };
    
     this.downloadArquivo = function(nmArquivo){
        window.open(window.serviceUrls.filter(function(x){return x.package == 'df';})[0].baseUrl+"/downloadArquivo?requestType=Row&row[DSARQUIVO]=" + nmArquivo);
    };
    
    this.voltarNomeZip = function(widget) {
        widget.getParent().activate();
        ScreenService.closePopup();
    };
    
    function validaNomeArquivo(nome) {
        return ((nome.indexOf("<") == -1) && 
                (nome.indexOf("%") == -1) && 
                (nome.indexOf("|") == -1) && 
                (nome.indexOf("\\") == -1)&& 
                (nome.indexOf("/") == -1) && 
                (nome.indexOf("<") == -1) && 
                (nome.indexOf(">") == -1) && 
                (nome.indexOf("?") == -1) && 
                (nome.indexOf(" ") == -1) && 
                (nome.indexOf(":") == -1));
    }
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Dfs61000_TransmissaoNfse_Controller", Dfs61000_TransmissaoNfse_Controller);
});


// FILE: ./assets/modules/saida/assets/js/controllers/Dfs65001_ParametrosNfse_Controller.js
function Dfs65001_ParametrosNfse_Controller(ScreenService, templateManager, restEngine, localStorageEngine, General, fileRepository, ZHPromise) {

    var inscUsePhrase = ['4895'];

    var self = this;
    
    this.salvar = function(widget) {
       if (widget.isValid()) {
            if (widget.currentRow.__is_new == true) {
                widget.currentRow.CDSENHACERTDIG_OLD = "";
                widget.currentRow.CDSENHAACESSOWS_OLD = "";
                widget.currentRow.__using_unit = false;
                widget.currentRow.__is_new_code = true;
                widget.currentRow.__new_code_pk = "PARAMNFSEINSCMUNI";
                widget.currentRow.__table_pk = "NRPARAMNFSEINSCMUNI";
                widget.currentRow.__cast_number = true;
                widget.currentRow.__new_code_length = 10;
                widget.currentRow.__columns_to_return = ["NRPARAMNFSEINSCMUNI", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
            }

            widget.saveRow().then(function(success) {
                if (success.dataset && success.dataset.columnsToReturn[0]) {
                    widget.currentRow.NRPARAMNFSEINSCMUNI = success.dataset.columnsToReturn[0].NRPARAMNFSEINSCMUNI;
                    widget.currentRow.NRORG               = success.dataset.columnsToReturn[0].NRORG;
                    widget.currentRow.DTINCLUSAO          = success.dataset.columnsToReturn[0].DTINCLUSAO;
                    widget.currentRow.CDOPERINCLUSAO      = success.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                    widget.currentRow.NRORGINCLUSAO       = success.dataset.columnsToReturn[0].NRORGINCLUSAO;

                    if (success.dataset.columnsToReturn[0].CDSENHACERTDIG) {
                        widget.currentRow.CDSENHACERTDIG = success.dataset.columnsToReturn[0].CDSENHACERTDIG;
                    }
                    
                    if (success.dataset.columnsToReturn[0].CDSENHAACESSOWS) {
                        widget.currentRow.CDSENHAACESSOWS = success.dataset.columnsToReturn[0].CDSENHAACESSOWS;
                    }
                    
                }
                
                widget.currentRow.changed = false;
                if (widget.currentRow.__is_new) {
                    widget.currentRow.__is_new = false;
                    widget.view();
                }
                else {
                    widget.view();
                }
                
                this.alternaMaxLength(true);
                this.alteraTemplateFraseSecreta(widget);
                
                General.changeWidgetAcessibility(widget.widgets, true);
                widget.reload();
            }.bind(this),
            function(error) {
                var errorMessage = error;
                if (error.data) {
                    errorMessage = error.data.error;
                }
                ScreenService.showMessage(errorMessage);
            });
        }
    };
    
    this.validaSaas = function(widget) {
        let nrorg = getLocalVar('NRORG');
        if(nrorg == 1) {
            widget.getField('IDLOCALSERVIDOR').dataSource.data = [
                {
                     "VALUE": "1",
                    "NAME": "Local"
                },
                {
                    "VALUE": "2",
                    "NAME": "Teknisa"
                }
            ];
        }
        else {
            widget.getField('IDLOCALSERVIDOR').dataSource.data = [{
                    "VALUE": "2",
                    "NAME": "Teknisa"
            }];
        }
    };
    
    this.editar = function(widget) {
        widget.currentRow.CDSENHACERTDIG_OLD = widget.currentRow.CDSENHACERTDIG;
        widget.currentRow.CDSENHAACESSOWS_OLD = widget.currentRow.CDSENHAACESSOWS;
        
        General.widgetEdit(widget);
        var fieldInscricao = widget.getField("CDINSCMUNI");
        let fieldEmpresa = widget.getField("NMEMPRESA");
        var cdempresa = widget.currentRow.CDEMPRESA;
        
        fieldInscricao.dataSourceFilter[0].value = cdempresa;
        fieldInscricao.readOnly = true;
        fieldEmpresa.readOnlyOnEdit = true;
        
        this.alternaMaxLength(false);
        this.alteraTemplateFraseSecreta(widget);

        fieldInscricao.reload();
        fieldEmpresa.reload();
    };
    
    this.controlaFieldInscMunc = function(fieldEmpresa, fieldInscricao) {
        if(fieldEmpresa.value()) fieldInscricao.readOnly = false;
        else fieldInscricao.readOnly = true;

        fieldInscricao.reload();
    };
    
    this.nmEmpresaOnChange = function(widget) {
        let fieldInscricao = widget.getField("CDINSCMUNI");
        let fieldEmpresa = widget.getField("NMEMPRESA");
        let cdempresa = widget.currentRow.CDEMPRESA;
        
        fieldInscricao.dataSourceFilter[0].value = cdempresa;
        fieldInscricao.clearValue();
        this.controlaFieldInscMunc(fieldEmpresa, fieldInscricao);
    };
    
    this.controlarCaminhoCertificado = function(field) {
        var fieldCaminhoCert = field.widget.getField("DSCAMINHOCERTDIG");
        if (field.value() == "1") {
            fieldCaminhoCert.readOnly = false;
            fieldCaminhoCert.placeholder = "";
        }
        else {
            fieldCaminhoCert.readOnly = true;
            fieldCaminhoCert.placeholder = "Servidor Teknisa";
        }
    };
    
    this.uploadFile = function(widget) {
        var wdgListaCertDig = templateManager.container.getWidget('wdgListaCertDig');
        if(widget.isValid()) {
            if(widget.currentRow.CERTIFICADO.length == 0) {
                ScreenService.showMessage('Adicione o certificado!');
            }
            else if(widget.currentRow.CERTIFICADO.length + wdgListaCertDig.dataSource.data.length > 2) {
                ScreenService.showMessage('Certificado já existe no servidor. Exclua e tente novamente');   
            }
            else {
                if(widget.getParent().currentRow.IDLOCALSERVIDOR === "1") this.uploadFileLocal(widget);
                else this.uploadFileFileServer(widget);
            }
        }
    };
    
    this.uploadFileLocal = function(widget) {
        widget.currentRow.NRPARAMNFSEINSCMUNI = widget.getParent().currentRow.NRPARAMNFSEINSCMUNI;
        widget.currentRow.CDEMPRESA           = widget.getParent().currentRow.CDEMPRESA;
        widget.currentRow.CDINSCMUNI          = widget.getParent().currentRow.CDINSCMUNI;
        widget.currentRow.NRORG               = widget.getParent().currentRow.NRORG;
        
        var dataSource = widget.dataSource;
        dataSource.data = [widget.currentRow];
        General.callRestEngine('df_saida#/saveCertificadoLocal', 'Row', widget.currentRow).then(function() {
            return fileRepository.saveInServer();
        }).then(function(data) {
            var wdgListaCertDig = templateManager.container.getWidget("wdgListaCertDig");
            self.createWidgetDataSource(wdgListaCertDig);
            self.closeAnexoSwipe(widget);
        }, function(error) {
            if(error.data.errorCode == 100) {
                ScreenService.showMessage(error.data.error);
            }
            console.error(error);
        });
    };
    
    this.uploadFileFileServer = function(widget) {
        widget.currentRow.NRPARAMNFSEINSCMUNI = widget.getParent().currentRow.NRPARAMNFSEINSCMUNI;
        widget.currentRow.CDEMPRESA           = widget.getParent().currentRow.CDEMPRESA;
        widget.currentRow.CDINSCMUNI          = widget.getParent().currentRow.CDINSCMUNI;
        widget.currentRow.NRORG               = widget.getParent().currentRow.NRORG;
        widget.currentRow.CDSENHACERTDIG      = widget.getParent().getParent().currentRow.CDSENHACERTDIG;

        var row = widget.currentRow;
        General.callRestEngine('df_saida#/saveCerticadoFileServer', 'Row', row).then(function() {
            var wdgListaCertDig = templateManager.container.getWidget("wdgListaCertDig");
            self.createWidgetDataSource(wdgListaCertDig);
            self.closeAnexoSwipe(widget);
        }, function(error) {
            if(error.data.errorCode == 101) {
             ScreenService.showMessage(error.data.error);
            }
        });
    };
    
    this.createWidgetDataSource = function(widget) {
        var nrparamnfseinscmuni = widget.getParent().currentRow.NRPARAMNFSEINSCMUNI;
        var cdempresa           = widget.getParent().currentRow.CDEMPRESA;
        var cdinscmuni          = widget.getParent().currentRow.CDINSCMUNI;
        var nrorg               = widget.getParent().currentRow.NRORG;
        var idlocalservidor     = widget.getParent().currentRow.IDLOCALSERVIDOR;
        
        widget.currentRow.NRPARAMNFSEINSCMUNI = nrparamnfseinscmuni;
        widget.currentRow.CDEMPRESA           = cdempresa;
        widget.currentRow.CDINSCMUNI          = cdinscmuni;
        widget.currentRow.NRORG               = nrorg;
        widget.currentRow.IDLOCALSERVIDOR     = idlocalservidor;
        
        var params = {
            "NRPARAMNFSEINSCMUNI": nrparamnfseinscmuni,
            "CDEMPRESA"          : cdempresa,
            "CDINSCMUNI"         : cdinscmuni,
            "IDLOCALSERVIDOR"    : idlocalservidor,
            "NRORG"              : nrorg
        };

        General.callRestEngine("df_saida#/paramnfse_certDig/createFileDataSet", "Row", params).then(function(success) {
            localStorageEngine.remove("df_saida#/paramnfse_certDig", "").then(function()
            {
                localStorageEngine.save("df_saida#/paramnfse_certDig", success.dataset.paramnfse_certDig).then(function()
                {
                    var widget = templateManager.container.getWidget("wdgListaCertDig");
                    widget.reload();
                });
            });
        },
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
    this.downloadFile = function(widget) {
        var body = document.body;
    
        this.downloadFilePromisse(widget, body).then((element) => {
            body.removeChild(element);
        });
        
    };
    
    this.downloadFilePromisse = function(widget, body) {
        var deferred = ZHPromise.defer();
       
        var element = document.createElement("a");
    
    
        element.href = widget.currentRow.path; 
    
        element.download = widget.currentRow.fileName;
        
        element.className = "hide-link";
    
        body.appendChild(element);
    
        if (element.fireEvent) {
            element.fireEvent("onclick");
        } else {
            var evObj = document.createEvent("MouseEvents");
            evObj.initEvent("click", true, false);
            element.dispatchEvent(evObj);
        }
        deferred.resolve(element);
        return deferred;
    };
    
    this.openAnexoSwipe = function(widget) {
        if(!widget.getParent().getParent().currentRow.CDSENHACERTDIG) {
            ScreenService.showMessage('Informe a senha do certificado antes de adicionar!');
        } else {
            widget.isVisible = true;
            ScreenService.openSwipe(widget).then(function(){
                widget.newRow();
            });
        }
    };
    
    this.closeAnexoSwipe = function(widget) {
        widget.dataSource.data.pop();
        ScreenService.closeSwipe();
        widget.isVisible = false;
    };
    
    this.excluiReg = function(args) {
        var widget = args.owner.widget;
        ScreenService.confirmMessage("Excluir Registro?", "question", function() {
            General.callRestEngine('df_saida#/paramnfseinscmuni/remove','Row', widget.currentRow).then(function() {
                ScreenService.closeSwipe();
                widget.reload();
            },function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
        });
    };
    
    this.deleteFile = function(widget) {
        var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
        widget.currentRow.IDLOCALSERVIDOR = widget.getParent().currentRow.IDLOCALSERVIDOR;
		var row = Util.clone(widget.currentRow, ignoredKeys);
		
        ScreenService.confirmMessage("Excluir Anexo?", "question", function() {
            restEngine.post('df_saida#/paramnfse_certDig/deleteFile',
            {
                "requestType": "Row",
                "row": row
            }, function(success)
            {
                var wdgListaCertDig = templateManager.container.getWidget('wdgListaCertDig');
                self.createWidgetDataSource(wdgListaCertDig);
                ScreenService.closeSwipe();
                ScreenService.showMessage("Anexo excluído.");
            }, undefined, function(error)
            {
                console.log(error);
            });
        }, null);
    };
    
    /** 
     * A função altera o template do campo CDFRASESECRETA, caso esteja sendo inserido um valor, ou seja, 
     * na adição ou ao editar e o valor deste seja nulo.
     * neste campo o template alterna para text, para facilitar a visualização. Caso contrário,
    ** assume o template password.
    */
    
    this.alteraTemplateFraseSecreta = function(widget) {
        var fieldFraseSecreta = widget.getField('CDFRASESECRETA');
        
        if((fieldFraseSecreta.value() == null || fieldFraseSecreta.value() == "") && widget.editing) {
            fieldFraseSecreta.template = "field/text-edit.html";        
        } else {
            fieldFraseSecreta.template = "field/password-edit.html";        
        }
        fieldFraseSecreta.reload();
    };
    /**
     * A função tem por condição acionar as ações de cancelar, voltar e salvar como verdadeiro
     * e falso quando acionar as ações de editar e adicionar. É necessária a alteração devido
     * a diferença entre o tamanho permitido da frase e o valor da frase criptografada
     * 
    */
    
    this.alternaMaxLength = function(condicao) {
        var widget = templateManager.container.getWidget('wdgParametrosNfseInscmuni');
        var fieldFraseSecreta = widget.getField('CDFRASESECRETA');
        
        if(condicao) {
            fieldFraseSecreta.maxlength = 100;
        } else {
            fieldFraseSecreta.maxlength = 20;
        }
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Dfs65001_ParametrosNfse_Controller", Dfs65001_ParametrosNfse_Controller);
});


// FILE: ./assets/modules/saida/assets/js/controllers/Dfs71000_DevolucaoDasTransferenciasDoOperadorLogistico_Controller.js
var Dfs71000_DevolucaoDasTransferenciasDoOperadorLogistico_Controller = function(Dfs71000_DevolucaoDasTransferenciasDoOperadorLogistico_Service, MessageService, ScreenService, templateManager, General, localStorageEngine, ZhFilterPreferences){
    
    this.preparaFiltro = function(widget) {
        let currentRow = widget.appliedFilter.FILTER_ROW;
        widget.dataSourceFilter = Dfs71000_DevolucaoDasTransferenciasDoOperadorLogistico_Service.buildFilter(["CDOPERLOG", "CDFILIAL", "CDSERIE", "NRNOTAFISC"], currentRow);
        widget.dataSourceFilter.push({name: "P_DTINI", operator: "=", value: currentRow.DTENTRSAID.start});
        widget.dataSourceFilter.push({name: "P_DTFIN", operator: "=", value: currentRow.DTENTRSAID.end});
        widget.reload();

    };
    
    this.alteraReadOnlyFilhos = function(field, desbloqueiaComValor = true) {
        let widget = field.widget;
        field.filhos.forEach((filho) => {
            let filhoField = widget.getField(filho);
            filhoField.readOnly =  ((field.value() == "") == desbloqueiaComValor);
            if(((field.value() == "") == desbloqueiaComValor)) widget.currentRow[filho] = "";
            if(filhoField.filhos) this.alteraReadOnlyFilhos(filhoField, true);
            widget.getField(filho).reload();
        });
    };
    
    this.blockFieldsFilhosAndSetEvents = function(widget, filter = true) {
        let fields = widget.fields;
        fields.forEach((field) => {
            if(field.filhos) {
                field.events.push(
                    {
                        "name": "FieldOnChange",
                        "code": "Dfs71000_DevolucaoDasTransferenciasDoOperadorLogistico_Controller.alteraReadOnlyFilhos(args.owner.field)"
                    }
                );
                field.filhos.forEach((filho) => {
                    field = widget.getField(filho);
                    if(!field.filterProperties) widget.filterProperties = {};
                    if(filter)  field.filterProperties.readOnly = true; 
                    else  widget.getField(filho).readOnly = true; 
                });
            }
        });
    };
    
    this.openPopup = function() {
        var widgetDevolucao = templateManager.container.getWidget('dadosDevolucao');
        widgetDevolucao.currentRow.CDTIPOOPER = '';
        widgetDevolucao.currentRow.NMTIPOOPER = '';
        widgetDevolucao.currentRow.CDSERIE = '';
        widgetDevolucao.currentRow.DSSERIE = '';
        General.callRestEngine("df#/crud/paramgeral", "FilterData", []).then((success) => {
            if(success.dataset.paramgeral[0].IDUTSERTPOP == 'S') {
                widgetDevolucao.getField('DSSERIE').isVisible = true;
                widgetDevolucao.getField('DSSERIE').readOnly  = true;
                widgetDevolucao.getField('NMTIPOOPER').events  = [{
                        "name": "FieldOnChange",
                        "code": "Dfs71000_DevolucaoDasTransferenciasDoOperadorLogistico_Controller.tipoOperOnChange(args.owner.field);"
                }];
            }
            else {
                widgetDevolucao.getField('DSSERIE').isVisible = false;
                widgetDevolucao.getField('NMTIPOOPER').events  = [];
            }
            
            Dfs71000_DevolucaoDasTransferenciasDoOperadorLogistico_Service.getDtProcessa().then(
                (data) => {
                    widgetDevolucao.currentRow.DTSAIDA = moment().format("DD/MM/YYYY");
                    widgetDevolucao.isVisible = true;
                    ScreenService.openPopup(widgetDevolucao);
                    widgetDevolucao.activate();
                }
            );
            
        });
        
    };
    
    this.tipoOperOnChange = function(field) {
        var serie = field.widget.getField('DSSERIE');
        if(field.value()) {
            serie.dataSourceFilter[0].value = field.widget.getParent().getParent().appliedFilter.FILTER_ROW.CDFILIAL;
            serie.dataSourceFilter[1].value = field.widget.currentRow.CDTIPOOPER;
            serie.readOnly  = false;
        }
        else {
            serie.readOnly  = true;
        }
    };
    
    this.closePopup = function(widget, widgetClose) {
        widget.isVisible = false;
        widget.dataSource.data = [];
        ScreenService.closePopup(widget);
        widgetClose = templateManager.container.getWidget(widgetClose);
        widgetClose.activate();
        widgetClose.reload();
        ScreenService.closePopup();
    };
    
    this.gerarDevolucao = function(widget) {
        if(widget.isValid()) {
            localStorageEngine.getDataSource('df_saida#/dadosAdic').then((success) => {
                var dadosAdicionais = success;
                var widgetNota       = templateManager.container.getWidget('devolucao_transfOpLogistico');
                var notas = widgetNota.getCheckedRows();
                var data = {
                    notas: notas,
                    dadosGeracao: widget.currentRow,
                    dadosAdicionais: dadosAdicionais
                };
                
                General.callRestEngine("df_saida#/gerarDevolucaoTransfOl", "DataSet", data).then((success) => {
                    this.closePopup(widget);
                    if(success.messages[0].message==="Não foi possível realizar a devolução das notas fiscais selecionadas."){
                        ScreenService.showMessage(success.messages[0].message);
                    }else{
                        var numNotas = success.messages[0].message;
                        var msg = "NUM_NOTAS_SUCESO";
                        MessageService.showMessageByCode(msg, {"NUMNOTAS": numNotas});
                    }
                    ScreenService.showMessage(success.messages[0].message);
                    if(success.dataset.errosDevolucao[0]) this.openPopupErros(success.dataset.errosDevolucao);
                });
            });
        }
    };
    
    this.openPopupErros = function(errosGeracao) {
        var widgetErros = templateManager.container.getWidget('errosGeracao');
        widgetErros.dataSource.data = errosGeracao;
        widgetErros.isVisible = true;
        ScreenService.openPopup(widgetErros);
        widgetErros.activate();
    };
    
    this.verificaReloadWidget = function(widget) {
        if(widget.inlineEditSave == false) widget.reload(); 
    };
    
    
    // Salvar o produto
    this.saveProdutos = function(widget) {
        widget.dataSource.data.forEach((produto) => {
            if(produto.changed) {
                if(Number(produto.QTDEVOLUCAOOL) == 0 || !produto.QTDEVOLUCAOOL ) {
                    ScreenService.showMessage("Quantidade devolvida deve ser maior que 0. " + produto.NMPRODUTO);
                }
                else if(Number(produto.QTDEVOLUCAOOL) > Number(produto.QTITEMPROD)) {
                    ScreenService.showMessage("Quantidade devolvida não pode ser maior que a quantidade original do produto. " + produto.NMPRODUTO);
                }
                else if(!produto.NMMOTIVODEV) {
                    ScreenService.showMessage("Motivo da devolução deve ser informado. " + produto.NMPRODUTO);
                }
                else {
                    General.callRestEngine("df#/crud/itemproduto/save", "Row", produto).then(function() {
                        widget.reload();
                        delete produto.changed_QTDEVOLUCAOOL;
                        delete produto.changed_NMMOTIVODEV;
                        delete produto.changed_CDMOTIVODEV;
                        delete produto.QTDEVOLUCAOOL_inline;
                        delete produto.NMMOTIVODEV_inline;
                        delete produto.CDMOTIVODEV_inline;
                        produto.changed = false;
                        widget.list();
                    });
                }
            
            }
        });
        
        
    };
    
    this.setDefaultFilter = function(widget) {
        localStorageEngine.setDataSource('df_saida#/dadosAdic', []);
        Dfs71000_DevolucaoDasTransferenciasDoOperadorLogistico_Service.getFilial().then((success) => {
            var row = Util.clone(widget.currentRow);
            if(success.dataset.filial[0].IDFUNCAOFILI == 'O') {
                row.CDOPERLOG = success.dataset.filial[0].CDFILIAL;
                row.NMOPERLOG = success.dataset.filial[0].NMFILIAL;
            }
            else {
                row.CDFILIAL = success.dataset.filial[0].CDFILIAL;
                row.NMFILIAL = success.dataset.filial[0].NMFILIAL;
            }
            var dataAtual = formatarData(new Date());
            row.DTENTRSAID = {start : dataAtual, end : dataAtual};
            
            ZhFilterPreferences.setOriginalFilter(widget.id, row);
            ZhFilterPreferences.openFilterWidget(widget, row);
        },
        (error) => {
            ZhFilterPreferences.openFilterWidget(widget);
            console.log(error);
        });
    };
    
    this.saveDadosAdic = function(widget) {
        var existe = false;
        if(widget.isValid()) {
            localStorageEngine.getDataSource('df_saida#/dadosAdic').then(
                (dataLocal) =>  {
                    var insertRow = {CDMENSAGEM: widget.currentRow.CDMENSAGEM, 
                                    DSMENSAGEM: widget.currentRow.DSMENSAGEM,
                                    CDFILIAL:  widget.getParent().currentRow.CDFILIAL,
                                    NRLANCTONF: widget.getParent().currentRow.NRLANCTONF
                    };
                    if(widget.currentRow.__is_new) {
                        dataLocal.map(
                            (row) => {
                                if(row.CDMENSAGEM == widget.currentRow.CDMENSAGEM 
                                    && row.DSMENSAGEM == widget.currentRow.DSMENSAGEM
                                    && row.CDFILIAL   == widget.getParent().currentRow.CDFILIAL
                                    && row.NRLANCTONF   == widget.getParent().currentRow.NRLANCTONF)
                                existe = true;
                            }        
                        );
                        if(!existe) {
                            dataLocal.push(insertRow);
                        }
                    }
                    else {
                        dataLocal = dataLocal.map((row) => {
                            if(row.CDMENSAGEM == widget.backupRow.CDMENSAGEM 
                                && row.DSMENSAGEM == widget.backupRow.DSMENSAGEM
                                && row.CDFILIAL   == widget.backupRow.CDFILIAL
                                && row.NRLANCTONF   == widget.backupRow.NRLANCTONF) {
                                    return insertRow;
                                }
                            else if(row.CDMENSAGEM == widget.currentRow.CDMENSAGEM 
                                    && row.DSMENSAGEM == widget.currentRow.DSMENSAGEM
                                    && row.CDFILIAL     == widget.getParent().currentRow.CDFILIAL
                                    && row.NRLANCTONF   == widget.getParent().currentRow.NRLANCTONF) {
                                existe = true;
                            }
                            else return row;
                        });
                    }
                    if(!existe) {
                        localStorageEngine.setDataSource('df_saida#/dadosAdic', dataLocal).then(() => {
                            ScreenService.goBack();
                            widget.reload();
                        });
                    }
                    else ScreenService.showMessage("Registro já existe.");
                }
            );
        }
    };
    
    this.deleteDadosAdic = function(widget) {
        localStorageEngine.getDataSource('df_saida#/dadosAdic').then(
            (dataLocal) =>  {
                dataLocal = dataLocal.map((row) => {
                    if(!(row.CDMENSAGEM == widget.currentRow.CDMENSAGEM 
                        && row.DSMENSAGEM == widget.currentRow.DSMENSAGEM
                        && row.CDFILIAL   == widget.currentRow.CDFILIAL
                        && row.NRLANCTONF   == widget.currentRow.NRLANCTONF)) return row;
                });
                localStorageEngine.setDataSource('df_saida#/dadosAdic', dataLocal).then(() => {
                    ScreenService.goBack();
                    widget.reload();
                });
            }
        );
    };
    
    this.limpaMensagem = function(widget) {
        var cdMensagem = widget.getField('CDMENSAGEM');
        if(cdMensagem.value()) {
            cdMensagem.setValue('');
            cdMensagem.reload();
        }
    };
    
    function formatarData(data) {
        var d = new Date(data),
        mes = '' + (d.getMonth() + 1),
        dia = '' + d.getDate(),
        ano = d.getFullYear();
    
        if (mes.length < 2) mes = '0' + mes;
        if (dia.length < 2) dia = '0' + dia;
    
        return [dia, mes, ano].join('/');
    }

    this.verificaItem = function(field, args) {
        var widget = field.widget;
        if(field.widget.currentRow.IDDEVOLVIDO == 'S') {
            widget.currentRow[field.name+'_inline'] = widget.currentRow[field.name];
            ScreenService.showMessage("Operação bloqueada. Item já utilizado para devolução do produto.");
        }
    };

};

Configuration(function (ContextRegister){
    ContextRegister.register('Dfs71000_DevolucaoDasTransferenciasDoOperadorLogistico_Controller', Dfs71000_DevolucaoDasTransferenciasDoOperadorLogistico_Controller);
});


// FILE: ./assets/modules/saida/assets/js/controllers/Dfs74000_CalculoReceitaBrutaSimplesNacional_Controller.js
function Dfs74000_CalculoReceitaBrutaSimplesNacional_Controller(ScreenService, templateManager,General, Dfs74000_CalculoReceitaBrutaSimplesNacional_Service) {

    this.controlaCampoTabelaSimplesNacional = function(widget){
        widget.currentRow.NMSIMPLESNACIONAL='';
        widget.currentRow.CDSIMPLESNACIONAL='';

        if(widget.currentRow.CDEMPRESA){
            widget.getField('NMSIMPLESNACIONAL').readOnly= false;
        }else{
            widget.getField('NMSIMPLESNACIONAL').readOnly= true;
        }
    };

    this.recarregarWidgetsTipoOperacaoReceita = function(widget){
        var widgetTipoOperacao = widget.widgets[0].widgets;
        var widgetReceita = widget.widgets[1].widgets[0];
        widgetTipoOperacao[0].reload();
        widgetTipoOperacao[1].reload();
        widgetReceita.reload();
        this.controlaActionsWidgetReceitaBruta(widgetReceita);
        if(widget.currentRow.CDEMPRESA && widget.currentRow.CDSIMPLESNACIONAL){
            widgetTipoOperacao[0].getAction('adicionarRow').readOnly = false;
            widgetTipoOperacao[1].getAction('adicionarRow').readOnly = false;
        }else{
            widgetTipoOperacao[0].getAction('adicionarRow').readOnly = true;
            widgetTipoOperacao[1].getAction('adicionarRow').readOnly = true;
        }
    };

    this.controlaActionsWidgetReceitaBruta = function(widgetReceita){
        var widgetFiltro = widgetReceita.parent.parent;
        var tabbedReceita = widgetReceita.parent;
        //caso todos os campos do filtro estejam preenchidos habilita a action de adiconar
        if(widgetFiltro.currentRow.CDEMPRESA && widgetFiltro.currentRow.CDSIMPLESNACIONAL && tabbedReceita.currentRow.P_ANO){
            widgetReceita.getAction('adicionarRow').readOnly = false;
        }
        else{
            widgetReceita.getAction('adicionarRow').readOnly = true;
        }
    };

    this.editarCalculoReceita = function (widget) {
        General.edit(widget);
        widget.currentRow.VRSALDOCOMPOE_OLD = widget.currentRow.VRSALDOCOMPOE;
        widget.currentRow.VRSALDODEDUZ_OLD = widget.currentRow.VRSALDODEDUZ;
    };

    this.saveTipoOperacao = function (widget){
        if(widget.name=='widgetCompoeReceita'){
            widget.currentRow.IDTIPOOPERREC='C';
        }else if(widget.name=='widgetDeduzReceita'){
            widget.currentRow.IDTIPOOPERREC='D';
        }
        widget.currentRow.CDSIMPLESNACIONAL = widget.parent.parent.currentRow.CDSIMPLESNACIONAL;
        var filter = [
            {
                name: "CDTIPOOPER",
                operator: "=",
                value: widget.currentRow.CDTIPOOPER
            },
            {
                name: "CDSIMPLESNACIONAL",
                operator: "=",
                value: widget.currentRow.CDSIMPLESNACIONAL
            },
            {
                name: "IDENTRSAIDOP",
                operator: "=",
                value: widget.currentRow.IDENTRSAIDOP
            }
        ];
        General.callRestEngine("df#/crud/tpopersimplesnac", "FilterData", filter).then((success) => {
            General.saveCrud(widget);
        });
    };
    
    this.updateITipoOperac = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Excluir Registro?'), 'question', function (success) {
            widget.currentRow.IDTIPOOPERREC = 'I';
            General.saveCrud(widget).then(() => {
                ScreenService.goBack();
                widget.reload();
            });
        });
    };

    this.saveReceitaBruta = function (widget){
        if(widget.isValid()){
            this.calculaValor(widget);
            var tabbedReceita = widget.parent;
            var tabbedCalculo = widget.parent.parent;
            if(widget.currentRow.__is_new){
                widget.currentRow.IDINFMANUAL       = 'M';
                widget.currentRow.CDEMPRESA         = tabbedCalculo.currentRow.CDEMPRESA;
                widget.currentRow.DTREFRECEITABR    = '01/'+widget.currentRow.MES+'/'+tabbedReceita.currentRow.P_ANO;
                widget.currentRow.CDSIMPLESNACIONAL = tabbedCalculo.currentRow.CDSIMPLESNACIONAL;
            }
            else{
                Dfs74000_CalculoReceitaBrutaSimplesNacional_Service.verificaValorAlterado(widget.currentRow);
            }
            Dfs74000_CalculoReceitaBrutaSimplesNacional_Service.validacoesSave(widget.currentRow).then(
                function(success) {
                    var mesAtual     = success.dataset.result.mesAtual;
                    var mesPosterior = success.dataset.result.mesPosterior;
                    var rows ={
                        'mesAtual':widget.currentRow,
                        'mesPosterior':mesPosterior
                    };
                    if(widget.currentRow.__is_new){
                        if(mesAtual.length == 0){
                            if(mesPosterior.length>0){
                                ScreenService.confirmMessage(ScreenService.i18n('Atenção! Ao confirmar esta operação o sistema irá desbloquear os meses posteriores ao mês corrente a ser excluído e irá recalcular a receita. Confirma a operação?'), 'question', function (success) {
                                    this.saveMes(widget,rows);

                                }.bind(this));
                            }else{
                                this.saveMes(widget,rows);
                            }
                        }else{
                            ScreenService.showMessage('Mês já cadastrado para o ano informado.');
                        }
                    }
                    else{
                        if(mesPosterior.length>0){
                            ScreenService.confirmMessage(ScreenService.i18n('Atenção! Ao confirmar esta operação o sistema irá desbloquear os meses posteriores ao mês corrente a ser excluído e irá recalcular a receita. Confirma a operação?'), 'question', function (success) {
                                this.saveMes(widget,rows);
                            }.bind(this));
                        }else{
                            this.saveMes(widget,rows);
                        }
                    }
                }.bind(this),
                function(callback){
                    if(callback.data){
                        ScreenService.showMessage(callback.data.error);
                    }
                }
            );

        }

    };

    this.saveMes = function(widget,rows){
        Dfs74000_CalculoReceitaBrutaSimplesNacional_Service.saveReceitaBruta(rows).then(
            function(Success) {
                widget.activate();
                if (widget.currentRow.__is_new) {
                    widget.list(true);
                    ScreenService.goBack();
                    widget.reload();
                } else {
                    widget.computeDescriptionOrder();
                    widget.view();
                }

                if(rows.mesPosterior.length>0){
                   widget.currentRow.MESCALCULADO = true;
                }

            }.bind(this),
            function(callback){
                if(callback.data){
                    ScreenService.showMessage(callback.data.error);
                }
            }
        );
    };

    this.deletarReceitaBruta = function (widget){
        Dfs74000_CalculoReceitaBrutaSimplesNacional_Service.buscaMesPosterior(widget.currentRow).then(
            function(success) {
               var mesPosterior = success.dataset.result;
               var rows ={
                    'mesAtual':widget.currentRow,
                    'mesPosterior':mesPosterior
                };
                if(mesPosterior.length>0){
                    ScreenService.confirmMessage(ScreenService.i18n('Atenção! Ao confirmar esta operação o sistema irá desbloquear os meses posteriores ao mês corrente a ser excluído e irá recalcular a receita. Confirma a operação?'), 'question', function (success) {

                        Dfs74000_CalculoReceitaBrutaSimplesNacional_Service.deleteMesReceita(rows).then(
                            function(success) {
                                ScreenService.closeSwipe().then(function () {
                                    widget.activate();
                                });
                                widget.reload();
                            }.bind(this),
                            function(callback){
                                if(callback.data){
                                    ScreenService.showMessage(callback.data.error);
                                }
                            }
                        );

                    }.bind(this));
                }else{
                    General.deleteCrud(widget);
                }
            }.bind(this),
            function(callback){
                if(callback.data){
                    ScreenService.showMessage(callback.data.error);
                }
            }
        );
    };

    this.calculaValor = function (widget){
        Dfs74000_CalculoReceitaBrutaSimplesNacional_Service.calculaValorTotalRceitaBruta(widget.currentRow);
    };

    this.aplicaFiltroAno = function(widget){
        var data = moment();
        widget.getField('P_ANO').value( data.format('YYYY') );
        widget.widgets[0].reload();
        this.controlaActionsWidgetReceitaBruta(widget.widgets[0]);
    };

    this.controlaActionsCalculoReceita = function(widget){
        if(!widget.currentRow.__is_new){
            var editar      = widget.getAction('editar');
            var excluir     = widget.getAction('DELETE');
            var calcular    = widget.getAction('calcularReceita');
            var bloquear    = widget.getAction('bloquearMes');
            var desbloquear = widget.getAction('desbloquearMes');
            switch(widget.currentRow.IDSTATUSMES){
                case'P':
                    editar.readOnly      = false;
                    excluir.readOnly     = false;
                    calcular.readOnly    = false;
                    bloquear.readOnly    = false;
                    desbloquear.readOnly = true;
                    break;
                case'C':
                    editar.readOnly      = false;
                    excluir.readOnly     = false;
                    calcular.readOnly    = false;
                    bloquear.readOnly    = false;
                    desbloquear.readOnly = true;
                    break;
                case'B':
                    editar.readOnly      = true;
                    excluir.readOnly     = true;
                    calcular.readOnly    = true;
                    bloquear.readOnly    = true;
                    desbloquear.readOnly = false;
                    break;
            }
        }
    };

    this.calcularReceitaBrutaMes = function(widget){
        Dfs74000_CalculoReceitaBrutaSimplesNacional_Service.buscaMesPosterior(widget.currentRow).then(
            function(Success) {
               var mesPosterior = Success.dataset.result;
               var rows ={
                    'mesAtual':widget.currentRow,
                    'mesPosterior':mesPosterior
                };
                if(mesPosterior.length>0){
                    ScreenService.confirmMessage(ScreenService.i18n('Atenção! Ao confirmar esta operação o sistema irá desbloquear os meses posteriores ao mês corrente e irá recalcular a receita. Confirma a operação?'), 'question', function (success) {

                        Dfs74000_CalculoReceitaBrutaSimplesNacional_Service.calculaReceitaBrutaMes(rows).then(
                            function(Success) {
                                widget.currentRow.IDINFMANUAL   = Success.dataset.receitaBruta.idinfmanual;
                                widget.currentRow.IDSTATUSMES   = Success.dataset.receitaBruta.idstatusmes;
                                widget.currentRow.VRSALDONEGMES = Success.dataset.receitaBruta.saldoNegativo;
                                widget.currentRow.VRSALDOCOMPOE = Success.dataset.receitaBruta.valorCompoem;
                                widget.currentRow.VRSALDODEDUZ  = Success.dataset.receitaBruta.valorDeduz;
                                widget.currentRow.VRRECEITABR   = Success.dataset.receitaBruta.valorTotal;
                                ScreenService.closePopup();
                                ScreenService.showMessage('Mês calculado com sucesso.');
                                this.controlaActionsCalculoReceita(widget);
                                widget.reload();
                                widget.currentRow.MESCALCULADO = true;
                            }.bind(this),
                            function(callback){
                                if(callback.data){
                                    ScreenService.showMessage(callback.data.error);
                                }
                            }
                        );
                    }.bind(this));
                }else{
                    Dfs74000_CalculoReceitaBrutaSimplesNacional_Service.calculaReceitaBrutaMes(rows).then(
                        function(Success) {
                                widget.currentRow.IDINFMANUAL   = Success.dataset.receitaBruta.idinfmanual;
                                widget.currentRow.IDSTATUSMES   = Success.dataset.receitaBruta.idstatusmes;
                                widget.currentRow.VRSALDONEGMES = Success.dataset.receitaBruta.saldoNegativo;
                                widget.currentRow.VRSALDOCOMPOE = Success.dataset.receitaBruta.valorCompoem;
                                widget.currentRow.VRSALDODEDUZ  = Success.dataset.receitaBruta.valorDeduz;
                                widget.currentRow.VRRECEITABR   = Success.dataset.receitaBruta.valorTotal;
                                ScreenService.closePopup();
                                ScreenService.showMessage('Mês calculado com sucesso.');
                                this.controlaActionsCalculoReceita(widget);
                                widget.reload();
                                widget.currentRow.MESCALCULADO = true;
                        }.bind(this),
                        function(callback){
                            if(callback.data){
                                ScreenService.showMessage(callback.data.error);
                            }
                        }
                    );
                }
            }.bind(this),
            function(callback){
                if(callback.data){
                    ScreenService.showMessage(callback.data.error);
                }
            }
        );
    };

    this.desbloquearMesReceita = function (widget){
        widget.currentRow.IDSTATUSMES = 'P'; 
        General.saveCrud(widget).then(
            function(Success) {
                ScreenService.closePopup();
                this.controlaActionsCalculoReceita(widget);
                ScreenService.showMessage('Mês desbloqueado com sucesso.');
            }.bind(this),
            function(callback){
                widget.currentRow.IDSTATUSMES = 'B';
                if(callback.data){
                    ScreenService.showMessage(callback.data.error);
                }
            }
        );
    };

    this.bloquerMesReceita = function (widget){
        if(widget.currentRow.IDINFMANUAL == 'M'){
            Dfs74000_CalculoReceitaBrutaSimplesNacional_Service.buscaNotasFiscaisMes(widget.currentRow).then(
                function(Success) {
                    var notasFiscais = Success.dataset.notasFiscais;
                    if(notasFiscais.length >0){
                        ScreenService.confirmMessage(ScreenService.i18n('Atenção! Existem notas lançadas no sistema para este mês. Deseja recalcular?'), 'question',
                            function (Success) {
                                Dfs74000_CalculoReceitaBrutaSimplesNacional_Service.bloquerMesReceita(widget.currentRow).then(
                                    function(Success) {
                                        widget.currentRow.IDINFMANUAL   = Success.dataset.receitaBruta.idinfmanual;
                                        widget.currentRow.IDSTATUSMES   = Success.dataset.receitaBruta.idstatusmes;
                                        widget.currentRow.VRSALDONEGMES = Success.dataset.receitaBruta.saldoNegativo;
                                        widget.currentRow.VRSALDOCOMPOE = Success.dataset.receitaBruta.valorCompoem;
                                        widget.currentRow.VRSALDODEDUZ  = Success.dataset.receitaBruta.valorDeduz;
                                        widget.currentRow.VRRECEITABR   = Success.dataset.receitaBruta.valorTotal;
                                        ScreenService.closePopup();
                                        ScreenService.showMessage('Mês bloqueado com sucesso.');
                                        this.controlaActionsCalculoReceita(widget);
                                        widget.reload();
                                        widget.currentRow.MESCALCULADO = true;
                                    }.bind(this),
                                    function(callback){
                                        if(callback.data){
                                            ScreenService.showMessage(callback.data.error);
                                        }
                                    }
                                );
                            }.bind(this),
                            function (error) {
                                this.bloqueiaMes(widget);
                            }.bind(this)
                        );
                    }else{
                        this.bloqueiaMes(widget);
                    }
                }.bind(this),
                function(callback){
                    if(callback.data){
                        ScreenService.showMessage(callback.data.error);
                    }
                }
            );
        }else if(widget.currentRow.IDINFMANUAL == 'C'){
           this.bloqueiaMes(widget);
        }
    };

    this.actionVoltar = function (widget){
        ScreenService.goBack();
        if(widget.currentRow.MESCALCULADO){
            widget.reload();
            widget.currentRow.MESCALCULADO = false;
        }
    };

    this.bloqueiaMes = function (widget){
        var status = widget.currentRow.IDSTATUSMES;
        widget.currentRow.IDSTATUSMES = 'B';
        General.saveCrud(widget).then(
            function(Success) {
                ScreenService.closePopup();
                ScreenService.showMessage('Mês bloqueado com sucesso.');
                this.controlaActionsCalculoReceita(widget);
            }.bind(this),
            function(callback){
                widget.currentRow.IDSTATUSMES = status;
                if(callback.data){
                    ScreenService.showMessage(callback.data.error);
                }
            }
        );
    };

}
Configuration(function (ContextRegister) {
    ContextRegister.register("Dfs74000_CalculoReceitaBrutaSimplesNacional_Controller", Dfs74000_CalculoReceitaBrutaSimplesNacional_Controller);
});


// FILE: ./assets/modules/saida/assets/js/controllers/Dfs77000_FilialRota_Controller.js
function Dfs77000_FilialRota_Controller(ScreenService, templateManager, restEngine, General) {

    var self = this;

    self.justInteger = function (campos){
        campos.forEach(function(campos){
            General.justInteger(campos);
        });
    };

    self.setDataSourceFilho = function(){
        var widgetPai = templateManager.container.getWidget('rotaentrega');
        widgetPai.widgets.forEach(function (filha){
            if(filha.name != 'filterPopup') filha.dataSourceFilter[0].value = widgetPai.currentRow.CDROTATRANS;
        });
    };

    self.setFilialRota = function(widget){
        widget.currentRow.CDFILIROTA = widget.currentRow.CDFILIAL;
    };

    self.saveFilialRota = function (widget){
        widget.currentRow.CDROTATRANS = widget.dataSourceFilter[0].value;
        General.widgetSave(widget);
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Dfs77000_FilialRota_Controller", Dfs77000_FilialRota_Controller);
});


// FILE: ./assets/modules/saida/assets/js/controllers/Dfs85000_ManifestoDocumentoEletronico_Controller.js
function Dfs85000_ManifestoDocumentoEletronico_Controller(Dfs85000_ManifestoDocumentoEletronico_Service, templateManager, General, ZhFilterPreferences, ScreenService, MessageService, ZHPromise) {
    
    let defaultMdfe = {
        CDEMPRESA: '',
        NMEMPRESA: '',
        CDINSCESTA: ''
    };
    
    let UP = 1;
    let DOWN = -1;
    
    this.setDefaultFilter = function(args) {
        var widget     = args.owner;
        var currentRow = widget.currentRow;
        var row        = { CDFILIAL: getLocalVar('CDFILIAL')};
        
        General.callRestEngine("df_entrada#/defaultFilterManif", "Row", row).then((defaultFilter) =>  {
            
            defaultFilter = defaultFilter.dataset.defaultFilter;
            var keys = ['CDEMPRESA', 'NMEMPRESA', 'NMRAZSOCEMPR', 'NRINSJURIEST', 'CDINSCESTA', 'DTEMISSAO'];
            var filter = Util.clone(currentRow, ["__rightActions", "__leftActions"]);
            keys.map((key) => {
                if(key == 'DTEMISSAO') {
                    filter.DTMDFE = {
                        start:  defaultFilter[key],
                        period: undefined,
                        end:    defaultFilter[key]
                    };
                }
                filter[key] = defaultFilter[key];
            });
            widget.getField('CDINSCESTA').readOnly = false;
            ZhFilterPreferences.setOriginalFilter(widget.id, filter);
            ZhFilterPreferences.openFilterWidget(widget, filter).then ( () => {
                var filterWidget = templateManager.container.getActiveWidget();
                filterWidget.actions.filter(action => action.name === "clear")[0].shortcutKey = "ctrl+backspace";
                filterWidget.getField('CDINSCESTA').readOnly = !filterWidget.currentRow.CDEMPRESA;
                filterWidget.getField("CDINSCESTA").dataSourceFilter[0].value = filterWidget.currentRow.CDEMPRESA;
            });
        },
        (error) => {
            ZhFilterPreferences.openFilterWidget(widget).then ( () => {
                var filterWidget = templateManager.container.getActiveWidget();
                filterWidget.actions.filter(action => action.name === "clear")[0].shortcutKey = "ctrl+backspace";
            });
        });
    };
    
    this.preparaFiltro = function(widget, args) {
        
        args.e.preventDefault();
        let currentRow = args.e.preference.FILTER_ROW;

        widget.dataSourceFilter = [];
        Dfs85000_ManifestoDocumentoEletronico_Service.buildFilter(["CDEMPRESA", "NRINSJURIEST", "CDINSCESTA", "CDVIATURA", "CDMOTORISTA", "NRMDFEINI", "NRMDFEFIN", "IDSTATUSMDFE", "IDTPAMBIENTEMDFE", "IDTIPOEMISMDFE"], currentRow, true)
        .then((filter) => {
            widget.dataSourceFilter = filter;

            widget.dataSourceFilter.push({name: "P_DTMDFEINI", operator: "=", value: currentRow.DTMDFE.start});
            widget.dataSourceFilter.push({name: "P_DTMDFEFIM", operator: "=", value: currentRow.DTMDFE.end});

            
            widget.reload().then(() => {
                widget.activate();
            });
            ScreenService.closePopup();
            
            defaultMdfe.CDEMPRESA   = currentRow.CDEMPRESA;
            defaultMdfe.CDINSCESTA  = currentRow.CDINSCESTA;
            defaultMdfe.NMEMPRESA   = currentRow.NMEMPRESA;
            
            var row = {CDEMPRESA: args.e.preference.FILTER_ROW.CDEMPRESA};
            General.callRestEngine("df_saida#/getDefaultMdfe", "Row", row).then((success) => {
                Dfs85000_ManifestoDocumentoEletronico_Service.setDefaultMdfe(success.dataset.mdfe);
            });
        });

    };
    
    this.controlaActionsMdfe = function(widget, actions) {
        var rowMdfe = templateManager.container.getWidget('manifesto_eletronico').currentRow;
        var readOnly = (rowMdfe.IDSTATUSMDFE == 'A' || rowMdfe.IDSTATUSMDFE == 'E' || rowMdfe.IDSTATUSMDFE == 'C');

        actions.forEach((actionName) => {
            var action = widget.getAction(actionName); 
            action.readOnly = readOnly; 
        });
    };
    
    this.controlaVisibiliadeCabecalho = function (widget) {
        var row = widget.currentRow;
        var status = row.IDSTATUSMDFE;
        
        var actionTransmitir = widget.getAction('actTransmitirMDFE');
        var actionCancelar   = widget.getAction('actCancelarMDFE');
        var actionEncerrar   = widget.getAction('actEncerrarMDFE');
        var actionConsulta   = widget.getAction('actConsultaMDFE');
        var actionDamdfe     = widget.getAction('actVisualizarDamdfe');
        
        let visibiltyTransmitir      = status == 'P' || status == 'R';
        let visibiltyCancelar        = status == 'A';
        let visiblityEncerrar        = status == 'A';
        let visibiltyConsultar       = true;
        let visibilityDamdfe         = status == 'A';
        
        actionTransmitir.isVisible = visibiltyTransmitir;
        actionCancelar.isVisible   = visibiltyCancelar;
        actionEncerrar.isVisible   = visiblityEncerrar;
        actionConsulta.isVisible   = visibiltyConsultar;
        actionDamdfe.isVisible     = visibilityDamdfe;
    };
    
    this.onChangeResponsavelSeguro = function(widget) {
        if(widget.currentRow.IDRESPSEGURO == '2') {
            widget.getField('NRINSJURRESP').validations     = {required: true};
            widget.getField('NRINSCSEGURADORA').validations = {required: true};
            widget.getField('NMSEGURADORA').validations     = {required: true};
            widget.getField('NRAPOLISE').validations        = {required: true};
            widget.getField('NRINSJURRESP').readOnly        = false;
            widget.currentRow.NRINSJURRESP = '';
        }  
        else {
            widget.getField('NRINSJURRESP').validations     = {required: true};
            widget.getField('NRINSCSEGURADORA').validations = {required: true};
            widget.getField('NMSEGURADORA').validations     = {required: true};
            widget.getField('NRAPOLISE').validations        = {};
            widget.getField('NRINSJURRESP').readOnly        = true;
            widget.currentRow.IDTPINSJURRESP = 'J';
            widget.currentRow.NRINSJURRESP = widget.getParent().currentRow.NRINSJURIEST;
            this.validaInscricaoResponsavel(widget.getField('NRINSJURRESP'));
        }
    };

    
    this.empresaOnChange = function(widget) {
        var activeWidget = templateManager.container.getActiveWidget();
        if (activeWidget.name == 'zhFilterPreferencesSelectConditions') {
            activeWidget.getField("CDINSCESTA").dataSourceFilter[0].value = activeWidget.currentRow.CDEMPRESA;
        }
        widget.currentRow.CDINSCESTA = '';
        if(!widget.currentRow.CDEMPRESA) {
            widget.getField('CDINSCESTA').readOnly = true;
        }
        else{
            widget.getField('CDINSCESTA').readOnly = false;
        }
        widget.getField('CDINSCESTA').reload();
    };
    
    this.novoMdfe = function(widget) {
        widget.newRow();
        let currentRow = widget.currentRow;   
        Dfs85000_ManifestoDocumentoEletronico_Service.getDefaultMdfe().then((defaultMdfe) => {
            currentRow = Object.assign(currentRow, defaultMdfe[0]);
            currentRow.__is_new = true;
            widget.getField('CDINSCESTA').reload();
            widget.getField('CDMODDOCFISCMDFE').setValue('58');
            currentRow.CDMODDOCFISCMDFE = '58';
            this.controlaStatusWidgets(widget, widget.name, false);
        });
    };
    
    this.salvarMDFE = function(widget) {
        widget.getField('CDMODDOCFISCMDFE').setValue('58');
        widget.currentRow.CDMODDOCFISCMDFE = '58';
        widget.currentRow.NRINSJURRESPGER = widget.currentRow.NRINSJURRESPGER.replace(/[^0-9]/g, '');
        if(widget.isValid()) {
            if(widget.currentRow.NRCIOT && !widget.currentRow.NRINSJURRESPGER) {
                MessageService.showMessageByCode("CIOT_SEM_INSCRICAO");
            }
            else if(!widget.currentRow.NRCIOT && widget.currentRow.NRINSJURRESPGER) {
                MessageService.showMessageByCode("INSCRICAO_SEM_CIOT");
            }
            else {
                if(widget.currentRow.NRINSJURRESPGER) {
                    if(widget.currentRow.NRINSJURRESPGER.length == 11 || widget.currentRow.NRINSJURRESPGER.length == 14) {
                        let objInscricao = General.inscricaoFormat(widget.currentRow.NRINSJURRESPGER);
                        if(objInscricao.valid) {
                            this.salvaMDFE(widget);
                        }
                        else {
                            MessageService.showMessageByCode("INSCRICAO_RESPONSAVEL_GERACAO_INVALIDA");
                        }
                    }
                    else {
                        MessageService.showMessageByCode("INSCRICAO_RESPONSAVEL_GERACAO_INVALIDA");
                    }
                }
                else {
                    this.salvaMDFE(widget);
                }
            }
        }
    };
    
    this.salvarSeguroCarga = function(widget) {
        if(widget.isValid()) {
            widget.currentRow.NRINSJURRESP     = widget.currentRow.NRINSJURRESP.replace(/[^0-9]/g, '');
            widget.currentRow.NRINSCSEGURADORA = widget.currentRow.NRINSCSEGURADORA.replace(/[^0-9]/g, '');
            if(widget.currentRow.NRINSJURRESP.length == 11 || widget.currentRow.NRINSJURRESP.length == 14) {
                var objInscricao = General.inscricaoFormat(widget.currentRow.NRINSJURRESP);
                if(objInscricao.valid) {
                    if(widget.currentRow.NRINSCSEGURADORA) {
                        if(widget.currentRow.NRINSCSEGURADORA.length == 14) {
                            var objInscricaoSeg = General.inscricaoFormat(widget.currentRow.NRINSCSEGURADORA);
                            if(objInscricaoSeg.valid) {
                                General.callRestEngine("df#/salvaSeguroCarga", "Row", widget.currentRow).then((success) => {
                                    MessageService.successNotificationByCode("REGISTRO_SALVO");
                                    if(widget.currentRow.__is_new) {
                                        ScreenService.closeSwipe();
                                        widget.list();
                                    }
                                    else {
                                        widget.view();
                                    }
                                    this.controlaStatusWidgets(widget, widget, true);
                                    this.controlaStatusWidgets(widget.getParent(), widget, true);
                                    widget.reload();
                                }).catch((error) => {
                                    ScreenService.errorNotification(error.data.error);
                                });
                            }
                            else {
                                MessageService.showMessageByCode("INSCRICAO_SEGURADORA_SEGURO_INVALIDA");                                        
                            }
                        }
                        else {
                            MessageService.showMessageByCode("INSCRICAO_SEGURADORA_SEGURO_INVALIDA");                                            
                        }
                    }
                    else {
                        General.callRestEngine("df#/salvaSeguroCarga", "Row", widget.currentRow).then((success) => {
                            MessageService.successNotificationByCode("REGISTRO_SALVO");
                            if(widget.currentRow.__is_new) {
                                ScreenService.closeSwipe();
                                widget.list();
                            }
                            else {
                                widget.view();
                            }         
                            this.controlaStatusWidgets(widget, widget, true);
                            this.controlaStatusWidgets(widget.getParent(), widget, true);
                            widget.reload();
                        }).catch((error) => {
                            ScreenService.errorNotification(error.data.error);
                        });
                    }
                }
                else {
                    MessageService.showMessageByCode("INSCRICAO_RESPONSAVEL_SEGURO_INVALIDA");
                }
            }
            else {
                MessageService.showMessageByCode("INSCRICAO_RESPONSAVEL_SEGURO_INVALIDA");
            }
                
        }  
    };
    
    this.editarSeguroCarga = function (widget) {
        General.edit(widget);
        this.controlaStatusWidgets(widget, widget.name, false);
        
        var nrinsjurresp = widget.currentRow.NRINSJURRESP;
        this.onChangeResponsavelSeguro(widget);
        
        if (widget.currentRow.IDRESPSEGURO == '2') {
            widget.getField("NRINSJURRESP").setValue(nrinsjurresp);
        }
        this.validaInscricaoResponsavel(widget.getField("NRINSJURRESP"), "IDTPINSJURRESP");
        this.validaInscricaoResponsavel(widget.getField("NRINSCSEGURADORA"), "");
    };
    
    this.salvaMDFE = function(widget) {
        widget.currentRow.NRINSJURRESPGER = widget.currentRow.NRINSJURRESPGER.replace(/[^0-9]/g, '');
        General.callRestEngine("df#/salvarMDFE", "Row", widget.currentRow).then((success) => {
            widget.currentRow.NRLANCTOMDFE = success.messages[0].message.NRLANCTOMDFE;
            this.controlaStatusWidgets(widget, widget.name, true);
            MessageService.successNotificationByCode("REGISTRO_SALVO");
            widget.view();
            widget.reload();
        }).catch((error) => {
            ScreenService.errorNotification(error.data.error);
        });  
    };
    
    this.adicionarWidgetFilha = function(widget, otherFields = []) {
        widget.newRow();
        widget.currentRow.NRLANCTOMDFE = widget.getParent().currentRow.NRLANCTOMDFE;
        widget.currentRow.CDVIATURA    = widget.getParent().currentRow.CDVIATURA;
        otherFields.forEach((field) => {
            widget.currentRow[field] = widget.getParent().currentRow[field]; 
        });
        this.controlaStatusWidgets(widget.getParent(), widget, false, [widget.name]);
    };
    
    this.adicionarAverbacao = function(widget, otherFields = []) {
        widget.newRow();
        widget.currentRow.NRLANCTOMDFE = widget.getParent().currentRow.NRLANCTOMDFE;
        widget.currentRow.CDVIATURA    = widget.getParent().currentRow.CDVIATURA;
        otherFields.forEach((field) => {
            widget.currentRow[field] = widget.getParent().currentRow[field]; 
        });
    };
    
    this.salvarGenerico = function(widget, dataSourceName, controlaStatus = true, fechaAba = true) {
        var route = "df#/" + dataSourceName;
        var row = Util.clone(widget.currentRow, ["__rightActions", "__leftActions"]);
        
        if(!widget.isValid()) {
            return;
        }
        
        General.callRestEngine(route, "Row", row).then((success) => {
            if(controlaStatus)  this.controlaStatusWidgets(widget.getParent(), widget.name, true);
            MessageService.successNotificationByCode("REGISTRO_SALVO");
            if(widget.currentRow.__is_new) {
                if (fechaAba) {
                    ScreenService.closeSwipe();
                }
                widget.list();
            }
            else {
                widget.view();
            }
            widget.reload();
        }).catch((error) => {
            ScreenService.errorNotification(error.data.error);
        });  
    };
    
    this.excluirGenerico = function (widget, dataSourceName, checkedRows = false, closeSwipe = false) {
        var route = "df#/" + dataSourceName;
        var row = Util.clone(widget.currentRow, ["__rightActions", "__leftActions"]);
        var saveData;
        var requestType;
        var confirmCode;
        var sucessCode;
        
        if (checkedRows) {
            let rows = widget.getCheckedRows();
            rows.forEach (saveRow => {
                delete saveRow.__rightActions;
                delete saveRow.__leftActions;
            });
            saveData = rows;
            requestType = "DataSet";
            confirmCode = "REGISTRO_EXCLUIR_SELECIONADOS";
            sucessCode = "REGISTRO_EXCLUIDO_SELECIONADOS";
        }
        else {
            saveData = row;
            requestType = "Row";
            confirmCode = "REGISTRO_EXCLUIR";
            sucessCode = "REGISTRO_EXCLUIDO";
        }
        
        MessageService.confirmMessageByCode(confirmCode).then (sucess => {
            General.callRestEngine(route, requestType, saveData).then (response => {
                if (closeSwipe) {
                    ScreenService.closeSwipe();  
                }
                widget.reload();
                MessageService.successNotificationByCode(sucessCode);
            }).catch((error) => {
                ScreenService.showMessage(error.data.error);
            });
        });
    };
    
    this.adicionarNotaRelacionada = function (widget) {
        var widgetRelacionar = widget.container.getWidget('relacionarNotas');
        
        widgetRelacionar.dataSourceFilter = [
            {
                "name": "CDEMPRESA",
                "operator": "=",
                "value": "@General.getFieldValue('CDEMPRESA', 'manifesto_eletronico')@"
            },
            {
                "name": "CDINSCESTA",
                "operator": "=",
                "value": "@General.getFieldValue('CDINSCESTA', 'manifesto_eletronico')@"
            }];
        
        ScreenService.openPopup(widgetRelacionar);
        this.controlaStatusWidgets(widget.getParent(), widgetRelacionar.name, false);
        
        widgetRelacionar.reload();
    };
    
    this.confimarRelacionamentoNotas = function (widget) {
        var widgetNotasRelacionadas = widget.container.getWidget('notasrelacionadas');
        
        Dfs85000_ManifestoDocumentoEletronico_Service.saveNotasRelacionadas(widget).then(response => {
            ScreenService.closePopup();
            this.controlaStatusWidgets(widget.getParent(), widget.name, true);
            widgetNotasRelacionadas.reload();
        }).catch(error => {
            ScreenService.showMessage(error.data.error);
        });
    };
    
    this.mudarOrdemNotaMdfe = function (widget, sentido) {
        var row = Util.clone(widget.currentRow, ["__rightActions", "__leftActions"]);
        row.SENTIDO = sentido;
        General.callRestEngine("df#/notasRelacionadasMdfe/alterarOrdem", "Row", row).then (response => {
            widget.reload();
        }).catch((error) => {
            ScreenService.errorNotification(error.data.error);
        });
    };
    
    this.percursoFieldOnChange = function (fieldChanged) {
        var widget = fieldChanged.widget;
        var fieldPais = widget.getField("NMPAIS");
        var fieldEstado = widget.getField("NMESTADO");
        var fieldMunicipio = widget.getField("NMMUNICIPIO");
        
        if (fieldChanged === fieldPais) {
            widget.currentRow.SGESTADO = "";
            widget.currentRow.NMESTADO = "";
            widget.currentRow.CDMUNICIPIO = "";
            widget.currentRow.NMMUNICIPIO = "";
        }
        else if (fieldChanged === fieldEstado) {
            widget.currentRow.CDMUNICIPIO = "";
            widget.currentRow.NMMUNICIPIO = "";
        }
        
        var pais = fieldPais.value();
        var estado = fieldEstado.value();
        var municipio = fieldMunicipio.value();
        
        if (!pais) {
            widget.currentRow.SGESTADO = "";
            widget.currentRow.NMESTADO = "";
            widget.currentRow.CDMUNICIPIO = "";
            widget.currentRow.NMMUNICIPIO = "";
            fieldEstado.readOnly = true;
            fieldMunicipio.readOnly = true;
            return;
        }
        
        if (!estado) {
            widget.currentRow.CDMUNICIPIO = "";
            widget.currentRow.NMMUNICIPIO = "";
            fieldEstado.readOnly = false;
            fieldMunicipio.readOnly = true;
            return;
        }
        
        fieldEstado.readOnly = false;
        fieldMunicipio.readOnly = false;
    };
    
    this.mudarOrdemPercursoMdfe = function (widget, sentido) {
        var row = Util.clone(widget.currentRow, ["__rightActions", "__leftActions"]);
        row.SENTIDO = sentido;
        General.callRestEngine("df#/percursoMdfe/alterarOrdem", "Row", row).then (response => {
            widget.reload();
        }).catch((error) => {
            ScreenService.errorNotification(error.data.error);
        });
    };
    
    this.buildFilterTransmissao = function (currentRow) {
        var row = JSON.parse(JSON.stringify(currentRow)); //Cria uma cópia
        
        var promise = ZHPromise.defer();
        
        for (let rowName in row) {
            if (['CDMOTORISTA', 'CDVIATURA'].includes(rowName)) {
                row[rowName] = [row[rowName]];
            }
        }
        
        row.IDSTATUSMDFE = '';
        row.IDTPAMBIENTEMDFE = '';
        row.IDTIPOEMISMDFE = '';
        row.CDMOTORISTA = '';
        
        Dfs85000_ManifestoDocumentoEletronico_Service.buildFilter(["CDEMPRESA", "NRINSJURIEST", "CDINSCESTA", "CDVIATURA", "CDMOTORISTA", "NRMDFEINI", "NRMDFEFIN", "IDSTATUSMDFE", "IDTPAMBIENTEMDFE", "IDTIPOEMISMDFE"], row, true)
        .then((filter) => {
            
            filter.push({name: "P_DTMDFEINI", operator: "=", value: moment(currentRow.DTMDFE, "DD/MM/YY hh:mm:ss").subtract(1, "days").format("DD/M/YYYY")});
            filter.push({name: "P_DTMDFEFIM", operator: "=", value: moment(currentRow.DTMDFE, "DD/MM/YY hh:mm:ss").add(1, "days").format("DD/M/YYYY")});
            filter.push({name: "NRLANCTOMDFE", operator: "=", value: currentRow.NRLANCTOMDFE});
            
            promise.resolve(filter);
        });
        
        return promise;
    };
    
    this.reloadRowMdfe = function (widget, filter) {
        var deffered = ZHPromise.defer();
        General.callRestEngine("df#/crud/mdfe_emissao", "FilterData", filter).then (response => {
            if (!response.dataset.mdfe_emissao.length) {
                return;
            }
            widget.currentRow = response.dataset.mdfe_emissao[0];
            for (var key in widget.dataSource.data) {
                if (widget.dataSource.data[key].NRLANCTOMDFE == widget.currentRow.NRLANCTOMDFE) {
                    widget.dataSource.data[key] = widget.currentRow;
                }
            }
            widget.reload();
            this.controlaVisibiliadeCabecalho(widget);
        });
        return deffered;
    };
    
    this.acaoMdfeSefaz = function (widget, route, message, cancelamento = false) {
        var self = this;
        var currentRow = widget.currentRow;
        route = "df#/" + route;
        
        if (!widget.isValid()) {
            return;
        }
    
        this.buildFilterTransmissao(currentRow).then((filter) => {
            if (cancelamento) {
                filter.push({name: "MOTCANCELA", operator: "=", value: currentRow.MOTCANCELA});
                if (currentRow.MOTCANCELA.length < 15) {
                    MessageService.showMessageByCode("MDFE_CANCELA_MENSAGEM_CURTA");
                    return;
                }
            }
            MessageService.confirmMessageByCode(message).then (success => {
                General.callRestEngine(route, "FilterData", filter).then (response => {
                    if (cancelamento) {
                        this.controlaStatusWidgets(widget.getParent(), widget.name, true);
                        ScreenService.closePopup();
                        widget = widget.getParent();
                    }
                    if (response.messages) {
                        var resposta = response.messages[0].message;
                        if (resposta.isError) {
                            MessageService.showMessageByCode(resposta.message, {"PARAMETRO" : resposta.motivo});
                        }
                        else {
                            MessageService.showMessageByCode(resposta.message);
                        }
                    }
                    else {
                        ScreenService.showMessage(response);
                    }
                    
                    self.reloadRowMdfe(widget, filter);
                    ScreenService.closePopup();
                }).catch((error) => {
                    self.reloadRowMdfe(widget, filter);
                    ScreenService.showMessage(error.data.error);
                    ScreenService.closePopup();
                });
            });
        });
    };
    
    this.cancelarMdfe = function (widget) {
        var widgetCancelamento = widget.container.getWidget('cancelamento');
        
        widgetCancelamento.currentRow = widget.currentRow;
        
        ScreenService.openPopup(widgetCancelamento);
        this.controlaStatusWidgets(widget.getParent(), widgetCancelamento.name, false);
        
        widgetCancelamento.reload();
    };
    
    this.visualizarDamdfe = function (widget) {
        var currentRow = widget.currentRow;
    
        this.buildFilterTransmissao(currentRow).then((filter) => {
                General.callRestEngine("df#/downloadDamdfe", "FilterData", filter).then (response => {
                    response.dataset.downloadDamdfe.forEach (fileUri => {
                        this.downloadArquivo(fileUri);
                    });
                }).catch((error) => {
                    ScreenService.showMessage(error.data.error);
                });
        });
        
    };
    
    /** 
     * Funções Gerais
     */
     this.controlaStatusWidgets = function(widgetPai, nomeWidgetAtual, status, ignoredWidgets) {
        var allWidgets = widgetPai.widgets;

        allWidgets.filter(widget => widget.name !== nomeWidgetAtual).forEach(widget => widget.enabled = status);

        if (widgetPai.name !== nomeWidgetAtual)
            widgetPai.enabled = status;

        if (Array.isArray(ignoredWidgets))
            allWidgets.filter(widget => ignoredWidgets.indexOf(widget.name) !== -1).forEach(widget => widget.enabled = !status);
    };
    
    this.validaInscricaoResponsavel = function(field, nmFieldTpInsc) {
        var value = field.value();
        if (!value) {
            return;
        }
        let obj = General.inscricaoFormat(value);
        var inscricoesValidas;
        if (nmFieldTpInsc) {
            inscricoesValidas = ["J", "F"];
        }
        else {
            inscricoesValidas = ["J"];
        }
        
        if(!obj.valid || !inscricoesValidas.includes(obj.tipo.cod)) {
            field.setValue('');
            if(nmFieldTpInsc) {
                field.widget.currentRow[nmFieldTpInsc] = '';
                switch (nmFieldTpInsc) {
                    case 'IDTPINSJURRESPGER': MessageService.showMessageByCode("INSCRICAO_SEGURADORA_SEGURO_INVALIDA"); break;
                    case 'IDTPINSJURRESP'   : MessageService.showMessageByCode("INSCRICAO_RESPONSAVEL_GERACAO_INVALIDA"); break;
                }
            }
            else {
                MessageService.showMessageByCode("INSCRICAO_RESPONSAVEL_SEGURO_INVALIDA");
            }
        }
        else {
            field.setValue(obj.value);
            field.widget.reload();
            if(nmFieldTpInsc) {
                field.widget.currentRow[nmFieldTpInsc] = obj.tipo.cod;
            }
        }
    };
    
    this.downloadArquivo = function(nmArquivo){
        window.open(window.serviceUrls.filter(function(x){return x.package == 'df';})[0].baseUrl+"/downloadArquivo?requestType=Row&row[DSARQUIVO]=" + nmArquivo);
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Dfs85000_ManifestoDocumentoEletronico_Controller", Dfs85000_ManifestoDocumentoEletronico_Controller);
});


// FILE: ./assets/modules/saida/assets/js/controllers/Dfs88000_Conhecimento_de_transporte_eletronico_Controller.js
function Dfs88000_Conhecimento_de_transporte_eletronico_Controller(ScreenService, templateManager, ZhFilterPreferences, General, MessageService, Dfs88000_Conhecimento_de_transporte_eletronico_Service) {
    
    //Nome das fields. Estilo: IDCAD + name[0], name[1] + NMRAZSOCCLIE, name[1] + NMRAZSOCFORN e NMRAZSOZ + name[2]. Exemplo: IDCADREMETENTE, REMETNMRAZSOCCLIE, REMETNMRAZSOCFORN e NMRAZSOCREME
    var fieldNames = [["REMETENTE", "REMET", "REME"], ["EXPEDIDOR", "EXP", "EXPE"], ["RECEBEDOR", "RECEB", "RECE"], ["DESTINATA", "DEST", "DEST"], ["TOMADOR", "TOMAD", "TOMA"]];
        
    
    this.widgetOnEnter = function (widget) {
        this.setDefaultFilterCTE(widget);
        let fieldStatus = widget.getField('IDSTATUSCTE');
        fieldStatus.setValue(fieldStatus.defaultValue);
    };
    
    this.setDefaultFilterCTE = function(widget) {
        var currentRow = widget.currentRow;
        var row        = { CDFILIAL: getLocalVar('CDFILIAL')};
        
        General.callRestEngine("df_saida#/defaultFilterCTE", "Row", row).then((defaultFilter) =>  {
            defaultFilter = defaultFilter.dataset.defaultFilter;
            var keys = ['DTEMISSAODEFAULT'];
            var filter = Util.clone(currentRow, ["__rightActions", "__leftActions"]);
            keys.map((key) => {
                if(key == 'DTEMISSAODEFAULT') {
                    filter.DTEMISSAO = {
                        start:  defaultFilter[key],
                        period: undefined,
                        end:    defaultFilter[key]
                    };
                }
                filter[key] = defaultFilter[key];
            });
            ZhFilterPreferences.setOriginalFilter(widget.id, filter);
            ZhFilterPreferences.openFilterWidget(widget, filter).then ( () => {
                var filterWidget = templateManager.container.getActiveWidget();
                filterWidget.actions.filter(action => action.name === "clear")[0].shortcutKey = "ctrl+backspace";
                filterWidget.getField('NRINSESTEMIT').readOnly = !filterWidget.currentRow.CDEMPRESA;
                filterWidget.getField("NRINSESTEMIT").dataSourceFilter[0].value = filterWidget.currentRow.CDEMPRESA;
                //Inicializa os campos "Status", Remetente", "Expedidor", "Recebedor", "Destinatário" e "Tomador" com a opção "Todos"
                for (let fieldName of ['IDSTATUSCTE', 'IDCADREMETENTE', 'IDCADEXPEDIDOR', 'IDCADRECEBEDOR', 'IDCADDESTINATA', 'IDCADTOMADOR']) {
                    filterWidget.getField(fieldName).setValue('');
                }
                filterWidget.getField('IDCADREMETENTE').setValue('');
            });
        },
        (error) => {
            ZhFilterPreferences.openFilterWidget(widget).then ( () => {
                var filterWidget = templateManager.container.getActiveWidget();
                filterWidget.actions.filter(action => action.name === "clear")[0].shortcutKey = "ctrl+backspace";
            });
        });
    };
    
    this.newDataSourceBeforeApply = function(args) {
        args.e.preventDefault();
        let row = args.e.preference.FILTER_ROW;
        let widget = args.owner;
        
        widget.dataSourceFilter = Dfs88000_Conhecimento_de_transporte_eletronico_Service.buildFilter(row);
        
        widget.reload().then ( () => {
            ScreenService.closePopup(); 
        });
        General.callRestEngine("df_saida#/getDefaultCte", "Row", row).then((success) => {
            Dfs88000_Conhecimento_de_transporte_eletronico_Service.setDefaultCte(success.dataset.cte);
        });
    };
    
    this.validacoesAfterMoveRow = function (widget) {
        this.setDataCampoInscricao(widget);
        this.controlaStatusWidgets(widget, widget.name, true);
    };
    
    this.empresaOnChange = function(widget) {
        let activeWidget = templateManager.container.getActiveWidget();
        
        if (activeWidget.name == 'zhFilterPreferencesSelectConditions')
            activeWidget.getField("NRINSESTEMIT").dataSourceFilter[0].value = activeWidget.currentRow.CDEMPRESA;
        
        widget.currentRow.CDINSCESTA = '';
        widget.currentRow.NRINSESTEMIT = '';
        if(!widget.currentRow.CDEMPRESA)   widget.getField('NRINSESTEMIT').readOnly = true;
        else widget.getField('NRINSESTEMIT').readOnly = false;
        
        this.limpaDadosEndereco(widget, "EMIT");
        
        widget.getField('NRINSESTEMIT').reload();
    };
    
    this.paisEstadoOnChange = function(fieldChanged) {
        var fieldPais, fieldEstado, fieldMunicipio;
        var widget = fieldChanged.widget;
        if (['NMPAISINISERV','NMESTADOINISERV'].includes(fieldChanged.name)) {
            fieldPais      =   widget.getField('NMPAISINISERV');
            fieldEstado    = widget.getField('NMESTADOINISERV');
            fieldMunicipio =  widget.getField('NMMUNICINISERV');
        }
        else if (['NMPAISFIMSERV','NMESTADOFIMSERV'].includes(fieldChanged.name)) {
            fieldPais      =   widget.getField('NMPAISFIMSERV');
            fieldEstado    = widget.getField('NMESTADOFIMSERV');
            fieldMunicipio =  widget.getField('NMMUNICFIMSERV');
        }
        else {
            return;
        }
        
        if (fieldChanged == fieldPais) {
            fieldEstado.setValue('');
            fieldEstado.reload();
            fieldMunicipio.readOnly = true;
        }
        fieldMunicipio.setValue('');
        fieldMunicipio.reload();
        
        var pais      =      fieldPais.value();
        var estado    =    fieldEstado.value();
        var municipio = fieldMunicipio.value();
        
        if (!pais) {
            fieldEstado.setValue('');
            fieldEstado.readOnly = true;
            fieldMunicipio.setValue('');
            fieldMunicipio.readOnly = true;
        }
        else {
            fieldEstado.readOnly = false;
            fieldMunicipio.readOnly = false;
        }
        if (!estado) {
            fieldMunicipio.setValue('');
            fieldMunicipio.readOnly = true;
        }
        else {
            fieldMunicipio.readOnly = false;
        }
    };
    
    this.retiradaOnChange = function(field) {
        var widget = field.widget;
        var fieldDetalhes = widget.getField('DSRETIRA');

        if (field.value() == field.valueToTrue) {
            fieldDetalhes.readOnly = false;
        }
        else {
            fieldDetalhes.readOnly = true;
            fieldDetalhes.setValue('');
        }
    };
    
    this.tipoInscricaoOnChange = function(widget) {
        this.alteraVisibilidadeCampoInscricao(widget, true);
    };
    
    this.limpaDadosEndereco = function (widget, typeName) {
        //Limpa os dados de endereço, telefone e e-mail.
        //typeName: nome dos tipos de campo. Ex: EMIT, REME, RECE
        for (let baseName of ['NRTELE', 'DSEMAIL', 'DSENDE', 'DSCOMPEND', 'NRCEP', 'NMPAIS', 'NMESTADO', 'NMMUNICIPIO', 'NMBAIR', 'CDPAIS', 'SGESTADO', 'CDMUNICIPIO']) {
            let fieldName = baseName + typeName;
            widget.currentRow[fieldName] = '';
        }
    };
    
    this.setDataCampoInscricao = function (widget) {
        let fieldNrinsestemit = widget.getField('NRINSESTEMIT');
        if (fieldNrinsestemit) {
            fieldNrinsestemit.setValue(widget.currentRow.NRINSESTEMIT);
            fieldNrinsestemit.reload();
        }
        
        for (let tipo of fieldNames) {
            let nameFieldTipo = "IDCAD" + tipo[0];
            let namefieldCliente = tipo[1] + "NMRAZSOCCLIE";
            let namefieldFornecedor = tipo[1] + "NMRAZSOCFORN";
            let nameRazSocField = "NMRAZSOC" + tipo[2];
            
            let fieldCliente = widget.getField(namefieldCliente);
            let fieldFornecedor = widget.getField(namefieldFornecedor);
            let razSoc = widget.currentRow[nameRazSocField];
            
            if (!fieldCliente || !fieldFornecedor) {
                continue;
            }
            
            switch (widget.currentRow[nameFieldTipo]) {
                case "C":
                    fieldCliente.setValue(razSoc);
                    break;
                case "F":
                    fieldFornecedor.setValue(razSoc);
                    break;
                default:
            }
            
        }
    };
    
    this.setFieldGroupVisibleStatus = function(fieldgroups, name, status){
	    var fieldgroup = fieldgroups.find(function(fieldgroup){
            return fieldgroup.name === name;
        });

        fieldgroup.isVisible = status;
	};
    
    this.alteraVisibilidadeCampoInscricao = function(widget, limpaValues = true) {
        if (widget.name == "main_cte") {
            return;
        }

        for (let tipo of fieldNames) {
            let nameFieldTipo = "IDCAD" + tipo[0];
            let namefieldCliente = tipo[1] + "NMRAZSOCCLIE";
            let namefieldFornecedor = tipo[1] + "NMRAZSOCFORN";
            
            let fieldCliente = widget.getField(namefieldCliente);
            let fieldFornecedor = widget.getField(namefieldFornecedor);
            
            if (!fieldCliente || !fieldFornecedor) {
                continue;
            }
            
            switch (widget.currentRow[nameFieldTipo]) {
                case "C":
                    this.validacaoCamposOutTipoFilter(fieldCliente, fieldFornecedor);
                    if (limpaValues) {
                        widget.currentRow[namefieldFornecedor] = null;
                        this.limpaDadosEndereco(widget, tipo[2]);
                    }
                    break;
                case "F":
                    this.validacaoCamposOutTipoFilter(fieldFornecedor, fieldCliente);
                    if (limpaValues) {
                        widget.currentRow[namefieldCliente] = null;
                        this.limpaDadosEndereco(widget, tipo[2]);
                    }
                    break;
                default:
                    this.validacaoCamposOutTipoFilter(fieldCliente, fieldFornecedor);
                    fieldCliente.readOnly = true;
                    delete fieldCliente.validations;
                    if (limpaValues) {
                        widget.currentRow[namefieldCliente] = null;
                        widget.currentRow[namefieldFornecedor] = null;
                        this.limpaDadosEndereco(widget, tipo[2]);
                    }
            }
        }
        
        widget.reload();
    };
    
    this.novoCte = function (widget) {
        widget.newRow();
        let currentRow = widget.currentRow;   
        Dfs88000_Conhecimento_de_transporte_eletronico_Service.getDefaultCte().then((defaultCte) => {
            this.controlaStatusWidgets(widget, widget.name, false);
            currentRow = Object.assign(currentRow, defaultCte[0]);
            currentRow.__is_new = true;
            
            widget.getField("NMFANTEMIT").reload();
            widget.getField("NRINSESTEMIT").reload();
            
            this.paisEstadoOnChange(widget.getField("NMPAISINISERV"));
            this.paisEstadoOnChange(widget.getField("NMPAISFIMSERV"));
            
            widget.getField("DSRETIRA").readOnly = true;
        });
    };
    
    this.salvarCte = function (widget) {
        var widgetCte = widget.name == 'main_cte' ? widget : widget.getParent();
        var currentRow = widget.currentRow;
        
        if (!widget.isValid()) {
            return;
        }
        
        //Ao criar um novo CT-e, alterar para a aba seguinte antes de salvar no banco. (Repetir até a aba de Tomador)
        if (currentRow.__is_new && widget.name != 'tomador_cte') {
            let nextWidget;
            
            if (widget.name == 'main_cte') {
                nextWidget = widget.widgets.filter( widget => widget.name == 'emitente_cte')[0];
            }
            else {
                let currentIndex;
                widget.getParent().widgets.forEach( (wdg, index) => {
                    if (wdg.name == widget.name) {
                        currentIndex = index;
                    }
                });
                nextWidget = widget.getParent().widgets[currentIndex+1];
            }
            widget.view();
            this.controlaStatusWidgets(widgetCte, nextWidget.name, false);
            nextWidget.enabled = true;
            nextWidget.activate();
            nextWidget.edit();
            return;
        }
        
        General.callRestEngine("df_saida#/salvarCte", "Row", currentRow).then (response => {
            let newRow = response.messages[0].message;
            if (newRow) {
                Object.assign(currentRow, newRow);
            }
            if (widget.name === 'totais_cte') {
                this.totaisChangeVisibility(widget, true);
                widget.widgets.forEach(widgetFilha => {
                    widgetFilha.actions.filter(action => action.name == "salvarTotais")[0].isVisible = false;
                    widgetFilha.actions.filter(action => action.name == "cancelar")[0].isVisible = false;
                    widgetFilha.actions.filter(action => action.name == "salvar")[0].isVisible = true;
                });

            }
            widget.view(); 
            widgetCte.view();
            widget.reload();
            this.controlaStatusWidgets(widgetCte, widget.name, true);
            this.setDataCampoInscricao(widget);
        }).catch(error => {
            ScreenService.showMessage(error.error);
        });
    };
    
    this.widgetCteOnEdit = function(widgetCte) {
        this.controlaStatusWidgets(widgetCte, 'main_cte', false);
        General.edit(widgetCte);
        
        //Define campos de País, Estado e Município para poderem ser alterados.
        let fieldPaisInicio = widgetCte.getField('NMPAISINISERV');
        let fieldEstadoInicio = widgetCte.getField('NMESTADOINISERV');
        let fieldMunicipioInicio = widgetCte.getField('NMMUNICINISERV');
        
        let fieldPaisFim = widgetCte.getField('NMPAISFIMSERV');
        let fieldEstadoFim = widgetCte.getField('NMESTADOFIMSERV');
        let fieldMunicipioFim = widgetCte.getField('NMMUNICFIMSERV');
        
        fieldPaisInicio.readOnly = false;
        fieldEstadoInicio.readOnly = false;
        fieldMunicipioInicio.readOnly = false;
        
        fieldPaisFim.readOnly = false;
        fieldEstadoFim.readOnly = false;
        fieldMunicipioFim.readOnly = false;
    };
    
    this.widgetFilhaOnEnter = function(widget) {
        widget.currentRow = widget.getParent().currentRow;
        this.setDataCampoInscricao(widget);
        this.alteraVisibilidadeCampoInscricao(widget, false);
        widget.redraw();
    };
    
    this.widgetFilhaOnEdit = function(widget) {
        var widgetCte = widget.getParent();
        this.controlaStatusWidgets(widgetCte, widget.name, false);
        this.setDataCampoInscricao(widget);
        General.edit(widget);
    };
    
    this.widgetFilhaOnCancel = function(widget) {
        var widgetCte = widget.getParent();
        var currentRow = widget.currentRow;
        
        //Se a row é nova, retorna para a grip principal, pois ela ainda não foi salva no banco.
        if (currentRow.__is_new) {
            ScreenService.closeSwipe();
            widget.getParent().dataSource.data.pop();
        }
        else {
            widget.view();
            //Retorna a row para seu valor antes da edição
            for (var i in widget.backupRow) {
				widget.currentRow[i] = widget.backupRow[i];
			}
        }
        this.controlaStatusWidgets(widgetCte, widget.name, true);
    };
    
    this.dadosAdicOnEnter = function (widget) {
        widget.reload();
    };
    
    this.dadosAdicOnCancel = function(widget) {
        var widgetCte = widget.getParent();
        
        widget.list();
        widget.reload();
        this.controlaStatusWidgets(widgetCte, widget.name, true);
    };
    
    this.dadosAdicOnSave = function (widget) {
        var currentRow = widget.currentRow;
        
        if (!widget.isValid()) return;
        
        //Se a row é nova, define valores de CDCTEDADOSADIC e CDCTE
        if (currentRow.__is_new) {
            //Busca maior CDCTEDADOSADIC já existente
            let maxCDCTEDADOSADIC = 0;
            widget.dataSource.data.forEach(row => {
                if (row.CDCTEDADOSADIC > maxCDCTEDADOSADIC) {
                    maxCDCTEDADOSADIC = row.CDCTEDADOSADIC;
                }
            });
            maxCDCTEDADOSADIC++;
            
            currentRow.CDCTEDADOSADIC = maxCDCTEDADOSADIC;
            currentRow.CDCTE = widget.getParent().currentRow.CDCTE;
        }
        
        widget.saveRow().then (() => {
            widget.list();
            widget.reload();
        }).catch(error => {
            ScreenService.showMessage(error.error);
        });
        
    };
    
    this.dadosAdicOnDelete = function (widget) {
        MessageService.confirmMessageByCode("REGISTRO_EXCLUIR").then (() => {
            widget.removeRow().then (() => {
                widget.list();
                widget.reload();
            });   
        });
    };
    
    this.totaisChangeValidations = function (widget) {
        var row = widget.currentRow;
        if(row.VRTOTAPRESTSERV != null)
            delete widget.getField('VRTOTAPRESTSERV').validations.required;
        else 
            widget.getField('VRTOTAPRESTSERV').validations.required = true;
       
        if(row.DSOUTCARCARGA != null)
            delete widget.getField('DSOUTCARCARGA').validations.required;
        else
            widget.getField('DSOUTCARCARGA').validations.required = true;
            
        if(row.VRTOTARECEBER != null)
            delete widget.getField('VRTOTARECEBER').validations.required;   
        else 
            widget.getField('VRTOTARECEBER').validations.required = true;
            
    };
    
    this.totaisOnEnter = function (widget) {
        this.totaisOnCancel(widget);
        this.widgetFilhaOnEnter(widget);
        this.totaisChangeValidations(widget);
        widget.reload();
    };
    
    //Coloca o widget Totais "manualmente" no modo de edição.
    //Isso é necessário pois o widget é do estilo tabbedWidget
    this.totaisOnEdit = function (widget) {
        var widgetCte = widget.getParent();
        this.controlaStatusWidgets(widgetCte, widget.name, false);
        
        widget.backupRow = Util.clone(widget.currentRow);
        
        this.totaisChangeVisibility(widget, true);
        
        widget.widgets.forEach(widgetFilha => {
            widgetFilha.actions.filter(action => action.name == "salvarTotais")[0].isVisible = true;
            widgetFilha.actions.filter(action => action.name == "cancelar")[0].isVisible = true;
            widgetFilha.actions.filter(action => action.name == "salvar")[0].isVisible = false;
        });
            
        widget.edit();
    };
    
    //Coloca o widget Totais "manualmente" no modo de exibição.
    //Isso é necessário pois o widget é do estilo tabbedWidget
    this.totaisOnCancel = function (widget) {
        var widgetCte = widget.getParent();
        this.controlaStatusWidgets(widgetCte, widget.name, true);
        
        if (widget.backupRow && (widget.currentRow.CDCTE == widget.backupRow.CDCTE)) {
            //Retorna a row para seu valor antes da edição
            for (var i in widget.backupRow) {
    			widget.currentRow[i] = widget.backupRow[i];
    		}  
        }
        
        this.totaisChangeVisibility(widget, false);
        this.totaisChangeValidations(widget);
        this.widgetFilhaOnEnter(widget);
        widget.reload();
    };
    
    
    this.totaisOnSave = function (widget) {
        widget.saveRow().then (() => {
            widget.list();
            this.totaisChangeValidations(widget);
            widget.reload();
            this.salvarCte(widget);
        }).catch(error => {
            ScreenService.showMessage(error.error);
        });
    };
    //Altera a visibilidade das ações e readOnly dos campos da widget Totais.
    //Parâmetro edicao: Indica se a widget deve ir para o modo de edição
    this.totaisChangeVisibility = function (widget, edicao ) {
        
        if(edicao) {
            widget.fields.forEach(field => {
                field.readOnly = false;
            });
            
            //Altera a visibilidade das ações das widget filhas
            widget.widgets.forEach(widgetFilha => {
                widgetFilha.actions.forEach(action => {
                    switch (action.name) {
                        case 'editar':
                        case 'voltar': action.isVisible = false; break;
                        
                        case 'salvar':
                        case 'cancelar': action.isVisible = true; break;
                        
                        default:
                    }
                });
            });
        } else {
            //Seta ReadOnly dos campos
            widget.fields.forEach(field => {
                field.readOnly = true;
            });
            //Altera a visibilidade das ações das widget filhas
            widget.widgets.forEach(widgetFilha => {
                widgetFilha.actions.forEach(action => {
                    switch (action.name) {
                        case 'editar':
                        case 'voltar': action.isVisible = true; break;
                        
                        case 'salvar':
                        case 'cancelar': action.isVisible = false; break;
                        
                        default:
                    }
                });
            });
        }
        
    };
    
    this.adicionarCmpValorCte = function (widgetTotais) {
        var widget = templateManager.container.getActiveWidget();
        widget.newRow();
        widget.reload();
        this.controlaStatusWidgets(widget, widget.name, true, widgetTotais);

    };
    
    this.widgetCteOnCancel = function (args) {
        var widget = args.owner.widget;
        General.backOrCancel(args);    
        this.controlaStatusWidgets(widget, widget.name, true);
    };
    
   /** 
    * Funções Gerais
    */
    
    this.validacaoCamposOutTipoFilter = function(principalField) {
        principalField.showOnView = true;
        principalField.showOnForm = true;
        principalField.isVisible = true;
        principalField.readOnly = false;
        principalField.readOnlyOnEdit = false;
        principalField.validations = {"required": true};

        for (var i = 1; i < arguments.length; i++) {
            arguments[i].showOnView = false;
            arguments[i].showOnForm = false;
            arguments[i].isVisible = false;
            arguments[i].readOnly = true;
            delete arguments[i].validations;
        }
    };
    
    this.controlaStatusWidgets = function(widgetPai, nomeWidgetAtual, status, ignoredWidgets) {
        var allWidgets = widgetPai.widgets;

        allWidgets.filter(widget => widget.name !== nomeWidgetAtual).forEach(widget => widget.enabled = status);

        if (widgetPai.name !== nomeWidgetAtual)
            widgetPai.enabled = status;

        if (Array.isArray(ignoredWidgets))
            allWidgets.filter(widget => ignoredWidgets.indexOf(widget.name) !== -1).forEach(widget => widget.enabled = !status);
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Dfs88000_Conhecimento_de_transporte_eletronico_Controller", Dfs88000_Conhecimento_de_transporte_eletronico_Controller);
});


// FILE: ./assets/modules/saida/assets/js/controllers/Grs15000_ExportacaoNfFaturamentoSap_Controller.js
function Grs15000_ExportacaoNfFaturamentoSap_Controller(ScreenService, templateManager, General, Grs15000_ExportacaoNfFaturamentoSap_Service, ZhFilterPreferences) {
    
    var msg = {
        "SUCESSO"         : "Notas Fiscais exportadas com sucesso.",
        "NOTAS_EXPORTADAS": "Existem notas que ja foram exportadas."
    };
    
    this.openFilterExportacao = function (widget){
        var dataAtual = moment().format('DD/MM/YYYY');
        var start = dataAtual;
        var end   = dataAtual;
        var rowFilter = {};
        
        rowFilter = {
 	        'DTENTRSAID': {start : start, end : end}
 	    };
 	    
 	    
 	    General.callRestEngine("df_saida#/getFilterExportSap", "Row", {}).then((response) => {
 	        if(response.dataset.filter) {
 	            rowFilter.CDEMPRESA = response.dataset.filter.CDEMPRESA;
 	            rowFilter.NMEMPRESA = response.dataset.filter.NMEMPRESA;
 	            rowFilter.CDFILIAL  = [response.dataset.filter.CDFILIAL];
 	            rowFilter.NMFILIAL  = [response.dataset.filter.NMFILIAL];
 	        }
            ZhFilterPreferences.setOriginalFilter(widget.id, rowFilter);
            ZhFilterPreferences.openFilterWidget(widget, rowFilter);
 	    },
 	    (error) => {
 	        ZhFilterPreferences.setOriginalFilter(widget.id, rowFilter);
            ZhFilterPreferences.openFilterWidget(widget, rowFilter);
 	    });
    };
    
    this.aplicaFiltro = function (widget, preference){
        var rowFiltro = preference.FILTER_ROW;
        var cdfilial  =  rowFiltro.CDFILIAL && rowFiltro.CDFILIAL.length>0 ?   "_" +  rowFiltro.CDFILIAL.toString().replace(/,/g, '_') + "_" : null;

        widget.dataSourceFilter = [
            {            
                "name":"CDEMPRESA",
                "operator":"=",
                "value":rowFiltro.CDEMPRESA
            },
            {
                "name":"DATAINI",
                "operator":"=",
                "value":rowFiltro.DTENTRSAID.start
            },
            {
                "name":"DATAFINAL",
                "operator":"=",
                "value":rowFiltro.DTENTRSAID.end
            },
            {
                "name":"CDFILIAL",
                "operator":"=",
                "value": cdfilial
            },
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": "OPERADOR_LOGADO"
            }
        ];
        
        if(rowFiltro.IMPORT)
            widget.dataSourceFilter.push(
                {
                    "name":"IMPORT",
                    "operator":"=",
                    "value":rowFiltro.IMPORT
                }
            );
            
        widget.reload();
        this.setStripeData(widget, preference);
    };
    
    this.exportarNf = function (widget){
        if(Grs15000_ExportacaoNfFaturamentoSap_Service.validaStatusNf(widget.getCheckedRows())){
            var rows = Grs15000_ExportacaoNfFaturamentoSap_Service.preparaRow(widget.getCheckedRows());
            General.callRestEngine('df_saida#/exportarNf','Row',rows).then(
                function (response)  {
                   widget.reload();
                   ScreenService.showMessage(msg.SUCESSO);
                },
                function (callback) {
                    if (callback.data) {
                        ScreenService.showMessage(callback.data.error);
                    }
                }  
            );    
        }else
            ScreenService.showMessage(msg.NOTAS_EXPORTADAS);
    };
    
    this.setStripeData = function(widget, preference) {
        var currentRow = preference.FILTER_ROW;
        var stripeWidget = templateManager.container.getWidget("stripe");

        stripeWidget.currentRow = {};
        stripeWidget.currentRow.NMEMPRESA   = currentRow.NMEMPRESA;
        stripeWidget.currentRow.DTENTRSAID  = 'De ' + currentRow.DTENTRSAID.start + ' até ' + currentRow.DTENTRSAID.end;
        stripeWidget.currentRow.CDFILIAL    =  currentRow.CDFILIAL;
        
        stripeWidget.getField('CDFILIAL').isVisible =  stripeWidget.currentRow.CDFILIAL ? true: false;
        
    };
    
    this.verificaEdicao = function (widget){
        widget.getAction('editar').readOnly = widget.currentRow.IMPORT !=='X' &&  widget.currentRow.IDAVISOCREDSAP ==='S' ? false: true; 
    };
    
    
   
    /*******************************************************************************************************
     * Se "Permite informar Aviso de Crédito" --  (this.verificaEdicao())
     * Se Aviso = "SIM " permitir alteração das colunas: Aviso, Cód. Único Aviso, Crédito e Descrição
     ****************************************************************************************************/
     this.HabilitaEdicao = function(widget) {
        if (widget.currentRow.IDAVISO === 'S'){
                widget.getField("CRCREDITO").readOnly = false; 
                  widget.getField("CDUNICOAVISO").readOnly = false; 
                   widget.getField("DSOBSERVCRED").readOnly = false; 
                    widget.getField("IDAVISO").readOnly = false; 
        } else if (widget.currentRow.IDAVISO === 'N'){
                widget.getField("CRCREDITO").readOnly = true; 
                  widget.getField("CDUNICOAVISO").readOnly = true; 
                   widget.getField("DSOBSERVCRED").readOnly = true; 
                    widget.getField("CRCREDITO").value (""); 
                    widget.getField("CDUNICOAVISO").value(""); 
                     widget.getField("DSOBSERVCRED").value(""); 
        } 
    };
    
    this.salvarNotaCredito = function (widget){
        var row = Grs15000_ExportacaoNfFaturamentoSap_Service.preparaRow(widget.currentRow);
        General.callRestEngine('df_saida#/salvarNotaCredito','Row',row).then(
            function (response)  {
                widget.activate();
                widget.view();
            },
            function (callback) {
                if (callback.data) {
                    ScreenService.showMessage(callback.data.error);
                }
            }  
        );
    };
    
    // Função criada somente para evitar o erro do zeedhi ao clicar na action no grid
    this.action = function(){};
    

    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Grs15000_ExportacaoNfFaturamentoSap_Controller", Grs15000_ExportacaoNfFaturamentoSap_Controller);
});


// FILE: ./assets/modules/relatorios/assets/js/controllers/Adm03000_ReNotasFiscais_Controller.js
function Adm03000_ReNotasFiscais_Controller(Adm03000_ReNotasFiscais_Service, ScreenService, General) {
    let widget;
    let REService = Adm03000_ReNotasFiscais_Service;

    this.checaTodosSomatorios = function(widget) {
        return this.checaSomatorio('NRCOUNTCGC', widget.getField("SOMACGC"))   &&
               this.checaSomatorio('NRCOUNTEMIS', widget.getField("SOMAEMIS")) &&
               this.checaSomatorio('VRTOTANOTA', widget.getField("SOMATOTA"))  &&
               this.checaSomatorio('NRCOUNTNUM', widget.getField("SOMANUMNF"));
    };

    this.geraRelatorio = function(widget) {
        if(widget.isValid() && this.checaTodosSomatorios(widget)) {
            if(widget.currentRow.IDENTRSAID === 'E') {
                if(REService.checaNotaEmpty()) {
                    ScreenService.showMessage('Não há dados a serem listados.');
                    return;
                }
            }

            widget.filter = new Object({});

            widget.filter.CDALMOXARIFE = widget.currentRow.CDALMOXARIFE || null;
            widget.filter.TAMNVTOT = widget.currentRow.TAMNVTOT;
            widget.filter.DATARE = widget.currentRow.DATARE;
            widget.filter.CDLOCALESTOQ = widget.currentRow.CDLOCALESTOQ || null;
            widget.filter.CDFILIAL = widget.currentRow.CDFILIAL;
            widget.filter.IDENTRSAID = widget.currentRow.IDENTRSAID;
            widget.filter.IDRETIICMS = widget.currentRow.IDRETIICMS;
            widget.filter.SALDOANT = widget.currentRow.SALDOANT || 0;
            widget.filter.TOTENTR = widget.currentRow.TOTENTR || 0;
            widget.filter.TOTSAID = widget.currentRow.TOTSAID || 0;
            widget.filter.__report_name = widget.currentRow.REPORT_NAME;

            REService.getNrorg().then(function(nrorg) {
                widget.filter.NRORG = nrorg;

                REService.buidlFilter(widget.filter).then(function(birtURL) {
                    window.open(birtURL.messages[0].message);
                }, function(error) {
                    ScreenService.showMessage(error.data.error);
                });
            });
        }
    };

    this.setParametersFilialLoagada = function(widgetGrid) {
        REService.setParametersFilial().then(function(params) {
            widget = widgetGrid;

            widget.currentRow.CDFILIAL = params[1];
            widget.getField('NMFILIAL').setValue(params[2]);
            widget.currentRow.IDRETIICMS = 'N';

            this.setReadAlmox(params[0]);
            this.setDataRe(params[3]);
            this.setReadSomato(widget.currentRow.IDENTRSAID);
        }.bind(this), function(error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    this.setParametersFilial = function() {
        widget.getField('DSALMOXARIFE').readOnly = true;
        widget.getField('DSALMOXARIFE').setValue(null);

        widget.getField('DSLOCALESTOQ').readOnly = true;
        widget.getField('DSLOCALESTOQ').setValue(null);

        widget.getField('DATARE').setValue(null);

        widget.getField('SOMACGC').setValue(null);
        widget.getField('SOMAEMIS').setValue(null);
        widget.getField('SOMATOTA').setValue(null);
        widget.getField('SOMANUMNF').setValue(null);

        REService.setParametersFilial().then(function(params) {
            this.setReadAlmox(params[0]);
            this.setDataRe(params[3]);
            this.setReadSomato(widget.currentRow.IDENTRSAID);
        }.bind(this), function(error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    this.setParamNota = ({CDFILIAL, CDALMOXARIFE = null, CDLOCALESTOQ = null, DATARE, IDENTRSAID}) =>
        REService.setParamNota(CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, DATARE)
            .then(this.setReadSomato.bind(this, IDENTRSAID))
            .catch(({data: {error: errorMessage}}) => ScreenService.showMessage(errorMessage));

    this.setReadAlmox = function(readOnly) {
        if(readOnly) {
            widget.getField('DSALMOXARIFE').readOnly = false;
            widget.getField('DSALMOXARIFE').reload();
        } else {
            widget.CDALMOXARIFE = ' ';
        }
    };

    this.setDataRe = function(data) {
        widget.getField('DATARE').setValue(data);
        widget.getField('DATARE').maxDate = data;
    };

    this.setReadLocal = function(value) {
        if(value === null) {
            widget.getField('DSLOCALESTOQ').readOnly = true;
        } else if(REService.checaUtilizaLocal()) {
            widget.getField('DSLOCALESTOQ').readOnly = false;
            widget.getField('DSLOCALESTOQ').reload();
        }
    };

    this.setReadSomato = function(value) {
        if(value === "E") {
            
            if(REService.allInvoicesImportedByXML())
                blockSumFields(widget);
            else {
                if(REService.checaIdconscgcre()) {
                    widget.getField('SOMACGC').readOnly = false;
                    setRequired(true, 'SOMACGC');
                } else {
                    widget.getField('SOMACGC').setValue(null);
                    setRequired(null, 'SOMACGC');
                }

                if(REService.checaIdconsemisre()) {
                    widget.getField('SOMAEMIS').readOnly = false;
                    setRequired(true, 'SOMAEMIS');
                } else {
                    widget.getField('SOMAEMIS').setValue(null);
                    setRequired(null, 'SOMAEMIS');
                }

                if(REService.checaIdconstotare()) {
                    widget.getField('SOMATOTA').readOnly = false;
                    setRequired(true, 'SOMATOTA');
                } else {
                    widget.getField('SOMATOTA').setValue(null);
                    setRequired(null, 'SOMATOTA');
                }

                if(REService.checaIdconsnumere()) {
                    widget.getField('SOMANUMNF').readOnly = false;
                    setRequired(true, 'SOMANUMNF');
                } else {
                    widget.getField('SOMANUMNF').setValue(null);
                    setRequired(null, 'SOMANUMNF');
                }
            }
        } else
            blockSumFields(widget);
    };

    const blockSumFields = widget => {
        const sumFields = ['SOMACGC', 'SOMAEMIS', 'SOMATOTA', 'SOMANUMNF'];
        const clearField = fieldName => {
            widget.getField(fieldName).readOnly = true;
            widget.getField(fieldName).setValue(null);
            setRequired(null, fieldName);
        };

        sumFields.forEach(clearField);
    };

    this.setDataIfMaskForTamNivel = function(field) {
        var levels = General.getLevelByMask("GER_CDPICTPROD", 1);

        for(var index in levels) {
            if(levels.hasOwnProperty(index)) {
                field.dataSource.data.push({
                    "name": levels[index].nivel,
                    "value": levels[index].nivel
                });
            }
        }

        field.setValue(1);
        this.setDefaultValue(field.widget);
    };

    this.reportSelect = function(value) {
        if(value === 'ADM03001') {
            widget.getField('IDENTRSAID').setValue('S');
            widget.getField('IDENTRSAID').readOnly = true;

            this.setReadSomato(widget.currentRow.IDENTRSAID);
        } else {
            widget.getField('IDENTRSAID').readOnly = false;
        }
    };

    const setRequired = (required, fieldName) => {
        if(required)
            widget.getField(fieldName).validations.required = true;
        else
            delete widget.getField(fieldName).validations.required;
    };

    this.checaSomatorio = function(nameSomatorio, field) {
        if(field.value() !== null) {
            if(nameSomatorio === 'NRCOUNTCGC') {
                if(REService.getValueValidSoma('NRCOUNTCGC') !== null) {
                    if(field.value() != REService.getValueValidSoma('NRCOUNTCGC')) {
                        field.setValue(null);

                        ScreenService.showMessage('Somatório do controle de C.G.C/C.P.F difere do cadastro.');
                        return false;
                    }
                } else if(parseFloat(field.value()) !== 0) {
                    field.setValue(null);

                    ScreenService.showMessage('Somatório do controle de C.G.C/C.P.F difere do cadastro.');
                    return false;
                }
            } else if(nameSomatorio === 'NRCOUNTEMIS') {
                if(REService.getValueValidSoma('NRCOUNTEMIS') !== null) {
                    if(field.value() != REService.getValueValidSoma('NRCOUNTEMIS')) {
                        field.setValue(null);

                        ScreenService.showMessage("Somatório dos dias da emissão de NFs difere do cadastro.");
                        return false;
                    }
                } else if(parseFloat(field.value()) !== 0) {
                    field.setValue(null);

                    ScreenService.showMessage("Somatório dos dias da emissão de NFs difere do cadastro.");
                    return false;
                }
            } else if(nameSomatorio === 'VRTOTANOTA') {
                if(REService.getValueValidSoma('VRTOTANOTA') !== null) {
                    var value = '';

                    if(field.value().search(',') != -1) {
                        value = parseFloat(field.value().replace(/\./g, '').replace(',', '.'));
                    } else {
                        value = parseFloat(field.value());
                    }

                    if(value - REService.getValueValidSoma('VRTOTANOTA') > 0.05 
                    || value - REService.getValueValidSoma('VRTOTANOTA') < -0.05) {
                        field.setValue(null);

                        ScreenService.showMessage("Somatório dos valores totais das NFs difere do cadastro.");
                        return false;
                    }
                } else if(parseFloat(field.value()) !== 0) {
                    field.setValue(null);

                    ScreenService.showMessage("Somatório dos valores totais das NFs difere do cadastro.");
                    return false;
                }
            } else if(nameSomatorio === 'NRCOUNTNUM') {
                if(REService.getValueValidSoma('NRCOUNTNUM') !== null) {
                    if(field.value() != REService.getValueValidSoma('NRCOUNTNUM')) {
                        field.setValue(null);

                        ScreenService.showMessage("Somatório dos números das NFs difere do cadastro.");
                        return false;
                    }
                } else if(parseFloat(field.value()) !== 0) {
                    field.setValue(null);

                    ScreenService.showMessage("Somatório dos números das NFs difere do cadastro.");
                    return false;
                }
            }

            return true;
        } else
            return !!field.readOnly && !(field.validations && field.validations.required);
    };

    this.setDefaultValue = function(widget) {
        widget.getField('REPORT_NAME').setValue('ADM03000');
        widget.getField('IDENTRSAID').setValue('E');
        widget.getField('IDRETIICMS').setValue('S');
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Adm03000_ReNotasFiscais_Controller', Adm03000_ReNotasFiscais_Controller);
});


// FILE: ./assets/modules/relatorios/assets/js/controllers/Dfe16000_ConsistenciasAfNf_Controller.js
function Dfe16000_ConsistenciasAfNf_Controller(ScreenService, Dfe16000_ConsistenciasAfNf_Service) {

	var changeDateFieldValues = function(currentRow) {
		currentRow.P_DTINICIAL = !currentRow.DTINTERVALO ? null : currentRow.DTINTERVALO.start;
		currentRow.P_DTFINAL = !currentRow.DTINTERVALO ? null : currentRow.DTINTERVALO.end;
	};

	var concatNotasList = function(currentRow) {
		if (Array.isArray(currentRow.CHAVE) && currentRow.CHAVE.length > 0) {
			currentRow.L_CHAVE = '_' + currentRow.CHAVE.join('_') + '_';
		}
		else {
			currentRow.L_CHAVE = null;
		}
	};

	var concatUnidadeList = function(currentRow) {
		if (currentRow.T_CDFILIAL == "L" && Array.isArray(currentRow.CDFILIAL) && currentRow.CDFILIAL.length > 0) {
			currentRow.L_CDFILIRECE = '_' + currentRow.CDFILIAL.join('_') + '_';
		}
		else {
			currentRow.L_CDFILIRECE = null;
		}
	};

	this.loadNotasField = function(widget) {
		var listaNotasField = widget.getField("CHAVE");
		var currentRow = widget.currentRow;
		changeDateFieldValues(currentRow);
		Dfe16000_ConsistenciasAfNf_Service.getMainQueryData(widget.currentRow).then(function(data) {
			listaNotasField.dataSource.data = data.dataset.mainQueryData;
		});
	};

	var openReport = function(widget) {
		widget.currentRow.__report_name = widget.currentRow.NMRELATORIO;
		Dfe16000_ConsistenciasAfNf_Service.generateReport(widget.currentRow).then(function(response) {
			if (response && response.messages) {
				var report = response.messages.pop();
				window.open(report.message);
			}
		}, function(error) {
			var errorMessage = 'Timeout';
			if (error.data) {
				errorMessage = ScreenService.i18n(error.data.error);
			}
			ScreenService.showMessage(errorMessage, 'E');
		});
	};

	this.setDefaultData = function(widget) {
		widget.currentRow.T_CDFILIAL = "T";
		widget.currentRow.NMRELATORIO = "DFE16000";
	};

	this.toggleReadOnlyNotasOnChange = function(widget) {
		var listaNotasField = widget.getField("CHAVE");
		var currentRow = widget.currentRow;
		concatUnidadeList(currentRow);
		changeDateFieldValues(currentRow);
		if ((currentRow.CDFILIAL) && (currentRow.DTINTERVALO)) {
			listaNotasField.readOnly = false;
		}
		else {
			listaNotasField.readOnly = true;
		}
		listaNotasField.setValue(null);
	};

	this.toggleReadOnlyUnidadesOnChange = function(widget) {
		var listaFilialField = widget.getField("CDFILIAL");
		if (widget.currentRow.T_CDFILIAL == "T") {
			listaFilialField.readOnly = true;
			listaFilialField.validations = {};
		}
		else {
			listaFilialField.readOnly = false;
			listaFilialField.validations = {required: true};
		}
		widget.currentRow.CDFILIAL = [];
	};

	this.validateReport = function(widget) {
		if (widget.isValid()) {
			concatUnidadeList(widget.currentRow);
			concatNotasList(widget.currentRow);
			changeDateFieldValues(widget.currentRow);
			Dfe16000_ConsistenciasAfNf_Service.checkDataExistance(widget.currentRow).then(function(thereIsData) {
				if (thereIsData) {
					openReport(widget);
				}
				else {
					ScreenService.showMessage(ScreenService.i18n("Não há dados a serem listados"));
				}
			}, function(error) {
				var errorMessage = 'Timeout';
				if (error.data) {
					errorMessage = ScreenService.i18n(error.data.error);
				}
				ScreenService.showMessage(errorMessage, 'E');
			});
		}
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('Dfe16000_ConsistenciasAfNf_Controller', Dfe16000_ConsistenciasAfNf_Controller);
});


// FILE: ./assets/modules/relatorios/assets/js/controllers/Dfe21100_RomaneioNotasFiscaisEntrada_Controller.js

function Dfe21100_RomaneioNotasFiscaisEntrada_Controller(ScreenService, General, templateManager, ZhFilterPreferences) {

	this.openFilterRomaneio = function (widget){
        var dataAtual = moment().format('DD/MM/YYYY');
        var start = dataAtual;
        var end   = dataAtual;
        var rowFilter = {};
        
        rowFilter = {
 	        'DTENTRADA': {start : start, end : end},
 	       
 	    };
 	    
 	    
 	    General.callRestEngine("df#/crud/filial", "FilterData", [{name: "CDFILIAL", operator:"=", value: "FILIAL_LOGADA"}]).then((response) => {
 	        if(response.dataset.filial[0]) {
 	            rowFilter.CDFILIAL  = response.dataset.filial[0].CDFILIAL;
 	            rowFilter.NMFILIAL  = response.dataset.filial[0].NMFILIAL;
 	        }
            ZhFilterPreferences.setOriginalFilter(widget.id, rowFilter);
            ZhFilterPreferences.openFilterWidget(widget, rowFilter);
 	    },
 	    (error) => {
 	        ZhFilterPreferences.setOriginalFilter(widget.id, rowFilter);
            ZhFilterPreferences.openFilterWidget(widget, rowFilter);
            
 	    });
    };
    

	/**********************************
	*
	*   Métodos de validação do filtro
	*
	***********************************/

	this.validateFilterFields = function (args, preference) {
		args.e.preventDefault();
		var filterPopup =templateManager.container.getWidget('romaneioEntrada');
		var currentRow = preference.FILTER_ROW;
		
		if (!(currentRow && dataLancamentoIsValid(currentRow))) {
			args.e.preventDefault();
		}
		else {
			// Método que concatena os códigos escolhidos nas listas para
			// poder utilizar como String na query
			concatenaListas(preference);

			// Atribui os valores do field interval aos campos necessários
		    if (currentRow.hasOwnProperty('DTENTRADA')){
				if (currentRow.DTENTRADA.hasOwnProperty('start' && 'end')){
					currentRow.DTINICIAL 	= currentRow.DTENTRADA.start;
					currentRow.DTFINAL 		= currentRow.DTENTRADA.end;
			    }
		    }
		    
		    if (currentRow.hasOwnProperty('DTLANCAMENTO')){
				if (currentRow.DTLANCAMENTO.hasOwnProperty('start' && 'end')){
					if(currentRow.DTLANCAMENTO.start) {
						currentRow.DTINILANC 	= currentRow.DTLANCAMENTO.start;
						currentRow.DTFINLANC 	= currentRow.DTLANCAMENTO.end;
					}
					else {
				    	currentRow.TDTLANC = 'T';
					}
			    }
			    else {
			    	currentRow.TDTLANC = 'T';
			    }
		    }
		    
		    // Anula as propriedades extras geradas pelo field interval
			delete currentRow.start;
			delete currentRow.end;
			delete currentRow.period;
			if (currentRow.DTLANCAMENTO) {
				delete currentRow.DTLANCAMENTO;
				delete currentRow.DTLANCAMENTO;
			}

			// Verifica se não há filial escolhida para filtrar por todas
			currentRow.TCDFILIAL	= (!!currentRow.CDFILIAL) ? currentRow.CDFILIAL : "T";

			// Método que concatena os códigos escolhidos nas listas para
			// poder utilizar como String na query
			if(currentRow.NMOPERADOR) {
				currentRow.CDOPERADOR = concatenaFields(filterPopup.getField('NMOPERADOR'), currentRow);
			}
			else {
				currentRow.CDOPERADOR = ' ';
			}

			if(currentRow.NMTIPOOPER) {
				currentRow.CDTIPOOPER = concatenaFields(filterPopup.getField('NMTIPOOPER'), currentRow);
			}
			else {
				currentRow.CDTIPOOPER = ' ';
			}
			this.changeFilter(args.owner, preference);
		}

	};

	// Como todos os parâmetros da query são necessários (mesmo que sejam nulos),
	// este método substitui o filtro gerado por outro baseado na currentRow,
	// pois o zhGridFilter possui uma validação muito complexa para inserir campos
	// no filtro, não compensando tentar mexer com visibilidade/valor dos fields
	// não inclusos
	this.changeFilter = function (widget, preference) {
		var currentRow = templateManager.container.getWidget('romaneioEntrada');
		var filters = [
            {
                "name": "CDTIPOOPER",
                "operator": "=",
                "value": preference.FILTER_ROW.CDTIPOOPER || null
            },
            {
                "name": "CDFORNECEDOR",
                "operator": "=",
                "value": preference.FILTER_ROW.LCDFORNECED || null
            },
            {
                "name": "CDCLIENTE",
                "operator": "=",
                "value": preference.FILTER_ROW.LCDCLIENTE || null
            },
            {
                "name": "TCDOPERADOR",
                "operator": "=",
                "value": preference.FILTER_ROW.TCDOPERADOR || 'T'
            },
            {
                "name": "CDFILIDESTI",
                "operator": "=",
                "value": preference.FILTER_ROW.LCDFILIDESTI || null
            },
            {
                "name": "TCDFORNECEDOR",
                "operator": "=",
                "value": preference.FILTER_ROW.TCDFORNECED || null
            },
            {
                "name": "TCDFILIDESTI",
                "operator": "=",
                "value": preference.FILTER_ROW.TCDFILIDESTI || null
            },
            {
                "name": "DTFINLANC",
                "operator": "=",
                "value": preference.FILTER_ROW.DTFINLANC || null
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": preference.FILTER_ROW.CDFILIAL || null
            },
            {
                "name": "TCDFILIAL",
                "operator": "=",
                "value": preference.FILTER_ROW.TCDFILIAL || null
            },
            {
                "name": "TCDTIPOOPER",
                "operator": "=",
                "value": preference.FILTER_ROW.TCDTIPOOPER || 'T'
            },
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": preference.FILTER_ROW.CDOPERADOR || null
            },
            {
                "name": "DATAINICIAL",
                "operator": "=",
                "value": preference.FILTER_ROW.DATAINICIAL || null
            },
            {
                "name": "TDTLANC",
                "operator": "=",
                "value": preference.FILTER_ROW.TDTLANC || null
            },
            {
                "name": "DATAFINAL",
                "operator": "=",
                "value": preference.FILTER_ROW.DATAFINAL || null
            },
            {
                "name": "DTINICIAL",
                "operator": "=",
                "value": preference.FILTER_ROW.DTINICIAL || null
            },
            {
                "name": "DTFINAL",
                "operator": "=",
                "value": preference.FILTER_ROW.DTFINAL || null
            },
            {
                "name": "DTINILANC",
                "operator": "=",
                "value": preference.FILTER_ROW.DTINILANC || null
            },
            {
                "name": "TCDCLIENTE",
                "operator": "=",
                "value": preference.FILTER_ROW.TCDCLIENTE || null
            }
        ];

        preference.CUSTOM_FILTER_ROW = Util.clone(preference.FILTER_ROW);
        if (preference.CUSTOM_FILTER_ROW.hasOwnProperty('DATAINICIAL')){
            delete preference.CUSTOM_FILTER_ROW.DATAINICIAL;
        }
        if (preference.CUSTOM_FILTER_ROW.hasOwnProperty('DATAFINAL')){
            delete preference.CUSTOM_FILTER_ROW.DATAFINAL;
        }
        
		widget.dataSourceFilter = filters;
		
		widget.reload().then((success) => {
			ScreenService.closePopup();
			widget.activate();
		});
	};

	/***********************************
	*
	*  Métodos de alteração de exibição
	*
	************************************/

	this.defineDestinatarioRemetente = function(widget) {
		var currentRow = widget.currentRow;
		var fieldDestrem = widget.getField(currentRow.selectName);

		if(currentRow.IDORIGDESTNF == 'C') {

			if (fieldDestrem.name != 'CDCLIENTE') {

				fieldDestrem.name = 'CDCLIENTE';
				fieldDestrem.fields = [];
				fieldDestrem.fields.push(newField(widget, 'NRINSJURCLIE', 'Inscrição', true, false, "field/text-edit.html", {type: "fix", params: {mask: "99.999.999/9999-99"}}));
				fieldDestrem.fields.push(newField(widget, 'NMRAZSOCCLIE', 'Cliente', true, false, "field/text-edit.html"));
				fieldDestrem.fields.push(newField(widget, 'CDCLIENTE', 'cdCliente', false, false, "field/text-edit.html"));
				fieldDestrem.dataSource = metaDataFactory.factoryDataSource({name: 'df#/crud/cliente', rest: true, itemsPerPage: 40});
				fieldDestrem.valueField = "CDCLIENTE";
				fieldDestrem.descriptionField = "NMRAZSOCCLIE";
				fieldDestrem.order = "NRINSJURCLIE";
				fieldDestrem.outData = {
					CDCLIENTE: "CDCLIENTE"
				};

				if (fieldDestrem.selectWidget) {
					delete fieldDestrem.selectWidget;
				}
				currentRow.selectName = "CDCLIENTE";
			}

		}
		else if(currentRow.IDORIGDESTNF == 'F') {

			if (fieldDestrem.name != 'CDFORNECED') {

				fieldDestrem.name = 'CDFORNECED';
				fieldDestrem.fields = [];
				fieldDestrem.fields.push(newField(widget, 'NRINSJURFORN', 'Inscrição', true, false, "field/text-edit.html", {type: "fix", params: {mask: "99.999.999/9999-99"}}));
				fieldDestrem.fields.push(newField(widget, 'NMRAZSOCFORN', 'Fornecedor', true, false, "field/text-edit.html"));
				fieldDestrem.fields.push(newField(widget, 'CDFORNECED', 'cdFornecedor', false, false, "field/text-edit.html"));
				fieldDestrem.dataSource = metaDataFactory.factoryDataSource({name: 'df#/crud/fornecedor', rest: true, itemsPerPage: 40});
				fieldDestrem.valueField = "CDFORNECED";
				fieldDestrem.descriptionField = "NMRAZSOCFORN";
				fieldDestrem.order = "NRINSJURFORN";
				fieldDestrem.outData = {
					CDFORNECED: "CDFORNECED"
				};

				if (fieldDestrem.selectWidget) {
					delete fieldDestrem.selectWidget;
				}
				currentRow.selectName = "CDFORNECED";
			}

		}
		else if(currentRow.IDORIGDESTNF == 'I') {

			if (fieldDestrem.name != 'CDFILIALDEST') {

				fieldDestrem.name = 'CDFILIALDEST';
				fieldDestrem.fields = [];
				fieldDestrem.fields.push(newField(widget, 'CDFILIAL', 'Código', true, false, "field/text-edit.html"));
				fieldDestrem.fields.push(newField(widget, 'NMFILIAL', 'Unidade', true, false, "field/text-edit.html"));
				fieldDestrem.dataSource = metaDataFactory.factoryDataSource({name: 'df#/crud/filial', rest: true, itemsPerPage: 40});
				fieldDestrem.valueField = "CDFILIAL";
				fieldDestrem.descriptionField = "NMFILIAL";
				fieldDestrem.order = "CDFILIAL";
				fieldDestrem.outData = {
					CDFILIAL: "CDFILIALDEST"
				};

				if (fieldDestrem.selectWidget) {
					delete fieldDestrem.selectWidget;
				}
				currentRow.selectName = "CDFILIALDEST";
			}
		}
	};

	this.limpaFieldsDependentes = function(currentRow) {
		currentRow.TCDCLIENTE = '';
		currentRow.CDCLIENTE = '';

		currentRow.TCDFORNECED = '';
		currentRow.CDFORNECED = '';

		currentRow.TCDFILIDESTI = '';
		currentRow.CDFILIDESTI = '';
	};

	var newField = function(widget, name, label, isVisible, readOnly, template, mask) {

		var newField = {
			"name": name,
			"label": label,
			"isVisible": isVisible,
			"showOnList": true,
			"readOnly": readOnly,
			"template": template,
			"mask": mask
		};

		return newField;
	};

	this.readOnlySelectDestRem = function(widget) {
		var destRemCheckbox = widget.getField('TDESTREM');
		widget.currentRow.selectName = widget.currentRow.selectName || "SELECTDESTREM";
		var destRemSelectField = widget.getField(widget.currentRow.selectName);

		if(widget.currentRow.IDORIGDESTNF == 'Todos') {
			destRemCheckbox.readOnly = true;
		}
		else {
			destRemCheckbox.readOnly = false;
		}

		destRemCheckbox.checked  = true;
		destRemCheckbox.setValue('T');

		destRemSelectField.readOnly = true;
		destRemSelectField.setValue("");
		destRemSelectField.validations = {};
	};


	/*************************************************
	*
	*  Métodos de validação dos dados para o filtro
	*
	*************************************************/
	var concatenaFields = function(selectField, currentRow) {
		var lista;
		lista = '_' + currentRow[selectField.name].join('_') + '_';
		return lista;
	};

	var concatenaListas = function(preference) {
		if(preference.FILTER_ROW.IDORIGDESTNF == 'C') {
			if(preference.FILTER_ROW.TDESTREM == 'T') {
				preference.FILTER_ROW.TCDCLIENTE = 'T';
				preference.FILTER_ROW.CDCLIENTE = '';
			}
			else {
				preference.FILTER_ROW.TCDCLIENTE = '';
				preference.FILTER_ROW.LCDCLIENTE = preference.FILTER_ROW.CDCLIENTE.join('_');
			}
		}
		else if(preference.FILTER_ROW.IDORIGDESTNF == 'F') {
			if(preference.FILTER_ROW.TDESTREM == 'T') {
				preference.FILTER_ROW.TCDFORNECED = 'T';
				preference.FILTER_ROW.CDFORNECED = '';
			}
			else {
				preference.FILTER_ROW.TCDFORNECED = '';
				preference.FILTER_ROW.LCDFORNECED = preference.FILTER_ROW.CDFORNECED.join('_');
			}
		}
		else if(preference.FILTER_ROW.IDORIGDESTNF == 'I') {
			if(preference.FILTER_ROW.TDESTREM == 'T') {
				preference.FILTER_ROW.TCDFILIDESTI = 'T';
				preference.FILTER_ROW.CDFILIDESTI = '';
			}
			else {
				preference.FILTER_ROW.TCDFILIDESTI = '';
				preference.FILTER_ROW.LCDFILIDESTI = preference.FILTER_ROW.CDFILIRECE.join('_');
			}
		}
		else {
			preference.FILTER_ROW.TCDCLIENTE 	= 'T';
			preference.FILTER_ROW.TCDFILIDESTI	= 'T';
			preference.FILTER_ROW.TCDFORNECED = 'T';
		}
		
		preference.CUSTOM_FILTER_ROW = Util.clone(preference.FILTER_ROW);
        if (preference.CUSTOM_FILTER_ROW.hasOwnProperty('TDESTREM')){
            delete preference.CUSTOM_FILTER_ROW.TDESTREM;
        }
	};

	// Verifica se ambas as datas de lançamento estão preenchidas ou vazias,
	// uma vez que elas não são requeridas, mas as duas devem ser preenchidas,
	// caso uma seja inserida
	var dataLancamentoIsValid = function(currentRow) {
		
		if(currentRow.DTLANCAMENTO) {
			currentRow.DTINILANC = currentRow.DTLANCAMENTO.start;
			currentRow.DTFINLANC = currentRow.DTLANCAMENTO.end;
			currentRow.TDTLANC	 = '';
		}
		else {
			currentRow.DTINILANC = '12/12/1899';
			currentRow.DTFINLANC = '12/12/1899';
			currentRow.TDTLANC	 = 'T';
		}
		return true;
	};

	this.verificaCampo = function(args,nomeCampoSelect,nomeCampoAlterar){
		var widget = args.owner.owner.widget;
		var campo =  widget.getField(nomeCampoAlterar);
		if(widget.getField(nomeCampoSelect).value()=='T'){
			campo.value("");
			campo.readOnly=true;
			delete(campo.validations.required);

		}else{
			campo.value("");
			campo.readOnly=false;
			campo.validations.required = true;
		}
	};

	this.campoDestinarioRemetente = function(args, preference) {
		var widget = args.owner.owner.widget;
		var currentRow = widget.currentRow;
		
		switch(widget.currentRow.IDORIGDESTNF) {
            case 'C':
            	widget.getField('TDESTREM').readOnly=false;
            	widget.getField('TDESTREM').value("");
            	currentRow.TCDFILIDESTI = null;
            	currentRow.TCDFORNECED = null;
            	currentRow.TCDCLIENTE = 'T';
            	currentRow.LCDFILIDESTI = '';
            	currentRow.LCDFORNECED = '';
            	currentRow.LCDCLIENTE = '';
            	//widget.getField('SELECTDESTREM').value("");
                break;

            case 'F':
            	widget.getField('TDESTREM').readOnly=false;
            	widget.getField('TDESTREM').value("");
            	currentRow.TCDFILIDESTI = null;
            	currentRow.TCDFORNECED = 'T';
            	currentRow.TCDCLIENTE = null;
            	currentRow.LCDFILIDESTI = '';
            	currentRow.LCDFORNECED = '';
            	currentRow.LCDCLIENTE = '';
	           	//widget.getField('SELECTDESTREM').value("");
                break;

            case 'I':
				widget.getField('TDESTREM').readOnly=false;
	           	widget.getField('TDESTREM').value("");
            	currentRow.TCDFILIDESTI = 'T';
            	currentRow.TCDFORNECED = null;
            	currentRow.TCDCLIENTE = null;
            	currentRow.LCDFILIDESTI = '';
            	currentRow.LCDFORNECED = '';
            	currentRow.LCDCLIENTE = '';
	           	//widget.getField('SELECTDESTREM').value("");
                break;

            default:
            	widget.getField('TDESTREM').readOnly=true;
            	widget.getField('TDESTREM').value("");
            	currentRow.TCDFILIDESTI = 'T';
            	currentRow.TCDFORNECED = 'T';
            	currentRow.TCDCLIENTE = 'T';
            	//widget.getField('SELECTDESTREM').value("");
                break;
        }
	};

	this.habilitaOrigDestNF = function(args) {
        var widget = args.owner.owner.widget;
        var TDESTREM = widget.getField('TDESTREM');
        if(TDESTREM.value() !== " "){
         	TDESTREM.setValue("T");
         	TDESTREM.reload();
        }
        switch(widget.currentRow.IDORIGDESTNF) {
            case 'C':
                widget.getField('CDFORNECED').isVisible = false;
                widget.getField('CDFILIRECE').isVisible = false;
                widget.getField('CDCLIENTE').isVisible  = true;

                widget.getField('CDFILIRECE').value('');
	            widget.getField('CDFORNECED').value('');
                break;
            case 'F':
                widget.getField('CDCLIENTE').isVisible  = false;
                widget.getField('CDFILIRECE').isVisible = false;
                widget.getField('CDFORNECED').isVisible = true;

                widget.getField('CDFILIRECE').value('');
	            widget.getField('CDCLIENTE').value('');
                break;
            case 'I':
                widget.getField('CDCLIENTE').isVisible  = false;
                widget.getField('CDFORNECED').isVisible = false;
                widget.getField('CDFILIRECE').isVisible = true;

                widget.getField('CDFORNECED').value('');
	            widget.getField('CDCLIENTE').value('');
                break;
            default:
                widget.getField('CDCLIENTE').isVisible  = false;
                widget.getField('CDFORNECED').isVisible = false;
                widget.getField('CDFILIRECE').isVisible = false;

				widget.getField('CDFILIRECE').value('');
                widget.getField('CDFORNECED').value('');
	            widget.getField('CDCLIENTE').value('');
                break;
        }
    };
    this.verificaCampoAtivado = function(args, preference){
    	var widget = args.owner.owner.widget;
    	var valorCampo = widget.currentRow.IDORIGDESTNF;
    	
    	if(widget.currentRow.TDESTREM !=='T'){
    		widget.getField('TDESTREM').setValue(" ");
    		widget.getField('TDESTREM').reload();
	    	switch(valorCampo) {
	            case 'C':
		            widget.getField('CDFILIRECE').readOnly  = true;
	                widget.getField('CDFORNECED').readOnly = true;
	                widget.getField('CDCLIENTE').readOnly  = false;
	                widget.getField('CDFILIRECE').value('');
	                widget.getField('CDFORNECED').value('');
	               	widget.getField('CDCLIENTE').validations.required = true;
	               	delete(widget.getField('CDFILIRECE').validations.required);
	                delete(widget.getField('CDFORNECED').validations.required);
	                break;
	            case 'F':
	                widget.getField('CDFILIRECE').readOnly  = true;
	                widget.getField('CDCLIENTE').readOnly = true;
	                widget.getField('CDFORNECED').readOnly  = false;
	                widget.getField('CDFILIRECE').value('');
	                widget.getField('CDCLIENTE').value('');
	               	widget.getField('CDFORNECED').validations.required = true;
	               	delete(widget.getField('CDFILIRECE').validations.required);
	                delete(widget.getField('CDCLIENTE').validations.required);
	                break;

	            case 'I':
	                widget.getField('CDCLIENTE').readOnly  = true;
	                widget.getField('CDFORNECED').readOnly = true;
	                widget.getField('CDFILIRECE').readOnly  = false;
	                widget.getField('CDCLIENTE').value('');
	                widget.getField('CDFORNECED').value('');
	               	widget.getField('CDFILIRECE').validations.required = true;
	               	delete(widget.getField('CDCLIENTE').validations.required);
	                delete(widget.getField('CDFORNECED').validations.required);
	                break;

	            default:
	                widget.getField('CDCLIENTE').readOnly  = true;
	                widget.getField('CDFORNECED').readOnly = true;
	                widget.getField('CDFILIRECE').readOnly  = true;
	                widget.getField('CDCLIENTE').value('');
	                widget.getField('CDFORNECED').value('');
	                widget.getField('CDFILIRECE').value('');
	               	delete(widget.getField('CDFILIRECE').validations.required);
	               	delete(widget.getField('CDCLIENTE').validations.required);
	                delete(widget.getField('CDFORNECED').validations.required);
	                break;
	        }
	        widget.currentRow = Util.clone(widget.currentRow);
	        if (widget.currentRow.hasOwnProperty('TDESTREM')){
	            delete widget.currentRow.TDESTREM;
	        }
	        widget.getField('TDESTREM').setValue(" "); 
			widget.getField('TDESTREM').reload();
		        
	    }else{
	    	widget.getField('CDCLIENTE').readOnly  = true;
	        widget.getField('CDFORNECED').readOnly = true;
	        widget.getField('CDFILIRECE').readOnly  = true;
	        widget.getField('CDCLIENTE').value('');
	        widget.getField('CDFORNECED').value('');
	        widget.getField('CDFILIRECE').value('');
	        delete(widget.getField('CDFILIRECE').validations.required);
	        delete(widget.getField('CDCLIENTE').validations.required);
	        delete(widget.getField('CDFORNECED').validations.required);
	        widget.getField('TDESTREM').setValue("T"); 
	        widget.getField('TDESTREM').reload();
	        widget.currentRow = Util.clone(widget.currentRow);
    		if (widget.currentRow.hasOwnProperty('TDESTREM')){
            	delete widget.currentRow.TDESTREM;
        }
        widget.getField('TDESTREM').setValue("T"); 
	    widget.getField('TDESTREM').reload();
	        
	    }
	 
	    

    };
}

Configuration(function(ContextRegister) {
	ContextRegister.register('Dfe21100_RomaneioNotasFiscaisEntrada_Controller', Dfe21100_RomaneioNotasFiscaisEntrada_Controller);
});


// FILE: ./assets/modules/relatorios/assets/js/controllers/Dfe23000_ComprasRealizadasPorMes_Controller.js
var Dfe23000_ComprasRealizadasPorMes_Controller = function(templateManager, ScreenService,  General, Dfe23000_ComprasRealizadasPorMes_Service, ZhFilterPreferences, ZHPromise, restEngine) {

    var openSwipe = false;
    var lastTotalizer = {};
    var lastTotalizerIndex = -1;

    this.openFilterCompras = function(widget){
        General.callRestEngine("df#/crud/filial", "FilterData", [{name: "CDFILIAL", operator: "=", value: "FILIAL_LOGADA"}]).then((success) => {
            var row = Util.clone(widget.currentRow);

            row.CDFILIAL = success.dataset.filial[0].CDFILIAL;
            row.NMFILIAL = success.dataset.filial[0].NMFILIAL;
            
            var dataAtual = formatarData(new Date());
                    
            var date = new Date();
            var primeiroDia = new Date(date.getFullYear(), date.getMonth(), 1);
            var ultimoDia = new Date(date.getFullYear(), date.getMonth() + 1, 0);
            
            row.DTENTRSAID       = {};
            row.DTENTRSAID.start = formatarData(primeiroDia);
            row.DTENTRSAID.end   = formatarData(ultimoDia);
                
            row.REPORT_NAME   = 'DFE32001';
            setLocalVar("REPORT_NAME", 'DFE32001');
            
            ZhFilterPreferences.setOriginalFilter(widget.id, row);
            ZhFilterPreferences.openFilterWidget(widget, row);
        },
        (error) => {
            ZhFilterPreferences.openFilterWidget(widget);
            console.log(error);
        });
    };
    
    function formatarData(data) {
        var d = new Date(data),
        mes = '' + (d.getMonth() + 1),
        dia = '' + d.getDate(),
        ano = d.getFullYear();
    
        if (mes.length < 2) mes = '0' + mes;
        if (dia.length < 2) dia = '0' + dia;
    
        return [dia, mes, ano].join('/');
    }
    
    this.setReportName = function(){
        var reportName = templateManager.container.getActiveWidget().getField('REPORT_NAME').value();
        setLocalVar("REPORT_NAME", reportName);
    };
    
	this.activateTab = function(widget) {
		if (!!openSwipe) {
			widget.activate();
			openSwipe = false;
		}
	};

    this.verAlmox = function(widget) {
        if(widget.currentRow.REPORT_NAME === 'DFE32004') {
            var currentRow = widget.currentRow;
            var fieldAmox = widget.getField('DSALMOXARIFE');
            if(currentRow.CDFILIAL){
                Dfe23000_ComprasRealizadasPorMes_Service.setParamFilial(currentRow.CDFILIAL).then(function(success) {
                    if(Dfe23000_ComprasRealizadasPorMes_Service.utilizaAlmox()){
                        fieldAmox.readOnly = false;
                    }  else {
                        fieldAmox.readOnly = true;
                    }
                    fieldAmox.setValue(null);
                }.bind(this), function(error) {
                    ScreenService.showMenssage(error.data.error);
                });
            } else {
                delete fieldAmox.validations.required;
                fieldAmox.readOnly = true;
                fieldAmox.setValue(null);
            }
        }
    };

    this.setMinMax = function(widget, fieldNameSet, fieldNameGet, minOrMax){
        if(minOrMax == 'max') {
            widget.getField(fieldNameSet).maxDate = widget.getField(fieldNameGet).value();
        } else {
            widget.getField(fieldNameSet).minDate = widget.getField(fieldNameGet).value();
        }
    };

    this.selectDataSource = function(args){
        args.e.preventDefault();
        var widgetByReport = templateManager.container.getWidget('widget_by_report');
        var widget = args.owner;
        
        var filterDefault = {
            name: '',
            operator: '=',
            value: ''
        };
        
        var filters = [];
        
           
        var cdprodini = {
            name: "CDPRODINI",
            value: templateManager.container.getActiveWidget().getField('CDPRODINI').value(), 
            operator: "="
        };
        
        var cdprodfim = {
            name: "CDPRODFIM",
            value: this.getProdFinMask(templateManager.container.getActiveWidget().getField('CDPRODFIM').value()), 
            operator: "="
        };
        
        filters.push(cdprodini);
        filters.push(cdprodfim);
        
        
        if(args.e.filters[0].value === 'DFE32001'){ //Compras Realizadas por Mês (ordenado por Mês)
            widgetByReport.dataSource.name = 'df_relatorios#/notafiscal_compras_realizadas_report_data';
            widget.dataSource.name = 'df_relatorios#/notafiscal_compras_realizadas_ordem_mes';
            widget.order = 'DATA';
            widget.getField('MES').isVisible = true;
            widget.getField('DATA').isVisible = true;
            widget.getField('CDARVPROD').isVisible = false;
            widget.getField('NMPRODUTO').isVisible = false;
            widget.getField('SGUNIDADE').isVisible = false;
        } else {                                          //Compras Realizadas por Mês (ordenado por Produto)
            widgetByReport.dataSource.name = 'df_relatorios#/notafiscal_compras_realizadas_report_prod';
            widget.dataSource.name = 'df_relatorios#/notafiscal_compras_realizadas_ordem_produto';
            widget.order = 'CDARVPROD';
            widget.getField('MES').isVisible = false;
            widget.getField('DATA').isVisible = false;
            widget.getField('CDARVPROD').isVisible = true;
            widget.getField('NMPRODUTO').isVisible = true;
            widget.getField('SGUNIDADE').isVisible = true;
        }
        widget.dataSourceFilter.forEach((filter) => {
            if(filter.name != 'REPORT_NAME' 
                && filter.name != 'CDPRODINI'
                && filter.name != 'CDPRODFIM'
                && filter.name != 'DTENTRSAID' 
                && filter.name != 'DTEMISSAO'
                && filter.name != 'DTENTRSAIDINI'
                && filter.name != 'DTENTRSAIDFIN'
                && filter.name != 'DTEMISSAOINI'
                && filter.name != 'DTEMISSAOFIN'
                && filter.name != 'T_DTEMISSAO'
                && filter.name != 'T_DTENTRSAID'){
                    delete filter.isCustomFilter;
                    filters.push(filter);
            }
        });
        
        var dtEntrSaid = templateManager.container.getActiveWidget().getField('DTENTRSAID').value();
        var dtEmissao  = templateManager.container.getActiveWidget().getField('DTEMISSAO').value();
        
        filterDefault.name  = 'DTENTRSAIDINI';
        filterDefault.value =  dtEntrSaid ? (dtEntrSaid.start || '') : '' ;
        filters.push(Util.clone(filterDefault));
        filterDefault.name  = 'DTENTRSAIDFIN';
        filterDefault.value =  dtEntrSaid ? (dtEntrSaid.end || '') : '' ;
        filters.push(Util.clone(filterDefault));
        
        filterDefault.name  = 'T_DTENTRSAID';
        filterDefault.value = dtEntrSaid ? ((dtEntrSaid.start) ? '' : 'T') : 'T' ;
        filters.push(Util.clone(filterDefault));
            

        filterDefault.name  = 'DTEMISSAOINI';
        filterDefault.value = dtEmissao ? (dtEmissao.start || '') : '' ;
        filters.push(Util.clone(filterDefault));
        
        filterDefault.name  = 'DTEMISSAOFIN';
        filterDefault.value = dtEmissao ? (dtEmissao.end || '') : '' ;
        filters.push(Util.clone(filterDefault));
        
        filterDefault.name  = 'T_DTEMISSAO';
        filterDefault.value = dtEmissao ? ((dtEmissao.start) ? '' : 'T') : 'T' ;
        filters.push(Util.clone(filterDefault));

        this.buildGrid(args.e.filters[0].value);
        widget.resizeGridColumns(true);
        
        widget.dataSourceFilter = filters;
        
        widget.reload();
        widget.redraw();
        
        
    };
    
    this.getProdFinMask = function(value) {

        var mascara = getLocalVar('GER_CDPICTPROD');
        mascara = mascara.split('.');
        var mascaraSemPonto =  "";
        for(var index in mascara){
            mascaraSemPonto = mascaraSemPonto + mascara[index];
        }
        mascara = mascaraSemPonto.length;
        var tamanhoValue = value.length;
        mascara = mascara - tamanhoValue;
        var maximo = 'z';
        value += maximo.repeat(mascara);
        
        return value;
    };
    
    this.validateFilterPopup = function(args) {
        
        var row = args.e.preference.FILTER_ROW;
        
        if(!row.DTENTRSAID && !row.DTEMISSAO) {
            ScreenService.showMessage('Data de entrada e/ou data de emissão devem ser informadas!');
            args.e.preventDefault();
        }
        else {
            General.validateFilterPopup(args);
        }
    };

    this.buildGrid = function(report){
        var widget = templateManager.container.getWidget('widget_by_report');
        var fieldAgrupa = widget.getField('AGRUPA');
        if(report === 'DFE32002') {//ORDENADOR POR PRODUTO
            fieldAgrupa.label = 'Produto';
            fieldAgrupa.template = 'field/text-edit.html';
            if(fieldAgrupa.dateFormat)
                delete fieldAgrupa.dateFormat;
            if(fieldAgrupa.displayFormat)
                delete fieldAgrupa.displayFormat;
        } else {//ORDENADO POR MÊS
            fieldAgrupa.label = 'Mês';
            fieldAgrupa.template = 'field/calendar.html';
                fieldAgrupa.dateFormat = 'DD/MM/YYYY';
            fieldAgrupa.displayFormat = 'MM/YYYY';
            
        }
    };

    this.buildGridAgruped = function(args, widget, event){
        this.lastTotalizerIndex = false;
        var widgetFilha = widget.widgets[0];
        var report;
        var dtIniMes, dtFinMes;
        
        if(event == "WidgetAfterMoveRow"){
            report = getLocalVar('REPORT_NAME');
        }else{
            report = args.e.filters[0].value;
        }
        //monta o filtro da widget filha conforme a row e o time do relatório
        widgetFilha.dataSourceFilter = angular.copy(widget.dataSourceFilter);
        
        widgetFilha.dataSourceFilter.forEach(filter => {

            if(report === 'DFE32001') {
                if(filter.name === 'DTENTRSAIDINI'){
                    dtIniMes = moment(widget.currentRow.DATA, 'DD/MM/YYYY');
                    if(dtIniMes.startOf('month').isAfter(moment(filter.value, 'DD/MM/YYYY'))) {
                         filter.value = moment(widget.currentRow.DATA, 'DD/MM/YYYY').startOf('month').format('DD/MM/YYYY');
                    }
                } else if(filter.name === 'DTENTRSAIDFIN'){
                    dtFinMes = moment(widget.currentRow.DATA, 'DD/MM/YYYY');
                    if(dtFinMes.endOf('month').isBefore(moment(filter.value, 'DD/MM/YYYY'))) {
                         filter.value = moment(widget.currentRow.DATA, 'DD/MM/YYYY').endOf('month').format('DD/MM/YYYY');
                    }
                } else if(filter.name === 'T_DTENTRSAID') {
                    filter.value = '';
                }    
                else if( filter.name === 'DTEMISSAOINI') {
                    filter.value = '';
                } else if (filter.name === 'DTEMISSAOFIN') {
                    filter.value = '';
                } else if(filter.name === 'T_DTEMISSAO') {
                    filter.value = 'T';
                }  
            }
            else {
                if( filter.name === 'CDPRODINI'){
                    filter.value = widget.currentRow.CDARVPROD;
                } else if( filter.name === 'CDPRODFIM'){
                    filter.value = widget.currentRow.CDARVPROD;
                }
            }
        });
        
        //Troca a label conforme a row
        if(report === 'DFE32001'){
            widgetFilha.label = widget.currentRow.MES + '/' + moment(widget.currentRow.DATA, 'DD/MM/YYYY').format('YYYY');
        } else {
            widgetFilha.label = widget.currentRow.NMPRODUTO;
        }
		openSwipe = true;
		widgetFilha.reload();
    };

    this.generateReport = function(reportType){
        var widgetByReport = templateManager.container.getWidget('widget_by_report');
        var widget = templateManager.container.getWidget('dfe23000_compras_realizadas_por_mes_grid');
        widgetByReport.dataSourceFilter = widget.dataSourceFilter;
        widgetByReport.reload().then(function() {
            widgetByReport.generateReport(widgetByReport, reportType);
        });
    };

    this.closeSwipe =function(widget){
        ScreenService.goBack().then(function(){
            widget.parent.activate();
        });
    };
    
    this.produtoFinalIsValid = function(widget) {
	    if(widget.currentRow.CDPRODUTOFIM && widget.currentRow.CDPRODUTOINI) {
    		if(widget.currentRow.CDPRODUTOFIM < widget.currentRow.CDPRODUTOINI) {
    			ScreenService.showMessage(ScreenService.i18n("Produto inicial deve ser menor ou igual ao produto final."));
    			widget.currentRow.CDPRODUTOFIM = null;
    			widget.currentRow.NMPRODFINAL = null;
    		}
    	widget.currentRow.CDPRODUTOFIM = widget.currentRow.CDPRODUTOFIM + "zzzzzzzzzz";
        widget.currentRow.CDPRODUTOFIM = widget.currentRow.CDPRODUTOFIM.substring(0, 10);
        widget.currentRow.CDPRODFIM = widget.currentRow.CDPRODFIM + "zzzzzzzzzz";
        widget.currentRow.CDPRODFIM = widget.currentRow.CDPRODFIM.substring(0, 10);
	    }
	};

	this.produtoInicialIsValid = function(widget) {
	    if(widget.currentRow.CDPRODUTOFIM && widget.currentRow.CDPRODUTOINI) {
    		if(widget.currentRow.CDPRODUTOFIM < widget.currentRow.CDPRODUTOINI) {
    			ScreenService.showMessage(ScreenService.i18n("Produto inicial deve ser menor ou igual ao produto final."));
    			widget.currentRow.CDPRODUTOINI = null;
    			widget.currentRow.NMPRODINICIO = null;
    		}
	    }
	};
	
	this.saveLastTotalizer = function(widget) {
	    if(widget.dataSource.data) {
    	    let length = widget.dataSource.data.length;
	        let total  = widget.dataSource.data[length - 1];
	        
	        this.lastTotalizer = total;
	        this.lastTotalizerIndex = total;
	    }    
	};
	
	this.totalizer = function(widget) {    
	    let length = widget.dataSource.data.length;
        let total  = widget.dataSource.data[length - 1];
	    
        
        if(this.lastTotalizerIndex) {
	        let { QUANTIDADE, VALORTOTAL } = widget.dataSource.data[this.lastTotalizerIndex];

            total.VALORTOTAL += VALORTOTAL;
            total.QUANTIDADE += QUANTIDADE;
            widget.dataSource.data = [  
                        ...widget.dataSource.data.slice(0, this.lastTotalizerIndex), 
                        ...widget.dataSource.data.slice(this.lastTotalizerIndex + 1, widget.dataSource.data.length)
            ];
            this.lastTotalizerIndex = widget.dataSource.data.length - 1;
        }  
        else {
            
            this.lastTotalizerIndex = length - 1;
            
        }           

	};

};

Configuration(function(ContextRegister) {
    ContextRegister.register('Dfe23000_ComprasRealizadasPorMes_Controller', Dfe23000_ComprasRealizadasPorMes_Controller);
});

// FILE: ./assets/modules/relatorios/assets/js/controllers/Dfe26000_InflacaoInterna_Controller.js
var Dfe26000_InflacaoInterna_Controller = function(ScreenService, ZhLayoutPreferences, General, templateManager, localStorageEngine, Dfe26000_InflacaoInterna_Service, ZhFilterPreferences) {

    //before apply filter
    this.changeCurrentRowBeforeFilter = function(widgetPopup, args) {
        var widget = templateManager.container.getWidget('dfe26000_inflacao_interna_grid');
        if (!widgetPopup) {
            args.e.preventDefault();
        }
        else {
            if (!this.validaNivel(widgetPopup.FILTER_ROW, args) && !this.compararDatas(widgetPopup.FILTER_ROW, args)) {
                if (widget.getField('DATAINICIAL').readOnly === false) {
                    var dataIni = moment(widgetPopup.FILTER_ROW.DATAINICIAL, 'DD/MM/YYYY');
                    var dataFin = moment(widgetPopup.FILTER_ROW.DATAFINAL, 'DD/MM/YYYY');
                    if (dataIni.isAfter(dataFin)) {
                        ScreenService.showMessage('A Data Inicial deve ser inferior a Data Final');
                        args.e.preventDefault();
                    }
                }
                else {
                    widgetPopup.FILTER_ROW.INIMES1 = moment(widgetPopup.FILTER_ROW.MES1, "MM/YYYY").startOf('month').format('DD/MM/YYYY');
                    widgetPopup.FILTER_ROW.FINMES1 = moment(widgetPopup.FILTER_ROW.MES1, "MM/YYYY").endOf('month').format('DD/MM/YYYY');
                    widgetPopup.FILTER_ROW.INIMES2 = moment(widgetPopup.FILTER_ROW.MES2, "MM/YYYY").startOf('month').format('DD/MM/YYYY');
                    widgetPopup.FILTER_ROW.FINMES2 = moment(widgetPopup.FILTER_ROW.MES2, "MM/YYYY").endOf('month').format('DD/MM/YYYY');
                }
                this.selectDataSourceByReport(widgetPopup);
            }
        }
    };

    //after apply filter
    this.buildDataSourceFilterAndGrid = function(widgetPopup, args) {
        var arrayIndex = [];
        var widget = templateManager.container.getWidget('dfe26000_inflacao_interna_grid');
        var field = widgetPopup.FILTER_ROW.CDOPERENTR;
        var mascara = getLocalVar('GER_CDPICTPROD');
        //corrige filtro versão nova

        if (!widget.dataSourceFilter.some(row => row.name == 'CDPRODFINAL')) {
            widget.dataSourceFilter.push({
                'name': "CDPRODFINAL",
                'operator': "=",
                'value': widgetPopup.FILTER_ROW.CDPRODFINAL
            });

        }
        else {
            //Atualiza valores CDPRODFINAL
            let row = widget.dataSourceFilter.filter(row => row.name == 'CDPRODFINAL');
            row[0].value = widgetPopup.FILTER_ROW.CDPRODFINAL;
        }

        if (!widget.dataSourceFilter.some(row => row.name == 'NMTIPOCUSTO')) {
            widget.dataSourceFilter.push({
                'name': "NMTIPOCUSTO",
                'operator': "=",
                'value': widgetPopup.FILTER_ROW.NMTIPOCUSTO
            });

        }
        else {
            //Atualiza valores NMTIPOCUSTO
            let row = widget.dataSourceFilter.filter(row => row.name == 'NMTIPOCUSTO');
            row[0].value = widgetPopup.FILTER_ROW.NMTIPOCUSTO;
        }

        if (!widget.dataSourceFilter.some(row => row.name == 'TIPOCUSTO')) {
            widget.dataSourceFilter.push({
                'name': "TIPOCUSTO",
                'operator': "=",
                'value': widgetPopup.FILTER_ROW.TIPOCUSTO
            });

        }
        else {
            let row = widget.dataSourceFilter.filter(row => row.name == 'TIPOCUSTO');
            row[0].value = widgetPopup.FILTER_ROW.TIPOCUSTO;

        }

        if (!widget.dataSourceFilter.some(row => row.name == 'CDPRODINI')) {
            widget.dataSourceFilter.push({
                'name': "CDPRODINI",
                'operator': "=",
                'value': widgetPopup.FILTER_ROW.CDPRODINI
            });

        }
        else {
            //Atualiza valores CDPRODINI
            let row = widget.dataSourceFilter.filter(row => row.name == 'CDPRODINI');
            row[0].value = widgetPopup.FILTER_ROW.CDPRODINI;
        }

        //Colocar z ao final do codigo do produto final , conforme o nuemro total de numeros da mascara da filial    
        for (var x = 0; x < widget.dataSourceFilter.length; x++) {
            if (widget.dataSourceFilter[x].name === 'CDPRODFINAL') {
                mascara = getLocalVar('GER_CDPICTPROD');
                mascara = mascara.split('.');
                var mascaraSemPonto = "";
                for (var index in mascara) {
                    mascaraSemPonto = mascaraSemPonto + mascara[index];
                }
                mascara = mascaraSemPonto.length;
                var tamanhoValue = widget.dataSourceFilter[x].value.length;
                mascara = mascara - tamanhoValue;
                var maximo = 'z';
                widget.dataSourceFilter[x].value = widget.dataSourceFilter[x].value + maximo.repeat(mascara);
                widgetPopup.FILTER_ROW.CDPRODFINAL = widget.dataSourceFilter[x].value;
            }
        }
        //Colocar a lista ou T no filtro de Operações
        if (field === 'Lista') {
            widget.dataSourceFilter.forEach(function(filter, index) {
                if (filter.name === 'CDOPERENTR' || filter.name === 'NMTIPOOPER') {
                    arrayIndex.push(index);
                }
            });
            this.buildDataSourceFilter(widget, arrayIndex);
            arrayIndex = [];
            var field2 = widgetPopup.FILTER_ROW.CDTIPOOPER;
            var string = '_' + field2.join('_') + '_';
            widget.dataSourceFilter.push({
                "name": "CDOPERENTR",
                "operator": "=",
                "value": string
            });
            widgetPopup.FILTER_ROW.CDOPERENTR = string;
            widget.filterData = Util.clone(widgetPopup.FILTER_ROW);
            widgetPopup.FILTER_ROW.CDOPERENTR = 'Lista';
        }
        else {
            widget.dataSourceFilter.forEach(function(filter, index) {
                if (filter.name === 'CDOPERENTR') {
                    filter.value = 'T';
                }
            });
            widgetPopup.FILTER_ROW.CDOPERENTR = 'T';
            widget.filterData = Util.clone(widgetPopup.FILTER_ROW);
            widgetPopup.FILTER_ROW.CDOPERENTR = 'Todos';
        }
        if (widgetPopup.FILTER_ROW.REPORT_NAME === 'Inflação Interna') {
            this.setShowOnlist(true, false, widget);
            let dtini = widget.dataSourceFilter.filter(row => row.name == 'DATAINICIAL');
            dtini[0].value = moment('01/' + widgetPopup.FILTER_ROW.MES1, 'DD/MM/YYYY').startOf('month').format('DD/MM/YYYY');

            let dtfim = widget.dataSourceFilter.filter(row => row.name == 'DATAFINAL');
            dtfim[0].value = moment('01/' + widgetPopup.FILTER_ROW.MES2, 'DD/MM/YYYY').endOf('month').format('DD/MM/YYYY');

            widget.dataSourceFilter.forEach(function(filter, index) {
                if (filter.name === 'CDGRUPFILI' || filter.name === 'REPORT_NAME' || filter.name === 'TAMNIVEL' || filter.name === 'INIMES1' || filter.name === 'FINMES1' || filter.name === 'INIMES2' || filter.name === 'FINMES2' || filter.name === 'MES1' || filter.name === 'MES2') {
                    arrayIndex.push(index);
                }
            });
        }
        else if (widgetPopup.FILTER_ROW.REPORT_NAME === 'Inflação Interna Consolidada') {
            //Campos do relatorio Inflação Interna Consolidada
            this.setShowOnListConsolidada(widget);
            widget.dataSourceFilter.forEach(function(filter, index) {
                if (filter.name === 'CDFILIAL' || filter.name === 'REPORT_NAME' || filter.name === 'TAMNIVEL' || filter.name === 'INIMES1' || filter.name === 'FINMES1' || filter.name === 'INIMES2' || filter.name === 'FINMES2' || filter.name === 'MES1' || filter.name === 'MES2') {
                    arrayIndex.push(index);
                }
            });
        }
        else {
            this.setShowOnlist(false, true, widget);
            if (!widget.dataSourceFilter.some(row => row.name == 'INIMES1'))
                widget.dataSourceFilter.push({
                    "name": "INIMES1",
                    "operator": "=",
                    "value": moment('01/' + widgetPopup.FILTER_ROW.MES1, 'DD/MM/YYYY').startOf('month').format('DD/MM/YYYY')
                });
            if (!widget.dataSourceFilter.some(row => row.name == 'FINMES1'))
                widget.dataSourceFilter.push({
                    "name": "FINMES1",
                    "operator": "=",
                    "value": moment('01/' + widgetPopup.FILTER_ROW.MES1, 'DD/MM/YYYY').endOf('month').format('DD/MM/YYYY')
                });
            if (!widget.dataSourceFilter.some(row => row.name == 'INIMES2'))
                widget.dataSourceFilter.push({
                    "name": "INIMES2",
                    "operator": "=",
                    "value": moment('01/' + widgetPopup.FILTER_ROW.MES2, 'DD/MM/YYYY').startOf('month').format('DD/MM/YYYY')
                });
            if (!widget.dataSourceFilter.some(row => row.name == 'FINMES2'))
                widget.dataSourceFilter.push({
                    "name": "FINMES2",
                    "operator": "=",
                    "value": moment('01/' + widgetPopup.FILTER_ROW.MES2, 'DD/MM/YYYY').endOf('month').format('DD/MM/YYYY')
                });
            widget.dataSourceFilter.forEach(function(filter, index) {
                if (filter.name === 'CDGRUPFILI' || filter.name === 'REPORT_NAME' || filter.name === 'TAMNIVEL' || filter.name === 'TIPOCUSTO' || filter.name === 'NMTIPOCUSTO' || filter.name === 'DATAINICIAL' || filter.name === 'DATAFINAL' || filter.name === 'MES1' || filter.name === 'MES2') {
                    arrayIndex.push(index);
                }
            });
        }

        this.buildDataSourceFilter(widget, arrayIndex);
        //voltar o valor do porduto final sem os Z no final
        for (var j = 0; j < widget.dataSourceFilter.length; j++) {
            if (widget.dataSourceFilter[j].name === 'CDPRODFINAL') {
                var newValue = widget.dataSourceFilter[j].value.split('z');
                widgetPopup.FILTER_ROW.CDPRODFINAL = newValue[0];
            }
        }
        widget.dataSourceFilter = widget.dataSourceFilter.filter(row => row.name != 'NMPRODINICIO' && row.name != 'NMPRODFINAL');

        widgetPopup.CUSTOM_FILTER_ROW = Util.clone(widgetPopup.FILTER_ROW);
        if (widgetPopup.CUSTOM_FILTER_ROW.hasOwnProperty('REPORT_NAME')) {
            delete widgetPopup.CUSTOM_FILTER_ROW.REPORT_NAME;
        }
        if (widgetPopup.CUSTOM_FILTER_ROW.hasOwnProperty('TAMNIVEL')) {
            delete widgetPopup.CUSTOM_FILTER_ROW.TAMNIVEL;
        }
        if (widgetPopup.CUSTOM_FILTER_ROW.hasOwnProperty('NMPRODINICIO')) {
            delete widgetPopup.CUSTOM_FILTER_ROW.NMPRODINICIO;
        }

        if (widgetPopup.CUSTOM_FILTER_ROW.hasOwnProperty('NMPRODFINAL')) {
            delete widgetPopup.CUSTOM_FILTER_ROW.NMPRODFINAL;
        }

        this.setLayout(args.owner);
        widget.activate().then(() => widget.reload());
    };

    this.buildDataSourceFilter = function(widget, indexs) {
        for (var i = indexs.length - 1; i >= 0; i--) {
            widget.dataSourceFilter.splice(indexs[i], 1);
        }
    };

    this.selectDataSourceByReport = function(widgetPopup) {
        var widget = templateManager.container.getWidget('dfe26000_inflacao_interna_grid');
        if (widgetPopup.FILTER_ROW.REPORT_NAME == 'Inflação Interna') {
            var dataSource26000 = "df_relatorios#/itlanctoest_inflacaointerna";
            widget.dataSource.name = dataSource26000;
        }
        else if (widgetPopup.FILTER_ROW.REPORT_NAME == 'Inflação de Compras') {
            var dataSource26001 = "df_relatorios#/itlanctoest_inflacaocompras";
            widget.dataSource.name = dataSource26001;
        }
        else if (widgetPopup.FILTER_ROW.REPORT_NAME == 'Inflação Interna Consolidada') {
            var dataSource26003 = "df_relatorios#/itlanctoest_inflacaointerna_consolidada";
            widget.dataSource.name = dataSource26003;
        }
        else {
            var dataSource26002 = "df_relatorios#/itlanctoest_inflcomprasmaisvol";
            widget.dataSource.name = dataSource26002;
        }
    };

    //onChange do campo REPORT_NAME do filtro
    this.setReadOnlyByReport = function(widgetPopup) {
        if (widgetPopup.getField('REPORT_NAME').value() == 'Inflação Interna') {
            widgetPopup.getField('DATAINICIAL').readOnly = true;
            widgetPopup.getField('DATAINICIAL').isVisible = false;
            if (!widgetPopup.getField('DATAINICIAL').validations)
                widgetPopup.getField('DATAINICIAL').validations = {};
            widgetPopup.getField('DATAINICIAL').validations.required = true;
            widgetPopup.getField('DATAINICIAL').setValue(moment().subtract(1, 'months').format("DD/MM/YYYY"));

            widgetPopup.getField('DATAFINAL').readOnly = true;
            widgetPopup.getField('DATAFINAL').isVisible = false;
            if (!widgetPopup.getField('DATAFINAL').validations)
                widgetPopup.getField('DATAFINAL').validations = {};
            widgetPopup.getField('DATAFINAL').validations.required = true;
            widgetPopup.getField('DATAFINAL').setValue(moment().format("DD/MM/YYYY"));

            widgetPopup.getField('NMTIPOCUSTO').readOnly = false;
            if (!widgetPopup.getField('NMTIPOCUSTO').validations)
                widgetPopup.getField('NMTIPOCUSTO').validations = {};
            widgetPopup.getField('NMTIPOCUSTO').validations.required = true;

            widgetPopup.getField('MES1').readOnly = false;
            widgetPopup.getField('MES1').isVisible = true;
            widgetPopup.getField('MES1').setValue(null);
            if (!widgetPopup.getField('MES1').validations)
                widgetPopup.getField('MES1').validations = {};
            if (widgetPopup.getField('MES1').validations.required)
                delete widgetPopup.getField('MES1').validations.required;

            widgetPopup.getField('MES2').readOnly = false;
            widgetPopup.getField('MES2').isVisible = true;
            widgetPopup.getField('MES2').setValue(null);
            if (!widgetPopup.getField('MES2').validations)
                widgetPopup.getField('MES2').validations = {};
            if (widgetPopup.getField('MES2').validations.required)
                delete widgetPopup.getField('MES2').validations.required;

            widgetPopup.getField('NMGRUPFILI').readOnly = true;
            widgetPopup.getField('NMGRUPFILI').setValue(null);
            if (!widgetPopup.getField('NMGRUPFILI').validations)
                widgetPopup.getField('NMGRUPFILI').validations = {};
            if (widgetPopup.getField('NMGRUPFILI').validations.required)
                delete widgetPopup.getField('NMGRUPFILI').validations.required;

            widgetPopup.getField('NMFILIAL').readOnly = false;
            if (!widgetPopup.getField('NMFILIAL').validations)
                widgetPopup.getField('NMFILIAL').validations = {};
            widgetPopup.getField('NMFILIAL').validations.required = true;

            if (widgetPopup.currentRow.$error) {
                widgetPopup.currentRow.$error.MES1 = [];
                widgetPopup.currentRow.$error.MES2 = [];
                widgetPopup.currentRow.$error.NMGRUPFILI = [];
            }

        }
        else if (widgetPopup.getField('REPORT_NAME').value() == 'Inflação Interna Consolidada') {
            widgetPopup.getField('DATAINICIAL').readOnly = false;
            widgetPopup.getField('DATAINICIAL').isVisible = true;
            if (!widgetPopup.getField('DATAINICIAL').validations)
                widgetPopup.getField('DATAINICIAL').validations = {};
            widgetPopup.getField('DATAINICIAL').validations.required = true;
            widgetPopup.getField('DATAINICIAL').setValue(moment().subtract(1, 'months').format("DD/MM/YYYY"));

            widgetPopup.getField('DATAFINAL').readOnly = false;
            widgetPopup.getField('DATAFINAL').isVisible = true;
            if (!widgetPopup.getField('DATAFINAL').validations)
                widgetPopup.getField('DATAFINAL').validations = {};
            widgetPopup.getField('DATAFINAL').validations.required = true;
            widgetPopup.getField('DATAFINAL').setValue(moment().format("DD/MM/YYYY"));

            widgetPopup.getField('NMTIPOCUSTO').readOnly = false;
            if (!widgetPopup.getField('NMTIPOCUSTO').validations)
                widgetPopup.getField('NMTIPOCUSTO').validations = {};
            widgetPopup.getField('NMTIPOCUSTO').validations.required = true;

            widgetPopup.getField('MES1').readOnly = true;
            widgetPopup.getField('MES1').isVisible = false;
            widgetPopup.getField('MES1').setValue(null);
            if (!widgetPopup.getField('MES1').validations)
                widgetPopup.getField('MES1').validations = {};
            if (widgetPopup.getField('MES1').validations.required)
                delete widgetPopup.getField('MES1').validations.required;

            widgetPopup.getField('MES2').readOnly = true;
            widgetPopup.getField('MES2').isVisible = false;
            widgetPopup.getField('MES2').setValue(null);
            if (!widgetPopup.getField('MES2').validations)
                widgetPopup.getField('MES2').validations = {};
            if (widgetPopup.getField('MES2').validations.required)
                delete widgetPopup.getField('MES2').validations.required;

            widgetPopup.getField('NMGRUPFILI').readOnly = false;
            if (!widgetPopup.getField('NMGRUPFILI').validations)
                widgetPopup.getField('NMGRUPFILI').validations = {};
            widgetPopup.getField('NMGRUPFILI').validations.required = true;


            widgetPopup.getField('NMFILIAL').readOnly = true;
            widgetPopup.getField('NMFILIAL').setValue(null);
            if (!widgetPopup.getField('NMFILIAL').validations)
                widgetPopup.getField('NMFILIAL').validations = {};
            if (widgetPopup.getField('NMFILIAL').validations.required)
                delete widgetPopup.getField('NMFILIAL').validations.required;


            if (widgetPopup.currentRow.$error) {
                widgetPopup.currentRow.$error.MES1 = [];
                widgetPopup.currentRow.$error.MES2 = [];
                widgetPopup.currentRow.$error.NMFILIAL = [];
            }

        }
        else {
            widgetPopup.getField('DATAINICIAL').readOnly = true;
            widgetPopup.getField('DATAINICIAL').isVisible = false;
            widgetPopup.getField('DATAINICIAL').setValue(null);
            if (!widgetPopup.getField('DATAINICIAL').validations)
                widgetPopup.getField('DATAINICIAL').validations = {};
            if (widgetPopup.getField('DATAINICIAL').validations.required)
                delete widgetPopup.getField('DATAINICIAL').validations;
            delete widgetPopup.currentRow.DTINICIAL;

            widgetPopup.getField('DATAFINAL').readOnly = true;
            widgetPopup.getField('DATAFINAL').isVisible = false;
            widgetPopup.getField('DATAFINAL').setValue(null);
            if (!widgetPopup.getField('DATAFINAL').validations)
                widgetPopup.getField('DATAFINAL').validations = {};
            if (widgetPopup.getField('DATAFINAL').validations.required)
                delete widgetPopup.getField('DATAFINAL').validations;
            delete widgetPopup.currentRow.DATAFINAL;

            widgetPopup.getField('NMTIPOCUSTO').readOnly = true;
            widgetPopup.getField('NMTIPOCUSTO').setValue(null);
            if (!widgetPopup.getField('NMTIPOCUSTO').validations)
                widgetPopup.getField('NMTIPOCUSTO').validations = {};
            if (widgetPopup.getField('NMTIPOCUSTO').validations.required)
                delete widgetPopup.getField('NMTIPOCUSTO').validations;

            widgetPopup.getField('MES1').readOnly = false;
            widgetPopup.getField('MES1').isVisible = true;
            if (!widgetPopup.getField('MES1').validations)
                widgetPopup.getField('MES1').validations = {};
            widgetPopup.getField('MES1').validations.required = true;
            widgetPopup.getField('MES2').readOnly = false;
            widgetPopup.getField('MES2').isVisible = true;
            if (!widgetPopup.getField('MES2').validations)
                widgetPopup.getField('MES2').validations = {};
            widgetPopup.getField('MES2').validations.required = true;

            widgetPopup.getField('NMGRUPFILI').readOnly = true;
            widgetPopup.getField('NMGRUPFILI').setValue(null);
            if (!widgetPopup.getField('NMGRUPFILI').validations)
                widgetPopup.getField('NMGRUPFILI').validations = {};
            if (widgetPopup.getField('NMGRUPFILI').validations.required)
                delete widgetPopup.getField('NMGRUPFILI').validations.required;

            widgetPopup.getField('NMFILIAL').readOnly = false;
            if (!widgetPopup.getField('NMFILIAL').validations)
                widgetPopup.getField('NMFILIAL').validations = {};
            widgetPopup.getField('NMFILIAL').validations.required = true;

            if (widgetPopup.currentRow.$error) {
                widgetPopup.currentRow.$error.DATAINICIAL = [];
                widgetPopup.currentRow.$error.DATAFINAL = [];
                widgetPopup.currentRow.$error.NMTIPOCUSTO = [];
                widgetPopup.currentRow.$error.NMGRUPFILI = [];
            }
        }
        this.setActionFLoatingControl(widgetPopup);
    };

    //onChange dos campos que seleciona lista ou todos, do filtro
    this.activeSelect = function(value, field) {
        field.setValue(field.value() || 'Todos');
        if (value == 'Lista') {
            field.readOnly = false;
            field.setValue(null);
            if (!field.filterProperties)
                field.filterProperties = {};
            field.filterProperties.validations = {};
            field.filterProperties.validations.required = true;
        }
        else {
            field.setValue(null);
            field.readOnly = true;
            delete field.filterProperties.validations.required;
            if (field.widget.currentRow.$error) {
                field.widget.currentRow.$error.NMTIPOOPER = [];
            }
        }
    };

    //onChange dos campos de datas, do filtro
    this.minMaxDate = function(minOrMax, date, field) {
        if (minOrMax == 'max') {
            field.maxDate = date;
        }
        else {
            field.minDate = date;
        }
    };

    this.validaNivel = function(currentRow, args) {
        if (currentRow.CDPRODINI && currentRow.CDPRODFINAL) {
            if (!General.comparaPratoProduto(currentRow.CDPRODINI, currentRow.CDPRODFINAL, getLocalVar('GER_CDPICTPROD'))) {
                ScreenService.showMessage("O Produto Final não pode ser maior que o Produto Inicial.");
                args.e.preventDefault();
                return true;
            }
        }
    };

    this.setShowOnlist = function(boolean1, boolean2, widget) {
        //Campos do relatorio Inflação Interna
        widget.getField('QTLANCTOEST').isVisible = boolean1;
        widget.getField('VRCUSTINIC').isVisible = boolean1;
        widget.getField('VRTOTAL').isVisible = boolean1;
        widget.getField('VRCUSTATUA').isVisible = boolean1;
        widget.getField('VRTOTAL2').isVisible = boolean1;

        //Campos do relatorio Inflação de Compras
        widget.getField('QTMES1').isVisible = boolean2;
        widget.getField('VRCUNIT1').isVisible = boolean2;
        widget.getField('VRMES1').isVisible = boolean2;
        widget.getField('QTMES2').isVisible = boolean2;
        widget.getField('VRCUNIT2').isVisible = boolean2;
        widget.getField('VRMES2').isVisible = boolean2;

        //Campos do relatorio Inflação Interna Consolidada
        widget.getField('MES_ANO').isVisible = false;
        widget.getField('CDFILIMOVI').isVisible = false;

        widget.resizeGridColumns(true);
    };

    this.setShowOnListConsolidada = function(widget) {

        //Campos do relatorio Inflação Interna

        widget.getField('VRTOTAL').isVisible = false;
        widget.getField('VRCUSTINIC').isVisible = true;
        widget.getField('VRTOTAL2').isVisible = false;

        //Campos do relatorio Inflação de Compras
        widget.getField('QTMES1').isVisible = false;
        widget.getField('VRCUNIT1').isVisible = false;
        widget.getField('VRMES1').isVisible = false;
        widget.getField('QTMES2').isVisible = false;
        widget.getField('VRCUNIT2').isVisible = false;
        widget.getField('VRMES2').isVisible = false;

        //Campos do relatorio Inflação Interna consolidada
        widget.getField('QTLANCTOEST').isVisible = true;
        widget.getField('CDFILIMOVI').isVisible = true;
        widget.getField('VRCUSTATUA').isVisible = true;
        widget.getField('MES_ANO').isVisible = true;
        
        widget.resizeGridColumns(true);
    };

    this.openFilterInflacao = function(args) {
        var inflacao_interna = templateManager.container.getWidget('dfe26000_inflacao_interna_grid');
        this.buildDataSorceIfMask(inflacao_interna.getField('TAMNIVEL'));
        var filterRow = {
            'NMTIPOCUSTO': 'Custo Médio Líquido Atual',
            'TIPOCUSTO': '1',
            'DATAFINAL': moment().format("DD/MM/YYYY"),
            'DATAINICIAL': moment().subtract(1, 'months').format("DD/MM/YYYY"),
            'CDPRODINI': null,
            'CDPRODFINAL': null,
            'CDOPERENTR': 'Todos',
            'NMPRODINICIO': null,
            'NMPRODFINAL': null,
            'REPORT_NAME': "Inflação Interna",
        };

        this.getFilialLogadaOperador().then(filialLogada => {
            var filial = filialLogada.dataset.filial[0];
            filterRow.NMFILIAL = filial.NMFILIAL;
            filterRow.CDFILIAL = filial.CDFILIAL;
            ZhFilterPreferences.setOriginalFilter(inflacao_interna.id, filterRow);
            ZhFilterPreferences.openFilterWidget(inflacao_interna, filterRow).then(() => {
                var popUp = templateManager.container.getActiveWidget('zhFilterPreferencesSelectConditions');
                this.setReadOnlyByReport(popUp);
            });

        });


    };

    this.buildDataSorceIfMask = function(field) {
        var levels = General.getLevelByMask("GER_CDPICTPROD", 1);
        for (var index in levels) {
            field.dataSource.data.push({
                "name": levels[index].nivel,
                "value": levels[index].nivel
            });
        }
    };

    this.generateReport = function(filterRow) {
        Dfe26000_InflacaoInterna_Service.generateReport(filterRow).then(function(birtURL) {
            window.open(birtURL.dataset.URL[0]);
        }, function(error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    this.compararDatas = function(currentRow, args) {
        if (currentRow.REPORT_NAME != "Inflação Interna") {
            var dataInicial = moment(currentRow.MES1, "MM/YYYY");
            var dataFinal = moment(currentRow.MES2, "MM/YYYY");
            if (dataInicial.isAfter(dataFinal)) {
                ScreenService.showMessage('Comparativo Mês 1 deve ser menor que Mês 2.');
                args.e.preventDefault();
                return true;  
                
            }
        }
    };

    this.getFilialLogadaOperador = function() {
        var filter = [{
            "name": "CDFILIAL",
            "operator": "=",
            "value": "FILIAL_LOGADA"
        }];
        return General.callRestEngine('df#/crud/filial', 'FilterData', filter);
    };

    this.setActionFLoatingControl = function(popup) {
        var inflacao = templateManager.container.getWidget('dfe26000_inflacao_interna_grid');
        var action = inflacao.floatingControl.actions.filter(action => action.name === 'startBirt');
        if(popup.getField('REPORT_NAME').value() == 'Inflação Interna Consolidada'){
            action[0].icon = 'xls';
            action[0].label= "Relatório XLSX";
            inflacao.setAppliedLayout('');
        }
        else {
            action[0].icon = 'pdf';
            action[0].label= "Relatório PDF";
        }
    };

    this.setLayout = function(widget) {
        if (widget.filterData.REPORT_NAME == 'Inflação Interna Consolidada') {
            ZhLayoutPreferences.getWidgetLayouts(widget.id).then(layouts => {
                var layoutConsolidada = layouts.filter(layout => layout.ID === 'inflacaoInternaConsolidadaAgrupamentoDeFiliais');
                ZhLayoutPreferences.applyLayout(widget, layoutConsolidada[0]);
            });
        }
        else {
            ZhLayoutPreferences.applyOriginalLayout(widget);
        }
        
    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Dfe26000_InflacaoInterna_Controller', Dfe26000_InflacaoInterna_Controller);
});


// FILE: ./assets/modules/relatorios/assets/js/controllers/Dfe60001_relatorio_manifestacao_destinatario_Controller.js
var Dfe60001_Relatorio_Manifestacao_Destinatario_Controller = function(General, templateManager, ScreenService, ZhReports) {
    var self = this;
    
    var messages = {
        confirmaDesconhecimento: 'Atenção! Após confirmar a operação não será mais possível importar o XML da mesma. Deseja confirmar a operação?',
        nenhumaNota: 'Não é possivel realizar a operação para a(s) nota(s) selecionadas.',
        confirmarOperacao: 'Atenção! Deseja confirmar a operação para todas as notas marcadas?'
        
    };
    
    this.filterBefore = [];
    
    var requestError = function(error){
        var errorMessage = "Ocorreu um erro ao tentar realizar a operação.";
        if (error && error.data) {
            errorMessage = error.data.error;
        }
        ScreenService.showMessage(errorMessage, "ERROR");
    };

    var validaCampoCnpj = function(manifestacaoArgs) {
        var cnpj = manifestacaoArgs.owner.field.widget.getField("NRINSJURIEST");
        cnpj.readOnly = !manifestacaoArgs.row.CDEMPRESA;
        cnpj.setValue('');
        validaCampoInscricaoEstadual(manifestacaoArgs);
    };

    var validaCampoInscricaoEstadual = function(manifestacaoArgs) {
        var inscricaoEstadual = manifestacaoArgs.owner.field.widget.getField("CDINSCESTA");
        inscricaoEstadual.readOnly = !manifestacaoArgs.row.NRINSJURIEST;
        inscricaoEstadual.setValue('');
    };

    this.empresaOnChange = function(manifestacaoArgs) {
        validaCampoCnpj(manifestacaoArgs);
    };

    this.cnpjOnChange = function(manifestacaoArgs) {
        validaCampoInscricaoEstadual(manifestacaoArgs);
    };

    var getManifestacaoWidget = function(){
        if(templateManager.container)
            return templateManager.container.getWidget("manifestacao");
    };

    this.getCurrentField = function(field) {
        var manifestacaoWidget = getManifestacaoWidget();
        if (manifestacaoWidget)
            return !manifestacaoWidget.currentRow[field] ? null : manifestacaoWidget.currentRow[field];
    };

    var getFilterPopupCurrentRow = function(args){
        return args.owner.appliedFilter.FILTER_ROW;
    };

    this.getEmpresaFiltrada = function(){
        var manifestacaoWidget = getManifestacaoWidget();
        if(manifestacaoWidget){
            return manifestacaoWidget.appliedFilter.FILTER_ROW.CDEMPRESA;
        }
    };
    
    this.coloreLegenda = function(widgetGrid){
        var widget = widgetGrid.container.getWidget('legendaStripe');
        setTimeout(()=>{
            widget.getField('LEGENDAGRAY').element[0].style.color = 'gray';
            widget.getField('LEGENDARED').element[0].style.color = 'red';
            widget.getField('LEGENDAGREEN').element[0].style.color = 'green';
            widget.getField('LEGENDABLUE').element[0].style.color = 'blue';
        },500);
    };

    this.getFieldFromFilterPopup = function(field, concat = true){
        var manifestacaoWidget = getManifestacaoWidget();
        if(manifestacaoWidget){
            var widgetAtiva = manifestacaoWidget.getParent().getActiveWidget();
            if(widgetAtiva.name == 'zhFilterPreferencesSelectConditions')
                if(widgetAtiva.currentRow[field]){
                    if(widgetAtiva.currentRow[field].length > 0 && concat)
                        return widgetAtiva.currentRow[field] || 'TD';//return widgetAtiva.currentRow[field].toString().replace(/,/g, '_');
                    else
                        return widgetAtiva.currentRow[field] || 'TD';
                }
                else    
                    return widgetAtiva.currentRow[field] || 'TD';
            else
                return manifestacaoWidget.appliedFilter.FILTER_ROW[field] || 'TD';
        }
    };

    this.idorigdestOnChange = function(filterArgs) {
        alteraVisibilidadeCampoInscricaoFiltro(filterArgs);
        limpaCamposOutDataSelectsTpOperacFiltro(filterArgs);
    };
    

    var alteraVisibilidadeCampoInscricaoFiltro = function(filterArgs) {
        var filtroWidget = filterArgs.owner.field.widget;
        var cliente = filtroWidget.getField("NMRAZSOCCLIE");
        var fornecedor = filtroWidget.getField("NMRAZSOCFORN");
        var filial = filtroWidget.getField("DSFILIEMIT");
        var tipoDestino = filterArgs.row.IDORIGDESTNF;

        switch (tipoDestino) {
            case "C":
                validacaoCamposOutTpOperacFilter(cliente, fornecedor, filial);
                break;
            case "F":
                validacaoCamposOutTpOperacFilter(fornecedor, cliente, filial);
                break;
            case "L":
                validacaoCamposOutTpOperacFilter(filial, fornecedor, cliente);
                break;
            default:
                filial.readOnly = true;
                cliente.readOnly = true;
                fornecedor.readOnly = true;
        }
    };

    var limpaCamposOutDataSelectsTpOperacFiltro = function(filterArgs)
    {
        var filtroWidget = filterArgs.owner.field.widget;
        var cabecalhoRow = filtroWidget.currentRow;
        var cliente = filtroWidget.getField("NMRAZSOCCLIE");
        var fornecedor = filtroWidget.getField("NMRAZSOCFORN");
        var filial = filtroWidget.getField("DSFILIEMIT");

        filial.value(null);

        cliente.value(null);
        cabecalhoRow.CDCLIENTE = null;

        fornecedor.value(null);
        cabecalhoRow.CDFORNECED = null;
        cabecalhoRow.IDSITUCADA = null;
        cabecalhoRow.IDBLQNFRELAFOAF = null;

        cabecalhoRow.NRINSJURNF = null;
        cabecalhoRow.NMINSJURNF = null;
        cabecalhoRow.IDTPINSJURNF = null;
    };

    var validacaoCamposOutTpOperacFilter = function(princialField) {
        princialField.showOnView = true;
        princialField.showOnForm = true;
        princialField.isVisible = true;
        princialField.readOnly = false;

        for (var i = 1; i < arguments.length; i++) {
            arguments[i].showOnView = false;
            arguments[i].showOnForm = false;
            arguments[i].isVisible = false;
            arguments[i].readOnly = true;
        }
    };
    
     this.ajustaFilterBeforeApply = function(args){
        var preference = args.e.preference;
        var manifestacaoWidget = args.owner;
        preference.CUSTOM_FILTER_ROW = Util.clone(preference.FILTER_ROW);
        manifestacaoWidget.dataSourceFilter.forEach(element =>{
            var eName = element.name;
            preference.CUSTOM_FILTER_ROW[eName] = element.value;
        });
    };

    this.criaDataSourceGridFilterAfterApply = function(args) {
        var manifestacaoWidget = args.owner;
        var dataSourceFilter = manifestacaoWidget.dataSourceFilter;
        /*dataSourceFilter = dataSourceFilter.filter(function(value) {
            return value.name !== 'P_DTEMISSAOINI' && value.name !== 'P_DTEMISSAOFIN' && (value.isCustomFilter !== true || value.name === 'DTEMISSAO'); 
        });
        var datasFiltro = dataSourceFilter.filter(function(value) {
            return value.name === "DTEMISSAO";
        });
        var filterDefault = {
            'operator': '='
        };
        datasFiltro.forEach(function(data) {
            filterDefault.name = 'P_' + data.name + 'INI';
            filterDefault.value = data.value[0];
            dataSourceFilter.push(Util.clone(filterDefault));
            filterDefault.name = 'P_' + data.name + 'FIN';
            filterDefault.value = data.value[1];
            dataSourceFilter.push(Util.clone(filterDefault));

            if(dataSourceFilter.indexOf(data) > 0)
                dataSourceFilter.splice(dataSourceFilter.indexOf(data), 1);
        });*/
        
        
        
        //manifestacaoWidget.dataSourceFilter = dataSourceFilter;
        
        /*setTimeout(()=>{
            manifestacaoWidget.reload();
        },600);*/
    };

    this.abrirPopupJustificativa = function(args){
        var notas = args.owner.widget.getCheckedRows().filter(function(notafiscal){
                    return !notafiscal.NRLANCTONF && notafiscal.IDSTATUSMANIF !== "3";
                });

        if(notas.length > 0){
            var popupJustificativa = args.owner.widget.container.getWidget('popupJustificativa');
            ScreenService.openPopup(popupJustificativa);
        }
        else{
            ScreenService.showMessage(messages.nenhumaNota);
        }
    };

    this.openPopupErros = function(erros) {
        var widgetErros = templateManager.container.getWidget('WidgetErros');
        widgetErros.dataSource.data = erros;
        widgetErros.isVisible = true;
        ScreenService.openPopup(widgetErros);
        widgetErros.activate();
    };
    
    this.closeErros = function(widget) {
        widget.isVisible = false;
        widget.dataSource.data = [];
        ScreenService.closePopup(widget);
        widget.parent.activate();
        widget.parent.unSelectAll();
        widget.parent.reload();
        ScreenService.closePopup();
    };

    
    this.openPopupResumo = function(sincronizacao) {
        var wdgResumo = templateManager.container.getWidget('RESUMOSINC');
        wdgResumo.isVisible = true;
        wdgResumo.currentRow = sincronizacao[0].DADOS;
        wdgResumo.widgets[0].dataSourceFilter[0].value = sincronizacao[0].CHAVES; 
        wdgResumo.widgets[0].dataSourceFilter[0].operator = "IN";
        wdgResumo.widgets[0].reload();
        ScreenService.openPopup(wdgResumo);
    };
    
    this.closePopupResumo = function() {
        var wdgSinc = templateManager.container.getWidget('RESUMOSINC');
        ScreenService.closePopup();
        wdgSinc.isVisible = false;
    };
    
    this.openPopupSincronizacao = function() {
        var wdgSinc = templateManager.container.getWidget('WidgetSincronizacao');
        wdgSinc.currentRow.FORMAPESQUISA    = "0";
        wdgSinc.isVisible = true;
        ScreenService.openPopup(wdgSinc);
    };
    
    this.closePopupSincronizacao = function() {
        var wdgSinc = templateManager.container.getWidget('WidgetSincronizacao');
        ScreenService.closePopup();
        wdgSinc.isVisible = false;
    };
    
    this.closePopup = function(widget) {
        widget.isVisible = false;
        widget.dataSource.data = [];
        ScreenService.closePopup(widget);
        widget.parent.activate();
        widget.parent.unSelectAll();
        widget.parent.reload();
        ScreenService.closePopup();
    };
    
    this.generateReport = function(args, format) {
        let customData = {}, sourceWidget = templateManager.container.getWidget('manifestacao'), 
            action = (format == 'PDF') ? 'pdfAction' : 'xlsAction', IDORIGDESTNF = sourceWidget.appliedFilter.FILTER_ROW.IDORIGDESTNF;
       /* if(IDORIGDESTNF) {

            if(IDORIGDESTNF == 'F') {
                sourceWidget.floatingControl[action].dataSourceName = 'manifestacao_destinatario_relatorio_fornecedor';
            }
            else if(IDORIGDESTNF == 'C')  {
                sourceWidget.floatingControl[action].dataSourceName = 'manifestacao_destinatario_relatorio_cliente';
            }
            else {
                sourceWidget.floatingControl[action].dataSourceName = 'manifestacao_destinatario_relatorio_filial';
            }
        }
        else {
            sourceWidget.floatingControl[action].dataSourceName = 'manifestacao_destinatario_relatorio';
        }
        var filtroData = [
            {
                "name": "P_DTEMISSAOINI",
                "operator": "=",
                "value": ""
            },
            {
                "name": "P_DTEMISSAOFIN",
                "operator": "=",
                "value": ""
            }
        ];
        this.filterBefore = Util.clone(sourceWidget.dataSourceFilter);
        sourceWidget.dataSourceFilter.forEach((filter, index) => {
            if(filter.name == 'IDORIGDESTNF') {
                sourceWidget.dataSourceFilter = [...sourceWidget.dataSourceFilter.slice(0, index), ...sourceWidget.dataSourceFilter.slice(index+1, sourceWidget.dataSourceFilter.length)];
            }
            else if(filter.name == 'DTEMISSAO') {
                filtroData[0].value = filter.value[0];
                
                filtroData[1].value = filter.value[1];
                //sourceWidget.dataSourceFilter = [...sourceWidget.dataSourceFilter.slice(0, index), ...sourceWidget.dataSourceFilter.slice(index+1, sourceWidget.dataSourceFilter.length)];
                sourceWidget.dataSourceFilter = [...sourceWidget.dataSourceFilter, ...filtroData];
                var filtro = sourceWidget.dataSourceFilter.filter(data => {
                   if(data.name != 'DTEMISSAO') {
                        if(data.value.indexOf('Dfe60001_Relatorio_Manifestacao_Destinatario_Controller') != -1) {
                            data.value = sourceWidget.appliedFilter.FILTER_ROW[data.name] || 'TD';
                        }
                        return data;
                   }
                });
                sourceWidget.dataSourceFilter = filtro;
            }
        });*/
        
        
        ZhReports.generateReport(customData, sourceWidget, this._previewCallback, function() {}, format);
        //templateManager.container.getWidget('manifestacao').dataSourceFilter = self.filterBefore;


    };
    
    this._previewCallback = function(data) {
        //templateManager.container.getWidget('manifestacao').dataSourceFilter = self.filterBefore;
        var popup_window = window.open(data.path, '_blank');
        if (!popup_window) {
            ScreenService.showMessage(ScreenService.i18n('Pop-up Blocker is enabled! Please add this site to your exception list.'));
        }
        if (callback) {
            callback(data);
        }
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfe60001_Relatorio_Manifestacao_Destinatario_Controller", Dfe60001_Relatorio_Manifestacao_Destinatario_Controller);
});

// FILE: ./assets/modules/relatorios/assets/js/controllers/Dfs21100_RomaneioNotasFiscaisSaida_Controller.js
var Dfs21100_RomaneioNotasFiscaisSaida_Controller = function(restEngine, templateManager, General, ZhFilterPreferences) {

	this.openFilterRomaneio = function (widget){
        var dataAtual = moment().format('DD/MM/YYYY');
        var start = dataAtual;
        var end   = dataAtual;
        var rowFilter = {};
        
        rowFilter = {
 	        'DTENTRSAIDFILTER': {start : start, end : end}
 	    };
 	  
 	  rowFilter.DTENTRSAIDFILTER.validations = { "requiredInterval": true };
 	  General.callRestEngine("df_saida#/getFilterExportSap", "Row", {}).then((response) => {
 	        if(response.dataset.filter) {
 	            rowFilter.CDEMPRESA = response.dataset.filter.CDEMPRESA;
 	            rowFilter.NMEMPRESA = response.dataset.filter.NMEMPRESA;
 	            rowFilter.CDFILIAL  = response.dataset.filter.CDFILIAL;
 	            rowFilter.NMFILIAL  = response.dataset.filter.NMFILIAL;
 	            rowFilter.CDINSCESTA = response.dataset.filter.CDINSCESTA;
 	            rowFilter.CDINSCMUNI = response.dataset.filter.CDINSCMUNI;
 	            
 	        }
            ZhFilterPreferences.setOriginalFilter(widget.id, rowFilter);
            ZhFilterPreferences.openFilterWidget(widget, rowFilter);
 	    },
 	    (error) => {
 	        ZhFilterPreferences.setOriginalFilter(widget.id, rowFilter);
            ZhFilterPreferences.openFilterWidget(widget, rowFilter);
 	    });
    };
    
    this.preparaFiltro = function(args) {
        args.e.preventDefault();
        var widget = args.owner;
        var filters = [];
        if(widget.appliedFilter) {
            args.e.filters.map((f) => {
                if(f.name != 'DTENTRSAIDFILTER') {
                    filters.push(f);
                }
                else {
                    filters.push({
                        name: "P_DTENTRSAIDINI",
                        operator: "=",
                        value: f.value[0]
                    });
                    filters.push({
                        name: "P_DTENTRSAIDFIN",
                        operator: "=",
                        value: f.value[1]
                    });
                }
            });     
        }
        args.owner.dataSourceFilter = filters;
        args.owner.reload();
    };
    
    this.getFieldValueFilter = function(fieldName, widgetName) {
        var container =  templateManager.container;
        
        if(container) {
            var filterWidget;
            var filterBeforeApply = templateManager.container.getActiveWidget();
            
            //test about how widget is... (field or filter)

    	    if(filterBeforeApply.label != 'Filtro'){		
    		    filterBeforeApply = filterBeforeApply.lastActiveWidget;
                filterWidget = filterBeforeApply;
                
            }
            
            if(filterBeforeApply.name == 'zhFilterPreferencesSelectConditions') {
            	filterWidget = filterBeforeApply;
            }
            
            else {
                var f = container.getWidget(widgetName);
                if(f) {
                     filterWidget = container.getWidget(widgetName).appliedFilter;
                }
               
            }
            
            // Insert values
            if(filterWidget) {
                var valueField;
                if(filterBeforeApply.name == 'zhFilterPreferencesSelectConditions')
                    valueField = filterWidget.currentRow[fieldName];
                    
                else
                    valueField = filterWidget.FILTER_ROW[fieldName];
                    
                return valueField || null;
            }
            else return null;
        }
        else return null;
    };
  
    this.habilitaOrigDestNF = function(args) {
        var widget = args.owner.owner.widget;
        switch(widget.currentRow.IDORIGDESTNF) {
            case 'C':
                widget.getField('CDFORNECED').isVisibleForm = false;
                widget.getField('CDFORNECED').isVisible     = false;
                widget.getField('CDFORNECED').showOnForm     = false;
                widget.getField('CDFORNECED').value("");

                widget.getField('CDFILIRECE').isVisibleForm   = false;
                widget.getField('CDFILIRECE').isVisible       = false;
                widget.getField('CDFILIRECE').showOnForm       = false;
                widget.getField('CDFILIRECE').value("");

                widget.getField('CDCLIENTE').isVisibleForm = true;
                widget.getField('CDCLIENTE').showOnForm     = true;
                widget.getField('CDCLIENTE').isVisible     = true;
                widget.getField('CDCLIENTE').validations.required = true;
                delete(widget.getField('CDFORNECED').validations.required );
                delete(widget.getField('CDFILIRECE').validations.required );
                break;
            case 'F':
                widget.getField('CDCLIENTE').isVisibleForm = false;
                widget.getField('CDCLIENTE').isVisible     = false;
                widget.getField('CDCLIENTE').showOnForm     = false;
                widget.getField('CDCLIENTE').value("");

                widget.getField('CDFILIRECE').isVisibleForm   = false;
                widget.getField('CDFILIRECE').isVisible       = false;
                widget.getField('CDFILIRECE').showOnForm       = false;
                widget.getField('CDFILIRECE').value("");

                widget.getField('CDFORNECED').isVisibleForm = true;
                widget.getField('CDFORNECED').isVisible     = true;
                widget.getField('CDFORNECED').showOnForm     = true;

                widget.getField('CDFORNECED').validations.required = true;
                delete(widget.getField('CDCLIENTE').validations.required );
                delete(widget.getField('CDFILIRECE').validations.required );
                break;
            case 'I':
                widget.getField('CDCLIENTE').isVisibleForm = false;
                widget.getField('CDCLIENTE').isVisible     = false;
                widget.getField('CDCLIENTE').showOnForm     = false;
                widget.getField('CDCLIENTE').value("");

                widget.getField('CDFORNECED').isVisibleForm = false;
                widget.getField('CDFORNECED').isVisible     = false;
                widget.getField('CDFORNECED').showOnForm     = false;
                widget.getField('CDFORNECED').value("");

                widget.getField('CDFILIRECE').isVisibleForm   = true;
                widget.getField('CDFILIRECE').isVisible       = true;
                widget.getField('CDFILIRECE').showOnForm       = true;

                widget.getField('CDFILIRECE').validations.required = true;
                delete(widget.getField('CDCLIENTE').validations.required );
                delete(widget.getField('CDFORNECED').validations.required );

                break;
            default:
                widget.getField('CDCLIENTE').isVisibleForm = false;
                widget.getField('CDCLIENTE').isVisible     = false;
                widget.getField('CDFORNECED').isVisibleForm = false;
                widget.getField('CDFORNECED').isVisible     = false;
                widget.getField('CDFILIRECE').isVisibleForm   = false;
                widget.getField('CDFILIRECE').isVisible       = false;
                widget.getField('CDCLIENTE').showOnForm     = false;
                widget.getField('CDFILIRECE').showOnForm       = false;
                widget.getField('CDFORNECED').showOnForm     = false;
                widget.getField('CDFILIRECE').value("");
                widget.getField('CDCLIENTE').value("");
                widget.getField('CDFORNECED').value("");
                delete(widget.getField('CDCLIENTE').validations.required );
                delete(widget.getField('CDFILIRECE').validations.required );
                delete(widget.getField('CDFORNECED').validations.required );
                break;
        }
    };

    this.verficaCampoEmpresa = function(args){
        var widget = args.owner.owner.widget;
        if (widget.currentRow.NMEMPRESA){
            widget.getField('NMREPRINSEST').readOnly = false;
            widget.currentRow.NMREPRINSEST = '';
            widget.getField('NMREPRINSEST').reload();
            widget.getField('NMREPRINSMUN').readOnly = false;
            widget.currentRow.NMREPRINSMUN = '';
            widget.getField('NMREPRINSMUN').reload();
            widget.getField('NMFILIAL').readOnly = false;
            widget.currentRow.NMFILIAL = '';
            widget.getField('NMFILIAL').reload();

        } else {
            widget.currentRow.NMREPRINSEST = '';
            widget.getField('NMREPRINSEST').readOnly = true;
            widget.currentRow.NMFILIAL = '';
            widget.getField('NMFILIAL').readOnly = true;
            widget.currentRow.NMREPRINSMUN = '';
            widget.getField('NMREPRINSMUN').readOnly = true;
        }
        widget.reload();

    };
    
    this.recarregarInscricaoFiltro = function (nomeCampo){
        var widget = templateManager.container.getWidget('romaneioNotasFiscaisSaidaGrid');
        var filtro = widget.appliedFilter;
        filtro.getField(nomeCampo).reload();
    };

    this.toggleInscricaoEstadual = function (field) {
        field.dataSource.data.forEach(inscricao => inscricao.NMREPRINSEST = inscricao.NMREPRINSEST === null ? inscricao.CDINSCESTA : inscricao.NMREPRINSEST );
    };
    
    this.toggleInscricaoMunicipal = function (field) {
        field.dataSource.data.forEach(inscricao => inscricao.NMREPRINSMUN = inscricao.NMREPRINSMUN === null ? inscricao.CDINSCMUNI : inscricao.CDINSCMUNI );
    };
    
};

Configuration(function(ContextRegister) {
	ContextRegister.register('Dfs21100_RomaneioNotasFiscaisSaida_Controller', Dfs21100_RomaneioNotasFiscaisSaida_Controller);
});

// FILE: ./assets/modules/relatorios/assets/js/controllers/Dfe13000_espelho_de_notas_fiscais.js
var Dfe13000_espelho_de_notas_fiscais = function(General, templateManager, ZHPromise, ScreenService, ZhReports,ZhFilterPreferences, espelhoNF_Repository) {
    
    this.openFilter = function(espelhoNFArgs) {
        var espelhoNFWidget = espelhoNFArgs.owner;
        setDefaultFilialFilter(espelhoNFWidget).then(defaultFilter =>{
            ZhFilterPreferences.setOriginalFilter(espelhoNFWidget.id, defaultFilter);
            ZhFilterPreferences.openFilterWidget(espelhoNFWidget, defaultFilter).then(response => {
                var filter = templateManager.container.getActiveWidget();
                var clearFilter = filter.actions.filter(action => action.name === 'clear')[0];
                var confirmFilter = filter.actions.filter(action => action.name === 'never')[0];
                
                clearFilter.shortcutKey = 'ctrl+backspace';
                clearFilter.tooltip = 'Limpar Filtro (CTRL + Backspace)';
                
                confirmFilter.shortcutKey = 'ctrl + enter';
                filter.reload();
                
            });
        });
    };
    
    var getEspelhoWidget = function(){
        if(templateManager.container)
            return templateManager.container.getWidget("espelho_de_notas_fiscais");
    };
    
    this.getFieldFromFilterPopup = function(field , data){
        var EspelhoWidget = getEspelhoWidget();
        if(EspelhoWidget){
            var widgetAtiva = EspelhoWidget.getParent().getActiveWidget();
            if (data){
                if( widgetAtiva.name == 'zhFilterPreferencesSelectConditions'){
                    return data ==='end'? widgetAtiva.currentRow[field].end || null : widgetAtiva.currentRow[field].start || null ;
                }else{
                    return  data ==='end'? EspelhoWidget.appliedFilter.FILTER_ROW[field].end  || null :  EspelhoWidget.appliedFilter.FILTER_ROW[field].start  || null ;
                }
            }else {
                if(widgetAtiva.name == 'zhFilterPreferencesSelectConditions')
                return widgetAtiva.currentRow[field] || null;
            else
                return EspelhoWidget.appliedFilter.FILTER_ROW[field] || null;
            }
        }
    };

    var setDefaultFilialFilter = function(cabecalhoWidget) {
        var popupFilter = Util.clone(cabecalhoWidget.currentRow);
        var promise = ZHPromise.defer(); 
        
        General.callRestEngine( "df#/filial_lancamento_notafiscal", "FilterData", [{name: "CDFILIAL", operator: "=", value: "FILIAL_LOGADA"}]).then( (filialLogada) =>  {
            
            var filial = filialLogada.dataset.filial_lancamento_notafiscal[0];
            popupFilter.NMFILIAL = filial.NMFILIAL;
            popupFilter.CDFILIAL = filial.CDFILIAL;
            popupFilter.NMEMPRESA = filial.NMEMPRESA;
            popupFilter.CDEMPRESA = filial.CDEMPRESA;
            popupFilter.CDINSCESTA = filial.CDINSCESTA;
            popupFilter.DSINSCESTA = filial.CDINSCESTA;
            popupFilter.IDENTRSAIDOP = "E";

            popupFilter.DTEMISSAO = {start:  moment().subtract(1, 'months').format('DD/MM/YYYY'), end: moment().format("DD/MM/YYYY")};        
                    
            promise.resolve(popupFilter);
            
        }); 
  
        return promise;
    };

    this.getFieldValueFilter = function(fieldName, widgetName) {
        var container =  templateManager.container;
        if(container) {
            var filterWidget;
            var filterBforeApply = templateManager.container.getActiveWidget();
            if(filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
              filterWidget = filterBforeApply;
            else {
                if(filterBforeApply.lastActiveWidget.name == 'zhFilterPreferencesSelectConditions'){
                    filterWidget = filterBforeApply.lastActiveWidget;
                    return filterWidget.currentRow[fieldName];
                }
                else{
                    var f = container.getWidget(widgetName);
                    if(f) {
                         filterWidget = container.getWidget(widgetName).appliedFilter;
                    }
                }
                    
               
            }
            
            if(filterWidget) {
                var valueField;
                if(filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
                    valueField = filterWidget.currentRow[fieldName];
                else
                    valueField = filterWidget.FILTER_ROW[fieldName];
                    
                return valueField || null;
            }
            
               else return null;
        }
        else return null;
    };
    
    this.empresaOnChange = function(widget) {
        validaCamposInscricao(widget);
        var unidade = widget.getField("NMFILIAL");
        var empresa = widget.getField("NMEMPRESA");
        var inscrestad = widget.getField("CDINSCESTA");
        
        if(empresa.value() === ""){
            widget.currentRow.CDFILIAL = "";
            widget.currentRow.NMFILIAL = "";
            widget.currentRow.CDINSCESTA = "";
            
            unidade.readOnly = true;
            inscrestad.readOnly = true;
            
        } else {
            widget.currentRow.CDFILIAL = "";
            widget.currentRow.NMFILIAL = "";
            widget.currentRow.CDINSCESTA = "";
            
            unidade.readOnly = false ;
            inscrestad.readOnly = false ;
        }
        
        unidade.reload();
        inscrestad.reload();
    };
    
    var validaCamposInscricao = function(widget){
        var inscEsta = widget.getField("CDINSCESTA");
        inscEsta.readOnly = !widget.currentRow.NMEMPRESA;

        if(inscEsta.readOnly){
            widget.currentRow.DSINSCESTA = '';
        }
    };
    
    this.validaFiltro = function(args){
        var espelhoWidget = args.owner;
        var dataSourceFilter = espelhoWidget.dataSourceFilter;
        var camposData = ['IDENTRSAIDOP','CDEMPRESA','CDFILIAL', 'NMFILIAL', 'DTEMISSAO' , 'REPORTNAME'];
        var datasRemover = dataSourceFilter.filter(function(field){
            return camposData.indexOf(field.name) >= 0;
        });
        if(datasRemover && datasRemover.length >= 0){
            datasRemover.forEach(function(data){
                dataSourceFilter.splice(dataSourceFilter.indexOf(data), 1);
            });
        }
        
        args.owner.reload();
    };
    
    this.geraRelatorio = function(widget){
        var rows = widget.getCheckedRows();
        //var REPORTNAME  = widget.appliedFilter.FILTER_ROW.REPORTNAME;
        if(rows.length > 0 ){
            validaDados(rows).then((NFsemProdutos) => {
                if (NFsemProdutos.length !== 0){
                        mensagemNaoDadosListado(NFsemProdutos);
                }else {
                    rows[0].REPORTNAME = "DFE13000";
                    espelhoNF_Repository.geraRelatorio(rows).then(function(birtURL){
                    window.open(birtURL.dataset.URL[0]);
                    }, function(error) {
                    ScreenService.showMessage(error.data.error);
                });
                } 
            });

        }else {
            ScreenService.showMessage("Selecione uma Nota Fiscal");
        }  
    };

    this.toggleEntradaSaida = function() {
        var widget = templateManager.container.getActiveWidget();
        var row =  widget.currentRow;

        if(row.IDENTRSAIDOP == 'E') {
            getEspelhoWidget().getField('DTENTRSAID').label = "Data de Entrada";
            
            getEspelhoWidget().getField('NRINSJURNF').showOnList = true;
            getEspelhoWidget().getField('NRINSJURNF').isVisible = true;

            getEspelhoWidget().getField('NMRAZAOORIGENTR').showOnList = true;
            getEspelhoWidget().getField('NMRAZAOORIGENTR').isVisible = true;

            getEspelhoWidget().getField('NMRAZAOORIGSAID').showOnList = false;
            getEspelhoWidget().getField('NMRAZAOORIGSAID').isVisible = false;

            getEspelhoWidget().getField('NMRAZAODESTSAID').showOnList = false;
            getEspelhoWidget().getField('NMRAZAODESTSAID').isVisible = false;

        } else {
            getEspelhoWidget().getField('DTENTRSAID').label = "Data de Saída";

            getEspelhoWidget().getField('NRINSJURNF').showOnList = false;
            getEspelhoWidget().getField('NRINSJURNF').isVisible = false;

            getEspelhoWidget().getField('NMRAZAOORIGENTR').showOnList = false;
            getEspelhoWidget().getField('NMRAZAOORIGENTR').isVisible = false;

            getEspelhoWidget().getField('NMRAZAOORIGSAID').showOnList = true;
            getEspelhoWidget().getField('NMRAZAOORIGSAID').isVisible = true;

            getEspelhoWidget().getField('NMRAZAODESTSAID').showOnList = true;
            getEspelhoWidget().getField('NMRAZAODESTSAID').isVisible = true;

        }
        
        getEspelhoWidget().reload();

    };
    
    var validaDados =  function(rows) {
        var promise = ZHPromise.defer(); 
        let NFsemProdutos = [];
        var rowsProcessadas = 0;
        rows.forEach((row) => {
            var filter =  [{name: "CDFILIAL",operator: "=",value: row.CDFILIAL},{name: "NRLANCTONF",operator: "=",value: row.NRLANCTONF}];
            General.callRestEngine( "df_relatorios#/crud/qryCDItemProduto", "FilterData", filter).then(function(itemProduto){
                 if (itemProduto.dataset.qryCDItemProduto.length == 0) {
                    NFsemProdutos.push(row.NRLANCTONF);
                 }
                 rowsProcessadas++;
                 if(rowsProcessadas == rows.length){ 
                     promise.resolve(NFsemProdutos);
                     
                 }
            }); 
        });

            
        return promise;
  
    };
    
    var mensagemNaoDadosListado =  function(rows) {
        let notas =" ";
        rows.forEach(function(row){
            notas += `, ${row}`;
        });
        
        ScreenService.showMessage("Não há dados listados para a(s) nota(s) com o(s) seguinte(s) número(s) de lançamento(s): " + notas.substr(2) );
    };
    
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfe13000_espelho_de_notas_fiscais", Dfe13000_espelho_de_notas_fiscais);
});
